"{ Package: 'stx:goodies' }"

ArrayedCollection variableSignedLongSubclass:#ShortRunArray
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Collections-Arrayed'
!

ShortRunArray comment:'This class is run-length encoded representation of short integer (e.g., 16bit signed integer values)'!

!ShortRunArray class methodsFor:'documentation'!

documentation
"
    adapted (well; actually: filedIn) from squeak
"
! !

!ShortRunArray class methodsFor:'instance creation'!

new: n
	"ShortRunArrays must be created with either
		someCollection as: ShortRunArray
	or by using
		ShortRunArray runs: runCollection values: valueCollection.
	"
	^self shouldNotImplement!

newFrom: aCollection
    "return a new ShortRunArray with elements taken from the argument, aCollection.
     (i.e. compress aCollection into a ShortRunArray)"

        | lastValue lastRun runs values |
        aCollection isEmpty ifTrue:[^self runs:#() values: #()].
        runs _ WriteStream on: (WordArray new: 100).
        values _ WriteStream on: (SignedWordArray "ShortIntegerArray" new: 100).
        lastValue _ aCollection first.
        lastRun _ 0.
        aCollection do:[:item|
                (item = lastValue and:[lastRun < 16r8000]) ifTrue:[
                        lastRun _ lastRun + 1.
                ] ifFalse:[
                        runs nextPut: lastRun.
                        values nextPut: lastValue.
                        lastRun _ 1.
                        lastValue _ item.
                ].
        ].
        runs nextPut: lastRun.
        values nextPut: lastValue.
        ^self runs: runs contents values: values contents
!

runs: runCollection values: valueCollection
	^(self basicNew: runCollection size) setRuns: runCollection values: valueCollection! !

!ShortRunArray class methodsFor:'class initialization'!

initialize
        "ShortRunArray initialize"
"/        Smalltalk addToStartUpList: self.
"/        LastSaveOrder _ #(42 42 42) as: self.
!

startUp
        "Check if the word order has changed from the last save"
"/        ((LastSaveOrder valueAtRun: 1) = 42 and:[(LastSaveOrder lengthAtRun: 1) = 3]) 
"/                ifTrue:[^self]. "Okay"
"/        ((LastSaveOrder lengthAtRun: 1) = 42 and:[(LastSaveOrder valueAtRun: 1) = 3]) 
"/                ifTrue:[^self swapRuns]. "Reverse guys"
"/        ^self error:'This must never happen'
!

swapRuns
	self allSubInstancesDo:[:inst| inst swapRuns]! !

!ShortRunArray methodsFor:'accessing'!

at: index
	"Return the short value at the given index"
	| rlIndex |
	index < 1 ifTrue:[^self errorSubscriptBounds: index].
	rlIndex _ index.
	self lengthsAndValuesDo:[:runLength :runValue|
		rlIndex <= runLength ifTrue:[^runValue].
		rlIndex _ rlIndex - runLength].
	"Not found. Must be out of range"
	^self errorSubscriptBounds: index!

at: index put: value
	"ShortRunArrays are read-only"
	^self shouldNotImplement.!

compressionRatio
	"Return the compression ratio.
	The compression ratio is computed based
	on how much space would be needed to
	store the receiver in a ShortIntegerArray"
	^(self size asFloat * 0.5) "Would need only half of the amount in ShortIntegerArray"
		/ (self runSize max: 1)!

lengthAtRun: index
	"Return the length of the run starting at the given index"
	^(self basicAt: index) bitShift: -16!

runSize
	"Return the number of runs in the receiver"
	^self basicSize!

size
	"Return the number of elements stored in the receiver"
	| n |
	n _ 0.
	"Note: The following loop is open-coded for speed"
	1 to: self basicSize do:[:i|
		n _ n + ((self basicAt: i) bitShift: -16).
	].
	^n!

species
        "Answer the preferred class for reconstructing the receiver."
        ^ SignedWordArray "ShortIntegerArray"
!

valueAtRun: index
	"Return the value of the run starting at the given index"
	| uShort |
	uShort _ (self basicAt: index) bitAnd: 16rFFFF.
	^(uShort bitAnd: 16r7FFF) - (uShort bitAnd: 16r8000)! !

!ShortRunArray methodsFor:'enumerating'!

do: aBlock
	"Evaluate aBlock with all elements of the receiver"
	self lengthsAndValuesDo:[:runLength :runValue|
		"Use to:do: instead of timesRepeat: for compiler optimization"
		1 to: runLength do:[:i|
			aBlock value: runValue.
		].
	].!

lengthsAndValuesDo: aBlock
	"Evaluate aBlock with the length and value of each run in the receiver"
	^self runsAndValuesDo: aBlock!

runsAndValuesDo: aBlock
	"Evaluate aBlock with the length and value of each run in the receiver"
	| basicValue length value |
	1 to: self basicSize do:[:i|
		basicValue _ self basicAt: i.
		length _ basicValue bitShift: -16.
		value _ basicValue bitAnd: 16rFFFF.
		value _ (value bitAnd: 16r7FFF) - (value bitAnd: 16r8000).
		aBlock value: length value: value.
	].!

valuesCollect: aBlock
	"Evaluate aBlock with each of the receiver's values as the argument. 
	Collect the resulting values into a collection like the receiver. Answer 
	the new collection."
	| newArray newValue |
	newArray _ self class basicNew: self basicSize.
	1 to: self runSize do:[:i|
		newValue _ aBlock value: (self valueAtRun: i).
		newArray setRunAt: i toLength: (self lengthAtRun: i) value: newValue.
	].
	^newArray!

valuesDo: aBlock
	self lengthsAndValuesDo:[:runLength :runValue| aBlock value: runValue]! !

!ShortRunArray methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: self class name; nextPutAll:' ( '.
	self lengthsAndValuesDo:[:runLength :runValue |
		aStream
			nextPutAll:' (';
			print: runLength;
			space;
			print: runValue;
			nextPut:$).
	].
	aStream nextPutAll:' )'.! !

!ShortRunArray methodsFor:'private'!

pvtAt: index
	"Private -- for swapping only"
	<primitive: 143>
	index isInteger ifTrue: [self errorSubscriptBounds: index].
	index isNumber ifTrue: [^ self at: index truncated].
	self errorNonIntegerIndex.
!

pvtAt: index put: value
	"Private -- for swapping only"
	<primitive: 144>
	index isInteger
		ifTrue: [
			(index >= 1 and: [index <= self size])
				ifTrue: [self errorImproperStore]
				ifFalse: [self errorSubscriptBounds: index]].
	index isNumber ifTrue: [^ self at: index truncated put: value].
	self errorNonIntegerIndex.
!

setRunAt: i toLength: runLength value: value
	(value < -16r7FFF or:[value > 16r8000]) ifTrue:[^self errorImproperStore].
	(runLength < 0 or:[runLength > 16rFFFF]) ifTrue:[^self errorImproperStore].
	self basicAt: i put: (runLength bitShift: 16) + 
		((value bitAnd: 16r7FFF) - (value bitAnd: -16r8000)).!

setRuns: runArray values: valueArray
	| runLength value |
	1 to: runArray size do:[:i|
		runLength _ runArray at: i.
		value _ valueArray at: i.
		self setRunAt: i toLength: runLength value: value.
	].!

swapRuns
	"Private -- swap length/value pairs in the receiver"
	| tmp |
	1 to: self basicSize do:[:i|
		tmp _ (self pvtAt: i * 2).
		self pvtAt: i * 2 put: (self pvtAt: i * 2 - 1).
		self pvtAt: i * 2 - 1 put: tmp.
	]! !

!ShortRunArray class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ShortRunArray.st,v 1.3 2000/09/25 14:31:27 cg Exp $'
! !
ShortRunArray initialize!
