"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#SysProProtocol
	instanceVariableNames:'protocolId conformsToProtocolIds messages protocolGroup
		specSections definesBehaviorOfClasses definesBehaviorOfList'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Protocols'
!


!SysProProtocol class methodsFor:'instance creation'!

newProtocolId: protocolSpec conformsToProtocols: conformsToSpec protocolGroup: group
	"Answer a new protocol with identifier protocolSpec that conforms to the protocols in conformsToSpec and grouped in group.
	Note: Currently only the protocols manager should create new protocols to ensure that they are properly managed (i.e. one important function of the manager is to insure no duplicate protocols exist)."
	| newProtocol conformsToList |
	protocolSpec first = $< ifFalse: [
		^self error: 'Protocol "', protocolSpec, '" not bracketed with ">".'].
	conformsToList := self conformsToProtocolIdsFrom: conformsToSpec.

	newProtocol := self basicNew.
	newProtocol setNewProtocolId: protocolSpec
		conformsToProtocolIds: conformsToList
		protocolGroup: group.
	^newProtocol
! !

!SysProProtocol class methodsFor:'constants/etc.'!

protocolANYId
	"Answer the protocol <ANY> identifier."

	^#'<ANY>'
! !

!SysProProtocol class methodsFor:'private'!

conformsToProtocolIdsFrom: conformsToProtocolsString
	"Private - Answer an <Array>, the conforms-to protocol id. list."
	| conformsToProtocolIdList protoIdStr strStream |
	conformsToProtocolIdList := OrderedCollection new.
	strStream := ReadStream on: conformsToProtocolsString.
	[strStream atEnd] whileFalse: [
		protoIdStr := (strStream upTo: $,) trimBlanks.
		protoIdStr notEmpty ifTrue: [
			conformsToProtocolIdList add: protoIdStr]].
	^conformsToProtocolIdList asArray
!

defaultMessageCollection
	"Private - Answer a <Set>, the default messageSpecification collection."

	^Set new
!

defaultSpecSectionsCollection
	"Private - Answer a <Dictionary>, the default SpecSections collection."

	^IdentityDictionary new
! !

!SysProProtocol methodsFor:'accessing'!

allConformsToProtocolIds
	"Answer the identifiers of all protocols to which the receiver conforms including super protocols."
	| tmpList |
	tmpList := OrderedCollection new.
	self addConformsToProtocolIdsTo: tmpList.
	^tmpList
!

allMessageSelectorProtocolDictionary
	"Answer a selector -> protocol dictionary of all selectors the receiver responds to including super protocols."
	| protocol aDict visitedProtocols set |
	aDict := Dictionary new.
	visitedProtocols := Set new.
	self
		addAllSelectorAndConformsToSelectorsTo: aDict
		visited: visitedProtocols.
	^aDict
!

allMessageSelectors
	"Answer all of selectors which make up the receiver's protocol and all protocols to which the receiver conforms."
	| tmpList protoMgr |
	protoMgr := self protoMgr.
	tmpList := self messageSelectors copy.
	self allConformsToProtocolIds do: [:protoId |
		tmpList addAll: (protoMgr protocolId: protoId) messageSelectors].
	^tmpList
!

conformsToProtocolIds
	"Answer the identifiers of protocols to which the receiver conforms."

	^conformsToProtocolIds copy
!

definesBehaviorOfList
	"Answer the set of classes, or metaclasses, the receiver defines the behavior of."

	^definesBehaviorOfList copy
!

messageAtSelector: selectorSpec

	^self messageAtSelector: selectorSpec
		ifNone: [self error: 'Protocol message "#', selectorSpec, '" not found.'].


	"Definition: <SystemProtocol> 
	Answer the message spec. at selectorSpec. 
	Parameters 
	selectorSpec	<symbol> 	uncaptured 
	Return Values 
	<SystemProtocolMsg>		state 
	Errors 
	A protocol named protocolName not found in the list of protocols.
	"
!

messageAtSelector: selectorSpec ifNone: notFoundBlock
	"Answer the receiver's message specification identified by selectorSpec or if not found evaluate notFoundBlock."
	| selector |
	selector := selectorSpec asSymbol.
	messages isNil ifTrue: [
		^notFoundBlock value].
	^messages 
		detect: [ :protocol | protocol messageSelector = selector]
		ifNone: [notFoundBlock value].
!

messageSelectors
	"Answer the set of message specification selectors of the receiver."

	messages isNil ifTrue: [
		^self class defaultMessageCollection].
	^messages collect: [ :msgSpec | msgSpec messageSelector]
!

messages
	"Answer the set of message specifications of the receiver."

	messages isNil ifTrue: [
		^self class defaultMessageCollection].
	^messages copy
!

protocolGroup
	"Answer the protocol group of the receiver."

	protocolGroup isNil ifTrue: [
		^'Unknown' copy].
	^protocolGroup copy
!

protocolId
	"Answer the protocol identifier of the receiver."

	^protocolId
!

sectionAt: sectionHeadingSpec
	"Answer the text of the receiver's specification section titled, sectionHeadingSpec, or an empty text if sectionHeadingSpec not found."

	specSections isNil ifTrue: [
		^String new].
	^specSections
		at: sectionHeadingSpec asSymbol
		ifAbsent: [String new]
!

sectionAt: sectionHeadingSpec put: sectionText
	"Set the receiver's specification section titled, sectionHeadingSpec, to the text, sectionText."

	specSections isNil ifTrue: [
		specSections := self class defaultSpecSectionsCollection].
	specSections
		at: sectionHeadingSpec asSymbol
		put: sectionText
! !

!SysProProtocol methodsFor:'adding'!

addConformsToProtocolId: protocolSpec 
	"Add the protocol with identifier protocolSpec as last in the list of protocols to which the receiver conforms."

	(conformsToProtocolIds includes: protocolSpec) ifFalse: [
		conformsToProtocolIds := conformsToProtocolIds copyWith: protocolSpec].
!

addMessage: newMessage
	"Add the new protocol message, newMessage, to the set of messages specified by the receiver.
	It is an error if the receiver already specifies a message with a selector that matches newMessage's selector."

	messages isNil ifFalse: [
		(messages includes: newMessage) ifTrue: [
			^self error: 'Protocol already specifies a message with selector:', newMessage messageSelector, '.']].
	messages isNil
		ifTrue: [messages := self class defaultMessageCollection].
	messages add: newMessage.
! !

!SysProProtocol methodsFor:'comparing'!

<= comperand
	"Answer true if the receiver is less than or equal to comperand.  Answer false otherwise."

	self == comperand ifTrue: [
		^true].	"No need for lengthy compare if same object."

	^protocolId asLowercase <= comperand protocolId asLowercase
! !

!SysProProtocol methodsFor:'displaying'!

displayOn: target 
	"Write to a stream a textual description of the receiver as a user would want to see it."

	target nextPutAll: protocolId
!

displayString
	"Answer a string that describes the receiver as a user would want to see it."
	| aStream |
	aStream := WriteStream with: String new.
	self displayOn: aStream.
	^aStream contents
! !

!SysProProtocol methodsFor:'enumerating'!

sectionHeadingsAndTextsDo: operation
	"Iteratively evaluate operation with each of the receiver's specification section headings and texts."

	specSections isNil ifTrue: [
		^self].
	specSections keysAndValuesDo: operation
! !

!SysProProtocol methodsFor:'filing'!

storeDefineBehaviorOn: target
	"Store the define-behavior-of-classes of the receiver in chunk format on target.  Not normally used.
	Note: I manually add a chunk terminator as this should not contain embedded terminators that must be doubled."

	(protocolId = self class protocolANYId) ifTrue: [
		^self].
	target
		nextPutAll: SysProProtocolsManager name;
		nextPutAll: ' protocol: ''';
		nextPutAll: protocolId asString;
		nextPut: $'; cr.
	target tab.
	definesBehaviorOfClasses
		ifTrue: [target nextPutAll: 'defineBehaviorOfClasses: ''']
		ifFalse: [target nextPutAll: 'defineBehaviorOfMetaclassesOfClasses: '''].
	self definesBehaviorStringOn: target.
	target nextPutAll: '''!!'; cr; cr.
!

storeOn: target withDefineBehavior: withDefineBehaviorFlag
	"Store the receiver in chunk format on the stream, target, optionally including the define-behavior-to-classes of the receiver depending on withDefineBehaviorFlag.
	Note: I manually add a chunk terminator except for spec. sections.  Only the spec. sections should contain embedded terminators that must be doubled."

	(protocolId = self class protocolANYId) ifTrue: [
		^self].
	target
		nextPutAll: SysProProtocolsManager name;
		nextPutAll: ' newProtocol: ''';
		nextPutAll: protocolId asString;
		nextPut: $'; cr;
		tab; nextPutAll: 'conformsToProtocols: '''.

	self conformsToStringOn: target.
	target
		nextPut: $'; cr;
		tab; nextPutAll: 'protocolGroup: ''';
		nextPutAll: protocolGroup asString;
		nextPutAll: '''!!'; cr; cr.

	withDefineBehaviorFlag ifTrue: [
		self storeDefineBehaviorOn: target].

	specSections isNil
		ifFalse: [	| sortedHeadings |
			sortedHeadings := specSections keys asSortedCollection
				sortBlock: [:x :y | x asLowercase <= y asLowercase].
			sortedHeadings do: [:secHeading|
				| secText secStream |
				secText := specSections at: secHeading.
				target
					nextPutAll: SysProProtocolsManager name;
					nextPutAll: ' protocol: ''';
					nextPutAll: protocolId asString;
					nextPut: $'; cr;
					tab; nextPutAll: 'section: ''';
					nextPutAll: secHeading asString;
					nextPut: $'; cr;
					tab; nextPutAll: 'text: '.
				secStream := WriteStream with: String new.	
				"Insure single qoutes doubled up:"
				secText asString printOn: secStream.
				target nextPutChunk: (secStream contents); cr]].

	messages isNil ifTrue: [
		^self].
	messages asSortedCollection do: [ :protoMsg |
		protoMsg
			storeOn: target
			inProtocolId: protocolId]
! !

!SysProProtocol methodsFor:'printing'!

printOn: target
	"Write to a stream a textual description of the receiver as a developer would want to see it."

	super printOn: target.
	target nextPut: $(;
		space;
		nextPutAll: protocolId;
		space;
		nextPut: $).

"The following is an alternative verbose format:

	target nextPutAll: protocolId.
	target nextPutAll: ' conformsTo['.
	self conformsToStringOn: target.
	definesBehaviorOfClasses
		ifTrue: [target nextPutAll: '] definesClasses[']
		ifFalse: [target nextPutAll: '] definesMetasOf['].
	self definesBehaviorStringOn: target.
	target nextPutAll: '] ('.
	self messageSelectors
		do: [:selector | target nextPut: $#; nextPutAll: selector]
		separatedBy: [target nextPutAll: ', '].
	target nextPut: $)
"
! !

!SysProProtocol methodsFor:'private'!

addAllSelectorAndConformsToSelectorsTo: aDict visited: visitedProtocols 
	"Private - Add all selectors of the receiver as selector key -> protocol value to dictionary, aDict, add the receiver id. to visitedProtocols and visit all protocols to which the receiver conforms not previously visited."

	self messageSelectors do: [:aMessageSelector |
		(aDict includesKey: aMessageSelector)
			ifFalse: [aDict at: aMessageSelector put: protocolId]].
	visitedProtocols add: protocolId.
	conformsToProtocolIds do: [:protoId |
		(visitedProtocols includes: protoId)
			ifFalse: [(self protoMgr protocolId: protoId)
				addAllSelectorAndConformsToSelectorsTo: aDict
				visited: visitedProtocols]]
!

addConformsToProtocolIdsTo: conformsToList
	"Private - Add the identifiers of all protocols to which the receiver conforms including super protocols.  Implementation helper method."
	| protoMgr |
	protoMgr := self protoMgr.
	conformsToProtocolIds do: [:protoId |
		(conformsToList includes: protoId)
			ifFalse: [conformsToList add: protoId]].
	conformsToProtocolIds do: [:protoId |
		(protoMgr protocolId: protoId)
			addConformsToProtocolIdsTo: conformsToList].
!

conformsToStringOn: aStream 
	"Private - Put the identifiers of protocols to which the receiver conforms on aStream in the formal format."

	conformsToProtocolIds
		do: [:protoId | aStream nextPutAll: protoId asString]
		separatedBy: [aStream nextPutAll: ', '].
!

definesBehaviorOfList: classOrMetaclassList isClasses: isClassesBoolean
	"Private - Set the receiver as defining the behavior of the <collection> of <classDescription>s in classOrMetaclassList.  If isClassBoolean is true the receiver defines the behavior of  classes, else that of the metaclasses of the classes in classOrMetaclassList."

	definesBehaviorOfList := classOrMetaclassList asSet.
	definesBehaviorOfClasses := isClassesBoolean
!

definesBehaviorStringOn: aStream 
	"Private - Put the instance class names of the <classDescription>s the receiver defines the behavior of on aStream in the formal format."

	definesBehaviorOfList
		do: [:aCls | aStream nextPutAll: aCls instanceClass name]
		separatedBy: [aStream nextPutAll: ' ']
!

errorSelectorNotFound: selector 
	"Private - Signal a protocol message selector not found error."

	self error: 'Protocol ' , self printString , ' message "' , selector , '" not found'
!

isDefiningBehaviorOfClasses
	"Private - Answer true if the receiver defines the behavior of classes, or false if the metaclasses, in definesBehaviorOfList."

	^definesBehaviorOfClasses = true
!

protoMgr 
	"Private - Answer the singleton system protocol manager."

	^SysProProtocolsManager curMgr
!

setNewProtocolId: protocolIdIn conformsToProtocolIds: conformsToList protocolGroup: groupString
	"Private - Set the initial state of the receiver."

	protocolId := protocolIdIn asSymbol.
	conformsToProtocolIds := (conformsToList collect: [:pId| pId asSymbol]) asArray.
	(groupString isNil or: [groupString isEmpty]) ifFalse: [
		protocolGroup := groupString]
! !

!SysProProtocol methodsFor:'public'!

removeAllSelectors: selectorSpecList
	"Remove from the receiver all the message specifications identified by the selectors in selectorSpecList.  Answer the message specifications removed."
	| resultProtoMsgList |
	resultProtoMsgList := self class defaultMessageCollection.
	selectorSpecList do: [ :selector |
		resultProtoMsgList add: (self removeSelector: selector)].
	^resultProtoMsgList
!

removeSelector: selectorSpec 
	"Remove from the receiver the message specification identified by selectorSpec.  Answer the message specification removed."

	^self
		removeSelector: selectorSpec asSymbol
		ifAbsent: [self errorSelectorNotFound: selectorSpec]
!

removeSelector: selectorSpec ifAbsent: notFoundBlock 
	"Remove from the receiver the message specification identified by selectorSpec or if absent evaluate notFoundBlock.  Answer the message specification removed."
	| aProtoMsg selector |
	messages isNil ifTrue: [
		^notFoundBlock value].
	selector := selectorSpec asSymbol.
	aProtoMsg := messages detect: [ :msgSpec |
		msgSpec messageSelector = selector] ifNone: [
			^notFoundBlock value].
	^messages remove: aProtoMsg
! !

!SysProProtocol class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocol.st,v 1.1 2002/06/21 11:08:19 penk Exp $'
! !
