"{ Package: 'stx:goodies/ansiTests/common' }"

Object subclass:#SysProProtocolsManager
	instanceVariableNames:'allProtocols behaviorProtocolIs'
	classVariableNames:'Current DefaultReturnTypeTable WhitespaceSeparators
		NumericProtocolIds OperatorTable'
	poolDictionaries:''
	category:'System-Protocols'
!


!SysProProtocolsManager class methodsFor:'accessing'!

curMgr
	"Answer the singleton instance of the receiver"

	^Current
!

defaultReturnProtocolIdReceiver: receiver operand: operand 
	"Answer the return value conforms-to protocol identifier of a message performing arithmetic, arithmetic progressions, and conversion on numerical quantities involving receiver and operand."
	| receiverProto operandProto convertedOperandType |
	receiverProto := self numericInstanceProtocolId: receiver.
	operandProto := self numericInstanceProtocolId: operand.
	convertedOperandType := (DefaultReturnTypeTable at: receiverProto)
				at: operandProto.
	convertedOperandType = #'<Fraction>' ifTrue: [
		^#'<rational>'].
	^convertedOperandType
!

numericInstanceProtocolId: number 
	"Answer the conforms-to protocol identifier of number."

	^NumericProtocolIds detect: [:protoId |
		Current
			behavior: number class
			conformsToProtocolId: protoId].
! !

!SysProProtocolsManager class methodsFor:'constants'!

whitespaceSeparators
	"Answer a collection of whitespace separator characters."

	^WhitespaceSeparators copy
! !

!SysProProtocolsManager class methodsFor:'examples'!

exampleCommonTasks
	"The following are tasks comonly performed:"
	"Initialize the list of protocol objects:"
	SysProProtocolsManager curMgr initializeProtocols.
	"Remove protocol object:"

	SysProProtocolsManager newProtocol: '<Bozo>'
	conformsToProtocols: '<Object>'
	protocolGroup: 'Junk'.
	SysProProtocolsManager protocol: '<Bozo>'
	defineBehaviorOfClasses: 'Object'.

	SysProProtocolsManager newProtocol: '<Bozo factory>'
	conformsToProtocols: '<Object class>'
	protocolGroup: 'Junk'.
	SysProProtocolsManager protocol: '<Bozo factory>'
	defineBehaviorOfMetaclassesOfClasses: 'Object'.



	"Answer the list of undefined conforms-to protocol name symbols:"
	SysProProtocolsManager  curMgr undefinedConformsToProtocolIds.
	"Answer the message at selector in protocol with id.:"
	(SysProProtocolsManager  curMgr protocolId: #'<number>')
		messageAtSelector: #'abs'.

	"Answer the list of conforms-to protocol name symbols:"
	(SysProProtocolsManager  curMgr protocolId: #'<Object class>') allConformsToProtocolIds.
	"-> Set (ANY Object instantiator classDescription )"
	"Answer the list of all of selectors which make up the protocol 
	and all protocols to which the it conforms:"
	(SysProProtocolsManager  curMgr protocolId: #'<Object class>') allMessageSelectors.


!

examplesMessagesAndReturnValuesOfProtocolId: protocolId 
	"Answer the a string containing the messages and their corresponding return values of protocol identified by, protocolId.  Example: 
	 
	SystemProtocolsManager current 
	xMessagesAndReturnValuesOfProtocolNamed: #'<instantiator>'
	"
	| protocol aStream |
	aStream := WriteStream with: String new.
	aStream cr.
	protocol := Current protocolId: protocolId.
	protocol messages asSortedCollection do: [:msg | 
		msg messageSelector printOn: aStream.
		aStream nextPutAll: '->'.
		msg isReturnValueSpecByRule
			ifTrue: [aStream nextPutAll: '[rule]']
			ifFalse: [msg returnValueList printOn: aStream].
		aStream cr].
	aStream contents
!

examplesMsgSpecListOfSelector: selector inProtocolId: protocolId 
	"Answer the list of msg specs for msg with selector, selector, in protocol identified by, protocolId.  Example: 
	 
	SystemProtocolsManager current 
	xMsgSpecListOfSelector: #',' 
	inProtocolNamed: #'<exceptionSelector>'
	"
	| msgSpec proto |
	proto := Current protocolId: protocolId.
	msgSpec := proto messageAtSelector: selector.
	^msgSpec isReturnValueSpecByRule
			ifTrue: [msgSpec returnValueWithRule]
			ifFalse: [msgSpec returnValueList]
!

examplesRemoveMsgSpecSelectors: selectorList fromProtocolId: protocolId 
	"Answer the list of msg specs with selectors, selectorList, after removing the msg specs from protocol identified by, protocolId.  Example: 
	SystemProtocolsManager current 
	xRemoveMsgSpecSelectors: #(#'name') 
	fromProtocolNamed: #'<classDescription>'
	"
	| proto |
	proto := Current protocolId: protocolId.
	^proto removeAllSelectors: selectorList
! !

!SysProProtocolsManager class methodsFor:'filing in'!

newProtocol: protocolSpec conformsToProtocols: conformsToSpec protocolGroup: group
	"Answer a new protocol with identifier protocolSpec that conforms to the protocols in conformsToSpec and grouped in group."
	| newProtocol |
	(Current includesProtocolId: protocolSpec) ifTrue: [
		^self error: 'Protocol "' , protocolSpec , '" already exists.'].
	newProtocol := Current
		addNewProtocolId: protocolSpec
		conformsToProtocolIds: conformsToSpec
		group: group.
	^newProtocol
!

protocol: protocolSpec defineBehaviorOfClasses: classNamesList
	"Set the protocol with identifier protocolSpec as defining the behavior of the classes named in classNamesList."
	| classNameList |
	classNameList := (classNamesList subStrings: WhitespaceSeparators)
		collect: [:clsName | clsName].
	Current
		protocolId: protocolSpec
		definesBehaviorOf: classNameList
		isClasses: true		
!

protocol: protocolSpec defineBehaviorOfMetaclassesOfClasses: classNamesList
	"Set the protocol with identifier protocolSpec as defining the behavior of the metaclasses of classes named in classNamesList."
	| classNameList |
	classNameList := (classNamesList subStrings: WhitespaceSeparators)
		collect: [:clsName | clsName].
	Current
		protocolId: protocolSpec
		definesBehaviorOf: classNameList
		isClasses: false
!

protocol: protocolSpec newSelector: selectorSpec parameters: parmList returnRule: returnValueRuleSource
	"Answer a new message (in protocol with identifier protocolSpec.) with selector selectorSpec, parameters parmList, and a return value rule source returnValueRuleSource."
	| newMsg |
	newMsg := SysProProtocolMessage
		newMessageSelector: selectorSpec
		parameters: parmList
		returnRule: returnValueRuleSource.
	(Current protocolId: protocolSpec)
		addMessage: newMsg.
	^newMsg
!

protocol: protocolSpec newSelector: selectorSpec parameters: parmList returnValues: returnValueList
	"Answer a new message (in protocol with identifier protocolSpec.) with selector selectorSpec, parameters parmList, and return values returnValueList."
	| newMsg |
	newMsg := SysProProtocolMessage
		newMessageSelector: selectorSpec
		parameters: parmList
		returnValues: returnValueList.
	(Current protocolId: protocolSpec)
		addMessage: newMsg.
	^newMsg
!

protocol: protocolSpec section: sectionHeading text: sectionText
	"Set the section titled, sectionHeading, to the text, sectionText, in protocol with identifier protocolSpec."
	(Current protocolId: protocolSpec)
		sectionAt: sectionHeading
		put: sectionText
!

protocol: protocolSpec selector: selectorSpec section: sectionHeading text: sectionText
	"Set the section titled, sectionHeading, to the text, sectionText, in message with selector, selectorSpec, in protocol with identifier protocolSpec."
	((Current protocolId: protocolSpec) messageAtSelector: selectorSpec)
		sectionAt: sectionHeading
		put: sectionText
! !

!SysProProtocolsManager class methodsFor:'initializing'!

initialize
	"Class initialization.
	Note: OperatorTable is initialized only when accessed.  Example: 
	 
		SysProProtocolsManager initialize
	"

	self initializeDefaultReturnTypeTable.
	NumericProtocolIds := #(#'<integer>' #'<Float>' #'<Fraction>' #'<scaledDecimal>').
	self initializeWhitespaceSeparators.
	Current := self basicNew.
	Current initializeProtocols. 
! !

!SysProProtocolsManager class methodsFor:'private'!

defaultProtocolIdCollection
	"Private - Answer an <Set>, the default protocol identifier collection."

	^Set new
!

initializeDefaultReturnTypeTable
	"Private - Initialize the two dimensional table for determining the protocol type a message with operand and receiver should return.  Example: 
	 
		SystemProtocolsManager initializeDefaultReturnTypeTable
	"
	| tmp |
	DefaultReturnTypeTable := IdentityDictionary new.
	tmp := IdentityDictionary new.
	tmp at: #'<integer>' put: #'<integer>'.
	tmp at: #'<scaledDecimal>' put: #'<scaledDecimal>'.
	tmp at: #'<Fraction>' put: #'<Fraction>'.
	tmp at: #'<Float>' put: #'<Float>'.
	DefaultReturnTypeTable at: #'<integer>' put: tmp.
	tmp := IdentityDictionary new.
	tmp at: #'<integer>' put: #'<scaledDecimal>'.
	tmp at: #'<scaledDecimal>' put: #'<scaledDecimal>'.
	tmp at: #'<Fraction>' put: #'<Fraction>'.
	tmp at: #'<Float>' put: #'<Float>'.
	DefaultReturnTypeTable at: #'<scaledDecimal>' put: tmp.
	tmp := IdentityDictionary new.
	tmp at: #'<integer>' put: #'<Fraction>'.
	tmp at: #'<scaledDecimal>' put: #'<Fraction>'.
	tmp at: #'<Fraction>' put: #'<Fraction>'.
	tmp at: #'<Float>' put: #'<Float>'.
	DefaultReturnTypeTable at: #'<Fraction>' put: tmp.
	tmp := IdentityDictionary new.
	tmp at: #'<integer>' put: #'<Float>'.
	tmp at: #'<scaledDecimal>' put: #'<Float>'.
	tmp at: #'<Fraction>' put: #'<Float>'.
	tmp at: #'<Float>' put: #'<Float>'.
	DefaultReturnTypeTable at: #'<Float>' put: tmp
!

initializeOperatorTable
	"Private - Initialize the <Dictionary> of Smalltalk operator key and text phrase substitute value used in generating test methods for a protocol test case.
	Note: The operator table is initialized only when accessed, so this must be run if the table is changed.  Example: 
	 
		SystemProtocolsManager initializeOperatorTable
	"

	OperatorTable := IdentityDictionary new.
	OperatorTable at: #'=' put: 'equalityOp'.
	OperatorTable at: #'==' put: 'identityOp'.
	OperatorTable at: #'~=' put: 'notEqualityOp'.
	OperatorTable at: #'~~' put: 'notIdentityOp'.
	OperatorTable at: #'&' put: 'andOp'.
	OperatorTable at: #'|' put: 'orOp'.
	OperatorTable at: #'<' put: 'lessThanOp'.
	OperatorTable at: #'<=' put: 'lessThanOrEqualToOp'.
	OperatorTable at: #'>' put: 'greaterThanOp'.
	OperatorTable at: #'>=' put: 'greaterThanOrEqualToOp'.
	OperatorTable at: #'*' put: 'multiplyOp'.
	OperatorTable at: #'+' put: 'addOp'.
	OperatorTable at: #'-' put: 'subtractOp'.
	OperatorTable at: #'/' put: 'divideOp'.
	OperatorTable at: #'//' put: 'integerDivideOp'.
	OperatorTable at: #'\\' put: 'remainderIntegerDivideOp'.
	OperatorTable at: #',' put: 'concatenateOp'
!

initializeWhitespaceSeparators
	"Private - Initialize the whitespace separators.  Example: 
	 
		SystemProtocolsManager initializeWhitespaceSeparators
	"
	WhitespaceSeparators := Array
		with: Character cr
		with: Character lf 
		with: Character space
		with: Character tab.
!

operatorTable
	"Private - Answer a <Dictionary> of Smalltalk operator key and text phrase substitute value used in generating test methods for a protocol test case."

	OperatorTable isNil ifTrue: [self initializeOperatorTable].
	^OperatorTable
! !

!SysProProtocolsManager class methodsFor:'public'!

testMethdodNameFrom: messageSelector
	"Answer a generated test suite stub method name from messageSelector. fixNum which may be incremented if a test methdod name is generated that must be fixed up by hand."
	| testSelName fixNum |
	messageSelector isBinarySelector
		ifTrue: [(self operatorTable includesKey: messageSelector)
			ifTrue: [testSelName := 'testX' , (self operatorTable at: messageSelector)]
			ifFalse: 
				[fixNum := fixNum + 1.
				testSelName := 'testXfixOp' , fixNum printString]]
		ifFalse: [testSelName := 'testX' ,
				(messageSelector asString collect: [:char | char = $:
					ifTrue: [$X]
					ifFalse: [char]])].
	^testSelName
!

wordsUppercaseFromProtocolId: protocolSpec
	"Answer a <String> from protocolSpec with whitespace, <, and > removed and words in uppercase."
	| wordsUppercase  |
	wordsUppercase := String new.
	((protocolSpec copyFrom: 2 to: (protocolSpec size - 1))
		subStrings: SysProProtocolsManager whitespaceSeparators)
	do: [:protoWord| 
		protoWord at: 1 put: (protoWord at: 1) asUppercase.
		wordsUppercase := wordsUppercase , protoWord].
	^wordsUppercase
!

wrkGenHelperAndTestClassesForProtocolIdList: protoIdList
	"Generate a test class and required helper classes for protocols with identifiers in protoIdList.  Note: no existing classes or methods are lost.  Example use:

	SysProProtocolsManager wrkGenHelperAndTestClassesForProtocolIdList:
		#('<Array factory>' '<Array>' '<boolean>')."
	| conformsToProtoIds protoIds |
	Transcript cr; nextPutAll: '***********'; cr.
	protoIdList do: [:protoId |
		self wrkGenTestClassForProtocolId: protoId].

	conformsToProtoIds := Set new.
	protoIdList do: [:cProtoId |	| proto |
		proto := Current protocolId: cProtoId.
		conformsToProtoIds addAll: (proto allConformsToProtocolIds)].
	conformsToProtoIds
		remove: (SysProProtocol protocolANYId)
		ifAbsent: [].
	conformsToProtoIds do: [:conToProtoId |
	 	self wrkGenHelperClassForProtocolId: conToProtoId].
!

wrkGenHelperClassForProtocolId: protocolId 
	"Private - Generate helper class with a suite of stub methods for the directly specified messages of protocol identified by, protocolName."
	| protocol helperClass |
	protocol := Current protocolId: protocolId.
	helperClass := self wrkHelperClassForProtocolId: protocolId.
	self
		wrkXprotocolIdXin: helperClass
		protocolId: protocolId.
	protocol messageSelectors do: [:msgSel |
		self
			wrkXselector: msgSel
			in: helperClass
			protocol: protocol].
!

wrkGenTestClassForProtocolId: protocolId
	"Private - Generate test class with a suite of stub methods for the directly specified messages of protocol identified by, protocolName."
	| protocol testClass |
	protocol := Current protocolId: protocolId.
	testClass := self wrkTestClassForProtocol: protocolId.
	self wrkXprotocolIdXin: testClass protocolId: protocolId.
	protocol messageSelectors do: [:msgSel |
		self
			wrkXselector: msgSel
			in: testClass
			protocol: protocol].
!

wrkHelperClassForProtocolId: protocolId 
	"Private - Answer a test helper class for the protocol identified by, protocolId."
	| protocol helperClass helperClassName |
	protocol := Current protocolId: protocolId.
	helperClassName := ((self wordsUppercaseFromProtocolId: protocolId), 'Helper') asSymbol.
	helperClass := Smalltalk
		at: helperClassName
		ifAbsent: [
			Transcript nextPutAll: ('Creating ', helperClassName); cr.
			(Smalltalk at: #'TestCaseHelper')
				subclass: helperClassName
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'Tests-ANSI-' , protocol protocolGroup].
	^helperClass
!

wrkTestClassForProtocol: protocolId 
	"Private - Answer a test main class for the protocol identified by, protocolId."
	| testClassName protocol testClass |
	protocol := Current protocolId: protocolId.
	testClassName := ((self wordsUppercaseFromProtocolId: protocolId), 'Test') asSymbol.
	testClass := Smalltalk
		at: testClassName
		ifAbsent: [
			Transcript nextPutAll: ('Creating ', testClassName); cr.
			(Smalltalk at: #'MainTestCase')
				subclass: testClassName
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: 'Tests-ANSI-', protocol protocolGroup].
	^testClass
!

wrkXprotocolIdXin: testClass protocolId: protocolId 
	"Private - Generate #protocolId: class method in ttestClass for protocolId."
	| catClass categories textStream testSelector |
	testSelector := 'protocolId'.
	(testClass class includesSelector: (testSelector asSymbol)) ifTrue: [
		^self].
	textStream := WriteStream with: String new.	
	textStream 
		nextPutAll: testSelector; cr;
		tab; nextPutAll: '"Answer the protocol identifier of the protocol the receiver tests."'; cr;
		cr;
		tab; nextPutAll: '^#''';
		nextPutAll: protocolId asString;
		nextPut: $';
		cr.

	catClass := self methodCategoryClass.
	categories := Array
		with: (catClass name: 'constants')
		with: catClass public.
	testClass class
		compile: textStream contents
		categories: categories.
!

wrkXselector: msgSel in: testClass protocol: protocol 
	"Private - Generate msgSel testselector method instance method in testClass for protocolId."
	| catClass categories textStream protocolId testSelector |
	testSelector := self testMethdodNameFrom: msgSel.
	(testClass includesSelector: (testSelector asSymbol)) ifTrue: [
		^self].
	protocolId := protocol protocolId.
	textStream := WriteStream with: String new.	
	textStream 
		nextPutAll: testSelector; cr;
		tab; nextPutAll: '" ', protocol protocolGroup, ' ', protocolId, ' "'; cr;
		tab; nextPutAll: 'self messageSelector: #''', msgSel, '''.'; cr; cr;
		tab; nextPutAll: 'self notDone.'; cr.

	catClass := self methodCategoryClass.
	categories := Array
		with: (catClass name: 'testing')
		with: catClass public.

	testClass
		compile: textStream contents
		categories: categories.
! !

!SysProProtocolsManager methodsFor:'accessing'!

allProtocolIds
	"Answer the identifiers of all protocols in the system."

	^(allProtocols collect: [:proto | proto protocolId]) asSet
!

allProtocolIdsSorted
	"Answer the identifiers of all protocols in the system sorted ignoring case."

	^(allProtocols collect: [:proto | proto protocolId]
	) asSortedCollection sortBlock: [:x :y | x asLowercase <= y asLowercase]
!

allProtocols
	"Answer all protocols in the system."

	^allProtocols
!

behavior: classOrMetaclass conformsToProtocolId: protocolSpec
	"Answer whether classOrMetaclass, or any class from which it inherits, conforms to the protocol identified by protocolSpec."
	| protoId behavior |
	protoId := protocolSpec asSymbol.
	protoId = SysProProtocol protocolANYId ifTrue: [
		^true].	"Note this is optimization to avoid going up to Object."

	behavior := classOrMetaclass.
	[behavior notNil] whileTrue: [	| behaviorProtoList |
		behaviorProtoList := behaviorProtocolIds
			at: behavior
			ifAbsent: [nil].
		behaviorProtoList isNil ifFalse: [
			(behaviorProtoList includes: protoId) ifTrue: [
				^true]].
		behavior := behavior superclass].
	^false
!

protocolId: protocolSpec 
	"Answer the protocol with identifier protocolSpec."

	^self
		protocolId: protocolSpec
		ifNone: [self protocolIdNotFound: protocolSpec]
!

protocolId: protocolSpec ifNone: notFoundBlock
	"Answer the protocol with identifier protocolSpec or if not found evaluate notFoundBlock."
	| protoId |
	protoId := protocolSpec asSymbol.
	^allProtocols detect: [ :proto | proto protocolId = protoId]
		ifNone: [notFoundBlock value]
!

undefinedConformsToProtocolIds
	"Answer all the undefined conforms-to protocol identifiers in the system."
	| undefConformsToProtoIds |
	undefConformsToProtoIds := self class defaultProtocolIdCollection.
	allProtocols do: [ :proto | 
		"Add all not-yet-defined conforms-to protocols:"
		proto conformsToProtocolIds do: [ :protoId | 
			(self includesProtocolId: protoId) ifFalse: [
				undefConformsToProtoIds add: protoId]].
		proto messages do: [ :protoMsg |
			"Add all not-yet-defined message parameter protocols:"
			protoMsg parameterList do: [ :parm |
				parm parmProtocolIds do: [ :protoId |
					(self includesProtocolId: protoId) ifFalse: [
						undefConformsToProtoIds add: protoId]]].
			"Add all not-yet-defined message return value protocols:"
			protoMsg isReturnValueSpecByRule ifFalse: [
				protoMsg returnValueList do: [ :returnValue |
					| retValProtolId |
					retValProtolId := returnValue returnValueProtocolId.
					(self includesProtocolId: (retValProtolId)) ifFalse: [
						undefConformsToProtoIds add: retValProtolId]]]]].
	^undefConformsToProtoIds
! !

!SysProProtocolsManager methodsFor:'filing'!

fileOutDefineBehaviorOfProtocolIds: protoIdList to: fileId
	"File out the define-behavior-to-classes of the protocols with identifiers in protoIdList to the file named fileId.  Not normally used."
	| aFileStream |
	aFileStream := FileStream write: fileId.
	[self
		storeDefineBehaviorOfProtocolIds: protoIdList
		on: aFileStream
	] ensure: [aFileStream close]
!

fileOutProtocolIds: protoIdList withDefineBehavior: withDefineBehaviorFlag to: fileId
	"File out the protocols with identifiers in protoIdList to the file named fileId, optionally including the define-behavior-to-classes of the protocols depending on withDefineBehaviorFlag."
	| aFileStream |
	aFileStream := FileStream write: fileId.
	[self
		storeProtocolIds: protoIdList
		withDefineBehavior: withDefineBehaviorFlag
		on: aFileStream
	] ensure: [aFileStream close]
!

storeDefineBehaviorOfProtocolIds: protoIdList on: target
	"Store the define-behavior-to-classes of the protocols with identifiers in protoIdList on stream, target.  Not normally used."

	protoIdList do: [:protoId |
		(self protocolId: protoId) storeDefineBehaviorOn: target].
!

storeProtocolIds: protoIdList withDefineBehavior: withDefineBehaviorFlag on: target
	"Store the protocols with identifiers in protoIdList on stream, target, optionally including the define-behavior-to-classes of the protocols depending on withDefineBehaviorFlag."

	protoIdList do: [:protoId |
		(self protocolId: protoId)
			storeOn: target
			withDefineBehavior: withDefineBehaviorFlag]
! !

!SysProProtocolsManager methodsFor:'private'!

addNewProtocolId: protocolSpec conformsToProtocolIds: conformsToString group: groupString
	"Private - Answer a new protocol with identifier protocolSpec that conforms to the protocols in conformsToString and grouped in groupString."
	| newProto |
	newProto := SysProProtocol
		newProtocolId: protocolSpec
		conformsToProtocols: conformsToString
		protocolGroup: groupString.
	allProtocols add: newProto.
	^newProto
!

initializeProtocols
	"Private - Initialize instance variables and add <ANY>."
	| protocolANY |
	allProtocols := Set new.
	behaviorProtocolIds := Dictionary new.

	protocolANY := SysProProtocol
		newProtocolId: SysProProtocol protocolANYId
		conformsToProtocols: Array new
		protocolGroup: 'ANSI-Special'.
	protocolANY
		sectionAt: 'Description'
		put: 'A protocol to which all other protocols conform.'.
	allProtocols add: protocolANY.
	protocolANY
		definesBehaviorOfList: (Array with: Object)
		isClasses: true.

!

protocolId: protocolSpec definesBehaviorOf: classNameList isClasses: isClassBoolean		
	"Private - Set the protocol with identifier protocolSpec as defining the behavior of the set of <classDescription>s named in classNamesList.  If isClassBoolean is true the protocol defines the behavior of  classes, else that of the metaclasses of the classes named in classNamesList.
classNameList is a <collection> of <readableString>s."
	| protocol protocolId classDescList |
	protocolId := protocolSpec asSymbol.
	classDescList := classNameList collect: [:clsName |
		isClassBoolean
			ifTrue: [Smalltalk at: (clsName asSymbol)]
			ifFalse: [(Smalltalk at: (clsName asSymbol)) class]].
	classDescList do: [:classOrMeta |	| protoIdList | 
		protoIdList := behaviorProtocolIds
			at: classOrMeta
			ifAbsent: [self class defaultProtocolIdCollection].
		protoIdList add: protocolId.
		behaviorProtocolIds
			at: classOrMeta
			put: protoIdList].

	protocol := self protocolId: protocolId.
	protocol
		definesBehaviorOfList: classDescList
		isClasses: isClassBoolean.
!

protocolIdNotFound: protocolSpec
	"Signal an error for protocol with identifier protocolSpec not found."

	self error: 'Protocol "' , protocolSpec, '" not found.'
! !

!SysProProtocolsManager methodsFor:'public'!

includesProtocolId: protocolSpec 
	"Answer whether the protocol with identifier protocolSpec exists."
	| protoId |
	protoId := protocolSpec asSymbol.
	allProtocols detect: [ :proto | proto protocolId = protoId] ifNone: [
		^false].
	^true
!

removeProtocolId: protocolSpec
	"Remove the protocol with identifier, protocolSpec, from the system."
	| targetProtocol protoId behaviorList |
	protoId := protocolSpec asSymbol.
	protoId = SysProProtocol protocolANYId ifTrue: [
		^self error: 'Protocol ' , protoId , ' can not be removed.'].
	targetProtocol := allProtocols remove: (self protocolId: protoId).
	behaviorList := targetProtocol definesBehaviorOfList.
	behaviorList notNil ifTrue: [
		behaviorList do: [ :classOrMetaclass |	| protoList |
			protoList := behaviorProtocolIds
				at: classOrMetaclass
				ifAbsent: [nil].
			protoList notNil ifTrue: [
				protoList remove: protoId.
				protoList isEmpty ifTrue: [
					behaviorProtocolIds removeKey: classOrMetaclass]]]].
!

renameProtocolId: oldProtocolSpec to: newProtocolSpec 
	"Rename the protocol identified by oldProtocolSpec to have the new identifier, newProtocolSpec."
	| newProto oldProto conformsTos targetStream classNameList |
	(self includesProtocolId: newProtocolSpec) ifTrue: [
		^self error: 'Duplicate protocol name: "' , newProtocolSpec , '".'].
	oldProto := self protocolId: oldProtocolSpec.
	self removeProtocolId: oldProtocolSpec.
	targetStream := WriteStream with: String new.
	oldProto conformsToProtocolIds
		do: [:protoId | targetStream nextPutAll: protoId asString]
		separatedBy: [targetStream nextPutAll: ', '].
	conformsTos := targetStream contents.
	newProto := SysProProtocol
		newProtocolId: newProtocolSpec
		conformsToProtocols: conformsTos
		protocolGroup: oldProto protocolGroup.
	allProtocols add: newProto.
	oldProto sectionHeadingsAndTextsDo: [:secHeading :secText |
		newProto
			sectionAt: secHeading
			put: secText].
	"New protocol defines the behavior of classes:"
	classNameList := oldProto definesBehaviorOfList
		collect: [:clsOrMetacls| clsOrMetacls instanceClass name].
	self
		protocolId: newProtocolSpec
		definesBehaviorOf: classNameList
		isClasses: oldProto isDefiningBehaviorOfClasses.		
! !

!SysProProtocolsManager class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/SysProProtocolsManager.st,v 1.1 2002/06/21 11:07:08 penk Exp $'
! !
SysProProtocolsManager initialize!
