"{ Package: 'stx:goodies/ansiTests/common' }"

TestCase subclass:#TestCaseUsingProtocols
	instanceVariableNames:'messageSelector protocolId'
	classVariableNames:'ProtoMgr SignalAnomalies DialectAndVersion SignalNotDone'
	poolDictionaries:''
	category:'SUnit-Protocol-Framework'
!


!TestCaseUsingProtocols class methodsFor:'accessing'!

testMethods
	"Answer a list of test class test method selectors.  This is the default filter for selecting test methods to run."

	^self selectors select: [:each| 'test*' match: each].
! !

!TestCaseUsingProtocols class methodsFor:'constants'!

protoMgr
	"Answer the singleton instance of the System Protocols manager."

	^SysProProtocolsManager curMgr
!

protocolId
	"Dummy for non-protocol subclasses.  Protocol subclasses must override this method."

	^#'<>'
! !

!TestCaseUsingProtocols class methodsFor:'initializing'!

initialize
        "Class initialization.  Example: 
         
                TestCaseUsingProtocols initialize"

        "Set known anomalies (enclosed in anomaly block) in test methods
        to signal an error:"
        self signalAnomalies: false.

        "Set unimplemented test methods to signal an error:"
        self signalNotDone: false.

        "Set <symbol> indicating this dialect and version:"
        DialectAndVersion := PU dialectAndVersionTag.

        "Set the singleton instance of the System Protocols manager:"
        ProtoMgr := self protoMgr.
! !

!TestCaseUsingProtocols class methodsFor:'options'!

signalAnomalies: aBoolean
	"Set whether code with anomalies in test methods are noted or not.  Example:

		TestCaseUsingProtocols signalAnomalies: false
		TestCaseUsingProtocols signalAnomalies: true"

	SignalAnomalies := aBoolean
!

signalNotDone: aBoolean
	"Set whether unimplemented test methods are noted or not.  Example:

		TestCaseUsingProtocols signalNotDone: false
		TestCaseUsingProtocols signalNotDone: true"

	SignalNotDone := aBoolean
! !

!TestCaseUsingProtocols class methodsFor:'private'!

addHelperConformsToTo: newTestSuite attachedTo: aMainTestClass coveredTests: coveredTestSelectors visited: coveredProtocolIds
	"Private - Add the conforms-to helper tests for the receiver's protocol.
	Coverage of all selectors a protocol conforms-to is done breadth first.  Tests for uncovered selectors of the immediate conforms-to protocols are added, then those of uncovered selectors of the conforms-to protocol's conforms-to protocol, etc.
	IMPORTANT: I am not certain breadth first is what the ANSI standard expects.  Depth first may be correct."
	| doHelperList |
	doHelperList := IdentityDictionary new.
	(ProtoMgr protocolId: (self protocolId))
	conformsToProtocolIds do: [:conformsToProtoId|
		(coveredProtocolIds includes: conformsToProtoId) ifFalse: [
			| helperClass |
			helperClass := TestCaseHelper allSubclasses
				detect: [:subcls| subcls protocolId = conformsToProtoId]
				ifNone: [self error: 'Helper for ', conformsToProtoId,
						' not implemented.'].
			"Add uncovered test selectors to TestSuite."
			helperClass testMethods do: [:testSelector|
				(coveredTestSelectors includes: testSelector) ifFalse: [
					newTestSuite addTest: (helperClass
						selector: testSelector
						attachedTo: aMainTestClass
						conformsToProtocolId: conformsToProtoId).
					coveredTestSelectors add: testSelector]].
			coveredProtocolIds add: conformsToProtoId.
			doHelperList
				at: conformsToProtoId
				put: helperClass]].
	doHelperList keysAndValuesDo: [:conformsToProtoId2 :helperClass2|
		helperClass2
			addHelperConformsToTo: newTestSuite
			attachedTo: aMainTestClass
			coveredTests: coveredTestSelectors
			visited: coveredProtocolIds].
!

signalAnomalies
	"Private - Answer whether code with anomalies in test methods are noted or not."

	^SignalAnomalies
!

signalNotDone
	"Private - Answer  whether unimplemented test methods are noted or not."

	^SignalNotDone
! !

!TestCaseUsingProtocols class methodsFor:'running'!

runTestCase: targetTestSelector
	"Run the test case testing method with the selector targetTestSelector."
	| test |
	test := self suite tests
		detect: [:aTestCase| aTestCase testSelector = targetTestSelector]
		ifNone: [
			^self error: 'No "#', targetTestSelector,
				'" test method found.'].
	test run hasPassed ifFalse: [
		test runCaseAsFailure].
! !

!TestCaseUsingProtocols class methodsFor:'testing'!

isAbstract
	"For TestCaseUsingProtocols, and TestCaseHelper and subclasses default is true so no TestCase instances built from it.
	Override for MainTestCase and abstract subclasses to answer true so no TestCase instances built from it, false for main tests that should build TestCase instances."

	^true
! !

!TestCaseUsingProtocols methodsFor:'accessing'!

messageSelector: selectorSpec
	"Set the selector of the protocol message about to be tested to a <symbol> from the <readableString> selectorSpec."

	messageSelector := selectorSpec asSymbol
!

testSelector
	"Answer a <symbol>, the selector of the test case test method to be tested.  Note: Accesses the TestCase variable."

	^testSelector
! !

!TestCaseUsingProtocols methodsFor:'asserting'!

conformingAssertMessageSelector 
	"Record a failure if the result of sending the message being tested to a typical instance (canonicalObject) evaluates to an object that does not conform to the return value specified for the message, messageSelector, in protocol with protocolId.
	See Also: TestCaseHelper>>#conformingAssertMessageSelector.
	Important: the message selector must first be set.
	Also note that this test really says nothing other than this message is implemented and the result of send it is of the specified type. 
	Example testing ANSI-Numeric (<integer>, #'even'):

	self messageSelector: #'even'.
	self conformingAssertMessageSelector."

	self conformingValue: [self canonicalObject perform: messageSelector]
		should: [:result| true].
!

conformingMessageSelectorBehavesLike: arrayOfTriplets 
	"Record a failure if (for each triplet in the arrayOfTriplets) the result of sending the message being tested to the triplet targetObject (with triplet arguments) evaluates to an object that does not conform to the return value specified for the message, messageSelector, in protocol with protocolId, or the result does not equal the triplet resultObject.
	Important: the message selector must first be set.
	Example testing Fundamental (<Character>, #'isDigit'):

	self messageSelector: #'isDigit'.

	self conformingMessageSelectorBehavesLike: #(#($1 true) #($A false) #($B false) #($b false) #($: false)).

	Note: Since there are no arguments to the message send, the triplet arguments can be omitted."

	arrayOfTriplets asANSITestArray do: [:triplet|
		| targetObject arguments resultObject |
		targetObject := triplet first.
		arguments := triplet
			copyFrom: 2
			to: (triplet size - 1).
		resultObject := triplet last.
		self conformingValue: [
				targetObject
					perform: messageSelector
					withArguments: arguments]
			should: [:r| r = resultObject]]
!

conformingValue: opBlock should: shouldBlock
	"Record a failure if shouldBlock evaluates to false, or opBlock evaluates to an object that does not conform to the return value specified for the message, messageSelector, in protocol with protocolId.  Example testing (<number>, #'='):

	self conformingValue: [2 = 2]
		should: [:result | result]"
	| protocolMsg opResult opResultClass |
	opResult := opBlock value.
	self assert: (shouldBlock value: opResult).
	protocolMsg := self
		msg: messageSelector
		inProtocol: protocolId.
	protocolMsg isReturnValueUnspecified ifTrue: [
		^self].
	opResultClass := opResult class.
	protocolMsg returnValueList do: [:msgReturn|
		(ProtoMgr
			behavior: opResultClass
			conformsToProtocolId: msgReturn returnValueProtocolId
		) ifTrue: [
			^true]].
	self signalFailure: ('-> ', opResult printString, ' does not conform to ', protocolMsg returnValueList displayString).
!

conformingValue: opBlock should: shouldBlock ruleReceiver: receiver 
	"Record a failure if shouldBlock evaluates to false, or opBlock evaluates to an object that does not conform to the protocol with the identifier returned by evaluating the <monadicBlock> rule block (with the receiver as a parameter) for message, messageSelector, in protocol with protocolId.  Example testing (<number>, #'integerPart'):

	| negFlt2 |
	negFlt2 := -2.0. 
	self conformingValue: [negFlt2 integerPart]
		should: [:result | result = -2.0] 
		ruleReceiver: negFlt2."
	| protocolMsg opResult msgReturnProtocolId |
	opResult := opBlock value.
	self assert: (shouldBlock value: opResult).
	protocolMsg := self
		msg: messageSelector
		inProtocol: protocolId.
	msgReturnProtocolId := protocolMsg returnValueWithRule
		returnProtocolIdRuleReceiver: receiver.
	(ProtoMgr
		behavior: opResult class
		conformsToProtocolId: msgReturnProtocolId
	) ifFalse: [	self signalFailure: (receiver printString, ' ',
				messageSelector,
				' -> ', opResult printString,
				' does not conform to ', msgReturnProtocolId)].
!

conformingValue: opBlock should: shouldBlock ruleReceiver: receiver operand: operand
	"Record a failure if shouldBlock evaluates to false, or opBlock evaluates to an object that does not conform to the protocol with the identifier returned by evaluating the two parameter rule block (with the receiver and operand) for message, messageSelector, in protocol with protocolId.
	Example testing (<number>, #'*'):

	| negFlt2 posInt2 |
	negFlt2 := -2.0.
	posInt2 := 2.
	self conformingValue: [negFlt2 * posInt2]
		should: [:result | result = -4.0] 
		ruleReceiver: negFlt2
		operand: posInt2."
	| protocolMsg opResult msgReturnProtocolId |
	opResult := opBlock value.
	self assert: (shouldBlock value: opResult).
	protocolMsg := self
		msg: messageSelector
		inProtocol: protocolId.
	msgReturnProtocolId := protocolMsg returnValueWithRule
		returnProtocolIdRuleReceiver: receiver
		operand: operand.
	(ProtoMgr
		behavior: opResult class
		conformsToProtocolId: msgReturnProtocolId
	) ifFalse: [	self signalFailure: (receiver printString, ' ',
				messageSelector,
				' ', operand printString, ' -> ', opResult printString,
				' does not conform to ', msgReturnProtocolId)].
!

conformingValue: opBlock shouldnt: shouldNotBlock
	"Record a failure if shouldBlock evaluates to true, or opBlock evaluates to an object that does not conform to the return value specified for the message, messageSelector, in protocol with protocolId.
	Example testing (<number>, #'='):

	self conformingValue: [2 = 3]
		should: [:result| result]"

	self
		conformingValue: opBlock
		should: [:result | (shouldNotBlock value: result) not] 
! !

!TestCaseUsingProtocols methodsFor:'comparing'!

compareStreamIgnoringLineTermTarget: targetStream result: resultStream
	"Answer a <boolean> indicating whether the contents of two streams are equal ignoring platform line termination differences.  Note: Extremely ugly.  Actually it compares line by line not including line termination.  Example:

	self should: [
			self compareStreamIgnoringLineTermTarget: targetStream
				result: resultStream]."

	resultStream reset.
	[targetStream atEnd and: [resultStream atEnd]] whileFalse: [
		| resultLine targetLine |
		resultLine := resultStream nextLine.
		targetLine := targetStream nextLine.
		resultLine = targetLine ifFalse: [
			^false]].
	^true

!

compareTextIgnoringLineTermTarget: targetText result: resultText
	"Answer a <boolean> indicating whether two <String>s are equal ignoring platform line termination differences.  Note: Extremely ugly.  Actually it compares line by line not including line termination.  Example:

	text := 'Line 1
Line 2'.
	self should: [
			self compareTextIgnoringLineTermTarget: text
				result: text copy]."
	| targetStream resultStream |
	targetStream := ReadStream on: targetText.
	resultStream  := ReadStream on: resultText.

	^self
		compareStreamIgnoringLineTermTarget: targetStream
		result: resultStream
! !

!TestCaseUsingProtocols methodsFor:'constants'!

fileNameXTestFileXjnk
	"Default TestCaseUsingProtocols test file name."

	^'TestFile.jnk' copy
!

lineDelimiter
	"Answer a <String> containing the host system end-of-line character sequence (Dolphin - a Cr/Lf pair).  Dialect specific."

	^String with: Character cr with: Character lf
! !

!TestCaseUsingProtocols methodsFor:'enumerating'!

numbers: numbers byNumbers: byNumbers answers: answers do: opBlock
	"Tests all type combinations of receiver and operand of a numeric message with each in numbers by evaluating opBlock with each in byNumbers verifying the correct result, answers.  See additional remarks below code."
	| numNdx byNdx |
	numNdx := 0.
	numbers do: [:number |
		numNdx := numNdx + 1.
		byNdx := 0.
		byNumbers do: [:byNumber | | answer |
			byNdx := byNdx + 1.
			answer := (answers at: numNdx) at: byNdx.
			opBlock
				value: number
				value: byNumber
				value: answer]].

"Additional remarks:
	numbers - a list of typical receiver values of one numeric type
		(example <integer>).
	byNumbers - a list of interesting operand values varying by all
		numeric types (example <integer>, <Float>, <ScaledDecimal>, etc.).
	answers - a list of lists of correct results.
	opBlock - a 3 parameter block (number, byNumber, and answer)
		normally a conforming assertion.

	Note: Complete coverage is achieved by all numeric protocols tested (automatically) with all combinations in byNumbers.

	Implementation: This method may be made better if refactored to more closely resemble #conformingMessageSelectorBehavesLike:.

	Example:
	'' Numeric <number> ''
	| canonicalNumbers mixedNums answers |
	self messageSelector: #'+'.

	''Limited numeric types, and block assert, to simplify example:''
	canonicalNumbers := #(2 0).
	mixedNums := #(0 2 2.0d0).
	answers := #( #(2 4 4.0d0) #(0 2 2.0d0) ).
	self numbers: canonicalNumbers
		byNumbers: mixedNums
		answers: answers
		do: [ :canonicalNumber :mixedNumber :answer|
			self conformingValue: [canonicalNumber + mixedNumber]
				should: [:r| r closeTo: answer]
				ruleReceiver: canonicalNumber
				operand: mixedNumber].
"
!

objects: objects answers: answers do: opBlock
	"Iterates over two matching lists, objects and answers.  See additional remarks below code."

	1 to: objects size do: [:ndx|
		opBlock
			value: (objects at: ndx)
			value: (answers at: ndx)].

"Additional remarks:

	objects	- a list of typical receivers.
	answers	- a list of correct results.

Implementation:
This just captures the general case of a common list of canonical objects reused with a specific message's results.  This method may be made better if refactored to more closely resemble #conformingMessageSelectorBehavesLike:.  For me this method did simplify the tests, but I think it still needs work.

Example:

	'' Numeric <number> ''
	| numbers answers |
	self messageSelector: #'abs'.

	''Limited numeric types to simplify example:''
	numbers := #(0 2 0.0d0 2.0d0 -2.0d0).
	answers := #(0 2 0.0d0 2.0d0  2.0d0).
	self objects: numbers answers: answers do: [:number :answer|
		self conformingValue: [number abs]
			should: [:r| r = answer]].
"
!

objects: objects1 objects: objects2 answers: answers do: opBlock
	"Iterates over three matching lists, objects1, objects2 and answers.  See additional remarks below code."

	1 to: objects1 size do: [:ndx|
		opBlock
			value: (objects1 at: ndx)
			value: (objects2 at: ndx)
			value: (answers at: ndx)].

"Additional remarks:

	objects1	- a list of typical receivers.
	objects2	- a list of values (in example receiver scale parameter).
	answers	- a list of correct results.

Implementation:
This just captures the general case of a common list of canonical objects reused with interesting parameters and a specific message's results.  This method may be made better if refactored to more closely resemble #conformingMessageSelectorBehavesLike:.  For me this method did simplify the tests, but I think it still needs work.

Example:

	'' Numeric <number> ''
	| numbers scales answers |
	self messageSelector: #'asScaledDecimal:'.

	''Limited numeric types, and block assert, to simplify example:''
	numbers := #(0 2 0.0d0 2.0d0 -2.0d0).
	scales := #(1 2).
	answers := OrderedCollection new.
	answers
		add: ScaledDecimal zero; add: (2 asScaledDecimal: 1);
		add: ScaledDecimal zero; add: (2.000d0 asScaledDecimal: 3);
		add: (-2.0000d0 asScaledDecimal: 4).

	self objects: numbers objects: scales answers: answers do: [
			:number :numberScale :answer|
		self conformingValue: [number asScaledDecimal: numberScale]
			should: [:r| r = answer]].
"
! !

!TestCaseUsingProtocols methodsFor:'error handling'!

anomaly: anomalousBlock for: dialectList 
	"Depending on SignalAnomalies selectively ignore SUnit test code in anomalousBlock that has some anomaly for the dialects and versions indicated with their <symbol> in dialectList.  See class>>#dialectAndVersion for recommended <symbol> of dialects and versions.
	Options are initially set to signal anomalies.  It is just easier sometimes to ignore non-working tests yet not miss anomalies one has not dealt with in a clutter of known anomalies.  See Also: TestCaseHelper>> #anomaly:forDialectAndTestCaseName:.  Example:

	self anomaly: [
		obj aMsg
	] for: #(#'DO30' ''2000/12/06 Harmon, R. Message not implemented''
		#'SQ27' ''2000/12/06 Harmon, R. Returns nil instead of true'')."

	SignalAnomalies ifTrue: [
		^anomalousBlock value].
	(dialectList includes: DialectAndVersion) ifFalse: [
		^anomalousBlock value].
!

notDone
	"If SignalNotDone is true signal an error for a method that is not done, else ignore."

	SignalNotDone ifTrue: [self error: 'not done']
! !

!TestCaseUsingProtocols methodsFor:'private'!

msg:  selectorSpec inProtocol: protocolSpec
	"Private - Answer the message specification, identified by selectorSpec, in the protocol with identifier protocolSpec."
	| protocolMsg newEx protocol |
	protocol := ProtoMgr protocolId: protocolSpec ifNone: [
		^self signalFailure: 'Protocol "', protocolSpec, '" not found.'].
	protocolMsg := protocol messageAtSelector: selectorSpec ifNone: [
		^self signalFailure:  selectorSpec, ' not in protocol ', protocolSpec].
	^protocolMsg
! !

!TestCaseUsingProtocols methodsFor:'running'!

debug
	"Run the receiver as a test case with a error.  The TestCase method is overridden as it uses a TestCase instance creation method, rather than a main or helper instance creation method."

	self runCase
!

debugAsFailure
	"Run the receiver as a test case with a failure.  The TestCase method is overridden as it uses a TestCase instance creation method, rather than a main or helper instance creation method."

	self runCaseAsFailure
! !

!TestCaseUsingProtocols class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/ansiTests/common/TestCaseUsingProtocols.st,v 1.1 2002/06/21 10:34:42 penk Exp $'
! !
TestCaseUsingProtocols initialize!
