"{ NameSpace: Benchmarks }"

Object subclass:#DBMethod
	instanceVariableNames:'outIndex block'
	classVariableNames:''
	poolDictionaries:''
	category:'benchmarks-DeltaBlue'
!

DBMethod comment:'I represent a method whose enforement procedure is stored in a Smalltalk block. Users may create custom methods by supplying an assignment expression string and a set of formal constrained variable names. See my instance creation protocol for further details.

Instance variables:
	outIndex                the index of my output variable (target of the assignment) in my constraint
	block           block to execute to enforce the constraint <BlockContext>
'!


!DBMethod class methodsFor:'instance creation'!

names: variableNames methodString: methodString
	"Create a constraint method from the given string, an assignment statement using the given variable names. For example:

		DBMethod names: #(a b c) methodString: 'a _ b * c'"

	^(super new)
		names: variableNames
		methodString: methodString! !

!DBMethod methodsFor:'constraint support'!

execute: variables
	"Execute myself to enforce my constraint on the given variables."

	block value: variables.!

outIndex
	"Answer the index of my output in the bindings array."

	^outIndex! !

!DBMethod methodsFor:'initialize-release'!

names: variableNames methodString: methodString
	"Initialize a method by compiling the given string considering the given collection of variable names to represent the parameters of the method (i.e. its inputs and outputs). A given variable may not be both an input and an output. Note: Any free variables in the methodString will be considered global (if they appear in 'Smalltalk') or temporary variables. The user is given a warning, however, since such cases are unusual and a free variable may indicate a typographical error."

	| insOutsTemps ins outs temps |
	insOutsTemps _ self extractInsOutsAndTemps: methodString using: variableNames.
	ins _ insOutsTemps at: 1.
	outs _ insOutsTemps at: 2.
	temps _ insOutsTemps at: 3.
	outIndex _ self outIndexForIns: ins outs: outs temps: temps all: variableNames.
	block _ Compiler
			evaluate:
				((self blockPrefixForIns: ins temps: temps args: variableNames),
				 methodString,
				 (self blockPostfixForOuts: outs allNames: variableNames))
			for: nil
			logged: false.
	(temps size > 0) ifTrue: [block fixTemps].!

release

	outIndex _ nil.
	block _ nil.! !

!DBMethod methodsFor:'printing'!

printOn: aStream

	aStream
		nextPutAll: 'DBMethod(', self asOop printString;
		nextPutAll: ' out: ', outIndex printString, ')'.! !

!DBMethod methodsFor:'private'!

blockPostfixForOuts: outNames allNames: allNames
	"Answer a string to be used as the postfix when creating a block for this method."

	| stream |
	"make a stream and add separator to terminate user's method string"
	stream _ WriteStream on: (String new).
	stream nextPutAll: '.'; cr.

	"build the expression postfix, creating assignments for all outputs"
	1 to: allNames size do:
		[: index |
		 (outNames includes: (allNames at: index)) ifTrue:
			[stream tab; nextPutAll: '(vars at: '.
			 stream nextPutAll: index printString, ') value: '.
			 stream nextPutAll: (allNames at: index), '.'; cr]].

	stream tab; nextPutAll: ']'; cr.
	^stream contents!

blockPrefixForIns: inNames temps: tempNames args: argNames
	"Answer a string to be used as the prefix when creating a block for a method with the given input names. All constraint variables are declared as temporaries, in addition to the temporary variables from the method string. Input variable temporaries are initialized from the argument vector."

	| stream |
	stream _ WriteStream on: (String new).

	"build the expression prefix, making all the variables look like temps"
	stream nextPutAll: '| '.
	argNames do: [: v | stream nextPutAll: v; space].
	tempNames do: [: v | stream nextPutAll: v; space].
	stream nextPutAll: '|'; cr.

	"build the block header and input assignments"
	stream tab; nextPutAll: '[: vars |'; cr.
	1 to: argNames size do:
		[: index |
		 (inNames includes: (argNames at: index)) ifTrue:
			[stream tab; nextPutAll: (argNames at: index), ' _ (vars at: '.
			stream nextPutAll: index printString.
			stream nextPutAll: ') value.'; cr]].

	stream tab.
	^stream contents!

extractInsOutsAndTemps: methodString using: allNames
	"Extract the input, output and temporary variable names from the Smalltalk expression represented by the given string. A temporary variable is one that is neither an input, an output, or a global. Answer an array containing the three lists (ins, outs, temps)."

	| s parseTree ins outs temps |
	s _ (String new: 200) writeStream.
	s nextPutAll: 'DoIt'; cr; cr.
	s tab; nextPutAll: '| '.
	allNames do: [: vName | s nextPutAll: vName; space].
	s nextPutAll: '|'; cr; tab; nextPutAll: methodString.
	parseTree _ self parse: s contents.
	ins _ parseTree referenced.
	outs _ parseTree assignedTo.
	temps _ parseTree allVariables select:
		[: v | ((allNames includes: v) not) &
			  ((Smalltalk includesKey: v) not)].
	^Array with: ins with: outs with: temps!

outIndexForIns: inNames outs: outNames temps: tempNames all: allNames
	"Answer the index of the method output in the constraint variables. Raise an error if the input and output arg lists are not disjoint or if there is not exactly one output. Warn the user if the method code has free variables (these will be made into temporaries)."

	| realOuts |
"
	realOuts _ outNames copy removeAll: tempNames; yourself.
"
	realOuts _ outNames copy.
	realOuts removeAll: tempNames.

	(realOuts size = 1) ifFalse:
		[self error: 'Constraints must have exactly one output variable'].

	outNames do:
		[: v |
		 ((inNames includes: v) and:
		  [allNames includes: v]) ifTrue:
			[self error: v asString, ' cannot be both input and output!!']].

	tempNames do:
		[: v |
		 Transcript show:
			'Warning: ''', v, ''' is assumed to be a temporary.'; cr].

	^allNames indexOf: (realOuts asOrderedCollection first)!

parse: methodString
	"Answer the Smalltalk parse tree for the given string."

	^SmalltalkCompiler
		parse: methodString
		class: UndefinedObject! !

!DBMethod class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/benchmarks/deltaBlue/DBMethod.st,v 1.4 1997/01/03 17:39:21 cg Exp $'
! !
