"{ Package: 'stx:goodies/compression' }"

Object subclass:#ZipEncoderNode
	instanceVariableNames:'value frequency height bitLength code parent left right'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Compression'
!

ZipEncoderNode comment:'ZipEncoderNode represents a node in a huffman tree for encoding ZipStreams.

Instance variables:
	value           <Integer>       - Encoded value
	frequency       <Integer>       - Number of occurrences of the encoded value
	height          <Integer>       - Height of the node in the tree
	bitLength       <Integer>       - bit length of the code
	code            <Integer>       - Assigned code for this node
	parent          <ZipEncoderNode>                - Parent of this node
	left                    <ZipEncoderNode>                - First child of this node
	right           <ZipEncoderNode>                - Second child of this node
'
!


!ZipEncoderNode class methodsFor:'instance creation'!

value: v frequency: f height: h
	^self new setValue: v frequency: f height: h
! !

!ZipEncoderNode methodsFor:'accessing'!

bitLength
	^bitLength ifNil:[0]
!

code
	^code ifNil:[0]
!

code: aCode
	self assert:[aCode >= 0 and:[(1 bitShift: bitLength) > aCode]].
	code _ aCode.
!

frequency
	^frequency
!

frequency: aNumber
	frequency _ aNumber
!

height
	^height
!

left
	^left
!

left: aNode
	aNode parent: self.
	left _ aNode.
!

parent
	^parent
!

parent: aNode
	parent _ aNode
!

right
	^right
!

right: aNode
	aNode parent: self.
	right _ aNode.
!

value
	^value
! !

!ZipEncoderNode methodsFor:'encoding'!

encodeBitLength: blCounts from: aTree
	| index |
	"Note: If bitLength is not nil then the tree must be broken"
	bitLength == nil ifFalse:[self error:'Huffman tree is broken'].
	parent = nil
		ifTrue:[bitLength _ 0]
		ifFalse:[bitLength _ parent bitLength + 1].
	self isLeaf ifTrue:[
		index _ bitLength + 1.
		blCounts at: index put: (blCounts at: index) + 1.
	] ifFalse:[
		left encodeBitLength: blCounts from: aTree.
		right encodeBitLength: blCounts from: aTree.
	].
!

rotateToHeight: maxHeight
	"Rotate the tree to achieve maxHeight depth"
	| newParent |
	height < 4 ifTrue:[^self].
	self left: (left rotateToHeight: maxHeight-1).
	self right: (right rotateToHeight: maxHeight-1).
	height _ (left height max: right height) + 1.
	height <= maxHeight ifTrue:[^self].
	(left height - right height) abs <= 2 ifTrue:[^self].
	left height < right height ifTrue:[
		right right height >= right left height ifTrue:[
			newParent _ right.
			self right: newParent left.
			newParent left: self.
		] ifFalse:[
			newParent _ right left.
			right left: newParent right.
			newParent right: right.
			self right: newParent left.
			newParent left: self.
		].
	] ifFalse:[
		left left height >= left right height ifTrue:[
			newParent _ left.
			self left: newParent right.
			newParent right: self.
		] ifFalse:[
			newParent _ left right.
			left right: newParent left.
			newParent left: left.
			self left: newParent right.
			newParent right: self.
		].
	].
	parent computeHeight.
	^parent
! !

!ZipEncoderNode methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPut:$(;
		nextPutAll:'value = '; print: value;
		nextPutAll:', freq = '; print: frequency;
		nextPutAll:', bitLength = '; print: bitLength;
		nextPutAll:', code = '; print: code;
		nextPutAll:', height = '; print: height;
	nextPut:$).
! !

!ZipEncoderNode methodsFor:'private'!

computeHeight
	^self isLeaf
		ifTrue:[height _ 0]
		ifFalse:[height _ (left computeHeight max: right computeHeight) + 1].
!

leafNodes
	self isLeaf
		ifTrue:[^Array with: self]
		ifFalse:[^left leafNodes, right leafNodes]
!

setBitLengthTo: bl
	bitLength _ bl
!

setValue: v frequency: f height: h
	value _ v.
	frequency _ f.
	height _ h.
! !

!ZipEncoderNode methodsFor:'testing'!

isLeaf
	^left == nil
! !

!ZipEncoderNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/compression/ZipEncoderNode.st,v 1.2 2005/04/28 17:27:33 cg Exp $'
! !
