"{ Package: 'stx:goodies/displayLists' }"

DisplayModel subclass:#DisplayItem
	instanceVariableNames:'origin corner color owner'
	classVariableNames:''
	poolDictionaries:''
	category:'Graphics-DisplayLists'
!

DisplayItem comment:'Instances of the subclasses of the abstract class DisplayItem are used as the items in display lists. They can generally display themselves on display media.
Instance Variables:
	origin	<Point> the object''s origin relative to its container (display list)
	corner	<Point> the object''s corner
	color	<Color> the object''s display color (or nil)
The subclasses add special display-related state and behavior such as an extent point and display method, or a visual item such as a string or image to display.
They generally implement displayOn: aGraphicsContext and bounds accessing methods.
'
!


!DisplayItem class methodsFor:'instance creation'!

instances
	"DependentsFields"
	"DisplayItem instances"

	^DisplayItem allSubInstances
!

model: aM origin: aPt
	"Answer an instance of DisplayItem with the given instance variables."
	^self new model: aM; origin: aPt
!

model: aM origin: aPt corner: cPt color: color
	"Answer an instance of DisplayItem with the given instance variables."
	^self new model: aM; origin: aPt; corner: cPt; color: color
!

origin: aPt
	"Answer an instance of DisplayItem with the given instance variables."
	^self new origin: aPt
!

origin: aPt corner: cPt
	"Answer an instance of DisplayItem with the given instance variables."
	^self new origin: aPt; corner: cPt
!

origin: aPt corner: cPt color: color
	"Answer an instance of DisplayItem with the given instance variables."
	^self new origin: aPt; corner: cPt; color: color
!

origin: aPt extent: ePt
	"Answer an instance of DisplayItem with the given instance variables."
	^self new origin: aPt; extent: ePt
!

origin: aPt extent: ePt color: color
	"Answer an instance of DisplayItem with the given instance variables."
	^self new origin: aPt; extent: ePt; color: color
! !

!DisplayItem methodsFor:'accessing'!

bottomLeft
	"Answer the point at the left edge of the bottom horizontal line of the receiver."
	^origin x @ corner y
!

bottomLeft: aPoint
	"Set the point at the left edge of the bottom horizontal line of the receiver."
	origin x: aPoint x.
	corner y: aPoint y
!

color
	"Answer the receiver's display color."
	^color
!

color: aVal
	"Set the receiver's display color to the argument."
	color := ((aVal isMemberOf: BlockContext)
		ifTrue: [aVal value]
		ifFalse: [aVal])
!

corner
	"Answer the receiver's display corner."
	^corner
!

corner: aVal
	"Set the receiver's display corner to the argument."
	corner := ((aVal isMemberOf: BlockContext)
		ifTrue: [aVal value]
		ifFalse: [aVal])
!

extent
	"Answer the receiver's extent."
	^corner - origin
!

extent: aPoint
	"Set the extent of the receiver."
	
	origin isNil
		ifTrue: [origin := 0@0].
	corner := origin rounded + aPoint rounded
!

offset: aPoint
	"Set the origin of the receiver."
	origin := aPoint
!

operatorForCorner: aPoint
	"Answer the message to send to change the given corner point."

	^nil		"default"
!

origin
	"Answer the receiver's display origin."
	^origin
!

origin: aPoint
	"Set the origin of the receiver."
	origin := aPoint
!

selectionHandles
	"Answer the points at which to display selection handles when the receiver is selected."
	^self boundingBox corners
!

topRight
	"Answer the point at the top right corner of the receiver's top horizontal line."
	^corner x @ origin y
!

topRight: aPoint
	"Set the point at the top right corner of the receiver's top horizontal line."
	corner x: aPoint x.
	origin y: aPoint y
! !

!DisplayItem methodsFor:'bounds accessing'!

boundingBox
	"Answer the receiver's actual bounding rectangle."
	| x1 y1 x2 y2 |
	x1 := origin x.
	y1 := origin y.
	x2 := corner x.
	y2 := corner y.
	^Rectangle origin: ((x1 min: x2)@(y1 min: y2)) corner: ((x1 max: x2)@(y1 max: y2))
!

bounds
	"Answer the receiver's actual bounding rectangle."

	^self boundingBox
!

computeBounds
	"Answer the receiver's actual bounding rectangle."

	^self boundingBox
! !

!DisplayItem methodsFor:'copying'!

copy
	"Answer a shallow copy of the receiver."

	| newMe |
	newMe := self shallowCopy.
	newMe container: nil.
	^newMe
! !

!DisplayItem methodsFor:'displaying'!

displayOn: aDisplayMedium
	"Display the receiver on the argument."
	self displayOn: aDisplayMedium at: origin
!

displayOn: aDisplayMedium at: aPoint
	"Display the receiver on the given medium with the given offset."
	self subclassResponsibility
! !

!DisplayItem methodsFor:'drawing'!

drawOn: aCanvas
	"Display the receiver on the given canvas."

	self drawOn: aCanvas at: 0@0
!

drawOn: aCanvas at: aPoint
	"Display the receiver on the given canvas at the given position."

	self subclassResponsibility
!

fullDrawOn: aCanvas
	"Display the receiver on the given canvas."

	self drawOn: aCanvas at: 0@0
! !

!DisplayItem methodsFor:'initialize-release'!

initialize
	"Initialize the instance variables of the receiver."
	origin := 0@0
! !

!DisplayItem methodsFor:'morphic'!

allMorphsDo: aBlock
	"Evaluate the given block for all morphs in this composite morph (including the receiver)."

	aBlock value: self.
!

allMorphsNotInPartsBinsDo: aBlock
	
	aBlock value: self
!

eventHandler
	^nil
!

fullBounds
	"Answer the receiver's actual bounding rectangle."

	^self boundingBox
!

fullContainsPoint: aPoint

	^self bounds containsPoint: aPoint
!

handlesMouseOver: evt
	"Do I want to receive mouseEnter: and mouseLeave: when the button is up and the hand is empty?  The default response is false, except if you have added sensitivity to mouseEnter: or mouseLeave:, using the on:send:to: mechanism." 

	self eventHandler ifNotNil: [^ self eventHandler handlesMouseOver: evt].
	^ false
!

handlesMouseOverDragging: evt
	"Return true if I want to receive mouseEnterDragging: and mouseLeaveDragging:"

	self eventHandler ifNotNil: [^ self eventHandler handlesMouseOverDragging: evt].
	^ false
!

layoutChanged
	"Ignored"
!

morphsAt: aPoint addTo: mList
	"Return a collection of all morphs in this morph structure that contain the given point, possibly including the receiver itself."

	(self boundingBox containsPoint: aPoint) 
		ifTrue: [mList addLast: self]
!

owner
	"Answer the receiver's owner."

	^owner
!

ownerChanged
	"Ignored"
!

privateFullMoveBy: aPoint
	"Translate the receiver's origin and corner by the argument."
	self origin: origin + aPoint.
	corner ifNotNil: [self corner: corner + aPoint]
!

privateOwner: object
	"Set the receiver's owner."

	owner := object
!

residesInPartsBin
	"Answer true if the receiver is, or has some ancestor owner who is, a parts bin"

	^false
!

snapToEdgeIfAppropriate
	"ignored"
!

transformFrom: uberMorph
	"Return a transform to be used to map coordinates in a morph above me into my local coordinates, or vice-versa. This is used to support scrolling, scaling, and/or rotation. This default implementation just returns my owner's transform or the identity transform if my owner is nil."

	owner ifNil: [^ MorphicTransform identity].
	^ owner transformFrom: uberMorph
!

trumpsMouseDown: evt
	"Return true if this morph wants to handle mouse down events even when the mouse is pressed under a submorph that also wishes to preepmpt mouse down events."

	^ false
!

unlockedMorphsAt: aPoint addTo: mList
	"Return a collection of all morphs in this morph structure that contain the given point, possibly including the receiver itself."

	(self boundingBox containsPoint: aPoint) 
		ifTrue: [mList addLast: self]
!

visible
	^true
!

wantsBalloon
	"no"

	^false
! !

!DisplayItem methodsFor:'printing'!

printCompleteOn: aStream depth: depth
	"Print the receiver in the given Stream."
	aStream cr.
	depth timesRepeat: [aStream tab].
	self printOn: aStream
!

printInstVarsOn: aStream
	"Print the receiver's relevant InstVars on the given Stream."
	model isNil ifFalse: [aStream nextPutAll: ' model: ', model class name].
	origin isNil ifFalse: [aStream nextPutAll: ' origin: '. origin printOn: aStream].
!

printOn: aStream
	"Print the receiver on the given Stream."
	aStream nextPutAll: ' (a ', self class name. 
	self printInstVarsOn: aStream.
	aStream nextPut: $).
! !

!DisplayItem methodsFor:'testing'!

isDisplayItem
	"Answer whether the receiver is a kind of DisplayItem"
	^true
!

isHit: aPoint within: distance
	"Answer whether the given point is a hit on the receiver to within the given distance."
	^(self boundingBox expandBy: distance) containsPoint: aPoint
! !

!DisplayItem methodsFor:'transforming'!

clipTo: clippingRect
	"Clip the receiver to the given rectangle."

	^self
!

scaledBy: aPoint
	"Scale the receiver's offset and extent by the argument; answer a copy."
	| new |
	new := self copy.
	new origin: self origin * aPoint.
	self corner == nil
		ifFalse: [new corner: self corner * aPoint].
	^new
!

translateBy: aPoint
	"Translate the receiver's origin and corner by the argument."

	self origin: origin + aPoint.
	corner ifNotNil:
		[self corner: corner + aPoint]
! !

!DisplayItem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayItem.st,v 1.1 2000/12/01 11:46:06 cg Exp $'
! !
