"{ Package: 'stx:goodies/displayLists' }"

View subclass:#DisplayListView
	instanceVariableNames:'displayList depth form background scrollOffset scale zoom
		backgroundColor foregroundColor pageSize pageOffset isPaged'
	classVariableNames:'SelectionHandle'
	poolDictionaries:''
	category:'Graphics-DisplayListViews'
!

DisplayListView comment:'DisplayListViews can display and scroll structured graphics display lists generated by various models or layout managers. They can scroll over very large display lists. They can have colored or gridded background pixmaps and can scroll/page on demand; they display their lists on their graphics contexts.

Subclasses generally override the displayOn: or displayOnImage methods, and add initialization or transformation methods.

Instance Variables
	displayList <DisplayList> the view''s display list
	form	 <Form> the view''s cached display pixel map (optional)
	background <Form or nil> the view''s background form (e.g., gridding)
	scale	 <Point> the display list''s layout scale factor or nil
	zoom	 <Point> the view''s zoom factor.
	backgroundColor <Color> graphics background color
	foregroundColor <Color> graphics display color
	scrollOffset <Point> the view scroll origin
	pageSize <Point> the cached page extent for very large lists (default = Display extent, make smaller for memory-starved machines)
	pageOffset <Point> the "page" we''re looking at.
	isPaged <Boolean> whether there are mote than 1 pages in the current list.

See the class examples for numerous ways of using display list views. The subclasses add knowledge of smart display list generation, background pixmap generation (e.g., gridding), display of item or x/y-range selection, x- or y-scaling or step/grid, property->color mapping, "clef forms" or other special pixmaps, multiple-model viewing, etc.
'
!

!DisplayListView class methodsFor:'documentation'!

about
	"Answer a string that describes the DisplayListView application."

	^'The DisplayListView is a general-purpose viewer/editor 
for Siren structured graphics display lists.
	The controller''s middle-button popup menu includes items 
for inspecting, redrawing, or adjusting the zoom or grid.'
! !

!DisplayListView class methodsFor:'instance creation'!

on: anEditor

	| me |
	me := self new initialize.
	me model: anEditor.
	^me
!

onList: dlist
	"DisplayListView example"

	^self onList: dlist size: 400@300
!

onList: dlist size: extent
	"Open a display list view on the given list."
	"DisplayListView example"

	^self onList: dlist size: extent withGrid: nil inColor: nil
!

onList: dlist size: extent withGrid: gPoint inColor: gColor 
	"Open a DLView's window with a background grid"
	"DisplayListView onList: (DisplayList exampleHierarchical) 
			size: 400@300 withGrid: 32@32 inColor: Color lightGray"

	| topView me editor |
	editor := DisplayListEditor on: dlist.
	me := self on: editor.
	gPoint == nil
		ifFalse: [me initializeWithGrid: gPoint color: gColor].
	topView := StandardSystemView new.
	topView model: editor.
	topView label: 'Display List Editor'.
	topView minimumSize: extent.
	topView borderWidth: 2.
	topView addSubView: me in: topView boundingBox borderWidth: 1.
	topView controller open
!

openOnList: dlist
	"IndentedListLayoutManager listExample display"
	"DisplayListView openOnList: (DisplayList linesX: 600 byY: 400)"

	| topView me editor |
	editor := DisplayListEditor on: dlist.
	me := self on: editor.
	topView := StandardSystemView new.
	topView model: editor.
	topView label: 'Display List Editor'.
	topView minimumSize: 300@200.
	topView maximumSize: 640@480.
	topView borderWidth: 2.
	topView addSubView: me in: topView boundingBox borderWidth: 1.
	topView controller open
!

openOnList: dlist size: extent withGrid: gPoint inColor: gColor 
	"Open a DLView's window with a background grid"
	"DisplayListView openOnList: (DisplayLine linesX: 800 byY: 800) 
			size: 400@300 withGrid: 64@32 inColor: Color lightGray"

	| topView me editor |
	editor := DisplayListEditor on: dlist.
	me := self on: editor.
	gPoint == nil
		ifFalse: [me initializeWithGrid: gPoint color: gColor].
	topView := StandardSystemView new.
	topView model: editor.
	topView label: 'Display List Editor'.
	topView minimumSize: extent.
	topView borderWidth: 2.
	topView addSubView: me in: topView boundingBox borderWidth: 1.
	topView controller open
! !

!DisplayListView class methodsFor:'class initialization'!

initialize
	"Set up the class variable."
	"DisplayListView initialize"

	SelectionHandle := Form extent: 8@8.
	SelectionHandle fill: ((0@0) extent: (8@8)) fillColor: Color white.
	SelectionHandle fill: ((1@1) extent: (6@6)) fillColor: Color black.
! !

!DisplayListView class methodsFor:'constant access'!

backgroundColor
	"Answer the default background color for display list views."
	"ColorValue constantNames"

	^Display depth > 1
		ifTrue: [Color "white"  "gray" veryLightGray]
		ifFalse: [Color white]
!

defaultCacheExtent
	"Answer the default extent of the receiver's instances' cache."

	^Display extent
!

foregroundColor
	"Answer the default foreground color for display list views."

	^Display depth > 1
		ifTrue: [Color red]
		ifFalse: [Color black]
!

inset
	"Answer the default display inset."

	^0@0
!

selectionHandle
	"Answer the class's selection handle pixmap"

	^SelectionHandle
! !

!DisplayListView class methodsFor:'examples'!

example1
	"Display lines"
	"DisplayListView example1"

	DisplayListView openOnList: (DisplayLine linesX: 800 byY: 600)
!

example2
	"Display rectangles."
	"DisplayListView example2"

	DisplayListView openOnList: (DisplayRectangle rectanglesX: 800 byY: 800)
!

exampleGridded
	"Open an example of a gridded-background display list view on the example hierarchical display list."
	"DisplayListView exampleGridded"

	DisplayListView onList: (DisplayList exampleHierarchical)
			size: 400@300
			withGrid: 32@32
			inColor: Color lightGray
!

exampleHierarchical
	"DisplayListView exampleHierarchical"

	self onList: (DisplayList exampleHierarchical) size: 400@400
!

exampleHierarchicalStars
	"DisplayListView exampleHierarchicalStars"

	self onList: (DisplayList exampleHierarchicalStarsX: 800 byY: 800 stars: 32) size: 400@400
!

hugeStringExample
	"Display *lots* of random strings in a very large space."
	"DisplayListView hugeStringExample"

	DisplayListView openOnList:
		(DisplayString stringsX: 8000 byY: 6000 items: 5000)
!

rectExample
	"Display rectangles."
	"DisplayListView rectExample"

	DisplayListView openOnList:
		(DisplayRectangle rectanglesX: Display width - 32 byY: Display height - 32)
!

stringExample
	"Display random strings."
	"DisplayListView stringExample"

	DisplayListView openOnList:
		(DisplayString stringsX: 800 byY: 600 items: 500)
! !

!DisplayListView methodsFor:'accessing'!

background: anObject
	"Set the receiver's 'background' to be anObject."

	background := anObject
!

backgroundColor
	"Answer the receiver's backgroundColor or the class default."

	^backgroundColor == nil
		ifTrue: [self class backgroundColor]
		ifFalse: [backgroundColor]
!

backgroundColor: aColor
	"Set the receiver's backgroundColor."

	backgroundColor := aColor
!

boundingBox
	"Answer the displayList's boundingBox."

	| rect |
	displayList == nil
		ifTrue: [^super boundingBox].
	rect := displayList boundingBox.
	^(rect origin) extent: (rect extent + (self class inset * 2))
!

displayList: aDisplayList
	"Set the receiver's display list."

	self list: aDisplayList
!

foregroundColor
	"Answer the receiver's foregroundColor or the default."

	^foregroundColor == nil
		ifTrue: [self class foregroundColor]
		ifFalse: [foregroundColor]
!

isColor
	"Answer whether or not to use colors in displaying."

	^Display depth > 1
!

isPaged
	"Answer whether or not the receiver's list is larger than 1 page."

	^isPaged
!

list
	"Answer the receiver's display list."

	^displayList
!

list: aDisplayList
	"Set the receiver's display list -- select the form size; use fixed 8-bit color."

	| ext |
	displayList := aDisplayList.
	ext := aDisplayList extent.
	ext > pageSize
		ifTrue: [isPaged := true. self halt.
			ext := pageSize]
		ifFalse: [isPaged := false.
			ext := (ext max: self class defaultCacheExtent)].
	form := Form extent: ext depth: 8
!

model: aDisplayListEditor
	"Accept the given display list editor as the model of the view."

	super model: aDisplayListEditor.
	displayList := model list.
	aDisplayListEditor displayList isNil
		ifFalse: [self list: aDisplayListEditor displayList].	
!

offsetX
	"Answer the receiver's 'offsetX.'"

	^scrollOffset x
!

offsetY
	"Answer the receiver's 'offsetY.'"

	^scrollOffset y
!

pageOffset
	"Answer the receiver's pageOffset."

	^pageOffset
!

pageSize
	"Answer the receiver's pageSize."

	^pageSize
!

scaleX
	"Answer the receiver's 'scaleX.'"

	^scale x
!

scaleX: aNumber
	"Set the receiver's 'scaleX' to be aNumber."

	scale x = aNumber ifTrue: [^self].
	scale := aNumber @ scale y
!

scaleY
	"Answer the receiver's 'scaleY.'"

	^scale y
!

scaleY: aNumber
	"Set the receiver's 'scaleY' to be aNumber."

	scale y = aNumber ifTrue: [^self].
	scale := scale x @ aNumber.
	self displayView
!

setScrollOffsetHolder: aValueHolder

	scrollOffset := aValueHolder.
! !

!DisplayListView methodsFor:'controller access'!

defaultControllerClass

	^DisplayListController
! !

!DisplayListView methodsFor:'displaying'!

display: box onForm: theForm
	"Display the receiver's display list on the cached form."

	^self display: box onForm: theForm at: 0@0
!

display: box onForm: theForm at: pt
	"Display the receiver's display list on the cached form."
	"Zoom as necessary, page, etc."

	(zoom = (1@1))
		ifTrue: [isPaged
			ifTrue: [(displayList itemsIntersecting: ((pageOffset * 0.75) extent: pageSize)) 
							displayOn: theForm at: pt clippingBox: box]
			ifFalse: [displayList displayOn: theForm at: pt clippingBox: box]]
		ifFalse: [isPaged
			ifTrue: [((displayList scaledBy: zoom) 
								itemsIntersecting: ((pageOffset * 0.75) extent: pageSize)) 
							displayOn: theForm at: pt clippingBox: box]
			ifFalse: [(displayList scaledBy: zoom) displayOn: theForm at: pt clippingBox: box]]
!

displaySelection: sel on: aDisplayMedium at: aPoint
	"Display little white-bordered black 'handles' at the corners of the selection."

	| pts |
	pts := displayList selectionHandlesFor: sel.
	sel ifNil: [^self].
	(zoom = (1@1))
		ifTrue: [pts do: [ :pt | SelectionHandle displayOn: aDisplayMedium at: (pt + aPoint - (4@4))]]
		ifFalse: [pts do: [ :pt | SelectionHandle displayOn: aDisplayMedium at: (pt * zoom + aPoint - (4@4))]]
!

displaySelectionOn: aDisplayMedium at: origin clippingBox: box
	"Display little white-bordered black 'handles' at the corners of the selection."

	| sel |
	sel := model selection.
	sel == nil ifTrue: [^self].
	sel hasItems
		ifFalse: [self displaySelection: sel on: aDisplayMedium at: origin]
		ifTrue: [sel do:
			[ :item | self displaySelection: item on: aDisplayMedium at: origin]]
!

displayView
	"Display the receiver's background form and display list on the Display."

	| box |
	super displayView.
	box := form boundingBox.
	Cursor execute showWhile:
		[form fillColor: self backgroundColor.
		background ifNotNil: [background displayOn: form].
		self display: box onForm: form.
		self displaySelectionOn: form at: 0@0 clippingBox: box].
	form displayOn: Display at: (self insetDisplayBox origin - scrollOffset) 
			clippingBox: self insetDisplayBox
!

redraw
        "Display the receiver's background form and display list on the Display."

        self displayView
!

redrawBackground
	"No-op here."

	^self
! !

!DisplayListView methodsFor:'initialize-release'!

initialize
	"Set up the default values of the receiver."

	super initialize.
	scale := 1@1.
	zoom := 1@1.
	scrollOffset := 0@0.
	pageSize := self class defaultCacheExtent.
	pageOffset := 0@0.
	isPaged := false
!

initializeWithGrid: gPoint color: gColor
	"Initialize the receiver's instance variables--the default is to cache."

	| cacheE line color |
	self initialize.
	cacheE := self class defaultCacheExtent.
	background := Form extent: cacheE depth: Display depth.
	background fillColor: DisplayListView backgroundColor.
	gColor isNil
		ifTrue: [color := Color black]
		ifFalse: [color := gColor].
	line _ BitBlt
		destForm: background
		sourceForm: nil
		fillColor: color
		combinationRule: Form over
		destOrigin: 0@0
		sourceOrigin: 0@0
		extent: 1@1
		clipRect: (0@0 extent: cacheE).
	gPoint x to: cacheE x by: gPoint x do:
		[ :x | line drawFrom: (x@0) to: (x @ cacheE y)].
	gPoint y to: cacheE y by: gPoint y do:
		[ :y | line drawFrom: (0@y) to: (cacheE x @ y)].
!

release
	"Remove the receiver as a dependent of its model."

	model release.
	model removeDependent: self.
	model := nil.
	displayList release.
	super release
! !

!DisplayListView methodsFor:'scrolling'!

pageDown
	"Scroll the receiver down by 3/4 of the window height."

	isPaged
		ifTrue: [self scrollBy: (0 @ (pageSize x * 0.75) truncated negated)]
		ifFalse: [self scrollDown]
!

pageLeft
	"Scroll the receiver left by 3/4 of the window width."

	isPaged
		ifTrue: [self scrollBy: ((pageSize x * 0.75) truncated @ 0)]
		ifFalse: [self scrollLeft]
!

pageRight
	"Scroll the receiver left by 3/4 of the window width."

	isPaged
		ifTrue: [self scrollBy: ((pageSize x * 0.75) truncated negated @ 0)]
		ifFalse: [self scrollRight]
!

pageUp
	"Scroll the receiver up by 3/4 of the window height."

	isPaged
		ifTrue: [self scrollBy: (0 @ (pageSize x * 0.75) truncated)]
		ifFalse: [self scrollUp]
!

relativePointFor: aPoint
	"Answer the transformed point representing the argument (faster than doing a full windowing transformation)."

	^aPoint - self insetDisplayBox origin / zoom + scrollOffset
!

scrollBy: aPoint
	"Scroll the receiver's display by the given point."

	scrollOffset := scrollOffset - aPoint.
	scrollOffset := scrollOffset max: (0@0).
	scrollOffset := scrollOffset min: (form extent - self insetDisplayBox extent).
	self displayView
!

scrollDown
	"Scroll the receiver down by 1/4 of the window height."

	self scrollBy: (0 @ (self insetDisplayBox width // 4) negated)
!

scrollEnd
	"Scroll the receiver to the lower-left."

	self scrollBy:  displayList bounds extent negated + scrollOffset 
				+ self insetDisplayBox extent

!

scrollHome
	"Scroll the receiver to 0@0."

	self scrollBy: scrollOffset
!

scrollLeft
	"Scroll the receiver left by 1/4 of the window width."

	self scrollBy: ((self insetDisplayBox width // 4)@0)
!

scrollRight
	"Scroll the receiver right by 1/4 of the window width."

	self scrollBy: ((self insetDisplayBox width // 4) negated @ 0)
!

scrollUp
	"Scroll the receiver up by 1/4 of the window height."

	self scrollBy: (0 @ (self insetDisplayBox width // 4))
! !

!DisplayListView methodsFor:'updating'!

update: anAspect with: anArg
	"Echo the model's changes, assuming a view is a dependent of the receiver."

	anAspect == #scaleX
		ifTrue: [^self scaleX: scale x * anArg].
	anAspect == #scaleY
		ifTrue: [^self scaleY: scale y * anArg].
	anAspect == #selection
		ifTrue: [^self displayView].
	super update: anAspect with: anArg
! !

!DisplayListView methodsFor:'zooming'!

zoomIn
	"Zoom in by 25%."

	zoom := zoom / 0.75.
	self redrawBackground.
	self displayView
!

zoomOut
	"Zoom out by 25%."

	zoom := zoom * 0.75.
	self redrawBackground.
	self displayView
!

zoomTo1
	"Zoom to scale 1.0."

	zoom := 1@1.
	self redrawBackground.
	self displayView
! !

!DisplayListView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/displayLists/DisplayListView.st,v 1.1 2000/12/01 15:19:50 cg Exp $'
! !
DisplayListView initialize!
