"{ Package: 'stx:goodies/gitocello' }"

Object subclass:#GCRegistry
	instanceVariableNames:'git'
	classVariableNames:'Repositories RepositoryRoot'
	poolDictionaries:''
	category:'Gitocello-Core'
!

GCRegistry class instanceVariableNames:'instance'

"
 No other class instance variables are inherited by this class.
"
!

GCRegistry comment:'In my class I keep all the information on where to create/find git repositories in the file system (RepositoryRoot) and which Repositories have been created and how far they are along compared to their Monticello history.

My instance (of which there is only one) is used to create, remove and update repositories 
through the interface exposed in the ''commands'' category.

I am well aware of the conversion to Gnu Smalltalk going on here, and I call the appropriate classes to do the conversion.'
!


!GCRegistry class methodsFor:'accessing'!

at: aPackage

	^ self repositories at: aPackage
! !

!GCRegistry class methodsFor:'class initialization'!

createDirectoryStructure: aString
	"Create the directory structure beneath the current image"
	| repoDir |
	repoDir := FileDirectory default.
	(aString findBetweenSubStrs: FileDirectory pathNameDelimiter asString)
		do: [:each |
			(repoDir directoryExists: each)
				ifFalse: [repoDir createDirectory: each].
			repoDir := repoDir directoryNamed: each].
	^ repoDir
!

initialize
	
	super initialize.
	Smalltalk addToStartUpList: self. 
	self startUp.
	self instance: nil.
!

new
	"One global Registry is enough"
	^ self instance 
		ifNil: [instance := super new]
		ifNotNil: [instance]
!

startUp

	self 
		repositoryRoot: 'package-cache', FileDirectory slash, 'git-repositories'.
	(self repositories keys copy do: [:each |
			(self repositoryRoot directoryNames includes: each)
				ifFalse: [Repositories removeKey: each]]).
! !

!GCRegistry class methodsFor:'class variables'!

instance
	
	^ instance
!

instance: anObject
	
	instance := anObject
!

repositories

	Repositories ifNil: [Repositories := Dictionary new].
	^ Repositories 
!

repositories: aDictionary

	Repositories := aDictionary
!

repositoryRoot
	"Return the FileDirectory"
	^ RepositoryRoot
!

repositoryRoot: aStringOrDirectory
	"Set the FileDirectory either directly or from String"
	RepositoryRoot := aStringOrDirectory isString 
		ifTrue: [self createDirectoryStructure: aStringOrDirectory]
		ifFalse: [self createDirectoryStructure: aStringOrDirectory pathName]
			
!

tracksPackage: aPackageNameOrMCPackage

	^ (self repositories keys includes: aPackageNameOrMCPackage)
		or: [self repositories keys includes: aPackageNameOrMCPackage name]
! !

!GCRegistry class methodsFor:'error handling'!

doesNotUnderstand: aMessage

	(self new respondsTo: aMessage selector)
		ifTrue: [^ self instance 
					perform: aMessage selector 
					withArguments: aMessage arguments].
	^ super doesNotUnderstand: aMessage
! !

!GCRegistry methodsFor:'accessing'!

at: aPackage

	^ self repositories at: aPackage
!

repositories

	^ Repositories
! !

!GCRegistry methodsFor:'commands'!

commit: aPackage
	"This is part of the class interface - try not to break it"
	| lastCommit ancestors wc |
	lastCommit := (GCRegistry at: aPackage) lastCommit.
	wc := (MCPackage named: aPackage) workingCopy.
	lastCommit ifNil: [lastCommit := (ListChooser
				chooseItemFrom: wc ancestry breadthFirstAncestors
				title: 'Please choose how far back you want to load and commit packages for import')].
	lastCommit ifNil: [^ self].
	ancestors := (wc ancestry allAncestorsOnPathTo: lastCommit) reverse.
	SystemChangeNotifier uniqueInstance doSilently: [
	ancestors
		do: [:ancestor |
			(wc repositoryGroup versionWithInfo: ancestor)
				ifNil: [self inform: 'Cannot find a package for ', ancestor name, ' in the repositories.',
								Character cr asString, 'I am continuing with the next version, ',
								Character cr asString, 'and will combine the commit messages']
				ifNotNilDo: [:version |
							version load.
							self 
								createFilesFor: aPackage;
								run: #add
									in: aPackage asString
									with: #();
								run: #commit:
									in: aPackage asString
									with: (Array with: (self commitMessagesFor: aPackage)) ]]
		displayingProgress: [:ancestor | 'Commiting ', ancestor name]].
!

createRepositoryFor: aPackage
	"This will do everything in its power to commit that package to git!!"
	(Repositories includesKey: aPackage) ifFalse: [
		Repositories add: (aPackage -> (GCPackage newFor: aPackage)).
		self createFilesFor: aPackage.
		self run: #init in: aPackage asString].
	self commit: aPackage
!

dumpImage
	"Dump all packages currently in the system"
	(SmalltalkImage current organization categories
	collect: [:c | c asString copyUpTo: $-]) asSet
		do: [:package | GCRegistry new createRepositoryFor: package].
!

pull: aPackage
	"This might be removed at some time in the future"
	self run: #pull in: aPackage asString
!

push: aPackage
	"This might be removed at some time in the future"
	self run: #push in: aPackage asString
!

removeRepositoryFor: aPackage
	"This will not touch the filesystem, only our registry"
	Repositories removeKey: aPackage ifAbsent: []
! !

!GCRegistry methodsFor:'fileOut'!

createFilesFor: aPackage
	"Create and convert source to Gnu Smalltalk syntax. Als create a package.xml"
	| files |
	files := (GCMapper newFor: aPackage)
		repoDir: (RepositoryRoot directoryNamed: aPackage);
		fileOutPackage;
		files.
	files do: [:file | GCGstConvertCommand convert: file from: 'squeak' to: 'gst'].
	(GCGstPackageWriter newFor: aPackage)
		repoDir: (RepositoryRoot directoryNamed: aPackage);
		fileOutPackageXml
! !

!GCRegistry methodsFor:'helpers'!

run: aGitCommand in: aPackage
	
	self run: aGitCommand in: aPackage with: (Array new: 0)
		
!

run: aGitCommand in: aPackage with: anArgumentsArray
	"Set the details we need and run"
	git 
		localPath: RepositoryRoot pathName, 
					FileDirectory pathNameDelimiter asString, 
					aPackage;
		perform: aGitCommand withArguments: anArgumentsArray
		
! !

!GCRegistry methodsFor:'initialize-release'!

initialize

	super initialize.
	git := GCGitWrapper new
! !

!GCRegistry methodsFor:'monticello-helpers'!

commitMessagesFor: aPackage
	"Creates a git-commit-msg from the non-commited Monticello versions"
	| messages ancestors |
	ancestors := ((MCPackage named: aPackage) 
		workingCopy ancestry breadthFirstAncestors 
			copyUpTo: (self at: aPackage) lastCommit).
	(ancestors size > 1 and: [self confirm: 'There are ', ancestors size, ' commits difference between the image and the Git repository. Should I commit all messages as one?'])
		ifTrue: [messages := self commitMessagesForAncestors: ancestors]
		ifFalse: [messages := self commitMessagesForAncestors:
				(ancestors ifEmpty: [{}] ifNotEmpty: [{ancestors at: 1}])].
	(self at: aPackage)
		lastCommit: ((MCPackage named: aPackage) workingCopy ancestry ancestors 
			ifEmpty: [nil]
			ifNotEmpty: [:o | o first]).
	^ self escapeForBash: messages contents
		
!

commitMessagesForAncestors: aCollection
	"Creates a git-commit-msg from the non-commited Monticello versions"
	| messages |
	messages := WriteStream on: (String new: 100).
	aCollection do: [:next | messages
								nextPutAll: next message;
								nextPut: Character space;
								nextPut: $(;
								nextPutAll: next summaryHeader;
								nextPut: $);
								cr; cr].
	^ messages contents
!

escapeForBash: aString

	^ (((aString 
		replaceAll: Character cr with: Character lf)
		replaceAll: $!! with: $.)
		replaceAll: $" with: $')
		replaceAll: $$ with: $S
		
!

storeVersion: aWorkingCopy

	| package |
	package := aWorkingCopy package name.
	self 
		pull: package;
		commit: package;
		push: package.
	
! !

!GCRegistry class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCRegistry.st,v 1.1 2011/08/31 15:58:33 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gitocello/GCRegistry.st,v 1.1 2011/08/31 15:58:33 cg Exp $'
! !

GCRegistry initialize!
