"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DatabaseAccessor
	instanceVariableNames:'connection currentLogin logging reusePreparedStatements
		deniedCommands mutex'
	classVariableNames:'LoggingEnabled'
	poolDictionaries:''
	category:'GlorpDatabase'
!

DatabaseAccessor comment:'This translates between the the general, portable Glorp protocol and whatever needs to be done to access a database in a particular smalltalk dialect. Most of the real work is done in its subclasses.

Subclasses must implement the following messages:
	accessing
		connectionClassForLogin:
	executing
		basicExecuteSQLString:
		externalDatabaseErrorSignal
		rowCount
	login
		loginIfError:
		logout
		showDialog:

Instance Variables:
	connection	<varies>	The underlying database connection
	currentLogin	<Login> The glorp-level login we are using
	logging	<Boolean>	Is logging on or off
	permittedCommands	<Collection of: DatabaseCommand class>	The list of commands we are permitted to use. This allows us to filter commands, indicating that a particular session should only be doing inserts right now, not updates, deletes, or queries. Or that it should only be doing writes, not reads.
	reusePreparedStatements	<Boolean>	Should we try to reuse prepared statements,that have the same SQL, or just re-prepare each time.

'
!


!DatabaseAccessor class methodsFor:'instance creation'!

classForThisPlatform

        Dialect isGNU ifTrue: [^Dialect smalltalkAt: #JdmDatabaseAccessor].
        Dialect isSqueak ifTrue: [^Dialect smalltalkAt: #SqueakDatabaseAccessor].
        Dialect isVisualWorks ifTrue: [^Dialect smalltalkAt: #'Glorp.VWDatabaseAccessor'].
        Dialect isVisualAge ifTrue: [^Dialect smalltalkAt: #VA55DatabaseAccessor].
        Dialect isDolphin ifTrue: [^Dialect smalltalkAt: #DolphinDatabaseAccessor].
        Dialect isObjectStudio ifTrue: [^Dialect smalltalkAt: #ObjectStudioDatabaseAccessor].
        Dialect isSmalltalkX ifTrue: [^Dialect smalltalkAt: #'Glorp::SmalltalkXDatabaseAccessor'].
        self error: 'unknown dialect'.
        ^self
!

forLogin: aLogin 
	^self classForThisPlatform new currentLogin: aLogin
!

new

	^super new initialize.
! !

!DatabaseAccessor class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DatabaseAccessor class methodsFor:'accessing'!

loggingEnabled
	LoggingEnabled isNil ifTrue: [LoggingEnabled := false].
	^LoggingEnabled
!

loggingEnabled: aBoolean 
	LoggingEnabled := aBoolean
! !

!DatabaseAccessor methodsFor:'accessing'!

connection
	^connection
!

connectionClass
	^self connectionClassForLogin: currentLogin
!

connectionClassForLogin: aLogin
	^self subclassResponsibility.
!

currentLogin
	^currentLogin
!

currentLogin: aLogin 
	currentLogin := aLogin
!

deniedCommands
	^deniedCommands.
!

deniedCommands: aCollection
	deniedCommands := aCollection.
!

encoding
	self subclassResponsibility.
!

numberOfPreparedStatements
	^0
!

platform
	^currentLogin database.
!

returnRowCount: aBoolean
	^self.
!

reusePreparedStatements
	^reusePreparedStatements and: [self useBinding].
!

reusePreparedStatements: aBoolean
	reusePreparedStatements := aBoolean.
!

useBinding
	^self platform useBinding.
! !

!DatabaseAccessor methodsFor:'copying'!

copy

	^self shallowCopy postCopy.
!

postCopy
! !

!DatabaseAccessor methodsFor:'cursors'!

cursorFor: aStream
	^GlorpCursoredStream on: aStream accessor: self.
!

cursorFor: answerStream command: aCommand
	^aCommand isNil
		ifTrue: [GlorpCursoredStream on: answerStream accessor: self]
		ifFalse: [GlorpCursoredStream on: answerStream for: aCommand accessor: self].
!

isCursorAtEnd: aCursor
	"Return true if aCursor is at its end. This implementation may not work for all, but it seems like a reasonable default"

	^aCursor atEnd.
! !

!DatabaseAccessor methodsFor:'executing'!

basicExecuteSQLString: aString
	self subclassResponsibility.
!

basicExecuteSQLString: aString bindingArray: aBindingArray
	self subclassResponsibility.
!

basicExecuteSQLStringNoResult: aString doing: aBlock
	^self basicExecuteSQLString: aString returnResult: false binding: nil doing: aBlock.
!

createSequence: aDatabaseSequence ifError: aBlock 
	self 
		execute: [self executeSQLStringNoResult: aDatabaseSequence creationString]
		ifError: aBlock
!

createTable: aGlorpDatabaseTable ifError: aBlock 
	"This method should be used to create a database table from aTable"
	aGlorpDatabaseTable isImaginary ifTrue: [^self].
	self 
		execute: [self executeSQLStringNoResult: (self platform createTableStatementStringFor: aGlorpDatabaseTable)]
		ifError: aBlock
!

createTableFKConstraints: aGLORBDatabaseTable ifError: aBlock 
	"This method should be used to define foreign key constraints for a database table from aTable"

	self execute: 
			[(self platform 
				createTableFKContraintsStatementStringsFor: aGLORBDatabaseTable) 
					do: [:ea | self executeSQLStringNoResult: ea]]
		ifError: aBlock
!

createTableIndexes: aGlorpDatabaseTable ifError: aBlock 
	"Create all indexes for this table"

	self execute: 
			[(self platform createTableIndexStatementStringsFor: aGlorpDatabaseTable) 
				do: [:ea | self executeSQLStringNoResult: ea]]
		ifError: aBlock
!

dropConstraint: aConstraint 
	self execute: [self executeSQLStringNoResult: aConstraint dropString]
		ifError: [ :ex | 	| msg|
		msg := ex messageText.
		Transcript show: msg.
		ex return: nil].
!

dropSequence: aDatabaseSequence ifAbsent: aBlock 
	self execute: [self executeSQLStringNoResult: aDatabaseSequence dropString]
		ifError: aBlock
!

dropSequences: anArray
	anArray
		do:
			[:each | 
			self
				dropSequence: each
				ifAbsent:
					[:ex | 
					Transcript show: (ex messageText ifNil: [ex printString]).
					ex return: nil]].
!

dropTable: aTable ifAbsent: aBlock 
	aTable isImaginary ifTrue: [^self].
	self execute: [aTable dropFromAccessor: self]
		ifError: aBlock
!

dropTableNamed: aString 
	self executeSQLStringNoResult: 'DROP TABLE ' , aString
!

dropTableNamed: aString ifAbsent: aBlock 
        self execute: [self executeSQLStringNoResult: 'DROP TABLE ' , aString]
                ifError: aBlock
!

dropTables: anArray 
"PostgreSQL drops foreign key constraints implicitly."
	anArray do: [:each | each dropForeignKeyConstraintsFromAccessor: self].
	anArray do: [:each | self dropTable: each ifAbsent: [ :ex | 	| msg|
		msg := ex messageText.
		Transcript show: msg.
		ex return: nil]].
!

errorsToCatch
	^Dialect error.
!

execute: aBlock

	^self execute: aBlock ifError: [:ex | ex pass].
!

execute: aBlock ifError: errorBlock

	^aBlock on: self errorsToCatch do: errorBlock.
!

executeCommand: command
	^self executeCommand: command returnCursor: false.
!

executeCommand: command returnCursor: aBoolean
	^mutex
		critical:
			[| result |
			self checkPermissionFor: command.
			result := [(self useBinding and: [command useBinding])
				ifTrue: [command executeBoundIn: self]
				ifFalse: [command executeUnboundIn: self]]
				on: Dialect error
				do: [:ex | self handleError: ex for: command].
			aBoolean ifTrue: [result] ifFalse: [result upToEnd]].
!

executeCommandBound: command
	"By default, we don't handle binding"
	^command executeUnboundIn: self.
!

executeCommandUnbound: aCommand
	| cursor |
	self log: aCommand sqlString.
	cursor := self basicExecuteSQLString: aCommand sqlString.
	cursor command: aCommand.
	^cursor.
!

executeSQLString: aString
	| result block time |
	self log: aString.
	block := [result := (self basicExecuteSQLString: aString) upToEnd].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self log: ['(' , (time / 1000.0) printString , ' s)'].
	^result.
!

executeSQLString: aString binding: aBindingArray
	| result block time |
	self log: aString.
	block := [ 
		result := (self basicExecuteSQLString: aString binding: aBindingArray) upToEnd].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self log: ['(' , (time / 1000.0) printString , ' s)'].
	^result.
!

executeSQLStringNoResult: aString
	^self executeSQLStringNoResult: aString doing: [].
!

executeSQLStringNoResult: aString doing: aBlock
	"Execute the string and discard the result. Evaluate aBlock before releasing the statement's answer stream"
	| block time |
	self log: aString.
	block := [
		self basicExecuteSQLStringNoResult: aString doing: aBlock].
	time := self logging
		ifTrue: [Time millisecondsToRun: block]
		ifFalse:
			[block value.
			0].
	self log: ['(' , (time / 1000.0) printString , ' s)'].
!

externalDatabaseErrorSignal

	self subclassResponsibility.
!

handleError: exception for: command
	| errorClass error |
	errorClass := command isReadCommand
		ifTrue: [GlorpDatabaseReadError]
		ifFalse: [GlorpDatabaseWriteError].
	error := errorClass new.
	error command: command.
	error databaseError: (self innerExceptionFor: exception).
	error accessor: self.
	error signal = true
		ifTrue:
			[self reset.
			self logout.
			self reLogin.
			exception restart]
		ifFalse: [error return: nil].
!

innerExceptionFor: exception
	^exception parameter.
!

rowCount
	^self subclassResponsibility
! !

!DatabaseAccessor methodsFor:'initializing'!

initialize
	deniedCommands := nil.
	mutex := Semaphore forMutualExclusion.
!

reset
	"Don't reset the permissions, because the replicator wants to reset sessions without changing their characteristics"
"	self permitEverything."
! !

!DatabaseAccessor methodsFor:'logging'!

log: aStringOrBlock
	| string |
	self logging ifFalse: [^self].
	string := aStringOrBlock isString
		ifTrue: [aStringOrBlock]
		ifFalse: [aStringOrBlock value].
	string size > 4000 ifTrue: [string := string copyFrom: 1 to: 4000].
	Transcript
		cr;
		show: string.
!

logError: anErrorObject 
	self log: anErrorObject printString
!

logging
	logging isNil ifTrue: [^self class loggingEnabled].
	^logging.
!

logging: aBoolean 
	logging := aBoolean
! !

!DatabaseAccessor methodsFor:'login'!

login
        
        | |
        self loginIfError: [:ex |   " ex halt: 'Login error'.  "
                ex pass].
        "Just to help avoid confusion if someone thinks they're getting a login object back from this"
        ^nil.
!

loginIfError: aBlock

	self subclassResponsibility.
!

logout
	^self subclassResponsibility.
!

showDialog: aString

	self subclassResponsibility.
! !

!DatabaseAccessor methodsFor:'permissions'!

alsoDenyCommand: aCommand
	self deniedCommands isNil
		ifTrue: [self denyCommand: aCommand]
		ifFalse:
			[self deniedCommands: self deniedCommands , (Array with: aCommand)].
!

beReadOnly
	self denyWrites.
!

checkPermissionFor: command
	deniedCommands isNil ifTrue: [^self].
	(deniedCommands includes: command class)
		ifTrue: [(GlorpIllegalCommand new command: command) signal].
!

denyCommand: aCommand
	(aCommand isBehavior) ifFalse: [self error: 'invalid command type'].
	self denyCommands: (Array with: aCommand).
!

denyCommands: aCollection
	deniedCommands := aCollection asArray.
!

denyReads
	deniedCommands := SelectCommand glorpAllSubclasses.
!

denyWrites
	deniedCommands := RowBasedCommand glorpAllSubclasses.
!

insertOnly
	self denyCommands: (Array with: UpdateCommand with: DeleteCommand).
!

permitEverything
	"Note that nil is treated as a flag indicating that all commands are allowed"
	deniedCommands := nil.
!

permitNothing
	deniedCommands := DatabaseCommand glorpAllSubclasses asArray.
!

permitsInserts
	deniedCommands isNil ifTrue: [^true].
	^(deniedCommands includes: InsertCommand) not.
!

permitsUpdates
	deniedCommands isNil ifTrue: [^true].
	^(deniedCommands includes: UpdateCommand) not.
!

resetPermissions
	self permitEverything.
! !

!DatabaseAccessor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DatabaseAccessor.st,v 1.1 2006/07/26 08:41:09 boris Exp $'
! !
