"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#DatabaseRow
	instanceVariableNames:'table contents shouldBeWritten owner forDeletion status
		oldVersion generated parent operation'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

DatabaseRow class instanceVariableNames:'missingFieldIndicator emptySlot'

"
 No other class instance variables are inherited by this class.
"
!

DatabaseRow comment:'This represents the data to be written out to a row. Database rows are normally stored in a rowmap, keyed according to their table and the object that did the primary writes to them. We expect that that''s only one object, although embedded values are an exception to that.

Instance Variables:

	table	<DatabaseTable>	The table holding the data
	contents	<IdentityDictionary>	Holds the fields with their values, indirectly through FieldValueWrapper instances.
	shouldBeWritten	<Boolean>	Normally true, but can be set false to suppress writing of a particular row. Used with embedded value mappings, where we create their row, unify it with the parent row, and suppress writing of the original row.
	owner	<Object>	The primary object that wrote into this row, would also be the key into the rowmap.
	forDeletion <Boolean> Is this row to be deleted or written.
	status <Symbol> Used for topological sort to determine write order. We keep a symbol indicating the "visited" status of the row while doing the sort. This doesn''t separate the sort from the row, using each row directly as a node in the graph. This is less general, but uses less memory than other alternatives.
	parent <DatabaseRow> Also used during sorting. Keep track of our parent node.
	oldVersion <DatabaseRow> The old version of us. Used if we increment a version field to keep track of what its value was before.
	generated <Boolean> True if we have already had field with generated types generated in the preWrite phase. This is needed because we generate all sequences before doing any writes. But if we use an explicit writeRow: operation, then we want to make sure that generated values are generated. And some generated types write over what was there before (e.g. optimistic lock fields). So we need a way of knowing if generation has already been done. Note that this doesn''t apply if using post-write sequence generation (i.e. identity columns) because then it''s all after the write. This makes certain things not work as well, but that''s what you get for using identity columns.
	'
!


!DatabaseRow class methodsFor:'instance creation'!

new

	^super new initialize.
!

newForTable: aTable

	^self new table: aTable.
!

newForTable: aTable withOwner: anObject

	^self new table: aTable; owner: anObject.
! !

!DatabaseRow class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!DatabaseRow class methodsFor:'private'!

emptySlot
	emptySlot == nil ifTrue: [emptySlot := Object new].
	^emptySlot.
!

missingFieldIndicator
	missingFieldIndicator == nil ifTrue: [missingFieldIndicator := Object new].
	^missingFieldIndicator.
! !

!DatabaseRow methodsFor:'accessing'!

= aRow
	^self equals: aRow.
!

at: aField

	^self at: aField ifAbsent: [self error: 'missing field'].
!

at: aField ifAbsent: absentBlock

	^(self wrapperAt: aField ifAbsent: [^absentBlock value]) contents.
!

at: aField ifAbsentPut: absentBlock

	| old |
	old := contents at: aField position.
	old == self class emptySlot 
		ifTrue: absentBlock
		ifFalse: [^old contents].
!

at: aField put: aValue 
	
	"For generated fields, we expect the real value to be provided later by the database, so don't write a nil value"
	
	| wrapper |
"	aValue isGlorpExpression ifTrue: [self error: 'cannot store expressions in rows']."
	aField table == self table ifFalse: [self error: 'Invalid table'].
	wrapper := self contentsAt: aField ifAbsentPut: [FieldValueWrapper new].
	(aValue isNil and: [aField isGenerated]) ifFalse: [wrapper contents: aValue].
	wrapper isNowContainedBy: self and: aField.
	^wrapper.
!

atFieldNamed: aString

	| field |
	field := table fieldNamed: aString.
	^self at: field
!

atFieldNamed: aString ifAbsent: aBlock

	| field |
	field := table fieldNamed: aString ifAbsent: aBlock.
	^self at: field ifAbsent: aBlock.
!

atFieldNamed: aString put: anObject

	| field |
	field := table fieldNamed: aString.
	^self at: field put: anObject
!

contentsAt: aField ifAbsentPut: absentBlock

	| old newValue |
	old := contents at: aField position.
	^old == self class emptySlot 
		ifTrue: [
			newValue := absentBlock value.
			contents at: aField position put: newValue.
			newValue]
		ifFalse: [old].
!

doesNotUnderstand: aMessage
	^self atFieldNamed: aMessage selector asString ifAbsent: [super doesNotUnderstand: aMessage].
!

fields
	^self table fields select: [:each | (self includesField: each)].
!

forDeletion
	^forDeletion.
!

forDeletion: aBoolean
	forDeletion := aBoolean.
!

includesField: aField
	^(contents at: aField position) ~~ self class emptySlot.
!

lockKeyFields

	^table lockKeyFields
!

nonGeneratedFields
	"Return a list of our fields that are not generated"
	| result |
	result := OrderedCollection new: contents size.
	self table fields do: [:field | 
		field isGenerated not ifTrue: [result add: field]].
	^result.
!

nonGeneratedFieldsWithValues
	"Return a list of our fields that a) are not generated or b) have values. That is, exclude values we expect the database to generate"
	| result |
	result := OrderedCollection new: contents size.
	self fieldsAndValidValuesDo: [:field :value | 
		(value notNil or: [field isGenerated not]) ifTrue: [result add: field]].
	^result.
!

nonPrimaryKeyFields
	| result |
	result := OrderedCollection new: contents size.
	self fieldsDo: [:field | 
		field isPrimaryKey ifFalse: [result add: field]].
	^result.
!

numberOfFields
	^contents
		inject: 0
		into:
			[:sum :each | sum + (each == self class emptySlot ifTrue: [0] ifFalse: [1])].
!

oldVersionAt: aField ifAbsent: aBlock
	oldVersion isNil ifTrue: [^aBlock value].
	^oldVersion at: aField ifAbsent: aBlock.
!

owner

	^owner.
!

owner: anObject

	owner := anObject.
!

parent
	^parent
!

parent: aDatabaseRow
	parent := aDatabaseRow.
!

primaryKey

	| |
	self table primaryKeyFields isEmpty ifTrue: [^nil].
	^self table hasCompositePrimaryKey 
		ifTrue: [
			self table primaryKeyFields 
				collect: [:each | 	self at: each]]
		ifFalse: [self at: self table primaryKeyFields first ifAbsent: [nil]].
!

status
	^status
!

status: aSymbol
	status := aSymbol
!

table
	"Private - Answer the value of the receiver's ''table'' instance variable."

	^table
!

values
	| result |
	result := OrderedCollection new.
	self keysAndValuesDo: [:key :value |
		result add: value].
	^result
!

wrapperAt: aField

	^self wrapperAt: aField ifAbsent: [self error: 'Field not found'].
!

wrapperAt: aField ifAbsent: aBlock

	| thing |
	contents isNil ifTrue: [^aBlock value].
	thing := contents at: aField position.
	^thing == self class emptySlot ifTrue: aBlock ifFalse: [thing].
!

wrapperAt: aField put: aWrapper
	| old |
	old := contents at: aField position.
	old == aWrapper ifTrue: [^self].
	contents at: aField position put: aWrapper.
	aWrapper isNowContainedBy: self and: aField.
! !

!DatabaseRow methodsFor:'api'!

unifyField: eachSourceField withField: eachTargetField inRow: targetRow
	| sourceWrapper targetWrapper |
	sourceWrapper := self wrapperAt: eachSourceField ifAbsent: [nil].
	targetWrapper := targetRow wrapperAt: eachTargetField ifAbsent: [nil].
	sourceWrapper isNil
		ifTrue:
			[targetWrapper isNil
				ifTrue:
					[targetWrapper := FieldValueWrapper new.
					targetRow wrapperAt: eachTargetField put: targetWrapper].
			^self wrapperAt: eachSourceField put: targetWrapper].
	targetWrapper isNil ifTrue: [^targetRow wrapperAt: eachTargetField put: sourceWrapper].
	sourceWrapper unifyWith: targetWrapper.
! !

!DatabaseRow methodsFor:'configuring'!

shouldBeWritten: aBoolean 
	shouldBeWritten := aBoolean
!

table: aDatabaseTable
	table := aDatabaseTable.
	contents := Array new: (aDatabaseTable fields size).
	contents atAllPut: self class emptySlot.
!

withAllFieldsIn: aRow 
	"aRow represents our original state. Make sure that we have all the fields in aRow, using nil values for any that are missing. This is needed if, e.g. we have been removed from a 1-many relationship, so we don't get a value generated for our foreign key, but we should still write it as a nil. We have to distinguish this from the case of a value that simply hasn't changed."

	aRow isEmpty ifTrue: [^self].
	oldVersion := aRow.
	self numberOfFields = table fields size ifTrue: [^self].
	aRow fieldsAndValidValuesDo: 
			[:eachField :eachValue | 
			(self includesField: eachField) ifFalse: [self at: eachField put: nil]].
	^self
! !

!DatabaseRow methodsFor:'enumerating'!

fieldValuesDo: aBlock 
	contents do: [:each | each == self class emptySlot ifFalse: [aBlock value: each]].
!

fieldsDo: aBlock 
	1 to: contents size do: [:i |
		| value |
		value := contents at: i.
		(value == self class emptySlot) ifFalse: [aBlock value: (self table fields at: i)]].
!

keysAndValuesDo: aBlock
	1 to: contents size do: [:i |
		| value | 
		value := contents at: i.
		value == self class emptySlot ifFalse: [
			aBlock value: (self table fields at: i) value: value contents]].
! !

!DatabaseRow methodsFor:'initializing'!

fieldsAndValidValuesDo: aBlock 
	"If iterating over fields and values, we include wrappers with no value assigned yet. This might or might not be what we want. This one just iterates over ones with actual values"
	self isEmpty ifTrue: [^self].
	table fields do: [:each |  | value |
		value := self at: each ifAbsent: [self class missingFieldIndicator].
		value == self class missingFieldIndicator ifFalse: [
			aBlock value: each value: value]].
!

fieldsAndValuesDo: aBlock 
	table isNil ifTrue: [^self].
	table fields do: [:each | aBlock value: each value: (self at: each ifAbsent: [self class missingFieldIndicator])]
!

initialize
	shouldBeWritten := true.
	forDeletion := false.
	generated := false.
! !

!DatabaseRow methodsFor:'printing'!

equalityStringForField: aDatabaseField

	| stream |
	stream := WriteStream on: (String new: 50).
	self printEqualityStringForField: aDatabaseField on: stream.
	^stream contents.
!

fieldsNeedingUpdate
	"Return a list of the fields that are different from the previous version (if we know what that was)"

	| result |
	oldVersion isNil ifTrue: [^self nonPrimaryKeyFields].
	result := OrderedCollection new: contents size.
	self fieldsDo: 
			[:each | 
			each isPrimaryKey 
				ifFalse: 
					[(oldVersion at: each ifAbsent: [self class missingFieldIndicator]) 
						= (self at: each) ifFalse: [result add: each]]].
	^result
!

printEqualityStringForField: aDatabaseField on: aStream 
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	aDatabaseField printNameOn: aStream withParameters: #().
	aStream nextPutAll: ' = '.
	self printValueOfField: aDatabaseField on: aStream
!

printEqualityTemplateForField: aDatabaseField on: aCommand
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	| bind |
	aDatabaseField printNameOn: aCommand withParameters: #().
	aCommand nextPutAll: ' = '.
	bind := aCommand canBind: (self at: aDatabaseField ifAbsent: [nil]) to: aDatabaseField type.
	bind
		ifTrue: [aCommand nextPutAll: '?']
		ifFalse: [self printValueOfField: aDatabaseField on: aCommand].
!

printEqualityTemplateForField: aDatabaseField on: aStream withBinding: aBoolean
	"Get around PostgreSQL bug.  Qualified names cannot appear in SET expression."
	aDatabaseField printNameOn: aStream withParameters: #().
	aStream nextPutAll: ' = '.
	aBoolean 
		ifTrue: [aStream nextPutAll: '?']
		ifFalse: [self printValueOfField: aDatabaseField on: aStream].
!

printFieldNamesOn: aWriteStream 
	GlorpHelper 
		do: [:each | aWriteStream nextPutAll: each name]
		for: self table fields
		separatedBy: [aWriteStream nextPutAll: ','].
!

printLockEqualityForField: aDatabaseField on: aStream
	| expression |
	expression := ((BaseExpression new getTable: aDatabaseField table) getField: aDatabaseField) get: #= withArguments: (Array with: (oldVersion at: aDatabaseField)).
	expression printSQLOn: aStream withParameters: #().
!

printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '(' , ((table isNil or: [table name isNil]) ifTrue: [''] ifFalse: [table name]),  ')'.
	aStream cr.
	self fieldsAndValuesDo: [:eachField :eachWrapper |
		aStream nextPutAll: '    '.
		eachField printOn: aStream.
		aStream nextPutAll: '->'.
		eachWrapper printOn: aStream.
		aStream cr.].
!

printPrimaryKeyStringOn: aStream 
	"If there is no primary key (i.e. this is a link table) use all the values that we have"
	| fields |
	fields := table primaryKeyFields isEmpty 
		ifTrue: [self fields]
		ifFalse: [table primaryKeyFields].
	GlorpHelper
		do:  [:eachField |
			self printEqualityStringForField: eachField on: aStream]
		for: fields
		separatedBy: [aStream nextPutAll: ' AND '].
!

printPrimaryKeyTemplateOn: aStream
	"If there is no primary key (i.e. this is a link table) use all the values that we have"
	| fields |
	fields := table primaryKeyFields isEmpty 
		ifTrue: [self fields]
		ifFalse: [table primaryKeyFields].
	GlorpHelper
		do:  [:eachField |
			self printEqualityTemplateForField: eachField on: aStream]
		for: fields
		separatedBy: [aStream nextPutAll: ' AND '].
!

printValueOfField: aDatabaseField on: aWriteStream 
	aDatabaseField type print: (self at: aDatabaseField ifAbsent: [nil]) on: aWriteStream.
! !

!DatabaseRow methodsFor:'private'!

at: aField forcePut: aValue 
	"Write the value. If there's a previous value, overwrite it. Use with extreme caution."
	| wrapper |
	aValue isGlorpExpression ifTrue: [self error: 'cannot store expressions in rows'].
	aField table == self table ifFalse: [self error: 'Invalid table'].
	wrapper := self contentsAt: aField ifAbsentPut: [FieldValueWrapper new].
	wrapper privateSetContents: aValue.
	^wrapper.
! !

!DatabaseRow methodsFor:'querying'!

expectsToGetValueFor: aField
	"Are we constrained to a field which is presumably going to get a value, which for the moment must means it's a serial type. This is primarily required for identity column databases, because in other cases we will pre-allocate sequence numbers, and this should never come up"
	| wrapper innerWrappers |
	wrapper := self wrapperAt: aField ifAbsent: [^false].
	innerWrappers := wrapper allInnerWrappers.
	(Array with: wrapper), innerWrappers do: [:eachWrapper |
		eachWrapper containedBy keysDo: [:eachField | (eachField ~~ aField and: [eachField type isSerial]) ifTrue: [^true]]].
	^false.
!

hasValueFor: aField

	^(self wrapperAt: aField ifAbsent: [^false]) hasValue.
!

shouldBeWritten
	table isNil ifTrue: [^shouldBeWritten].
	^shouldBeWritten and: [table shouldBeWritten].
! !

!DatabaseRow methodsFor:'relationships'!

relatedRowsIn: aRowMap
	| relatedRows |
	relatedRows := OrderedCollection new.
	self table foreignKeyConstraints
		do:
			[:eachConstraint | relatedRows addAll: (eachConstraint rowsRelatedTo: self)].
	^relatedRows.
! !

!DatabaseRow methodsFor:'sequencing'!

needsValueFromSequence: aSequence 
	self table fields do: 
			[:each | 
			((self shouldGenerateValueFor: each) and: [each type sequence == aSequence]) 
				ifTrue: [^true]].
	^false
!

postWriteAssignSequencesUsing: anAccessor

	self table fields do: [:each |
		(self hasValueFor: each) ifFalse: [
			each type postWriteAssignSequenceValueFor: each in: self using: anAccessor]].
!

preWriteAssignSequencesUsing: anAccessor
	"Skip if we've already been generated"
	generated ifTrue: [^self].
	self table fields do: 
			[:each | 
			(self shouldGenerateValueFor: each) 
				ifTrue: 
					[each type 
						preWriteAssignSequenceValueFor: each
						in: self
						using: anAccessor]].
	generated := true.
!

shouldGenerateValueFor: aField
	aField isGenerated ifFalse: [^false].
	^aField type generatesOverExistingValues
		ifTrue: [true]
		ifFalse: [(self hasValueFor: aField) not].
! !

!DatabaseRow methodsFor:'testing'!

equals: aRow 
	self forDeletion = aRow forDeletion ifFalse: [^false].
	self fieldsAndValuesDo: 
			[:eachField :eachWrapper | 
			| otherValue |
			otherValue := aRow at: eachField ifAbsent: [self class missingFieldIndicator].
			eachWrapper = otherValue ifFalse: [^false]].
	^true
!

hash
	| basic |
	basic := self table hash.
	self fieldsAndValuesDo: 
			[:eachField :eachValue | 
			basic := basic bitXor: eachField hash.
			basic := basic bitXor: eachValue hash].
	^basic.
!

isEmpty
	| empty |
	contents isNil ifTrue: [^true].
	empty := self class emptySlot.
	contents do: [:each | each == empty ifFalse: [^false]].
	^true.
! !

!DatabaseRow class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/DatabaseRow.st,v 1.1 2006/07/26 08:00:14 boris Exp $'
! !
