"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Descriptor
	instanceVariableNames:'tables mappings system mappedFields cachePolicy typeResolver
		mapsPrimaryKeys keyFields classModel multipleTableJoin
		imaginaryTableReplacements mappingsForFields'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

Descriptor comment:'For each persistent class, there is a descriptor. This is what tells GLORP how to read and write objects of this class. The descriptor is not actually a very complicated object, because most of the information is in the mappings described below. But the descriptor does keep hold of a few per-class pieces of information, notably

Instance Variables:
	cachePolicy	<CachePolicy>	The cache policy to use. If left blank, default to what the system specifies.
	describedClass	<Class>	The persistent class that we map.
	mappedFields	<Collection of: DatabaseField>	The database fields we have mappings for (cached computed value)
	mappings	<(Collection of: Mapping)>	Our mappings.
	mapsPrimaryKeys	<Object>	Do we map the primary keys (cached computed value)
	multipleTableCriteria	<(Collection of: Join)>	description of multipleTableCriteria
	system	<DescriptorSystem>	The descriptor system that we belong to.
	tables	<(SequenceableCollection of: (DatabaseTable))>	The tables that we map
	typeResolver	<TypeResolver>	description of typeResolver

'
!


!Descriptor class methodsFor:'instance creation'!

new

	^super new initialize.
! !

!Descriptor class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Descriptor methodsFor:'As yet unclassified'!

imaginaryTableReplacements
	imaginaryTableReplacements isNil ifTrue: [imaginaryTableReplacements := OrderedCollection new: 4].
	^imaginaryTableReplacements.
! !

!Descriptor methodsFor:'accessing'!

addMapping: aMapping 
	self basicAddMapping: aMapping.
	aMapping hasAttribute ifTrue: [
		(aMapping classModel attributeNamed: aMapping attribute name) == aMapping attribute ifFalse: [self error: 'If you have a classModel you must use newMapping: rather than addMapping:']].
!

addMultipleTableJoin: anExpression 
	self multipleTableJoin add: anExpression
!

addTable: aDatabaseTable 
	tables add: aDatabaseTable.
!

allMappingsForField: aField 
	"Return all of the mappings that use this field"
	^mappings select: [:each | each mappedFields includes: aField]
!

basicAddMapping: aMapping 
	mappings := mappings, (Array with: aMapping).
	aMapping descriptor: self.
	self resetMappedFields.
!

cachePolicy
	cachePolicy isNil ifTrue: [^system cachePolicy].
	^cachePolicy
!

cachePolicy: aCachePolicy
	cachePolicy:= aCachePolicy
!

classModel
	^classModel
!

classModel: aGlorpClassModel
	classModel := aGlorpClassModel
!

describedClass
	^classModel describedClass.
!

describedClass: aClass
	classModel := self system classModelFor: aClass.
!

directMappingForField: aField 
	"Return a single, direct mapping for this field. There may conceivably be more than one, but they all have to agree, so it shouldn't matter as far as the value. There may also be none."

	^(self mappingsForFields at: aField ifAbsent: [^nil]) 
		detect: [:each | each isRelationship not]
		ifNone: [nil]
!

directMappingsForField: aField 
	"Return all the relationship mappings that use this field"

	^(self mappingsForFields at: aField ifAbsent: [^#()]) 
		reject: [:each | each isRelationship].
!

fieldsForSelectStatement
	| myFields inheritedFields |
	myFields := self mappedFields.
	inheritedFields := self typeResolver fieldsForSelectStatement.
	^inheritedFields isEmpty ifTrue: [myFields] ifFalse: [myFields, inheritedFields].
!

imaginaryTableIsDefinedByRelationFrom: sourceAttributeName to: targetAttributeName
	self imaginaryTableReplacements add: (Array with: sourceAttributeName with: targetAttributeName).
!

initialize

	mappings := Array new.
	tables := OrderedCollection new: 1.
!

keyFields
	"Return the fields we use as primary keys for caching purposes. Typically, these are just going to be the primary keys of our primary table. Objects intended to be embedded won't map the primary keys at all. Weird objects may supply their own keys (e.g. functions rather than fields directly)"

	^keyFields == nil
		ifTrue:
			[self mapsPrimaryKeys
				ifTrue: [self primaryTable primaryKeyFields]
				ifFalse: [nil]]
		ifFalse: [keyFields].
!

keyFields: aFieldCollection
	"Set the fields we use as primary keys for caching purposes. Typically, these are just going to be the primary keys of our primary table"
	keyFields := aFieldCollection.
!

mappedFields
	"Return all the fields that are mapped, in the order that they occur in the table. This specifically omits fields that are not in the selected set of tables (so e.g. even if a mapping uses a link table field, if it's not in the list of tables to select from, we don't want it). We also include mapped, non-constant expressions, which we would normally expect to be functions"
	mappedFields isNil ifTrue: [
		| fieldSet |
		fieldSet := IdentitySet new: mappings size.
		mappings do: [:each | fieldSet addAll: each mappedFields].
		mappedFields := OrderedCollection new.
		tables do: [:each |
			each fields do: [:eachField | (fieldSet includes: eachField) ifTrue: [mappedFields add: eachField]]].
		fieldSet do: [:each | 
 			(each isGlorpExpression and: [each isConstantExpression not]) ifTrue: [mappedFields add: each]]].
	^mappedFields.
!

mappingForAttributeNamed: aSymbol

	^mappings detect: [:each | each attributeName == aSymbol] ifNone: [
		self isForAssociation ifTrue: [self valueDescriptor mappingForAttributeNamed: aSymbol]]
!

mappingForField: aField

	^(mappingsForFields at: aField ifAbsent: [^nil]) first.
!

mappingsForFields
	"Return a dictionary from fields to the mappings that contain them. This specifically omits fields that are not in the selected set of tables (so e.g. even if a mapping uses a link table field, if it's not in the list of tables to select from, we don't want it). We also include mapped, non-constant expressions, which we would normally expect to be functions"
	mappingsForFields isNil ifTrue: [
		mappingsForFields := IdentityDictionary new: mappings size.
		mappings do: [:eachMapping | 
			eachMapping mappedFields do: [:eachField | 
				(mappingsForFields at: eachField ifAbsentPut: [OrderedCollection new]) add: eachMapping]]].
	^mappingsForFields.
!

multipleTableJoin
	multipleTableJoin isNil
		ifTrue: [multipleTableJoin := OrderedCollection new: 1].
	^multipleTableJoin.
!

primaryTable
	^tables first.
!

relationshipMappingForField: aField 
	"Return a single, direct mapping for this field. There may conceivably be more than one, but they all have to agree, so it shouldn't matter as far as the value. There may also be none."

	^(self mappingsForFields at: aField ifAbsent: [^nil]) 
		detect: [:each | each isRelationship]
		ifNone: [nil]
!

relationshipMappingsForField: aField 
	"Return all the relationship mappings that use this field"

	^(self mappingsForFields at: aField ifAbsent: [^#()]) 
		select: [:each | each isRelationship].
!

resetMappedFields
	mappedFields := nil.
!

session

	^system session.
!

system
	^system
!

system: anObject
	system := anObject
!

table
	^tables first.
!

table: aDatabaseTable

	tables add: aDatabaseTable.
!

tables
	^tables
!

typeMapping
	^mappings detect: [ :each | each isTypeMapping ] ifNone: [
		| mapping |
		mapping := IdentityTypeMapping new.
		self addMapping: mapping.
		mapping]
!

typeMapping: aMapping
	self addMapping: aMapping
!

typeResolver
	typeResolver isNil
		ifTrue: [  IdentityTypeResolver new register: self  ].
	^typeResolver
!

typeResolver: anObject
	typeResolver := anObject
!

valueDescriptor
	| valueMapping |
	valueMapping := self mappingForAttributeNamed: #value.
	valueMapping isNil ifTrue: [^nil].
	^valueMapping referenceDescriptor.
! !

!Descriptor methodsFor:'creating mappings'!

newMapping: aClass
	| mapping |
	mapping := aClass new.
	self basicAddMapping: mapping.
	^mapping.
! !

!Descriptor methodsFor:'deleting'!

createDeleteRowFor: anObject table: aTable in: aRowMap
	"Create records for rows that require deletion"

	(self primaryKeyMappingsFor: aTable)
		do: [:each | 
			each first mapFromObject: anObject intoRowsIn: aRowMap].
!

createDeleteRowsFor: anObject in: aRowMap
	"Create records for rows that require deletion"

	|  |
	anObject class == self describedClass ifFalse: [self error: 'wrong descriptor for this object'].
	"If we don't map primary keys, then we're something that gets embedded, and a delete isn't necessary"
	self mapsPrimaryKeys ifFalse: [^self]. 

	self tables do: [:eachTable |
		self createDeleteRowFor: anObject table: eachTable in: aRowMap.
		aRowMap rowForTable: eachTable withKey: anObject ifPresent: [:eachRow | eachRow forDeletion: true]].
	"It's possible that we might not have any direct mapping for a secondary table's primary keys, so 
allow the multiple table criteria to specify them if that's the only one. If they're not, then they don't do any harm"
	self multipleTableJoin do: [:each |
		each mapFromSource: anObject andTarget: anObject intoRowsIn: aRowMap.
		each allTables do: [:eachTable |
			aRowMap rowForTable: eachTable withKey: anObject ifPresent: [:eachRow | eachRow forDeletion: true]]].

	self mappings do: [:each | each  createDeleteRowFor: anObject in: aRowMap].
!

primaryKeyMappingsFor: aTable
	"Return our primary key mappings as a collection of mappings, one collection per primary key"

	| allMappings |
	allMappings := Dictionary new: aTable primaryKeyFields size.
	aTable primaryKeyFields
		do: [:each | 
			| mappingsForField |
			mappingsForField := self mappingsForFields at: each ifAbsent: [nil].
			mappingsForField isNil ifFalse: [allMappings at: each put: mappingsForField]].
	^allMappings.
! !

!Descriptor methodsFor:'internal'!

buildReplacementTableSelect
	"We have an imaginary table which is defined by selecting from our attributes instead of from the table. Build the query for that."

	| sourceMappings targetMappings mainQuery subQuery |
	mainQuery := nil.
	sourceMappings := imaginaryTableReplacements
		collect: [:each | self mappingForAttributeNamed: each first].
	targetMappings := imaginaryTableReplacements
		collect: [:each | self mappingForAttributeNamed: each last].
	sourceMappings
		with: targetMappings
		do:
			[:eachSourceMapping :eachTargetMapping | 
			subQuery := self
				buildReplacementTableSelectFrom: eachSourceMapping
				to: eachTargetMapping.
			mainQuery := mainQuery isNil
				ifTrue: [subQuery]
				ifFalse: [mainQuery unionAll: subQuery]].
	mainQuery session: self session.
	self primaryTable replacementSubSelect: mainQuery.
!

buildReplacementTableSelectFrom: sourceMapping to: targetMapping
	"Build a select statement that combines the two mappings. We expect that both are relationships from a single object whose table is imaginary, so we need to knit together the information from the mappings to join the real tables directly together."

	| fullJoin targetJoin sourceJoin subQuery join |
	subQuery := Query returningManyOf: sourceMapping referenceClass.
	subQuery session: self session.	
	subQuery setUpExpressions.
	"Invert because these will both start with the imaginary one on the left and we want to merge them together"
	sourceJoin := (sourceMapping join
		replacing: self primaryTable
		withEquivalenceFrom: targetMapping join) inverseJoin.
	targetJoin := targetMapping join
		replacing: self primaryTable
		withEquivalenceFrom: sourceMapping join.
	fullJoin := sourceJoin = targetJoin
		ifTrue: [sourceJoin]
		ifFalse: [sourceJoin , targetJoin].
	join := fullJoin asGeneralGlorpExpression asExpressionJoiningSource: subQuery baseExpression toTarget: subQuery baseExpression.
	subQuery addJoin: join.
	sourceMapping usesLinkTable ifTrue: [subQuery addJoin: (sourceMapping  expressionFromLinkToReferenceTableWithBase: subQuery baseExpression)].
	targetMapping usesLinkTable ifTrue: [subQuery addJoin: (targetMapping  expressionFromLinkToReferenceTableWithBase: subQuery baseExpression)].
	self
		setRetrievalExpressionsInReplacementTableForQuery: subQuery
		fromMapping: sourceMapping
		toMapping: targetMapping.
	^subQuery.
!

describedConcreteClassFor: row withBuilder: builder
	"Lookup the class that is represented by the row when there is a possibility
		of this row representing any class within a hierarchy. "
	^self typeResolver describedConcreteClassFor: row withBuilder: builder descriptor: self
!

readBackNewRowInformationFor: anObject
!

referencedIndependentObjectsFrom: anObject do: aBlock

	mappings do: [:each |
		(each referencedIndependentObjectsFrom: anObject) do: [:eachReferencedObject |
			aBlock value: eachReferencedObject]].
!

referencedIndependentObjectsWithMappingsFrom: anObject do: aBlock

	mappings do: [:each |
		(each referencedIndependentObjectsFrom: anObject) do: [:eachReferencedObject |
			aBlock value: eachReferencedObject value: each]].
!

setRetrievalExpressionsInReplacementTableForQuery: subQuery fromMapping: sourceMapping toMapping: targetMapping
	self primaryTable fields
		do:
			[:each | 
			| sourceReplacement replacement expression |
			sourceReplacement := sourceMapping join targetForSource: each.
			replacement := sourceReplacement isNil
				ifTrue: [targetMapping join targetForSource: each]
				ifFalse: [sourceReplacement].
			expression := replacement isGlorpExpression
				ifTrue: [replacement] 
				ifFalse: [(BaseExpression new getTable: replacement table) getField: replacement].
			expression alias: each name.
			subQuery retrieve: expression].
! !

!Descriptor methodsFor:'mapping'!

createRowsFor: anObject in: aRowMap

	|  |
	anObject class == self describedClass ifFalse: [self error: 'wrong descriptor for this object'].
	mappings do: [:each |
		each stopIfDebuggingWrite.
		each mapFromObject: anObject intoRowsIn: aRowMap].

	"This is a bit of a hack/heuristic. If the multiple table join is an outer join, and the row doesn't exist yet, then assume we won't need it and don't do the join. It means none of our mappings wrote to it. Hope that in that case no one else's will. It seems reasonable that we wouldn't map an object to multiple tables unless we planned to map some field within it, but it's still a bit of a risk."
	self multipleTableJoin do: [:each |
		(each isOuterJoin not or: [
			each allTables anySatisfy: [:eachTable | aRowMap includesRowForTable: eachTable withKey: anObject]])
				ifTrue: [each mapFromSource: anObject andTarget: anObject intoRowsIn: aRowMap]].
!

mappings
	^ReadStream on: mappings
!

populateObject: anObject inBuilder: anElementBuilder
	"Answer an object using the values for the specified fields."

	mappings
		do: [:each | 
			each stopIfDebuggingRead.
			each mapObject: anObject inElementBuilder: anElementBuilder].
!

primaryKeyExpressionFor: anObject basedOn: anExpression relation: aSymbol
	"anObject can be either a real domain object, or a constant expression containing it, or an expression representing the object"

	| expression value pkMappings |
	value := anObject isGlorpExpression
		ifTrue:
			[anObject isConstantExpression ifTrue: [anObject value] ifFalse: [anObject]]
		ifFalse:
			[anObject class == self describedClass
				ifFalse: [self error: 'Wrong descriptor for this object'].
			anObject].
	self isForAssociation ifTrue: [
		"We just rewrite as self value = x"
		^((anExpression get: #value) get: aSymbol withArguments: (Array with:  value)) rewriteEquality].
	expression := nil.
	pkMappings := self primaryKeyMappings.
	(pkMappings isEmpty ifTrue: [self mappings] ifFalse: [pkMappings])
		do:
			[:each | 
			| clause |
			clause := each
				expressionFor: value
				basedOn: anExpression
				relation: aSymbol.
			expression := aSymbol = #<> ifTrue: [clause OR: expression] ifFalse: [clause AND: expression]].
	^expression.
!

primaryKeyFor: anObject
	| fields |
	anObject class == self describedClass
		ifFalse: [self error: 'Wrong descriptor for this object'].
	fields := self primaryTable primaryKeyFields.
	^fields size = 1
		ifTrue: [self valueOfField: (fields at: 1) fromObject: anObject]
		ifFalse:
			[fields collect: [:each | self valueOfField: each fromObject: anObject]].
!

primaryKeyMappings
	"Return things that map to our primary key fields. Prefer direct mappings"
	| fields pkMappings |
	fields := self primaryTable primaryKeyFields.
	pkMappings := OrderedCollection new: fields size.
	fields do: [:each |  | mapping |
		mapping := self directMappingForField: each.
		mapping isNil ifTrue: [mapping := self mappingForField: each].
		mapping isNil ifFalse: [pkMappings add: mapping]].
	^pkMappings.
!

readBackNewRowInformationFor: anObject in: aRowMap 
	anObject yourSelf class == self describedClass ifFalse: [self error: 'wrong descriptor for this object'].
	mappings do: [:each | each readBackNewRowInformationFor: anObject fromRowsIn: aRowMap]
!

valueOfField: aField fromObject: anObject
	| directMapping notFound |
	directMapping := self directMappingForField: aField.
	directMapping isNil
		ifFalse: [^directMapping valueOfField: aField fromObject: anObject].
	notFound := Object new.
	(self relationshipMappingsForField: aField)
		do:
			[:each | 
			| candidate |
			(each hasValueThatCanProvideField: aField from: anObject)
				ifTrue: [
					candidate := each valueOfField: aField fromObject: anObject ifNone: [notFound].
					candidate == notFound ifFalse: [^candidate]]].
	^nil
! !

!Descriptor methodsFor:'printing'!

printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: '('.
	self describedClass printOn: aStream.
	aStream nextPutAll: ')'.
! !

!Descriptor methodsFor:'testing'!

computeMapsPrimaryKeys
	| primaryKeyFields |
	primaryKeyFields := self primaryTable primaryKeyFields.
	primaryKeyFields isEmpty ifTrue: [^false].
	primaryKeyFields
		do: [:each | (self mappedFields anySatisfy: [ :eachMappedField |
			eachMappedField = each]) ifFalse: [^false]].
	^true.
!

isForAssociation
	^self describedClass == Association.
!

isTypeMappingRoot
	^self typeResolver isTypeMappingRoot: self
!

mapsPrimaryKeys
	mapsPrimaryKeys isNil ifTrue: [mapsPrimaryKeys := self computeMapsPrimaryKeys].
	^mapsPrimaryKeys
!

supportsOrdering

	typeResolver isNil ifTrue: [^true].
	^typeResolver class ~= HorizontalTypeResolver.
! !

!Descriptor methodsFor:'tracing'!

defaultTracing
	^Tracing new.
!

setupTracing: aTracing
	"Find all the other objects that need to be read when this one is read"

	self trace: aTracing context: aTracing base.
!

trace: aTracing context: anExpression
	"For each mapping, check if the relationship is involved in the set of things
to be read"
	mappings do: [:each |
		each trace: aTracing context: anExpression].
! !

!Descriptor methodsFor:'type mapping'!

allDescribedConcreteClasses
	^ self typeMapping allDescribedConcreteClasses
!

registerTypeResolver: aResolver
	self registerTypeResolver: aResolver abstract: false
!

registerTypeResolver: aResolver abstract: shouldBeAbstract
	shouldBeAbstract ifFalse: [ self beAbstract ].
	aResolver register: self.
!

typeMappingCriteriaIn: base
	| r l |
	r := ConstantExpression for: self typeMapping keys.
	l := FieldExpression forField: self typeMapping field basedOn: base.
	^l in: r
! !

!Descriptor methodsFor:'type resolution'!

classesRequiringIndependentQueries

	^ self typeResolver classesRequiringIndependentQueriesFor: self describedClass
!

typeMappingRoot
	^self typeResolver typeMappingRoot
!

typeMappingRootDescriptor
	^self typeResolver typeMappingRootDescriptor
! !

!Descriptor methodsFor:'validation'!

validate
	"Do any tests we can to make sure that this definition makes sense in this context"

	| readableMappings attributeNames |
	self mappings do: [:eachMapping |
		eachMapping validate].
	"Validate that we have at most one readable mapping for an instance variable"
	readableMappings := self mappings select: [:each | each canRead and: [each isTypeMapping not and: [each attributeName isEmpty not]]].
	attributeNames := readableMappings collect: [:each | each attributeName].
	(attributeNames asSet size = readableMappings size) ifFalse: [self error: 'multiple readable mappings for one instance variable'].
! !

!Descriptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Descriptor.st,v 1.1 2006/07/26 08:03:27 boris Exp $'
! !
