"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#ElementBuilder
	instanceVariableNames:'instance requiresPopulating key expression cursor
		fieldTranslations isExpired row query'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

ElementBuilder comment:'This is the abstract superclass of builders. These assemble information, either primitive data or objects, from a database result set.

Subclasses must implement the following messages:
	building objects
		buildObjectFrom:
		findInstanceForRow:useProxy:
	selecting fields
		fieldsFromMyPerspective

Instance Variables:
	expression	<MappingExpression>	The expression we''re mapping. e.g. if the query is reading people, this might be the expression corresponding to "each address", meaning that we build the address object related to the main Person instance by the given relationship. 
	fieldTranslations	<Array of: Integer>	 The translation of the field positions from where they are in the descriptor to where they are in the row we''re reading. This is done so we can read the rows efficiently, by index, rather than doing lots of dictionary lookups by name. If we''re doing a simple read, the translations will probably be a no-op, but if we read multiple objects, some of the fields will be in different positions than they are in our table definition.
	instance	<Object>	The thing we''re constructing.
	isExpired	<Boolean>	If our instance is in cache, we use that instead. However, if the instance has expired, then we do something different (most likely force a refresh) than if it''s still alive.
	key	<Object>	The key for this row. This is lazily computed, and "self" is used a special marker to indicate that it hasn''t been computed yet.
	query	<AbstractReadQuery>	the query that we''re building results for.
	requiresPopulating	<Boolean>	Do we need to populate the object. Will be false if the object was found in cache and hasn''t expired.
	row	<Array>	The database results. May actually be a result set row of some sort rather than an array, depending on the dialect, but should always respond to indexing protocol.

'
!


!ElementBuilder class methodsFor:'instance creation'!

basicFor: anExpression in: aQuery

	^(self basicClassFor: anExpression in: aQuery) new
		expression: anExpression;
		query: aQuery;
		setup.
!

for: anExpression in: aQuery

	^(self classFor: anExpression in: aQuery) new
		expression: anExpression;
		query: aQuery;
		setup.
!

new

	^super new initialize.
! !

!ElementBuilder class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ElementBuilder class methodsFor:'private'!

basicClassFor: anExpression in: aQuery

	^(anExpression hasDescriptor and: [anExpression mapsSimpleObject not])
		ifTrue: [ObjectBuilder] 
		ifFalse: [DataElementBuilder].
!

classFor: anExpression in: aQuery
	
	((aQuery returnsDictionary and: [anExpression == aQuery baseExpression])
		or: [anExpression returnsDictionary])
			ifTrue: [^DictionaryBuilder].
	^self basicClassFor: anExpression in: aQuery.
! !

!ElementBuilder methodsFor:'accessing'!

canCauseDuplicateRows
	^false.
!

cursor
	^cursor
!

cursor: aGlorpCursoredStream
	cursor := aGlorpCursoredStream
!

expression
	^expression
!

expression: anExpression
	expression := anExpression.
!

fieldTranslations
	^fieldTranslations
!

fieldTranslations: aDictionary 
	fieldTranslations := aDictionary.
!

instance
	^instance
!

instanceValue
	^instance
!

query
	^query
!

query: aQuery 
	query := aQuery
!

requiresDistinct

	^expression requiresDistinct.
!

requiresPopulating
	^true
!

returnValueIn: aCursoredStream
	^self instance
!

row
	^row
!

row: anArray
	"Since nil is a possible key value, use self as a special marker to indicate we haven't found the key yet"
	row == anArray ifFalse: [
		key := self].
	row := anArray.
!

session
	^expression descriptor session.
!

subBuilders
	^#().
!

system
	^self session system.
!

withAllSubBuilders
	^Array with: self.
! !

!ElementBuilder methodsFor:'building objects'!

buildObjectFrom: anArray 
	self subclassResponsibility.
!

findInstanceForRow: aRow proxyType: proxyType
	self subclassResponsibility.
!

knitResultIn: aSimpleQuery
	"Connect up our built object with any other builders that use the same thing"

	^self.
!

registerObjectInUnitOfWork
	"If there is a current unit of work, then we must register in it, after population because that way the state is already in place. The nil checks are mostly for safety during unit tests, as those conditions should never occur in real use"
	query isNil ifTrue: [^self].
	query session isNil ifTrue: [^self].
	query session register: instance.
! !

!ElementBuilder methodsFor:'copying'!

copy
	^self shallowCopy postCopy.
!

postCopy
	instance := nil.
	row := nil.
	requiresPopulating := false.
	isExpired := false.
!

reconcileWithBuilders: oldCollection andCopy: newCollection
! !

!ElementBuilder methodsFor:'executing'!

hasFieldTranslations
	^self fieldTranslations notNil
! !

!ElementBuilder methodsFor:'initializing'!

initialize
!

setup
	"Do any initialization required once we've got our parameters"
	^self.
! !

!ElementBuilder methodsFor:'selecting fields'!

fieldsForSelectStatement

	^self subclassResponsibility.
!

fieldsFromMyPerspective
	self subclassResponsibility.
! !

!ElementBuilder methodsFor:'testing'!

buildsObjects
	^false.
! !

!ElementBuilder methodsFor:'translating fields'!

translateFieldPosition: aDatabaseField 
	fieldTranslations isNil ifTrue: [^aDatabaseField position].
	^fieldTranslations at: aDatabaseField.
!

valueOf: anExpression
	^expression valueInBuilder: self.
!

valueOfField: aField
	"aField is either a database field, or a constant expression containing a non-varying value that isn't derived from the row"
	(aField class == ConstantExpression) ifTrue: [^aField value].
	^self row atIndex: (self translateFieldPosition: aField).
!

valueOfField: aField in: aRow
	"Since the elementBuilder now holds the row, #valueOfField: is preferred protocol, but some things (e.g. ad hoc mapping blocks) might still be using this, so left for compatibility"
	(aField class == ConstantExpression) ifTrue: [^aField value].
	^aRow atIndex: (self translateFieldPosition: aField).
! !

!ElementBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ElementBuilder.st,v 1.1 2006/07/26 08:04:41 boris Exp $'
! !
