"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#Join
	instanceVariableNames:'sources targets base mapping outerJoin'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

Join comment:'This is a specialized variety of expression that is more constrained and is used for defining relationships. It has two main purposes
 - ease of construction: Relationships are normally defined by field to field equality expressions (my foreign key field = his primary key field). These are more tedious to create via block expressions, so this provides a simpler syntax.
 - constrained semantics. These define both read and write for the relationship, so fully general expressions won''t work (most notably, relations other than equality are hard to write). Using a primaryKeyExpression ensures that we satisfy these constraints.

I''m not completely sure this class is a good idea. It makes for an annoying assymetry between different kinds of expressions. This is especially notable now that we allow sources to be constants. It''s possible that all we need is an expression constructor that generates real expressions, but with more convenient syntax and ensuring that the constraints are met.

Note that although these are typically fk=pk, it''s allowed to be the other way around -- i.e. our object-level relationships can be the opposite of the way the fk''s "point" in the database.

Instance Variables:
	base	<BaseExpression>	The base on which we are built. Mostly used if we want to convert this into a real expression.
	sources	<SequenceableCollection of: (DatabaseField | ConstantExpression)> The source fields (typically the foreign keys)
	targets	<SequenceableCollection of: DatabaseField> The target fields (typically the targets of the foreign keys)

'
!


!Join class methodsFor:'instance creation'!

from: aField to: anotherField

	^self new
		addSource: aField target: anotherField.
!

from: from1Field to: to1Field
from: from2Field to: to2Field

	^self new
		addSource: from1Field target: to1Field;
		addSource: from2Field target: to2Field.
!

from: from1Field to: to1Field
from: from2Field to: to2Field
from: from3Field to: to3Field

	^self new
		addSource: from1Field target: to1Field;
		addSource: from2Field target: to2Field;
		addSource: from3Field target: to3Field.
!

from: from1Field to: to1Field
from: from2Field to: to2Field
from: from3Field to: to3Field
from: from4Field to: to4Field

	^self new
		addSource: from1Field target: to1Field;
		addSource: from2Field target: to2Field;
		addSource: from3Field target: to3Field;		
		addSource: from4Field target: to4Field
!

fromAll: sourceFields toAll: targetFields

	^self new
		sourceFields: sourceFields targetFields: targetFields.
!

new

	^super new initialize.
! !

!Join class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!Join methodsFor:'accessing'!

allSourceFields

	^sources
!

allTables

	^(targets collect: [:each | each table]) asSet.
!

allTargetFields

	^targets
!

base
	^base.
!

base: aBaseExpression
	base := aBaseExpression.
!

from: aField to: anotherField

	^self addSource: aField target: anotherField.
!

hasDescriptor

	^false.
!

numberOfParameters
	^sources size
!

outerJoin
	^outerJoin.
!

outerJoin: aBoolean
	outerJoin := aBoolean.
!

queryLevelBaseExpression
	^self ultimateBaseExpression.
!

targetKeys

	^targets.
!

ultimateBaseExpression

	^base.
! !

!Join methodsFor:'api'!

addSource: aField target: anotherField 
	| value targetValue |
	value := (self isConstant: aField) 
				ifTrue: [ConstantExpression for: aField]
				ifFalse: [aField].
	sources add: value.
	targetValue := anotherField class == SimpleQuery
		ifTrue: [anotherField asGlorpExpressionOn: base] 
		ifFalse: [anotherField].
	(self isConstant: targetValue) ifTrue: [
		self error: 'You are attempting to set a constant value as the target of a relationship. I suspect you want to set it on the source instead'].
	targets add: targetValue
!

asGlorpExpression

	^self.
!

asGlorpExpressionForDescriptor: aDescriptor

	base descriptor: aDescriptor.
!

asGlorpExpressionOn: aBaseExpression
	| copy |
	copy := self copy.
	copy base: aBaseExpression.
	copy outerJoin: outerJoin.
	^copy.
!

asOuterJoin
	^self copy beOuterJoin.
!

beOuterJoin
	outerJoin := true.
!

mapFromSource: sourceObject andTarget: targetObject intoRowsIn: aRowMap
	(sourceObject isNil or: [targetObject isNil]) ifTrue: [^self].
	1
		to: sources size
		do:
			[:i | 
			| eachSourceField eachTargetField |
			eachSourceField := sources at: i.
			eachTargetField := targets at: i.
			(self isConstant: eachSourceField)
				ifTrue: [aRowMap at: eachTargetField key: targetObject put: eachSourceField value]
				ifFalse:
					[aRowMap
						unifyField: eachSourceField
						key: sourceObject
						withField: eachTargetField
						key: targetObject]].
! !

!Join methodsFor:'comparing'!

= aJoin
	aJoin class == self class ifFalse: [^false].
	aJoin allSourceFields = sources ifFalse: [^false].
	aJoin allTargetFields = targets ifFalse: [^false].
	^true.
! !

!Join methodsFor:'converting'!

, aJoin
	| newJoin |
	aJoin isNil ifTrue: [^self].
	aJoin == self ifTrue: [^self].
	newJoin := (self class fromAll: (sources, aJoin allSourceFields) toAll: (targets, aJoin targetKeys)) removeDuplicates.
	"If either one of the contributors is an outer join, make the result be an outer join"
	newJoin outerJoin: (self outerJoin | aJoin outerJoin).
	^newJoin
!

asExpressionJoiningSource: source toTarget: target
	"Create a general expression which represents this relationship where the values of the targets (which are normally parameters) are supplied out of the context provided by 'target' and the source fields are referencing things out of the context of source. Produces something suitable for ANDing into an expression when doing a join
   Example: If we had CUSTOMER.ADDRESS_ID = ADDRESS.ID as a parameter, and we want to AND this into an expression [:customer | customer address street = 'Main'] then we have customer as a base, and we get 
	(customer.ADDRESS.STREET = 'Main') AND (customer.CUSTOMER.ADDRESS_ID = customer.ADDRESS.ID)
	The primary key expression for the relationship has been translated into field references into the customer and address tables in a particular context."

	| sourceFieldExpression targetFieldExpression completeExpression relationExpression |
	completeExpression := nil.
	sources with: targets
		do: 	[:sourceField :targetField | 
			sourceFieldExpression := source getFieldKnownNotToBeAName: sourceField.
			targetFieldExpression := target getFieldKnownNotToBeAName: targetField.
			relationExpression := sourceFieldExpression equals: targetFieldExpression.
			completeExpression := relationExpression AND: completeExpression].
	outerJoin ifTrue: [completeExpression beOuterJoin].
	^completeExpression
!

asGeneralGlorpExpression
	"Convert this to a 'normal' expression representing the same information"

	| main clause |
	main := nil.
	sources with: targets
		do: 
			[:eachSource :eachTarget | 
			| srcExp targetExp |
			srcExp := self sourceExpressionFor: eachSource.
			targetExp := self targetExpressionFor: eachTarget.
			"Reversing the order is important because the source is the parameter, and sql won't accept '27 = FOO' "
			clause := targetExp equals: srcExp.
			main := main == nil ifTrue: [clause] ifFalse: [main AND: clause]].
	outerJoin ifTrue: [main beOuterJoin].
	^main
!

asGeneralGlorpExpressionWithFields
	"The normal conversion of a join is to an expression equating fields and parameters. Instead, generate one with field to field comparisons"

	| main clause |
	main := nil.
	sources with: targets
		do: 
			[:eachSource :eachTarget | 
			| srcExp targetExp |
			srcExp := self targetExpressionFor: eachSource.
			targetExp := self targetExpressionFor: eachTarget.
			clause := srcExp equals: targetExp.
			main := main == nil ifTrue: [clause] ifFalse: [main AND: clause]].
	outerJoin ifTrue: [main beOuterJoin].
	^main
!

generalExpressionPart
	^nil.
!

inverseJoin
	| inverse |
	inverse := self class new
		base: base;
		sourceFields: targets
		targetFields: sources.
	inverse mapping: self mapping.
	^inverse.
!

isConstant: aTarget
	"The target can be either a constant (which gets turned into a ConstantExpression) or (usually) a DatabaseField, representing a parameter to the query"
	(aTarget class == DatabaseField) ifTrue: [^false].
	(aTarget isGlorpExpression and: [aTarget isConstantExpression not]) ifTrue: [^false].
	^true.
!

removeDuplicates
	| existing newSources newTargets |
	existing := Set new.
	newSources := OrderedCollection new: sources size.
	newTargets := OrderedCollection new: targets size.
	sources
		with: targets
		do:
			[:eachSource :eachTarget | 
			| association |
			association := eachSource -> eachTarget.
			(existing includes: association)
				ifFalse:
					[existing add: association.
					newSources add: eachSource.
					newTargets add: eachTarget]].
	sources := newSources.
	targets := newTargets.
!

replacing: aTable withEquivalenceFrom: aJoin
	"Create a new join in which references to fields in aTable have been replaced with the equivalent based on a Join (we assume that aTable references are in the sources of aJoin). If the field doesn't occur in the translation, just remove it"
	| newSources newTargets newSource |
	newSources := OrderedCollection new.
	newTargets := OrderedCollection new.
	self fieldsDo: [:eachSource :eachTarget |
		(eachSource isGlorpExpression not and: [eachSource table = aTable])
			ifTrue: [
				newSource := aJoin targetForSource: eachSource.
				newSource isNil ifFalse: [
					newSources add: newSource.
					newTargets add: eachTarget]]].
	^self class fromAll: newSources toAll: newTargets.
!

sourceExpressionFor: source
	
	^((self isConstant: source) or: [source isMappable not]) "This might be a subselect"
		ifTrue: [source]
		ifFalse: [base getParameter: source].
!

targetExpressionFor: eachTarget 
	^(self isConstant: eachTarget)
		ifTrue: [eachTarget]
		ifFalse: [
			eachTarget isGlorpExpression 
			ifTrue: [eachTarget] 
			ifFalse: [(base getTable: eachTarget table) getField: eachTarget]].
! !

!Join methodsFor:'debugging'!

mapping
	"Return the mapping (if there is one) that is responsible for. Not needed for the code, but might be a useful debugging aid if trying to figure out where a @#%#@% join came from"
	^mapping.
!

mapping: aMapping
	mapping := aMapping.
! !

!Join methodsFor:'initialize'!

initialize

	sources := OrderedCollection new: 2.
	targets := OrderedCollection new: 2.
	base := BaseExpression new.
	outerJoin := false.
!

sourceFields: sourceFields targetFields: targetFields
	sources := sourceFields.
	targets := targetFields.
! !

!Join methodsFor:'iterating'!

do: aBlock skipping: aSet
	"Joins act as if they don't have children, and just evaluate for themselves"
	| |
	(aSet includes: self) ifTrue: [^self].
	aSet add: self.
	aBlock value: self.
!

fieldsDo: aBlock

	sources with: targets do: aBlock.
!

mappableFieldsDo: aBlock

	sources with: targets do: [:source :target |
		source isMappable ifTrue: [aBlock value: source value: target]].
! !

!Join methodsFor:'preparing'!

additionalExpressions
	^#()
!

additionalExpressionsIn: aQuery 
	^#()
!

allTablesToPrint

	^targets inject: Set new into: [:sum :each | 
		sum add: each table. sum].
!

prepareIn: aQuery 
	"Do nothing."
	aQuery whereClause: self asGeneralGlorpExpression.
	aQuery whereClause prepareIn: aQuery.
!

rebuildOn: aBaseExpression startingFrom: anExpression withOuterScopeBase: ultimateBaseExpression
	^self asGlorpExpressionOn: aBaseExpression.
!

replaceJoinsWithInverse
	"Replace any actual Join objects we have with their inverse. Only useful if we have something that has mixed joins and general expressions, where we assume we don't want to mess with the general ones, because they're not field-level. As actual joins, we ignore this, because it will be done at the parent level"
	^self.
!

rewriteEquality
	^self.
!

sourceForTarget: aField

	| index |
	index := targets indexOf: aField.
	index = 0 ifTrue: [^nil].
	^sources at: index.
!

targetForSource: aField

	| index |
	index := sources indexOf: aField.
	index = 0 ifTrue: [^nil].
	^targets at: index.
! !

!Join methodsFor:'primary keys'!

primaryKeyFromDictionary: aDictionary
	"Given a set of parameters, return a primary key suitable for retrieving our target. Return either a value for the key, nil for no key found, or an array for compound keys."
	
	sources size = 1 ifTrue: [^aDictionary at: sources first ifAbsent: [nil]].
	^sources collect: [:each | aDictionary at: each ifAbsent: [nil]].
! !

!Join methodsFor:'printing'!

printOn: aStream

	sources with: targets do: [:source :target |
		aStream nextPut: $(.
		source printSQLOn: aStream withParameters: #().
		aStream nextPutAll: ' = '.
		target printSQLOn: aStream withParameters: #().
		aStream nextPutAll: ') ']
!

printSQLOn: aStream withParameters: aDictionary 
        1 to: sources size
                do: 
                        [:i | 
                        | eachTarget eachSource sourceValue |
                        eachTarget := targets at: i.
                        eachSource := sources at: i.
                        eachTarget printSQLOn: aStream withParameters: aDictionary.
                        sourceValue := (self isConstant: eachSource) ifTrue: [eachSource value] ifFalse: [aDictionary at: eachSource].
                        sourceValue isNil 
                                ifTrue: [aStream nextPutAll: ' IS NULL ']
                                ifFalse: 
                                    [aStream nextPutAll: ' = '.
                                     sourceValue isString
                                        ifTrue: [sourceValue storeOn: aStream]
                                        ifFalse: [sourceValue printOn: aStream]
                                    ].
                        i = targets size ifFalse: [aStream nextPutAll: ' AND ']]
! !

!Join methodsFor:'testing'!

isGlorpExpression
	^true.
!

isJoin
	^true.
!

isOuterJoin
	^outerJoin.
! !

!Join class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__Join.st,v 1.1 2006/07/26 08:18:31 boris Exp $'
! !
