"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#TimedProxyReaper
	instanceVariableNames:'weakValueDictionary keysSortedByExpiryTime indexOffset mutex
		reaperThread delay maximumWork nextId timeout done startTime'
	classVariableNames:''
	poolDictionaries:''
	category:'GlorpCore'
!

TimedProxyReaper comment:'Overview:

A TimedProxyReaper collects TimedProxies and, using a background process, uninstantiates when they are due to expire. You can control the number of seconds between each invocation of the background process, with the message #delay:, and you can control the amount of work the background process will do each time it runs with the message #maximumWork:.

API Methods:

delay: seconds
	"Set the number of seconds to delay between invocation of the reaper background process."

maximumWorks: milliseconds
	"Set the maximum length of time in milliseconds that the reaper will run for before returning control to other processes."


Instance Variables:

weakValueDictionary 				TimedProxies, indexed by key.
keysSortedByExpiryTime 				Keys, sorted by expiry time.
indexOffset							Number of elements removed from the front of keysSortedByExpiryTime.
									(i.e. the "absolute" offset of the 1st element of keysSortedByExpiryTime, as if no elements
									had been removed).
mutex 								Semaphore to guard access to keysSortedByExpiryTime and weakValueDictionary.
reaperThread 						The process that uninstantiates and removes expired TimedProxies.
delay 								The length of time, in seconds, to delay between invocation of reaperThread.
maximumWork 						The maximum length of time, in milliseconds, for the reaperThread to run before yielding control.'
!


!TimedProxyReaper class methodsFor:'As yet unclassified'!

delay: seconds
	^self new delay: seconds
!

delay: seconds maximumWork: milliseconds
	^self new delay: seconds; maximumWork: milliseconds
!

new
	^super new initialize
! !

!TimedProxyReaper class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!TimedProxyReaper methodsFor:'accessing'!

delay
	"Answer the delay, in seconds, between runs of the reaper process."

	^delay
!

delay: seconds
	"Set delay between runs of the reaper process in seconds."

	delay := seconds
!

maximumWork
	"Answer the maximum amount of time the reaper thread will run for before yielding."

	^maximumWork
!

maximumWork: milliseconds
	"Set the maximum amount of time the reaper thread will run for before yielding."

	maximumWork := milliseconds
!

nextSequenceNumber

	| id |
	id := nextId.
	nextId := nextId + 1.
	^id
! !

!TimedProxyReaper methodsFor:'caching'!

add: aTimedProxy
	[mutex wait.
	self noWaitAdd: aTimedProxy] ensure: [mutex signal].
	self restartIfNecessary.
!

at: id ifAbsent: errorBlock
	| result |
	[mutex wait.
	result := weakValueDictionary at: id ifAbsent: [^errorBlock value]]
		ensure: [mutex signal].
	^result.
!

includes: aTimedProxy

	self at: aTimedProxy glorpId ifAbsent: [^false].
	^true
!

noWaitAdd: aTimedProxy

	| association id |
	aTimedProxy glorpId isNil ifTrue: [aTimedProxy glorpId: self nextSequenceNumber].
	id := aTimedProxy glorpId.	
	association := Association key: id value: aTimedProxy glorpTimeToDie.
	weakValueDictionary at: id put: aTimedProxy.
	aTimedProxy glorpAssociation: association.
	keysSortedByExpiryTime add: association.
	aTimedProxy glorpReaper: self.
!

remove: aTimedProxy ifAbsent: errorBlock

	^self removeKey: aTimedProxy glorpId ifAbsent: errorBlock.
!

removeKey: key ifAbsent: errorBlock
	"Note: The key is NOT removed from keysSortedByExpiryTime. The reaper
	will eventually reap the given key, and ignore it since it is already gone.
	Removing objects from the middle of keysSortedByExpiryTime will screw
	up the indexing of objects that are registered later. Don't do it."

	| result |
	[mutex wait.
	result := weakValueDictionary removeKey: key ifAbsent: [^errorBlock value]]
		ensure: [mutex signal].
	^result.
!

touch: aTimedProxy
	[| association |
	mutex wait.
	association := aTimedProxy glorpAssociation.
	association isNil ifTrue: [^self].
	association key: nil.	"This indicates that the key should not be reaped at this time."
	self noWaitAdd: aTimedProxy] ensure: [mutex signal].
! !

!TimedProxyReaper methodsFor:'initializing'!

initialize

	done := false.
	self delay: 5. "seconds"
	self maximumWork: 100. "milliseconds"
	nextId := 1.
	weakValueDictionary := Dialect weakValueDictionaryClass new.
	weakValueDictionary manager: self.

	"Note: The sort used by keysSortedByExpiryTime _must_ be stable."
	keysSortedByExpiryTime := SortedCollection sortBlock: [:associationA :associationB | associationA value "timeToDie" <= associationB value].

	indexOffset := 0.
	mutex := Semaphore forMutualExclusion.
	self forkReaperThread
!

release

	done := true.
	maximumWork := 0.
	reaperThread notNil ifTrue: [reaperThread terminate].
	reaperThread := nil.
! !

!TimedProxyReaper methodsFor:'reaping'!

finish
"	Transcript cr; show: 'Done', (Time millisecondClockValue // 1000) printString."
	done := true.
!

forkReaperThread

	| block |
	block := [
		[done] whileFalse: [
			(Delay forSeconds: self delay) wait.
			self reapExpiredObjects]].
	reaperThread := block forkAt: Processor userBackgroundPriority
!

mournKeyOf: anEphemeron
	"Don't really need to do anything."

	anEphemeron value uninstantiate.
!

privateReapObjects
	"Private - Go through all of the objects (sorted by time) and remove those which have expired from the receiver's weakValueDictionary.
	Answer the number of objects reaped."

	| numberReaped timesUp |
	startTime := Time millisecondClockValue.
	numberReaped := 0.
	timesUp := false.
	[(Delay forMilliseconds: 100) wait.
	timesUp := true] forkAt: Processor activeProcess priority + 1.
	keysSortedByExpiryTime
		do:
			[:association | 
			done | timesUp ifTrue: [^numberReaped].
			(self shouldReap: association)
				ifTrue:
					[| timedProxy |
					numberReaped := numberReaped + 1.
					association key isNil
						ifFalse:
							["Been touched in the meantime. Just skip over it, but don't reap"
							timedProxy := weakValueDictionary
								removeKey: association key
								ifAbsent: [nil].
							timedProxy class == TimedProxy ifTrue: [timedProxy uninstantiate]]]
				ifFalse: [^numberReaped]].
	^numberReaped.
!

reapExpiredObjects
	| numberReaped |
	[mutex wait.
	keysSortedByExpiryTime isEmpty ifTrue: [self finish].
	done ifTrue: [^self].
	numberReaped := self privateReapObjects.	"If the cache is empty, shut down"
	numberReaped timesRepeat: [keysSortedByExpiryTime removeFirst].
	indexOffset := indexOffset + numberReaped] ensure: [mutex signal].
!

restartIfNecessary
	[mutex wait.
	done
		ifTrue:
			["Transcript cr; show: 'Restarting', (Time millisecondClockValue // 1000) printString."
			done := false.
			self forkReaperThread]] ensure: [mutex signal].
!

shouldReap: anAssociation
	"Check that either we've passed the start time, or that there's been a wraparound in the millisecond value and we're now larger than it by some ridiculous margin, which we'll call 24 hours."

	| expireTime milliseconds shouldReap |
	anAssociation key isNil ifTrue: [^true].  "It's been touched. Skip over but don't reap, handled elsewhere"
	expireTime := anAssociation value.
	milliseconds := "24 * 60 * 60 * 1000" 86400000.
	shouldReap := expireTime <= startTime or: [expireTime - milliseconds > startTime].
	^shouldReap.
! !

!TimedProxyReaper class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/Glorp__TimedProxyReaper.st,v 1.1 2006/07/26 08:38:02 boris Exp $'
! !
