"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#ObjectTransaction
	instanceVariableNames:'undoMap collectionMementos'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

ObjectTransaction comment:'An ObjectTransaction knows how to remember the state of objects and revert them back to that state later on. It does this by making a *shallow* copy of the registered objects and everything connected to them, and then putting that into an identity dictionary keyed by the originals.

If you have to undo, you push the state from the shallow copies back into the originals.

Yes, that works, and it''s all you have to do. It even handles collections become:ing different sizes.

This is fairly independent of GLORP. You could use this mechanism in general, if you provided your own mechanism for figuring out what to register, or even just uncommented the one in here.

Instance Variables:
	undoMap	<IdentityDictionary>	 The dictionary of originals->copies.

'
!


!ObjectTransaction class methodsFor:'instance creation'!

new
	^super new initialize
! !

!ObjectTransaction class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!ObjectTransaction methodsFor:'accessing'!

collectionMementos
	^collectionMementos.
!

undoMap
	^undoMap
! !

!ObjectTransaction methodsFor:'begin/commit/abort'!

abort
	undoMap keysAndValuesDo: [:original :copy | self restoreStateOf: original toThatOf: copy]
!

abort: original
	| copy |
	copy := undoMap at: original ifAbsent: [^self].
	self restoreStateOf: original toThatOf: copy
!

begin
	self initializeUndoMap
!

commit
	self initializeUndoMap
! !

!ObjectTransaction methodsFor:'initializing'!

initialize
	self initializeUndoMap.
!

initializeUndoMap

	undoMap := IdentityDictionary new: 1000.
	collectionMementos := IdentityDictionary new: 200.
! !

!ObjectTransaction methodsFor:'private/registering'!

shallowCopyOf: anObject ifNotNeeded: aBlock

	| copy |
	copy := anObject shallowCopy.
	^copy == anObject 
		ifTrue: [aBlock value]
		ifFalse: [copy]
! !

!ObjectTransaction methodsFor:'private/restoring'!

isShapeOf: original differentThanThatOf: copy

	^original class ~~ copy class or: [original glorpBasicSize ~= copy glorpBasicSize]
!

restoreIndexedInstanceVariablesOf: original toThoseOf: copy

	copy glorpStartOfIndexedInstVars to: copy glorpEndOfIndexedInstVars do: [:index |
		original isString 
			ifTrue: [original at: index put: (copy at: index)]
			ifFalse: [original basicAt: index put: (copy basicAt: index)]].
!

restoreNamedInstanceVariablesOf: original toThoseOf: copy

	1 to: copy class instSize do: [:index |
		original instVarAt: index put: (copy instVarAt: index)]
!

restoreShapeOf: original toThatOf: copy

	| newOriginal |
	(copy class isBits or: [copy class isVariable])
		ifTrue: [newOriginal := copy class basicNew: copy glorpBasicSize]
		ifFalse: [newOriginal := copy class basicNew].
	original become: newOriginal.
!

restoreStateOf: original toThatOf: copy 

	(self isShapeOf: original differentThanThatOf: copy) 
		ifTrue: [self restoreShapeOf: original toThatOf: copy].
	self restoreNamedInstanceVariablesOf: original toThoseOf: copy.
	self restoreIndexedInstanceVariablesOf: original toThoseOf: copy
! !

!ObjectTransaction methodsFor:'registering'!

isRegistered: anObject 
	"Note: We can never have a situation where a proxy is registered but its contents aren't, so we don't have to worry about that ambiguous case."
	| realObject |
	realObject := self realObjectFor: anObject ifNone: [^false].
	^undoMap includesKey: realObject.
!

needsCollectionMementoFor: aCollection
	"We can avoid the need for a collection copy if the collection has no internal structure - that is, if we'll be able to loop over a simple shallow copy. This is dialect and collection class dependent"
	^Dialect needsCollectionMementoFor: aCollection.
!

realObjectFor: anObject 
	"If this is a proxy, return the contents (if available). Otherwise, return nil"
	^self realObjectFor: anObject ifNone: [nil].
!

realObjectFor: anObject ifNone: aBlock
	"If this is a proxy, return the contents (if available). Otherwise, evaluate the block"
	^anObject isGlorpProxy 
		ifTrue: [anObject isInstantiated ifTrue: [anObject getValue] ifFalse: [aBlock value]]
		ifFalse: [anObject]
!

register: anObject 
	"Make anObject be a member of the current transaction. Return the object if registered, or nil otherwise"

	| copy realObject |
	(self requiresRegistrationFor: anObject) ifFalse: [^nil].
	realObject := self realObjectFor: anObject ifNone: [^nil].
	copy := self shallowCopyOf: realObject ifNotNeeded: [^nil].
	undoMap at: realObject put: copy.
	self registerTransientInternalsOfCollection: realObject.
	^realObject
!

registerTransientInternalsOfCollection: aCollection
	"If this is a collection, then we may need to register any internal structures it has, e.g. an internal array. This is implementation dependent for the collection. We will also explicitly exclude strings"

	aCollection glorpIsCollection ifFalse: [^self].
	aCollection class isBits ifTrue: [^self].
	(self needsCollectionMementoFor: aCollection)
		ifTrue: [collectionMementos at: aCollection put: aCollection copy].
	aCollection glorpRegisterCollectionInternalsIn: self.
!

registeredObjectsDo: aBlock
	"Iterate over all our objects. Note that this will include objects without descriptors. Be sure we're iterating over a copy of the keys, because this will add objects to the undoMap. Allow the block to return a collection which we will then add to the list of things to process, until there's nothing further"

	| newAdditions previousVersion |
	newAdditions := undoMap keys.
	previousVersion := newAdditions.
	[newAdditions isEmpty]
		whileFalse:
			[newAdditions do: aBlock.
			newAdditions := OrderedCollection new.
			undoMap
				keysDo:
					[:eachKey | (previousVersion includes: eachKey) ifFalse: [newAdditions add: eachKey]].
			newAdditions isEmpty ifFalse: [previousVersion := undoMap keys]].
!

requiresRegistrationFor: anObject

	| realObject |
	realObject := self realObjectFor: anObject ifNone: [^false].
	^(self isRegistered: realObject) not.
! !

!ObjectTransaction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/ObjectTransaction.st,v 1.1 2006/07/26 08:18:19 boris Exp $'
! !
