"{ Package: 'stx:goodies/glorp0_4' }"

"{ NameSpace: Glorp }"

Object subclass:#UnitOfWork
	instanceVariableNames:'session transaction deletedObjects newObjects rowMap commitPlan
		deletePlan rowsByTable succeeded linkRowsForUpdate commitPhase'
	classVariableNames:''
	poolDictionaries:''
	category:'Glorp'
!

UnitOfWork comment:'A UnitOfWork keeps track of objects which might potentially be modified and lets you roll them back or commit the changes into the database.

Instance Variables:
	newObjects	<IdentitySet of: Object>	The objects registered with us as being new (i.e. not existing already in the database)
	session	<Session>	The session in which this is all taking place.
	transaction	<ObjectTransaction>	Keeps track of the original object state so that we can revert it.
	rowMap	<RowMap>	A holder for the rows when we are writing out changes.
	commitPlan	<(OrderedCollection of: DatabaseRow)>	The list of rows to be written, in order. Constructed by topological sorting the contents of the row map.
	calculatingRows <Boolean> Are we in the process of computing the rows to be written. If so, we should not be doing reads and registrations - it''s probably an error in the mapping code by accidentally forcing a read.

'
!


!UnitOfWork class methodsFor:'instance creation'!

new

	^super new initialize.
! !

!UnitOfWork class methodsFor:'LICENSE'!

LICENSE

	^'Copyright 2000-2004 Alan Knight.
This class is part of the GLORP system (see http://www.glorp.org), licensed under the GNU Lesser General Public License, with clarifications with respect to Smalltalk library usage (LGPL(S)). This code is distributed WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE . See the package comment, or the COPYING.TXT file that should accompany this distribution, or the GNU Lesser General Public License.'
! !

!UnitOfWork methodsFor:'accessing'!

correspondenceMap
	^transaction undoMap
!

newObjects
	newObjects isNil ifTrue: [newObjects := IdentitySet new].
	^newObjects
!

numberOfRows
	^commitPlan size + deletePlan size.
!

rowsToWrite
	^commitPlan.
!

session
	^session
!

session: aGlorpSession
	session := aGlorpSession
!

system
	^session system.
! !

!UnitOfWork methodsFor:'begin/commit/abort'!

abort
	self reinitialize.
!

abort: anObject
	transaction abort: anObject
!

begin
	self reinitialize.
!

commit
	[self preCommit.
	self writeRows] ifCurtailed: [succeeded := false].
	succeeded
		ifTrue: [self postCommit]
		ifFalse:
			[GlorpTransactionFailure signal].
!

createMementoRowMapFor: objects 
	"Create a rowmap for the objects whose state was already known. We subtract this from the rowmap of all known objects to get the rows that need to be written. New objects are also registered, so we only generate rows here for non-new objects"

	| localRowMap |
	localRowMap := RowMapForMementos 
				withCorrespondenceMap: self correspondenceMap.
	localRowMap collectionMementos: transaction collectionMementos.
	objects keysAndValuesDo: 
			[:original :memento | 
			(self newObjects includes: original) 
				ifFalse: [session createRowsFor: memento in: localRowMap]].
	^localRowMap
!

createRowMapFor: objects
	| localRowMap |
	localRowMap := RowMap new: objects size.
	objects do: [:each | session createRowsFor: each in: localRowMap].
	^localRowMap
!

createRows
	self createRowsForPartialWrites.
!

createRowsForCompleteWrites
	"reference implementation. not called from anywhere"

	self registeredObjectsDo: [:eachObject | session createRowsFor: eachObject in: rowMap].
	deletedObjects do: [:eachObject | session createDeleteRowsFor: eachObject in: rowMap].
!

createRowsForPartialWrites
	| registeredObjectsRowMap mementoObjectsRowMap |
	registeredObjectsRowMap := self createRowMapFor: self registeredObjects.
	deletedObjects 
		do: [:eachObject | session createDeleteRowsFor: eachObject in: registeredObjectsRowMap].
	mementoObjectsRowMap := self createMementoRowMapFor: self mementoObjects.
	rowMap := registeredObjectsRowMap differenceFrom: mementoObjectsRowMap.
!

mementoObjects
	"Warning: Excessive cleverness!!!!!! The mementoObjects we want to iterate over are the values in the correspondenceMap dictionary. We were getting the values and returning them, but if all we need to do is iterate, then the dictionary itself works fine"
	^self correspondenceMap.
!

postCommit
	self sendPostWriteNotification.
	self updateSessionCache.
	self validateRowsPostWrite.
!

preCommit
	commitPhase := #registering.
	self registerTransitiveClosure.
	commitPhase := #building.
	self createRows.
	linkRowsForUpdate := rowMap collapseMatchingInsertsAndDeletes.
	self buildCommitPlan.
	commitPhase := nil.
	self sendPreWriteNotification.
	self validateRows.
!

propagateDeletes
	| deletionQueue |
	deletionQueue := OrderedCollection new: deletedObjects size.
	deletionQueue addAll: deletedObjects.
	[deletionQueue isEmpty]
		whileFalse:
			[| each |
			each := deletionQueue removeFirst.
			(self session descriptorFor: each) mappings
				do:
					[:eachMapping | 
					eachMapping isExclusive
						ifTrue:
							[eachMapping
								loopOverAttributeOf: each
								doing:
									[:index :subObject | 
									| realSubObject |
									realSubObject := subObject yourSelf.
									(realSubObject notNil and: [(deletedObjects includes: realSubObject) not])
										ifTrue:
											[session delete: subObject.
											deletionQueue add: realSubObject]]]]].
!

registerTransitiveClosure
	"Look for new objects reachable from currently registered objects, and propogate deletions"

	self
		registeredObjectsDo:
			[:eachObject | self registerTransitiveClosureFrom: eachObject].
	self propagateDeletes.
!

registeredObjects
	^self correspondenceMap keys
!

rollback
	self abort.
!

validateRows
	"Perform basic validation. Always test for equal named but non-identical tables, a sign of a malformed  system or other loss of identity. Also delegate to the descriptorSystem, which may have useful application-specific validations to do."
	
	| tables tableNames |
	tables := Set new.
	rowMap rowsDo: [:each |
		tables add: each table].
	tableNames := tables collect: [:each | each qualifiedName].
	tables asSet size = tableNames asSet size ifFalse: [self error: 'multiple table objects with the same name'].
	self system validateRows: self.
!

validateRowsPostWrite
	"Perform basic validation after writing is done. It's better to validate beforehand, but some things are easier to catch afterwards"
	
	self system validateRowsPostWrite: self.
! !

!UnitOfWork methodsFor:'deletion'!

delete: anObject
	(self isRegistered: anObject) ifFalse: [session register: anObject].
	deletedObjects add: anObject.
	(session embeddedObjectsFor: anObject) do: [:each | self delete: each].
!

hasPendingDeletions

	^deletedObjects isEmpty not.
!

willDelete: anObject
	"Return true if we will delete anObject when we commit."	
	^deletedObjects includes: anObject.
! !

!UnitOfWork methodsFor:'enumerating'!

registeredObjectsDo: aBlock

	transaction registeredObjectsDo: [:each |
		(each glorpIsCollection or: [session hasDescriptorFor: each]) ifTrue: [
			aBlock value: each]].
!

rowsForTable: aTable do: aBlock

	rowMap rowsForTable: aTable do: aBlock.
! !

!UnitOfWork methodsFor:'initializing'!

initialize

	transaction := ObjectTransaction new.
	self reinitialize
!

reinitialize

	rowMap := RowMap new.
	commitPhase := nil.
	deletedObjects := IdentitySet new.
	commitPlan := OrderedCollection new.
	deletePlan := OrderedCollection new.
	transaction abort.
! !

!UnitOfWork methodsFor:'private'!

deletedObjects
	^deletedObjects.
!

privateGetRowMap

	^rowMap
!

privateGetTransaction

	^transaction.
!

registerAsNew: anObject 
	anObject isNil ifTrue: [^nil].
	commitPhase == #building ifTrue: [self halt]. "Should not happen. Probably indicates that we're triggering proxies while firing the mappings"
	self newObjects add: anObject.
	self register: anObject.
	^anObject
!

sendPostWriteNotification
	self 
		registeredObjectsDo: [:eachObject | session sendPostWriteEventTo: eachObject]
!

sendPreWriteNotification
	self 
		registeredObjectsDo: [:eachObject | session sendPreWriteEventTo: eachObject]
! !

!UnitOfWork methodsFor:'private/mapping'!

addObject: eachObject toCacheKeyedBy: key 
	
	self session cacheAt: key put: eachObject.
!

addToCommitPlan: aRow
	aRow shouldBeWritten ifFalse: [^self].
	commitPlan add: aRow.
!

addToDeletePlan: aRow
	
	deletePlan add: aRow.
!

assignSequenceNumbers
	commitPlan do: [:each | each preWriteAssignSequencesUsing: self session accessor].
!

buildCommitPlan
	| tablesInCommitOrder |
	commitPlan := OrderedCollection new.
	deletePlan := OrderedCollection new.
	tablesInCommitOrder := session tablesInCommitOrder.
	tablesInCommitOrder do: 
			[:eachTable | 
			self rowsForTable: eachTable
				do: 
					[:eachRow | 
					eachRow forDeletion 
						ifTrue: [self addToDeletePlan: eachRow]
						ifFalse: [self addToCommitPlan: eachRow]]].
	self sortCommitPlanRows.
!

instantiateIfRequiredFor: anObject mapping: eachMapping
	"Sometimes we have to instantiate the targets if they weren't. Specifically, if there's a relationship where the target has a foreign key to us. e.g. if X has a 1-many to Y, and we don't instantiate the collection of Y, but then replace it with some other collection. The Y's keys have to be updated, so we need to make sure they're read. There are other cases, too"

	| originalTarget targetObject mapping instantiated |
	mapping := eachMapping applicableMappingForObject: anObject.
	mapping isRelationship ifFalse: [^false].
	mapping readOnly ifTrue: [^false].
	targetObject := mapping getValueFrom: anObject.
	(self isNewObject: anObject)
		ifTrue:
			[^self instantiateIfRequiredForNewObjectReferenceTo: targetObject].
	instantiated := (self willDelete: anObject)
		ifTrue:
			[self instantiateIfRequiredForDeletedObjectReferencing: targetObject from: anObject in: eachMapping]
		ifFalse: [false].
	originalTarget := self originalValueFor: anObject mapping: mapping.
	^instantiated or: [self
		instantiateIfRequiredForReferenceChangedFrom: originalTarget
		to: targetObject].
!

instantiateIfRequiredForDeletedObjectReferencing: targetObject from: anObject in: aMapping
	"If we are deleted, then we need to have at least one mapping for each of our primary keys instantiated. If we're lucky, there's a direct mapping, and it's easy. Note that this only looks for the primary table. This might not be correct, but it's not clear, and for secondary tables there's a multiple table join that can be used without needing a mapping. And I'm not prepared to think about secondary tables whose keys are mapped only as relationships"
	| mappingsToUseForPrimaryKeys |
	aMapping mapsPrimaryKeys ifFalse: [^false].	
	mappingsToUseForPrimaryKeys := aMapping descriptor primaryKeyMappingsFor: aMapping descriptor primaryTable.
	"For any of the fields we map, is there anyone else who can get us the value. If not, instantiate us. It might be better to optimize this a bit to look for e.g. one-one mappings before to-many."
	aMapping mappedFields do: [:each |
		| mappings |
		mappings := mappingsToUseForPrimaryKeys at: each ifAbsent: [nil].
		mappings isNil ifFalse: [
			(mappings allSatisfy: [:eachMapping | 
				(eachMapping hasValueThatCanProvideField: each from: anObject) not])
					ifTrue: [
						targetObject yourSelf.
						^true]]].
	^false.
!

instantiateIfRequiredForNewObjectReferenceTo: targetObject
	"If it's a new object, then we always need to instantiate"
	targetObject isGlorpProxy ifFalse: [^false].
	targetObject getValue.
	^true.
!

instantiateIfRequiredForReferenceChangedFrom: originalTarget to: newTarget
	"For an existing relationship, we need to instantiate if the object we're referring to changed, and either was or is now a proxy. Actually we might get away without if it used to be, but now isn't, but better safe than sorry."

	| wasProxy  |
	originalTarget == newTarget ifTrue: [^false].
	wasProxy := false.
	originalTarget isGlorpProxy ifTrue: [wasProxy := true. originalTarget getValue].
	newTarget isGlorpProxy ifTrue: [wasProxy := true. newTarget getValue].
	^wasProxy.
!

originalValueFor: anObject mapping: eachMapping
	| memento |
	memento := transaction undoMap at: anObject.
	^eachMapping getValueFrom: memento.
!

readBackNewRowInformation
	| changedObjects |
	changedObjects := rowMap objects.
	changedObjects
		do: 
			[:each | 
			| descriptor |
			descriptor := session descriptorFor: each.
			descriptor isNil ifFalse: [descriptor readBackNewRowInformationFor: each in: rowMap]]
!

registerTransitiveClosureFrom: anObject

	| descriptor |
	anObject glorpIsCollection ifTrue: [
		anObject do: [:each | session register: each].
		^self].

	descriptor := session descriptorFor: anObject class.
	descriptor isNil ifTrue: [^self].
	descriptor mappings do: [:eachMapping |
			self instantiateIfRequiredFor: anObject mapping: eachMapping].
	descriptor referencedIndependentObjectsFrom: anObject do: [:eachObject |
		session register: eachObject].
!

relatedRowsFor: each
	| rows extraRows |
	rows := each relatedRowsIn: rowMap.
	extraRows := self session system additionalRelatedRowsFor: each in: rowMap.
	extraRows isNil ifFalse: [rows addAll: extraRows].
	^rows.
!

reserveSequenceNumbers
	rowsByTable := commitPlan glorpGroupedBy: [:each | each table].
	rowsByTable keysAndValuesDo: [:eachTable :rows | 
		eachTable sequences do: [:eachSequence | 
			eachSequence 
				reserveSequenceNumbers: (
					rows inject: 0 into: [:sum :each | 
						(each needsValueFromSequence: eachSequence) 
							ifTrue: [sum + 1] ifFalse: [sum]])
					in: self session
					for: eachTable]]
!

sortCommitPlanRows
	| sorter |
	sorter := GlorpRowSorter new
		getChildrenVia: [:each | self relatedRowsFor: each].
	commitPlan := sorter sort: commitPlan.
	sorter := GlorpRowSorter new
		getChildrenVia: [:each | self relatedRowsFor: each].
	deletePlan := sorter sort: deletePlan.
!

updateSessionCache
	rowMap
		keysAndValuesDo: [:eachObject :eachRow | 
			eachRow shouldBeWritten ifTrue: [
				self updateSessionCacheFor: eachObject withRow: eachRow]].
	deletedObjects do: [:each |
		session cacheRemoveObject: each].
!

updateSessionCacheFor: anObject withRow: aRow 
	| key |
	(rowMap isRowMapKey: anObject) ifTrue: [^self].  "Not cachable"
	key := aRow primaryKey.
	key isNil ifTrue: [^self].
	(session cacheContainsObjectForClass: anObject class key: key) 
		ifFalse: [self addObject: anObject toCacheKeyedBy: key].
!

writeRows

	succeeded := true.
	self reserveSequenceNumbers.
	self assignSequenceNumbers.
	self session tablesInCommitOrder do: [:eachTable | self writeRowsForTable: eachTable].
	deletePlan reverseDo: [:eachRow | | rowSucceeded |
		rowSucceeded := session writeRow: eachRow.
		succeeded := succeeded & rowSucceeded].
	self readBackNewRowInformation.
!

writeRowsForTable: aTable

	| rows rowsByOperation |

	rows := rowsByTable at: aTable ifAbsent: [#()].
	^rows isEmpty ifFalse: [
		rowsByOperation := rows glorpGroupedBy: [:eachRow | session shouldInsert: eachRow].
		rowsByOperation do: [:eachListOfHomogeneousRows | 
			succeeded := succeeded & (session writeHomogeneousRows: eachListOfHomogeneousRows)]].
! !

!UnitOfWork methodsFor:'registering'!

isRegistered: anObject

	^transaction isRegistered: anObject.
!

register: anObject 

	| realObject |
	commitPhase == #building ifTrue: [self halt]. "Should not happen. Probably indicates that we're triggering proxies while firing the mappings"
	realObject := transaction register: anObject.
	self registerTransitiveClosureFrom: realObject.
! !

!UnitOfWork methodsFor:'testing'!

forcesUpdateFor: aDatabaseRow
	^linkRowsForUpdate notNil and: [linkRowsForUpdate includes: aDatabaseRow].
!

isNewObject: anObject 
	^self newObjects includes: anObject.
!

shouldFilterDeletions
	"Should we filter deleted objects from the results of queries. We don't do this if we're in the process of registering the transitive closure, because any fetches we do there are in order to get information we need for the commit (e.g. primary keys). If we filtered the deleted objects, then we may not be able to figure out primary keys for the objects to be deleted"
	^commitPhase ~~ #registering.
! !

!UnitOfWork class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/glorp0_4/UnitOfWork.st,v 1.1 2006/07/26 08:18:43 boris Exp $'
! !
