"{ Package: 'stx:goodies/gofer' }"

Object subclass:#Gofer
	instanceVariableNames:'references repositories errorBlock packageCacheRepository
		resolvedReferencesCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Gofer-Core'
!

Gofer comment:': Gofer, a person who runs errands. Origin 1960s: from go for, i.e. go and fetch.
: ''''The New Oxford American Dictionary''''
!! Synopsis
Gofer is a small tool on top of Monticello that loads, updates, merges, diffs, reverts, commits, recompiles and unloads groups of Monticello packages. Contrary to existing tools Gofer makes sure that these operations are performed as clean as possible:
- Gofer treats packages from one or more repository in one operation.
- Gofer works with fixed versions or tries to find the "latest" version using a given package name.
- Gofer automatically assigns repositories to all packages, so that the other tools are ready to be used on individual packages.
- Gofer makes sure that there is only one repository instance registered for a single physical location.
- Gofer works with Monticello dependencies and uniformly treats them like the primary package.
- Gofer prefers to work with faster repositories if there is a choice.
- Gofer cleans up after Monticello, no empty class categories and no empty method protocols are to be expected.
- Gofer supports operations to sync remote and local repositories with each other.
!! Installation
Gofer is included with the latest Pharo and GemStone distributions. To update to the latest version you can use Gofer itself:
== Gofer upgrade
In case you are missing Gofer in your image, grab it from *http://source.lukas-renggli.ch/gofer.html*.
!! Description
Gofer is very simple by design, the basic useage scenario is always the same and consists of three steps:
# You specify one or more Monticello repository URLs. You can do this using the methods ==url:==, ==url:username:password:== (HTTP, FTP), ==directory:==, or ==repository:== if you need full control. You might also use the convenience methods like ==squeaksource:==, ==wiresong:==, or ==gemsource:== for well known repositories. Additionally the following settings are available:
#- Gofer implicitly declares the local package cache as a repository. To disable the local package cache use the method ==disablePackageCache==, to re-enable use ==enablePackageCache==.
#- Gofer throws an error if a repository is not reachable. To silently ignore repository erros use the message ==disableRepositoryErrors==, to re-enable use ==enableRepositoryErrors==.
# You specify one or more Monticello packages you want to work with, by adding them to the Gofer instance. Use ==version:== to add a specific version, or use ==package:== to add the "latest" version in the given repository. Furthermore there is ==package:constraint:== that allows you to further constraint the version to be loaded in a block passed in as the second argument.
# You specify one or more actions to be performed on the specified packages:
| ==load==	| Load the specified packages.
| ==update==	| Update the specified packages.
| ==merge==	| Merge the specified packages into their working copies.
| ==localChanges==	| Answer the changes between the base version and the working copy.
| ==browseLocalChanges==	| Browse the changes between the base version and the working copy.
| ==remoteChanges==	| Answer the changes between the working copy and the remote changes.
| ==browseRemoteChanges==	| Browse the changes between the working copy and the remote changes.
| ==cleanup==	| Cleans the specified packages.
| ==commit==	| Commit the modified specified packages.
| ==commit:==	| Commit the modified specified packages with the given commit message.
| ==revert==	| Revert the specified packages to the currently loaded version.
| ==recompile==	| Recompile the specified packages.
| ==reinitialize==	| Call the class side initializers on the specified packages.
| ==unload==	| Unload the specified packages.
| ==fetch==     | Download versions from remote repositories into the local cache.
| ==push==      | Upload local versions from local cache into remote repositories.
!! Example
To use Gofer to update to exact versions of the Kom Server, the ''latest'' code of Seaside 2.8 and the ''latest'' code of the Scriptaculous package that is committed by the author with the initials ''lr'' one could evaluate:
== Gofer new
==     squeaksource: ''KomHttpServer'';
==     version: ''DynamicBindings-gc.7'';
==     version: ''KomServices-gc.19'';
==     version: ''KomHttpServer-gc.32'';
==     update.
== Gofer new
==     squeaksource: ''Seaside'';
==     package: ''Seaside2.8a'';
==     package: ''Scriptaculous'' constraint: [ :version | version author = ''lr'' ];
==     load'
!


!Gofer class methodsFor:'instance creation'!

it
	^ self new
!

new
	^ self basicNew initialize
! !

!Gofer class methodsFor:'private'!

gofer
	"Create a Gofer instance of Gofer."

	^ self new
		renggli: 'gofer';
		package: 'Gofer-Core';
		package: 'Gofer-Tests';
		yourself
!

upgrade
	"Update Gofer to the latest version using itself."
	
	| working |
	[ self gofer load ]
		on: Error
		do: [ :err | err retry ].
	self new unload
		unregister: (MCWorkingCopy 
			forPackage: (MCPackage named: 'Gofer')).
	self gofer recompile; cleanup
! !


!Gofer methodsFor:'accessing'!

allResolved
        "Answer all sorted references within the configured repositories."
        
        | resolved |
        resolved := OrderedCollection new.
        self repositories
                do: [ :repository | resolved addAll: (self allResolvedIn: repository) ].
        ^ (resolved asSortedCollection:[:a :b | a <= b]) asArray

    "Modified: / 20-08-2011 / 19:38:15 / cg"
!

allResolvedIn: aRepository
	"Answer all sorted references within aRepository. For efficiency cache the references."

	^ (resolvedReferencesCache ifNil: [ resolvedReferencesCache := Dictionary new ])
		at: aRepository ifAbsentPut: [ self basicReferencesIn: aRepository ]
!

references
	"Answer the configured references."

	^ Array withAll: references
!

repositories
	"Answer the configured monticello repositories."

	|  result |
	result := OrderedCollection withAll: repositories.
	packageCacheRepository isNil
		ifFalse: [ result addFirst: packageCacheRepository ].
	^ result asArray
!

resolved
	"Answer the resolved references of the receiver."

	^ self references collect: [ :each | each resolveWith: self ]
! !

!Gofer methodsFor:'copying'!

postCopy
	references := references copy.
	repositories := repositories copy.
	resolvedReferencesCache := nil
! !

!Gofer methodsFor:'deprecated'!

addPackage: aString
	self deprecated: 'Instead of #addPackage: simply use #package:'.
	self package: aString
!

addPackage: aString constraint: aBlock
	self deprecated: 'Instead of #addPackage:constraint: simply use #package:constraint:'.
	self package: aString constraint: aBlock
!

addVersion: aString
	self deprecated: 'Instead of #addVersion: simply use #version:'.
	self version: aString
! !

!Gofer methodsFor:'initialization'!

initialize
	self enablePackageCache; enableRepositoryErrors.
	references := OrderedCollection new.
	repositories := OrderedCollection new
! !

!Gofer methodsFor:'operations'!

cleanup
	"Cleans the specified packages."
	
	^ self execute: GoferCleanup
!

commit
	"Commit the modified packages."

	^ self execute: GoferCommit
!

commit: aString
	"Commit the modified packages with the given commit message."

	^ self execute: GoferCommit do: [ :operation | operation message: aString ]
!

fetch
	"Download versions from remote repositories into the local cache."

	^ self execute: GoferFetch
!

load
	"Load the specified packages into the image."
	
	^ self execute: GoferLoad
!

localChanges
	"Answer the changes between the base version and the working copy."
	
	^ self execute: GoferLocalChanges
!

merge
	"Merge the specified packages into their working copies."
	
	^ self execute: GoferMerge
!

push
	"Upload local versions from local cache into remote repositories."

	^ self execute: GoferPush
!

recompile
	"Recompile the specified packages."

	^ self execute: GoferRecompile
!

reinitialize
	"Calls the class side initializers on all package code."
	
	^ self execute: GoferReinitialize
!

remoteChanges
	"Display the changes between the working copy and the remote changes."
	
	^ self execute: GoferRemoteChanges
!

revert
	"Revert the specified packages to the currently loaded version."
	
	^ self execute: GoferRevert
!

unload
	"Unload the specified packages."
	
	^ self execute: GoferUnload
!

update
	"Update the specified packages."
	
	^ self execute: GoferUpdate
! !

!Gofer methodsFor:'operations-ui'!

browseLocalChanges
	"Browse the changes between the base version and the working copy."
	
	^ self execute: GoferBrowseLocalChanges
!

browseRemoteChanges
	"Browse the changes between the working copy and the remote changes."
	
	^ self execute: GoferBrowseRemoteChanges
! !

!Gofer methodsFor:'private'!

basicReferencesIn: aRepository
        ^ [ (aRepository goferReferences asSortedCollection:[:a :b | a <= b]) asArray ] 
                on: GoferRepositoryError
                do: errorBlock

    "Modified: / 20-08-2011 / 19:38:24 / cg"
!

execute: anOperationClass
	^ self execute: anOperationClass do: nil
!

execute: anOperationClass do: aBlock
	| operation result |
	operation := anOperationClass on: self copy.
	aBlock isNil
		ifFalse: [ aBlock value: operation ].
	^ operation execute
! !

!Gofer methodsFor:'references'!

package: aString
	"Add the package aString to the receiver. aString is a package name as it appears in the Monticello Browser, something like 'Magritte-Seaside'. Gofer will try to resolve this name to an actual version in one of the configured repositories before doing something with the package. Gofer sorts all the versions in all the repositories according to branch name (versions without a branch are preferred), version number, author name and repository priority. The top hit of this sorted list is eventually going to be loaded and used."

	references addLast: (GoferPackageReference name: aString)
!

package: aString constraint: aOneArgumentBlock
	"Add the package aString to the receiver, but constraint the resulting versions further with aOneArgumentBlock. For details on the package, see #package:. The following example defines various constraints: 
	
		aGofer package: 'Magritte-Seaside' constraint: [ :version |
			version author = 'lr'
				and: [ version branch = 'trial'
				and: [ version versionNumber > 120 ] ] ]"

	references addLast: (GoferConstraintReference name: aString constraint: aOneArgumentBlock)
!

version: aString
	"Add the version aString to the receiver. aString is a version name as it appears in the Monticello Repository Browser, something like 'Magritte-Seaside-lr.334'. Gofer will try to resolve this name to one of the configured repositories before loading the code."

	references addLast: (GoferVersionReference name: aString)
! !

!Gofer methodsFor:'repositories'!

directory: aDirectoryOrString
	"Add a file-system repository at aDirectoryOrString."

	| repository |
	repository := (aDirectoryOrString isString and: [ aDirectoryOrString endsWith: '*' ])
		ifTrue: [ 
			((Smalltalk globals
				at: #MCSubDirectoryRepository
				ifAbsent: [ self error: aDirectoryOrString printString , ' is an unsupported repository type' ]) new)
				directory: (FileDirectory on: aDirectoryOrString allButLast);
				yourself ]
		ifFalse: [ 
			(MCDirectoryRepository new)
				directory:
						(aDirectoryOrString isString
								ifTrue: [ FileDirectory on: aDirectoryOrString ]
								ifFalse: [ aDirectoryOrString ]);
				yourself ].
	self repository: repository
!

repository: aRepository
	"Add aRepository to the repository configuration. If there is already a repository defined in the global configuration with that URL take this one instead."
	
	| repository |
	repository := MCRepositoryGroup default repositories
		detect: [ :each | each = aRepository ]
		ifNone: [ aRepository ].
	repositories addLast: repository
!

url: anUrlString
	"Add anUrlString as a repository for the following package operations."

	self url: anUrlString username: String new password: String new
!

url: anUrlString username: aUsernameString password: aPasswordString
	"Add anUrlString as a repository for the following package operations."

	| repository |
	repository := (anUrlString beginsWith: 'ftp://')
		ifTrue: [ 
			(Smalltalk globals
				at: #MCFtpRepository
				ifAbsent: [ self error: anUrlString printString , ' is an unsupported repository type' ])
				host: ((anUrlString allButFirst: 6) copyUpTo: $/)
				directory: ((anUrlString allButFirst: 6) copyAfter: $/)
				user: aUsernameString
				password: aPasswordString ]
		ifFalse: [ MCHttpRepository location: anUrlString user: aUsernameString password: aPasswordString ].
	self repository: repository
! !

!Gofer methodsFor:'repositories-options'!

disablePackageCache
	"Disable the use of the package-cache repository."

	packageCacheRepository := nil
!

disableRepositoryErrors
	"Silently swallow all repository errors."

	errorBlock := [ :error | error resume: #() ]
!

enablePackageCache
	"Enable the use of the package-cache repository."

	packageCacheRepository := MCCacheRepository default
!

enableRepositoryErrors
	"Throw an exception when repositories are not available."

	errorBlock := [ :error | error pass ]
! !

!Gofer methodsFor:'repositories-places'!

blueplane: aString
	self url: 'http://squeaksource.blueplane.jp/' , aString
!

croquet: aString
	self url: 'http://hedgehog.software.umn.edu:8888/' , aString
!

gemsource: aString
	self url: 'http://seaside.gemstone.com/ss/' , aString
!

impara: aString
	self url: 'http://source.impara.de/' , aString
!

renggli: aString
	self url: 'http://source.lukas-renggli.ch/' , aString
!

saltypickle: aString
	self url: 'http://squeak.saltypickle.com/' , aString
!

squeakfoundation: aString
	self url: 'http://source.squeakfoundation.org/' , aString
!

squeaksource: aString
	self url: 'http://www.squeaksource.com/' , aString
!

wiresong: aString
	self url: 'http://source.wiresong.ca/' , aString
! !

!Gofer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/gofer/Gofer.st,v 1.2 2011/08/20 17:39:05 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/gofer/Gofer.st,v 1.2 2011/08/20 17:39:05 cg Exp $'
! !
