"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Object subclass:#BoundaryConstraint
	instanceVariableNames:'line isStart filledFigure isUpdating'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Constraints'
!

BoundaryConstraint comment:'BoundaryConstraint represents a constraint that is placed between a filled figure and a line figure. It computes the lines position so that it is attached to the edge of the filled figure. This constraint only works if the filled figure answers true when sent the containsPoint: message with the filled figures center.

Instance Variables:
	filledFigure    <Figure>  the filled figure that answers true to "filledFigure containsPoint: filledFigure center"
	isStart <Boolean> setting the start or stop point of the line
	isUpdating      <Boolean> are we currently setting the lines values (avoids infinite recursion)
	line    <PolylineFigure>  the line that is attached'
!


!BoundaryConstraint class methodsFor:'instance creation'!

forFigure: aFigure startLine: aLineFigure
	^(self new) figure: aFigure line: aLineFigure;
		isStart: true;
		updateLinePoint;
		yourself
!

forFigure: aFigure stopLine: aLineFigure
	^(self new) figure: aFigure line: aLineFigure;
		isStart: false;
		updateLinePoint;
		yourself
! !

!BoundaryConstraint methodsFor:'initialize-release'!

figure: aFigure line: aLineFigure
	filledFigure := aFigure.
	line := aLineFigure.
	isUpdating := false.
	filledFigure addDependent: self.
	line addDependent: self
!

isStart: aBoolean
	isStart := aBoolean
!

release
	filledFigure removeDependent: self.
	line removeDependent: self.
	super release
! !

!BoundaryConstraint methodsFor:'private'!

currentPoint
	^isStart
		ifTrue: [line startPoint]
		ifFalse: [line stopPoint]
!

findPointOnBoundaryBetweenInside: aPoint andOutside: anotherPoint
	| newPoint insidePoint outsidePoint |
	insidePoint := aPoint.
	outsidePoint := anotherPoint.
	[outsidePoint between: insidePoint - 1 and: insidePoint + 1] whileFalse:
			[newPoint := (insidePoint + outsidePoint) / 2.0.
			(filledFigure containsPoint: newPoint)
				ifTrue: [insidePoint := newPoint]
				ifFalse: [outsidePoint := newPoint]].
	^outsidePoint
!

movePointTo: aPoint
	isStart
		ifTrue: [line startPoint: aPoint]
		ifFalse: [line stopPoint: aPoint]
!

previousPoint
	^isStart
		ifTrue: [line pointAt: 2]
		ifFalse: [line pointAt: line pointsSize - 1]
! !

!BoundaryConstraint methodsFor:'updating'!

update: anAspectSymbol with: aParameter from: aSender
	anAspectSymbol == #deleted ifTrue: [^self release].
	aSender == line ifTrue: [^self updateFromLineWith: anAspectSymbol].
	aSender == filledFigure ifTrue: [^self updateLinePoint].
	super
		update: anAspectSymbol
		with: aParameter
		from: aSender
!

updateFromLineWith: anAspectSymbol
	isUpdating ifTrue: [^self].
	isStart
		ifTrue: [anAspectSymbol == #stopPoint ifTrue: [^self]]
		ifFalse: [anAspectSymbol == #startPoint ifTrue: [^self]].
	self updateLinePoint
!

updateLinePoint
	isUpdating := true.

	[| current previous direction inside outside |
	current := self currentPoint.
	previous := self previousPoint.
	(filledFigure containsPoint: current)
		ifTrue:
			[(filledFigure containsPoint: previous)
				ifTrue:
					[direction := current - previous.
					direction := direction = (0 @ 0)
								ifTrue: [1 @ 0]
								ifFalse: [direction unitVector].
					outside := previous + (direction * (filledFigure extent r + 1)).
					inside := previous]
				ifFalse:
					[inside := current.
					outside := previous]]
		ifFalse:
			[inside := filledFigure center.
			outside := current].
	self movePointTo: (self findPointOnBoundaryBetweenInside: inside
				andOutside: outside)]
			valueNowOrOnUnwindDo: [isUpdating := false]
! !

!BoundaryConstraint class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__BoundaryConstraint.st,v 1.1 2006/03/13 16:58:05 cg Exp $'
! !
