"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#CompositeFigure
	instanceVariableNames:'components'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Framework'
!

CompositeFigure comment:'CompositeFigure is a Figure that groups other figures into a single figure (composite pattern).

Instance Variables:
	components      <SequenceableCollection of: Figure>       the figures I''m grouping

'
!


!CompositeFigure methodsFor:'accessing'!

components
	^components
! !

!CompositeFigure methodsFor:'adding'!

add: aFigure
	| figureBounds |
	components add: aFigure.
	aFigure container: self.
	figureBounds := aFigure bounds.
	self invalidateRectangle: figureBounds.
	self mergeBounds: figureBounds.
	^aFigure
!

addAll: aCollection
	"For efficiency"

	| figureBounds |
	aCollection isEmpty ifTrue: [^aCollection].
	components addAll: aCollection.
	aCollection do: [:each | each container: self].
	figureBounds := aCollection inject: nil into: [:sum :each | sum isNil
					ifTrue: [each bounds]
					ifFalse: [sum merge: each bounds]].
	self invalidateRectangle: figureBounds.
	self mergeBounds: figureBounds.
	^aCollection
!

addLast: aFigure
	components addFirst: aFigure.
	aFigure container: self.
	self invalidateRectangle: aFigure bounds.
	self mergeBounds: aFigure bounds.
	^aFigure
! !

!CompositeFigure methodsFor:'broadcast'!

downcastEvent: aKey with: aParameter from: anInitiator
	"This is a message passed down the view structure to all subparts."

	super downcastEvent: aKey
		with: aParameter
		from: anInitiator.
	components do:
			[:each |
			each downcastEvent: aKey
				with: aParameter
				from: anInitiator]
!

downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator
	"This is a message passed down the view structure to some single
	part. Answer true if we accepted the event, or false if it should be
	passed on to whatever's behind us."

	components size to: 1
		by: -1
		do:
			[:i |
			((components at: i) downcastLocalEvent: aKey
				with: aParameter
				at: aPoint
				from: anInitiator) ifTrue: [^true]].
	^super downcastLocalEvent: aKey
		with: aParameter
		at: aPoint
		from: anInitiator
!

flushCoordinateCaches
	"Flush caches that relate to coordinate translations between this component
	and its container"

	super flushCoordinateCaches.
	components do: [:each | each flushCoordinateCaches]
!

newGraphicsDevice: aGraphicsDevice
	"Forward the newGraphicsDevice: aGraphicsDevice messages
	to the receiver's components."

	super newGraphicsDevice: aGraphicsDevice.
	components do: [:component | component newGraphicsDevice: aGraphicsDevice]
! !

!CompositeFigure methodsFor:'control'!

componentWantingControl
	"Answer the first component that wishes to take control.  Answer nil if there is
	no component wanting control."

	components size to: 1
		by: -1
		do:
			[:i |
			| obj |
			obj := (components at: i) objectWantingControl.
			obj notNil ifTrue: [^obj]].
	^nil
! !

!CompositeFigure methodsFor:'copying'!

postCopy
	super postCopy.
	components := components collect: [:each | each copy]
! !

!CompositeFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	| rect |
	rect := aGraphicsContext clippingBounds
				intersect: (self preferredBounds expandedBy: 1).
	aGraphicsContext clippingRectangle: rect.
	components
		do: [:each | (each intersects: rect) ifTrue: [each displayOn: aGraphicsContext copy]]
! !

!CompositeFigure methodsFor:'initialize-release'!

flushCaches
	super flushCaches.
	components do: [:each | each flushCaches]
!

initialize
	super initialize.
	components := OrderedCollection new
!

release
	components do: [:each | each release].
	super release
! !

!CompositeFigure methodsFor:'private'!

compositionBoundsFor: aComponent
	^aComponent preferredBounds
!

computePreferredBounds
	| rect |
	components isEmpty ifTrue: [^0 @ 0 extent: 0 @ 0].
	rect := components first bounds.
	2 to: components size
		do: [:i | rect := rect merge: (components at: i) preferredBounds].
	^rect
!

mergeBounds: aRectangle
	bounds isNil ifTrue: [^self].
	self setBoundsTo: (bounds merge: aRectangle)
! !

!CompositeFigure methodsFor:'removing'!

remove: aFigure
	| rect |
	rect := aFigure bounds.
	aFigure container: nil.
	components remove: aFigure ifAbsent: [].
	self invalidateRectangle: rect.
	^aFigure
!

removeAll: aCollection
	| rect |
	aCollection isEmpty ifTrue: [^aCollection].
	rect := aCollection inject: nil into: [:sum :each | sum isNil
					ifTrue: [each bounds]
					ifFalse: [sum merge: each bounds]].
	aCollection
		do:
			[:each |
			each container: nil.
			components remove: each ifAbsent: []].
	self invalidateRectangle: rect.
	^aCollection
! !

!CompositeFigure methodsFor:'sorting'!

bringToFront: aFigure
	| index |
	index := components indexOf: aFigure ifAbsent: [^self].
	index to: components size - 1
		do: [:i | components at: i put: (components at: i + 1)].
	components at: components size put: aFigure.
	self invalidateRectangle: aFigure bounds
!

sendToBack: aFigure
	| index |
	index := components indexOf: aFigure ifAbsent: [^self].
	index to: 2
		by: -1
		do: [:i | components at: i put: (components at: i - 1)].
	components at: 1 put: aFigure.
	self invalidateRectangle: aFigure bounds
! !

!CompositeFigure methodsFor:'testing'!

containsPoint: aPoint
	^components contains: [:each | each containsPoint: aPoint]
!

isComposite
	^true
! !

!CompositeFigure methodsFor:'transforming'!

basicTranslateBy: aPoint
	components do: [:each | each translateBy: aPoint]
! !

!CompositeFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__CompositeFigure.st,v 1.1 2006/03/13 16:55:50 cg Exp $'
! !
