"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

CompositeFigure subclass:#Drawing
	instanceVariableNames:'selections handles controller'
	classVariableNames:'CopiedFigures'
	poolDictionaries:''
	category:'HotDraw-Framework'
!

Drawing comment:'Drawing is a view that allows the user to interact with the figure. In addition to the composite figure behavior, it cooperates with a controller to allow the user to manipulate the figures.

Instance Variables:
	controller      <DrawingController>       my controller
	handles <SequenceableCollection of: Handle>       handles that the user can interact with, only selections will have their handles displayed
	selections      <Collection of: Figure>   the figures that are currently selected by the user. Selected figures may be treated differently by different tools

Class Variables:
	CopiedFigures   <Collection of: Figure>   a collection of figures that were copied or cut from a drawing (for cut/paste operations)

'
!


!Drawing class methodsFor:'accessing'!

clearCutBuffer
	CopiedFigures := nil
! !

!Drawing class methodsFor:'parcel load/unload/save'!

postLoadActionFor: aParcel
	| comment |
	comment := aParcel comment.
	comment := comment
					copyFrom: (comment indexOfSubCollection: 'SUMMARY' startingAt: 1) + 9
					to: comment size.
	(ComposedTextView
			createOn: (TextCollector with: comment)
			label: 'HotDraw Installation Workspace'
			icon: (Icon constantNamed: #workspace))
		openIn: (30@250 extent: 500@260)

	"self postLoadActionFor: (Parcel parcelNamed: 'HotDraw Framework')"
! !

!Drawing class methodsFor:'private'!

figures: aCollection
	^(self new)
		setComponents: aCollection;
		yourself
! !

!Drawing methodsFor:'accessing'!

drawing
	^self
!

edit
	^DrawingEditor openOnDrawing: self
!

figureAt: aPoint
	"Drawings contain all points, if no figures contain this point, return self"

	| figure |
	handles size to: 1
		by: -1
		do:
			[:i |
			figure := (handles at: i) figureAt: aPoint.
			figure notNil ifTrue: [^figure]].
	components size to: 1
		by: -1
		do:
			[:i |
			figure := (components at: i) figureAt: aPoint.
			figure notNil ifTrue: [^figure]].
	^self
!

figures
	^components copy
!

figuresIn: aRectangle
	^components select: [:each | aRectangle contains: each]
!

menuAt: aPoint
	| mb |
	mb := MenuBuilder new.
	mb
		addCopyCutPaste;
		line;
		add: 'save...' -> #saveDrawing;
		add: 'load...' -> #loadDrawing;
		line;
		add: 'print' -> #print;
		add: 'save as postscript...' -> #savePostscript;
		line;
		add: 'inspect...' -> #inspect;
		line;
		add: 'group' -> #group;
		add: 'ungroup' -> #ungroup;
		line.
	self addLineColorMenuTo: mb.
	self addFillColorMenuTo: mb.
	self addLineWidthMenuTo: mb.
	self addModelMenuTo: mb at: aPoint.
	^mb menu
! !

!Drawing methodsFor:'broadcast'!

downcastEvent: aKey with: aParameter from: anInitiator
	"This is a message passed down the view structure to all subparts."

	super
		downcastEvent: aKey
		with: aParameter
		from: anInitiator.
	handles do:
			[:each |
			each
				downcastEvent: aKey
				with: aParameter
				from: anInitiator]
!

downcastLocalEvent: aKey with: aParameter at: aPoint from: anInitiator
	"This is a message passed down the view structure to some single
	part. Answer true if we accepted the event, or false if it should be
	passed on to whatever's behind us."

	handles size to: 1
		by: -1
		do:
			[:i |
			((handles at: i)
				downcastLocalEvent: aKey
				with: aParameter
				at: aPoint
				from: anInitiator) ifTrue: [^true]].
	^super
		downcastLocalEvent: aKey
		with: aParameter
		at: aPoint
		from: anInitiator
!

flushCoordinateCaches
	"Flush caches that relate to coordinate translations between this component
	and its container"

	super flushCoordinateCaches.
	handles do: [:each | each flushCoordinateCaches]
!

newGraphicsDevice: aGraphicsDevice
	"Forward the newGraphicsDevice: aGraphicsDevice messages
	to the receiver's components."

	super newGraphicsDevice: aGraphicsDevice.
	handles do: [:component | component newGraphicsDevice: aGraphicsDevice]
! !

!Drawing methodsFor:'control'!

componentWantingControl
	"Answer the first component that wishes to take control.  Answer nil if there is
	no component wanting control."

	| obj |
	handles size to: 1
		by: -1
		do:
			[:i |
			obj := (handles at: i) objectWantingControl.
			obj notNil ifTrue: [^obj]].
	^super componentWantingControl
!

objectWantingControl
	"The receiver is in a control hierarchy and the container is asking
	for an object that wants control.  If no control is desired then the receiver
	answers nil.  If control is wanted then the receiver answers the
	control object."

	| obj |
	obj := super objectWantingControl.
	obj notNil ifTrue: [^obj].
	^self componentWantingControl
! !

!Drawing methodsFor:'controller accessing'!

controller
	"Answer the receiver`s current controller. If the receiver's controller
	is nil (the default case), an initialized instance of the receiver's
	default controller is installed and returned."

	^self getController
!

controller: aController
	"Set the receiver's controller to aController.  An instance of NoController can be
	specified to indicate that the receiver will not have a controller.  The model of
	aController is set to the receiver's model."

	self setController: aController
!

defaultController
	"Answer an initialized instance of the receiver's default controller.
	Subclasses should redefine this message only if the default controller
	instances need to be initialized in a nonstandard way."

	^self defaultControllerClass new
!

defaultControllerClass
	^DrawingController
! !

!Drawing methodsFor:'copying'!

postCopy
	super postCopy.
	selections := nil.
	handles := #().
	controller := nil
! !

!Drawing methodsFor:'displaying'!

displayOn: aGraphicsContext
	| clipRect |
	clipRect := aGraphicsContext clippingBounds.
	aGraphicsContext paint: self backgroundColor.
	clipRect displayFilledOn: aGraphicsContext.
	super displayOn: aGraphicsContext.
	handles do:
			[:each |
			(each intersects: clipRect)
				ifTrue: [each displayOn: aGraphicsContext copy]]
!

invalidateRectangle: aRectangle repairNow: aBoolean
	"Hacked to avoid some off by one errors. Different OS's have different definitions of how to draw
	rectangles. Instead of trying to do something smart, just expand the rectangle..."

	^super invalidateRectangle: (aRectangle expandedBy: 1 @ 1) repairNow: aBoolean
!

invalidateRectangle: aRectangle repairNow: aBoolean forComponent: aFigure
	"Hacked to avoid some off by one errors. Different OS's have different definitions of how to draw
	rectangles. Instead of trying to do something smart, just expand the rectangle..."

	^super
		invalidateRectangle: (aRectangle expandedBy: 1 @ 1)
		repairNow: aBoolean
		forComponent: aFigure
! !

!Drawing methodsFor:'event driven'!

handlerForMouseEvent: aMouseEvent
	"The receiver is in a control hierarchy and the container is asking
	for an object that wants control. If no control is desired then the
	receiver answers nil. If control is wanted then the receiver answers the
	control object."

	self controller currentTool passInputDown
		ifTrue:
			[| obj |
			components size to: 1
				by: -1
				do:
					[:i |
					obj := (components at: i) handlerForMouseEvent: aMouseEvent.
					obj notNil ifTrue: [^obj]].
			^super handlerForMouseEvent: aMouseEvent]
		ifFalse: [^self controller]
!

updateSpot: type
	"Forward this message to all components."

	components do: [:cmp | cmp updateSpot: type]
! !

!Drawing methodsFor:'fileIn/Out'!

representBinaryOn: binWriter
	"Don't try to store our whole representation (just store our figures)."

	^MessageSend
		receiver: self class
		selector: #figures:
		argument: components
! !

!Drawing methodsFor:'initialize-release'!

initialize
	super initialize.
	selections := OrderedCollection new.
	handles := OrderedCollection new.
	self
		lineWidth: 1;   "Set the default properties of figures, someone must do this..."
		lineColor: ColorValue black;
		fillColor: ColorValue white;
		selectable: false;
		moveable: false;
		removeable: false;
		connectable: false
!

release
	Cursor normal show.
	super release
! !

!Drawing methodsFor:'menu actions'!

copySelection
	CopiedFigures := self selections collect: [:each | each copy]
!

cut
	CopiedFigures := self selections copy.
	self removeAll: self selections copy
!

group
	| figures newFigure |
	figures := selections copy.
	figures isEmpty ifTrue: [^self].
	self removeAll: figures.
	newFigure := CompositeFigure new.
	newFigure addAll: figures.
	self
		add: newFigure;
		selection: newFigure
!

loadDrawing
	| name boss drawing |
	name := Dialog requestFileName: 'Load drawing from:'.
	name isEmpty ifTrue: [^self].
	name asFilename exists ifFalse: [^Dialog warn: 'File does not exist'].
	boss := (#{BinaryObjectStorage} valueOrDo: [^self])
				onOld: name asFilename readStream.
	drawing := Cursor read showWhile: [[boss next] ensure: [boss close]].
	drawing
		container: container;
		model: model;
		privateSetController: controller.
	self oneWayBecome: drawing.
	self invalidate
!

paste
	CopiedFigures isNil ifTrue: [^self].
	self addAll: (CopiedFigures collect: [:each | each copy])
!

print
	| ps parent scale wrapper |
	parent := container.
	bounds := self computePreferredBounds expandedBy: 10 @ 10.
	ps := Printer startPrintJobNamed: 'HotDraw.postscript'.
	ps setLandscape: self extent x > self extent y.
	scale := (ps extent x / bounds extent x min: ps extent y / bounds extent y)
				min: 1.

	[wrapper := (ScalingWrapper on: self) scale: scale.
	self flushCaches.
	(MockMedium
		on: wrapper
		in: ps bounds
		for: ps)
		newGraphicsDevice: ps graphicsDevice;
		display]
			valueNowOrOnUnwindDo:
				[container := parent.
				self flushCaches.
				ps print]
!

saveDrawing
	| name boss oldSelections |
	name := Dialog requestFileName: 'Save drawing to:'.
	name isEmpty ifTrue: [^self].
	oldSelections := self selections.
	self
		clearSelections;
		flushCaches.
	boss := (#{BinaryObjectStorage} valueOrDo: [^self]) onNew: name asFilename writeStream.
	Cursor write showWhile:
			[[boss nextPut: self] ensure:
					[boss close.
					self selections: oldSelections]]
!

savePostscript
	| scale filename file gc drawingBounds |
	filename := Dialog request: 'Enter filename for output:'
				initialAnswer: 'drawing.ps'.
	filename isEmpty ifTrue: [^self].
	drawingBounds := self computePreferredBounds expandedBy: 10 @ 10.
	file := PostScriptFile named: filename.
	file setLandscape: drawingBounds extent x > drawingBounds extent y.
	gc := file graphicsContext.
	scale := gc clippingBounds extent x / drawingBounds extent x
				min: gc clippingBounds extent y / drawingBounds extent y.
	gc scaleBy: (scale min: 1).
	self flushCaches.
	self displayOn: gc.
	file close
!

ungroup
	| figures compositeFigures |
	compositeFigures := selections select: [:each | each isComposite].
	compositeFigures isEmpty ifTrue: [^self].
	figures := OrderedCollection new.
	compositeFigures do: [:each | figures addAll: each components].
	self
		removeAll: compositeFigures;
		addAll: figures;
		selections: figures
! !

!Drawing methodsFor:'private'!

changedPreferredBounds: oldPreferredBounds forComponent: aVisualComponent
	| newBounds newRect |
	newBounds := aVisualComponent bounds.
	self
		updatePreferredBoundsWith: (newBounds origin corner: newBounds corner + 1).     "Rectangle draw differently on different platforms"
	newRect := oldPreferredBounds isNil
				ifTrue: [newBounds]
				ifFalse: [oldPreferredBounds merge: newBounds].
	newRect origin: newRect origin - 1.
	newRect corner: newRect corner + 1.
	self invalidateRectangle: newRect repairNow: false
!

getController
	"Answer the receiver`s current controller. If the receiver's controller
	is nil (the default case), an initialized instance of the receiver's
	default controller is installed and returned."

	controller isNil ifTrue: [self setController: self defaultController].
	^controller
!

privateSetController: aController
	controller := aController
!

setComponents: aCollection
	components := aCollection.
	bounds := nil
!

setController: aController
	"Set the receiver's controller to aController. If aController is not nil, its view is set to the receiver
	and its model is set to the receiver's model."

	aController notNil
		ifTrue:
			[aController view: self.
			aController model: model].
	controller := aController
!

updatePreferredBoundsWith: aRectangle
	| oldBounds |
	oldBounds := bounds.
	bounds := bounds isNil
				ifTrue: [aRectangle]
				ifFalse: [bounds merge: aRectangle].
	oldBounds = bounds ifFalse: [self changedPreferredBounds: oldBounds].
	self invalidateRectangle: aRectangle repairNow: false
! !

!Drawing methodsFor:'removing'!

remove: aFigure
	aFigure isRemoveable ifFalse: [^self].
	self deselectFigure: aFigure.
	super remove: aFigure.
	^aFigure
!

removeAll: aCollection
	| toRemove |
	toRemove := aCollection select: [:each | each isRemoveable].
	self deselectFigures: toRemove.
	super removeAll: toRemove.
	^toRemove
! !

!Drawing methodsFor:'selection'!

clearSelections
	selections do: [:each | each deselect].
	selections := OrderedCollection new.
	handles do:
			[:each |
			each
				invalidate;
				container: nil].
	handles := OrderedCollection new
!

deselectFigure: aFigure
	selections remove: aFigure ifAbsent: [^self].
	aFigure deselect.
	handles := handles reject:
					[:each |
					| value |
					(value := each owner == aFigure)
						ifTrue:
							[each
								invalidate;
								container: nil].
					value]
!

deselectFigures: aCollection
	aCollection
		do:
			[:each |
			selections remove: each ifAbsent: [].
			each deselect].
	handles := handles
				reject:
					[:each |
					| value |
					(value := aCollection includes: each owner)
						ifTrue: [each invalidate; container: nil].
					value]
!

isSelected: aFigure
	^aFigure isSelected
!

selectFigure: aFigure
	| newHandles |
	aFigure isSelectable ifFalse: [^self].
	aFigure select.
	selections add: aFigure.
	newHandles := aFigure handles.
	handles addAll: newHandles.
	newHandles do:
			[:each |
			self mergeBounds: each bounds.
			each
				container: self;
				invalidate]
!

selection
	^selections isEmpty
		ifTrue: [nil]
		ifFalse: [selections first]
!

selection: aFigure
	self clearSelections.
	self selectFigure: aFigure
!

selections
	^selections
!

selections: aCollection
	self clearSelections.
	aCollection do: [:each | self selectFigure: each]
!

toggleSelection: aFigure
	(self isSelected: aFigure)
		ifTrue: [self deselectFigure: aFigure]
		ifFalse: [self selectFigure: aFigure]
!

toggleSelections: aCollection
	aCollection do: [:each | self toggleSelection: each]
! !

!Drawing methodsFor:'testing'!

containsPoint: aPoint
	"Drawings contain everything"

	^true
!

isDrawing
	^true
! !

!Drawing class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__Drawing.st,v 1.1 2006/03/13 16:55:24 cg Exp $'
! !
