"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

VisualPart subclass:#Figure
	instanceVariableNames:'dependents state attributes model bounds'
	classVariableNames:'Selected Connectable Invisible Removeable Filled Selectable
		Moveable'
	poolDictionaries:''
	category:'HotDraw-Framework'
!


!Figure class methodsFor:'instance creation'!

createAt: aPoint
	^(self new)
		translateTo: aPoint;
		yourself
! !

!Figure class methodsFor:'class initialization'!

initialize
    Selectable := 1.
    Moveable := 2.
    Invisible := 4. 
    Removeable := 8.
    Filled := 16.
    Selected := 32.
    Connectable := 64.

    "
     self initialize
    "
! !

!Figure methodsFor:'accessing'!

constrain: aSelector to: aMessageSend
	"Constrain yourself to aMessageSend's value. If aSelector is a setter, then
	send the message directly. If its a getter, then use the translateBy: message
	to move to the new position. See PositionConstraint for details."

	^PositionConstraint send: aSelector
		to: self
		with: aMessageSend
!

container: aContainer
	aContainer isNil ifTrue: [self delete].
	super container: aContainer
!

drawing
	"Return the drawing that you are located in or nil if you are not in any drawing right now."

	container isNil ifTrue: [^nil].
	^container drawing
!

figureAt: aPoint
	self isVisible ifFalse: [^nil].
	^(self containsPoint: aPoint) ifTrue: [self] ifFalse: [nil]
!

handles
	| handles |
	handles := OrderedCollection withAll: (TrackHandle allCornersOf: self).
	handles add: (Handle connectionOn: self at: #center).
	^handles
!

menuAt: aPoint
	"Figures have a menu that pops-up when clicked on by the yellow mouse button."

	| mb |
	mb := MenuBuilder new.
	mb
		addCopyCutPaste;
		line;
		add: 'inspect...' -> #inspect;
		line.
	self addLineColorMenuTo: mb.
	self addFillColorMenuTo: mb.
	self addLineWidthMenuTo: mb.
	self addModelMenuTo: mb at: aPoint.
	^mb menu
!

model
	^model
!

model: anObject
	model := anObject.
	model addDependent: self
!

owner
	^self
! !

!Figure methodsFor:'attribute accessing'!

attributes
	^attributes isNil
		ifTrue: [attributes := FigureAttributes new]
		ifFalse: [attributes]
!

beInvisible
	self setBit: Invisible to: true.
	self invalidate
!

beVisible
	self setBit: Invisible to: false.
	self invalidate
!

connectable: aBoolean
	self setBit: Connectable to: aBoolean
!

deselect
	self setBit: Selected to: false
!

fillColor
	^(attributes isNil or: [attributes fillColor isNil])
		ifTrue:
			[container isNil ifTrue: [self backgroundColor] ifFalse: [container fillColor]]
		ifFalse: [attributes fillColor]
!

fillColor: aColorValue
	self attributes fillColor: aColorValue.
	self invalidate
!

isSelected
	^(state bitAnd: Selected) ~~ 0
!

lineColor
	^(attributes isNil or: [attributes lineColor isNil])
		ifTrue:
			[container isNil ifTrue: [self foregroundColor] ifFalse: [container lineColor]]
		ifFalse: [attributes lineColor]
!

lineColor: aColorValue
	self attributes lineColor: aColorValue.
	self invalidate
!

lineWidth
	^(attributes isNil or: [attributes lineWidth isNil])
		ifTrue: [container isNil ifTrue: [1] ifFalse: [container lineWidth]]
		ifFalse: [attributes lineWidth]
!

lineWidth: anInteger
	self attributes lineWidth: anInteger.
	self recomputePreferredBounds
!

moveable: aBoolean
	self setBit: Moveable to: aBoolean not
!

opaque
	self setBit: Filled to: true.
	self invalidate
!

removeable: aBoolean
	self setBit: Removeable to: aBoolean not
!

select
	self setBit: Selected to: true
!

selectable: aBoolean
	self setBit: Selectable to: aBoolean not
!

transparent
	self setBit: Filled to: false.
	self invalidate
! !

!Figure methodsFor:'bounding box accessing'!

bottom
	^self bounds bottom
!

bottomCenter
	^self bounds bottomCenter
!

bottomLeft
	^self bounds bottomLeft
!

bottomLeft: aPoint
	self setBoundsTo: (Rectangle vertex: aPoint x @ self origin y
				vertex: self corner x @ aPoint y)
!

bottomRight
	^self bounds bottomRight
!

bottomRight: aPoint
	self setBoundsTo: (Rectangle vertex: self origin vertex: aPoint)
!

center
	^self bounds center
!

corner
	^self bounds corner
!

corner: aPoint
	self bottomRight: aPoint
!

extent
	^self bounds extent
!

left
	^self bounds left
!

leftCenter
	^self bounds leftCenter
!

offCenter: deltaPoint
	^self center + deltaPoint
!

offCorner: deltaPoint
	^self corner + deltaPoint
!

offOrigin: deltaPoint
	^self origin + deltaPoint
!

origin
	^self bounds origin
!

origin: aPoint
	self topLeft: aPoint
!

position: aPoint offset: offsetPoint
	^self origin + (self extent * aPoint) + offsetPoint
!

right
	^self bounds right
!

rightCenter
	^self bounds rightCenter
!

top
	^self bounds top
!

topCenter
	^self bounds topCenter
!

topLeft
	^self bounds topLeft
!

topLeft: aPoint
	self setBoundsTo: (Rectangle vertex: aPoint vertex: self corner)
!

topRight
	^self bounds topRight
!

topRight: aPoint
	self setBoundsTo: (Rectangle vertex: self origin x @ aPoint y
				vertex: aPoint x @ self corner y)
! !

!Figure methodsFor:'bounds accessing'!

computePreferredBounds
	"Compute the preferredBounds of the figure. For some figures, the preferredBounds
	is always set so this method is never called. For the other figures, this should
	return a rectangle of the bounds."

	^bounds
!

preferredBounds
	^bounds isNil
		ifTrue: [bounds := self computePreferredBounds]
		ifFalse: [bounds]
!

recomputePreferredBounds
	^self setBoundsTo: self computePreferredBounds
! !

!Figure methodsFor:'changing'!

changed: anAspectSymbol with: aParameter
	super changed: anAspectSymbol with: aParameter.
	self invalidate
!

delete
	"This message is sent when a figure is being deleted. Default behavior is to notify dependents. For
	example, this will let a line delete itself when the figures it is attached to are deleted."

	self changed: #deleted with: nil
!

deletionUpdateFrom: aFigure
	"aFigure was removed from the drawing. I don't know what you want to do, so I just remove myself as a
	deletion dependent of aFigure. For lines the default action is to remove the line."

	aFigure removeDependent: self.
	self removeDependent: aFigure
!

update: anAspectSymbol with: aParameter from: aSender
	"Somebody has justed changed. If it is as a result of a deletion dependent then it will be #deleted."

	anAspectSymbol = #deleted ifTrue: [^self deletionUpdateFrom: aSender].
	super update: anAspectSymbol
		with: aParameter
		from: aSender
! !

!Figure methodsFor:'connection'!

canConnectFromFigure: aFigure
	"Can we connect from aFigure. We might want some additional checking to verify that
	this is a legal operation for our drawing's sematics."

	^true
!

canConnectFromPoint: aPoint
	"Should we allow outgoing connections from aPoint?"

	^self canBeConnected
!

canConnectFromPoint: myPoint to: aFigure at: figurePoint
	^(self canConnectFromPoint: myPoint) and:
			[(aFigure canConnectToPoint: figurePoint) and:
					[(self canConnectToFigure: aFigure)
						and: [aFigure canConnectFromFigure: self]]]
!

canConnectToFigure: aFigure
	"Can we connect to aFigure. We might want some additional checking to verify that
	this is a legal operation for our drawing's sematics."

	^self canBeConnected and: [self ~~ aFigure]
!

canConnectToPoint: aPoint
	"Can we allow a connection to aPoint? If we have several 'ports' we might only allow
	certain connections to some ports."

	^self canBeConnected
!

connectFromPoint: myPoint to: aFigure at: figurePoint
	"Return a figure that represents the connection from self to aFigure (using myPoint
	and figurePoint as the connection points)."

	| newFigure |
	newFigure := self connectionFigureClass connect: self to: aFigure.
	self createStartConnectionConstraintFor: newFigure at: myPoint.
	aFigure createStopConnectionConstraintFor: newFigure at: figurePoint.
	^newFigure
!

connectionFigureClass
	"The type of figure that we should use for connections"

	^LineFigure
!

connectionPositions
	"If we don't use a boundary constraint, try to guess a good location for the end point of the line."

	^#(#origin #corner #topRight #bottomLeft #center #topCenter #rightCenter #bottomCenter #leftCenter)
!

createStartConnectionConstraintFor: newFigure at: myPoint
	"If we are a filled figure, then try connecting to our boundary (our center
	must be inside us). If not filled, then try to find what part we are connecting
	to/from. Subclasses can override to connect to figure parts."

	self isOpaque
		ifTrue: [BoundaryConstraint forFigure: self startLine: newFigure]
		ifFalse:
			[| position |
			position := self shortestPositionTo: myPoint.
			PositionConstraint send: #startPoint:
				to: newFigure
				with: (MessageSend receiver: self selector: position)]
!

createStopConnectionConstraintFor: newFigure at: myPoint
	"If we are a filled figure, then try connecting to our boundary (our center
	must be inside us). If not filled, then try to find what part we are connecting
	to/from. Subclasses can override to connect to figure parts."

	self isOpaque
		ifTrue: [BoundaryConstraint forFigure: self stopLine: newFigure]
		ifFalse:
			[| position |
			position := self shortestPositionTo: myPoint.
			PositionConstraint send: #stopPoint:
				to: newFigure
				with: (MessageSend receiver: self selector: position)]
!

shortestPositionTo: myPoint
	"Return a symbol from #connectionPositions that is the shortest distance from myPoint."

	| min position positions |
	positions := self connectionPositions.
	positions isEmpty ifTrue: [^#center].
	min := myPoint dist: (self perform: positions first).
	position := positions first.
	positions do:
			[:each |
			| dist |
			dist := myPoint dist: (self perform: each).
			dist < min
				ifTrue:
					[min := dist.
					position := each]].
	^position
! !

!Figure methodsFor:'copying'!

postCopy
	super postCopy.
	container := nil.
	dependents := nil.
	bounds := self computePreferredBounds.
	self deselect
! !

!Figure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	^self subclassResponsibility
!

displayOn: aGraphicsContext
	self isVisible ifTrue:
			[self isSelected
				ifTrue: [self displaySelectedFigureOn: aGraphicsContext]
				ifFalse: [self displayFigureOn: aGraphicsContext]]
!

displaySelectedFigureOn: aGraphicsContext
	^self displayFigureOn: aGraphicsContext
! !

!Figure methodsFor:'initialize-release'!

initialize
	super initialize.
	model := nil.
	state := Connectable
!

release
	model removeDependent: self.
	^super release
! !

!Figure methodsFor:'private'!

addFillColorMenuTo: aMenuBuilder
	aMenuBuilder beginSubMenuLabeled: 'fill color'.
	ColorValue constantNames do:
			[:each |
			aMenuBuilder
				add: each asString -> [self fillColor: (ColorValue perform: each)]].
	aMenuBuilder endSubMenu
!

addLineColorMenuTo: aMenuBuilder
	aMenuBuilder beginSubMenuLabeled: 'line color'.
	ColorValue constantNames do:
			[:each |
			aMenuBuilder
				add: each asString -> [self lineColor: (ColorValue perform: each)]].
	aMenuBuilder endSubMenu
!

addLineWidthMenuTo: aMenuBuilder
	aMenuBuilder beginSubMenuLabeled: 'line width'.
	1 to: 5
		do:
			[:i |
			aMenuBuilder
				add: (i printString , (i == 1 ifTrue: [' pixel'] ifFalse: [' pixels']))
						-> [self lineWidth: i]].
	aMenuBuilder endSubMenu
!

addModelMenuTo: aMenuBuilder at: aPoint
	(model respondsTo: #menuAt:)
		ifTrue:
			[aMenuBuilder
				line;
				add: 'Model' -> (model menuAt: aPoint)]
		ifFalse:
			[(model respondsTo: #menu)
				ifTrue:
					[aMenuBuilder
						line;
						add: 'Model' -> model menu]]
!

addVisibilityMenuTo: aMenuBuilder
	aMenuBuilder
		beginSubMenuLabeled: 'visibility';
		add: 'transparent' -> #transparent;
		add: 'opaque' -> #opaque;
		endSubMenu
!

myDependents
	"Answer the receiver's dependents or nil. Copied down from Model to make dependency checking faster."

	^dependents
!

myDependents: dependentsOrNil
	"Set the receivers dependents. Copied down from Model to make dependency checking faster."

	dependents := dependentsOrNil
!

setBit: mask to: aBoolean
	state := state bitOr: mask.
	aBoolean ifFalse: [state := state bitXor: mask]
! !

!Figure methodsFor:'testing'!

canBeConnected
	^(state bitAnd: Connectable) ~~ 0
!

containedBy: aRectangle
	"Are you contained in aRectangle?"

	^aRectangle contains: self bounds
!

isComposite
	^false
!

isConnectionFigure
	^false
!

isDrawing
	^false
!

isHandle
	^false
!

isMoveable
	^(state bitAnd: Moveable) == 0
!

isOpaque
	^(state bitAnd: Filled) ~~ 0
!

isRemoveable
	^(state bitAnd: Removeable) == 0
!

isSelectable
	^(state bitAnd: Selectable) == 0
!

isStateFigure
	^false
!

isVisible
	^(state bitAnd: Invisible) == 0
! !

!Figure methodsFor:'transforming'!

align: alignmentPoint with: relativePoint
	self translateBy: relativePoint - alignmentPoint
!

basicTranslateBy: aPoint
	"This method is called by #translateBy:. The #translateBy: method has already moved our
	bounds. If we have other points that need to be moved, we need to move them also."
!

scaleBy: aPoint
	| center |
	center := self center.
	self align: center with: (center scaledBy: aPoint)
!

setBoundsTo: aRectangle
	"We have moved to aRectangle. Update our bounds, and invalidate our old and new positions."

	| oldBounds |
	oldBounds := self preferredBounds.
	bounds := aRectangle.
	self changedPreferredBounds: oldBounds.
	self changed
!

translateBy: aPoint
	"Move ourself by aPoint. Instead of overriding this method,
	subclasses will probably just need to override basicTranslateBy:"

	| oldBounds |
	self isMoveable ifFalse: [^self].
	aPoint = (0 @ 0) ifTrue: [^self].
	oldBounds := self preferredBounds.
	self translatePreferredBoundsBy: aPoint.
	self basicTranslateBy: aPoint.
	self changedPreferredBounds: oldBounds.
	self changed
!

translatePreferredBoundsBy: aPoint
	bounds := self preferredBounds translatedBy: aPoint
!

translateTo: aPoint
	"Move origin to aPoint. This used to move corner to aPoint, which seemed wierd."

	self translateBy: aPoint - self origin
! !

!Figure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__Figure.st,v 1.1 2006/03/13 16:57:11 cg Exp $'
! !

Figure initialize!
