"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Model subclass:#ObjectWorldFigureModel
	instanceVariableNames:'figure name animationMethods model pause stayVisible'
	classVariableNames:'DefaultMovement DefaultOrbit DebugFlag'
	poolDictionaries:''
	category:'HotDraw-ObjectWorld'
!

ObjectWorldFigureModel comment:'a MetaFigure for the ObjectWorld application.
=========================
Assumes that a figure must implement :
	rotateBy:     Rotate the figure by the given angle.

Instance Variables:

name <String>
    The name of the figure in the ObjectWorld.  This is the string associated with the figure by the user.

animationMethods  <Collection>
    A collection of the method the figure should apply to itself to perform a step in its animation.

model <ObjectParser>
   The ObjectWorld that the figure belongs to.

pause <Boolean>
   Indicator for the figure as to whether or not it will perform its animation step.  This allows each figure to control whether or not it wants to perform a step in its
   animation.

stayVisible <Boolean>
   Indicator telling the figure if it should stay within the visible bounds given to it as part of an animation step.

Class Variables:
DebugFlag <Boolean>
   Indicator telling the figure to perform debug operations.

DefaultGrow <Integer>
   Amount to grow a figure when the grow or shrink message is sent to a figure.

DefaultRotate <Float>
   Angle to rotate a figure when the turnRight or turnLeft message is sent to a figure.

DefaultMovement <Integer>
   Amount to move a figure when it is sent the moveLeft, moveRight, moveUp, or moveDown messages are sent to it.

DefaultOrbit <Float>
   Angle to rotate the figure around the object it is orbiting.

DefaultSize <Integer>
   Size in pixels to create the initial figure.'
!


!ObjectWorldFigureModel class methodsFor:'instance creation'!

new
	^super new initialize
!

on: aFigure
	| model |
	model := self new.
	model figure: aFigure.
	aFigure model: model
! !

!ObjectWorldFigureModel class methodsFor:'accessing'!

debugFlag
	^DebugFlag
!

debugFlag: aBoolean
	"Control the class' DebugFlag: if true, then handle doesNotUnderstand
	messages in the usual way, i.e. raise an exception. If false (the default),
	then override doesNotUnderstand: to just flash the view to signal the
	error to the user."

	DebugFlag := aBoolean
!

defaultMovement
	"Return the default amount of movement for a figure"

	^DefaultMovement
!

defaultMovement: anInteger
	"Set the default amount of movement for a figure to anInteger"

	DefaultMovement := anInteger
!

defaultOrbit
	^DefaultOrbit
!

defaultOrbit: aFloat
	DefaultOrbit := aFloat
! !

!ObjectWorldFigureModel methodsFor:'accessing'!

menu
	| mb |
	mb := MenuBuilder new.
	mb add: 'Rename...' -> #rename.
	^mb menu
! !

!ObjectWorldFigureModel methodsFor:'animating'!

animate: aMethod
	"Add aMethod to the list of methods to invoke for animation on this
	figure. aMethod is either a block or the name of a message to send."

	| block |
	block := aMethod isSymbol
				ifTrue:
					[(self respondsTo: aMethod)
						ifTrue: [[:fig | fig perform: aMethod]]
						ifFalse: [^self error: aMethod , ' is an illegal message']]
				ifFalse: [aMethod].
	animationMethods addFirst: block
!

checkVisible: drawingBounds oldBounds: oldBounds
	| newBox xDirection yDirection |
	stayVisible ifFalse: [^self].
	newBox := self bounds.
	xDirection := newBox left - oldBounds left.
	yDirection := newBox top - oldBounds top.
	xDirection < 0 & (newBox right < drawingBounds left)
		ifTrue: [self translateBy: (drawingBounds width + newBox width) @ 0].
	xDirection > 0 & (newBox left > drawingBounds right)
		ifTrue: [self translateBy: (drawingBounds width + newBox width) negated @ 0].
	yDirection < 0 & (newBox bottom < drawingBounds top)
		ifTrue: [self translateBy: 0 @ (drawingBounds height + newBox height)].
	yDirection > 0 & (newBox top > drawingBounds bottom)
		ifTrue: [self translateBy: 0 @ (drawingBounds height + newBox height) negated]
!

home
	"Tell this Figure to move to the center of the visible part of the view. "

	| newCenter bounds |
	bounds := model drawing bounds.
	newCenter := bounds center - self center.
	figure translateBy: newCenter
!

pause
	"Suspend animation for this Figure."

	pause := true
!

resume
	"Begin animation for this Figure."

	pause := false
!

stayVisible
	"Require that this Figure stay within the visible part of the view."

	stayVisible := true
!

stepIn: drawingBounds
	"Perform all the methods in the animation list on this Figure. This is
	meant to move the figure one step through its animation. If the pause
	indicator is set, skip animation for this figure. If animation is to occur,
	then once it has moved through one step, see if the figure is supposed
	to stay within the visible bounds of drawing that were passed in. If so,
	make sure the user can still see the figure, if not, move it to the opposite
	side of the drawing based on the direction the figure is traveling in."

	| oldBounds |
	oldBounds := self bounds.
	pause ifTrue: [^self].
	animationMethods do: [:method | method value: self].
	self checkVisible: drawingBounds oldBounds: oldBounds
!

stop
	"Stop all the methods in the animation list on this Figure. This is an
	   ObjectWorld extension. This method is intended to be invoked
	 by the ObjectWorld user."

	[animationMethods isEmpty] whileFalse: [animationMethods removeFirst]
!

stop: aMethod
	"Stop the given method from the animation list for the figure.
	   This is an ObjectWorld extension. This method is intended to be invoked
	 by the ObjectWorld user."

	| methodSymbol |
	methodSymbol := aMethod asSymbol.
	(animationMethods includes: methodSymbol)
		ifTrue: [animationMethods remove: methodSymbol ifAbsent: []]
!

unbounded
	stayVisible := false
! !

!ObjectWorldFigureModel methodsFor:'figure'!

bounds
	^figure bounds
!

center
	^figure center
!

changed
	^figure changed
!

damage
	^figure damage
!

number
	^figure number
!

number: aNumber
	^figure number: aNumber
!

translateBy: aPoint
	^figure translateBy: aPoint
! !

!ObjectWorldFigureModel methodsFor:'initialize-release'!

figure
	^figure
!

figure: aFigure
	figure := aFigure
!

initialize
	"Create the animation list, start the figure out with pausing turned off
	   and let it roam outside the visible part of the drawing. "

	animationMethods := OrderedCollection new.
	pause := false.
	stayVisible := false
!

model: aModel
	model := aModel
!

name
	"Return the name of the figure."

	^name
!

name: aString
	"Register the receiver with the local model. Check if the receiver has been registered
	before (i.e. if the receiver already has a non-null name), in which case the receiver
	must unregister itself before re-registering under the new name."

	name isNil ifFalse: [model unregister: self].
	model register: self as: aString.
	name := aString
! !

!ObjectWorldFigureModel methodsFor:'private'!

doesNotUnderstand: aMessage
	"Override the same method in the Object class to allow us to
	signal the user that they made an error by flashing the view,
	rather than by raising an exception and popping up a window."

	self class debugFlag
		ifTrue: [^super doesNotUnderstand: aMessage]
		ifFalse: [model parser view flash]
!

rename
	"Let the ObjectWorld user change the name of the receiver with a dialogue."

	| newname defaultName |
	defaultName := name isNil ifTrue: [''] ifFalse: [name].
	newname := Dialog request: 'Enter name of object:'
				initialAnswer: defaultName.
	self name: newname
! !

!ObjectWorldFigureModel methodsFor:'removing'!

remove
	"Remove the receiving Figure from its Drawing."

	figure changed.
	self unregister.
	model drawing remove: self
!

unregister
	"Remove the receiving Figure from its Drawing."

	model unregister: self
! !

!ObjectWorldFigureModel methodsFor:'transforming'!

moveDown
	"Move the receiving Figure downwards the default distance."

	self moveDown: self class defaultMovement
!

moveDown: anInteger
	"Move the receiving Figure downwards the specified distance.
	Intended for the ObjectWorld user."

	self translateBy: 0 @ anInteger
!

moveLeft
	"Move the receiving Figure left the default distance.
	Intended for the ObjectWorld user."

	self moveLeft: self class defaultMovement
!

moveLeft: anInteger
	"Move the receiving Figure left the specified distance.
	Intended for the ObjectWorld user."

	self translateBy: anInteger negated @ 0
!

moveRight
	"Move the receiving Figure right the default distance.
	Intended for the ObjectWorld user."

	self moveRight: self class defaultMovement
!

moveRight: anInteger
	"Move the receiving Figure right the specified distance.
	Intended for the ObjectWorld user."

	self translateBy: anInteger @ 0
!

moveUp
	"Move the receiving Figure upwards the default distance.
	Intended for the ObjectWorld user."

	self moveUp: self class defaultMovement
!

moveUp: anInteger
	"Move the receiving Figure upwards the specified distance.
	Intended for the ObjectWorld user."

	self translateBy: 0 @ anInteger negated
!

orbit: aFigure
	"Make the receiving Figure move in an 'orbit' around aFigure.
	To do this, we rotate the receiver about the center of aFigure.
	This is an ObjectWorld user method."

	| deltaX deltaY sin cos newX newY |
	deltaX := (self center x - aFigure center x) asDouble.
	deltaY := (self center y - aFigure center y) asDouble.
	sin := self class defaultOrbit sin.
	cos := self class defaultOrbit cos.
	newX := aFigure center x + (deltaX * cos) - (deltaY * sin) - self center x.
	newY := aFigure center y + (deltaY * cos) + (deltaX * sin) - self center y.
	figure translateBy: newX @ newY
! !

!ObjectWorldFigureModel class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__ObjectWorldFigureModel.st,v 1.1 2006/03/13 16:57:50 cg Exp $'
! !
