"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#PolylineFigure
	instanceVariableNames:'points'
	classVariableNames:''
	poolDictionaries:''
	category:'HotDraw-Figures'
!

PolylineFigure comment:'PolylineFigure is a figure that is represented by a series of points. The default polyline figure just draws straight lines between these points, but other subclasses can draw curved lines through the points. If this figure is filled, then the start and end points should be the same.

Instance Variables:
	points  <SequenceableCollection of: Point>        our points

'
!


!PolylineFigure class methodsFor:'instance creation'!

connect: aFigure to: anotherFigure
	| figure |
	figure := self withPoints: (Array with: aFigure center with: anotherFigure center).
	aFigure addDependent: figure.
	anotherFigure addDependent: figure.
	^figure
!

createAt: aPoint
	^self withPoints: (Array with: aPoint with: aPoint)
!

start: aPoint stop: anotherPoint
	^self withPoints: (Array with: aPoint with: anotherPoint)
!

withPoints: aCollection
	^self new points: aCollection
! !

!PolylineFigure methodsFor:'accessing'!

centerOfSegment: anIndex
	^((points at: anIndex) + (points at: anIndex + 1)) // 2
!

handles
	| handles |
	handles := OrderedCollection new.
	1 to: points size - 1
		do:
			[:i |
			handles
				add: ((i == 1 ifTrue: [TrackHandle] ifFalse: [IndexedTrackHandle])
						pointAtIndex: i
						of: self).
			handles add: (TentativePositionHandle forSegment: i of: self)].
	handles add: (TrackHandle pointAtIndex: points size of: self).
	^handles
!

pointAt: anIndex
	^points at: anIndex
!

pointAt: anIndex put: aValue
	| oldBounds |
	oldBounds := self preferredBounds.
	points at: anIndex put: aValue.
	bounds := self computePreferredBounds.
	self changedPreferredBounds: oldBounds.
	anIndex == 1 ifTrue: [^self changed: #startPoint].
	anIndex == points size ifTrue: [^self changed: #stopPoint].
	self changed
!

pointsSize
	^points size
!

startPoint
	^points first
!

startPoint: aPoint
	self pointAt: 1 put: aPoint
!

stopPoint
	^points last
!

stopPoint: aPoint
	self pointAt: points size put: aPoint
! !

!PolylineFigure methodsFor:'adding'!

addPoint: aPoint
	points add: aPoint.
	self updatePreferredBoundsWith: aPoint.
	^aPoint
!

addPoint: aPoint beforeIndex: aPosition
	points add: aPoint beforeIndex: aPosition.
	self updatePreferredBoundsWith: aPoint.
	^aPoint
! !

!PolylineFigure methodsFor:'bounds accessing'!

computePreferredBounds
	| rect |
	rect := points inject: (points first extent: 0 @ 0)
				into:
					[:sum :point |
					sum
						origin: (sum origin min: point);
						corner: (sum corner max: point)].
	rect corner: rect corner + (self lineWidth // 2) + 1.   "Add one, so we get the horizontal/vertical lines"
	rect origin: rect origin - (self lineWidth // 2).
	^rect
! !

!PolylineFigure methodsFor:'changing'!

deletionUpdateFrom: aFigure
	"Since we're probably connected to the figure that was deleted, remove ourself"

	super deletionUpdateFrom: aFigure.
	self drawing notNil ifTrue: [self drawing remove: self]
! !

!PolylineFigure methodsFor:'converting'!

asGeometric
	^Polyline vertices: points
! !

!PolylineFigure methodsFor:'copying'!

postCopy
	super postCopy.
	points := points copy
! !

!PolylineFigure methodsFor:'displaying'!

displayFigureOn: aGraphicsContext
	| geometric |
	geometric := self asGeometric.
	self isOpaque ifTrue:
			[aGraphicsContext paint: self fillColor.
			geometric displayFilledOn: aGraphicsContext].
	self lineWidth > 0 ifTrue:
			[aGraphicsContext lineWidth: self lineWidth.
			aGraphicsContext paint: self lineColor.
			geometric displayStrokedOn: aGraphicsContext]
! !

!PolylineFigure methodsFor:'initialize-release'!

initialize
	super initialize.
	points := #()
!

points: aCollection
	points := aCollection asOrderedCollection
! !

!PolylineFigure methodsFor:'private'!

updatePreferredBoundsWith: aPoint
	| oldBounds |
	oldBounds := self preferredBounds.
	bounds := (oldBounds origin min: aPoint) corner: (oldBounds corner max: aPoint).
	oldBounds ~= bounds ifTrue: [self changedPreferredBounds: oldBounds]
! !

!PolylineFigure methodsFor:'removing'!

removePointAtIndex: aPosition
	| isSelected |
	(isSelected := self isSelected)
		ifTrue: [self drawing deselectFigure: self].
	points size > 2 ifFalse: [^self].
	points removeAtIndex: aPosition.
	self recomputePreferredBounds.
	isSelected ifTrue: [self drawing selectFigure: self]
! !

!PolylineFigure methodsFor:'testing'!

containsPoint: aPoint
	^self isOpaque
		ifTrue:
			[self asGeometric regionIntersects: (aPoint - 3 - (self lineWidth // 2)
						corner: aPoint + 3 + (self lineWidth // 2))]
		ifFalse:
			[self asGeometric outlineIntersects: (aPoint - 3 - (self lineWidth // 2)
						corner: aPoint + 3 + (self lineWidth // 2))]
! !

!PolylineFigure methodsFor:'transforming'!

basicTranslateBy: aPoint
	points := points collect: [:each | each + aPoint]
! !

!PolylineFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__PolylineFigure.st,v 1.1 2006/03/13 16:59:21 cg Exp $'
! !
