"{ Package: 'stx:goodies/hotdraw' }"

"{ NameSpace: HotDraw }"

Figure subclass:#TextFigure
	instanceVariableNames:'paragraph selection'
	classVariableNames:'InsertionPoint'
	poolDictionaries:''
	category:'HotDraw-Figures'
!

TextFigure comment:'TextFigure is a Figure that represents text.

Instance Variables:
	paragraph       <Paragraph>       the text that is displayed
	selection       <Interval | nil>  the selected text interval, when the figure is part of the drawing''s selection, then this selection is also highlighted

Class Variables:
	InsertionPoint  <Image>   an image of the ^ that is displayed at the insertion point.

'
!


!TextFigure class methodsFor:'instance creation'!

createAt: aPoint with: aString
	^self string: aString at: aPoint
!

paragraph: aParagraph at: aPoint
	^self new setParagraph: aParagraph origin: aPoint
!

string: aString
	^self string: aString at: 0 @ 0
!

string: aString at: aPoint
	^self paragraph: aString asComposedText at: aPoint
! !

!TextFigure methodsFor:'accessing'!

copyFrom: startIndex to: stopIndex
	^paragraph asText copyFrom: startIndex to: stopIndex
!

handles
	| handles |
	handles := OrderedCollection new.
	handles add: (Handle connectionOn: self at: #origin).
	handles add: ((TrackHandle topRightOf: self)
				moveBlock: [:aPoint | self moveMarginTo: aPoint]).
	^handles
!

indexForPoint: aPoint
	"Answer the index into my text that most closely corresponds to aPoint."

	^(paragraph characterBlockAtPoint: aPoint - self origin) stringIndex
!

margin
	^self extent x
!

margin: aNumber
	paragraph compositionWidth: aNumber rounded.
	self recomputePreferredBounds.
	^aNumber
!

menuAt: aPoint
	| mb |
	mb := MenuBuilder new.
	self addJustificationMenuTo: mb.
	self addVisibilityMenuTo: mb.
	self addStyleMenuTo: mb.
	^(mb menu)
		addItem: ((MenuItem labeled: 'other')
					submenu: (super menuAt: aPoint);
					yourself);
		yourself
!

paragraph
	^paragraph
!

print: anObject
	self string: anObject printString
!

replaceFrom: startIndex to: stopIndex with: aText
	paragraph := (paragraph copy)
				replaceFrom: startIndex
					to: stopIndex
					with: aText;
				yourself.
	self margin: paragraph rightMarginForComposition
!

replaceSelectionWith: aText
	self
		replaceFrom: selection first
		to: selection last - 1
		with: aText
!

selectedIndices
	^selection
!

setIndices: newIndices
	"Set the indices of the highlighted part --- newly added"

	newIndices notNil
		ifTrue:
			[(selection notNil and: [selection first = newIndices first
				and: [selection last = newIndices last]])
					ifFalse:
						[selection := newIndices.
						self changed]]
		ifFalse:
			[selection notNil
				ifTrue:
					[selection := nil.
					self changed]]
!

string
	^paragraph asString
!

string: aString
	self text: aString asText
!

text: aText
	paragraph := aText asComposedText
! !

!TextFigure methodsFor:'attribute accessing'!

deselect
	super deselect.
	selection := nil.
	self changed
!

select
	super select.
	self changed
! !

!TextFigure methodsFor:'bounds accessing'!

computePreferredBounds
	| extent |
	extent := paragraph width @ paragraph height.
	^self origin extent: extent
! !

!TextFigure methodsFor:'copying'!

postCopy
	super postCopy.
	paragraph := paragraph asString asComposedText
! !

!TextFigure methodsFor:'defaults'!

defaultCompositionWidth
	^100
! !

!TextFigure methodsFor:'displaying'!

displayCaretOn: aGraphicsContext
	"Show a caret inbetween characters anInteger and anInteger+1. Actually, reverse the
	image of a caret at this point. Thus we can toggle the caret on and off by calling this method
	repeatedly."

	| ipExtent aPoint aBlock |
	ipExtent := InsertionPoint extent.
	aBlock := self paragraph characterBlockForIndex: selection first.
	aPoint := (aBlock left - (ipExtent x // 2))
				@ (aBlock top + self paragraph textStyle baseline) extent: ipExtent.
	InsertionPoint displayOn: aGraphicsContext at: self origin + aPoint origin
!

displayEditOn: aGraphicsContext
	selection isNil ifTrue: [^self].
	selection first = selection last
		ifTrue: [self displayCaretOn: aGraphicsContext]
		ifFalse: [self highlightOn: aGraphicsContext]
!

displayFigureOn: aGraphicsContext
	"If I am opaque, draw a box in the background color obscuring those figures beneath me."

	self isOpaque
		ifTrue:
			[aGraphicsContext
				paint: self fillColor;
				displayRectangle: self bounds].
	aGraphicsContext
		paint: self lineColor;
		display: paragraph at: self origin
!

displaySelectedFigureOn: aGraphicsContext
	self displayFigureOn: aGraphicsContext.
	self displayEditOn: aGraphicsContext
!

highlightOn: aGraphicsContext
	"Display the image of my text from start to stop. Note they may not be in the proper
	order."

	| startPoint stopPoint lineRange first last start stop |
	start := selection first.
	stop := selection last.
	start = stop ifTrue: [^self].
	startPoint := paragraph characterBlockForIndex: (start min: stop).
	stopPoint := paragraph characterBlockForIndex: (start max: stop).
	aGraphicsContext paint: self selectionBackgroundColor.
	startPoint top = stopPoint top
		ifTrue:
			[aGraphicsContext displayRectangle: (startPoint origin + self origin
						corner: stopPoint bottomLeft + self origin).
			aGraphicsContext paint: self selectionForegroundColor.
			^paragraph
				displayFromCharacter: startPoint stringIndex
				to: stopPoint stringIndex - 1
				startX: startPoint left + self origin x
				forTranslation: self origin
				on: aGraphicsContext].
	lineRange := paragraph
				lineRangeFor: (startPoint origin corner: stopPoint corner).
	first := (paragraph lineAt: lineRange first) first.
	last := (paragraph lineAt: lineRange last) last.
	first := first max: startPoint stringIndex.
	last := last min: stopPoint stringIndex - 1.
	aGraphicsContext displayRectangle: (startPoint origin + self origin
				corner: self extent x @ startPoint bottom + self origin).
	aGraphicsContext
		displayRectangle: (self origin x @ (startPoint bottom + self origin y)
				corner: self extent x @ stopPoint top + self origin).
	aGraphicsContext
		displayRectangle: (self origin x @ (startPoint bottom + self origin y)
				corner: stopPoint bottomLeft + self origin).
	aGraphicsContext paint: self selectionForegroundColor.
	paragraph
		displayFromCharacter: first
		to: last
		startX: startPoint left + self origin x
		forTranslation: self origin
		on: aGraphicsContext
! !

!TextFigure methodsFor:'formatting'!

alignUsing: aSymbol
	self margin: self margin.
	paragraph textStyle: paragraph textStyle copy.
	paragraph perform: aSymbol.
	self changed
!

styleUsing: aSymbol
	| start stop |
	selection isNil
		ifTrue:
			[start := 1.
			stop := paragraph asString size]
		ifFalse:
			[start := selection first.
			stop := selection last - 1].
	self paragraph asText
		emphasizeFrom: start
		to: stop
		with: aSymbol.
	self recompose
! !

!TextFigure methodsFor:'initialize-release'!

setParagraph: aParagraph origin: aPoint
	paragraph := aParagraph.
	paragraph width = 0
		ifTrue: [paragraph compositionWidth: self defaultCompositionWidth].
	bounds := aPoint extent: 0 @ 0.
	self
		recomputePreferredBounds;
		setIndices: nil
! !

!TextFigure methodsFor:'private'!

addJustificationMenuTo: aMenuBuilder
	aMenuBuilder beginSubMenuLabeled: 'justification'.
	#('flush left' 'centered' 'flush right' 'justified')
		with: #(#leftFlush #centered #rightFlush #justified)
		do: [:label :symbol | aMenuBuilder add: label -> [self alignUsing: symbol]].
	aMenuBuilder endSubMenu
!

addStyleMenuTo: aMenuBuilder
	aMenuBuilder beginSubMenuLabeled: 'style'.
	#(#bold #italic #underline #strikeout #default #large #small #fixed)
		do: [:each | aMenuBuilder add: each -> [self styleUsing: each]].
	aMenuBuilder endSubMenu
! !

!TextFigure methodsFor:'transforming'!

moveMarginTo: aPoint
	| newMargin |
	newMargin := (aPoint - self origin) x.
	newMargin <= 0 ifTrue: [^self].
	newMargin = self extent x ifTrue: [^self].
	self margin: newMargin
!

recompose
	"Recompose my paragraph, resetting the width and possibly the right margin."

	self margin: paragraph composeAll
!

textStyle: aTextStyle
	aTextStyle alignment = paragraph textStyle alignment
		ifTrue: [paragraph textStyle: aTextStyle]
		ifFalse:
			[paragraph textStyle: ((aTextStyle copy)
						alignment: paragraph textStyle alignment;
						yourself)]
! !

!TextFigure class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/hotdraw/HotDraw__TextFigure.st,v 1.1 2006/03/13 16:52:58 cg Exp $'
! !
