"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Number subclass:#AlgebraicNumber
	instanceVariableNames:'polynomial interval'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

AlgebraicNumber comment:'An algebraic number is stored, in this implementation, as a  square free polynomial and a ClosedInterval wich contains an only root of the polynoial ( to determine the number of roots I use Sturn). For example: Raiz de 2 aproximationError: 1e-10
For another examples see sqrtOf: anInteger. For examples of some methods, see the category examples.
'
!


!AlgebraicNumber class methodsFor:'instance creation'!

fromNumber: aNumber
	" answer a new AlgebraicNumber that can represent aNumber"
	^ self
		polynomial: (RaisedVariable name: #x deg: 1) - aNumber
		interval: (ClosedInterval left: aNumber-1 right: aNumber+1)
!

polynomial: aPolynomial interval: anInterval
	"Private - creates a new Algebraic number with the interval anInterval and the polynomial apolynomial"
	^ self new polynomial: aPolynomial ; interval: anInterval
!

root: anInteger of: anotherInteger
	"Public - creates a new Algebraic Number which is the anIntegerth root of anotherInteger. For example:
root: 3 of: 5"
	anInteger even ifTrue: [
		anotherInteger > 0 ifFalse: [ ^ self error: 'no existe la raiz'].
		^self polynomial: (RaisedVariable name: #x deg: anInteger) - anotherInteger
		interval: (ClosedInterval left: 1 right: anotherInteger)].
	^self polynomial: (RaisedVariable name: #x deg: anInteger) - anotherInteger
		interval: ((anotherInteger >= 0) ifTrue: [ClosedInterval left: 1 right: anotherInteger]
			ifFalse: [ClosedInterval left: anotherInteger right: -1])
!

sqrt2
	" Public - gives the algebraic number sqrt of 2 , which is a particular case of the sqrtOf: method "
	^ self
		polynomial: (RaisedVariable name: #x deg: 2) - 2
		interval: (ClosedInterval left: 1 right: 2)
!

sqrtOf: anInteger
	" Public - gives a new AlgebraicNumber which is the square root of aNumber. For example: 
 sqrtOf: 5"
	"anInteger sqrt isInteger ifTrue: [^ anInteger sqrt]."
	^ self
		polynomial: (RaisedVariable name: #x deg: 2) - anInteger
		interval: (ClosedInterval left: 1 right: anInteger)
! !

!AlgebraicNumber class methodsFor:'examples'!

* anAlgebraicNumber
	"For more examples, you can play in the Morphic Word. Raiz de 2 * Raiz cubica de 5   = " 
!

+ anAlgebraicNumber
	"For more examples, you can play in the Morphic Word. Raiz de 2 + Raiz cubica de 5 = " 
!

aproximationError: aNumber
	"For more examples, you can play in the Morphic Word. Raiz de 2 aproximationError: 0.000001 "
!

reciprocal
        "For more examples, you can play in the Morphic Word.  Raiz de 2  reciprocal "

        AlgebraicNumber sqrt2.

        AlgebraicNumber sqrt2 reciprocal.
! !

!AlgebraicNumber methodsFor:'accessing'!

interval
	"Public - answer the interval who includes me as the only root of the polynomial. For example:  Raiz de 2  interval "
	^ interval
!

polynomial
	"Public - answer the polynomial who anules me. For example:  Raiz de 2  polynomial "
	^ polynomial
! !

!AlgebraicNumber methodsFor:'arithmetic'!

* anObject
		"Public - gives a new algebraic number which is the product of myself and anAlgebraicNumber. For examples see the category examples in class"
	| x y poly inter |
	anObject isNumber 
		ifTrue: [anObject isNull
			ifTrue: [0]
			ifFalse: [
				anObject isAlgebraic
					ifTrue:[
						x _ RaisedVariable name: #x.
						y _ RaisedVariable name: #y.
						poly _ ((self polynomialAt: x/y)*(y raisedToInteger: self polynomial deg) resultant: (anObject polynomialAt: y) in: y) squareFree.
						[inter _ ClosedInterval
							center: ( self interval center * anObject interval center)
							delta: (self interval right abs * anObject aproximationError + (anObject interval left abs * self aproximationError)).
						(poly rootsIn: inter) = 1] whileFalse: [
							self refine.
							anObject refine].
						^ self class polynomial: poly * poly denominator interval: inter]]].
	^ (anObject adaptAlgebraicNumber: self) * (anObject adaptToAlgebraicNumber)
!

+ anObject
        "Public - gives a new algebraic number which is the sum of myself and anAlgebraicNumber. For examples see the category examples in class."
        | x y poly inter |
        anObject isNumber 
                ifTrue: [anObject isAlgebraic
                        ifTrue:[
                                x _ RaisedVariable name: #x.
                                y _ RaisedVariable name: #y.
                                poly _ ((self polynomialAt: x-y) resultant: (anObject polynomialAt: y) in: y) squareFree.
                                [inter _ ClosedInterval new
                                        left: self left+anObject left;
                                        right: self right+anObject right.
                                        (poly rootsIn: inter) = 1] whileFalse: [
                                                self refine.
                                                anObject refine].
                                ^ self class polynomial: poly interval: inter]
                        ifFalse: [
                                ^ self class new
                                        polynomial: (self polynomialAt: self polynomial variable asMonomial - anObject);
                                        interval: self interval + anObject]]
                ifFalse: [^ (anObject adaptAlgebraicNumber: self) + (anObject adaptToAlgebraicNumber)]

    "Modified: / 04-10-2006 / 15:34:05 / cg"
!

- anAlgebraicNumber
	"Public - gives a new algebraic number which is the diference between myself and anAlgebraicNumber"
	^ self + (anAlgebraicNumber negated)
!

/ anAlgebraicNumber
	"Public - gives a new algebraic number which is the division of myself and anAlgebraicNumber"
	^ self * (anAlgebraicNumber reciprocal)	
!

negated
	"Public - gives my aditive inverse. For example:  Raiz de 2  negated"
	^ self class new polynomial: (self polynomial valueAt: self polynomial variable * -1) asSimplePolynomial;
	 interval: self interval negated
!

reciprocal
	"Public - gives my inverse as algebraic number. For examples see the category examples in class."
	| left right |
	(self isNull) ifTrue: [ ^ self error: 'el cero no tiene inverso'].
	self interval left isNull
		ifTrue: [ left _ -1]
		ifFalse: [ left _ self interval left].
	self interval right isNull
		ifTrue: [right _ 1]
		ifFalse: [ right _ self interval right].
	^ self class
		polynomial: (self polynomial reverse)
		interval: (ClosedInterval left: right reciprocal right: left reciprocal).
! !

!AlgebraicNumber methodsFor:'comparing'!

< anObject
        "Public - determines if I'm < anObject. For example:  Raiz de 2  <  Raiz de 3 "
        | value |
        anObject isNumber 
                ifTrue: [anObject isAlgebraic
                        ifTrue:[ "Algebraic Number"
                                ^ (self interval intersects: anObject interval)
                                        ifFalse: ["Disjoint intervals" 
                                                self right < anObject left]
                                        ifTrue: ["Intersecting intervals"
                                                self refine.
                                                anObject refine.
                                                self < anObject]]
                        ifFalse: ["non Algebraic Number"
                                ^ (self right < anObject) or: [(anObject > self left) and: [
                                                ((value _ self polynomialAt: anObject) isNull not) and: [
                                                        value sign = self polynomialAtRight sign]]]]]
                ifFalse: [^ (anObject adaptAlgebraicNumber: self) < (anObject adaptToAlgebraicNumber)]

    "Modified: / 04-10-2006 / 15:34:42 / cg"
!

<= aNumber
	"Public - determines if I'm <= aNumber "
	^ self < aNumber or: [self = aNumber]
!

= anAlgebraicNumber

	"Public - determines if i'm equal to anAlgebraicNumber. For example:  Raiz de 2   =   Raiz de 3 "
	| temp |
	^ anAlgebraicNumber isAlgebraic
		ifTrue:[
			temp _ self polynomial gcd: (anAlgebraicNumber polynomial).
			((Sturn new polynomial: temp) sturnIn: (anAlgebraicNumber interval)) = 1]
		ifFalse: [(anAlgebraicNumber adaptAlgebraicNumber: self) = (anAlgebraicNumber adaptToAlgebraicNumber)]
!

> aNumber
	"Public - determines if I'm > anumber "
	^ (self <= aNumber) not
!

>= aNumber
	"Public - answer if I'm >= aNumber "
	^ (self < aNumber) not
!

~= aNumber
	"Public - says if I'm not aNumber "
	^ (self = aNumber) not
! !

!AlgebraicNumber methodsFor:'converting'!

adaptInteger: anInteger
	"Public - converts an integer so it can be added with an algebraic number "
	^ self class fromNumber: anInteger
!

adaptToInteger
	" Private  adapts me so I can make arithmethic operations with an integer"
	^ self
!

asAlgebraicNumber
	^ self
!

asFloat
	^ (self aproximationError: (10 raisedToInteger: ((1.0/3) asString size - 2) negated)) center asFloat
!

asFraction
	^ self center asFraction
! !

!AlgebraicNumber methodsFor:'printing'!

printOn: aStream
	"self polynomial printOn: aStream.
	aStream cr.
	self interval printOn: aStream.
	aStream cr."
	self interval center asFloat printOn: aStream.
! !

!AlgebraicNumber methodsFor:'private'!

center
	" private - answers value in middle of my interval"
	^ self interval center
!

hasRootsAtLeftOf: aNumber
	" public - answer wheather my polynomial has a root inside my interval,
	but at left of aNumber"
	^ (self polynomialAt: self left) sign*(self polynomialAt: aNumber) sign <= 0
!

hasRootsAtRightOf: aNumber
	" public - answer wheather my polynomial has a root inside my interval,
	but at right of aNumber"
	^ (self polynomialAt: self right) sign*(self polynomialAt: aNumber) sign <= 0
!

interval: anInterval
	"Private - assigns the value to the instance variable "
	interval _ anInterval
!

left
	" private - answers value nearer negative infinity in my interval"
	^ self interval left
!

left: aNumber
	" private - set the value nearer negative infinity in my interval"
	self interval left: aNumber
!

polynomial: aPolynomial
	"Private - assigns the value to the instance variable "
	polynomial _ aPolynomial
!

polynomialAt: anObject
	" private - evaluate my polynomial at my left"
	^ self polynomial valueAt: anObject
!

polynomialAtLeft
	" private - evaluate my polynomial at my left"
	^ self polynomialAt: self left
!

polynomialAtMiddle
	" private - evaluate my polynomial at my left"
	^ self polynomialAt: self center
!

polynomialAtRight
	" private - evaluate my polynomial at my left"
	^ self polynomialAt: self right
!

right
	" private - answers value nearer positive infinity in my interval"
	^ self interval right
!

right: aNumber
	" private - set the value nearer positive infinity in my interval"
	self interval right: aNumber
! !

!AlgebraicNumber methodsFor:'refining'!

aproximationError
	" public - answer the error for printing me. For example:  Raiz de 2  aproximationError "	
	^ (self right - self left) / 2
!

aproximationError: aNumber
	"public - I will refine my interval so my printString represents
	me with an error smaller that aNumber"
	| m |
	[ self aproximationError > aNumber] whileTrue: [
		self polynomialAtLeft sign = ( m _ self polynomialAtMiddle) sign
			ifTrue: [self left: self center]
			ifFalse: [
				m isNull
					ifTrue: [
						self left: (m _ self center).
						self right: m]
					ifFalse: [self right: self center]].
		self updateMorph]
!

refine
	" public - reduce aproximationError to half. For example:  Raiz de 2  refine"
	self aproximationError: self aproximationError / 2
! !

!AlgebraicNumber methodsFor:'testing'!

isAlgebraic
	"Public - says if I'm algebraic "
	^ true
!

isNull
	"Public - answer if i'm the cero algebraic number. For example:  Raiz de 2   isNull "
	(self interval includes: 0) ifFalse: [^ false]
		 ifTrue: [ (self polynomial valueAt: 0) isNull
			ifTrue: [ ^ true]].
	^ false		
!

isNumber
	"Public - says if I'm a number "
	^ true
! !

!AlgebraicNumber class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/AlgebraicNumber.st,v 1.3 2006/10/04 13:34:47 cg Exp $'
! !
