"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#Matrix
	instanceVariableNames:'rows matrixReducer'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

!Matrix class methodsFor:'documentation'!

documentation
"
    Matrix's instances represent Matrices...       

    Matrices can be easly created with fromString: or newFrom: methods:    

        Matrix 
            newFrom:        
                ' 1 54  6 8 98        
                  3  4 5  5 6       
                  3  2 2 2 4       
                  4 2 2 54 3'       

    It accepts many representations: 
      a line is divided by Character cr or $| 
      and each line is treated as a Tuple. 
      $( $) $  and $, are skeeped in Tuple #fromString: 
      so this all mean the same: 

        Matrix 
            newFrom:     
                '1 2 3    
                 4 5 6    
                 7 8 9'   

        Matrix 
            newFrom:   
                '(1 2 3)   
                 (4 5 6)   
                 (7 8 9)'   

        Matrix 
            fromString:  
                '(1,2,3)  
                 (4,5,6)  
                 (7,8,9)'  

      and also...  

        Matrix 
            newFrom:  
                '(1(((,,)(               2,,,))))3  
                 )))),,,4))()()()(5,,,,                 6  
                )))))7))))))8(((((((,,,,,,)))))9(()))()'  

    Matrices can be added, and multiplied:  

        |a b|  

        a := Matrix fromString:  
            '(1,2,3)  
             (4,5,6)  
             (7,8,9)'. 
        b := Matrix fromString:  
            '(1,2,3)  
             (4,5,6)  
             (7,8,9)'. 
        a + b 

    and:
        |a b|

        a := Matrix fromString:  
            '(1,2,3)  
             (4,5,6)  
             (7,8,9)'. 
        b := Matrix fromString:  
            '(1,2,3)  
             (4,5,6)  
             (7,8,9)'. 
        a * b 

    and they can be reduced using a MatrixReducer: 

        (Matrix 
            fromString:
                '1 2 3 
                4 5 6 
                7 8 9' ) 
                reduced 

    and 

        (Matrix fromString:
            '1 2 3 
            4 5 6 
            7 8 90' ) det
"
! !

!Matrix class methodsFor:'instance creation'!

fromString: aString
    "public - return a new Matrix filled in with values from a string. See documentation
"

    |answer|
	answer _ self new.
    (aString findTokens: (Array with: Character cr with: $|)) do:
		[:eachLine | answer addRow: (answer rowClass fromString: eachLine)].
    ^answer.
!

identity: anInteger
	| answer |
	answer _ Matrix new.
	1 to: anInteger do: [ :each | answer addRow: ( Tuple canonic: each dim: anInteger)].
	^ answer
	
!

newFrom: anObject
	(anObject isKindOf: String) ifTrue: [^self fromString: anObject].
	^nil
! !

!Matrix class methodsFor:'defaults'!

defaultMatrixReducer
	^ MatrixReducer
! !

!Matrix methodsFor:'accessing'!

at: aPoint
    "private - returns element at row@col in the matrix"
    ^(self row: aPoint y) at: aPoint x
!

at: aPoint put: anElement
    "private - sets element at row@col in the matrix to be anElement"
    ^(self row: aPoint y) at: aPoint x put: anElement
!

colDim
    "public - return number of rows (col dimension)"
    ^self rows size
!

matrixReducer
	matrixReducer isNil ifTrue: [ self initializeMatrixReducer ].
	^ matrixReducer
!

matrixReducer: aMatrixReducer
	matrixReducer _ aMatrixReducer 
!

row: anIndex
    "private - returns the row number anIndex"
    ^self rows at: anIndex
!

row: anIndex put: aTuple
    " sets the row anIndex = aTuple"
    ^self rows at: anIndex put: aTuple
!

rowDim
    "public - return number of rows (col dimension)"
    ^self rows first size
! !

!Matrix methodsFor:'adding'!

addRow: aRow
    "private - adds a row at the bottom of the matrix"
    ^self rows add: aRow
! !

!Matrix methodsFor:'arithmetic'!

* anObject
	"public - returns a new matrix that is self * anObject"
	| newRow answer |
	anObject isMatrix ifTrue:[
	answer _ self class new.
	self rows do: [:myRow |
		newRow _ nil.
		1 to: self rowDim do: [:each | 
			newRow _ newRow
				ifNil: [(anObject row: each) * (myRow at: each)]
				ifNotNil: [newRow + ((anObject row: each) * (myRow at: each))]].
		answer addRow: newRow].
	^ answer].
	anObject isNumber ifTrue: [^ self collect: [:each | each * anObject]]
!

+ aMatrix
    "public - returns a new matrix that is self + aMatrix"
    |answer|
    answer := self class new.
    1 to: self colDim do: [:i |
        answer addRow: (self row: i) + (aMatrix row: i)].
    ^answer
!

- aMatrix
    "public - returns a new matrix that is self - aMatrix"
    ^ self + aMatrix negated
!

negated
    "public - returns a new matrix that is -self"
    |answer|
    answer _ self class new.
    self rows do: [:aRow |
        answer addRow: aRow negated].
    ^answer
!

squared
	" public - answer myself squared"
	^ self * self
! !

!Matrix methodsFor:'comparing'!

= aMatrix
	1 to: self colDim do: [:eachIndex |
		(self row: eachIndex) = (aMatrix row: eachIndex) ifFalse: [^false]].
	^true
!

isMatrix
	^ true
! !

!Matrix methodsFor:'converting'!

asGraph
	" public - answer a Graph whos Matrix representation is me"
	| answer |
	answer _ Graph new.
	1 to: self colDim do: [:eachRow |
		1 to: self rowDim do: [:eachCol |
			(self at: eachCol @ eachRow) = 0 ifFalse: [
				answer addArrowFrom: eachCol to: eachRow weighted: (self at: eachCol @ eachRow)]]].
	^ answer
! !

!Matrix methodsFor:'copying'!

copy
^ self class new rows: self rows copy
! !

!Matrix methodsFor:'enumerating'!

collect: aBlock
	| answer |
	answer _ self class new.
	self rows do: [:eachRow | answer addRow: (eachRow collect: aBlock)].
	^ answer
! !

!Matrix methodsFor:'initialization'!

initializeMatrixReducer
	self matrixReducer: self class defaultMatrixReducer new
!

initializeRows
	"private - sets the rows to be an empty OrderedCollection"
	^ self rows: OrderedCollection new
!

rowClass
	" answer the proper class for my rows"
	^ Tuple
! !

!Matrix methodsFor:'matrix functions'!

det
	^ (self matrixReducer matrix: self) det
!

nullRows
	" public - answer a collection of the indexes of my null rows"
	^ (1 to: self rowDim) select: [:each | (self row: each) isNull]
!

permuteRow: anInteger withRow: bnInteger
	| aux |
	aux _ self row: anInteger.
	self row: anInteger put: (self row: bnInteger).
	self row: bnInteger put: aux.
!

reduced
	^ (self matrixReducer matrix: self) reducedMatrix
!

row: anIndex add: aTuple
    " friend - adds to row anIndex the row aTuple"
    ^self rows at: anIndex put: (aTuple + (self row: anIndex)).
!

transposed
	" public - answer my transposed matrix"
	| answer newRow |
	answer _ self class new.
	1 to: self rowDim do: [:eachCol |
		newRow _ self rowClass new.
		self rows do: [:each |
			newRow add: (each at: eachCol)].
		answer addRow: newRow].
	^ answer
! !

!Matrix methodsFor:'printing'!

printOn: aStream
    "public - returns the representation of self as a String"
	1 to: self colDim do: [:eachRow |    
        (self row: eachRow) printOn: aStream.
        aStream cr].
! !

!Matrix methodsFor:'private'!

rows
    "private - returns the value of the internal variable rows"
	rows isNil ifTrue: [self initializeRows].
    ^rows
!

rows: aValue
    "private - assigns the value of the internal variable rows
    and returns self"
    rows := aValue.
    ^self
! !

!Matrix class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Matrix.st,v 1.1 2002/11/22 19:48:10 cg Exp $'
! !
