"{ Package: 'stx:goodies/math/algebraicNumbers' }"

"{ NameSpace: Math }"

Object subclass:#Polynomial
	instanceVariableNames:'monomials'
	classVariableNames:''
	poolDictionaries:''
	category:'Math-SqMath-Algebra'
!

Polynomial comment:'A polynomial is stored as a sorted collection of monomials with the lexicographic order. There are a lot of different ways of creating a polynomial. At first you have to define the variables which will be included in the polynomial. For example:
|x y|
x _ RaisedVariable name: #x .
y _ RaisedVariable name: #y.
x*x*y + (2*x) + y

Once the Polynomial is defined, there are a lot of things one would like to ask him (variables,  degIn: x, degIn: y, derivativeIn: x, derivativeIn: y, head, lc, value: x at: 3, isNull. You can play with a lot of other examples and functions between two polynomials in Morphic Word.

| x y |
x _ Monomial variable: ''x''.
y _ Monomial variable: ''y''. 
(y*y*y*x*11) + (y*y*2) + (x*x*x*3) derivativeIn: x


|x|
x _ Variable name: #x .
(x*x + (2*x) + 1) gcd: ( x*x - 1)

|x y|
x _ Variable name: #x .
y _ Variable name: #y.
(x*x*y + (2*y) + (3*y*x)) gcd: (x*y + y) in: x


|x|
x _ Variable name: #x .
(3*x*x + (2*x) + 6) monic


Polynomial new nullMonomial

|x|
x _ RaisedVariable name: #x.
(x*x + (2*x) + 2) squared resultant: ((x*x + (2*x) + 2) squared derivative) in: x

|x y|
x _ Variable name: #x .
y _ Variable name: #y.
(x*x*y + (2*x) + y) reversedIn: x
'
!


!Polynomial class methodsFor:'instance creation'!

fromMonicMonomial: aMonicMonomial
	^ Polynomial new add: aMonicMonomial asMonomial
!

fromMonomial: aMonomial
	^ Polynomial new add: aMonomial
!

fromNumber: aNumber
	" public - answer a Polynomial that represents a Number"
	^ self new add: aNumber asMonomial
!

with: aMonomial
	" public - answer a Polynomial with a monomial"
	^ self new add: aMonomial
!

with: aMonomial with: anotherMonomial
	" public - answer a Polynomial with two monomials"
	^ self new add: aMonomial; add: anotherMonomial
! !

!Polynomial class methodsFor:'defaults'!

defaultEuclideanFunction
	^ [:a | a isNumber ifTrue: [0] ifFalse: [a deg]]
!

defaultSortBlock
	^ [:a :b | b >= a]
! !

!Polynomial class methodsFor:'examples'!

example1
"
        |x y|
        x _ RaisedVariable name: #x .
        y _ RaisedVariable name: #y.
        ^ x*x*y + (2*x) + y
"
!

example2
	|x y|
	x _ RaisedVariable name: #x .
	y _ RaisedVariable name: #y.
	^ x + (2*y)
!

example3
	|x|
	x _ RaisedVariable name: #x .
	^ x + 1
! !

!Polynomial methodsFor:'accessing'!

add: aMonomial
	" add myself a Monomial "
	aMonomial coef isNull not ifTrue: [self monomials add: aMonomial]
!

coefAt: aMonicMonomial 
	" answers the coeficient which multiplies aMonicMonomial in self, or null if 
	aMonicMonomial is not present"
	^ (self monomialAt: aMonicMonomial ifNone: [self nullMonomial]) coef
!

coefAt: aMonomial put: aCoeficient
	"Private - sets the coeficient which multiplies aMonomial "
	self remove: aMonomial.
	aCoeficient isNull ifFalse: [self add: (aMonomial copy coef: aCoeficient)]
!

remove: aMonicMonomial 
	" public - removes Monomial that matches aMonicMonomial"
	self monomials remove: (self monomialAt: aMonicMonomial ifNone: [nil]) ifAbsent: [nil]
!

variable
	" public - answer my first variable. if I'm a one variable polynomial, it's my only variable"
	^ self variables isEmpty
		ifTrue: [RaisedVariable new]
		ifFalse: [self variables last copy deg: 1]
!

variableWith: aPolynomial
	" public - answer my variable. If it's none, answer aPolynomial's variable"
	^ self variables isEmpty
		ifTrue: [aPolynomial variable]
		ifFalse: [self variable]
!

variables
	" public - answer a SortedCollection filled up with all my variables"
	| answer |
	answer _ Set new.
	self monomialsDo: [:eachMon |
		answer addAll: (eachMon variables collect: [:eachVar |
			eachVar copy deg: 1])].
	^ answer
		asSortedCollection: (answer detect: [:one | one isNil not] ifNone: [RaisedVariable new]) class sortBlock
! !

!Polynomial methodsFor:'arithmetic'!

* anObject
	"Public - answer the result of multiplying me with anObject. For example:
	(y + 2*x + y*x^2) * (2*y + x) =
	For other examples you can play in Morphic Word"
	| answer |
	^ anObject isPolynomial
		ifTrue: [
			answer _ self class new.
			self monomialsDo: [:eachMyMon |
				anObject monomialsDo: [:eachHisMon |
					answer sum: eachMyMon * eachHisMon]].
			answer]
		ifFalse: [(anObject adaptPolynomial: self)*(anObject adaptToPolynomial)]
!

+ anObject
	" public - answer the result of adding anObject to me.
	For example: (y + 2*x + y*x^2) + (2*y + x) ="
	| answer |
	self isNull ifTrue: [^ anObject].
	anObject isNull ifTrue: [^ self].
	^ anObject isPolynomial
		ifTrue: [
			answer _ self copy.
			anObject monomialsDo: [:eachMonomial | answer sum: eachMonomial].
			answer]
		ifFalse: [(anObject adaptPolynomial: self)+(anObject adaptToPolynomial)] 
!

- aPolynomial 
	"Public - gives the sustraction between me and aPolynomial"
	^ self + aPolynomial negated
!

// aPolynomial
	" public - answer a Polynomial that represents the quotient of myself /
	 aPolynomial. (it will work only in an euclidean domain).
	For example: (y + 2*x + y*x^2) // (x + 1) = "
	^ (self dividedBy: aPolynomial in: (self variableWith: aPolynomial)) at: 1
!

\\ aPolynomial 
	" public - answer a Polynomial that represents the remainder of myself / aPolynomial. (it will work only in an euclidean domain).
	For example: (y + 2*x + y*x^2) \\ (x + 1) = "
	^ (self dividedBy: aPolynomial in: self variable) at: 2
!

dividedBy: aPolynomial in: aVariable
	" public - answer myself divided by a Polynomial expressed as polynomials in aVariable"
	| r g q factor var |
	r _ self asPolynomialIn: aVariable.
	g _ aPolynomial asPolynomial asPolynomialIn: aVariable.
	"g isMonic ifFalse: [self error: 'only divsion by monical polynomials allowed']."
	q _ Polynomial new.
	var _ aVariable copy.
	[r deg < g deg or: [r isNull]] whileFalse: [
		factor _ ((var deg: r deg - g deg) * r lc * g lc reciprocal) asPolynomialIn: aVariable.
		q _ q + factor.
		r _ r - (g*factor)].
	^ Array with: q with: r
!

negated
	"Public - gives my aditive inverse "
	^ self collect: [:eachMonomial | eachMonomial negated]
!

privateAdd: aPolynomialOrNumber
	"Private - it's an internal way of adding two polynomials"
	| aPolynomial |
	aPolynomial _ (aPolynomialOrNumber isKindOf: Number)
				ifTrue: [Monomial fromNumber: aPolynomialOrNumber]
				ifFalse: [aPolynomialOrNumber].
	aPolynomial monomialsDo: [:eachMonomial | self sum: eachMonomial].
!

quo: anObject
	" needed for compatibility with SmallInteger //"
	^ self // anObject
!

raisedToInteger: anInteger
	"Public - gives the answer of multiplying me anInteger times"
	
|mid|

	
anInteger < 0 ifTrue: [^(self raisedToInteger: anInteger negated) reciprocal].
	
anInteger = 0 ifTrue: [^1].
	
anInteger = 1 ifTrue: [^self].
	
mid _ anInteger // 2.
	
^ (self raisedToInteger: (anInteger - mid)) * (self raisedToInteger: mid)
!

squared
	"Public - answer my squared"
	^ self * self
! !

!Polynomial methodsFor:'converting'!

, anObject
	"Public - answer a Tuple with myself and anObject"
	^ (Tuple new add: self ; add: anObject)
!

adaptFraction: aFraction
	^ aFraction asPolynomial
!

adaptInteger: anInteger
	^ anInteger asPolynomial
!

adaptMonicMonomial: aMonicMonomial
	^ aMonicMonomial asPolynomial
!

adaptMonomial: aMonomial
	^ self class new add: aMonomial 
!

adaptToFraction
	^ self
!

adaptToInteger
	^ self
!

adaptToMonicMonomial
	^ self
!

adaptToMonomial
	^ self
!

adaptToVariable
	^ self
!

adaptVariable: aVariable
	^ aVariable asPolynomial
!

asPolynomial
	^ self
!

asPolynomialIn: aVariable 
	" public - answer my degree"
	^ self asSimplePolynomial asPolynomial collect: [:eachMon | eachMon asMonomialIn: aVariable]
!

asSimplePolynomial
	^ self collect: [:eachMon | eachMon asSimplePolynomial]
!

asTuple
	" public - answer my coeficints as a Tuple.
	Usefull for filling up a Matrix with my coeficients"
	| answer var |
	answer _ (Smalltalk at: #Tuple) new.
	var _ self variable copy.
	0 to: self deg do: [:each |
		answer add: (self coefAt: (var deg: each) asMonicMonomial) asSimplePolynomial].
	^ answer
!

asTupleIn: aVariable
	" public - answer my coeficints as a Tuple.
	Usefull for filling up a Matrix with my coeficients"
	^ (self asPolynomialIn: aVariable) asTuple
! !

!Polynomial methodsFor:'copying'!

copy
	"Private - creates a copy of myself"
	^ self class new monomials: self monomials copy
! !

!Polynomial methodsFor:'enumerating'!

collect: aBlock 
	" public - creates a new polynomial after collecting for monomials"
	| answer |
	answer _ 0.
	self monomialsDo: [:eachMon | answer _ answer + (aBlock value: eachMon)].
	^ answer
!

monomialsDo: aBlock
	"Public - evaluates aBlock in each monomial "
	self monomials reversed do: aBlock
! !

!Polynomial methodsFor:'initialization'!

initializeMonomials
        "Private - initialize the instance variable"
        self monomials: (SortedCollection sortBlock: Monomial sortBlock)
! !

!Polynomial methodsFor:'polynomial functions'!

deg
	" public - answer my degree. See ? for examples"
	^ self head deg
!

degIn: aVariable 
	" public - answer my degree in aVariable. For examples see ?"
	^ (self asPolynomialIn: aVariable) deg
!

denominator
	"Public - gives the lcm for the denominators of the polinomyal if it's in a single variable"
	| answer |
	answer _ 1.
	self asSimplePolynomial monomialsDo: [ :each |
		each coef isFraction ifTrue: [
			answer _ answer lcm: each coef denominator]].
	^ answer
!

derivative
	" public - answer my derivative in my first variable. For examples see the ?"
	^ self derivativeIn: self variable
!

derivativeIn: aVariable 
	" public - answer my derivative computed at aVariable. For example: 
	 y^3*x*11 + y^2*2 + x^3*3 derivativeIn: x	"
	^ self collect: [:eachMon | eachMon derivativeIn: aVariable]
!

gcd: aPolynomial 
	" Public - answer the gcd betwen myself and a polynomial , both expressed in aVariable. For example:  (1 + x + x^2) gcd: ( 1 - x^2). See also  gcd: in: "
	^ (self gcd: aPolynomial in: self variable)
!

gcd: aPolynomial in: aVariable

	" Public - answer the gcd betwen myself and a polynomial , both expressed in aVariable. For example:  (x^3*y + 2*y + 3*y*x) gcd: (x*y + y) in: x"
	|r1 r2 aux|
	
r1 _ self asPolynomialIn: aVariable.
	r2 _ aPolynomial asPolynomialIn: aVariable.
	[r2 deg = 0] whileFalse: [
		aux _ r1 \\ r2.
		r1 _ r2.
		r2 _ aux].
	^ r2 isNull
		ifTrue: [r1]
		ifFalse: [1]
!

head
	" public - answer my greater monomial. For an example see ?"
	^ self monomials isEmpty
		ifTrue: [self nullMonomial]
		ifFalse: [self monomials last]
!

lc
	" public - answer my leading coeficient. For an example see ?"
	^ self head coef asSimplePolynomial
!

monic
	" public - answer a monic polynomial that is my multiple. For example: (6 + 2*x + 3*x^2) monic"
	^ (self * self lc reciprocal) asSimplePolynomial
!

nullMonomial
	"Public - answer a monomial that represents the correct monoial for zero
	that is with the coeficient of the apropiate class. null "
	^ 0 asMonomial
	
!

resultant: aPolynomial in: aVariable
	"Public - gives a new polynomial which is the resultant between me and aPolynomial in the variable aVariable . For example:
|x|
x _ RaisedVariable name: #x.
(x*x + (2*x) + 2) squared resultant: ((x*x + (2*x) + 2) squared derivative) in: x ="
	| pol1 pol2 matrix size1 size2 |
	pol1 _ (self asTupleIn: aVariable) reverse.
	pol2 _ (aPolynomial asTupleIn: aVariable) reverse.
	size1 _ (pol1 size - 1).
	size2 _ (pol2 size - 1).
	(size2 - 1) timesRepeat: [pol1 add: 0].
	(size1 - 1) timesRepeat: [pol2 add: 0].
	matrix _ Matrix new.
	size2 timesRepeat: [matrix addRow: pol1.
							pol1 _ pol1 shift].
	size1 timesRepeat: [matrix addRow: pol2.
							pol2 _ pol2 shift].
	^ (EuclideanMatrixReducer new matrix: matrix euclideanFunction: self class defaultEuclideanFunction) det
!

reverse
	" public - answer a Polynomial with coeficient reversed.
	Usefull for computing the minimal polynomial of an algebraic number
	given it's reciprocal's minimal>For examples see reversedIn:"
	^ self reversedIn: self variable
!

reversedIn: aVariable
	" public - answer a Polynomial with coeficient reversed.
	Usefull for computing the minimal polynomial of an algebraic number
	given it's reciprocal's minimal. For example:  (2*x + y + y*x^2) reversedIn: x"
	| deg |
	deg _ self degIn: aVariable.
	^ (self asPolynomialIn: aVariable) collect: [:eachMon |
		eachMon copy at: aVariable put: deg - eachMon deg]	
!

rootsFrom: aNumber to: anotherNumber

	"Public - gives the number of roots between aNumber an anotherNumber using the Sturn algorithm. For example see:  rootsIn:"
	^(Sturn new polynomial: self) sturnIn: (ClosedInterval left: aNumber right: anotherNumber)
!

rootsIn: anInterval
	"Public - gives the number of roots between aNumber an anotherNumber using the Sturn algorithm. For examples see  sturn"
	^ anInterval length = 0
		ifTrue: [(self valueAt: anInterval left) = 0 ifTrue: [1] ifFalse: [0]]
		ifFalse: [(Sturn new polynomial: self) sturnIn: anInterval]
!

squareFree
	" public - answer a monic polynomial with the same roots as me, but with order 1
	 (x+1)*(x+1) squareFree
	"
	| r1 r2 |
	r1 _ self monic.
	r2 _ r1 derivative.
	^ (r1 // (r1 gcd: r2)).
!

value: aVariable at: anObject
	" public - evaluates myself when aVariable is anObject. For examples see ?"
	^ self collect: [:eachMon | eachMon value: aVariable at: anObject]
!

valueAt: anObject
	"Public - evaluates my first variable at anObject. For examples see ? "
	^ self value: self variable at: anObject
! !

!Polynomial methodsFor:'printing'!

printOn: aStream 
	| first |
	self isNull
		ifTrue: [0 printOn: aStream]
		ifFalse: [
			first _ true.
			aStream nextPut: $(.
			self
				monomialsDo: 
					[:eachMonomial | 
					first
						ifTrue: [first _ false]
						ifFalse: [aStream nextPutAll: ' + '].
					eachMonomial printOn: aStream].
			aStream nextPut: $)]
! !

!Polynomial methodsFor:'private'!

monomialAt: aMonomial ifNone: exceptionBlock 
	" private - answers the monomial whos variables matches aMonicMonomial's
	if none value axceptionBlock"
	| index |
	index _ self monomials findFirst: [:eachMon |
		aMonomial variables = eachMon variables].
	^ index ~= 0
		ifTrue: [ self monomials at: index]
		ifFalse: [ exceptionBlock value]
!

monomials
	monomials ifNil: [self initializeMonomials].
	^ monomials
!

monomials: aCollectionOfMonomials
	monomials _ aCollectionOfMonomials
!

sum: aMonomial 
	| c |
	c _ (self coefAt: aMonomial)
				+ aMonomial coef.
	self coefAt: aMonomial put: c
! !

!Polynomial methodsFor:'testing'!

= aPolynomial
	"Public - determines if I'm equal to aPolynomial "
	^ (self - aPolynomial) asSimplePolynomial isNull
!

isMonic
	"Public - answer if I'm monis "
	^ self head isMonic
!

isNull
	"Public - answer if I'm the null polynomial "
	self monomialsDo: [:eachMonomial | eachMonomial isNull ifFalse: [^ false]].
	^ true
!

isPolynomial
	"Public - answer if I'm a polynomial "
	^ true
!

negative
	^ false
! !

!Polynomial class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/algebraicNumbers/Polynomial.st,v 1.1 2002/11/22 19:48:32 cg Exp $'
! !
