"{ Package: 'stx:goodies/math/fibonacci' }"

"{ NameSpace: Math }"

Object subclass:#FibonacciCalculator
	instanceVariableNames:'cache m lambda muh'
	classVariableNames:'Cache'
	poolDictionaries:''
	category:'Math-Algorithms'
!

!FibonacciCalculator class methodsFor:'documentation'!

documentation
"
    My instances know how to compute the Fibonacci Sequence. 
    I'm a the standard recursive but slower FibonacciCalculator, but some of my subclasses are very inteligent.

    See coments in my #lambdaMuh, in my subclasses comments (to see something very interesting), 
    and in my subclasses' #positiveF: method. 
    Heres a table of the running time of the diferent implementations in my P233 and my 486DX-2 80MHz

    Time millisecondsToRun: [Split2FibonacciCalculator new emptyClassCache f: 15000] 10014 74937
    Time millisecondsToRun: [Split1FibonacciCalculator new emptyClassCache f: 15000] 11246 84189 
    Time millisecondsToRun: [SplittingFibonacciCalculator new emptyClassCache f: 15000] 11546 85580
    Time millisecondsToRun: [BinaryFibonacciCalculator new emptyClassCache f: 15000] 12738 94873
    Time millisecondsToRun: [MatrixFibonacciCalculator new emptyClassCache f: 15000] 20409 151326

    st/x P233 timings:
    Time millisecondsToRun: [Split2FibonacciCalculator new emptyClassCache f: 15000] 90
    Time millisecondsToRun: [Split1FibonacciCalculator new emptyClassCache f: 15000] 145
    Time millisecondsToRun: [SplittingFibonacciCalculator new emptyClassCache f: 15000] 153
    Time millisecondsToRun: [BinaryFibonacciCalculator new emptyClassCache f: 15000] 99
    Time millisecondsToRun: [MatrixFibonacciCalculator new emptyClassCache f: 15000] 155
    Time millisecondsToRun: [15000 fib]   57 (!!!!!!)

    [comments:]
        cg: those numbers are for Squeak...
            ... st/x is not too bad :-; compare above timings with:
            Time millisecondsToRun: [15000 fib]   
"
! !

!FibonacciCalculator class methodsFor:'initialization'!

initializeCache
	" private - Called to initialize Cache class variable.
	this Cache is a collection of precalculated Fibonachi numbers,
	but are cached only power of two and powers of two minus 1 Fibonachi numbers"
	Cache _	Dictionary new.
	self cacheAdd: 0@0.
	self cacheAdd: 1@1.
! !

!FibonacciCalculator class methodsFor:'access'!

cache
	Cache ifNil: [self initializeCache].
	^ Cache
!

cacheAdd: aPoint
	self cache at: aPoint x put: aPoint y.
! !

!FibonacciCalculator class methodsFor:'maintainment'!

emptyCache
	" public - empty the cache of Fibonachi Numbers"
	self initializeCache.
! !

!FibonacciCalculator methodsFor:'accessing'!

f: anIndex 
	" public - Answer the value of the anIndex term of the Fibonacci  
	sequence"
	| answer |
	 
	answer _ self normalize: (anIndex negative
		ifTrue: [ anIndex even
			ifTrue: [(self f: anIndex negated) negated]
			ifFalse: [self f: anIndex negated]]
		ifFalse: [ (self knows: anIndex)
			ifTrue: [self cache at: anIndex]
			ifFalse: [self positiveF: anIndex]]).
	^ answer.
!

lambda

	" public - reffer to coments (within source) of lambdaMuh"
	lambda ifNil: [self lambdaMuh].
	^ lambda.
!

m
	^ m
!

m: anIntegerOrNil
	m _ anIntegerOrNil.
	self initializeCache
!

muh
	" public - reffer to coments (within source) of lambdaMuh"
	muh ifNil: [self lambdaMuh].
	^ muh
! !

!FibonacciCalculator methodsFor:'cache'!

cache
	cache ifNil: [self initializeCache].
	^ cache
!

cache: aDictionary
	^ cache _ aDictionary
!

cacheAdd: aPoint
	self cache at: aPoint x put: aPoint y.
!

knows: anIndex 
	" answers whether anIndex is cached or not.
	If the class' cache knows it, get it from there and adapt the modulus
	befor adding to my own cache"
	(self cache includesKey: anIndex)
		ifTrue: [^ true].
	(self class cache includesKey: anIndex)
		ifTrue: 
			[self cacheAdd: anIndex @ (self normalize: (self class cache at: anIndex)).
			^ true].
	^ false
! !

!FibonacciCalculator methodsFor:'computing'!

lambdaMuh
	" private - calculates lambda and muh, calculating both at once can be done faster. Where
	lambda is the period of Fibonacci's sequence modulus m, and
	muh is the offset where the sequence starts to be periodic.

	It uses the fact that given a sequence (Xn) in a set of m different values defined with 
	Xn+1=f(Xn-1,Xn) if there exist d such that Xn=Xn+d and Xn+1=Xn+d+1 then Xn+d+i=Xn+i for 
	any i >= 0. 
	 
	From a set of m elements can be taken only m^2 different pairs, so for any sequence 
	deffined as (Xn) there exists such d, and it's called lambda. And the smallest n such that 
	Xn+d=Xn is called muh.

	This message is here to be sent to instances of my class that calculates Fibonacci
	sequence modulus some m. (look at #m: and #f:)

	This algorithm is adapted from one in Knuth's Seminumerical Algorithms,
	Exercise 3.1.6 (pg. 7) and it's solution (pg. 517).
	The difference is that here Xn+1=f(Xn-1,Xn) and there it's Xn+1=f(Xn)"

	| n xn1 xn xi1 xi copy |
	n _ 1.
	copy _ self copy.
	[(xn1 _ self f: n - 1) = (copy f: 2 * n - 1) and: [(xn _ (self f: n)) = (copy f: 2 * n)]]
		whileFalse: [n _ n + 1].
	" at this point Xn=X2*n and Xn-1 = X2*n-1, so from this n the sequence will repeat.
	  xn _ Xn, and xn1 _ Xn-1"

	muh _ 0.
	lambda _ n.
	[(self f: muh) = (xi1 _ copy f: n + muh) and: [(self f: muh+1) = (xi _ copy f: n + muh+1)]]
		whileFalse: [
			(lambda > muh and: [xi = xn and: [xn1 = xi1]]) ifTrue: [lambda _ muh].
			muh _ muh + 1].

	" here muh is the smaller value such that Xmuh=Xmuh+n, this is the desired muh,
	  and lambda is the period of the sequence"
!

positiveF: anIndex 
	" public - computes Fibonachi number for index anIndex"
	^ (self f: anIndex - 1) + (self f: anIndex - 2)
! !

!FibonacciCalculator methodsFor:'initialization'!

initializeCache
	self zero: 0 one: 1.
!

initializeLambdaMuh
	muh _ lambda _ nil.
!

zero: zero one: one
	self initializeLambdaMuh.
	self m
		ifNil: [self cache: self class cache]
		ifNotNil: [self cache:
			(Dictionary new
				at: 0 put: zero;
				at: 1 put: one;
				yourself)]
! !

!FibonacciCalculator methodsFor:'maintainment'!

emptyCache
	self class emptyCache.
	self cache: nil.
!

emptyClassCache
	self class emptyCache.
! !

!FibonacciCalculator methodsFor:'private'!

normalize: aNumber
	" private - if my m (modulus) is not nil answer aNumber mod m, else answer aNumber"
	^ aNumber 
		perform: (m ifNil: [#+]
				ifNotNil: [#\\])
		with: (m ifNil: [0]
				ifNotNil: [m])
! !

!FibonacciCalculator class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/fibonacci/FibonacciCalculator.st,v 1.2 2003/06/21 10:22:05 cg Exp $'
! !
