"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearBasis
	instanceVariableNames:'matrix'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearBasis class methodsFor:'documentation'!

documentation
"
    My instances represent basis for linear spaces (of a finite dimension).
    Evaluating:

        LinearBasis canonic: 3

    I answer the canonic base for the three dimensional space of tuples.
    Also I can answer a base from a set of generators. Try this:

        LinearBasis generators: (OrderedCollection
            with: (1, -1, 0, 3)
            with: (2, 0, 8, 1)
            with: (4, -2, 8, 7))
"
! !

!LinearBasis class methodsFor:'instance creation'!

canonic: anInteger
    "Answer the canonic base of dimension anInteger."

    | answer |
    answer := self new.
    1 to: anInteger do: [:each | answer add: (Tuple e: each dim: anInteger)].
    ^answer
!

generators: aTupleCollection
    "Answer a new instance of the receiver generating
    the same linear space as aTupleCollection."

    | answer |
    answer := self new.
    aTupleCollection do: [:each |(answer generates: each) ifFalse: [answer add: each]].
    ^answer
!

new

    ^super new initialize
! !

!LinearBasis methodsFor:'ST/V methods'!

add: aTuple
    "Public - Append the vector aTuple to the receiver."

    self matrix addRow: aTuple
!

ambient
	"Answer the ambient space of the receiver."

	^self dim = 0
		ifTrue: [LinearSubspace new]
		ifFalse: [(self at: 1) ambient]
	
!

at: anInteger
    "Answer the element with index anInteger of the receiver."

    ^self matrix row: anInteger
!

changeTo: aLinearBase
    "Answer the Matrix for changing from the receiver to aLinearBase."

    | answer |
    answer := Matrix new.
    self do: [:each | answer addRow: (aLinearBase coordinatesOf: each)].
    ^answer transposed
!

coordinatesOf: aVector
    "Public - Answer the coordinates of aVector when expressed as a linear combination
    of the receiver. Answer nil if aVector is independent from the receiver."

    | m reducer |
    m := matrix copy.
    m addRow: aVector.
    reducer := MatrixReducer matrix: m transposed.
    ^reducer rank > self dim
        ifTrue: [nil]
        ifFalse: [reducer reduced lastColumn transposed]
!

copy
	"Answer a copy of the receiver."

	| answer |
	answer := self class new.
	self do: [:each | answer add: each copy].
	^answer
!

dim
    "Answer the dimension of the linear space generated by the receiver."

    ^self matrix colDim
!

do: aBlock
    "For each vector in the receiver, evaluate the one argument block aBlock
    with that vector as the argument."

    self matrix rowsDo: aBlock
!

generates: aVector
    "Answer true if aVector is dependent from the receiver. Else answer false."

    ^(self coordinatesOf: aVector) notNil
!

initialize

    self matrix: Matrix new
!

matrix
    "Answer the matrix of row vectors."

    ^matrix
!

matrix: aMatrix
    "Private - Change the matrix of row vectors."

    matrix := aMatrix
!

printOn: aStream
    "Append a String representation of the receiver on aStream."

    | first |
    first := true.
    aStream nextPutAll: '\{'.
    self matrix rowsDo: [:each |
        first
            ifTrue: [first := false]
            ifFalse: [aStream nextPutAll: '; '].
        each printOn: aStream].
    aStream nextPutAll: '\}'
        
!

union: aLinearBasis
	"Answer a LinearBasis consisting of the union of the receiver and aLinearBasis.
	This methods assumes that both basis are independent."

	| answer |
	answer := self copy.
	aLinearBasis do: [:each | answer add: each].
	^answer
! !

!LinearBasis class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearBasis.st,v 1.1 2002/11/22 19:43:48 cg Exp $'
! !
