"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearSubspace
	instanceVariableNames:'basis linearSystem'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

LinearSubspace class instanceVariableNames:'spaces'

"
 No other class instance variables are inherited by this class.
"
!

!LinearSubspace class methodsFor:'documentation'!

documentation
"
    My instances represent linear subspaces of a finite dimension.
    I can be instantiated from a set of generatior, from a linear system
    or from a basis.

    For an example try this:

        (LinearSubspace dim: 3) inspect

    Or this one:

        | s |
        s := LinearSubspace generators: (OrderedCollection
            with: (1, -1, 0, 3)
            with: (2, 0, 8, 1)
            with: (4, -2, 8, 7)).
        s linearSystem; inspect

    Or this other:
        | t |
        t := LinearSubspace linearSystem: (LinearSystem new
            add: (2, 3, 5, -1) asLinearForm asEquation;
            add: (0, 1, 17, 3) asLinearForm asEquation;
            yourself).
        t base; inspect
"
! !

!LinearSubspace class methodsFor:'instance creation'!

basis: aLinearBasis
    "Answer a new instance of the receiver with base aLinearBasis."

    ^self basis: aLinearBasis linearSystem: nil
!

basis: aLinearBasis linearSystem: aLinearSystem
    "Answer a new instance of the receiver with base aLinearBasis."

    ^self new
        basis: aLinearBasis;
        linearSystem: aLinearSystem

!

dim: anInteger
	"Answer an instance of the receiver representing
	the linear space of dimension anInteger."

	^self spaces at: anInteger ifAbsent: [self privateDim: anInteger]
!

generators: aCollection
    "Answer a new instance of the receiver with generators
    in aCollection."

    ^self basis: (LinearBasis generators: aCollection)
!

linearSystem: aLinearSystem
    "Answer a new instance of the receiver defined by aLinearSystem."

    ^self basis: nil linearSystem: aLinearSystem
!

nullDim: anInteger
	"Answer the null subspace of ambient dimension anInteger."

	| sys |
	sys := LinearSystem new.
	1 to: anInteger do: [:each | sys add: ((LinearForm e: each dim: anInteger) eq: 0)].
	^self linearSystem: sys
!

privateDim: anInteger
	"Private - Add a new instance of the receiver representing the linear space
	of dimension anInteger to the spaces dictionary. Answer the new space."

	^self spaces at: anInteger put: (self basis: (LinearBasis canonic: anInteger))
!

spaces
	"Answer a the dictionary of all instantiated linear spaces."

	^spaces ifNil: [self spaces: Dictionary new; spaces]
!

spaces: aDictionary
	"Private - Change the dictionary of all instantiated linear spaces."

	spaces := aDictionary
! !

!LinearSubspace methodsFor:'ST/V methods'!

ambient
	"Answer the ambient space of the receiver."

	^self basis ambient
!

asLinearVariety

    ^LinearVariety subspace: self translation: (Tuple nullDim: self ambient dim)
!

asSubspace

    ^self
!

basis
    "Answer a basis of the receiver."

    ^basis ifNil: [self privateBasis]
!

basis: aLinearBasis
    "Private - Change the basis of the receiver."

    basis := aLinearBasis.
!

codim
    "Answer the codimension of the receiver."

    ^self ambient dim - self dim
!

dim
    "Answer the dimension of the receiver."

    ^self basis dim
!

includes: aTupleOrSubspace
    "Answer true if aTupleOrSubspace is included in the receiver."

    | space |
    space := aTupleOrSubspace asSubspace.
    space basis do: [:each |
        self linearSystem do: [:some | (each verifies: some) ifFalse: [^false]]].
    ^true
!

intersection: aLinearSubspace
    "Answer the intersection of the receiver and aLinearSubspace."

    | equations |
    equations := LinearSystem new.
    self linearSystem do: [:each | equations add: each].
    aLinearSubspace linearSystem do: [:each | equations add: each].
    ^self class linearSystem: equations
!

linearSystem
    "Answer a linear system defining the receiver."

    ^linearSystem ifNil: [self privateLinearSystem]
!

linearSystem: aLinearSystem
    "Private - Change the linear system defining the receiver."

    linearSystem := aLinearSystem
!

orthogonal
    "Answer the orthogonal complement of the receiver."

    | oBasis |
    oBasis := LinearBasis new.
    self linearSystem do: [:each | oBasis add: each linearForm asTuple].
    ^self class new basis: oBasis
!

printOn: aStream
    "Append a String representation of the receiver on aStream."

    | first |
    first := true.
    aStream nextPut: $<.
    self basis do: [:each |
        first
            ifTrue: [first := false]
            ifFalse: [aStream nextPutAll: '; '].
        each printOn: aStream].
    aStream nextPut: $>
!

privateBasis
    "Private - Compute a LinearBasis from the linearSystem of the receiver."

    self basis: self linearSystem kernelBasis.
    ^self basis
!

privateLinearSystem
    "Private - Compute a LinearSystem from the base of the receiver."

    | dual sys |
    dual := LinearSystem new.
    self basis do: [:each | dual add: each asLinearForm asEquation].
    sys := LinearSystem new.
    dual kernel basis do: [:each | sys add: each asLinearForm asEquation].
    self linearSystem: sys.
    ^self linearSystem
!

subspace

    ^self
! !

!LinearSubspace methodsFor:'arithmetic'!

+ aLinearSubspace
    "Answer the sum of the receiver and aLinearSubspace."

    | generators |
    generators := OrderedCollection new.
    self basis do: [:each | generators add: each].
    aLinearSubspace basis do: [:each | generators add: each].
    ^self class generators: generators
! !

!LinearSubspace class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearSubspace.st,v 1.1 2002/11/22 19:44:20 cg Exp $'
! !
