"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearSystem
	instanceVariableNames:'equations solver'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearSystem class methodsFor:'documentation'!

documentation
"
    My instances represent systems of linear equations, i.e.
    $$
        \cases{
            f_1 = b_1    \cr
            f_2 = b_2    \cr
            \dots            \cr
            f_m = b_n    \cr
        }
    $$
    where each $f_h$ is a linear form and each $b_h$ is a constant value.

    For an example try this:

        | sys |
        sys := LinearSystem new
            add: ((2, 3, 5, -1) asLinearForm eq: 2);
            add: ((0, 1, 17, 3) asLinearForm eq: -7);
            yourself.
        sys particular inspect.
        sys kernel inspect
"
! !

!LinearSystem class methodsFor:'instance creation'!

new

    ^super new initialize
! !

!LinearSystem class methodsFor:'defaults'!

defaultSolverClass
    "Answer the default Class for a solver of instances
    of the receiver."

    ^LinearSystemSolver
!

whiteSpace
    "Private - Answer the white space string separating each entry
    in the #printOn: method."

    ^'    '
! !

!LinearSystem methodsFor:'ST/V methods'!

add: aLinearEquation
    "Add aLinearEquation to the receiver."

    self equations add: aLinearEquation
!

at: anInteger
    "Answer the LinearEquation with index anInteger."

    ^self equations at: anInteger
!

dim
    "Answer the dimension of the ambient space of the solutions."

    ^self equations isEmpty
        ifTrue: [0]
        ifFalse: [self equations first linearForm dim]
!

do: aBlock
    "For each vector in the receiver, evaluate the one argument block aBlock
    with that vector as the argument."

    self equations do: aBlock
!

equations
    "Private - Answer the collection of equations of the receiver."

    ^equations
!

equations: aLinearEquationCollection
    "Private - Change the collection of equations of the receiver."

    equations := aLinearEquationCollection
!

homogeneousSystem
    "Answer the homogeneous LinearSystem associated to the receiver."

    | answer |
    answer := self class new.
    self do: [:each | answer add: (each linearForm eq: 0)].
    ^answer
!

initialize
    "Initialize the receiver."

    self equations: OrderedCollection new

!

isHomogeneous
	"Answer true if the receiver is homogeneous. Else answer false."

	self do: [:each |each value = 0 ifFalse: [^false]].
	^true
!

isSolution: aTuple
	"Answer true if aTuple is a solution of the receiver."

	self do: [:each | (each isSolution: aTuple) ifFalse: [^false]].
	^true
!

kernel
    "Answer the kernel of the receiver (a LinearSubspace)."

    ^LinearSubspace basis: self kernelBasis
!

kernelBasis
    "Answer a LinearBasis for the kernel of the receiver."

    ^self solver kernelBasis
!

matrix
    "Answer the matrix of coefficients of the receiver."

    | answer |
    answer := Matrix new.
    self do: [:each | answer addRow: each asTuple].
    ^answer
!

particular
    "Answer a paritcular solution for the the receiver.
    Answer nil if the receiver is not solvable."

    ^self solver particular
!

printOn: aStream
    "Append a String representation of the receiver on aStream using TeX sintax."

    self equations size = 1 ifTrue: [
        self equations first printOn: aStream.
        ^self].
    aStream nextPutAll: '\eqalign{'; cr.
    self do: [:each |
        aStream nextPutAll: self class whiteSpace.
        each linearForm printOn: aStream.
        aStream nextPutAll: self class whiteSpace, '&=', self class whiteSpace.
        each value printOn: aStream.
        aStream nextPutAll: self class whiteSpace, '\cr'; cr].
    aStream nextPut: $}; cr
!

solver
    "Answer the solver associated to the receiver."

    ^solver ifNil: [
        self solver: self class defaultSolverClass new.
        solver]
!

solver: aLinearSystemSolver
    "Public - Change the system solver for the receiver."

    solver := aLinearSystemSolver.
    solver system: self.
! !

!LinearSystem class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearSystem.st,v 1.1 2002/11/22 19:44:04 cg Exp $'
! !
