"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearSystemSolver
	instanceVariableNames:'system reducer kernel particular'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearSystemSolver class methodsFor:'documentation'!

documentation
"
   TeX syntax

    \input defs
    \magnification=1200
    \beginsection LinearSystemSolver $\rangle\rangle$ \#documentation

    Given a linear system $Ax=b$, we reduce the matrix $A$ to obtain a new matrix $R$.
    This matrix has canonical vectors $e_i$ in the independent columns and anything in the
    dependent ones. We call $R_1, R_2, \dots$ the rows of $R$;  $C_{i_1}, C_{i_2}, \dots$
    the independent columns of $R$, and $D$ the set of indexes for the dependent columns.

    \smallskip
    \ni The homogeneous system $Ax=0$ has as many independent solutions as indexes
    has~$D$. Thus, for each $d \in D$ we construct a solution $s_d$ using the formula:
    $$
        s_d = e_d - \sum_h \langle R_h; e_d\rangle  e_{i_h}    \eqno(*)
    $$
    where $\langle R_h; e_d\rangle$ stands for the inner product between the row $R_h$
    and the canonical vector~$e_d$.

    \smallskip
    \ni To prove that $(*)$ actually gives the correct solutions we note that
    $$
        \langle e_{i_h}; R_k\rangle  = \delta_{hk}
    $$
    since $R$ is reduced and thus the submatrix of its independent columns equals the
    identetity matrix or rank $r$. Using this fact and taking the inner product of $(*)$ by
    $R_k$, we obtain:
    $$
        \langle s_d; R_k\rangle  = \langle e_d; R_k\rangle  - \langle R_k; e_d\rangle = 0.
    $$
    This ensures that all rows of $R\cdot s_d$ will vanish. So $s_d$ is a solution of the
    associated homogeneous system.

    \smallskip
    \ni The same arguments show that
    $$
        v = \sum_h b_h e_{i_h}
    $$
    is a paritcular solution for the system when it is solvable.

    \bye
"
! !

!LinearSystemSolver class methodsFor:'instance creation'!

new

    ^super new initialize
!

system: aLinearSystem
    "Answer a new instance of the receiver for solving aLinearSystem."

    ^self new system: aLinearSystem
! !

!LinearSystemSolver methodsFor:'ST/V methods'!

dim
    "Answer the dimension of the space of the solutions of the system."

    ^self system dim
!

initialize
    "Initialize the receiver."

    self reducer: MatrixReducer new
!

isSolvable
    "Answer true if the system has any solution."

    ^self isSolved
        ifTrue: [self reducer dependentColumns includes: self dim + 1]
        ifFalse: [self solve; isSolvable]
!

isSolved
    "Private - Answer true if the receiver has solved the system."

    ^self privateKernel notNil
!

kernelBasis
    "Public - Answer a LinearBasis for the kernel of the system."

    ^self isSolved
        ifTrue: [self privateKernel]
        ifFalse: [self solve; privateKernel]
!

particular
    "Public - Answer a particular solution of the system.
    Answer nil if the system is not solvable."

    ^self isSolved
        ifTrue: [self isSolvable
            ifTrue: [particular]
            ifFalse: [nil]]
        ifFalse: [self solve; particular]

!

particular: anObject
    "Change the particular solution of the system."

    particular := anObject
    
!

privateKernel
    "Private - Answer the instance variable kernel (a LinearBase)."

    ^kernel
!

privateKernel: aLinearBase
    "Private - Change the Linear Base for the kernel."

    kernel := aLinearBase
    
!

reducer
    "Private - Answer the reducer to solve the system."

    ^reducer
!

reducer: anObject
    "Change the reducer to solve the system."

    reducer := anObject
!

solve
    "Solve the system. (See the documentation of the class)"

        | ker index indepCols lastCol part |
        self isSolved ifTrue: [^self].
        ker := LinearBasis new.
        index := 0.
        indepCols := self reducer independentColumns collect: [:each | index := index + 1. index -> each].
        (self reducer dependentColumns removeLast; yourself) do: [:each |
                | canonical  solution |
                canonical := Tuple e: each dim: self dim + 1.
                solution := canonical.
                indepCols do: [:some |
                        solution := solution -
                                ((Tuple e: some value dim: self dim + 1) * ((reducer reduced row: some key) scalar: canonical))].
                ker add: (solution copyFrom: 1 to: self dim)].
        self privateKernel: ker.
        indepCols last value <= self dim ifTrue: [
        lastCol := self reducer reduced lastColumn.
                part := Tuple nullDim: self dim.
                indepCols do: [:some | part := part + ((Tuple e: some value dim: self dim) * (lastCol at: some key))].
                self particular: part]
!

system
    "Answer the system to solve."

    ^system
!

system: aLinearSystem
    "Change the system to solve."

    system := aLinearSystem.
    self reducer matrix: aLinearSystem matrix.
    self privateKernel: nil.
    self particular: nil
! !

!LinearSystemSolver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearSystemSolver.st,v 1.1 2002/11/22 19:44:09 cg Exp $'
! !
