"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearTransformation
	instanceVariableNames:'forms domain'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearTransformation class methodsFor:'documentation'!

documentation
"
    My instances represent linear transformations between
    linear spaces of finite dimension.

    For an example try this:

    | f |
    f := LinearTransformation new
        addForm: (2, 3, 5, -1) asLinearForm;
        addForm: (0, 1, 17, 3) asLinearForm;
        yourself.
        f domain inspect.
    f kernel inspect.
    f image inspect

        Or this one

        | s f |
        s := LinearSubspace generators: (OrderedCollection with: (1, 1, 1) with: (2, 0, -2)).
        f := LinearTransformation projectionOn: s.
        f inspect
"
! !

!LinearTransformation class methodsFor:'instance creation'!

fromMatrix: aMatrix
	"Answer a new instance of the receiver with the rows of aMatrix
	as the coeficients of the linear forms."

	| answer |
	answer := self new.
	aMatrix rowsDo: [:each | answer addForm: each asLinearForm].
	^answer
	
!

new

    ^super new initialize
!

projectionOn: aLinearSubspace
	"Answer a new instance of the reciever representing the projection on
	aLinearSubspace."

	| basis orthogonal matrix newMatrix |
	orthogonal := aLinearSubspace orthogonal.
	basis := aLinearSubspace basis union: orthogonal basis.
	matrix := Matrix new.
	aLinearSubspace basis do: [:each | matrix addRow: each ].
	aLinearSubspace orthogonal dim timesRepeat: [matrix addNullRow].
	^self fromMatrix: matrix transposed * ((LinearBasis canonic: basis dim) changeTo: basis)

	
! !

!LinearTransformation methodsFor:'ST/V methods'!

addForm: aLinearForm
    "Append aLinearForm as the last form of the receiver."

    self forms add: aLinearForm
!

asLinearTransformation
	"Answer the LinearTransformation defined by the receiver."

	^self
!

at: anInteger
    "Answer the linear form number anInteger."

    ^self forms at: anInteger
!

at: anInteger put: aLinearForm
    "Change the linear form number anInteger."

    self forms at: anInteger put: aLinearForm
!

codomainDim
    "Answer the dimension of the codomain of the receiver."

    ^self forms size
!

do: aBlock
    "For each linear form in the receiver evaluate the one argument
    block aBlock with that form as the argument."

    self forms do: aBlock
!

domain
	"Answer the domain of the receiver."

	^self privateDomain ifNil: [
		self forms isNil
			ifTrue: [LinearSubspace new]
			ifFalse: [self domain: self forms first domain; privateDomain]]
!

domain: aLinearSubspace
	"Change the domain of the receiver."

	domain := aLinearSubspace
!

fiberAt: vector
    "Answer the LinearVariety representing the fiber of the reciever
    at vector, i.e., the inverse image of vector by the receiver.
    Answer nil if vector is not in the image of the receiver."

    | sys coordinate |
    sys := LinearSystem new.
    coordinate := 1.
    self do: [:each |
        sys add: (each eq: (vector at: coordinate)).
        coordinate := coordinate + 1].
    ^LinearVariety fromLinearSystem: sys
!

forms
    "Private - Answer the Collection of forms of the receiver."

    ^forms
!

forms: aLinearFormCollection
    "Private - Change the Collection of forms of the receiver."

    forms := aLinearFormCollection
!

image
    "Answer image of the receiver (a LinearSubspace)."

    | generators |
    generators := OrderedCollection new: self domain dim.
    self matrix transposed rowsDo: [:each | generators add: each].
    ^LinearSubspace generators: generators
!

initialize

    self forms: OrderedCollection new
!

inverse
	"Answer the inverse of the receiver. Answer nil if the receiver
	is not an isomorphism."

	^self domain dim = self codomainDim
		ifTrue: [self leftInverse]
		ifFalse: [nil]
!

kernel
    "Answer the kernel of the receiver (a LinearSubspace)."

    | sys |
    sys := LinearSystem new.
    self do: [:each | sys add: each asEquation].
    ^sys kernel
!

leftInverse
	"Answer the left inverse of the receiver. Answer nil if the receiver
	is not a monomorphism."

	^self class fromMatrix: self matrix leftInverse
!

matrix
    "Answer the matrix in the canonical basis."

    | answer |
    answer := Matrix new.
    self do: [:each | answer addRow: each asTuple].
    ^answer
!

matrixFrom: domainBasis to: codomainBasis
	"Answer the matrix of the receiver using the given basis."

	| answer |
	answer := Matrix new.
	domainBasis do: [:each |
		answer add: (codomainBasis coordinatesOf: (self valueAt: each))].
	^answer transposed
!

printOn: aStream
	"Append a String representation of the receiver on aStream."

	| first |
	first := true.
	aStream nextPut: $(.
	self do: [:each |
		first
			ifTrue: [first := false]
			ifFalse: [	aStream nextPutAll: ', '].
		each printOn: aStream].
	aStream nextPut: $)
!

privateDomain
	"Private - Answer the domain of the receiver."

	^domain
!

restrictTo: aLinearSubspace
	"Change the domain of the receiver to aLinearSubspace."

	self domain: aLinearSubspace
!

valueAt: aTuple
    "Answer the result of evaluating the receiver at aTuple."

    | answer |
    answer := Tuple new: self codomainDim.
    1 to: self codomainDim do: [:each |
        answer at: each put: ((self at: each) valueAt: aTuple)].
    ^answer
    
! !

!LinearTransformation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearTransformation.st,v 1.1 2002/11/22 19:44:31 cg Exp $'
! !
