"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#LinearVariety
	instanceVariableNames:'translation subspace'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!LinearVariety class methodsFor:'documentation'!

documentation
"
        My instances represent linear varieties of finite dimension; i.e., translations
        of subspaces by a vector.

        LinearVarieties appear when considering (not homogeneous) LinearSystems
        or the set of all vectors in the domain of a LinearTransformation that are
        mapped to a given vector in the codomain of the transformation (which is
        called the fiber of the vector).

        For an example try this:

    | f v |
    f := LinearTransformation new
        addForm: (2, 3, 5, -1) asLinearForm;
        addForm: (0, 1, 17, 3) asLinearForm;
        yourself.
        v := f fiberAt: (8, -1).
        v inspect

        Or this one:

        | s w |
        s := LinearSubspace generators: (OrderedCollection
                with: (1, -1, 0, 3)
                with: (2, 0, 8, 1)
                with: (4, -2, 8, 7)).
        w := s asLinearVariety + (1, 1, 1, -1).
        w inspect
"
! !

!LinearVariety class methodsFor:'instance creation'!

fromLinearSystem: aLinearSystem
    "Answer an instance of the receiver representing
    the variety of solutions of aLinearSystem."

    | vector |
    ^(vector := aLinearSystem particular) isNil
        ifTrue: [nil]
        ifFalse: [self subspace: aLinearSystem kernel translation: vector]
!

subspace: aLinearSubspace translation: vector
    "Answer an instance of the receiver representing the translation
    of aLinearSubspace by vector."

    ^self new
        subspace: aLinearSubspace;
        translation: vector
! !

!LinearVariety methodsFor:'ST/V methods'!

ambient
    "Public - Answer the ambient space of the receiver."

    ^self translation ambient
!

asLinearVariety
	"Answer the LinearVariety defined by the receiver."

	^self
!

codim
    "Answer the codimension of the receiver."

    ^self subspace codim
!

dim
    "Public - Answer the dimension of the receiver."

    ^self subspace dim
!

includes: aLinearObject
    "Public - Answer true if the receiver includes aLinearObject."

	| variety |
	(aLinearObject isKindOf: Tuple) ifTrue: [^self includesTuple: aLinearObject].
	variety := aLinearObject asLinearVariety.
	^(self includesTuple: variety translation)
		and: [self subspace includes: variety subspace]
!

includesTuple: aTuple
    "Answer true if the receiver includes aTuple."

	^self linearSystem isSolution: aTuple
!

intersection: aLinearVariety
    "Answer a LinearVariety: the inteserction of the receiver and aLinearVariety."

    | sys |
    sys := LinearSystem new.
    self linearSystem do: [:each | sys add: each].
    aLinearVariety linearSystem do: [:each | sys add: each].
    ^self class fromLinearSystem: sys
!

isParallelTo: aLinearVarietyOrSubspace
    "Answer true if the receiver is parallel to aLinearVarietyOrSubspace."

    ^aLinearVarietyOrSubspace subspace includes: self subspace
!

isSubspace
	"Answer true if the receiver is a subspace."

	^self translation isNull or: [self linearSystem isHomogeneous]
!

linearSystem
    "Answer a LinearSystem defining the receiver."

    | ker answer part |
    ker := self subspace linearSystem.
    answer := LinearSystem new.
    part := self translation.
    ker do: [:each | answer add: (each linearForm eq: (each linearForm valueAt: part))].
    ^answer
!

orthogonalBy: vector
	"Answer a LinearVariety orthogonal to the receiver and passing trough vector."

	^self class subspace: self subspace orthogonal translation: vector
!

printOn: aStream
	"Append a String representantion of the receiver on aStream."

	self subspace printOn: aStream.
	self isSubspace ifFalse: [
	aStream	nextPutAll: ' + '.
	self translation printOn: aStream]
!

subspace
	"Answer the subspace parallel to the receiver."

	^subspace
!

subspace: aLinearSubspace
	"Change the subspace parallel to the receiver."

	subspace := aLinearSubspace
!

translation
    "Answer the particular point included in the receiver."

    ^translation
!

translation: vectorOrTuple
    "Change the particular point included in the receiver."

    translation := vectorOrTuple
! !

!LinearVariety methodsFor:'arithmetic'!

+ vector
    "Answer the translation of the receiver by vector."

    ^self class subspace: self subspace translation: self translation + vector
! !

!LinearVariety class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/LinearVariety.st,v 1.1 2002/11/22 19:44:15 cg Exp $'
! !
