"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#Matrix
	instanceVariableNames:'rows reducer'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!Matrix class methodsFor:'documentation'!

documentation
"
    My instances represent matrices. Each matrix has a reducer, i.e., an object
        encapsulating a reduction algorithm. See the documentation of MatrixeRducer
        for further details.

    For an example try: 

    (Matrix fromString:
        '0 1 2 3 -2 7
        1 3 -5 6 2 0
        9 1 8 0 -1 4') inspect
"
! !

!Matrix class methodsFor:'instance creation'!

fromString: aString
    "Answer a new instance of the receiver with the entries
    read from aString."

    | stream answer |
    stream := aString readStream.
    answer := self new.
    [stream atEnd] whileFalse: [
        answer addRow: (Tuple from:
            (stream nextLine asArrayOfSubstrings collect: [:each | each asNumber]))].
    ^answer
!

identity: anInteger
    "Answer the identity Matrix of dimension anInteger."

    | answer |
    answer := self new.
    1 to: anInteger do: [:each | answer addRow: (Tuple e: each dim: anInteger)].
    ^answer
!

new
    "Answer a new instance of the receiver."

    ^super new initialize
!

null: anInteger
    "Answer the null Matrix of dimension anInteger."

    | answer |
    answer := self new.
    1 to: anInteger do: [:each | answer addRow: (Tuple nullDim: anInteger)].
    ^answer
! !

!Matrix class methodsFor:'defaults'!

defaultReducerClass
    "Answer the default Class of objects to reduce a matrix."

    ^MatrixReducer
!

whiteSpace
    "Answer the white space string separating each entry
    in the #printOn: method."

    ^'    '
! !

!Matrix methodsFor:'ST/V methods'!

addNullRow
    "Append a null row as the last row of the receiver."

	self rows add: (Tuple nullDim: self rowDim)
!

addRow: aTuple
    "Append aTuple as the last row of the receiver.
    Answer aTuple."

    ^self rows add: aTuple
!

asMatrix
    "Answer a Matrix with the entries of the receiver."

    ^self
!

at: aPoint
    "Answer the entry of the receiver in position column @ row = aPoint."

    ^(self row: aPoint y) at: aPoint x
!

at: aPoint put: anObject
    "Change the entry of the receiver in position column @ row = aPoint."

    ^(self row: aPoint y) at: aPoint x put: anObject

!

atRow: anInteger put: aTuple
    "Change the row anInteger of the receiver with aTuple. Answer aTuple."

    ^self rows at: anInteger put: aTuple

!

basicProduct: aMatrix
    "Private - Answer the product of the receiver by aMatrix."

    | answer row items |
    answer := self class new.
    items := self rowDim.
    1 to: self colDim do: [:rIndex |
        row := Tuple new: aMatrix rowDim.
        1 to: row dim do: [:cIndex |
            | sum |
            sum := 0.
            1 to: items do: [:each |
                sum := (self at: each @ rIndex) * (aMatrix at: cIndex @ each) + sum].
            row at: cIndex put: sum].
        answer addRow: row].
    ^answer
!

colDim
    "Answer the column size of the receiver."

    ^self rows size
!

collect: aBlock
    "Public - For each element in the receiver, evaluate aBlock with
    that element as the argument.  Answer a new Tuple containing
    the results as its coordinantes from the aBlock evaluations."

    | answer |
    answer := self class new.
    self rowsDo: [:each | answer addRow: each copy].
    ^answer

!

column: anInteger
    "Answer the column number anInteger of the receiver.
    The first column is number 1."

    | answer |
    answer := Column new: self colDim.
    1 to: answer dim do: [:each | answer at: each put: (self at: anInteger @ each)].
    ^answer
!

copy
    "Answer a copy of the receiver."

    | answer |
    answer := self class new.
    self rowsDo: [:each | answer addRow: each copy].
    ^answer
!

det
    "Public - Answer the determinant of the receiver."

    ^self reducer det
!

divideRow: anInteger by: aNumber
    "Replace the row anInteger of the matrix with its division by aNumber."

    self atRow: anInteger put: (self row: anInteger) / aNumber
!

hash
    "Answer the hash value of the receiver."

    ^self trace hash
!

initialize
    "Private - Initialize the receiver."

    self rows: OrderedCollection new.
!

lastColumn
    "Answer the last Column of the receiver."

    ^self column: self rowDim
!

leftInverse
	"Answer the left inverse to the receiver."

	^self reducer inverse
!

negated
    "Answer the negated of the receiver."

    | answer |
    answer := self class new.
    self rowsDo: [:each | answer addRow: each negated].
    ^answer
!

printOn: aStream
    "Append a String representation of the receiver on aStream."

    aStream nextPutAll: '\pmatrix{'; cr.
    self rowsDo: [:each |
        each do: [:some |
            aStream
                nextPutAll: self class whiteSpace asString;
                nextPut: $&.
            some printOn: aStream].
        aStream nextPutAll: self class whiteSpace asString, '\cr'; cr].
    aStream nextPut: $}
!

rank
    "Answer the rank of the receiver."

    ^self reducer rank
!

reduced
    "Answer the reduced (fully triangulated) by rows matrix associated to the receiver."

    ^self reducer reduced
!

reducer
    "Answer the reducer associated to the receiver."

    ^reducer ifNil: [
        self reducer: self class defaultReducerClass new.
        reducer]
!

reducer: aMatrixReducer
    "Public - Change the matrix reducer for the receiver.
    The reducer is the object for triangulation operations."

    reducer := aMatrixReducer.
    reducer matrix: self.
!

row: anInteger
    "Answer the row number anInteger of the receiver.
    The first row is number 1."

    ^rows at: anInteger
!

rowDim
    "Answer the row size of the receiver."

    ^self rows isEmpty
        ifTrue: [0]
        ifFalse: [(self row: 1) dim]
!

rows
    "Answer the Collection of rows of the receiver."

    ^rows
!

rows: aTupleCollection
    "Private - Change the Collection of rows of the receiver."

    rows := aTupleCollection
!

rowsDo: aBlock
    "For each row in the receiver evaluate the one argument block aBlock
    with that row as the argument."

    self rows do: aBlock
!

scalarProduct: aNumber
    "Private - Answer the product of the receiver by aNumber."

    | answer |
    answer := self class new.
    self rowsDo: [:each | answer addRow: each * aNumber].
    ^answer
!

swapRows: anInteger and: anotherInteger
    "Swap the rows of the receiver."

    | swap |
    swap := self row: anInteger.
    self atRow: anInteger put: (self row: anotherInteger).
    self atRow: anotherInteger put: swap
!

trace
    "Public - Answer the trace of the receiver."

    | answer |
    answer := 0.
    1 to: (self colDim min: self rowDim) do: [:each | answer := answer + each].
    ^answer
!

transposed
    "Answer the transpose of the receiver."

    | answer |
    answer := self class new.
    1 to: self rowDim do: [:each | answer addRow: (self column: each) transposed].
    ^answer
! !

!Matrix methodsFor:'arithmetic'!

* aNumberOrMatrix
    "Answer the product of the receiver by aNumberOrMatrix."

    ^aNumberOrMatrix isNumber
        ifTrue: [self scalarProduct: aNumberOrMatrix]
        ifFalse: [self basicProduct: aNumberOrMatrix asMatrix]
!

+ aMatrix
    "Answer the sum of the receiver and aMatrix."

    | answer |
    answer := self class new.
    1 to: self colDim do: [:each | answer addRow: (self row: each) + (aMatrix row: each)].
    ^answer
!

- aMatrix
    "Answer the difference of the receiver minus aMatrix."

    | answer |
    answer := self class new.
    1 to: self colDim do: [:each | answer addRow: (self row: each) - (aMatrix row: each)].
    ^answer
!

/ aNumber
    "Public - Answer the division of the receiver by aNumber."

    | answer |
    answer := self class new.
    self rowsDo: [:each | answer addRow: each / aNumber].
    ^answer
! !

!Matrix methodsFor:'comparing'!

= aMatrix
    "Answer true if the receiver has the same rows as aMatrix."

    ^aMatrix class = self class and: [
        aMatrix rowDim = self rowDim and: [
            aMatrix colDim = self colDim and: [
                ((1 to: self colDim)
                    detect: [:one |    (aMatrix row: one) ~= (self row: one)]
                    ifNone: [nil]) isNil]]]
! !

!Matrix class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/Matrix.st,v 1.1 2002/11/22 19:43:26 cg Exp $'
! !
