"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#MatrixReducer
	instanceVariableNames:'matrix currentPivot pivots usedRows isReduced inverse det'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!MatrixReducer class methodsFor:'documentation'!

documentation
"
    My instances encapsulate algorithms for reducing matrices. Different kind
        of matrices need different reducing algorithms because of the allowed
        operations on rows and columns. The operations allowed in a given algorithm
        depend on the ring where the entries of the matrix are to be considered. A
        rational matrix, i.e., a matrix with rational coefficients, can be reduced using
        divisions (multiplication by reciprocals). On the other hand, a matrix with
        integer or polynomials coefficients should be reduced without reciprocals.
        Performance considerations should also be cosidered when using one algorthm
        or the other. (Note: the implementation given here has only one Gauss algorithm
        and that algorithm uses reciprocals).

    For an example try:

        | matrix reducer |
        matrix := (Matrix fromString:
            '0 1 2 3 -2 7
            1 3 -5 6 2 0
            9 1 8 0 -1 4').
        reducer := MatrixReducer matrix: matrix.
        matrix reducer: reducer.
        reducer reduce.
        matrix inspect.
        reducer inspect
"
! !

!MatrixReducer class methodsFor:'instance creation'!

matrix: aMatrix
    "Answer a new instance of the receiver for the matrix aMatrix."

    ^self new matrix: aMatrix
!

new
    "Answer a new instance of the receiver."

    ^super new initialize
! !

!MatrixReducer methodsFor:'ST/V methods'!

addPivot: aPoint
    "Private - Add aPoint to the pivots of the receiver."

    self pivots add: aPoint.
    self currentPivot: aPoint.
    self usedRows add: aPoint y

!

addRow: anIndex by: aNumber toRow: rowIndex
    "Private - Replace the row rowIndex of the matrix by its sum with
    row anIndex multiplied by aNumber."

    | row rowBy |
    row := self matrix row: rowIndex.
    rowBy := self matrix row: anIndex.
    self currentPivot x to: self rowDim do: [:cIndex |
        row
            at: cIndex
            put: ((row at: cIndex) + ((rowBy at: cIndex) * aNumber))].
    self inverse
        atRow: rowIndex
        put: (self inverse row: anIndex) * aNumber + (self inverse row: rowIndex).
!

colDim
    "Answer the dimesion of the columns of the matix in the receiver."

    ^self matrix colDim
!

currentPivot
    "Private - Answer the current pivot of the receiver."

    ^currentPivot
!

currentPivot: aPoint
    "Private - Change the current pivot of the receiver."

    currentPivot := aPoint

!

dependentColumns
    "Public - Answer the Collection of indexes of dependent columns."

    | indep |
	indep := self independentColumns.
	^(1 to: self rowDim) reject: [:each | indep includes: each]
!

det
    "Public - Answer the determinant of the receiver."

    ^self isReduced
        ifTrue: [self privateDet]
        ifFalse: [self reduce; privateDet]
!

det: aNumber
    "Private - Change the determinant of the receiver."

    det := aNumber
!

divideRow: anInteger by: aNumber
    "Private - Replace the row anInteger of the matrix with its division by aNumber."

    aNumber = 1 ifTrue: [^self].
    self currentPivot x to: self rowDim do: [:cIndex |
        self matrix at: cIndex @ anInteger put: (self matrix at: cIndex @ anInteger) / aNumber].
    self inverse divideRow: anInteger by: aNumber.
    self det: self privateDet * aNumber
!

independentColumns
    "Public - Answer the Collection of indexes of independent columns."

    ^self isReduced
		ifTrue: [self pivots collect: [:each | each x]]
		ifFalse: [self reduce; independentColumns]
!

initialize

    self det: 1.
    self currentPivot: 0 @ 0.
    self pivots: OrderedCollection new.
    self usedRows: OrderedCollection new.
    self isReduced: false

!

inverse
    "Private - Answer the left inverse matrix of the matrix."

    ^inverse
!

inverse: aMatrix
    "Private - Change the left inverse matrix of the matrix."

    inverse := aMatrix
!

isReduced
    "Private - Answer true if the receiver is reduced.
    Else answer false."

    ^isReduced
!

isReduced: aBoolean
    "Private - Change the value of the reduced flag to aBoolean."

    isReduced := aBoolean
!

leftInverse
    "Public - Answer the left inverse matrix of the matrix."

    ^self isReduced
        ifTrue: [self inverse]
        ifFalse: [self reduce; inverse]

!

matrix
    "Private - Answer the matrix of the receiver."

    ^matrix
!

matrix: aMatrix
    "Public - Start over a new matrix."

    self initialize.
    self inverse: (aMatrix class identity: aMatrix colDim).
    matrix := aMatrix copy
!

nextPivot
    "Private - Advance the position to the next available pivot."

    | col used rows row r min |
    col := self currentPivot x + 1.
    used := self usedRows.
    [col <= self rowDim and: [
    (rows := (1 to: self colDim) select: [:rIndex |
        (used includes: rIndex) not and: [
        (self matrix at: col @ rIndex) ~= 0]]) isEmpty]] whileTrue:
        [col := col + 1].
    col > self rowDim ifTrue: [^nil].
    row := rows
        detect: [:rIndex | (self matrix at: col @ rIndex) abs = 1]
        ifNone: [
            r := rows first.
            min := (self matrix at: col @ r) abs.
            rows do: [:each |
                (self matrix at: col @ each) abs < min ifTrue: [
                    r := each.
                    min := (self matrix at: col @ r) abs]].
            r].
    self addPivot: col @ row.
    ^col @ row
!

pivots
    "Private - Answer the collection of positions
    used by pivots in the receiver."

    ^pivots
!

pivots: aPointCollection
    "Private - Change the collection of positions
    used by pivots in the receiver."

    pivots := aPointCollection
!

privateDet
    "Private - Answer the determinant of the receiver."

    ^det
!

rank
    "Answer the rank of the receiver."

    ^self isReduced
        ifTrue: [self pivots size]
        ifFalse: [self reduce; rank]
!

reduce
    "Private - Reduce the matrix using the current pivot."

    | row |
    self isReduced ifTrue: [^self].
    row := 1.
    [self nextPivot isNil] whileFalse: [
        self swapRow: row.
        row := row + 1.
        self reduceStep].
    self isReduced: true
!

reduceStep
    "Private - Perform one step reduction of the matrix using the current pivot."

    | row col |
    row := self currentPivot y.
    col := self currentPivot x.
    self divideRow: row by: (self matrix at: self currentPivot).
    1 to: self colDim do: [:rIndex |
        | value |
        (rIndex = row or: [(value := self matrix at: col @ rIndex) = 0]) ifFalse: [
            self addRow: row by: value negated toRow: rIndex]]
!

reduced
    "Public - Answer the reduced (full triangulated) matrix."

    ^self isReduced
        ifTrue: [self matrix]
        ifFalse: [self reduce; matrix]
!

rowDim
    "Answer the dimesion of the rows of the matix in the receiver."

    ^self matrix rowDim
!

swapRow: anInteger
    "Private - Swap the row anInteger with the row of the current pivot."

    | swap |
    anInteger = self currentPivot y ifTrue: [^self].
    self matrix swapRows: anInteger and: self currentPivot y.
    self inverse swapRows: anInteger and: self currentPivot y.
    self usedRows remove: self currentPivot y.
    self usedRows add: anInteger.
    self currentPivot y: anInteger.
    self det: self privateDet negated
!

unusedRowsDo: aBlock
    "Private - For each row in the matrix not used by a pivot
    evaluate the one argument block aBlock with that row as
    an argument."

    | used |
    used := self usedRows.
    1 to: self colDim do: [:rIndex |
        (used includes: rIndex) ifFalse: [aBlock value: (self matrix row: rIndex)]]
!

usedCols
    "Private - Answer the collection of cols of all the pivots
    of the receiver."

    ^self pivots collect: [:each | each x]
!

usedRows
    "Private - Answer the collection of rows of all the pivots
    of the receiver."

    ^usedRows
!

usedRows: aCollection
    "Private - Change the collection of rows of all the pivots
    of the receiver."

    usedRows := aCollection
! !

!MatrixReducer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/MatrixReducer.st,v 1.1 2002/11/22 19:43:53 cg Exp $'
! !
