"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#Polygon
	instanceVariableNames:'vertices position'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!Polygon class methodsFor:'documentation'!

documentation
"
        My instances represent polygons in an ambient space of
        any given dimension.

        Each polygon has vertices and edges joining them. The last vertex
        is joined to the first one by an edge (i.e., the polygon is closed).
        The edges of a Polygon are Segments. See the Segment class for
        further documentation.

        Vertices and edges are numerated. The edge with origin in vertex
        k has the index k.

        For an example try this:

        | p |
        p := Polygon fromVertices: (OrderedCollection
                with: (3, 1, 2) with: (-1, 0, 8) with: (17, 5, -3)).
        p barycentre.

        Also a Stream protocol is provided. The stream is endless since it
        wraps around the last vertex but this situation is signaled by an atEnd
        condition.

        For an example try this:

        | p |
        p := Polygon new.
        p add: (3, 1, 2); add: (-1, 0, 8); add: (17, 5, -3); add: (-6, 0, 1); yourself.
        p next.
        [p atEnd] whileFalse: [p next].
        p position.
        p next.

        An algorithm is provided that responds if a given point is 'interior' to
        a plane polygon.

        For an example try this:

        | p q |
        p := Polygon fromVertices: (OrderedCollection
                with: (3, 1, 2) with: (-1, 0, 8) with: (17, 5, -3)).
        q := p barycentre.
        p isInterior: q
"
! !

!Polygon class methodsFor:'instance creation'!

fromPoints: aPointCollection

    ^self new vertices: (aPointCollection collect: [:each | (each x, each y)])
!

fromVertices: aTupleCollection
	"Answer a new intances of the receiver using aTupleCollection as the
	collection of vertices."

    ^self new vertices: aTupleCollection
!

new

	^super new initialize
! !

!Polygon class methodsFor:'defaults'!

edgeClass
    "Answer the class modeling the edges of the
    instance of the receiver."

    ^Segment
! !

!Polygon methodsFor:'ST/V methods'!

add: aTuple
	"Public - Add aTuple as a new vertex of the receiver.
	Answer aTuple."

	^self vertices add: aTuple
!

ambient
	"Answer the ambient space containing the receiver."

	^self size = 0
		ifTrue: [LinearSubspace new]
		ifFalse: [self first ambient]
!

atEnd
	"Answer true if the position is on the last vertex of the receiver."

	^self position = self size
!

barycentre
    "Answer the Tuple corresponding to the centre of mass of the receiver."

    | answer |
    answer := self first class nullDim: self ambient dim.
    self do: [:each | answer := answer + each].
    ^answer / self size
!

basicDiameter2
	"Private - Answer the squared diameter of the receiver. Use a naive algorthm."

	| answer n |
	n := self size.
	answer := 0.
	1 to: n - 1 do: [:each |
		each + 1 to: n do: [:some |
			answer := ((self vertex: each) distance2To: (self vertex: some)) max: answer]].
	^answer
!

diameter
	"Public - Answer the diameter of the receiver."

	^self diameter2 sqrt
!

diameter2
	"Public - Answer the squared diameter of the receiver."

	^self basicDiameter2
!

do: aBlock
    "Answer the receiver. For each vertex of the receiver evaluate
    the one argumento block aBlock with that vertex as the argument."

    self vertices do: aBlock
!

edge: anInteger
    "Answer the edge that goes from vertex  anInteger to the
    following vertex of the receiver."

    | next |
    next := (anInteger = self size
        ifTrue: [1]
        ifFalse: [anInteger + 1]).
    ^self edgeClass
        origin: (self vertex: anInteger)
        destiny: (self vertex: next)
!

edgeClass
    "Answer the class modeling the edges of the receiver."

    ^self class edgeClass
!

edges
    "Answer the collection of edges of the receiver."

    ^(1 to: self size) collect: [:each | self edge: each]
!

farthest
	"Answer the vertex wich has the greatest distance to the barycenter
	of the receiver."

	| b dist answer |
	b := self barycentre.
	answer := self first.
	dist := b distance2To: answer.
	self do: [:each |
		| d |
		d := b distance2To: each.
		d > dist ifTrue: [
			answer := each.
			dist := d]].
	^answer
!

first
	"Answer the first vertex of the receiver."

	^self vertices first
!

initialize
	"Initialize the receiver."

	self vertices: OrderedCollection new.
	self position: 0
!

inner
	"Answer the vertex wich has the smallest distance to the barycenter
	of the receiver."

	| b dist answer |
	b := self barycentre.
	answer := self first.
	dist := b distance2To: answer.
	self do: [:each |
		| d |
		d := b distance2To: each.
		d < dist ifTrue: [
			answer := each.
			dist := d]].
	^answer
!

isBoundary: aTuple
    "Answer true if aTuple is on the boundary of the receiver."

    ^(self edges
        detect: [:one | one includes: aTuple]
        ifNone: []) notNil


!

isInterior: aTuple
	"Answer true if aTuple is in the boundary or the interior of the shape
	delimieted by the receiver."

	| variety segments p q edge line intersections dir coord |
	(variety := self variety) dim = 2 ifFalse: [self error: 'Polygon is not laying on a plane'].
	(variety includes: aTuple) ifFalse: [^false].
	(self isBoundary: aTuple) ifTrue: [^true].

	"Initialize some variables for the sake of efficiency."
	segments := self edges.

	"Choose an edge whose line does not include aTuple. Such an edge must exist
	because the receiver generates a variety of dimension 2."
	edge := segments detect: [:one | (one line includes: aTuple) not].

	"Chosee a point p on edge such that the line joining aTuple and p
	does not overlap any edge of the receiver."
	(1 / (self size +1) to: self size by: 1 / (self size +1)) detect: [:some |
		p := edge direction * some + edge origin.
		line := (Segment origin: aTuple destiny: p) line.
		(segments detect: [:one | one line includes: line] ifNone: []) isNil
		].

	"Count the intersections of the receiver with the semirect from aTuple to p."
	intersections := 0.
	segments do: [:each |
		q := (each line intersection: line).
		q notNil ifTrue: [
			q :=  q translation.
			(each includes: q) ifTrue: [
				dir := p - aTuple.
				coord := (1 to: self ambient dim) detect: [:one | (dir at: one) ~= 0].
				((q - aTuple at: coord) = 0 or: [(q - aTuple at: coord) sign = (dir at: coord) sign])
				ifTrue: [intersections := intersections + 1]
			]
		]
	].
	^intersections odd.
!

next
	"Answer the vertex in the next position of the receiver."

	self position: self position + 1.
	self position > self size ifTrue: [self position: 1].
	^self vertices at: self position
!

perimeter
    "Answer the perimeter of the receiver."

    | answer |
    answer := 0.
    self edges
        do: [:each | answer := answer + each length].
    ^answer
!

position
	"Answer the position of the receiver."

	^position
!

position: anInteger
	"Change the position of the receiver."

	position := anInteger
!

reset
	"Change the current position to the first vertex."

	self position: 0
!

size
    "Answer the number of vertices of the receiver."

    ^self vertices size
!

variety
	"Answer the linear variety generated by the receiver."

	| first generators |
	first := self first.
	generators := OrderedCollection new: self size.
	self do: [:each | generators add: each - first].
	^LinearVariety
		subspace: (LinearSubspace generators: generators)
		translation: first
!

vertex: anInteger
    "Answer a Tuple that is the vertex number anInteger of the receiver."

    ^vertices at: anInteger

!

vertices
    "Answer the collection of vertices of the receiver."

    ^vertices
!

vertices: aTupleCollection
    "Change the vertices of the receiver."

    vertices := aTupleCollection asOrderedCollection
! !

!Polygon class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/Polygon.st,v 1.1 2002/11/22 19:43:37 cg Exp $'
! !
