"{ Package: 'stx:goodies/math/linearAlgebra' }"

"{ NameSpace: Math2 }"

Object subclass:#Tuple
	instanceVariableNames:'entries'
	classVariableNames:''
	poolDictionaries:''
	category:'V''Age classes'
!

!Tuple class methodsFor:'documentation'!

documentation
"
    My instances represent row vectors given by coordinates.

    For an example try this:

    (3, -7, 12, 4) inspect

    Or this one:

    | e1 e2 e3 |
    e1 := Tuple e: 1 dim: 3.
    e2 := Tuple e: 2 dim: 3.
    e3 := Tuple e: 3 dim: 3.
    (3*e1 + (5*e2) - (7*e3)) inspect
"
! !

!Tuple class methodsFor:'instance creation'!

e: indexInteger dim: dimInteger
    "Answer the canonical tuple of dimInteger coordinates with index indexInteger."

    ^(self new: dimInteger)
        at: indexInteger put: 1;
        yourself
!

from: aCollection
    "Public - Answer a new instance of the receiver with the entries taken from aCollection."

    ^self new entries: aCollection
!

fromPoint: aPoint
    "Public - Answer a new instance of the receiver with the
    coordinats of aPoint."

    ^self new entries: (Array with: aPoint x with: aPoint y)
!

new: anInteger
    "Private - Answer a new instance of the receiver with anInteger coordinates."

    ^self from: ((Array new: anInteger) atAllPut: 0; yourself)
!

nullDim: dimInteger
    "Answer the null tuple of dimInteger coordinates."

    ^self new: dimInteger
! !

!Tuple methodsFor:'ST/V methods'!

ambient
	"Answer the ambient space of the receiver."

	^self dim = 0
		ifTrue: [LinearSubspace new]
		ifFalse: [LinearSubspace dim: self dim]
!

asLinearForm
    "Answer a LinearForm with the coefficients of the receiver."

    ^LinearForm from: self entries
!

asLinearVariety
	"Answer the LinearVariety consisting in the singleton of the receiver."

	^LinearVariety subspace: (LinearSubspace nullDim: self dim) translation: self
!

asMatrix
    "Answer a Matrix with the entries of the receiver."

    ^Matrix new addRow: self; yourself
!

asPoint
    "If the receiver has dimesnion 2, answer the Point of the with
    the coordinates of the receiver. Else signal an error."

    ^self dim = 2
        ifTrue: [(self at: 1) @ (self at: 2)]
        ifFalse: [self error: 'Invalid dimension.']
!

asSubspace
	"Answer the LinearSubspace generated by the receiver."

	^LinearSubspace generators: (OrderedCollection with: self)
!

at: anInteger
    "Public - Answer the coordinate number anInteger of the receiver."

    ^self entries at: anInteger
!

at: anInteger put: anObject
    "Public - Chante the coordinate number anInteger of the receiver
    with anObject. Answer anObject."

    ^self entries at: anInteger put: anObject

!

collect: aBlock
    "Public - For each element in the receiver, evaluate aBlock with
    that element as the argument.  Answer a new Tuple containing
    the results as its coordinantes from the aBlock evaluations."

    ^self class from: (entries collect: aBlock)
!

copy
    "Answer a copy of the receiver."

    ^self class new entries: self entries shallowCopy
!

copyFrom: startIndex to: stopIndex
    "Answer a copy of the receiver within the given indexes."

    ^self class from: (self entries copyFrom: startIndex to: stopIndex)
!

dim
    "Public - Answer the number of coordinates of the receiver."

    ^self entries size
!

distance2To: aTuple
	"Public - Answer the squared distance between the receiver and aTuple."

	^(aTuple isKindOf: Tuple)
		ifTrue: [(self - aTuple) norm2]
		ifFalse: [aTuple distance2To: self]

!

distanceTo: aTuple
	"Public - Answer the distance between the receiver and aTuple."

	^(self distance2To: aTuple) sqrt

!

do: aBlock
    "For each entry en the receiver evaluate the one argument block aBlock
    with that entry as the argument."

    self entries do: aBlock
!

dual
    "Answer the LinearForm whose coefficients are the coordinates of the receiver."

    ^LinearForm from: self entries
!

entries
    "Private - Answer the Array of entries of the receiver."

    ^entries
!

entries: aCollection
    "Private - Change the entries of the receiver."

    entries := aCollection asArray
!

hash
    "Answer the hash value of the receiver."

    ^self entries hash
!

isNull
    "Answer true if the receiver is the null tuple."

    1 to: self dim do: [:each |
        (self at: each) = 0 ifFalse: [^false]].
    ^true
!

negated
    "Public - Answer the negated, coordinate by coordinate, of the receiver."

    ^self collect: [:each | each negated]
!

norm
    "Public - Answer the norm of the receiver."

    ^(self scalar: self) sqrt
!

norm2
    "Public - Answer the squared norm of the receiver."

    ^self scalar: self
!

printOn: aStream
    "Public - Append a String representation of the receiver to aStream."

    | first |
    first := true.
    aStream nextPut: $(.
    1 to: self dim do: [:each |
        first
            ifTrue: [first := false]
            ifFalse: [    aStream nextPutAll: ', '].
        aStream nextPutAll: (self at: each) printString].
    aStream nextPut: $)
!

scalar: aTuple
    "Public - Answer the scalar product of the reciver and aTuple."

    | answer |
    answer := 0.
    1 to: self dim do: [:each |
        answer := (self at: each) * (aTuple at: each) + answer].
    ^answer
!

transposed

    ^Column from: self entries
!

verifies: aLinearEquation
    "Answer true if the receiver is a solution of aLinearForm."

    ^aLinearEquation isSolution: self
! !

!Tuple methodsFor:'arithmetic'!

* aNumber
    "Public - Answer the product of the receiver by aNumber."

    ^self collect: [:each | each * aNumber]
!

+ aTuple
    "Public - Answer the sum, coordinate by coordinate, of the receiver and aTuple."

    | answer |
    answer := self class new: self dim.
    1 to: self dim do: [:each |
        answer at: each put: (self at: each) + (aTuple at: each)].
    ^answer
!

, anObject
    "Answer a new Tuple appending anObject to the receiver."

    ^self class from: self entries, (Array with: anObject)
!

- aTuple
    "Public - Answer the difference, coordinate by coordinate, of the receiver and aTuple."

    | answer |
    answer := self class new: self dim.
    1 to: self dim do: [:each |
        answer at: each put: (self at: each) - (aTuple at: each)].
    ^answer
!

/ aNumber
    "Public - Answer the division of the receiver by aNumber."

    ^self collect: [:each | each / aNumber]
! !

!Tuple methodsFor:'comparing'!

= aTuple
    "Answer true if the receiver has the same coordinates as aTuple."

    ^(self class = aTuple class) and: [
        self dim = aTuple dim and: [
        self entries = aTuple entries]]
! !

!Tuple class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/math/linearAlgebra/Tuple.st,v 1.1 2002/11/22 19:43:42 cg Exp $'
! !
