"{ Package: 'stx:goodies/measurement' }"

Object subclass:#Unit
	instanceVariableNames:'key'
	classVariableNames:'UnitAllocationLock Unitless UnitsByKey UnitsByName'
	poolDictionaries:''
	category:'Measurements'
!

!Unit class methodsFor:'documentation'!

documentation
"
    Measurements and Units package by Travis Griggs.

    1 asMeter
"
! !

!Unit class methodsFor:'instance creation'!

of: aString 

        ^UnitsByName at: aString ifAbsentPut: [self byKey: (KeyOnlyUnit fromString: aString) key]
! !

!Unit class methodsFor:'accessing'!

unitless
	^Unitless
!

unitsByKey
	^UnitsByKey
! !

!Unit class methodsFor:'class initialization'!

initialize
        "self initialize"

        UnitsByKey := Dictionary new.
        UnitsByName := Dictionary new.
        UnitAllocationLock := RecursionLock new.

        "/ cg: added for ST/X - load extensions
        ((Smalltalk respondsTo:#isSmalltalkX) and:[Smalltalk isSmalltalkX]) ifTrue:[
            (Fraction implements:#unitNameFromKey) ifFalse:[
                |packageDir|

                packageDir := Smalltalk getPackageDirectoryForPackage:self package.
                packageDir notNil ifTrue:[
                    (packageDir construct:'extensions.st') fileIn    
                ]
            ].
            BaseUnit notNil ifTrue:[BaseUnit autoload]. 
            DerivedUnit autoload. "KeyOnlyUnit autoload"
            Celsius autoload.  Farenheit autoload.  Kelvin autoload. Rankine autoload.
            IncompatibleUnitsError autoload.  
            Measurement autoload.
        ].

        KeyOnlyUnit notNil ifTrue:[
            self installUnits
        ].
!

installUnitless
	Unitless := BaseUnit new setName: '' key: 1.
	self addUnit: Unitless
!

installUnits
        "self installUnits"

        self
                "/ initialize;
                installUnitless.
        BaseUnit installBaseUnits.
        DerivedUnit installDerivedUnits
! !

!Unit class methodsFor:'parsing'!

basicUnitPartIn: aString 
	| rs |
	rs := aString readStream.
	^rs nextUntil: [:char | char isDigit]
!

denominatorPartIn: aString 
	| bag rs numPiece |
	bag := Bag new.
	rs := aString readStream.
	[rs atEnd] whileFalse: 
			[rs skipThrough: $/.
			numPiece := rs upTo: $/.
			bag addAll: (self parseUnitTokensIn: numPiece)].
	^bag
!

numeratorPartIn: aString 
	| bag rs numPiece |
	bag := Bag new.
	rs := aString readStream.
	[rs atEnd] whileFalse: 
			[numPiece := rs upTo: $/.
			bag addAll: (self parseUnitTokensIn: numPiece).
			rs skipThrough: $/].
	^bag
!

parseUnitTokensIn: aString 
	| rs unitParts piece |
	rs := aString readStream.
	unitParts := Bag new.
	[rs atEnd] whileFalse: 
			[piece := rs upTo: $..
			piece isEmpty 
				ifFalse: 
					[unitParts add: (self basicUnitPartIn: piece)
						withOccurrences: (self unitPowerPartIn: piece)]].
	^unitParts
!

unitPowerPartIn: aString 
	| rs |
	rs := aString readStream.
	rs skipWhile: [:char | char isDigit not].
	^rs atEnd ifTrue: [1] ifFalse: [Integer readFrom: rs]
! !

!Unit class methodsFor:'private'!

addUnit: aUnit 
	UnitAllocationLock
		critical: 
			[UnitsByName at: aUnit name put: aUnit.
			UnitsByKey at: aUnit key put: aUnit].
	^aUnit
!

baseKeyImpliedBy: aKey 
	| numerator denominator |
	numerator := 1.
	denominator := 1.
	aKey numerator = 1 
		ifFalse: 
			[aKey numerator primeFactors valuesAndCountsDo: 
					[:prime :count | 
					numerator := numerator * ((UnitsByKey at: prime) baseUnit key ** count)]].
	aKey denominator = 1 
		ifFalse: 
			[aKey denominator primeFactors valuesAndCountsDo: 
					[:prime :count | 
					denominator := denominator 
								* ((UnitsByKey at: prime) baseUnit key ** count)]].
	^numerator / denominator
!

byKey: aKey 
	^UnitsByKey at: aKey ifAbsent: [self addUnit: (self createFromKey: aKey)]
!

conversionFactorImpliedBy: aKey 
	| numerator denominator |
	numerator := 1.
	denominator := 1.
	aKey numerator = 1 
		ifFalse: 
			[aKey numerator primeFactors valuesAndCountsDo: 
					[:prime :count | 
					numerator := numerator * ((UnitsByKey at: prime) conversion ** count)]].
	aKey denominator = 1 
		ifFalse: 
			[aKey denominator primeFactors valuesAndCountsDo: 
					[:prime :count | 
					denominator := denominator * ((UnitsByKey at: prime) conversion ** count)]].
	^numerator / denominator
!

createFromKey: aKey 
	| baseKey |
	baseKey := self baseKeyImpliedBy: aKey.
	^baseKey = aKey
		ifTrue: [BaseUnit new setName: aKey unitNameFromKey key: aKey]
		ifFalse: [DerivedUnit new
				setName: aKey unitNameFromKey
				key: aKey
				conversion: (self conversionFactorImpliedBy: aKey)
				base: (self byKey: baseKey)]
!

nextPrime
	^UnitAllocationLock critical: [UnitsByKey isEmpty
			ifTrue: [2]
			ifFalse: [(UnitsByKey keys inject: 1 into: [:max :each | (each isInteger and: [each isPrime])
						ifTrue: [each max: max]
						ifFalse: [max]]) nextPrime]]
! !

!Unit methodsFor:'accessing'!

* aUnit 
	^KeyOnlyUnit key: key * aUnit key
!

/ aUnit 
	^KeyOnlyUnit key: key / aUnit key
!

allUnits
	^UnitsByKey values asSortedCollection: [:a :b | a name < b name]
!

baseUnit
	^self subclassResponsibility
!

canonicalUnit
	^self
!

compatibleUnits
	| result base |
	result := OrderedCollection new.
	base := self baseUnit.
	UnitAllocationLock critical: [UnitsByKey values do: [:each | each baseUnit = base ifTrue: [result add: each]]].
	^result
!

conversion
	^self subclassResponsibility
!

convert: aMeasurement 
	aMeasurement baseUnit == self baseUnit ifFalse: [self incompatibleUnitsError].
	^Measurement magnitude: aMeasurement baseMagnitude / self conversion unit: self
!

incompatibleUnitsError
	^IncompatibleUnitsError raiseSignal
!

name
	^self subclassResponsibility
!

reciprocal
	^KeyOnlyUnit key: 1 / key
! !

!Unit methodsFor:'comparing'!

= aUnit 
	^key = aUnit key
!

hash
	^key hash
! !

!Unit methodsFor:'testing'!

isBaseUnit
	^false
! !

!Unit class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/measurement/Unit.st,v 1.10 2003/10/20 15:44:56 penk Exp $'
! !

Unit initialize!
