"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageElement subclass:#HeaderField
	instanceVariableNames:'name'
	classVariableNames:'FieldsRegistry'
	poolDictionaries:''
	category:'NetClients-MIME'
!

HeaderField comment:'This class represents header fields in RFC822 and MIME messages. Related RFCs are
	rfc822  - ftp://uu.net/inet/rfc/rfc822.Z
	rfc2045
	rfc2046
	rfc2047
	rfc2048
	rfc2183

Each field has name and value. The general syntax of RFC822 fields is
	name: value<CrLf>
Fields can occupy more than one line using line folding techniques. Values can be structured or unstructured. Unstructured value is considered to be text (RFC822 defines text as any sequence terminated by CrLf,
but MIME specifies how non-ASCII characters can be encoded into text, so there is some decoding
involved when text lexems are parsed or composed. This class provides enough logic to process
unstructured fields. Its subclasses implement structure fields.
Structured fields have defined syntax. For each structured field a subclass of HeaderField is created
to capture its syntax and semantics. Many fields share common syntax (i.e. address fields in RFC822,
date and version fields across a number of protocols. All fields with the same syntax can be handled
with  one class unless semantical differences justify in troduction of a separate class.
Fields are responsible for parsing themselves from a stream and composing (writing) themselves
on stream. Parsing is done with a help of a scanner which has interface of RFC822Scanner.
Fields specific to protocols derived from RFC822 use specialized scanners that support protocol-
specific syntax.
A field can parse itself from a string, stream or RFC822 scanner. Examples:

	ContentTypeField fromLine: ''content-type: text/html''
	HeaderField readFrom: ''content-type: text/html'' readStream
	HeaderField parse: (RFC822Scanner on: ''content-type: text/html'' readStream)

Internally, string is wrapped in a stream, stream is wrapped in RFC822 scanner. Parsing is invoked
by calling parse:.
Parsing and composing is always done on the instance side; class methods create new instance
and invoke instance side method to complete parsing. The only processing done on the class side
is reading field name and finding proper header field class for specified field name (field factory).
Note: the current factory scheme is based on scanning subclasses and might need a little more work
because each protocol may need its own field factory. Currently a subclass implements class method
#fieldNames that answers an array of field types that this class recognizes.

Each field also stores its source. This is significant for mail protocol because allows to reproduce fields
verbatim if a message containing the field is to be resent, forwarded, etc. RFC822 and other standards
strongly advise from any rewriting of message fields if it is not absolutely necessary. Source is stored in
inherited instance variable source which is a stream; the source is the stream''s contents. Subclasses may
disable storing source. See MessageElement for more details.
Typically fields are built not by MimeEntity itself, but rather by MessageBuilder. MessageBuilder decides whether to store source in a field. One other application of storing source -- if a message is read from mail file containing multiple message, a builder stores StreamSegment as method source. This way each message element knows its exact location in the file and fields can be changed and written back to the file in place one-by-one. Stamp (unfortunately) depends on this capability.

Instance Variables:

	name    <String>        description of variable''s function

Class Variables:

	FieldsRegistry  <IdentityDictionary>     used to store field factory bindings;'
!

!HeaderField class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 2004 by eXept Software AG
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
! !

!HeaderField class methodsFor:'instance creation'!

name: aname
" Answer new instance of field corresponding to field's name. For now, treat all fields as unstructured "

	^(self fieldClassForName: aname) new name: aname; yourself
!

new
	^super new
! !

!HeaderField class methodsFor:'class initialization'!

fieldsRegistry
	"(self fieldsRegistry)"
	FieldsRegistry isEmpty
		ifTrue: [ self registerClassFields].

	^FieldsRegistry
!

initialize
	"(((self initialize)))"
	self resetRegistry.
!

registerClassFields
	HeaderField allSubclasses
		do: [ :cl |
			cl fieldNames do: [ :fname | FieldsRegistry at: fname put: cl ]]
!

resetRegistry
	FieldsRegistry := Dictionary new: 5.
! !

!HeaderField class methodsFor:'defaults'!

default
	^self new
! !

!HeaderField class methodsFor:'parsing'!

defaultFieldClass
	^HeaderField
!

fieldClassForName: fieldName

	^self fieldsRegistry at:  fieldName asLowercase ifAbsent: [self defaultFieldClass].
!

fieldNames
	^#()
!

fromLine: aString
" For compatibility with Swazoo "

	^self readFrom: aString readStream
!

readFieldNameFrom: rfc822Stream
	| fname |

	fname := rfc822Stream scanFieldName.
	rfc822Stream skipWhiteSpace.
	^(rfc822Stream atEnd or: [(rfc822Stream peekFor: $:) not])
		ifTrue: [ nil ]
		ifFalse: [fname asLowercase]
!

readFrom: aStream
	"Reads and parses message header contents from the message stream; answers an instance of header field"

	| fname scanner |
	scanner := self scannerOn: aStream.
	(fname := self readFieldNameFrom: scanner) isNil
		ifTrue: [^nil]
		ifFalse: [^(self name: fname)
				source: aStream;
				parse: scanner]
!

scannerType
	^MimeScanner
! !

!HeaderField methodsFor:'accessing'!

canMerge
	^false
!

canonicalFieldName
	| s |

	s := name copy.
	(s isNil or: [ s isEmpty]) ifTrue: [ ^String new ].
	s at: 1 put: s first asUppercase.                       " Capitalize first letter "
	^ s
!

canonicalValue
" Override as necessary "
	^self value
!

name
	^name
!

name: aString
	name := aString
!

values
" Multi-valued fields override this method "
	^Array with: self value
!

values: anArray
" Multi-valued fields override this method "
	anArray size ~= 1 ifTrue: [ ^self error: (#CantSetMultipleValuesInField << #net >> 'Can''t set multiple values in this field') ].
	self value: anArray first
! !

!HeaderField methodsFor:'composing'!

writeOn: aStream
	self parent ~~ nil
		ifTrue: [       self writeOn: aStream encoding: self parent headerCharset ]
		ifFalse: [ self printOn: aStream ]
!

writeOn: aStream encoding: aString
"Encoding should be added to value if needed "
	self
		printNameOn: aStream;
		writeValueOn: aStream encoding: aString.
!

writeValueOn: aStream encoding: aString
	| val rfcString |
	(val := self value) notNil
		ifTrue:
			[ rfcString :=  (MimeEncodedWordCoDec shouldBeEncoded: val)
				ifTrue: [ (MimeEncodedWordCoDec encodeText: val asBase64Using: aString) ]
				ifFalse: [ val ].
			aStream nextPutAll: rfcString]
! !

!HeaderField methodsFor:'parsing'!

parse: rfc822Stream
" Generic parser for unstructured fields. Copy everything up to CRLF. Scanner handles end of line rules and answers cr when end of line is seen. Scanner also folds linear white space answering space character in place of <CRLF space+> "

        [value := self doParse: rfc822Stream.]
                on: Error
                do: [ :ex |
                        Transcript cr; show:(#ErrorParsingMimeHeader << #net >> ('Error parsing mime header: <1s>  <2s>'
                        expandMacrosWith: self name
                        with: ex errorString)); cr.
                        value := nil ].
! !

!HeaderField methodsFor:'printing'!

printNameOn: aStream

	aStream
		nextPutAll: self canonicalFieldName;
		nextPut: $: ;
		space.
!

printOn: aStream indent: level
	super printOn: aStream indent: level.
	self
		printNameOn: aStream;
		printValueOn: aStream.
!

printValueOn: aStream
	| val |
	(val := self value) notNil
		ifTrue:
			[ (val isKindOf: String)
				ifTrue: [ aStream nextPutAll:  val]
				ifFalse: [ val printOn: aStream ] ]
! !

!HeaderField methodsFor:'private'!

doParse: rfc822Stream
" Generic parser for unstructured fields. This method is typically overridden for fields with known syntax to do more detailed parsing "

	^rfc822Stream skipWhiteSpace; scanText
! !

!HeaderField methodsFor:'private-initialize'!

valueFrom: aString
" Swazoo compatibility"

	^self readFrom: aString readStream
! !

!HeaderField methodsFor:'visiting'!

acceptVisit: aVisitor  with: argument
	^aVisitor acceptHeaderField: self with: argument
! !

!HeaderField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__HeaderField.st,v 1.6 2009/09/29 20:09:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__HeaderField.st,v 1.6 2009/09/29 20:09:06 cg Exp $'
! !

HeaderField initialize!
