"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#IMAPProtocolInterpreter
	instanceVariableNames:'client socket connectionStream responseStream commandSequencer
		readResponseSignal commandsInProgress connectionMutex
		responseReaderProcess'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-IMAP'
!

IMAPProtocolInterpreter comment:'IMAP Protocol Interpreter manages IMAP server connection and provides interface for sending IMAP commands and handling IMAP responses.

Instance Variables:
	client  <IMAPClient>    Instance of IMAPClient to handle unsolicited responses
	socket  <Socket>        Socket for inter-process communication
	connectionStream        <Stream>        Is used to read from and write to the external connection stream.
	responseStream  <Stream>        Is used for testing purpose to execute script respresenting complete execution of one or more commands.
	commandSequencer        <IMAPCommandSequencer>  Creates for each command identifier called a "tag"
	readResponseMutex       <Semaphore>     Protects access to commandsInProgress
	readResponseSignal      <Semaphore>     Is signalled when commandsInProgress is non-empty
	commandsInProgress      <List>  List of outstanding commands that has been sent to IMAP server and has not completed yet
	connectionMutex <Semaphore>     Protects access to the connectionStream

 IMAP Protocol Interpreter owns and controls access to IMAP server socket connection. The only interface between IMAPClient and PI is through
	executeCommand:
	connect
	disconnect

IMAP implements sharing mechanisms as follows:
PI''s Shared Resources:
IMAP Connection (connectionMutex, connectionStream responseStream) - IMAP connection is shared between command sender and response reader loop. A process must own the connection before it accesses the connection.
Response List (readResponseSignal commandsInProgress) -- this is a LIFO list of commands waiting for their response.
Command Processing
 Once a command is sent, it is added to the response list and readResponseSignal is signalled. It then adds the command to the list of commands waiting for reply. This list is LIFO list, so the last command will be the first to handle server responses. It is managed by IMAP response reader loop. IMAP Response reader loop is a separate thread that is blocked on the semaphore asociated with the response list. This semaphore''s excess signals count is equal to the number of commands in response list. Therefore, when there are outstanding commands waiting for a response, this loop will be constantly reading next response from the server and dispatching it. Responses are read from the server and parsed, so that an appropriate instance is created for the next response. The response is checked with the IMAPClient if this response is one of untagged responses that have to be recorded by the client.The response is then offered to every outstanding command beginning with the most recent. In this sense the most recent command provides ''context'' for processing responses, especially for untagged responses containing mailbox and message data. This process continues until a command consumes this response by answering true. Each command, when given a request, tried to process it. If it recognizes the response as its, it typically adds the response to the list of responses associated with this request and answers that it consumed the response. If the response is a completion (tagged) response for this command, the execution of this command is complete. At this point the command is removed from the list of outstanding commands, command''s semaphore is signalled and the excess signal counter for the list''s semaphore is decremented by one. If the list is empty, the list semaphore''s excess counter will be zero and response reader loop will be blocked on the semaphore.
Even though this response can be processed by a command in outstanding command list, this command will not indicate it has consumed this response, so it will eventually get to the protocol interpreter.

When processing a response, response reader loop should do the following:

	Lock IMAP connection
	Read next response
	Process the response
	If some command is now complete
		Remove it from response list
		Decrement response loop semaphore Unlock IMAP connection

'
!


!IMAPProtocolInterpreter class methodsFor:'instance creation'!

connectToHost: hostName port: aNumber

	^self new connectToHost: hostName port: aNumber
!

new
        ^self basicNew initialize
! !

!IMAPProtocolInterpreter class methodsFor:'debugging'!

debugLabelsAndValues
        ^List with: ('IMAP Client Messages'-> #IMAPClient)
              with: ('IMAP Server Messages'-> #IMAPServer).
!

log: aStringOrBlock level: aLevel

        Transcript showCR:aStringOrBlock value.
! !

!IMAPProtocolInterpreter methodsFor:'accessing'!

client

	^client
!

client: imapClient

	client := imapClient
!

commandPrefix: aString
	commandSequencer prefix: aString
!

connectionStream

	^connectionStream
!

responseStream

	^responseStream
! !

!IMAPProtocolInterpreter methodsFor:'connection'!

closeConnection
        |process|

        socket isNil
                ifFalse:
                        [
                        socket notNil ifTrue:[
                            socket close.
                        ].
                        socket := nil.                         " To prevent re-entry "
                        connectionStream notNil ifTrue:[
                            connectionStream close.
                            connectionStream := nil.
                        ].
                        responseStream notNil ifTrue:[
                            responseStream close.
                            responseStream := nil.
                        ].
                        process := responseReaderProcess.
                        process notNil ifTrue:[
                            process terminate.
                        ].
                        responseReaderProcess := nil.

                        " Response stream variable exists separately only to be able to set up test scripts, no need to do anything about it "
                        self connectionIsClosed]
!

closed
	^socket == nil
!

connectToHost: hostName port: portNumber

        self resetCommandSequence.
        socket := Socket newTCPclientToHost: hostName port: portNumber.
        socket lineEndCRLF.

        self responseStream:socket. 
        self connectionStream:socket. 

        self connectionIsReady.
        self getResponse.
!

disconnect
	self closeConnection
! !

!IMAPProtocolInterpreter methodsFor:'constants and defaults'!

defaultCommandPrefix
	^'imapv4_'
!

defaultResponseClass

	^ IMAPResponse
! !

!IMAPProtocolInterpreter methodsFor:'events'!

commandIsDone: command
    commandsInProgress remove: command ifAbsent: [^self].
    readResponseSignal wait
!

commandIsInProgress: command
	commandsInProgress addFirst: command.
	readResponseSignal signal
!

connectionIsClosed

	self liveAcrossSnapshot
		ifTrue: [ObjectMemory removeDependent: self].
!

connectionIsReady
	commandSequencer reset.
	self liveAcrossSnapshot
		ifTrue: [ObjectMemory addDependent: self].
!

isAnyCommandInProgress
	commandsInProgress isEmpty
!

isNoCommandInProgress
	"For testing purpose"

	^ commandsInProgress isEmpty
! !

!IMAPProtocolInterpreter methodsFor:'initialize-release'!

initialize

        connectionMutex := Semaphore forMutualExclusion.
        readResponseSignal := Semaphore new.
        commandsInProgress := List new: 4.
        commandSequencer := IMAPCommandSequencer newPrefix: self defaultCommandPrefix.
        responseReaderProcess := self responseReaderLoop fork.
!

release
	self closeConnection.
! !

!IMAPProtocolInterpreter methodsFor:'private'!

commandsInProgress

	^commandsInProgress
!

connectionStream: stream
" This is mostly for debugging purposes "

        Smalltalk isSmalltalkX ifTrue:[
            connectionStream := CrLfStream on:stream
        ] ifFalse:[
            connectionStream := stream
        ].
!

createSocket: hostName port: portNumber
        "Return a SocketAccessor instance or nil if a connection can not be made."

        ^ Socket newTCPclientToHost: hostName port: portNumber
!

nextCommandSequenceNumber

	^ commandSequencer next.
!

resetCommandSequence

	commandSequencer reset
!

responseReaderLoop
        ^[[[[readResponseSignal wait; signal.
                connectionMutex critical: [self handleNextResponse]
        ] repeat]
                on: Error
                do:
                        [:exc |
                        "An exception is being raised. If it is non-proceedable,
                        save it so that the Promise will raise it instead of returning a value."
                        self terminateCommandsWith: exc.
                        exc return]]
                on: Process terminateSignal
                do:
                        [:exc |
                        self terminateCommandsWith: ((Exception newException) setSignal: Exception; parameter: exc; errorString: nil).
                        exc return]]
!

responseStream: stream
" This is ONLY for debugging purposes "

        responseStream := stream
!

terminateCommandsWith: exception
	commandsInProgress do: [:command | command terminateWithException: exception]
! !

!IMAPProtocolInterpreter methodsFor:'public'!

executeCommand: aCommand
        aCommand sequenceID isNil
                ifTrue: [aCommand sequenceID: self nextCommandSequenceNumber].
        self class log: ['----------------------------------']  level: #IMAPClient.
        self class log: ['C: ' , aCommand printString] level: #IMAPClient.

        ^[ connectionMutex critical:    [ aCommand execute ]]
                on: OsInaccessibleError
                do:     [:ex |
                        self terminateCommandsWith: ex.
                        aCommand terminateWithException: ex.
                        self closeConnection.
                        NetClientError signalWith: #connection message: 'Server is not accessible'.
                        ex return: aCommand]
! !

!IMAPProtocolInterpreter methodsFor:'responses'!

getResponse
        | resp |
        ^ [     resp := self defaultResponseClass readFrom: self responseStream.
                self class log: ['S: ' , resp printLog] level: #IMAPServer.
                resp]
        on: OsInaccessibleError
        do:     [:ex |
                        self terminateCommandsWith: ex.
                        self closeConnection.
                        NetClientError signalWith: #connection message: 'Server is not accessible.']
!

handle: aResponse

	^self client handle: aResponse
!

handleNextResponse
	| resp |
	resp := self getResponse.
	self lookAtResponse: resp.
	commandsInProgress
			detect: [:command | command handle: resp]
			ifNone: [self handle: resp]
!

lookAtResponse: resp
	"Some responses must be recorded at all times. Insert logic here"
	self client handle: resp
! !

!IMAPProtocolInterpreter methodsFor:'sending tokens'!

argumentAsAssociation: argument
	(argument isKindOf: Association) ifTrue: [^argument].
	argument isNil ifTrue: [^'NIL'].
	argument isCharacters ifTrue: [^#string->argument].
	(argument isKindOf: Number) ifTrue: [^#number->argument].
	argument isSequenceable ifTrue: [^#parentthesizedList->argument].
	^argument.
!

literalStringAsBytes: string
        | output input char |
        output := (ByteArray new: string size) writeStream.
        input := string readStream.
        [input atEnd]
                whileFalse:
                        [char := input next.

                        char == Character cr 
                                ifTrue:
                                        [output nextPut: Character return asInteger.
                                         output nextPut: Character lf asInteger.
                                        ]
                                ifFalse:[char == Character return ifTrue:[
                                            output nextPut: Character return asInteger.
                                            output nextPut: Character lf asInteger.
                                            input peekFor: Character lf
                                         ] ifFalse:[
                                            output nextPut: char asInteger.]]].

        ^output contents
!

sendLiteralString: string
        | resp bytes continue |

        bytes := self literalStringAsBytes: string.
        IMAPScanner printLiteralStringLength: bytes on: self connectionStream.
        continue := true.

        [continue and: [|atEnd|
                        self responseStream blocking:false.
                        atEnd := self responseStream readWaitWithTimeoutMs:1000. "/ atEnd.
                        self responseStream blocking:true.
                        atEnd not]]
                whileTrue:
                        [
#stx.
                        resp := self responseStream upTo: Character return "cr" .

self responseStream peek == Character nl ifTrue:[
    self responseStream next
].
                        self class log: ['S: ' , resp] level: #IMAPClient.
                        resp first == $+ ifTrue: [
                                [self connectionStream binary.
                                IMAPScanner printLiteralStringContents: bytes on: self connectionStream stream.
                                continue := false]
                                        ensure: [self connectionStream text]]]
!

sendToken: token tokenType: tokenType
	tokenType = #literalString
		ifTrue: [self sendLiteralString: token]
		ifFalse: [IMAPScanner printToken: token tokenType: tokenType on: self connectionStream]
!

sendTokenList: listOfTokens
	| assoc |
	listOfTokens
		do:
			[:arg |
			assoc := self argumentAsAssociation: arg.
			self sendToken: assoc value tokenType: assoc key]
		separatedBy: [self connectionStream space]
! !

!IMAPProtocolInterpreter methodsFor:'updating'!

liveAcrossSnapshot
	^false
!

update: aSymbol
	"Dependents of ObjectMemory are sent update:
	#returnFromSnapshot when a snapshot is started."

	self liveAcrossSnapshot
		ifTrue:
			[aSymbol == #returnFromSnapshot
				ifTrue: [self closeConnection].
			"(aSymbol == #aboutToSnapshot or: [aSymbol == #aboutToQuit])
				ifTrue: [self closeConnection]."].

	super update: aSymbol
! !

!IMAPProtocolInterpreter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPProtocolInterpreter.st,v 1.7 2010/06/14 10:52:08 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPProtocolInterpreter.st,v 1.7 2010/06/14 10:52:08 cg Exp $'
! !
