"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#IMAPResponse
	instanceVariableNames:'source cmdName value parameters'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-IMAP'
!

IMAPResponse comment:'IMAPResponse is implemented to read and parse responses directly from the IMAP server socket stream.

Instance Variables:

	cmdName <String>        The name of a command
	value   <Collection>    The response value
	source  <Stream>        Row server response
	parameters      <Collection>     The parameters hold the fetch command message data and represented as a collection. For example: for the fetch command 1 FETCH (BODY[HEADER]) the response parameters are:
#(  ''BODY[HEADER]''  ''Date: Wed, 17 Jul 1996 02:23:25 -0700 (PDT)...'')

In order to adequately parse a response, response parser has to read up to three tokens. If the first response is a tag, the response is tagged status response and can be parsed immediately. In this case tag becomes response''s command name and the rest of response is parsed and placed in response value. Otherwise, second token is read. If this token starts with a letter, this second token becomes response''s command name, the remainder or response is deep tokenized and put in value. So for the following response
IMAPResponse readFrom: ''* FLAGS (\Seen \Answered \Deleted)'' readStream

cmdName = ''FLAGS'' and
value = #(''\Seen'' ''\Answered'' ''\Deleted'')

If the second token starts with a digit, the third token is read. This third token becomes response command name and the second token becomes reponse value. So for the following response:

IMAPResponse readFrom: ''* 3 EXPUNGE'' readStream

cmdName = ''EXPUNGE'' and
value = 3

Status responses as well as tagged responses may have bracketed status as in

''* OK [UIDVALIDITY 11349] UIDVALIDITY value''

(pain to handle because brackets syntactically are atom-chars). This bracketed expression is parsed into value. ''OK'' is stored into status field.

Every parsed response contains ''command name'' and response value. Command name is used to match a response to potentially multiple active commands. Command name is also useful for user agent when user agent needs information in specific response or set of responses. A standard situation is when user agent needs values of all responses whose command name is the same as the name of their command.


'
!


!IMAPResponse class methodsFor:'parsing, general'!

canParse: cmdName
	^false
!

defaultResponseClass
    ^IMAPResponse
!

parse: scanner

    | theToken theResponse |

    scanner sourceTrailOn.
    theToken := scanner nextToken.
"/    theToken isNil ifTrue: [ ^nil ].

    "
    IMAP Server responses are classified as either tagged or untagged.
    Untagged responses begin with either the asterisk or plus sign, while tagged responses begin with the command id.
    "

    theResponse := (#($* $+) includes: theToken)
        ifTrue: [ self parseUntagged: scanner withStar: theToken == $* ]
        ifFalse: [ self parseTagged: scanner withTag: theToken ].

    scanner upTo: Character nl.

    theResponse source: scanner sourceTrail.
    ^theResponse
!

parse: scanner with: cmdName
	^self new
			parse: scanner with: cmdName
!

parse: scanner with: cmdName with: code
	^self new
		parse: scanner
		with: cmdName
		with: code
!

parseContinuationResponse: scanner
    ^IMAPContinuationResponse new
!

parseTagged: scanner with: tag
	|   status |

	status := scanner nextToken.
	^(self parserTypeForTaggedStatus: status) parse: scanner tag: tag status: status.
!

parseTagged: scanner withTag: tag

    | status |

    status := scanner nextToken.

    ^(self parserTypeForTaggedStatus: status)
	parse: scanner
	tag: tag
	status: status
!

parseUntagged: scanner with: starOrPlus
	| token token2 |
	starOrPlus == $+ ifTrue: [^self parseContinuationResponse: scanner].
	token := scanner nextToken.
	^token first isAlphabetic
		ifTrue: [(self parserForUntaggedCommand: token)
						parse: scanner with: token]
		ifFalse:
			[token2 := scanner nextToken.
			(self parserForUntaggedCommand: token2)
						parse: scanner with: token2 with: token]
!

parseUntagged: scanner withStar: isStar

    | token token2 |

    "An untagged responses might be a continuation responses.
    These begin with the plus sign rather than the asterisk."

    isStar
	ifFalse: [ ^self parseContinuationResponse: scanner ].

    token := scanner nextToken.

    "At this point, we know the response is untagged, but IMAP's untagged responses are not well designed.
    Some responses provide numeric data first, response or condition name second, while others do it the other way around.
    What we are doing here is determining what order these things are in, and then doing the parsing accordingly."

    ^(token first isLetter)
	ifTrue: [
	    (self parserForUntaggedResponse: token)
		parse: scanner with: token
	    ]
	ifFalse: [
	    token2 := scanner nextToken.
	    (self parserForUntaggedResponse: token2)
		parse: scanner forCommandOrConditionNamed: token2 withValue: token
	    ]
!

parserForUntaggedCommand: cmdName
	^IMAPResponse allSubclasses
				detect: [:each | each canParse: cmdName ]
				ifNone: [self defaultResponseClass].
!

parserForUntaggedResponse: responseName

    | properSubclass |

    properSubclass := IMAPResponse allSubclasses
	detect: [ : each | each canParse: responseName ]
	ifNone: [ self defaultResponseClass ].

    ^properSubclass new
!

parserTypeForTaggedStatus: status
	^IMAPResponseTagged
!

readFrom: stream

    ^self parse: (self scannerOn: stream)
!

scannerOn: stream
    ^IMAPScanner on: stream
! !

!IMAPResponse methodsFor:'accessing'!

cmdName
    ^cmdName
!

cmdName: aString
    cmdName := aString
!

parameters
	^parameters
!

parameters: aCollection
	parameters := aCollection.
	value isNil ifTrue: [ value := parameters ].
!

source
	^source
!

source: aString
	source := aString
!

tag
    ^nil
!

value
    ^value
!

value: aValue
    value := aValue
! !

!IMAPResponse methodsFor:'parsing, general'!

parse: scanner
	" Nothing more to do here "
	self parameters: scanner deepTokenize
!

parse: scanner forCommandOrConditionNamed: commandOrConditionName withValue: codeValue

    self cmdName: commandOrConditionName.
    self value: codeValue.

    self parse: scanner
!

parse: scanner with: commandConditionOrStatusName

    self cmdName: commandConditionOrStatusName.

    self parse: scanner
!

parse: scanner with: cmdNameString with: codeValue

	self cmdName: cmdNameString.
	self value: codeValue.
	self parse: scanner
!

scanFrom: scanner

    self value: scanner deepTokenize
!

scanFrom: scanner forCommandOrConditionNamed: commandOrConditionName withValue: codeValue

    self cmdName: commandOrConditionName.
    self value: codeValue.

    self scanFrom: scanner
!

scanFrom: scanner with: commandConditionOrStatusName

    self cmdName: commandConditionOrStatusName.

    self scanFrom: scanner
! !

!IMAPResponse methodsFor:'printing'!

printLog

	^ self source
!

printOn: stream

	source notNil ifTrue: [ stream nextPutAll: source ]
! !

!IMAPResponse methodsFor:'testing'!

hasTag: aString
    ^false
!

isContinuationResponse
    ^false
!

isStatusResponse
    ^false
!

terminatesCommand: aCommand
	^false
! !

!IMAPResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPResponse.st,v 1.1 2004/11/12 10:46:13 penk Exp $'
! !
