"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MailScanner subclass:#IMAPScanner
	instanceVariableNames:''
	classVariableNames:'AtomMask TextMask QuotedTextMask QuotedPairChar'
	poolDictionaries:''
	category:'NetClients-IMAP'
!


!IMAPScanner class methodsFor:'character classification'!

atomSpecials
	" These characters cannot occur inside an atom"
	^'(){%*"\'
!

specials
	^self atomSpecials
! !

!IMAPScanner class methodsFor:'class initialization'!

initClassificationTable
	super initClassificationTable.
	self initClassificationTableWith: TextMask when:
		[:c | c ~~ Character return ].
	self initClassificationTableWith: AtomMask when:
		[:c | c > Character space and: [ (self atomSpecials includes: c) not] ].
	self initClassificationTableWith: QuotedTextMask when:
		[:c | c ~~ $" and: [ c ~~ $\ and: [ c ~~ Character return ]]].
!

initialize
	" IMAPScanner initialize "

	self initializeConstants; initClassificationTable
!

initializeConstants
	AtomMask := 256.
	QuotedTextMask := 4096.
	TextMask := 8192.
! !

!IMAPScanner class methodsFor:'printing'!

defaultTokenType
	^#string
!

printAtom: atom on: stream
	atom isNil
		ifTrue: [stream nextPutAll: 'NIL']
		ifFalse: [stream nextPutAll: atom ]
!

printIMAPString: value on: stream
	"Print string as either atom or quoted text"

	value isNil ifTrue: [ self printNilOn: stream].
	(self shouldBeQuoted: value)
		ifTrue: [self printQuotedText: value on: stream ]
		ifFalse: [self printAtom: value on: stream]
!

printLiteralString: aString on: stream
	"Note that this method is good for printing but not for sending.
	IMAP requires sender to send string length, then wait for continuation response"

	self printLiteralStringLength: aString on: stream.
	self printLiteralStringContents: aString on: stream
!

printLiteralStringContents: aString on: stream
	stream nextPutAll: aString asString
!

printLiteralStringLength: aString on: stream
	stream nextPut: ${.
	aString size printOn: stream.
	stream nextPut: $}.
	stream class == Net::CrLfStream ifTrue:[
	    stream nextPut:Character nl
	] ifFalse:[
	    stream nextPut:Character return; nextPut:Character nl
	]
!

printNilOn: stream
	stream nextPutAll: 'NIL'
!

printParenthesizedList: arrayOfAssociations on: stream
	"In order to accurately print parenthesized list, we need to know
	token types of every element. This is applied recursively"

	stream nextPut: $(.
	self printTokenList: arrayOfAssociations on: stream.
	stream nextPut: $)
!

printToken: value tokenType: aSymbol on: stream
	aSymbol = #string ifTrue: [^self printIMAPString: value on: stream].
	aSymbol = #literalString ifTrue: [^self printLiteralString: value on: stream].
	aSymbol = #atom ifTrue: [^self printAtom: value on: stream].
	aSymbol = #quotedText ifTrue: [^self printQuotedText: value on: stream].
	aSymbol = #nil ifTrue: [^self printNilOn: stream].
	aSymbol = #parenthesizedList ifTrue: [^self printParenthesizedList: value on: stream].  "Invalid token type"
	aSymbol = #special ifTrue: [^stream nextPut: value].
	self shouldImplement
!

stringAsAssociation: string
	(self shouldBeQuoted: string) ifFalse: [^#atom -> string].
	(string first == $\ and:
	[string size > 1 and:
	[self shouldBeQuoted: (string copyFrom: 2 to: string size) not]])
		ifTrue: [^#atom -> string].
	^#quotedText -> string
!

tokenAsAssociation: token
	(token isKindOf: Association) ifTrue: [^token].
	token isNil ifTrue: [^'NIL'].
	token isCharacters ifTrue: [^self stringAsAssociation: token].
	(token isKindOf: Number) ifTrue: [^#number -> token].
	token isSequenceable ifTrue: [^#parentthesizedList -> token].
	^token
! !

!IMAPScanner class methodsFor:'testing'!

isAtomChar: char
	^((self classificationTable at: char asInteger + 1) bitAnd: AtomMask) ~= 0
!

shouldBeQuoted: string
	^(string detect: [ :char | (self isAtomChar: char) not ] ifNone: [ nil ]) notNil
! !

!IMAPScanner class methodsFor:'utility, parsing'!

tokenizeBracketedGroup: string
" If argument contains bracketed group, break it into pre-group, group and post-group.
Answer nil if there is no bracketed group or 3-element array if there is one "

	| first last |
	first := string indexOf: $[.
	first = 0 ifTrue: [ ^nil ].
	last := string indexOf: $].
	^Array
		with: (string copyFrom: 1 to: first - 1)
		with: (string copyFrom: first + 1 to: last - 1)
		with: (string copyFrom: last + 1 to: string size).
! !

!IMAPScanner methodsFor:'multi-character scans'!

doSpecialScanProcessing
	"Hacks that require special handling of IMAP tokens go here.
	The most frustrating one for us was handling of message/mailbox flags that have format \<atom> as
	in \Seen. The problem is that $\ is not an atom-char, so these flags are tokenized as #( $\ 'Seen').
	We make heuristical decision here if current token is $\ immediately followed by a letter. We will
	then read next token and merge $\ and next token answering a string. This is ONLY applied inside a
	parenthesized list"

	(token == $\ and: [(self classificationMaskFor: self peek)
			anyMask: AlphabeticMask])
		ifTrue:
			[self nextToken.
			token := '\' , token.
			tokenType := #string]
!

scanAtom
	"atom = 1*<any CHAR except atom-specials (which includes atomSpecials, space and CTLs)>"

	token := self scanAtomToken.
	(token match: 'NIL')
		ifTrue:
			["RFC2060 defines NIL as a special atom type, atoms are not case-sensitive"
			token := nil.
			tokenType := #nil]
		ifFalse: [tokenType := #atom].
	^token
!

scanAtomToken
	"atom = 1*<any CHAR except atom-specials (which includes atomSpecials, space and CTLs)>"

	^self
		scanWhile: [self matchCharacterType: AtomMask]
		do:
			[:stream :char |
			stream nextPut: char.
			char == $[
					ifTrue: [stream nextPutAll: self xScanBrackets]]
!

scanLiteralText
	"<{> nnn <}> <CRLF> <nnn bytes>"

	| nbytes string |
	nbytes := self scanLiteralTextLength.
	string := self nextBytesAsString: nbytes.
	token := string copyReplaceAll: (String with: Character return with: Character lf) with: (String with: Character cr).
	tokenType := #literalString.
	^token
!

scanLiteralTextLength
	"<{> nnn <}> <CRLF>"

	" We are positioned at the first brace character "
	token := self scanToken: [ self matchCharacterType: DigitMask ] delimitedBy: '{}' notify: 'Malformed literal length'.
	self upTo: Character cr.
	^Integer readFrom: token readStream
!

scanParenthesizedList
	| stream |
	stream := (Array new: 4) writeStream.
	self mustMatch: $( notify: 'Parenthesized list should begin with ('.
	self deepTokenizeUntil: [token == $)]
		do: [self doSpecialScanProcessing. stream nextPut: token].
	token ~~ $) ifTrue: [self notify: 'Non-terminated parenthsized list'].
	token := stream contents.
	tokenType := #parenthesizedList.
	^token
!

scanParenthesizedListAsAssociation
	| stream |
	stream := (Array new: 4) writeStream.
	self mustMatch: $( notify: 'Parenthesized list should begin with ('.
	self deepTokenizeAsAssociationUntil: [token == $)]
		do: [:assoc | self doSpecialScanProcessing. stream nextPut: (tokenType->token)].
	token ~~ $) ifTrue: [self notify: 'Non-terminated parenthsized list'].
	token := stream contents.
	tokenType := #parenthesizedList.
	^tokenType->token
!

scanQuotedChar
	"Scan possible quoted character. If the current char is $\, read in next character and make it a quoted
	string character"

	(hereChar == $\)
		ifTrue:
			[self step.
			classificationMask := QuotedTextMask.
			^true]
		ifFalse: [^false]
!

scanQuotedText
" quoted-string = <""> *(quoted_char / quoted-pair) <"">
  quoted_char    =  <any CHAR except <""> and <\>"

	" We are positioned at the first double quote character "
	token := self scanToken: [ self scanQuotedChar; matchCharacterType: QuotedTextMask ] delimitedBy: '""' notify: 'Unmatched quoted text'.
	tokenType := #quotedText.
	^token.
!

scanText
	"RFC822: text = <Any CHAR, including bare CR & bare LF, but not including CRLF. This is a 'catchall' category and cannot be tokenized. Text is used only to read values of unstructured fields"

	^self
		skipWhiteSpace;
		scanWhile: [ (self matchCharacterType: CRLFMask) not ].
! !

!IMAPScanner methodsFor:'printing'!

printLiteralString: aString on: stream
	self class printLiteralStringLength: aString on: stream.
	self class printLiteralStringContents: aString on: stream.
! !

!IMAPScanner methodsFor:'private'!

nextBytesAsString: nbytes
	| str |
	^source isExternalStream
		ifTrue: [
			[self binary.
			str := (source next: nbytes) asString.
"/                        str := str copyReplaceString:(String crlf) withString:Character cr asString.
			self sourceTrailNextPutAll: str.
			str]
				ensure: [self text]]
		ifFalse: [super next: nbytes]
!

nextIMAPToken
	| char |
	self skipWhiteSpace.
	char := self peek.
	char isNil                              "end of input"
		ifTrue: [tokenType := #doIt.
			^token := nil].
	char == $" ifTrue: [^self scanQuotedText].
	char == ${ ifTrue: [^self scanLiteralText].
	((char < Character space) or: [self specials includes: char])
		ifTrue: [tokenType := #special.         " Special character. Make it token value and set token type "
				^token := self next.].
	(self matchCharacterType: AtomMask)
		ifTrue: [^self scanAtom].
	tokenType := #doIt.
	token := char.
	^token
!

xScanBrackets
	^(self scanUntil: [hereChar == $] ]), ']'.
! !

!IMAPScanner methodsFor:'tokenization'!

deepNextToken
	^self nextToken == $(
		ifTrue: [self stepBack; scanParenthesizedList]
		ifFalse: [token]
!

deepNextTokenAsAssociation
	^self nextToken == $(
		ifTrue: [self stepBack; scanParenthesizedListAsAssociation]
		ifFalse: [tokenType->token].
!

deepTokenize
	| stream |
	stream := (Array new: 4) writeStream.

	[self deepNextToken.
	tokenType = #doIt or: [token == Character cr ]]
		whileFalse: [stream nextPut: token].
	token == Character cr ifTrue: [ self stepBack ].
	^stream contents
!

deepTokenizeAsAssociation
	| stream assoc |
	stream := (Array new: 4) writeStream.

	[assoc := self deepNextTokenAsAssociation.
	assoc key = #doIt]
		whileFalse: [stream nextPut: assoc].
	^stream contents
!

deepTokenizeAsAssociationUntil: aBlock do: actionBlock
	| assoc |

	[self skipWhiteSpace.
	assoc := self deepNextTokenAsAssociation.
	assoc key = #doIt or: aBlock]
		whileFalse: [actionBlock value: assoc]
!

deepTokenizeUntil: aBlock do: actionBlock
	[self skipWhiteSpace; deepNextToken. tokenType == #doIt or: aBlock]
		whileFalse:
			[actionBlock value].
!

nextToken
	^self nextIMAPToken
!

specials
	^self class atomSpecials
! !

!IMAPScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__IMAPScanner.st,v 1.6 2004/11/18 10:15:40 cg Exp $'
! !

IMAPScanner initialize!
