"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MimeEntityBot subclass:#MailMessageBot
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MailMessageBot comment:'This class provides a ''robot'' to perform common tasks on mail messages. Mail message is a specialization
of generic rfc822 and Mime messages. A mail message has text and zero or more attachments. Mail message''s
text can have a number of different representations as follows:

-Simple message. This message has no attachments and its contents is text of the message. The message
should have text content type (''text/*''); its subtype defines text formatting (''plain'', ''rtf'', ''html'').
-Multipart message. This message has multipart type (''multipart/*''). Multipart message contains zero or more
parts. One of the subparts is the text part. It may be simple or multipart/alternative part.
Multipart may contain multiple alternative representations of message text. This is done by enclosing
all alternative representations into multipart/alternative part.
Mail message itself may be multipart/alternative. In this case the message has no attachments, just alternative
representations of its text. Otherwise, top-level message has type multipart/mixed. The first part of this message
is its text part. It is either simple text part, or multipart/alternative part.

This class provides methods to query and scan mail message for its text (part), attachments, etc. and to
perform common transformations (add/remove attachment, replace message text, mutate message from
multipart into single part and vice versa) in such a way to preserve message structural consistency. This
includes changing and/or mutating message body, fields and contents

Text query methods allow to specify a list of preferences. This is useful when client can display text in more
than one format. Preferences can be specified as array or desired formats i.e. #(''html'' ''plain'' ''*''). The default is #(''plain'' ''*'')
'
!


!MailMessageBot methodsFor:'private'!

bestMatchFrom: coll preferences: prefArray
	| sorted candidates |
	^coll isEmpty
		ifTrue: [nil]
		ifFalse:
			[(prefArray isNil or: [prefArray isEmpty]) ifTrue: [ ^coll first ].
			" Then select only those that match at least one condition "
			candidates := coll select: [ :candidate | (self indexOf: candidate subtype in: prefArray) > 0 ].
			candidates isEmpty ifTrue: [ ^nil ].
			sorted := (SortedCollection sortBlock:
				[:x :y | (self indexOf: x subtype in: prefArray) < (self indexOf: y subtype in: prefArray)])
				addAll: candidates;
				yourself.
				sorted first]
!

indexOf: aString in: seqColl

	1 to: seqColl size
		do: [:idx | ((seqColl at: idx) match: aString) ifTrue: [^idx]].
	^0
! !

!MailMessageBot methodsFor:'tasks'!

allTextPartsFor: aMailMessage
" Answered sequencable collection of all text parts in the message "
	^self findTextPartCandidatesFor: aMailMessage.
!

applyTransferEncodingTo: aMimeEntity

	(self attachmentsFor: aMimeEntity)
		do: [ :attachment | self prepareForTransport: attachment encodeAs: 'base64' ]
!

attachmentsFor: aMimeEntity
	| textPart attachments |
	textPart := self textPartFor: aMimeEntity preferences: nil.

	textPart parent isNil ifTrue: [ ^OrderedCollection new ].               " Simple entity -- no attachments "

	(textPart parent isMultipartAlternative) ifTrue: [textPart := textPart parent].
	textPart == aMimeEntity ifTrue: [ ^OrderedCollection new ].

	attachments := OrderedCollection new: 4.
	self
		for: aMimeEntity
		maxDepth: 2
		partsDo: [:part | (part ~~ textPart and: [part ~~ aMimeEntity])
				ifTrue: [attachments add: part]].
	^attachments
!

findMultipartAlternativeTextPartsFor: aMimeEntity
	"Answer all text subparts of specified entity "

	^aMimeEntity parts select: [:subpart | subpart isTextContentType].
!

findMultipartMixedTextPartsFor: aMimeEntity
	"If my body is multipart/mixed, the first text part is message text.
	Otherwise, the first multipart/alternative part contains alternative
	representations of message text"

	| part |
	part := aMimeEntity parts detect: [:subpart | subpart isTextContentType]
				ifNone: [nil].

	^part notNil
		ifTrue: [Array with: part]
		ifFalse:        " Try multipart/alternative now "
			[part := aMimeEntity parts detect: [:subpart | subpart isMultipartAlternative]
						ifNone: [^Array new].
			self findMultipartAlternativeTextPartsFor: part]
!

findTextPartCandidatesFor: aMimeEntity
	aMimeEntity isSimple ifTrue: [^Array with: aMimeEntity].
	^aMimeEntity isMultipartMixed
		ifTrue: [self findMultipartMixedTextPartsFor: aMimeEntity]
		ifFalse: [self findMultipartAlternativeTextPartsFor: aMimeEntity]
!

highLevelTextPartFor: aMimeEntity
" Answer part that either text part itself, or its parent if the part is inside multipart/alternative part "

	| textPart |

	textPart := self textPartFor: aMimeEntity preferences: #('*').
	^(textPart isNil or: [textPart parent isNil or: [textPart parent isMultipart not]])
		ifTrue: [textPart]
		ifFalse: [textPart parent]
!

prepareForTransport: aMimeEntity
	self checkStructure: aMimeEntity.
	self applyTransferEncodingTo: aMimeEntity.
!

prepareForTransport: aMimeEntity encodeAs: encodingName

	self for: aMimeEntity partsDo: [ :part |
		part hasContentTransferEncoding
			ifFalse: [ self applyTransferEncodingTo: part encodeAs: encodingName]].
	^aMimeEntity
!

replaceTextFor: aMimeEntity with: aString
" Replace text can result in one of three scenarios:
	- For simple part text is this part's contents, so all three of aMimeEntity, textPart and containingPart
		are the same. Simply replace part's contents with new text.
	- For multipart/mixed entity containg simple text part aMimeEntity and containingPart are the same.
		Simply replace contents of text part
	- For multipart/mixed entity containing multipart/alternative text part, containingTextPart is
		multipart/alternative part containing textPart. The transformation replaces this multipart/alternative
		part with simple text part "

	| partWithText partToReplace |

	(partWithText := self textPartFor: aMimeEntity preferences: #('*')) isNil
		ifTrue: [^self error: (#InvalidMessage << #net >> 'Invalid message')].

	" Replace part's text "
	partWithText contents: aString.
	(partWithText ~~ aMimeEntity and: [partWithText parent isMultipartAlternative])
		ifTrue:
			[" If this is first-level multipart message, replace its body with simple body,
			otherwise replace it in its parent with (simple) text part "

			partToReplace := partWithText parent.
			(aMimeEntity == partToReplace)
				ifTrue: [aMimeEntity replaceFrom: partWithText]
				ifFalse: [partToReplace parent replacePart: partToReplace with: partWithText ]]
!

textFor: aMimeEntity
" Answer default text representation for specified message "

	^self textFor: aMimeEntity preferences: #('plain' '*')
!

textFor: aMimeEntity preferences: anArray
" Answer mail message text according to the list of preferences "
	| entity |
	^(entity := self textPartFor: aMimeEntity preferences: anArray) isNil
		ifTrue: [ nil ]
		ifFalse: [ entity removeContentTransferEncoding; contents ]
!

textPartFor: aMimeEntity
" Answer default text part for specified message "

	^self textPartFor: aMimeEntity preferences: #('plain' '*')
!

textPartFor: aMimeEntity preferences: anArray
" Answer part containing message text according to the list of preferences. The list is
the form of #(bestPref nextBestPref ...). For example #('html' 'plain' '*') "

	| candidates |

	" First get all candidate text parts "
	candidates := self findTextPartCandidatesFor: aMimeEntity.
	^self bestMatchFrom: candidates preferences: anArray
! !

!MailMessageBot class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MailMessageBot.st,v 1.1 2004/11/12 10:43:44 penk Exp $'
! !
