"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#Mailbox
	instanceVariableNames:'user server lastMessage keepMail deleteMail deliveryQueue
		pollingProcess delaySemaphore sync settingsSync connection'
	classVariableNames:'DoLogging'
	poolDictionaries:''
	category:'NetClients-Mailbox'
!

Mailbox comment:'The Mailbox class and its subclasses provide a high-level interface that makes it simple to check mailboxes.

Instance Variables:

	user    <NetUser>       the mailbox user
	server  <HostSpec>      network server
	lastMessage     <Integer>       last message number
	keepMail        <Boolean>       keep mail indicatior
	deleteMail      <Boolean>       delete mail indicatoe
	deliveryQueue   <SharedQueue>  holds letter in transit
	pollingProcess  <Process>       retrieve new messages from the mail server
	delaySemaphore  <Semaphore>     provides synchronized communication between pollingProcess and main process
	sync    <RecursionLock> protects access to mailbox
	settingsSync    <RecursionLock> protects access to ''profile.smr'' file (mailbox settings)
	connection      <NetClient>      specific net client, currently supported IMAP and POP3'
!


!Mailbox class methodsFor:'initialization'!

initialize
    DoLogging := true.
! !

!Mailbox class methodsFor:'instance creation'!

new
        ^self basicNew initialize
!

startFetchMailProcessWithPriority: priority from: serverName for: user password: password to: deliveryQueue pollingInterval: pollTime

	| mailbox popUser |
	mailbox := self new.
	popUser := NetUser username: user password: password.
	mailbox
		user: popUser;
		server: serverName;
		deliveryQueue: deliveryQueue;
		loadSettings.
	^mailbox
		startPollingAt: (priority isNil ifTrue: [self defaultPollingPriority] ifFalse: [priority])
		every: pollTime
!

user: popUser
	| mailbox |
	mailbox := self new.
	^mailbox
		user: popUser;
		yourself
!

user: popUser server: server
	"Pop3Mailbox
		user: PopUser request
		server: (Dialog request: 'Pop Server:' initialAnswer: 'sulu.cincom.com')"

	| mailbox |
	mailbox := self new.
	^mailbox
		user: popUser;
		server: server;
		yourself
! !

!Mailbox class methodsFor:'constants'!

defaultPollingPriority
	^30
!

userClass
	^NetUser
! !

!Mailbox class methodsFor:'debugging'!

doLog
    ^ DoLogging ? false
!

log: aStringOrBlock
    self log: aStringOrBlock level: nil
!

log: aStringOrBlock level: aLevel
    Transcript showCR:aStringOrBlock value.
!

registerToDebug

    "/ LogEnvironment default addToDebug: self
! !

!Mailbox methodsFor:'accessing'!

connectedDo: aBlock
	| shouldDisconnect |

	(shouldDisconnect := connection isNil)
			ifTrue:
				[connection := self createConnection.].
	^aBlock ensure: [shouldDisconnect
			ifTrue:
				[self logoutAndDisconnect.
				]]
!

connection

	^connection
!

deleteMail
	^deleteMail isNil
		ifTrue: [deleteMail := true]
		ifFalse: [deleteMail]
!

deliveryQueue
	^deliveryQueue notNil
		ifTrue: [deliveryQueue]
		ifFalse: [ deliveryQueue := SharedQueue new]
!

deliveryQueue: sharedQueue
	deliveryQueue := sharedQueue
!

keepMail
	^keepMail isNil
		ifTrue: [keepMail := true]
		ifFalse: [keepMail]
!

lastMessage
	^lastMessage notNil
		ifTrue: [ lastMessage]
		ifFalse: [ lastMessage := 0]
!

lastMessage: aNumber
	lastMessage := aNumber
!

logout
	connection notNil
		ifTrue:
			[connection logout.
			"connection := nil" #stx.]
!

logoutAndDisconnect
	connection notNil
		ifTrue:
			["connection logout."
			connection disconnect.
			connection := nil]
!

server

	^server
!

server: aValue

	server := aValue
!

serverType
	self subclassResponsibility
!

terminate
	pollingProcess isNil ifFalse:
		[pollingProcess terminate.
		"We should find a way to clean up transactions in progress."]
!

user

	^user
!

user: aValue

	user := aValue
! !

!Mailbox methodsFor:'commands'!

allHeaders
	self subclassResponsibility
!

anyNewMail
	self subclassResponsibility
!

deleteMessageNumbers:aSequenceOfUids
	"selete messages from server"
	self subclassResponsibility
!

getMessage: number

	self subclassResponsibility
!

messageCount
	"How many messages are on the server?"
	self subclassResponsibility
!

pollForNewMail
	delaySemaphore ~~ nil ifTrue: [delaySemaphore signal]
! !

!Mailbox methodsFor:'debugging'!

debug: aStringOrBlock level: aLevel
    Transcript showCR:aStringOrBlock value.
!

log: aStringOrBlock level: aLevel
	self class log: aStringOrBlock level: aLevel
! !

!Mailbox methodsFor:'initialize-release'!

initialize
	sync := RecursionLock new.
	settingsSync := RecursionLock new.
! !

!Mailbox methodsFor:'mail delivery'!

startPollingAt: priority every: seconds
	pollingProcess isNil ifFalse: [pollingProcess terminate].
	pollingProcess := [
			[| delay |
			[self anyNewMail] whileTrue: [self deliverNewMail].
			delay := Delay forSeconds: seconds.
			delaySemaphore := delay delaySemaphore.
			delay wait] repeat]
				forkAt: priority
"
lastMessage := 0. self deliverAllMail.
"
! !

!Mailbox methodsFor:'private'!

clientSpecies
	self subclassResponsibility
!

createConnection

	(user isNil or: [server isNil]) ifTrue: [^nil].
	connection := self clientSpecies connectToHost: self server.
	[connection user: self user; login]
		on: NetClientError
		do: [:ex | | newUser |
			ex parameter == #connection ifTrue: [ ex pass].
			ex parameter == #login
				ifTrue:
					[(newUser := self requestNewUser)  isNil ifTrue: [ex pass].
					self user: newUser.
					ex retry]
				ifFalse: [ex pass]].
	^connection
!

deleteUids: aSequenceOfUids
	"Delete all the letters on the server.  Ignore non-existent ones"

	^sync critical:
		[(deliveryQueue isNil
		 or: [deliveryQueue isEmpty]) ifFalse:
			[self error: 'trying to delete letters when delivery queue is not empty.\This will mess up message numbers.' withCRs].
		 self connectedDo: [
			connection isNil ifTrue:
				[self class log: ('could not login to delete letters on <1s>' expandMacrosWith: server).
				 ^false].
			lastMessage := self deleteMessageNumbers: aSequenceOfUids.
			self saveSettings.
			true]]
!

deliver: aLetterInTransit
	|activeInstance|
	self deliveryQueue nextPut: aLetterInTransit.
	MailReader notNil ifTrue:[
		(activeInstance := MailReader activeInstance) ~~ nil ifTrue:
			[activeInstance checkStatus]]
!

loadSettings
	| data filename lines |
	settingsSync critical:
		[(filename := self settingsFilename asFilename) definitelyExists ifFalse: [ lastMessage := 0. ^self].
		data := filename contentsOfEntireFile].
	lines := data tokensBasedOn: $, .
	lastMessage := (Number readFrom: (lines at: 2) readStream).
!

notifyServerMessageRead: uid
	^nil
!

requestNewUser
        | username password cancel |
        username := self user username asValue.
        password := '' asValue.
        cancel := (DialogBox new)
                        setInitialGap;
                        addMessage: ('Your login failed for <1s>' expandMacrosWith: server) centered: true;
                        addGap;
                        addMessage: 'Please try again.' centered: true;
                        addGap;
                        addMessage: 'Username' textLine: username boundary: 0.4;
                        addGap;
                        addMessage: 'Password' textLine: password type: #password boundary: 0.4;
                        addGap;
                        addOK: [true];
                        openDialog;
                        cancel.
        cancel value ifTrue: [^nil].
        ^NetUser new
                username: username value;
                password: password value;
                yourself
!

safelyExecute: block
" Ensure there is a connection; execute a block and make sure response is OK.
Raise an appropriate exception if there is an error. Make sure connection is released
if we had to acquire it "

	^sync critical: [ self connectedDo: block.]
!

saveSettings
	self saveSettings: lastMessage
!

saveSettings: n
	settingsSync critical:
		[| output |output := self settingsFilename asFilename writeStream.
		[output
			nextPutAll: self server;
			nextPut: $, ;
			print: n] ensure: [output close]]
!

settingsFilename
	| basename |
	basename := self server, '.mailbox'.
	MailFile notNil ifTrue:[
	    ^ MailFile fileName: basename
	].
	^ basename asFilename
! !

!Mailbox class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Mailbox.st,v 1.6 2010/06/14 10:52:27 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__Mailbox.st,v 1.6 2010/06/14 10:52:27 cg Exp $'
! !

Mailbox initialize!
