"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

ScalarField subclass:#MailboxListField
	instanceVariableNames:'addresses'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MailboxListField comment:'This class represents field whose value is a list of mailboxes as defined by RFC822. Several field
types in RFC822 messages (from, cc, to, etc) are mailbox list fields. Mailbox specs allows to specify
address-spec as well as an alias. MIME allows to encode non-ASCII characters in the alias part.
Addresses can be mailbox or mail group addresses.

Value of this field is list of addresses (strings). Addresses can be read or set using messages:
	addresses
	addresses:

When setting addresses, caller specified a collection of strings. Each string can be a single address or a sequence of comma-separated addresses. Among other things, this handles non-ascii characters in the alias part of the address. Address strings are always parsed and canonicalized inside. This class uses NetworkAddressDescriptor class to parse and compose address strings.

Mailbox list examples:
	Gourmets:  Pompous Person <WhoZiWhatZit@Cordon-Bleu>,
		Childs@WGBH.Boston, Galloping Gourmet@
		ANT.Down-Under (Australian National Television),
		Cheapie@Discount-Liquors;,
       Cruisers:  Port@Portugal, Jones@SEA;,
	 Another@Somewhere.SomeOrg

The formal grammar is as follows:

     address     =  mailbox                      ; one addressee
		 /  group                        ; named list

     group       =  phrase ":" [#mailbox] ";"

     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec

     route-addr  =  "<" [route] addr-spec ">"

     route       =  1#("@" domain) ":"           ; path-relative

     addr-spec   =  local-part "@" domain        ; global address

     local-part  =  word *("." word)             ; uninterpreted
						 ; case-preserved

     domain      =  sub-domain *("." sub-domain)

     sub-domain  =  domain-ref / domain-literal

     domain-ref  =  atom                         ; symbolic reference

'
!


!MailboxListField class methodsFor:'parsing'!

fieldNames
	^#('from' 'to' 'reply-to' 'cc' 'bcc' 'resent-reply-to' 'resent-from' 'resent-to' 'resent-cc' 'resent-bcc' )
! !

!MailboxListField class methodsFor:'printing'!

addressSeparator
	^$,
!

printAddressList: collection on: aStream
	collection do: [:addr | aStream nextPutAll: addr]
		separatedBy: [aStream nextPut: self addressSeparator]
! !

!MailboxListField methodsFor:'accessing'!

addAddress: aString
	^self addAddresses: (Array with: aString)
!

addAddresses: aCollection
	addresses isNil ifTrue: [ addresses := OrderedCollection new].
	addresses add: aCollection.
	self value addAll: (self sanitize: aCollection)
!

address
	^self addresses isEmpty
		ifTrue: [nil]
		ifFalse: [self addresses first]
!

address: aString
" MailboxListField default address: 'Some person<person@acme.com>' "
	self addresses: (OrderedCollection with: aString)
!

addresses
	^addresses
!

addresses: aCollection
	 addresses := aCollection.
	self value: (self sanitize: aCollection)
!

addressesEncodedWith: aString
	^self addresses collect: [ :addressDesc | addressDesc stringWithEncoding: aString ]
!

values
" Multi-valued fields override this method "
	^self value
!

values: anArray
" Multi-valued fields override this method "
	self value: anArray
! !

!MailboxListField methodsFor:'composing'!

writeAddressesOn: aStream encoding: aString
	self class printAddressList: (self addressesEncodedWith: aString ) on: aStream
!

writeStructureOn: aStream encoding: aString
	self writeAddressesOn: aStream encoding: aString.
! !

!MailboxListField methodsFor:'initialization'!

initialize
	super initialize.
	value := OrderedCollection new.
! !

!MailboxListField methodsFor:'parsing'!

doParse: rfc822Stream
" HeaderField fromLine: 'To       :  George Jones <Group@Some-Reg.An-Org>,
		 Al.Neuman@MAD.Publisher' "
	| pos |

	pos := rfc822Stream position.
	addresses :=  [NetworkAddressDescriptor addressesFrom: rfc822Stream]
					on: Error
					do: [ :ex |
						Array with:
							(NetworkAddressDescriptor new
								comment: (rfc822Stream position: pos; nextLine);
								yourself) ].
	^(self sanitize: addresses) asOrderedCollection
! !

!MailboxListField methodsFor:'printing'!

printAddressesOn: aStream
	self class printAddressList: (self sanitize: self addresses) on: aStream
!

printStructureOn: aStream
	self printAddressesOn: aStream.
! !

!MailboxListField methodsFor:'private'!

sanitize: aCollection
" Make sure collection of addresses is comprised of strings "
	^aCollection collect: [:address | self sanitizeAddress: address]
!

sanitize: aCollection encoding: aString
" Make sure collection of addresses is comprised of strings "
	^aCollection collect: [:address | self sanitizeAddress: address encoding: aString ]
! !

!MailboxListField class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MailboxListField.st,v 1.1 2004/11/12 10:44:39 penk Exp $'
! !
