"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageHandler subclass:#MessageBuildHandler
	instanceVariableNames:'internalStream'
	classVariableNames:'RemoveContentTransferEncoding'
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessageBuildHandler comment:'This class is responsible for putting message together from pieces parsed by the message parser. Specifically, this class knows what type of entity to create based on the context (parent entity, content type, etc.) and what to do with message body.
The sequence of building message is as follows:
	Parser invokes startEntity:. Builder replies with an instance of message header. Message header provides class factory for header fields.
	Parser parser fields and invokes endHeader:. Builder constructs appropriate entity based on the header and answers this entity
	Parser parses entity''s body. If it is multipart entity, the process is nested for body parts

'
!


!MessageBuildHandler class methodsFor:'instance creation'!

build: aMimeEntity from: rfc822Scanner
	^self new build: aMimeEntity from: rfc822Scanner
!

buildHeaderFor: aMimeEntity from: rfc822Scanner
	^self new buildHeaderFor: aMimeEntity from: rfc822Scanner
!

parse: rfc822Scanner
	^self new parse: rfc822Scanner
!

parse: aMimeEntity from: rfc822Scanner
	^self new parse: aMimeEntity from: rfc822Scanner
!

parseHeader: rfc822Scanner
	^self new parseHeader: rfc822Scanner
!

parseHeaderFor: aMimeEntity from: rfc822Scanner
	^self new parseHeaderFor: aMimeEntity from: rfc822Scanner
!

read: entity from: aStream
	^self new read: entity from: aStream
!

readFrom: aStream
	^self new readFrom: aStream
!

readHeaderFor: aMimeEntity from: aStream
	^self new readHeaderFor: aMimeEntity from: aStream
!

readHeaderFrom: aStream
	^self new readHeaderFrom: aStream
! !

!MessageBuildHandler class methodsFor:'defaults'!

removeContentTransferEncoding
	^RemoveContentTransferEncoding isNil
		ifTrue: [ RemoveContentTransferEncoding := false ]
		ifFalse: [ RemoveContentTransferEncoding ]
!

removeContentTransferEncoding: aBoolean
	RemoveContentTransferEncoding := aBoolean
! !

!MessageBuildHandler methodsFor:'accessing'!

fieldFactory
" Answer field factory or default field factory if none is specified. Different protocols may have different field factories (i.e. Mime vs. HTTP) "

	^self context at: #fieldFactory ifAbsent: [ self defaultFieldFactory ]
!

fieldFactory: anObject
" Set field factory. Different protocols may have different field factories (i.e. Mime vs. HTTP). Field factory should respond to messages:
	readFrom:
	readNameFrom:
	scannerType
	fieldClassForName:
"
	self context at: #fieldFactory put: anObject
!

initialEntity
	^self context at: #initialEntity ifAbsent: [ nil ]
!

initialEntity: anObject
" Set initial entity. This is used if we start with a known entity to build. It will be assigned as root context's entity during the first entity callback "
	self context at: #initialEntity put: anObject
!

messageHeaderClass
" Answer message header class "

	^self context at: #messageHeaderClass ifAbsent: [ MessageHeader ]
!

messageHeaderClass: aClass
" Set message header class "

	self context at: #messageHeaderClass put: aClass
!

scannerType
	^self context at: #scannerType ifAbsent: [ self defaultScannerType ]
!

scannerType: aClass
	self context at: #scannerType put: aClass
!

useSourceStream
" This option directs builder to use stream segments pointing to the actual
source stream this message is built from. It is used mainly because of
compatibility with STAMP, so that we can change message flags stored as
extension fields directly in the folder file "

	^self context at: #useSourceStream ifAbsent: [ false ]
!

useSourceStream: aBoolean
	self context at: #useSourceStream put: aBoolean
! !

!MessageBuildHandler methodsFor:'action callbacks'!

endBody: aBody
" By now we have accumulated body octets.
Remove content transfer encoding and decode body source using current charset "

        (aBody isSimple  and: [self useSourceStream not])
                ifTrue:
                        [ self setBodySource: aBody.
Smalltalk isSmalltalkX ifFalse:[
                        aBody isBinary
                                ifFalse:
                                        [internalStream
                                                ifTrue: [ aBody source lineEndCR]
                                                ifFalse: [ aBody source lineEndCRLF]]]].

        ^true
!

endBodyPart: aMimeEntity
	self entity addPart: aMimeEntity
!

endHeader: aMessageHeader
	self entity isNil
		ifTrue: [ self createEntityFor: aMessageHeader]
		ifFalse: [self entity header: aMessageHeader].
	^self entity
!

startBody: aMessageBody

	(super startBody: aMessageBody)
		ifFalse: [^false].
	self privateStartBody: aMessageBody.
	^true
!

startBodyStream: aStream length: lengthOrNil
" Add specified number of bytes from input stream to our source stream.
This callback is executed once for mail messages; HTTP messages may
consist of multiple chunks, each chunk will result in calling this method.
Input stream octets should be interpreted using the current encoding of the
body. So the input stream is converted into octets; when the body has been
parsed, the bytes are encoded into encoded stream using propoer encoding.
This is done in endBody:.

Unfortunately, we cannot assume that input stream can be set into binary mode
because we want to be able to handle messages stored in internal streams
or in files (latter case we can't even rely on line end convention being CRLF).
So we read input stream as characters in ISO8859-1 (not us-ascii because
some malformed messages may containg non-us-ascii characters in the body. "

	| body counter |

	body := self entity body.
	self useSourceStream
		ifTrue: [counter := lengthOrNil notNil
							ifTrue: [aStream position + lengthOrNil - 1]
							ifFalse: [aStream size].
				body source: (aStream from: aStream position to: counter)]
		ifFalse:
			[self privateStartBody: body stream: aStream length: lengthOrNil]
!

startEntity: aMimeEntityOrNil
	self pushEntityContext: aMimeEntityOrNil.
	(aMimeEntityOrNil isNil and: [self isRootContext and: [self initialEntity notNil]])
		ifTrue: [self entity: self initialEntity].
	^true
!

startHeader: aMessageHeaderOrNil
	^aMessageHeaderOrNil isNil
		ifTrue: [self messageHeaderClass new]
		ifFalse: [aMessageHeaderOrNil]
! !

!MessageBuildHandler methodsFor:'defaults'!

defaultFieldFactory
	^HeaderField
!

defaultPartEntityType
	^MimeEntity
!

defaultRootEntityType
	^MimeEntity
!

defaultScannerType
	^MimeScanner
!

dispatcherClass
	^MessageParser
! !

!MessageBuildHandler methodsFor:'part factory'!

createEntityFor: aMessageHeader
" Add actual logic for message factory "

	| entity |

	entity := self entityType new
		header: aMessageHeader;
		yourself.
	self entity: entity.
	^entity
!

entityType
	^self isRootContext
		ifTrue: [ self defaultRootEntityType]
		ifFalse: [self defaultPartEntityType]
!

scannerOn: aStream
	^self scannerType on: aStream
! !

!MessageBuildHandler methodsFor:'private'!

handleStartBodyException: exception

	Transcript cr; show: ((#ErrorParsingMimeBody1s << #net >> 'Error parsing mime body: <1s>')
			expandMacrosWith: exception errorString); cr.
	ParsingSimpleBodyError new
		messageText: exception messageText;
		parameter: exception;
		raiseSignal
!

privateStartBody: aMessageBody

        (aMessageBody isSimple and: [self useSourceStream not])
                ifTrue:
                        [
                        Smalltalk isSmalltalkX ifTrue:[
                            aMessageBody source: (ByteArray new: 1024) writeStream.
                        ] ifFalse:[
                            aMessageBody source: ((ByteArray new: 1024) withEncoding: #ISO8859_1) writeStream.
                        ].
                        aMessageBody isBinary
                                ifTrue: [aMessageBody source binary]
                                ifFalse:
                                        [
                                         Smalltalk isSmalltalkX ifFalse:[
                                             internalStream
                                                ifTrue: [ aMessageBody source lineEndCR]
                                                ifFalse: [ aMessageBody source lineEndCRLF]]]]
!

privateStartBody: body stream: aStream length: lengthOrNil
" Copy the body (next counter bytes or to the end of the stream if nil) "
        | binary lineEndConvention counter c |

Smalltalk isSmalltalkX ifFalse:[
        binary := body isBinary.
        lineEndConvention := aStream lineEndConvention.
        binary
                ifTrue: [aStream binary]
                ifFalse: [aStream lineEndTransparent].
].
        counter := lengthOrNil.
        [
                [(counter == nil or: [counter > 0]) and: [aStream atEnd not]]
                        whileTrue:
                                [c := aStream next.
                                binary ifTrue: [c := c asInteger].
                                body source nextPut: c.
                                counter notNil ifTrue: [counter := counter - 1]].
                aStream text.
Smalltalk isSmalltalkX ifFalse:[
                aStream lineEndConvention: lineEndConvention.
].

        ]  on: Error
          do: [ :ex | self handleStartBodyException: ex ]
!

setBodySource: aBody
"Remove content transfer encoding and decode body source using current charset "

        | lineEndConvention |
        aBody source "isInternal stx" isExternalStream 
                ifTrue: [ lineEndConvention := aBody source lineEndConvention].

        (self class removeContentTransferEncoding
                and: [ aBody parent contentTransferEncoding ~~ nil
                                and: [#('base64' 'quoted-printable') includes: aBody parent contentTransferEncoding asLowercase]])
                                ifTrue:
                                        [aBody source: aBody source encodedContents asString readStream.
                                        aBody parent removeContentTransferEncoding]
                                ifFalse:
                                        [ 
                                            Smalltalk isSmalltalkX ifTrue:[
                                                aBody source:  (CharacterEncoder encodeString:aBody source contents from: aBody parent charset into:#unicode) readStream    
                                            ] ifFalse:[
                                                aBody source:  (aBody source encodedContents withEncoding: aBody parent charset ) readStream    
                                            ]
                                        ].

        lineEndConvention ~~ nil
                ifTrue: [ aBody source lineEndConvention:lineEndConvention].
! !

!MessageBuildHandler methodsFor:'tasks'!

build: aMimeEntity from: aStream
	^self parse: aMimeEntity from: (self scannerOn: aStream)
!

buildHeaderFor: aMimeEntity from: aStream
	^self parseHeaderFor: aMimeEntity from: (self scannerOn: aStream)
!

parse: rfc822Scanner
        self initializeEntityContext.
        internalStream := rfc822Scanner source "isInternal stx" isExternalStream not.
        ^self dispatcher parse: rfc822Scanner
!

parse: aMimeEntity from: rfc822Scanner
	^self
		initialEntity: aMimeEntity;
		parse: rfc822Scanner
!

parseBody: aMimeEntity from: rfc822Scanner
        self initializeEntityContext.
        self startEntity: aMimeEntity.
        internalStream := rfc822Scanner source "isInternal stx" isExternalStream not.
        ^self dispatcher parseBody: self entity body from: rfc822Scanner
!

parseHeader: rfc822Scanner
	^self headerOnly: true; parse: rfc822Scanner
!

parseHeaderFor: aMimeEntity from: rfc822Scanner
	^self
		initialEntity: aMimeEntity;
		parseHeader: rfc822Scanner
!

read: aMimeEntity from: aStream
	^self parse: aMimeEntity from: (self scannerOn: aStream)
!

readFrom: aStream
	^self parse: (self scannerOn: aStream)
!

readHeaderFor: aMimeEntity from: aStream
	^self parseHeaderFor: aMimeEntity from: (self scannerOn: aStream)
!

readHeaderFrom: aStream
	^self parseHeader: (self scannerOn: aStream)
! !

!MessageBuildHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageBuildHandler.st,v 1.3 2010/03/09 12:14:22 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageBuildHandler.st,v 1.3 2010/03/09 12:14:22 cg Exp $'
! !
