"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#MessageDispatcher
	instanceVariableNames:'handler'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessageDispatcher comment:'This class is an abstract superclass for all classes that traverse structure of RFC822, Mime, HTTP,
etc. messages. Subclasses perform tasks such as controlling building (parsing) of a message,
printing, searching, etc. Structure traversing is implemented via visitor pattern. Visitor pattern
(GoF, p 331-344) allows to separate concerns and tasks from the hierarchical message structure.
Thus, we can employ different strategies while building a message depending on the usage
context such as parsing only to a certain depth, caching data in memory vs. storing pointers
to real external streams, parsing external bodies as just text vs. actually accessing remote data
and others. Similarly, we can do some very sophisticated searches and data mining inside complex
messages, develop nice printing mechanisms, message transformation schemes (answers,
forwarding, etc).

Visitor implements method
	visit: element
		^element acceptVisit: self

Element in turns double-dispatches this message back to the visitor. Usually method invoked depends
on the nature of the element. This allows visitor to do different things as different elements are
visited. For example
	HeaderField>>acceptVisit: aVisitor
		aVisitor acceptHeaderField: self

All double-dispatch methods are im the category
	visiting -- double dispatching

Instance Variables:
'
!


!MessageDispatcher class methodsFor:'instance creation'!

handler: aMessageHandler
	^self new
		handler: aMessageHandler
!

new
        ^self basicNew initialize
! !

!MessageDispatcher methodsFor:'accessing'!

handler
	^handler
!

handler: aMessageHandler
	handler := aMessageHandler
! !

!MessageDispatcher methodsFor:'actions'!

action: aSymbol with: argument1
	^self handler notNil
		ifTrue: [self handler action: aSymbol with: argument1]
!

action: aSymbol with: argument1 with: argument2
	^self handler notNil
		ifTrue: [self handler action: aSymbol with: argument1 with: argument2]
! !

!MessageDispatcher methodsFor:'initialization'!

initialize
! !

!MessageDispatcher methodsFor:'visiting'!

visit: aMessageElement
	self visit: aMessageElement with: nil
!

visit: aMessageElement with: argument
	^aMessageElement notNil
		ifTrue: [aMessageElement acceptVisit: self with: argument]
		ifFalse: [nil]
! !

!MessageDispatcher methodsFor:'visiting -- double dispatching'!

acceptHeaderField: aHeaderField with: argument
	^aHeaderField
!

acceptMessageHeader: aMessageHeader with: argument
	[(self action: #startHeader: with: aMessageHeader)
		ifTrue: [aMessageHeader fieldsDo: [ :field | self visit: field ]]]
		ensure: [self action: #endHeader: with: aMessageHeader].
	^aMessageHeader
!

acceptMimeEntity: aMimeEntity  with: argument
	[(self action: #startEntity: with: aMimeEntity) ifTrue:
		[self visit: aMimeEntity header;
			visit: aMimeEntity body]]
		ensure: [self action: #endEntity: with: aMimeEntity].
	^aMimeEntity
!

acceptMultipartBody: aMultipartBody with: argument
	[(self action: #startBody: with: aMultipartBody)
		ifTrue:
			[aMultipartBody parts do:
				[ :part |
					[(self action: #startBodyPart: with: part)
						ifTrue: [self visit: part]]
				ensure: [self action: #endBodyPart: with: part]]]]
		ensure: [self action: #endBody: with: aMultipartBody].
	^aMultipartBody
!

acceptRFC822Entity: anRFC822Message with: argument
" Default behavior is the same as of RFC822 message's superclass "
	^self acceptMimeEntity: anRFC822Message
!

acceptSimpleBody: aSimpleBody  with: argument
	| aStream |
	[(self action: #startBody: with: aSimpleBody)
		ifTrue:
			[[ aStream := self action: #getSimpleBodyStream: with: aSimpleBody.
			self action: #startBodyStream: with: aStream]
				ensure:         [ aStream ~~nil ifTrue: [ aStream close ]]]]
		ensure: [self action: #endBody: with: aSimpleBody].
	^aSimpleBody
! !

!MessageDispatcher class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageDispatcher.st,v 1.2 2010/06/14 10:52:46 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageDispatcher.st,v 1.2 2010/06/14 10:52:46 cg Exp $'
! !
