"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageElement subclass:#MessageHeader
	instanceVariableNames:'registry'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessageHeader comment:'I represent header fields in rfc822 messages. RFC822 messages may have repeated fields, order of field
appearance is significant, and there are manadatory rules that certain fields should precede all other
fields. This class encapsulates all this logic

Fields are represented by an instance of OrderedMultiValuedDictionary to preserve order of field appearance.
Value at a key is always an ordered collection of values. Each occurrence of the field adds another
element to this collection. When a field is removed and its value is an empty collection, the whole
entry is removed.

Usually, when fields are parsed from a message, their sequence is already correct. If fields are added
dynamically, some fields may appear in the wrong order, so part of preparing a message for transport
is re-arranging fields to ensure correct sequence


'
!


!MessageHeader class methodsFor:'instance creation'!

newFor: aMimeEntity
	^self new parent: aMimeEntity
! !

!MessageHeader methodsFor:'accessing'!

allAt: name
	^self registry at: name asLowercase ifAbsent: [OrderedCollection new]
!

at: name
	^self at: name at: 1
!

at: name at: occurrenceIndex
	^self at: name at: occurrenceIndex ifAbsent: [ nil ]
!

at: name at: occurrenceIndex ifAbsent: aNiladicBlock
	| namedFields |

	namedFields := self registry at: name asLowercase ifAbsent: [^aNiladicBlock value ].
	^namedFields size >= occurrenceIndex
		ifTrue: [namedFields at: occurrenceIndex]
		ifFalse: [ aNiladicBlock value ]
!

at: name ifAbsent: aNiladicBlock
	^self at: name at: 1 ifAbsent: aNiladicBlock
!

at: name ifAbsentPut: aNiladicBlock
	^self at: name ifAbsent: [self at: name put: aNiladicBlock value]
!

at: name put: field
	field name: name.
	self
		removeKey: name;
		add: field.
	^field
!

fields
	^value
!

fields: aCollection
" Replace all fields with new set of fields "
	self initialize; addAll: aCollection
!

indexOf: field
	^self fields indexOf: field
!

parent: aMimeEntity
	super parent: aMimeEntity.
	self fieldsDo: [ :field | field parent: aMimeEntity ].
! !

!MessageHeader methodsFor:'adding'!

add: field
	self register: field.
	self fields add: field
!

add: field after: object
	self register: field.
	self fields add: field after: object
!

add: field afterIndex: occurrenceIndex
	self register: field.
	self fields add: field afterIndex: occurrenceIndex
!

add: field before: object
	self register: field.
	self fields add: field before: object
!

add: field beforeIndex: occurrenceIndex
	self register: field.
	self fields add: field beforeIndex: occurrenceIndex
!

addAll: collectionOfFields
	collectionOfFields do: [:field | self add: field]
! !

!MessageHeader methodsFor:'composing'!

writeOn: aStream
	self fields do: [:field | field writeOn: aStream]
! !

!MessageHeader methodsFor:'copying'!

postCopy
	| oldFields |

	oldFields := value.
	self initialize.
	oldFields do: [ :field | self add: field copy ]
! !

!MessageHeader methodsFor:'enumerating'!

fieldsDo: aMonadicBlock
	self fields do: aMonadicBlock
! !

!MessageHeader methodsFor:'initialization'!

initialize
	registry := Dictionary new: 10.
	value := OrderedCollection new: 10.
! !

!MessageHeader methodsFor:'printing'!

printOn: aStream indent: level
	aStream
		nextPutAll: self class name, ': ';
		nextPutAll: self registry keys printString;
		cr
! !

!MessageHeader methodsFor:'private'!

deregister: field
	"Remove field from registry"

	| namedFields |
	namedFields := self registry at: field name asLowercase ifAbsent: [^self].
	namedFields remove: field.
	namedFields isEmpty ifTrue: [self registry removeKey: field name asLowercase]
!

register: field
" Register field in field registry "
	field parent: self parent.
	(self registry at: field name asLowercase ifAbsentPut: [ OrderedCollection new ]) add: field
!

registry
	^registry
! !

!MessageHeader methodsFor:'removing'!

remove: field
	field notNil
		ifTrue:
			[self fields remove: field.
			self deregister: field]
!

removeAll: collectionOfFields
	collectionOfFields do: [ :field | self remove: field ]
!

removeAllAt: name
	self removeAll: (self allAt: name)
!

removeKey: fieldName
	self removeAll: (self allAt: fieldName)
! !

!MessageHeader methodsFor:'sorting'!

arrangeFieldsInOrder: anArrayOfFieldNames
" Make sure specified fields occur before all other fields in specified order "

	value := self fieldsInOrder: anArrayOfFieldNames
!

fieldsInOrder: anArrayOfFieldNames
" Answer fields sorted such that specified fields occur before all other fields"

	| definedFields |

	definedFields := OrderedCollection new: self fields size.
	anArrayOfFieldNames do: [ :fieldName | definedFields addAll: (self allAt: fieldName)].
	definedFields addAll: (self fields reject: [ :fieldx | definedFields includes: fieldx]).
	^definedFields
! !

!MessageHeader methodsFor:'visiting'!

acceptVisit: aVisitor with: argument
	^aVisitor acceptMessageHeader: self with: argument
! !

!MessageHeader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageHeader.st,v 1.1 2004/11/12 10:41:13 penk Exp $'
! !
