"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageDispatcher subclass:#MessageParser
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessageParser comment:'This class parses Mime entities. It is responsible for parsing elements of the entity and notifying MessageBuildHandler who actually contructs the message. This division of labor is designed so that different policies can be implemented on the builder level. For example, we may choose to process some parts of the message as they are decoded without waiting for the whole message to be fully parsed; we may choose to create different entities to represent parsed entity (MimeEntity vs. MailMessage) depending on what would we expect to do with the message, etc.
Being a subclass of MessageDispatcher, MessageParser ''walks'' message being parsed and executes callbacks to its message builder when it arrives at certain points in the message. It also expects certain data to come from the builder. Specifically, it expects the builder to answer an instance of message header for #startEntity: callback and tp answer an instance of MimeEntity in response to #endHeader: callback.

'
!


!MessageParser class methodsFor:'instance creation'!

readFrom: aStream
	^self readFrom: aStream builder: self defaultBuilder
!

readFrom: aStream builder: aMessageBuildHandler
	^self new
		handler: aMessageBuildHandler;
		readFrom: aStream
! !

!MessageParser class methodsFor:'builders'!

defaultBuilder
	^self mailBuilder
!

mailBuilder
	^MessageBuildHandler new
! !

!MessageParser methodsFor:'defaults'!

encoder
	^#UTF_8
! !

!MessageParser methodsFor:'parsing'!

parse: rfc822Scanner
        | header entity |
        self initialize.
        self action: #startEntity: with: nil.
        header := self action: #startHeader: with: nil.
        self parseHeader: header from: rfc822Scanner.
        entity := self action: #endHeader: with: header.
        entity source: rfc822Scanner asStream.
        self parseBody: entity body from: rfc822Scanner.
        ^self action: #endEntity: with: entity.
!

parseBody: body from: rfc822Scanner

        | outStream  |

        (self action: #startBody: with: body)
                ifTrue:
                        [rfc822Scanner source "isInternal stx" isExternalStream
                                ifFalse:
                                        [
                                            |rawString decodedString|

                                            rawString := rfc822Scanner source upToEnd.
                                            Smalltalk isSmalltalkX ifTrue:[
                                                rawString := rawString copyReplaceString:(String crlf) withString:Character cr asString.
                                                decodedString := CharacterEncoder encodeString:rawString from:self encoder into:#unicode.
                                                "/ rawString size > 0 ifTrue:[self halt.].
                                                rfc822Scanner source:(decodedString readStream)
                                            ] ifFalse:[
                                                outStream := rawString asByteArrayEncoding: self encoder.
                                                rfc822Scanner source:  (outStream withEncoding: self encoder ) readStream.
                                                rfc822Scanner source lineEndCR 
                                            ].
                                        ].
                        "After parsing header fields the current position is looking at LF, skip it"
                        Smalltalk isSmalltalkX ifFalse:[
                            rfc822Scanner peekFor: Character lf.
                        ].
                        body isSimple
                                ifTrue: [ self parseSimpleBody: body from: rfc822Scanner]
                                ifFalse:        [self parseMultipartBody: body from: rfc822Scanner].
                        self action: #endBody: with: body].
        ^body
!

parseHeader: header from: rfc822Scanner
" Parse all fields. Add fields to specified header "
	| field factory newEndPosition lastEndPosition |
	lastEndPosition := 1.
	newEndPosition := 0.
	factory := self handler fieldFactory.
	[(rfc822Scanner atEnd
		or: [(rfc822Scanner peekFor: Character cr) or: [rfc822Scanner peekFor: Character lf]])
				or:[ lastEndPosition == newEndPosition]]
		whileFalse:
			[lastEndPosition := rfc822Scanner position.
			self
				scanFieldFrom: rfc822Scanner
				do:     [:fieldStream |
					(field := factory readFrom: fieldStream) notNil
						ifTrue:
							[(field canMerge and: [(header at: field name) notNil])
								ifTrue: [ (header at: field name) add: field]
								ifFalse: [ header add: field]]].
				newEndPosition := rfc822Scanner position. ].
	^header
!

parseMultipartBody: aMultipartBody from: rfc82Scanner
	" Build multipart body recursively "

	aMultipartBody
		scanPartsFrom: rfc82Scanner
		useSourceStream: self useSourceStream
		do: [ :sourceStream | | part |
			self action: #startBodyPart: with: nil.
			part := self parse: (self scannerOn: sourceStream).
			self action: #endBodyPart: with: part.
			part].
!

parseSimpleBody: body from: rfc822Scanner
	self action: #startBodyStream:length: with: rfc822Scanner asStream with: body contentLength
!

scannerOn: aStream
	^self handler scannerOn: aStream
! !

!MessageParser methodsFor:'private'!

scanFieldFrom: rfc822Scanner do: aBlock
        |  startPos endPos  fieldString fieldStream |

        " Scan to the end of the field and stop before terminating CrLf "
        startPos := rfc822Scanner position.
        fieldString := rfc822Scanner nextLineWithoutTerminator.
        endPos := rfc822Scanner position.
        fieldStream := self useSourceStream
                                        ifTrue: [ rfc822Scanner asStream from: startPos to: endPos ]
                                        ifFalse: [ fieldString readStream ].

        " Parse field using stream segment "
        aBlock value: fieldStream.

        rfc822Scanner
                position: endPos;
                peekFor: Character return;
                peekFor: Character lf.                  " Skip through CrLf "
! !

!MessageParser methodsFor:'tasks'!

parseEntity: aMimeEntity from: rfc822Scanner
	| header entity body |

	[self initialize.
	self action: #startEntity: with: aMimeEntity.
	header := aMimeEntity isNil
		ifTrue: [MessageHeader new]
		ifFalse: [aMimeEntity header].
	self visit: header with: rfc822Scanner.
	entity := self handler entity.
	body := self visit: entity body with: rfc822Scanner]
		ensure: [rfc822Scanner close].
!

readFrom: aStream
	^self parse: (self scannerOn: aStream)
! !

!MessageParser methodsFor:'testing'!

useSourceStream
	^self handler useSourceStream
! !

!MessageParser methodsFor:'visiting -- double dispatching'!

acceptMessageHeader: aMessageHeader with: rfc822Scanner
	[(self action: #startHeader: with: aMessageHeader)
		ifTrue: [self parseHeader: aMessageHeader from: rfc822Scanner]]
		ensure: [self action: #endHeader: with: aMessageHeader].
	^aMessageHeader
! !

!MessageParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageParser.st,v 1.5 2010/03/09 12:09:42 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessageParser.st,v 1.5 2010/03/09 12:09:42 cg Exp $'
! !
