"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageHandler subclass:#MessagePrintHandler
	instanceVariableNames:'stream'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MessagePrintHandler comment:'This class implements printing and writing messages on a stream. The message is rendered according to rfc822 syntax rules.
	writeOn: writes message on output stream in full conformance of rfc822 syntax, so it can be used to send message directly to SMTP server without creating intermediate streams. If some constituent entities require content transfer encoding, it is applied dynamically and transfer-encoded output is written directly on output stream with no intermediate buffers created. Header fields are arranged in the order imposed by rfc822.
	printOn: is used for printing, so the logic is slightly different to enhance readability of the output. First, nested entities are indented according to their nesting level. Secondly, binary data is not shown and in general on a digest of each entity''s contents is displayed.

Instance Variables:

	stream  <Stream>        output stream'
!


!MessagePrintHandler class methodsFor:'instance creation'!

on: aStream
	^self new
		setStream: aStream
!

print: message on: aStream
	^self printMessage: message on: aStream
!

print: message on: aStream indent: level
	^self printMessage: message on: aStream indent: level
!

printMessage: message on: aStream
	^self new printMessage: message on: aStream
!

write: message on: aStream
	^self writeMessage: message on: aStream
!

writeHeaderFor: message on: aStream
	^self writeMessageHeaderFor: message on: aStream
!

writeMessage: message on: aStream
	^self new writeMessage: message on: aStream
!

writeMessageHeaderFor: message on: aStream
	^self new writeMessageHeaderFor: message on: aStream
! !

!MessagePrintHandler methodsFor:'accessing'!

digestMode
	^self context at: #digestMode ifAbsent: [false]
!

digestMode: aBoolean
	self context at: #digestMode put: aBoolean
!

fieldStream
" field stream is used to convert field values into streams. field stream is reused for performance reasons "
	^self context at: #fieldStream ifAbsentPut: [(String new: 256) writeStream]
!

headerFieldOrder
" This method answers a collection of field names that must appear in  the given order before all other fields. This is used to impose correct field sequence for rfc822 messages because rfc822 postulates certain order of fields "
	^self entityContext at: #headerFieldOrder ifAbsent: [ OrderedCollection new ]
!

headerFieldOrder: anArrayOrFieldNames
	^self entityContext at: #headerFieldOrder put: anArrayOrFieldNames
!

indentLines
	^self context at: #indentLines ifAbsent: [false]
!

indentLines: aBoolean
	self context at: #indentLines put: aBoolean
!

indentation
	^self context at: #indentation ifAbsent: [0]
!

indentation: anInteger
	^self context at: #indentation put: anInteger
!

maxLineLength
	^self context at: #maxLineLength ifAbsent: [1024]
!

maxLineLength: anInteger
	self context at: #maxLineLength put: anInteger
!

preferFieldSource
" Answers true if field's source is preferred to its parsed values, false otherwise. If application resends, routes of forwards messages, all mail protocols strongly advise against and changes in message header fields. These changes are unavoidable if all fields are to be parsed and later composed when a message is resent/forwarded. In this situation a wise approach is to prefer source, so fields are copied to the output exactly the way thay have been received "

	^self context at: #preferFieldSource ifAbsent: [false]
!

preferFieldSource: aBoolean
	self context at: #preferFieldSource put: false
!

setStream: aStream
	stream := aStream
!

stream
	^stream
!

transferEncodingType
" Answer method of encoding for the body of this entity or nil if no transfer encoding is necessary "

	^self entityContext at: #transferEncodingType ifAbsent: [ nil ]
!

transferEncodingType: aSymbol
" Answer method of encoding for the body of this entity or nil if no transfer encoding is necessary "

	^self entityContext at: #transferEncodingType put: aSymbol
!

transferFields
	^self entityContext at: #transferFields ifAbsentPut: [ OrderedCollection new ]
!

transferFields: aCollection
	self entityContext at: #transferFields put: aCollection
! !

!MessagePrintHandler methodsFor:'action callbacks'!

endBody: aMessageBody
" Print conclusion if any "
" Reset line end convention for the stream for the boundary delimiter if it is multipart body "
        stream "isInternal stx" isExternalStream ifTrue: [ stream lineEndCRLF ].
        self printConclusion: aMessageBody
!

endBodyPart: aMimeEntity
	self cr
!

endHeader: aMessageHeader
	"Print header fields"

	[self
		setBoundary: self entity;
		headerFieldOrder: self entity transferFieldOrder;
		setTransferContext: self entity;
		printHeader: aMessageHeader
	]
		ensure: [self cr].
	^super endHeader: aMessageHeader
!

getSimpleBodyStream: aSimpleBody

	aSimpleBody source == nil  ifTrue: [^String new readStream].

	^(self digestMode
		ifTrue: [ aSimpleBody source ]
		ifFalse: [aSimpleBody valueAsStream ]) reset
!

startBody: aMessageBody

	^(super startBody: aMessageBody)
		ifTrue: [self printPreambule: aMessageBody]
		ifFalse: [false]
!

startBodyPart: aMimeEntity
	self printLine: '--' ,  self entity boundary.
	^true
!

startBodyStream: aStream
	[self digestMode
		ifTrue: [ self printDigestOf: aStream ]
		ifFalse: [self printBodyStream: aStream]]
		ensure: [aStream close].
! !

!MessagePrintHandler methodsFor:'printing utilities'!

cr
	self stream cr
!

printBodyStream: aStream
	self encode: aStream as: self transferEncodingType
!

printLine: aString

	self tab.
	self stream nextPutAll: aString.
	self cr
!

printPreambule: aMessageBody
" Print preamble if any "
	| aStream |
	(aStream := aMessageBody preamble) notNil
			ifTrue: [ aStream linesDo: [:line | self printLine: line ]].
	^true
!

tab
	self indentLines ifTrue: [ self stream tab: self level + self indentation ]
! !

!MessagePrintHandler methodsFor:'private'!

contentTransferEncodingFor: aMimeEntity
	^'base64'
!

defaultBoundaryFor: aMimeEntity
" Compute default boundary for specified Mime entity. Boundary is a string that
could not occur inside any of the parts. For now just use constant plus a random number "

	^'=_STAMPed_MAIL', Random new next printString, '_='
!

encode: aStream as: encodingName
	encodingName isNil
		ifTrue: [[aStream atEnd] whileFalse: [stream nextPut: aStream next]].
	encodingName = 'base64'
		ifTrue: [MimeScanner encode: aStream asBase64To: stream].
	encodingName = 'quoted-printable'
		ifTrue: [MimeScanner encode: aStream asQuotedPrintableTo: stream].
!

fieldValueAsStreamFor: aHeaderField
	^(self preferFieldSource and: [aHeaderField source notNil])
		ifTrue: [aHeaderField source]
		ifFalse:
			[self digestMode
				ifTrue: [ aHeaderField printOn: self fieldStream reset ]
				ifFalse: [ aHeaderField writeOn: self fieldStream reset ].
				self fieldStream contents readStream]
!

printConclusion: aMessageBody
	| aStream |
	(aStream := aMessageBody conclusion) notNil
		ifTrue: [aStream linesDo: [:line | self printLine: line ]].
!

printDigestOf: aStream
        " Print short digest from source stream onto aStream "
        | contents binary fname |

        aStream  isWritable ifTrue: [^nil].
        fname := self entity fileName.
        binary := self entity isBinaryContentType or: [(aStream respondsTo: #isBinary) and: [aStream isBinary]].

        fname notNil
                ifTrue: [self printLine: '------ from ', fname, ' (', aStream size printString, ' bytes) -----']
                ifFalse: [binary ifTrue: [self printLine: '------ (', aStream size printString, ' bytes) -----']].
        binary
                ifFalse: [contents := (aStream nextAvailable: 512).
                                aStream atEnd ifFalse: [contents := contents, '...'].
                                contents readStream linesDo: [ :line | self printLine: line ]].
        (fname notNil and: [binary not]) ifTrue: [self printLine: '---------------------------------------------------']
!

printHeader: aMessageHeader
" Print header fields "
	| fields |

	fields := (aMessageHeader fieldsInOrder: self headerFieldOrder)
				addAll: self transferFields;
				yourself.
	fields do: [ :field | self printHeaderField: field ]
!

printHeaderField: aHeaderField
" Get string representation of the field, make sure all lines are withing max line length and write all this on output stream. Lines may be folded to fit into max length (cr followed by tab is inserted). Field itself may fold lines, so field's representation is first broken into lines and then each line is folded if necessary. a cr is written at the end even if the field's source did not have cr "

	(self fieldValueAsStreamFor: aHeaderField)
		linesDo: [ :line | self printLine: line]
!

setBoundary: aMimeEntity
	aMimeEntity needsBoundary
		ifTrue: [aMimeEntity boundary: (self defaultBoundaryFor: aMimeEntity)].
!

setTransferContext: aMimeEntity
" Set transfer encoding context for this entity. Transfer encoding logic may depend on the protocol domain (e-mail uses content-transfer-encoding, HTTP uses transfer-codings). We want to avoid producing intermediate results, specifically applying transfer encoding as a separate step. So we collect all the fields inside and apply transfer encoding directly on output stream.
This method should be rewritten to use transfer encoding policies, then it will be truly usable across different protocols "

	| contentTransferEncoding |

	(self digestMode not and: [aMimeEntity needsContentTransferEncoding])
		ifTrue:
			[contentTransferEncoding := self contentTransferEncodingFor: aMimeEntity.
			self transferFields add: (ScalarField contentTransferEncoding: contentTransferEncoding).
			self transferEncodingType: contentTransferEncoding]
! !

!MessagePrintHandler methodsFor:'tasks'!

printMessage: aMimeEntity on: aStream
	self printMessageElement: aMimeEntity on: aStream indent: 1
!

printMessage: aMimeEntity on: aStream indent: level

	^self printMessageElement: aMimeEntity on: aStream indent: level
!

printMessageElement: aMessageElement on: aStream
	^self printMessageElement: aMessageElement on: aStream indent: 0
!

printMessageElement: aMessageElement on: aStream indent: level
	self
		digestMode: true;
		indentLines: true;
		indentation: level;
		setStream: aStream;
		initializeEntityContext.

	aMessageElement parent notNil
		ifTrue: [ self pushEntityContext: aMessageElement parent ].
	self visit: aMessageElement
!

writeMessage: aMimeEntity on: aStream
	self
		digestMode: false;
		indentLines: false;
		setStream: aStream;
		initializeEntityContext;
		visit: aMimeEntity
!

writeMessageElement: aMessageElement on: aStream
	self
		digestMode: false;
		indentLines: false;
		setStream: aStream;
		initializeEntityContext;
		entity: aMessageElement parent.

	aMessageElement parent notNil
		ifTrue: [ self pushEntityContext: aMessageElement parent ].
	self visit: aMessageElement
!

writeMessageHeaderFor: aMimeEntity on: aStream
	self
		headerOnly: true;
		writeMessage: aMimeEntity on: aStream
! !

!MessagePrintHandler class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessagePrintHandler.st,v 1.4 2010/03/09 12:14:20 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MessagePrintHandler.st,v 1.4 2010/03/09 12:14:20 cg Exp $'
! !
