"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageElement subclass:#MimeEntity
	instanceVariableNames:'header builder headerCharset'
	classVariableNames:'EditorRegistry'
	poolDictionaries:''
	category:'NetClients-MIME'
!

MimeEntity comment:'I represent Mime entity as defined by RFC2045. Mime entities form a basis for mail messages and HTTP. A Mime entity is basically and object that has header and a body. Header consists of fields each of which has a name and (potentially complex) value. A body can be either a simple or multipart body. Simple body has contents, which is either a string, or byte array. Multipart body consists of multiple parts, each of which is a Mime entity in itself.
Header fields describe entity contents or provide additional information. Header fields are represented by hierarchy of classes starting at HeaderFields. Some fields have known structure and syntax, the rest are considered to be just a text field. Mime defines a few standard fields, they all start with content-*. The most important field is content-type field. It defines type of information that entity represents (see rfc2045 for details). MimeEntity provides methods to access/set aspects of content-type field:
	type            (default ''text'')
	subtype         (''plain'')
	contentType (=type/subtype      default ''text/plain'')
	charset         (default charset from parent entity (RFC822Message/HttpEntity) or if the parent is nil - ''iso-8859-1'')
	boundary        (default nil, should be specified for multipart entities)
	headerCharset (default nil, is used to encode header fields that have non ASCII characters)
Other notable fields include content-length (contentLength) and content-disposition (fileName).
Fields may occur more than once and sometimes have to occur in particular sequence. MimeEntity allows multiple fields with the same name and remembers field order. There are various methods that allow to manipulate fields.

Methods are provide to access and travers entity body and parts. Mime entity allows to add and remove parts. An entity is created as a simple entity by default. As parts are added, it will mutate itself into multipart entity transparently. It is also possible to convert multiupart entity back into a single-part one, merging fields in the process.

MimeEntity also supports Visitor patterns. Nodes can be traversed by an outside visitor. This allows to put domain- and application-specific logic into the visitor and create visitors specific to Mail, HTTP, etc. See hierarchy of MessageVisitor

Instance Variables:
	header  <MessageHeader>  Message header contains header fields
	builder <MessageBuilder>  Message builder -- used to construct message and perform tasks on the message


Shared Variables:
	EditorRegistry  <Dictionary key: String value: ApplicationModel>  Viewers keyed by content-type
'
!

!MimeEntity class methodsFor:'documentation'!

syntaxOfMultiPartMimeBodies
" From RFC 2046: Media Types                  November 1996

   The Content-Type field for multipart entities requires one parameter,
   'boundary'. The boundary delimiter line is then defined as a line
   consisting entirely of two hyphen characters ($-, decimal value 45)
   followed by the boundary parameter value from the Content-Type header
   field, optional linear whitespace, and a terminating CRLF.

   WARNING TO IMPLEMENTORS:  The grammar for parameters on the Content-
   type field is such that it is often necessary to enclose the boundary
   parameter values in quotes on the Content-type line.  This is not
   always necessary, but never hurts. Implementors should be sure to
   study the grammar carefully in order to avoid producing invalid
   Content-type fields.  Thus, a typical 'multipart' Content-Type header
   field might look like this:

     Content-Type: multipart/mixed; boundary=gc0p4Jq0M2Yt08j34c0p

   But the following is not valid:

     Content-Type: multipart/mixed; boundary=gc0pJq0M:08jU534c0p

   (because of the colon) and must instead be represented as

     Content-Type: multipart/mixed; boundary=""gc0pJq0M:08jU534c0p""

   This Content-Type value indicates that the content consists of one or
   more parts, each with a structure that is syntactically identical to
   an RFC 822 message, except that the header area is allowed to be
   completely empty, and that the parts are each preceded by the line

     --gc0pJq0M:08jU534c0p

   The boundary delimiter MUST occur at the beginning of a line, i.e.,
   following a CRLF, and the initial CRLF is considered to be attached
   to the boundary delimiter line rather than part of the preceding
   part.  The boundary may be followed by zero or more characters of
   linear whitespace. It is then terminated by either another CRLF and
   the header fields for the next part, or by two CRLFs, in which case
   there are no header fields for the next part.  If no Content-Type
   field is present it is assumed to be 'message/rfc822' in a
   'multipart/digest' and 'text/plain' otherwise.

   NOTE:  The CRLF preceding the boundary delimiter line is conceptually
   attached to the boundary so that it is possible to have a part that
   does not end with a CRLF (line  break).  Body parts that must be
   considered to end with line breaks, therefore, must have two CRLFs
   preceding the boundary delimiter line, the first of which is part of
   the preceding body part, and the second of which is part of the
   encapsulation boundary.
"
! !

!MimeEntity class methodsFor:'instance creation'!

fileName: fileNameOrString
	" self fileName: 'ftp.st' "
	| stream |
	stream := fileNameOrString asFilename readStream lineEndTransparent.
	^[self source: stream.] ensure: [ stream close ]
!

mimeTypeForFile: fileName
	^MimeTypeDescriptor guessFromExtension: fileName asFilename extension
!

newTextHTML
	" self newTextHTML "
	^self new
		contentType: MimeTypeDescriptor contentTypeHtml;
		charset: self defaultCharset;
		yourself.
!

newTextPlain
	" self newTextPlain "
	^self new
		contentType: MimeTypeDescriptor contentTypePlainText;
		charset: self  defaultCharset;
		yourself.
!

source: aStream
" Construct new simple Mime entity conaining specified stream
Example:
	self source: 'ftp.st' asFilename readStream lineEndTransparent "

	^self stream: aStream
!

stream: aStream
" Construct new simple Mime entity conaining specified stream
Example:
	self stream: 'ftp.st' asFilename readStream lineEndTransparent "

	| entity |
	entity := self new.
	(aStream respondsTo: #fileName)
		ifTrue: [entity fileName: aStream fileName tail;
					contentType: (self mimeTypeForFile: aStream fileName)].
	^entity
		body: (SimpleBody new source: aStream);
		yourself
! !

!MimeEntity class methodsFor:'class initialization'!

initialize
	"MimeEntity initialize"

	(EditorRegistry := Dictionary new)
		at: 'text/plain' put: SinglePartPresenter.
"               at: 'text/html' put:    "
! !

!MimeEntity class methodsFor:'constants'!

contentLengthFieldName
	^'content-length'
!

contentTypeFieldName
	^'content-type'
!

defaultBuilderType
	^MessageBuildHandler
!

defaultPrinterType
	^MessagePrintHandler
! !

!MimeEntity class methodsFor:'defaults'!

defaultCharset

	^MimeTypeDescriptor iso8859x1
! !

!MimeEntity class methodsFor:'parsing'!

builder
	^self defaultBuilderType new
!

parse: rfc822Scanner
	^self builder parse: rfc822Scanner
!

printer
	^self defaultPrinterType new
!

readFrom: aStream
	^self builder readFrom: aStream
!

readHeaderFrom: stream
	^self builder readHeaderFrom: stream
!

scannerType
	^MimeScanner
! !

!MimeEntity class methodsFor:'presentation'!

defaultEditorType
	^SinglePartPresenter
!

editor
!

editorTypes
	^EditorRegistry
!

editorTypesAt: aType
	^self editorTypes at: aType ifAbsent: [ self defaultEditorType ]
! !

!MimeEntity methodsFor:'accessing'!

body

	^value isNil
		ifTrue: [ value := self newBody ]
		ifFalse: [ value ]
!

body: aMessageBody
	value := aMessageBody.
	aMessageBody parent: self.
!

boundary
	^self contentTypeField boundary
!

boundary: aString
	^self contentTypeField boundary: aString
!

builder
	^self defaultBuilderType new
!

charset
	^self charsetIfNone:
			[ self parent ~~ nil
				ifTrue: [       self parent defaultCharset]
				ifFalse: [ self defaultCharset] ]
!

charset: aName
	self isSimple
		ifTrue: [ (self getFieldAt: 'content-type') charset: aName ]
!

charsetIfNone: aBlock
	^self contentTypeField charsetIfNone: aBlock
!

contentId
	^self fieldValueAt: 'content-id'
!

contentId: aString
	self fieldValueAt: 'content-id' put: aString
!

contentLength
	^self fieldValueAt: 'content-length'
!

contentLength: anInteger
	self fieldValueAt: 'content-length' put: anInteger
!

contentTransferEncoding
	^self fieldValueAt: 'content-transfer-encoding'
!

contentTransferEncoding: encodingNameString
	self fieldValueAt: 'content-transfer-encoding'  put: encodingNameString
!

contentType
	^(self fieldAt: 'content-type' ifAbsent: [ self defaultContentTypeField  ]) contentType
!

contentType: aString
	('text/plain' match: aString)
		ifTrue: [self  contentType: aString charset: self defaultCharset ]
		ifFalse: [self fieldAt: 'content-type' put: (ContentTypeField contentType: aString)]
!

contentType: aString charset: cString
	self fieldAt: 'content-type' put: (ContentTypeField contentType: aString charset: cString)
!

contentTypeEncoding
	" Answer symbolic name of the character encoding for the decoded body of this entity "

	^self contentTypeEncodingType
!

contentTypeEncodingType
" Answer symbol representing character encoding that we use to decode the body of this entity. This
encoding is entirely determined by the current content type field as follows:
	If content type is text or multipart, then value of charset parameter is used
		content-type: text/plain; charset = iso-8859-1
	If no charset is present, default charset encoding is used
	If content type is binary, binary encoding is used
"

	^self isBinaryContentType
		ifTrue: [ #binary ]
		ifFalse: [self charset notNil
					ifTrue: [self charset]
					ifFalse: [#default]]
!

contentTypeField
	^(self fieldAt: 'content-type' ifAbsent: [ self defaultContentTypeField ])
!

contentTypeSpecies
" Answer ByteArray if my Mime type is binary, String otherwise. See also contentsSpecies "

	^self isBinaryContentType
		ifTrue: [ ByteArray ]
		ifFalse: [ String ]
!

contentsSpecies
" Answer ByteArray if I contain bytes, String if I containg character info. Note
that if content transfer encoding is present, it is always String. See also contentTypeSpecies "

	^self isBinary
		ifTrue: [ ByteArray ]
		ifFalse: [ String ]
!

encoding
	" Answer symbolic name of the character encoding for the body of this entity "

	^self encodingType ~~ nil
		ifTrue: [ self encodingType ]
		ifFalse: [ MimeTypeDescriptor encodingForCharset: #default ]
!

encodingType
" Answer symbol representing character encoding that we use to read the body of this entity. This
encoding depends on the current content type and content transfer encoding.
If transfer encoding is not present, then encoding is determined by content-type
field as follows:
	If content type is text or multipart, then value of charset parameter is used
		content-type: text/plain; charset = iso-8859-1
	If no charset is present, default charset encoding is used
	If content type is binary, binary encoding is used
If transfer encoding is present, default charset encoding is used
"

	^self isBinary
		ifTrue: [ #binary ]
		ifFalse: [self charsetIfNone: [#default]]
!

fields

	^self header fields
!

fields: aValue
	self header fields: aValue
!

headerCharset
"       headerCharset == nil
		ifTrue:
			[headerCharset :=
				( self charset ~~ nil
					ifTrue: [ self charset ]
					ifFalse:
						[self parent ~~ nil
							ifTrue: [ self parent headerCharset ]
							ifFalse: [ MimeTypeDescriptor defaultCharacterSet ]])].
"
	^headerCharset
!

headerCharset: aString
	headerCharset := aString
!

majorType
	^self contentTypeField subtype
!

mimeType
    Smalltalk isSmalltalkX ifTrue:[    
        ^ MIMETypes mimeTypeFromString: self contentType asLowercase
    ] ifFalse:[
        ^MimeTypeDescriptor guessFromContentType: self contentType
    ].
!

mimeVersion
	^self fieldValueAt: 'mime-version'
!

mimeVersion: aString
	self fieldValueAt: 'mime-version' put: aString
!

subtype
	^self contentTypeField subtype
!

type
	^self contentTypeField type
!

type: aString
	^self contentTypeField type: aString
! !

!MimeEntity methodsFor:'accessing body parts'!

addPart: part
	part parent: self.
	self body addPart: part.
!

contents
	^self body value
!

contents: aString
	^self body value: aString
!

contentsStream
	^self body valueAsStream
!

decodedContents
	^self body decodedValue
!

decodedContentsWith: aSymbol
	^self body decodedValueWith: aSymbol
!

indexOfPart: aPart
	^self parts indexOf: aPart
!

newBody
	^MimeMessageBody newBodyFor: self
!

partAt: index
	^self parts at: index
!

partAt: index put: aPart
	self parts at: index put: aPart
!

parts
	^ self body parts
!

removePart: aMimeEntity
	^self body removePart: aMimeEntity
!

removePartAt: index
	^self body removePartAt: index
! !

!MimeEntity methodsFor:'accessing fields'!

addField: field
	" This method will check if the field exists already; if yes, if it can be merged into the existing field and, if yes, merge it. Otherwise, add as a new field "

	" Implement field merge"
	self header add: field.
!

arrangeFieldsInOrder: anArrayOfFieldNames
	self header arrangeFieldsInOrder: anArrayOfFieldNames
!

fieldAt: aString
	^self header at: aString
!

fieldAt: aString ifAbsent: aNiladicBlock
	^self header at: aString ifAbsent: aNiladicBlock
!

fieldAt: aString ifAbsentPut: aNiladicBlock
	^self header at: aString ifAbsentPut: aNiladicBlock
!

fieldAt: aString put: field
	^self header at: aString put: field
!

fieldValueAt: fieldName
" Answer value of specified field or nil "
	^self fieldValueAt: fieldName ifAbsent: [ nil ]
!

fieldValueAt: fieldName put: aValue
" Ensures at least field's placeholder exists and sets field's value "
	^(self getFieldAt: fieldName ) value: aValue
!

fieldValuesAt: aString
" Answer all fields with specified names in their order of appearance "
	^(self fieldsAt: aString)
		inject: OrderedCollection new into: [ :list :field | list addAll: field values. list ]
!

fieldsAt: aString
" Answer all fields with specified names in their order of appearance "
	^self header allAt: aString
!

getValueFor: fieldName
	^self getValueFor: fieldName ifAbsent: [ nil ]
!

getValueFor: fieldName ifAbsent: aBlock
" For compatibility only "
	^self fieldValueAt: fieldName ifAbsent: [ ^aBlock value ]
!

header

	^header isNil
		ifTrue: [ header := self newHeader ]
		ifFalse: [ header ]
!

header: aMessageHeader

	header := aMessageHeader.
	header parent: self
!

indexOfField: aField
	^self header indexOf: aField
!

indexOfFieldNamed: aFieldName
	^self header indexOf: (self fieldAt: aFieldName ifAbsent: [^0])
!

newHeader
	^MessageHeader newFor: self
!

removeField: aHeaderField
" Remove header field "

	^self header remove: aHeaderField
!

removeFieldAt: fieldName
" Remove field with specified name "

	^self header removeKey: fieldName
! !

!MimeEntity methodsFor:'accessing fields and body parts'!

fieldValueAt: fieldName ifAbsent: aBlock

	^(self fieldAt: fieldName ifAbsent: [ ^aBlock value ]) value
!

getFieldAt: aString
" Make sure we have either field or its stub so we can put real value in it. Default implementation
of #default in HeaderField just creates a new instance of the field. Some subclasses override
this method to provide more specific stub creation "

	^self header
		at: aString
		ifAbsentPut: [(HeaderField fieldClassForName: aString) default name: aString].
! !

!MimeEntity methodsFor:'composing'!

writeContentsOn: anEncodedStream
	^self body writeContentsOn: anEncodedStream
!

writeOn: aStream
	self printer writeMessage: self on: aStream
! !

!MimeEntity methodsFor:'content-disposition'!

contentDisposition
	^self fieldAt: 'content-disposition' ifAbsent: [nil]
!

contentDisposition: aString
" aString should be 'inline' or 'attachment' "
	(self getFieldAt: 'content-disposition') disposition: aString
!

contentDispositionSize
	^(self fieldAt: 'content-disposition' ifAbsent: [^nil]) size
!

contentDispositionSize: anInteger
	(self getFieldAt: 'content-disposition') size: anInteger
!

fileName

	^(self fieldAt: 'content-disposition' ifAbsent: [^nil]) fileName
!

fileName: aString

	(self getFieldAt: 'content-type')  fileName: aString. "For eudora"
	^(self getFieldAt: 'content-disposition') fileName: aString
! !

!MimeEntity methodsFor:'copying'!

postCopy
	super postCopy.
	header := header copy parent: self.
	value := value copy parent: self.
! !

!MimeEntity methodsFor:'defaults'!

defaultBot
	^MimeEntityBot new
!

defaultBuilderType
	^self class defaultBuilderType
!

defaultCharset

	^self class defaultCharset
!

defaultContentDispositionField
	^ContentDispositionField default
!

defaultContentType
" Answer default content type string. Use Mime rules, subclasses may override "

        ^self parent notNil
                ifTrue: [ self parent defaultContentTypeForParts ]
                ifFalse: [ MimeTypeDescriptor defaultContentTypeString ]
!

defaultContentTypeField
" Answer default content-type field "
	^ContentTypeField contentType: self defaultContentType charset: self defaultCharset
!

defaultContentTypeForParts
" Answer default content-type field for body parts in multipart message. This method uses Mime rules. It can be overridden, i.e. for HTTP. Mime specifies that the default content-type is text/plain; charset = 'us-ascii' except in multipart/digest entities where the default content-type is message/rfc822 "
	^self contentType ~= MimeTypeDescriptor contentTypeMultipartDigest
		ifTrue: [ MimeTypeDescriptor defaultContentTypeString ]
		ifFalse: [ MimeTypeDescriptor contentTypeMessageRfc822 ]
!

defaultPrinterType
	^self class defaultPrinterType
!

defaultTransferEncodingName
" Answer default transfer encoding for this entity. Somewhat arbitrarily we use quoted-printable
for text entities (which assumes we have largely ASCII-based text, saving space), and base64
for everything else (because it is most likely binary data, so base64 is more appropriate "

	^self isTextContentType
		ifTrue: [ 'quoted-printable' ]
		ifFalse: [ 'base64' ]
! !

!MimeEntity methodsFor:'mime version'!

addMimeVersion
	self mimeVersion: '1.0'
! !

!MimeEntity methodsFor:'mutating'!

replaceFrom: replacementPart
" Replace your guts from another entity. Copy all the fields into yourself
replacing your own fields if same name. Replace your own body with
the replacement entity's body "

	" Merge fields from the replacement part into the message "
	replacementPart fields do: [ :field | self fieldAt: field name put: field ].
	self body: replacementPart body
!

replacePart: originalPart with: replacementPart
	| index |

	index := self parts indexOf: originalPart.
	index = 0 ifTrue: [ ^self error: (#OriginalPartNotFound << #net >> 'Original part not found') ].
	self partAt: index put: replacementPart
! !

!MimeEntity methodsFor:'parsing'!

parse: rfc822Scanner
	^self builder parse: self from: rfc822Scanner
!

parseBody: rfc822Scanner
	^self builder parseBody: self from: rfc822Scanner
! !

!MimeEntity methodsFor:'presentation'!

edit

	self editor openWith: self
!

editor
	^ (self class editorTypesAt: self contentType) new
! !

!MimeEntity methodsFor:'printing'!

displayString
"For presenter to display something useful in place of the entity contents.
For text entity this is simply entity contents, but for binary entities only a short
digest is shown"


        | stream |
        Smalltalk isSmalltalkX ifTrue:[
            ^ super displayString
        ].
        stream := (String new: 128) writeStream.
        self printForDisplayOn: stream.
        ^stream contents
!

printForDisplayOn: stream
"For presenter to display something useful in place of the entity contents.
For text entity this is simply entity contents, but for binary entities only a short
digest is shown"

	self body printForDisplayOn: stream
!

printHeaderOn: aStream
	self printer printMessageElement: self header on: aStream
!

printOn: aStream indent: level
	self printer printMessage: self on: aStream indent: level.
!

printer
	^self defaultPrinterType new
! !

!MimeEntity methodsFor:'private'!

bot
	^self defaultBot
! !

!MimeEntity methodsFor:'testing'!

hasBoundary
	^self boundary notNil
!

hasContentTransferEncoding
	^self contentTransferEncoding notNil
!

isApplication

	^ self type match: 'application'
!

isBase64Type
	^self contentTransferEncoding notNil
		ifTrue: [ self contentTransferEncoding match: 'base64']
		ifFalse: [ false ]
!

isBinary
" Answers true if my contents are represented as text or binary. If content
transfer encoding is present, my contents are text even though this text
encodes binary information "

	^self isBinaryContentType and: [self hasContentTransferEncoding not]
!

isBinaryContentType
	^self mimeType isBinary
!

isHtmlText

	^ self isTextContentType and: [self subtype = 'html' ]
!

isMultipart
	^self contentTypeField isMultipart
!

isMultipartAlternative

	^self contentType match: 'multipart/alternative'
!

isMultipartMixed

	^self contentType match: 'multipart/mixed'
!

isPlainText

	^ self isTextContentType and: [ self subtype = 'plain' ]
!

isPrintedQuotableType
	^self contentTransferEncoding notNil
		ifTrue: [ self contentTransferEncoding match: 'quoted-printable' ]
		ifFalse: [ false ]
!

isSimple
	^self isMultipart not
!

isText
" Answers true is message body contains binary data. This method will
always answer true in the presence of content transfer encoding. See also
isTextContentType "

	^ self isBinary not
!

isTextContentType
" Answers true is message content type corresponds to binary data. See also
isText "

	^ self mimeType isText
!

needsBoundary
	^self isMultipart and: [self boundary isNil]
!

needsContentLength
	^false
!

needsContentTransferEncoding

	^self hasContentTransferEncoding not and: [self body needsContentTransferEncoding]
! !

!MimeEntity methodsFor:'transfer encodings'!

addContentTransferEncoding: encodingNameString
" Transfer-encode message. encodingNameString is one of: 'base64', 'quoted-printable', ... "

	self contentTransferEncoding: encodingNameString.
	self body encodeAs: encodingNameString.
!

removeContentTransferEncoding
	"Transfer-encode message. encodingNameString is one of: 'base64', 'quoted-printable', ..."

	self hasContentTransferEncoding
		ifTrue:
			[self body decodeAs: self contentTransferEncoding asLowercase.
			self removeFieldAt: 'content-transfer-encoding']
!

transferFieldOrder
" Answer fields that should appear in specific order during transfer "
	^OrderedCollection new
! !

!MimeEntity methodsFor:'visiting'!

acceptVisit: aVisitor with: argument
	^aVisitor acceptMimeEntity: self with: argument
! !

!MimeEntity class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MimeEntity.st,v 1.3 2009/01/13 15:24:37 stefan Exp $'
! !

MimeEntity initialize!
