"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MessageHandler subclass:#MimeEntityBot
	instanceVariableNames:'entityCallback keepSearching'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MimeEntityBot comment:'This class is a ''robot'' that can be used to search Mime entities for information and to perform some transformations of Mime entities.
Methods in ''enumeration'' provide search facilities to find and collect parts matching certain criteria. Searches can be restricted to some level of nesting, which is convenient when searching for message text parts and attachments.
Methods in ''tasks'' provides facilities to check message structure, add/remove transfer encoding and also to correctly mutate message. Examples include adding part to a simple message which mutates it froma simple message to multi-part message. Another example is dropping all representations of message text except plain text one, which may require to mutate message from multipart/alternative to simple, or replacing multipart/alternative part with a simple part. This class encapsulates all the logistics and complexity involved in these transformations providing easy to use high-level methods.

Instance Variables:

	keepSearching   <Boolean>  Internal -- set to terminate message traversal
	partCallback    <BlockContext>  Internal -- callback when a node is traversed
'
!


!MimeEntityBot methodsFor:'accessing'!

entityCallback
	^entityCallback
!

entityCallback: aMonadicBlock
	entityCallback := aMonadicBlock
!

keepSearching
	^keepSearching
!

keepSearching: aBoolean
	keepSearching := aBoolean
! !

!MimeEntityBot methodsFor:'action callbacks'!

startEntity: aMimeEntityOrNil
	^(super startEntity: aMimeEntityOrNil)
		ifTrue: [self entityCallback isNil
			ifTrue: [true]
			ifFalse: [self entityCallback value: aMimeEntityOrNil]]
		ifFalse: [false]
! !

!MimeEntityBot methodsFor:'enumerating'!

for: aMimeEntity maxDepth: depthLimit partsDo: aBlock
	self
		for: aMimeEntity
		partsDo: [: part | self level <= (depthLimit - 1) ifTrue: [aBlock value: part]]
!

for: aMimeEntity partsDetect: aBlock ifNone: exceptionBlock
	self
		for: aMimeEntity
		partsDo: [:part | (aBlock value: part) ifTrue: [^part]].
	^exceptionBlock value
!

for: aMimeEntity partsDo: aBlock
	self for: aMimeEntity while: [ true ] partsDo: aBlock
!

for: aMimeEntity while: condBlock partsDo: aBlock
	self entityCallback: [:part | condBlock value
			ifTrue: [aBlock value: part. true]
			ifFalse: [self keepSearching: false. false]].
	self traverse: aMimeEntity.
	^aMimeEntity
!

traverse: aMimeEntity
	self
		initializeEntityContext;
		keepSearching: true;
		visit: aMimeEntity
! !

!MimeEntityBot methodsFor:'initialization'!

initialize
	super initialize.
	keepSearching := true
! !

!MimeEntityBot methodsFor:'tasks'!

addFileAttachmentFor: aMessage fileName: aFilename
	| part |
	aMessage addPart: (part := MimeEntity fileName: aFilename).
	^part
!

applyTransferEncodingTo: part
	self applyTransferEncodingTo: part encodeAs: self defaultEncodingName
!

applyTransferEncodingTo: part encodeAs: encodingNameOrNil
	| encoding |

	encoding := encodingNameOrNil isNil
					ifTrue: [ part defaultTransferEncodingName ]
					ifFalse: [encodingNameOrNil].
	part addContentTransferEncoding: encoding.
!

attachmentsFor: aMimeEntity
	^aMimeEntity isMultipart
		ifTrue: [ aMimeEntity parts ]
		ifFalse: [ OrderedCollection new ]
!

checkStructure: aMimeEntity
	" Make sure all fields are in order mandated by RFC822 "

	self for: aMimeEntity partsDo:
		[ :part | part arrangeFieldsInOrder: aMimeEntity transferFieldOrder ].

	aMimeEntity mimeVersion isNil
		ifTrue: [aMimeEntity addMimeVersion].

	" Make sure all parts have assigned boundaries "
	self for: aMimeEntity partsDo: [ :part |
		part needsBoundary
			ifTrue: [part boundary: (self defaultBoundaryFor: part)]].
!

defaultBoundaryFor: aMimeEntity
" Compute default boundary for specified Mime entity. Boundary is a string that
could not occur inside any of the parts. For now just use constant plus a random number "

	^'=_STAMPed_MAIL', Random new next printString, '_='
!

defaultEncodingName
	^'base64'
!

makeMultipart: rfc822Message
" Make single-part mail message a multipart one. Assumption that is made
here: aMimeEntity is a top-level RFC822 message. We create a new Mime
entity to contain message text and Mime fields (content-type, content-length, etc).
These fields are removed from the top-level message and new content-type is
set. This content type is somewhat arbitrarily set to miltipart/mixed. The message
is then given new miltipart body and new part is added to the body "

	| messagePart |
	messagePart := self makeSinglepartFrom: rfc822Message.
	" Remove non-mime fields from the top-level message "

	rfc822Message
		fields: (rfc822Message fields reject: [ :field |  'content*' match: field name ]);
		contentType: 'multipart/mixed';
		body: MultipartBody new.
	messagePart ~~ nil
		ifTrue: [ rfc822Message addPart: messagePart].
	^rfc822Message
!

makeSinglepart: rfc822Message textPart: textPart
" Mutate multipart mail message into a single-part one. Assumption that is made
here: rfc822Message is a top-level RFC822 message that has only one part.
We create new simple part and copy the part's contents into message contents.
We then copy and all the part's fields into the top-level message fields. Simple body
becomes the top level message's body "

	" Merge fields from the part into the message "
	textPart fields do: [ :field | rfc822Message addField: field ].

	rfc822Message
		body: SimpleBody new;
		contents: textPart contents.
	^rfc822Message
!

makeSinglepartFrom: aMimeEntity
" Make single-part from mime entity. Return nil if mime entity has empty body and  header fields "
	| messagePart chSet |
	messagePart := MimeEntity new.
	" Copy Mime fields from the message "
	aMimeEntity fields
		do: [ :field |
			('content*' match: field name)
				ifTrue: [ messagePart addField: field copy] ].
	aMimeEntity contents notNil
		ifTrue: [       messagePart contents: aMimeEntity contents].

	^(messagePart fields isEmpty and: [ messagePart contents == nil ])
		ifTrue: [nil]
		ifFalse:
			[(chSet := messagePart charset ) ~~ nil ifTrue: [ aMimeEntity charset: chSet ].
			messagePart fields isEmpty ifTrue: [ messagePart charset: aMimeEntity defaultCharset].
			messagePart]
!

validate: aMimeEntity
" Run sanity checks on an entity "

	" 1. Validate structure "
	self
		for: aMimeEntity
		partsDo: [:part |
			part body parent == part ifFalse: [ self error: (#BodyParentIsWrong << #net >> 'Body parent is wrong') ].
			part fields do: [ :field | field parent == part ifFalse: [ self error: (#FieldParentIsWrong << #net >> 'Field parent is wrong') ]].
			part parent notNil
				ifTrue: [ (part parent isMultipart and: [ part parent parts includes: part ])
						ifFalse: [self error: (#InvalidMessageStructure << #net >> 'Invalid message structure') ]].
			(part isMultipart
				ifTrue: [ part body isKindOf: MultipartBody ]
				ifFalse: [ part body isKindOf: SimpleBody ])
					ifFalse: [self error: (#BodyTypeMismatch << #net >> 'Body type mismatch') ].
			part isMultipart
				ifTrue: [part boundary notNil
					ifFalse: [self error: (#ShouldHaveBoundary << #net >> 'Should have boundary') ]].
		].
!

writeContentsOf: aMimeEntity on: anEncodedStream
	aMimeEntity removeContentTransferEncoding.
	aMimeEntity isBinary
		ifTrue: [anEncodedStream binary]
		ifFalse: [anEncodedStream text].
	aMimeEntity writeContentsOn: anEncodedStream.
! !

!MimeEntityBot class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MimeEntityBot.st,v 1.1 2004/11/12 10:46:34 penk Exp $'
! !
