"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

RFC822Scanner subclass:#MimeScanner
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MimeScanner comment:'MimeScanner implements Mime encoding rules specified in RFC2047 (Message Header Extensions for Non-Ascii Text). The notation extends RFC822 describing encoding lexical elements text, phrase and word. MimeScanner extends RFC822 scanner. It has the same set of lexems. When either text, phrase or word is scanned, it is scanned using rfc822 syntax but Mime character  table. Once lexem is scanned, then Mime encoding rules for this lexem are applied so that non-ASCII characters can be decoded.'
!


!MimeScanner class methodsFor:'printing'!

printPhrase: phrase on: stream
	MimeEncodedWordCoDec decodePhrase: phrase printOn: stream
! !

!MimeScanner class methodsFor:'testing'!

isTokenChar: char
	^((self classificationTable at: char asInteger + 1) bitAnd: TokenMask) ~= 0
!

needsTransferEncoding: aStream mimeType: aMimeTypeDescriptor
	"Answer true if the object needs to be transfer-encoded because it contains
	8-bit values or lines are too long, etc.
	For now on, answer true for non-binary streams. Scan character strings looking for 8-bit characters
	and answer true if one is found. Otherwise answer false"
	| savePos |
	aMimeTypeDescriptor isBinary ifTrue: [ ^true ].
	((aStream respondsTo: #isBinary) and: [aStream isBinary])
		ifTrue: [^true].

	savePos := aStream position.
	[aStream reset.
	[aStream atEnd]
		whileFalse: [aStream next asInteger > 127 ifTrue: [^true]]]
		ensure: [aStream position: savePos].
	^false
!

shouldBeQuoted: string
	^(string detect: [ :char | (self isTokenChar: char) not ] ifNone: [ nil ]) notNil
! !

!MimeScanner class methodsFor:'text encoding/decoding'!

decodeBase64From: inStream to: outStream
	"Decode input stream from base64 to output stream. outStream is an encoded stream "

	| string |
	[inStream atEnd]
		whileFalse:
			[(string := inStream upTo: Character return) notEmpty
				ifTrue: [       self decodeBase64From: 1 to: string size in: string on: outStream]].
!

decodeBase64From: startIndex to: endIndex in: aString
	"Decode aString from startIndex to endIndex in base64."

	| codeChars decoder output index cr endChars end limit padding data sz |

	codeChars := 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.
	decoder := (0 to: 255) collect: [:n| (codeChars indexOf: (n + 1) asCharacter) - 1].
	decoder replaceAll: -1 with: 0.
	output := (data := ByteString new: endIndex - startIndex * 3 // 4) writeStream.
	index := startIndex.
	cr := Character return.
	"There is padding at the end of a base64 message if the content is not a multiple of
	 3 bytes in length.  The padding is either two ='s to pad-out a trailing byte, 1 = to
	 pad out a trailing pair of bytes, or no padding.  Here we count the padding.  After
	 processing the message we cut-back by the amount of padding."
	end := endIndex min: (sz := aString size).
	endChars := codeChars, (String with: $=).
	[(endChars includes: (aString at: end))
	and: [end = endIndex or: [(aString at: end + 1) = cr]]] whileFalse: [end := end - 1].
	limit := end.
	padding := 0.
	[(aString at: end) == $=] whileTrue: [padding := padding - 1. end := end - 1].
	[index <= limit] whileTrue:
		[| triple |
		triple :=    ((decoder at: (aString at: index     ) asInteger) bitShift: 18)
				+ ((decoder at: (aString at: index + 1) asInteger) bitShift: 12)
				+ ((decoder at: (aString at: index + 2) asInteger) bitShift:  6)
				+  (decoder at: (aString at: index + 3) asInteger).
		output nextPut: (Character value: (triple digitAt: 3)).
		output nextPut: (Character value: (triple digitAt: 2)).
		output nextPut: (Character value: (triple digitAt: 1)).
		index := index + 4.
		[(index > sz or: [(aString at: index) = cr])
		and: [index <= limit]] whileTrue: [index := index + 1]].
	padding ~= 0 ifTrue: [output skip: padding].
	^data copyFrom: 1 to: output position.
!

decodeBase64From: startIndex to: endIndex in: aString on: output
	"Decode aString from startIndex to endIndex in base64."

	| codeChars decoder index cr endChars end limit padding sz |

	codeChars := 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.
	decoder := (0 to: 255) collect: [:n| (codeChars indexOf: (n + 1) asCharacter) - 1].
	decoder replaceAll: -1 with: 0.
	index := startIndex.
	cr := Character return.
	"There is padding at the end of a base64 message if the content is not a multiple of
	 3 bytes in length.  The padding is either two ='s to pad-out a trailing byte, 1 = to
	 pad out a trailing pair of bytes, or no padding.  Here we count the padding.  After
	 processing the message we cut-back by the amount of padding."
	end := endIndex min: (sz := aString size).
	endChars := codeChars, (String with: $=).
	[(endChars includes: (aString at: end))
	and: [end = endIndex or: [(aString at: end + 1) = cr]]] whileFalse: [end := end - 1].
	limit := end.
	padding := 0.
	[(aString at: end) == $=] whileTrue: [padding := padding - 1. end := end - 1].
	[index <= limit] whileTrue:
		[| triple |
		triple :=    ((decoder at: (aString at: index     ) asInteger) bitShift: 18)
				+ ((decoder at: (aString at: index + 1) asInteger) bitShift: 12)
				+ ((decoder at: (aString at: index + 2) asInteger) bitShift:  6)
				+  (decoder at: (aString at: index + 3) asInteger).
		output nextPut: (triple digitAt: 3).
		output nextPut: (triple digitAt: 2).
		output nextPut: (triple digitAt: 1).
		index := index + 4.
		[(index > sz or: [(aString at: index) = cr])
		and: [index <= limit]] whileTrue: [index := index + 1]].
	padding ~= 0 ifTrue: [output skip: padding].
	^output
!

decodeQuotedPrintableFrom: input
	"Decode aString from startIndex to endIndex in quoted-printable."

	| output char n1 n2 |
	output := (ByteString new: input size ) writeStream.
	[input atEnd] whileFalse:
		[char := input next.
		$= == char
			ifTrue:
				[('0123456789ABCDEF' includes: (n1 := input next)) ifTrue:
					[n2 := input next.
					output nextPut: ((n1 digitValue bitShift: 4) + n2 digitValue) asCharacter]]
			ifFalse:
				[output nextPut: char]].
	^output contents
!

decodeQuotedPrintableFrom: startIndex to: endIndex in: aString
	"Decode aString from startIndex to endIndex in quoted-printable."

	^self decodeQuotedPrintableFrom: (ReadStream on: aString from: startIndex to: endIndex)
!

decodeUUEncodedFrom: startIndex to: farEndIndex in: aString
	"decode aString from startIndex to farEndIndex as uuencode-encoded"
	| endIndex i cr space output data |
	endIndex := farEndIndex - 2.
	[endIndex <= startIndex
	or: [(aString at: endIndex + 1) = $e
		and: [(aString at: endIndex + 2) = $n
		and: [(aString at: endIndex + 3) = $d]]]]
		whileFalse:
			[endIndex := endIndex - 1].
	i := (aString
			findString: 'begin'
			startingAt: startIndex
			ignoreCase: true
			useWildcards: false) first.
	i = 0 ifTrue: [i := startIndex].
	cr := Character return.
	space := Character space asInteger.
	output := (data := ByteString new: endIndex - startIndex * 3 // 4) writeStream.
	[[i < endIndex
	and: [(aString at: i) ~= cr]] whileTrue: [i := i + 1].
	i < endIndex] whileTrue:
		[| count |
		count := (aString at: (i := i + 1)) asInteger - space bitAnd: 8r77.
		i := i + 1.
		count = 0
			ifTrue: [i := endIndex]
			ifFalse:
				[[count > 0] whileTrue:
					[| m n o p |
					m := (aString at: i) asInteger - space bitAnd: 8r77.
					n := (aString at: i + 1) asInteger - space bitAnd: 8r77.
					o :=  (aString at: i + 2) asInteger - space bitAnd: 8r77.
					p :=  (aString at: i + 3) asInteger - space bitAnd: 8r77.
					count >= 1 ifTrue:
						[output nextPut: (Character value: (m bitShift: 2) + (n bitShift: -4)).
						count >= 2 ifTrue:
							[output nextPut: (Character value: ((n bitShift: 4) + (o bitShift: -2) bitAnd: 16rFF)).
							count >= 3 ifTrue:
								[output nextPut: (Character value: ((o bitShift: 6) + p bitAnd: 16rFF))]]].
					i := i + 4.
					count := count - 3]]].
	^data copyFrom: 1 to: output position
!

encode: inStream asBase64To: outStream
        "Encode the input as base64 to outStream."
        | codeChars buffer lineLength count end column a b c in convertToByteArray chunkSize temp |
        codeChars := 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'.
        column := 0.

        " Ugly code to ensure the the underlying array is bytes "

        in := inStream.

        [in binary] on: Error do: [ :ex | ex return].
        convertToByteArray := in contentsSpecies includesBehavior: String.
        lineLength := 76.
        chunkSize := (lineLength / 4 * 3) asInteger.
        [convertToByteArray
                ifTrue:
                        [temp := in nextAvailable: chunkSize.
"/                        buffer := temp asByteArrayEncoding: temp stringEncoding streamEncodingType.
                        buffer := temp asByteArray.
                        count := buffer size]
                ifFalse:
                        [buffer := ByteArray new: chunkSize.
                        count := in nextAvailableBytes: buffer size into: buffer startingAt: 1].
        end := count // 3 * 3.
        1 to: end by: 3 do:
                [:i|
                (column := column + 4) > lineLength ifTrue:
                        [outStream cr. column := 0].
                a := buffer at: i.
                b := buffer at: i + 1.
                c := buffer at: i + 2.
                outStream
                        nextPut: (codeChars at: (a bitShift: -2) + 1);
                        nextPut: (codeChars at: (((a bitShift: 4) + (b bitShift: -4)) bitAnd: 16r3F) + 1);
                        nextPut: (codeChars at: (((b bitShift: 2) + (c bitShift: -6)) bitAnd: 16r3F) + 1);
                        nextPut: (codeChars at: (c bitAnd: 16r3F) + 1)].
        (column := column + 4) > lineLength ifTrue:
                [outStream cr. column := 0].
        count - end = 1 ifTrue: "one byte left to write"
                [a := buffer at: end + 1.
                 outStream
                        nextPut: (codeChars at: (a bitShift: -2) + 1);
                        nextPut: (codeChars at: ((a bitShift: 4) bitAnd: 16r3F) + 1);
                        nextPut: $=;
                        nextPut: $=].
        count - end = 2 ifTrue: "two bytes left to write"
                [a := buffer at: end + 1.
                 b := buffer at: end + 2.
                 outStream
                        nextPut: (codeChars at: (a bitShift: -2) + 1);
                        nextPut: (codeChars at: (((a bitShift: 4) + (b bitShift: -4)) bitAnd: 16r3F) + 1);
                        nextPut: (codeChars at: ((b bitShift: 2) bitAnd: 16r3F) + 1);
                        nextPut: $=].
        count > 0] whileTrue
!

encode: inStream asQuotedPrintableTo: outStream
	"Encode the input as printed-quotable to outStream.  Ensure that leading '.'
	 characters are encoded so that mailers which interpret '.' at the beginning
	 of line as an end-of-message indicator are not confused by '.'s in the body
	 of a message."
	| cr tab spaceCode encoder column ch code |
	cr := Character return.
	tab := Character tab.
	spaceCode := Character space asInteger.
	encoder := '0123456789ABCDEF'.
	column := 0.
	[(ch := inStream next) == nil] whileFalse:
		[ch = cr
			ifTrue:
				[outStream nextPut: cr.
				 column := 0]
			ifFalse:
				[(column := column + 1) >= 76 ifTrue:
					[outStream nextPut: $=; nextPut: Character return. column := 1].
				((code := ch asInteger) < 127
				  and: [(code >= spaceCode or: [ch == tab])
				  and: [ch ~~ $. or: [column > 1]]])
					ifTrue:
						[outStream nextPut: ch]
					ifFalse:
						[outStream
							nextPut: $=;
							nextPut: (encoder at: (code bitShift: -4) + 1);
							nextPut: (encoder at: (code bitAnd: 16rF) + 1).
						column := column + 2]]]
! !

!MimeScanner methodsFor:'constants'!

boundaryDashes
	^'--'
! !

!MimeScanner methodsFor:'multi-character scans'!

scanText
" Parse text as defined in RFC822 grammar, then apply the rules of RFC2047 for encoded words in Text fields. An encoded word inside text field may appear immediately following a white space character "

	| text |
	text := super scanText.
	^MimeEncodedWordCoDec decodeText: text.
!

scanToBoundary: boundary
" Scan for specified boundary (RFC2046, p5.1). Answer scanned text from current position up to the beginning of the boundary. Stream is positioned after end of the boundary line
Second element is either #next or #last. #next means the boundary found is not the last one. #last means the boundary is the closing boundary for the multi-part body (that is, it looks like '--<boundary>--) "

	| string |

	string := self upToAll: (self boundaryDashes , boundary).
	self upTo: Character return.                                        " Skip to the end of line "
	^string.
!

scanToken
" MIME and HTTP: token  =  1*<any CHAR except tspecials, SPACE and CTLs>. That is, 'token' is analogous to RFC822 'atom' except set of Mime's set of tspecials characters includes three more characters as compared to set of 'specials' in RFC822"

	token := self scanTokenMask: TokenMask.
	tokenType := #token.
	^token
! !

!MimeScanner methodsFor:'printing'!

printPhrase: phrase on: stream
	self class printPhrase: phrase on: stream
! !

!MimeScanner methodsFor:'private'!

decodeCommentString: commentString
	^MimeEncodedWordCoDec decodeComment: commentString
!

nextMimeToken
	| char |
	self skipWhiteSpace.
	char := self peek.
	char isNil                              "end of input"
		ifTrue: [tokenType := #doIt.
			^token := nil].
	char == $( ifTrue: [^self scanComment; nextToken].
	char == $" ifTrue: [^self scanQuotedText].
	(self specials includes: char)
		ifTrue: [tokenType := #special.         " Special character. Make it token value and set token type "
				^token := self next.].
	(self matchCharacterType: TokenMask)
		ifTrue: [^self scanToken].
	tokenType := #doIt.
	token := char.
	^token
!

scanCommentOn: streamOrNil
	"scan comment copying on specified stream. Look for MIME 'encoded words' (RFC2047) and decoded them if identified"

	token := super scanCommentOn: streamOrNil.
	^self decodeCommentString: token
! !

!MimeScanner methodsFor:'tokenization'!

nextToken
	^self nextMimeToken
!

specials
" This method is provided to encapsulate lexical differences between RFC822 on one side, and MIME, HTTP on the other side. MIME definiton of 'tspecials' is the same as the RFC 822 definition of ''specials' with the addition of the three characters </>, <?>, and <=>, and the removal of <.>. To present uniform tokenization interface, this method is overridden in Mime scanner "

	^self class tspecials
! !

!MimeScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MimeScanner.st,v 1.3 2009/03/12 16:33:37 ca Exp $'
! !
