"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MimeMessageBody subclass:#MultipartBody
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

MultipartBody comment:'This class represents multi-part MIME message bodies. Multi-part bodies contain
one or more body parts, each preceded by a boundary delimiter line,
and the last one followed by a closing boundary delimiter line.
After its boundary delimiter line, each body part then consists of a
header area, a blank line, and a body area.  Thus a body part is
similar to an RFC 822 message in syntax, but different in meaning.

This class is responsible for parsing body structure from source stream
and composing body structure to output stream. It parses out individual
entities and lets entities parse themselves. Value of multi-part body is an array of
Mime entities. Value (and hence parsing) is initialized lazily.

Multi-part bodies are defined in section 5.1 of RFC2046 (ftp://ftp.uu.net/inet/rfc/rfc2046.Z).
See documentation method for more details.

An example of multi-part message:

     From: Nathaniel Borenstein <nsb@bellcore.com>
     To: Ned Freed <ned@innosoft.com>
     Date: Sun, 21 Mar 1993 23:56:48 -0800 (PST)
     Subject: Sample message
     MIME-Version: 1.0
     Content-type: multipart/mixed; boundary="simple boundary"

     This is the preamble.  It is to be ignored, though it
     is a handy place for composition agents to include an
     explanatory note to non-MIME conformant readers.

     --simple boundary

     This is implicitly typed plain US-ASCII text.
     It does NOT end with a linebreak.
     --simple boundary
     Content-type: text/plain; charset=us-ascii

     This is explicitly typed plain US-ASCII text.
     It DOES end with a linebreak.

     --simple boundary--

     This is the epilogue.  It is also to be ignored.

'
!

!MultipartBody class methodsFor:'documentation'!

documentation

" From RFC2046: Multipurpose Internet Mail Extensions (MIME) Part Two:
			      Media Types

5.1.  Multipart Media Type

   In the case of multipart entities, in which one or more different
   sets of data are combined in a single body, a 'multipart' media type
   field must appear in the entity's header.  The body must then contain
   one or more body parts, each preceded by a boundary delimiter line,
   and the last one followed by a closing boundary delimiter line.
   After its boundary delimiter line, each body part then consists of a
   header area, a blank line, and a body area.  Thus a body part is
   similar to an RFC 822 message in syntax, but different in meaning.

   A body part is an entity and hence is NOT to be interpreted as
   actually being an RFC 822 message.  To begin with, NO header fields
   are actually required in body parts.  A body part that starts with a
   blank line, therefore, is allowed and is a body part for which all
   default values are to be assumed.  In such a case, the absence of a
   Content-Type header usually indicates that the corresponding body has
   a content-type of 'text/plain; charset=US-ASCII'.

   The only header fields that have defined meaning for body parts are
   those the names of which begin with 'Content-'.  All other header
   fields may be ignored in body parts.  Although they should generally
   be retained if at all possible, they may be discarded by gateways if
   necessary.  Such other fields are permitted to appear in body parts
   but must not be depended on.  'X-' fields may be created for
   experimental or private purposes, with the recognition that the
   information they contain may be lost at some gateways.

   NOTE:  The distinction between an RFC 822 message and a body part is
   subtle, but important.  A gateway between Internet and X.400 mail,
   for example, must be able to tell the difference between a body part
   that contains an image and a body part that contains an encapsulated
   message, the body of which is a JPEG image.  In order to represent
   the latter, the body part must have 'Content-Type: message/rfc822',
   and its body (after the blank line) must be the encapsulated message,
   with its own 'Content-Type: image/jpeg' header field.  The use of
   similar syntax facilitates the conversion of messages to body parts,
   and vice versa, but the distinction between the two must be
   understood by implementors.  (For the special case in which parts
   actually are messages, a 'digest' subtype is also defined.)

   As stated previously, each body part is preceded by a boundary
   delimiter line that contains the boundary delimiter.  The boundary
   delimiter MUST NOT appear inside any of the encapsulated parts, on a
   line by itself or as the prefix of any line.  This implies that it is
   crucial that the composing agent be able to choose and specify a
   unique boundary parameter value that does not contain the boundary
   parameter value of an enclosing multipart as a prefix.

   All present and future subtypes of the 'multipart' type must use an
   identical syntax.  Subtypes may differ in their semantics, and may
   impose additional restrictions on syntax, but must conform to the
   required syntax for the 'multipart' type.  This requirement ensures
   that all conformant user agents will at least be able to recognize
   and separate the parts of any multipart entity, even those of an
   unrecognized subtype.

   As stated in the definition of the Content-Transfer-Encoding field
   [RFC 2045], no encoding other than '7bit', '8bit', or 'binary' is
   permitted for entities of type 'multipart'.  The 'multipart' boundary
   delimiters and header fields are always represented as 7bit US-ASCII
   in any case (though the header fields may encode non-US-ASCII header
   text as per RFC 2047) and data within the body parts can be encoded
   on a part-by-part basis, with Content-Transfer-Encoding fields for
   each appropriate body part.

5.1.1.  Common Syntax

   This section defines a common syntax for subtypes of 'multipart'.
   All subtypes of 'multipart' must use this syntax.  A simple example
   of a multipart message also appears in this section.  An example of a
   more complex multipart message is given in RFC 2049.

   The Content-Type field for multipart entities requires one parameter,
   'boundary'. The boundary delimiter line is then defined as a line
   consisting entirely of two hyphen characters ('-', decimal value 45)
   followed by the boundary parameter value from the Content-Type header
   field, optional linear whitespace, and a terminating CRLF.

   NOTE:  The hyphens are for rough compatibility with the earlier RFC
   934 method of message encapsulation, and for ease of searching for
   the boundaries in some implementations.  However, it should be noted
   that multipart messages are NOT completely compatible with RFC 934
   encapsulations; in particular, they do not obey RFC 934 quoting
   conventions for embedded lines that begin with hyphens.  This
   mechanism was chosen over the RFC 934 mechanism because the latter
   causes lines to grow with each level of quoting.  The combination of
   this growth with the fact that SMTP implementations sometimes wrap
   long lines made the RFC 934 mechanism unsuitable for use in the event
   that deeply-nested multipart structuring is ever desired.

   WARNING TO IMPLEMENTORS:  The grammar for parameters on the Content-
   type field is such that it is often necessary to enclose the boundary
   parameter values in quotes on the Content-type line.  This is not
   always necessary, but never hurts. Implementors should be sure to
   study the grammar carefully in order to avoid producing invalid
   Content-type fields.  Thus, a typical 'multipart' Content-Type header
   field might look like this:

     Content-Type: multipart/mixed; boundary=gc0p4Jq0M2Yt08j34c0p

   But the following is not valid:

     Content-Type: multipart/mixed; boundary=gc0pJq0M:08jU534c0p

   (because of the colon) and must instead be represented as

     Content-Type: multipart/mixed; boundary='gc0pJq0M:08jU534c0p'

   This Content-Type value indicates that the content consists of one or
   more parts, each with a structure that is syntactically identical to
   an RFC 822 message, except that the header area is allowed to be
   completely empty, and that the parts are each preceded by the line

     --gc0pJq0M:08jU534c0p

   The boundary delimiter MUST occur at the beginning of a line, i.e.,
   following a CRLF, and the initial CRLF is considered to be attached
   to the boundary delimiter line rather than part of the preceding
   part.  The boundary may be followed by zero or more characters of
   linear whitespace. It is then terminated by either another CRLF and
   the header fields for the next part, or by two CRLFs, in which case
   there are no header fields for the next part.  If no Content-Type
   field is present it is assumed to be 'message/rfc822' in a
   'multipart/digest' and 'text/plain' otherwise.

   NOTE:  The CRLF preceding the boundary delimiter line is conceptually
   attached to the boundary so that it is possible to have a part that
   does not end with a CRLF (line  break).  Body parts that must be
   considered to end with line breaks, therefore, must have two CRLFs
   preceding the boundary delimiter line, the first of which is part of
   the preceding body part, and the second of which is part of the
   encapsulation boundary.

   Boundary delimiters must not appear within the encapsulated material,
   and must be no longer than 70 characters, not counting the two
   leading hyphens.

   The boundary delimiter line following the last body part is a
   distinguished delimiter that indicates that no further body parts
   will follow.  Such a delimiter line is identical to the previous
   delimiter lines, with the addition of two more hyphens after the
   boundary parameter value.

     --gc0pJq0M:08jU534c0p--

   NOTE TO IMPLEMENTORS:  Boundary string comparisons must compare the
   boundary value with the beginning of each candidate line.  An exact
   match of the entire candidate line is not required; it is sufficient
   that the boundary appear in its entirety following the CRLF.

   There appears to be room for additional information prior to the
   first boundary delimiter line and following the final boundary
   delimiter line.  These areas should generally be left blank, and
   implementations must ignore anything that appears before the first
   boundary delimiter line or after the last one.

   NOTE:  These 'preamble' and 'epilogue' areas are generally not used
   because of the lack of proper typing of these parts and the lack of
   clear semantics for handling these areas at gateways, particularly
   X.400 gateways.  However, rather than leaving the preamble area
   blank, many MIME implementations have found this to be a convenient
   place to insert an explanatory note for recipients who read the
   message with pre-MIME software, since such notes will be ignored by
   MIME-compliant software.

   NOTE:  Because boundary delimiters must not appear in the body parts
   being encapsulated, a user agent must exercise care to choose a
   unique boundary parameter value.  The boundary parameter value in the
   example above could have been the result of an algorithm designed to
   produce boundary delimiters with a very low probability of already
   existing in the data to be encapsulated without having to prescan the
   data.  Alternate algorithms might result in more 'readable' boundary
   delimiters for a recipient with an old user agent, but would require
   more attention to the possibility that the boundary delimiter might
   appear at the beginning of some line in the encapsulated part.  The
   simplest boundary delimiter line possible is something like '---',
   with a closing boundary delimiter line of '-----'.

   As a very simple example, the following multipart message has two
   parts, both of them plain text, one of them explicitly typed and one
   of them implicitly typed:

     From: Nathaniel Borenstein <nsb@bellcore.com>
     To: Ned Freed <ned@innosoft.com>
     Date: Sun, 21 Mar 1993 23:56:48 -0800 (PST)
     Subject: Sample message
     MIME-Version: 1.0
     Content-type: multipart/mixed; boundary='simple boundary'

     This is the preamble.  It is to be ignored, though it
     is a handy place for composition agents to include an
     explanatory note to non-MIME conformant readers.

     --simple boundary

     This is implicitly typed plain US-ASCII text.
     It does NOT end with a linebreak.
     --simple boundary
     Content-type: text/plain; charset=us-ascii

     This is explicitly typed plain US-ASCII text.
     It DOES end with a linebreak.

     --simple boundary--

     This is the epilogue.  It is also to be ignored.

   The use of a media type of 'multipart' in a body part within another
   'multipart' entity is explicitly allowed.  In such cases, for obvious
   reasons, care must be taken to ensure that each nested 'multipart'
   entity uses a different boundary delimiter.  See RFC 2049 for an
   example of nested 'multipart' entities.

   The use of the 'multipart' media type with only a single body part
   may be useful in certain contexts, and is explicitly permitted.

   NOTE: Experience has shown that a 'multipart' media type with a
   single body part is useful for sending non-text media types.  It has
   the advantage of providing the preamble as a place to include
   decoding instructions.  In addition, a number of SMTP gateways move
   or remove the MIME headers, and a clever MIME decoder can take a good
   guess at multipart boundaries even in the absence of the Content-Type
   header and thereby successfully decode the message.

   The only mandatory global parameter for the 'multipart' media type is
   the boundary parameter, which consists of 1 to 70 characters from a
   set of characters known to be very robust through mail gateways, and
   NOT ending with white space. (If a boundary delimiter line appears to
   end with white space, the white space must be presumed to have been
   added by a gateway, and must be deleted.)  It is formally specified
   by the following BNF:

     boundary := 0*69<bchars> bcharsnospace

     bchars := bcharsnospace / ' '

     bcharsnospace := DIGIT / ALPHA / ''' / '(' / ')' /
		      '+' / '_' / ',' / '-' / '.' /
		      '/' / ':' / '=' / '?'

   Overall, the body of a 'multipart' entity may be specified as
   follows:

     dash-boundary := '--' boundary
		      ; boundary taken from the value of
		      ; boundary parameter of the
		      ; Content-Type field.

     multipart-body := [preamble CRLF]
		       dash-boundary transport-padding CRLF
		       body-part *encapsulation
		       close-delimiter transport-padding
		       [CRLF epilogue]

     transport-padding := *LWSP-char
			  ; Composers MUST NOT generate
			  ; non-zero length transport
			  ; padding, but receivers MUST
			  ; be able to handle padding
			  ; added by message transports.

     encapsulation := delimiter transport-padding
		      CRLF body-part

     delimiter := CRLF dash-boundary

     close-delimiter := delimiter '--'

     preamble := discard-text

     epilogue := discard-text

     discard-text := *(*text CRLF) *text
		     ; May be ignored or discarded.

     body-part := MIME-part-headers [CRLF *OCTET]
		  ; Lines in a body-part must not start
		  ; with the specified dash-boundary and
		  ; the delimiter must not appear anywhere
		  ; in the body part.  Note that the
		  ; semantics of a body-part differ from
		  ; the semantics of a message, as
		  ; described in the text.

     OCTET := <any 0-255 octet value>

   IMPORTANT:  The free insertion of linear-white-space and RFC 822
   comments between the elements shown in this BNF is NOT allowed since
   this BNF does not specify a structured header field.

   NOTE:  In certain transport enclaves, RFC 822 restrictions such as
   the one that limits bodies to printable US-ASCII characters may not
   be in force. (That is, the transport domains may exist that resemble
   standard Internet mail transport as specified in RFC 821 and assumed
   by RFC 822, but without certain restrictions.) The relaxation of
   these restrictions should be construed as locally extending the
   definition of bodies, for example to include octets outside of the
   US-ASCII range, as long as these extensions are supported by the
   transport and adequately documented in the Content- Transfer-Encoding
   header field.  However, in no event are headers (either message
   headers or body part headers) allowed to contain anything other than
   US-ASCII characters.

   NOTE:  Conspicuously missing from the 'multipart' type is a notion of
   structured, related body parts. It is recommended that those wishing
   to provide more structured or integrated multipart messaging
   facilities should define subtypes of multipart that are syntactically
   identical but define relationships between the various parts. For
   example, subtypes of multipart could be defined that include a
   distinguished part which in turn is used to specify the relationships
   between the other parts, probably referring to them by their
   Content-ID field.  Old implementations will not recognize the new
   subtype if this approach is used, but will treat it as
   multipart/mixed and will thus be able to show the user the parts that
   are recognized.

5.1.2.  Handling Nested Messages and Multiparts

   The 'message/rfc822' subtype defined in a subsequent section of this
   document has no terminating condition other than running out of data.
   Similarly, an improperly truncated 'multipart' entity may not have
   any terminating boundary marker, and can turn up operationally due to
   mail system malfunctions.

   It is essential that such entities be handled correctly when they are
   themselves imbedded inside of another 'multipart' structure.  MIME
   implementations are therefore required to recognize outer level
   boundary markers at ANY level of inner nesting.  It is not sufficient
   to only check for the next expected marker or other terminating
   condition.

"
! !

!MultipartBody methodsFor:'accessing'!

addPart: part
	self parts add: part
!

boundary
	^self parent boundary
!

parent: aMimeEntity
	super parent: aMimeEntity.
	" Re-parent all my parts "
	self parts do: [ :part | part parent: aMimeEntity ]
!

partAt: anInteger
	^self parts at: anInteger
!

parts
	^value
!

parts: aCollection
	value := aCollection.
	value do: [ :part | part parent: self ].
!

removePart: aMimeEntity
	self parts remove: aMimeEntity
!

removePartAt: anInteger
	self parts remove: (self partAt: anInteger)
! !

!MultipartBody methodsFor:'constants'!

boundaryDashes
	^'--'
! !

!MultipartBody methodsFor:'copying'!

postCopy
	super postCopy.
	value := value collect: [ :part | part copy ].
! !

!MultipartBody methodsFor:'initialization'!

initialize
	self parts: (OrderedCollection new: 4)
! !

!MultipartBody methodsFor:'printing'!

conclusion
	^(self boundaryDashes, self boundary, self boundaryDashes) readStream
!

preamble
	^'This message is in Mime format\\' withCRs readStream
!

printStructureOn: aStream indent: level do: aBlock
	| boundary |

	(boundary := self boundary) isNil
					ifTrue: [ boundary := (#NoBoudarySpecified << #net >> '*** No Boudary Specified!!!!!! ***') ].
	self printLine: 'This message is in MIME format.' on: aStream indent: level.
	aStream cr.
	self parts
		do:
			[:part |
			self printLine: '--' ,  boundary on: aStream indent: level.
			aBlock value: part.
			aStream cr].
	self printLine: ('--' ,  boundary, '--') on: aStream indent: level.
! !

!MultipartBody methodsFor:'private -- parsing'!

doParse: scanner
	"Parse multi-part body structure "

	self
		scanPartsFrom: scanner
		useSourceStream: false
		do: [ :sourceStream | | part |
			(part := MimeEntity readFrom: sourceStream) notNil
				ifTrue: [self addPart: part]].
	^self parts
!

scanPartsFrom: scanner useSourceStream: aBoolean do: aBlock
	"Parse multi-part body structure "

	|  boundary |

	boundary := self boundary.
	"Skip to the first boundary, ignore text in between"
	boundary isNil ifTrue: [^self error: (#MissingBoundaryInMultipartBody << #net >> 'Missing boundary in multi-part body')].
	scanner scanToBoundary: boundary.
	scanner atEnd ifTrue: [^self error: (#MissingBoundaryInMultipartBody << #net >> 'Missing boundary in multi-part body')].
	[self
		scanToBoundary: boundary
		from: scanner asStream
		useSourceStream: aBoolean
		do:
			[:partStream :lastPart |
			aBlock value: partStream.
			lastPart ifTrue: [^self]]] repeat
!

scanToBoundary: boundary from: stream useSourceStream: aBoolean do: aDyadicBlock
	"Scan for specified boundary (RFC2046, p5.1). Answer two-element array. First element is read
	stream on scanned text from current position up to the beginning of the boundary. Second element
	is either #next or #last. #next means the boundary found is not the last one. #last means the
	boundary is the closing boundary for the multi-part body (that is, it looks like '--<boundary>--)"

	| pattern partStream startPos isLastPart nextPos savePos lineEnd str |

	lineEnd := stream lineEndConvention.
"The boundary delimiter MUST occur at the beginning of a line, i.e.,   following a CRLF, and the initial CRLF is considered to be attached to the boundary delimiter line rather than part of the preceding part. For internal stream the boundary delimiter can start from CR"
	str := (lineEnd = LineEndCR
		ifTrue: [String with: Character cr ]
		ifFalse: [ String with: Character cr with: Character lf ]).
	pattern := str, self boundaryDashes , boundary.

	[ stream lineEndTransparent.

	partStream := aBoolean
				ifTrue:
					[startPos := stream position.
					stream skipToAll: pattern.
					stream asStream from: startPos to: stream position]
				ifFalse: [(stream upToAndSkipThroughAll: pattern) readStream].

	"Skip over pattern"
	savePos := stream position.
	isLastPart := (stream atEnd or:
					[[(stream next: 2) = self boundaryDashes]
						on: Stream incompleteNextCountSignal
						do: [:ex | ex return: true]])
				ifTrue: [true]
				ifFalse:        [stream position: savePos.
						false].

	"Skip to the end of line"
	stream upToAndSkipThroughAll: str.

	] ensure: [ stream lineEndConvention: lineEnd ].

	nextPos := stream position.
	[aDyadicBlock value: partStream value: isLastPart]
		ensure: [stream position: nextPos]
! !

!MultipartBody methodsFor:'testing'!

isMultipart
	^true
!

isSimple
	^false
!

notEmpty
	^true
! !

!MultipartBody methodsFor:'transfer encoding'!

decodeAs: encodingName
	" Do nothing -- content transfer encoding does not apply to multipart bodies "
!

encodeAs: encodingName
	" Do nothing -- content transfer encoding does not apply to multipart bodies "
!

needsContentTransferEncoding
	^false
! !

!MultipartBody methodsFor:'visiting'!

acceptVisit: aVisitor with: argument
	^aVisitor acceptMultipartBody: self with: argument
! !

!MultipartBody methodsFor:'writing'!

writeContentsOn: aStream
	self shouldNotImplement
! !

!MultipartBody class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__MultipartBody.st,v 1.1 2004/11/12 10:42:28 penk Exp $'
! !
