"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#NetClient
	instanceVariableNames:'hostName portNumber user reporter clientPI retries timeout'
	classVariableNames:''
	poolDictionaries:'Net::TCP'
	category:'NetClients-Framework'
!

NetClient comment:'NetClient class is abstract class to support high level interface to the Internet protocols. The class defines the connection protocol for the subclasses.

Instance Variables:

	user    <NetUser>       Provides user name and password to login
	hostName        <String>        Provides the host name to connect
	portNumber      <Number>        Provides the port number to connect
	clientPI        <IMAPProtocolInterpreter>       Responsible for sending commands to the host and receiving responses
	retries <Number>        The number of attempts to connect to the host
	timeout <Number>        The number of milliseconds to wait for the command completion
	delaySeconds    <Number>        The number of milliseconds to wait between retries'
!


!NetClient class methodsFor:'instance creation'!

connectToHost: aHostname
"Create NetClient object, establish a connection to the host <aString>"
	^self new connectToHost: aHostname port: nil
!

connectToHost: aHostname port: aPort
"Create NetClient object, establish a connection to the host <aString> and port <aString>"
	^self new connectToHost: aHostname port: aPort
!

host: aHostname
"Create NetClient object, set up the host <aString>"
	^(self new)
		hostName: aHostname;
		yourself
!

host: aHostname port: aPort
"Create NetClient object, set up the host <aString> and port <aString>"
	^(self new)
		hostName: aHostname;
		portNumber: aPort;
		yourself
!

loginToHost: aHostName asUser: userString withPassword: passwdString
"Create NetClient object, connect and login to the host <aString>  using the specified user name<aString>&  password<aString> "
	^self loginToHost: aHostName port: nil asUser: userString withPassword: passwdString
!

loginToHost: aHostName port: aNumber asUser: userString withPassword: passwdString
"Create NetClient object, connect and login to the host <aString>  using the specified user name<aString>,  password<aString> & port <aNumber>"
	^self new
		loginToHost: aHostName
		port: aNumber
		asUser: userString
		withPassword: passwdString
!

loginUser: userString withPassword: passwdString
    ^self loginUser: userString withPassword: passwdString
! !

!NetClient class methodsFor:'debugging'!

log: aString level: aLevel
    Transcript showCR:aString.
    "/ LogEnvironment default log: aString level: aLevel
!

registerToDebug
    "/ LogEnvironment default addToDebug: self
! !

!NetClient class methodsFor:'defaults'!

defaultPortNumber
    ^ self subclassResponsibility
! !

!NetClient methodsFor:'abstract'!

protocolInterpreter
    self subclassResponsibility
!

protocolInterpreterClass
    self subclassResponsibility
! !

!NetClient methodsFor:'accessing'!

clientPI
    clientPI isNil ifTrue:[
	clientPI := self protocolInterpreterClass new.
    ].
    ^clientPI
!

clientPI: aProtocolInterpreter
    clientPI := aProtocolInterpreter
!

hostName
    ^hostName
!

hostName: aString
    hostName := aString
!

password

    ^self user password
!

portNumber

    (portNumber isNil or:[portNumber = 0]) ifTrue: [ ^ self defaultPortNumber ].
    ^portNumber
!

portNumber: aNumber
    portNumber := aNumber
!

reporter
    reporter isNil ifTrue: [ reporter := Reporter new ].
    ^reporter
!

reporter: aReporter
    reporter := aReporter
!

retries
	retries isNil ifTrue: [retries := self defaultRetries].
	^retries
!

retries: aNumber
	retries := aNumber
!

timeout

	timeout isNil ifTrue: [ timeout := self defaultTimeout].
	^timeout
!

timeout: aNumber

	timeout := aNumber
!

user
    ^user
!

user: aNetUser
    user := aNetUser
!

username
    ^user username
!

username: usernameString password: passwdString
    user := NetUser username: usernameString password: passwdString
! !

!NetClient methodsFor:'connection'!

close
    ^self logout
!

connect
	"Establish a connection to the host"

	^self clientPI connectToHost: self hostName port: self portNumber.
!

connectIfClosed
    self clientPI closed
	ifTrue: [self connect ]
!

connectToHost: aString
	"Establish a connection to the host <aString>."

	^ self connectToHost: aString port: nil
!

connectToHost: aString port: aNumber
	"Establish a connection to the host <aString>."

	self hostName: aString.
	self portNumber: aNumber.
	self connect.
!

disconnect
	"Disconnect from the host"
	self logout.
	clientPI disconnect.
	clientPI := nil
!

isConnected
	^SubclassResponsibilityError
!

login
!

loginAsUser: userString withPassword: passwdString
"Login to the host using the specified user name <aString>  and password<aString>"
	user := NetUser username: userString password: passwdString.
	self loginToHost
!

loginToHost
"Login to the host"
	self login
!

loginToHost: aHostName asUser: userString withPassword: passwdString
"Connect and login to the host <aString>  using the specified user name<aString> and password<aString>"
	^self
		loginToHost: aHostName
		port: nil
		asUser: userString
		withPassword: passwdString
!

loginToHost: aHostName port: aNumber asUser: userString withPassword: passwdString
"Connect and login to the host <aString>  using the specified user name<aString> and password<aString>"
	self connectToHost: aHostName port: aNumber.
	self loginAsUser: userString withPassword: passwdString.
!

logout
!

reconnect
	"Reconnect to the host"
	self disconnect.
	self connect.
! !

!NetClient methodsFor:'defaults'!

defaultDelaySeconds
	^10
!

defaultPortNumber
	^self class defaultPortNumber
!

defaultRetries
	^5
!

defaultTimeout
	^20000
! !

!NetClient methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: 'a', self class name.
	self hostName notNil
		ifTrue: [aStream
					nextPutAll: ' on: ';
					nextPutAll: self hostName;
					nextPut: $:;
					nextPutAll: self portNumber printString].
! !

!NetClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetClient.st,v 1.3 2009/07/27 16:53:36 cg Exp $'
! !
