"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

NetworkEntityDescriptor subclass:#NetworkAddressDescriptor
	instanceVariableNames:'domain localPart route'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

NetworkAddressDescriptor comment:'This class represents RFC822 network addresses. An address may have address part, alias and comment. Each one is optional.

Address syntax from rfc822 is as follows:

     address     =  mailbox                      ; one addressee
		 /  group                        ; named list

     group       =  phrase ":" [#mailbox] ";"

     mailbox     =  addr-spec                    ; simple address
		 /  phrase route-addr            ; name & addr-spec

     route-addr  =  "<" [route] addr-spec ">"

     route       =  1#("@" domain) ":"           ; path-relative

     addr-spec   =  local-part "@" domain        ; global address

     local-part  =  word *("." word)             ; uninterpreted
						 ; case-preserved

     domain      =  sub-domain *("." sub-domain)

     sub-domain  =  domain-ref / domain-literal

     domain-ref  =  atom                         ; symbolic reference


Instance Variables:
	domain  <SequencableCollection of Strings>  Domain parts domainString answers domain string
	localPart       <SequencableCollection of Strings>  Parts of local part. localPartString answers string
	route   <Collection | EncodedStream | PositionableStream>  Route part of the address. routeString answers route string
'
!


!NetworkAddressDescriptor class methodsFor:'parsing'!

parser
	^self scannerType new
!

scannerType
	^NetworkAddressParser
! !

!NetworkAddressDescriptor class methodsFor:'utility'!

addressFrom: aStream
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp'."
	"self addressesFrom: 'Kazuki Yasumatsu <kyasu@crl.fujixerox.co.jp>'."
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp (Kazuki Yasumatsu)'."
	"self addressesFrom: ' kyasu1, kyasu2, Kazuki Yasumatsu <kyasu3>, kyasu4 (Kazuki Yasumatsu)'."
	"self addressesFrom: ' foo bar, kyasu1, ,  Kazuki Yasumatsu <kyasu2> <kyasu3> (<foo> (foo bar), bar)'."

	^self scannerType addressFrom: aStream
!

addressesFrom: stream
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp' readStream."
	"self addressesFrom: 'Kazuki Yasumatsu <kyasu@crl.fujixerox.co.jp>' readStream."
	"self addressesFrom: 'kyasu@crl.fujixerox.co.jp (Kazuki Yasumatsu)' readStream."
	"self addressesFrom: ' kyasu1, kyasu2, Kazuki Yasumatsu <kyasu3>, kyasu4 (Kazuki Yasumatsu)' readStream."
	"self addressesFrom: ' foo bar, kyasu1, ,  Kazuki Yasumatsu <kyasu2> <kyasu3> (<foo> (foo bar), bar)' readStream."

	^self scannerType addressesFrom: stream asStream
! !

!NetworkAddressDescriptor methodsFor:'accessing'!

addressSpecString
	^self printStringSelector: #printAddressSpecOn:
!

aliasString
	^self printStringSelector: #printAliasOn:
!

commentString
	^self printStringSelector: #printCommentOn:
!

domain

	^domain
!

domain: aValue

	domain := aValue
!

domainString
	^self printStringSelector: #printDomainOn:
!

localPart

	^localPart
!

localPart: aValue

	localPart := aValue
!

localPartString
	^self printStringSelector: #printLocalPartOn:
!

route

	^route
!

route: aValue

	route := aValue
!

routeString
	^self printStringSelector: #printRouteOn:
! !

!NetworkAddressDescriptor methodsFor:'composing'!

writeCanonicalValueOn: stream encoding: aString
	alias notNil
		ifTrue:
			[self writeAliasOn: stream encoding: aString.
			self writeCommentOn: stream encoding: aString.
			self printRouteAddressOn: stream]
		ifFalse:        [ self printAddressSpecOn: stream].
! !

!NetworkAddressDescriptor methodsFor:'initialization'!

initialize
	localPart := Array new.
! !

!NetworkAddressDescriptor methodsFor:'printing'!

printAddressSpecOn: stream
	self hasAddressSpec
		ifTrue:
			[self printLocalPartOn: stream.
			stream nextPut: $@.
			self printDomainOn: stream]
		ifFalse:
			[(alias notNil and: [ alias notEmpty ])
				ifTrue: [self scannerType printWord: alias on: stream ]]
!

printCanonicalValueOn: stream
	alias notNil
		ifTrue:
			[self printAliasOn: stream.
			self printRouteAddressOn: stream]
		ifFalse: [self printAddressSpecOn: stream].
!

printDomainOn: stream
	self scannerType printDomain: domain on: stream
!

printLocalPartOn: stream
	localPart do: [ :token | self scannerType printWord: token on: stream ] separatedBy: [ stream nextPut: $. ]
!

printRouteAddressOn: stream
	(route notNil or: [self hasAddressSpec])
		ifTrue:
			[stream nextPut: $<.
			self printRouteOn: stream; printAddressSpecOn: stream.
			stream nextPut: $>]
!

printRouteOn: stream
	(route notNil and: [route isEmpty not])
		ifTrue:
			[route
				do:
					[:domainx |
					stream space; nextPut: $@.
					self scannerType printDomain: domainx on: stream.
					stream nextPut: $:].
			stream space]
! !

!NetworkAddressDescriptor methodsFor:'private'!

printStringSelector: sel
	| stream |
	stream := (String new: 64) writeStream.
	self perform: sel with: stream.
	^stream contents
! !

!NetworkAddressDescriptor methodsFor:'testing'!

hasAddressSpec
	^localPart notNil and: [localPart isEmpty not and: [domain notNil and: [domain isEmpty not]]]
! !

!NetworkAddressDescriptor class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__NetworkAddressDescriptor.st,v 1.1 2004/11/12 10:40:41 penk Exp $'
! !
