"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MailScanner subclass:#RFC822Scanner
	instanceVariableNames:''
	classVariableNames:'QuotedPairChar SimpleTimeZones DomainTextMask HeaderNameMask
		HeaderNameDelimiterChar CommentMask TextMask AtomMask
		QuotedPairMask TokenMask QuotedTextMask'
	poolDictionaries:''
	category:'NetClients-Framework'
!

RFC822Scanner comment:'This class performs scanning/parsing of rfc822 messages. It implements scanning routines for rfc822 lexical types (see ''multi-character scans'') and general tokenization logic.

Shared Variables:
	AtomMask        <Integer>  Mask for rfc822 Atoms
	CommentMask     <Integer>  Mask for rfc822 Comments ()
	DomainTextMask  <Integer>  Mask for rfc822 Domain Text
	HeaderNameDelimiterChar <Character>  $:
	HeaderNameMask  <Integer>  Mask for rfc822 header field names
	QuotedPairChar  <Character>  $\
	QuotedPairMask  <Integer>  Mask for rfc822 quoted-pair (\x)
	QuotedTextMask  <Integer>  Mask for rfc822 quoted-text
	SimpleTimeZones <Dictionary key: ?type? value: ?type?>  comment
	TextMask        <Integer>  Mask for rfc822 text
	TokenMask       <Integer>  Mask for rfc822 tokens
'
!


!RFC822Scanner class methodsFor:'character classification'!

specials
	" Note that definition of this set varies from standard to standard, so this method needs to be overridden for specialized parsers "
	^'()<>@,;:\".[]'
!

tspecials
	" tspecials in MIME and HTTP. It is derived from RCC822 specials with addition of </>, <?>, <=> and remova; of <.> "
	^'()<>@,;:\"/[]?='
! !

!RFC822Scanner class methodsFor:'class initialization'!

initClassificationTable
	super initClassificationTable.
	self initClassificationTableWith: HeaderNameMask when:
		[:c | c > Character space and: [c ~~ $:]].
	self initClassificationTableWith: TextMask when:
		[:c | c ~~ Character return ].
	self initClassificationTableWith: AtomMask when:
		[:c | c > Character space and: [ (self specials includes: c) not ] ].
	self initClassificationTableWith: TokenMask when:
		[:c | c > Character space and: [ (self tspecials includes: c) not ] ].
	self initClassificationTableWith: QuotedTextMask when:
		[:c | c ~~ $" and: [ c ~~ $\ and: [ c ~~ Character return ]]].
	self initClassificationTableWith: DomainTextMask when:
		[:c | ('[]\' includes: c) not and: [ c ~~ Character return ]].
	self initClassificationTableWith: CommentMask when:
		[:c | c ~~ $( and: [ c ~~ $) and: [ c ~~ $\ and: [ c ~~ Character return ]]]].
!

initialize
	" RFC822Scanner initialize "

	self initializeConstants; initClassificationTable
!

initializeConstants
	AtomMask := 256.
	CommentMask := 512.
	DomainTextMask := 1024.
	HeaderNameMask := 2048.
	QuotedTextMask := 4096.
	TextMask := 8192.
	TokenMask := 16384.
	QuotedPairMask := (QuotedTextMask bitOr: CommentMask) bitOr: DomainTextMask.
	QuotedPairChar := $\.
	HeaderNameDelimiterChar := $:.

	"Install TimeZone constants."

	SimpleTimeZones := Dictionary new.

	"Universal Time"
	SimpleTimeZones at: 'UT' put: 0.
	SimpleTimeZones at: 'GMT' put: 0.

	"For North America."
	SimpleTimeZones at: 'EST' put: -5.
	SimpleTimeZones at: 'EDT' put: -4.
	SimpleTimeZones at: 'CST' put: -6.
	SimpleTimeZones at: 'CDT' put: -5.
	SimpleTimeZones at: 'MST' put: -7.
	SimpleTimeZones at: 'MDT' put: -6.
	SimpleTimeZones at: 'PST' put: -8.
	SimpleTimeZones at: 'PDT' put: -7.

	"For Europe."
	SimpleTimeZones at: 'BST' put: 0.
	SimpleTimeZones at: 'WET' put: 0.
	SimpleTimeZones at: 'MET' put: 1.
	SimpleTimeZones at: 'EET' put: 2.

	"For Japan."
	SimpleTimeZones at: 'JST' put: 9
! !

!RFC822Scanner class methodsFor:'from Network Clients'!

correctDateForY2K: date
" Arbitrarily decide that if year is >2025, subtract one century "
	date year > 2025
		ifTrue: [
		    Smalltalk isSmalltalkX ifTrue:[
			^ Date newDay:date day month:date month year:(date year - 100)
		    ] ifFalse:[
			date day: date day year: date year - 100
		    ].
		].
	^date
!

currentGMTDateAndTime

	^TimeZone null
		convertGMT: Time secondClock
		do:
			[:date :seconds |
			Array with: date with: (Time fromSeconds: seconds)]
!

dateAndTimeFrom: aString
	"RFC822Scanner dateAndTimeFrom: '6 Dec 88 10:16:08 +0900 (Tuesday)'."
	"RFC822Scanner dateAndTimeFrom: '12 Dec 88 10:16:08 +0900 (Tuesday)'."
	"RFC822Scanner dateAndTimeFrom: 'Fri, 31 Mar 89 09:13:20 +0900'."
	"RFC822Scanner dateAndTimeFrom: 'Tue, 18 Apr 89 23:29:47 +0900'."
	"RFC822Scanner dateAndTimeFrom: 'Tue, 23 May 89 13:52:12 JST'."
	"RFC822Scanner dateAndTimeFrom: 'Thu, 1 Dec 88 17:13:27 jst'."
	"RFC822Scanner dateAndTimeFrom: 'Sat, 15 Jul 95 14:36:22 0900'."
	"RFC822Scanner dateAndTimeFrom: '2-Nov-86 10:43:42 PST'."
	"RFC822Scanner dateAndTimeFrom: 'Friday, 21-Jul-95 04:04:55 GMT'."
	"RFC822Scanner dateAndTimeFrom: 'Jul 10 11:06:40 1995'."
	"RFC822Scanner dateAndTimeFrom: 'Jul 10 11:06:40 JST 1995'."
	"RFC822Scanner dateAndTimeFrom: 'Mon Jul 10 11:06:40 1995'."
	"RFC822Scanner dateAndTimeFrom: 'Mon Jul 10 11:06:40 JST 1995'."
	"RFC822Scanner dateAndTimeFrom: '(6 December 1988 10:16:08 am )'."
	"RFC822Scanner dateAndTimeFrom: '(12 December 1988 10:16:08 am )'."
	"RFC822Scanner dateAndTimeFrom: ''."

	| rfcString |
	aString size <= 10      "may be illegal format"
		ifTrue: [^self currentGMTDateAndTime].
	rfcString := self normalizeDateAndTimeString: aString.
	^self readRFC822DateAndTimeFrom: rfcString readStream
!

dateStringFrom: dateAndTime
	"Return Date dateAndTimeNow in the Unix mail format"

	" self dateAndTimeFrom: (Array with: Date today with: Time now) "

	| stream temp indx d seconds hours minutes |
	stream := WriteStream on: (String new: 40).
	indx := dateAndTime first weekdayIndex.
	d := #(#Mon #Tue #Wed #Thr #Fri #Sat #Sun) at: indx.
	stream nextPutAll: d.
	stream nextPutAll: ', '.
	dateAndTime first printOn: stream format: #(1 2 3 $  2 1).
	stream nextPutAll: ((temp := dateAndTime last hours) < 10
				ifTrue: [' 0']
				ifFalse: [' ']).
	temp printOn: stream.
	stream nextPutAll: ((temp := dateAndTime last minutes) < 10
				ifTrue: [':0']
				ifFalse: [':']).
	temp printOn: stream.
	stream nextPutAll: ((temp := dateAndTime last seconds) < 10
				ifTrue: [':0']
				ifFalse: [':']).
	temp printOn: stream.

	seconds := Time secondClock.
Smalltalk isSmalltalkX ifTrue:[
	seconds := Timestamp now utcOffset
] ifFalse:[
	seconds := seconds
				- (TimeZone default convertLocalSecondsToGMT: seconds).
].
	seconds negative
		ifTrue: [stream nextPutAll: ' -']
		ifFalse: [stream nextPutAll: ' +'].
	hours := seconds abs // 3600.
	hours abs < 10 ifTrue: [stream nextPut: $0].
	stream print: hours abs.
	minutes := seconds abs \\ 3600 // 60.
	minutes abs < 10 ifTrue: [stream nextPut: $0].
	stream print: minutes.
	^stream contents
!

defaultTimeZoneDifference
	^TimeZone default instVarAt: 1
!

initializeTimeZones
	"RFC822Scanner initializeTimeZones."
	"Install TimeZone constants."

	SimpleTimeZones := Dictionary new.

	"Universal Time"
	SimpleTimeZones at: 'UT' put: 0.
	SimpleTimeZones at: 'GMT' put: 0.

	"For North America."
	SimpleTimeZones at: 'EST' put: -5.
	SimpleTimeZones at: 'EDT' put: -4.
	SimpleTimeZones at: 'CST' put: -6.
	SimpleTimeZones at: 'CDT' put: -5.
	SimpleTimeZones at: 'MST' put: -7.
	SimpleTimeZones at: 'MDT' put: -6.
	SimpleTimeZones at: 'PST' put: -8.
	SimpleTimeZones at: 'PDT' put: -7.

	"For Europe."
	SimpleTimeZones at: 'BST' put: 0.
	SimpleTimeZones at: 'WET' put: 0.
	SimpleTimeZones at: 'MET' put: 1.
	SimpleTimeZones at: 'EET' put: 2.

	"For Japan."
	SimpleTimeZones at: 'JST' put: 9
!

normalizeDateAndTimeString: aString
	"RFC822 formats"
	"RFC822Scanner normalizeDateAndTimeString: '6 Dec 88 10:16:08 +0900 (Tuesday)'."
	"RFC822Scanner normalizeDateAndTimeString: 'Tue, 18 Apr 89 23:29:47 +0900'."
	"RFC822Scanner normalizeDateAndTimeString: 'Tue, 18 Apr 89 23:29:47 0900'."
	"RFC822Scanner normalizeDateAndTimeString: 'Tue, 23 May 89 13:52:12 JST'."
	"RFC822Scanner normalizeDateAndTimeString: '2-Nov-86 10:43:42 PST'."
	"Other formats"
	"RFC822Scanner normalizeDateAndTimeString: 'Jul 10 11:06:40 1995'."
	"RFC822Scanner normalizeDateAndTimeString: 'Jul 10 11:06:40 JST 1995'."
	"RFC822Scanner normalizeDateAndTimeString: 'Mon Jul 10 11:06:40 1995'."
	"RFC822Scanner normalizeDateAndTimeString: 'Mon Jul 10 11:06:40 JST 1995'."

	| head tail read str1 str2 write |
	aString size < 6 ifTrue: [^aString].
	head := aString copyFrom: 1 to: aString size - 5.
	(head indexOf: $,) > 0 ifTrue: [^aString].
	tail := aString copyFrom: aString size - 4 to: aString size.
	read := tail readStream.
	(read next = Character space and:
	[read next isDigit and:
	[read next isDigit and:
	[read next isDigit and:
	[read next isDigit]]]])
		ifFalse: [^aString].
	read := head readStream.
	str1 := read through: Character space.
	str2 := read through: Character space.
	(str1 isEmpty or: [str2 isEmpty]) ifTrue: [^aString].
	str2 first isDigit
		ifFalse:
			[str1 := str2.
			str2 := read through: Character space.
			(str2 isEmpty or: [str2 first isDigit not]) ifTrue: [^aString]].
	read atEnd ifTrue: [^aString].
	write := WriteStream on: (String new: 32).
	write nextPutAll: str2;
		nextPutAll: str1;
		nextPutAll: (tail copyFrom: 4 to: 5);
		space;
		nextPutAll: read.
	^write contents
!

readDateFrom: aStream
	"date   =  1*2DIGIT month 2DIGIT
	month   =  'Jan'  /  'Feb' /  'Mar'  /  'Apr'
			/  'May'  /  'Jun' /  'Jul'  /  'Aug'
			/  'Sep'  /  'Oct' /  'Nov'  /  'Dec'"
	"RFC822Scanner readDateFrom: '01 Jan 95' readStream."
	"RFC822Scanner readDateFrom: '1 Jan 95' readStream."
	"RFC822Scanner readDateFrom: '23 Jan 95' readStream."
	"RFC822Scanner readDateFrom: '23-Jan-95' readStream."
	"RFC822Scanner readDateFrom: 'Jan 23 95' readStream."
	"RFC822Scanner readDateFrom: 'Jan 23 1995' readStream."

	Smalltalk isSmalltalkX ifTrue:[
	    ^ Date readFrom:aStream
	].

	^(TimestampPrintPolicy newFor: #rfc822) reader readDateFrom: aStream
!

readRFC822DateAndTimeFrom: aStream
	"date-time      =  [ day ',' ] date time
	day                     =  'Mon'  / 'Tue' /  'Wed'  / 'Thu'
				/  'Fri'  / 'Sat' /  'Sun'"
	"RFC822Scanner readRFC822DateAndTimeFrom: '6 Dec 88 10:16:08 +0900 (Tuesday)' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: '12 Dec 88 10:16:08 +0900 (Tuesday)' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: 'Date: Mon, 7 Nov 4004 21:52:25 -0800 (PST)' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: 'Fri, 31 Mar 89 09:13:20 +0900' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: 'Tue, 18 Apr 89 23:29:47 +0900' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: 'Tue, 23 May 89 13:52:12 JST' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: 'Thu, 1 Dec 88 17:13:27 jst' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: '2-Nov-86 10:43:42 PST' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: '(6 December 1988 10:16:08 am )' readStream."
	"RFC822Scanner readRFC822DateAndTimeFrom: '(12 December 1988 10:16:08 am )' readStream."

	| char date time |
	[aStream atEnd or:
	[char := aStream peek.
	char isDigit]] whileFalse:
		[aStream next].
	aStream atEnd ifTrue: [^self currentGMTDateAndTime].
	date := self readDateFrom: aStream.
	self correctDateForY2K: date.
	aStream skipSeparators.
	time := self readTimeFrom: aStream.
	^List with: date with: time
!

readTimeFrom: aStream
	"time   = hour zone
	hour    = 2DIGIT ':' 2DIGIT [':' 2DIGIT]
	zone    = 'UT' / 'GMT'
	/ 'EST' / 'EDT'
	/ 'CST' / 'CDT'
	/ 'MST' / 'MDT'
	/ 'PST' / 'PDT'
	/ 1ALPHA
	/ ( ('+' / '-') 4DIGIT )"
	"RFC822Scanner readTimeFrom: ' 00:31:53 +0200' readStream."
	"RFC822Scanner readTimeFrom: ' 20:09:28 -0400'readStream."
	"RFC822Scanner readTimeFrom: ' 04:46:04 -0300' readStream."
	"RFC822Scanner readTimeFrom: ' 19:03:47 -0700' readStream."
	"RFC822Scanner readTimeFrom: '21:16:08 JST' readStream."
	"RFC822Scanner readTimeFrom: '21:16:08 jst' readStream."
	"RFC822Scanner readTimeFrom: '21:16:08 MET' readStream."
	"RFC822Scanner readTimeFrom: '21:16:08 -0900' readStream."
	"RFC822Scanner readTimeFrom: '21:16:08 0900' readStream."
	"RFC822Scanner readTimeFrom: '12:16:08 pm' readStream."
	"Smalltalk time"
	"RFC822Scanner readTimeFrom: '12:16' readStream."
	"No timezone"
	"RFC822Scanner readTimeFrom: '12:16:08' readStream."
	"No timezone"
	| "hour minute" write char timezone time sec |
	aStream skipSeparators.
	time := Time readFrom: aStream.
	aStream skipSeparators.
	write := WriteStream on: (String new: 8).
	[aStream atEnd
		or:
			[char := aStream next.
			char isSeparator]]
		whileFalse: [write nextPut: char].
	timezone := write contents asUppercase.
	timezone isEmpty ifTrue: [ ^time ].
	('#####' match: timezone)
		ifTrue:
			[sec := 60*60* (( (timezone copyFrom: 2 to: 3) asNumber) + (timezone copyFrom: 4 to: 5) asNumber)].

	(SimpleTimeZones at: timezone ifAbsent: [nil]) notNil
		ifTrue:
			[ sec := 60*60* ( (SimpleTimeZones at: timezone) asNumber).
			time := time subtractTime: (Time fromSeconds: sec) ]
		ifFalse: [('+####' match: timezone)
				ifTrue:
					[ time := time subtractTime:  (Time fromSeconds: sec) ]
				ifFalse: [('-####' match: timezone)
						ifTrue: [ time := time addTime: (Time fromSeconds: sec) ]]].

"/ ST/X bug: always uses localtime for Time
Smalltalk isSmalltalkX ifTrue:[
    time := time addSeconds:(Timestamp now utcOffset).
].
	^time
! !

!RFC822Scanner class methodsFor:'printing'!

defaultTokenType
	^#word
!

printComment: comment on: stream
	comment notNil ifTrue: [
		stream nextPut: $(.
		comment
			do:
				[:char |
				(self isCommentChar: char)
					ifFalse: [stream nextPut: $\].
				stream nextPut: char].
		stream nextPut: $).
	]
!

printDomain: domainx on: stream
	"Domainx is an array of domain segments"

	domainx notNil ifTrue: [domainx do: [:word | self printWord: word on: stream]
			separatedBy: [stream nextPut: $.]]
!

printPhrase: phrase on: stream
	phrase do: [ :word | stream nextPutAll: word] separatedBy: [ stream space ]
!

printWord: str on: stream
	"Print word as either atom or quoted text"

	(self shouldBeQuoted: str)
		ifTrue: [stream nextPut: $"; nextPutAll: str; nextPut: $"]
		ifFalse: [stream nextPutAll: str]
! !

!RFC822Scanner class methodsFor:'testing'!

isAtomChar: char
	^((self classificationTable at: char asInteger + 1) bitAnd: AtomMask) ~= 0
!

isCommentChar: char
	^((self classificationTable at: char asInteger + 1) bitAnd: CommentMask) ~= 0
!

shouldBeQuoted: string
	^(string detect: [ :char | (self isAtomChar: char) not ] ifNone: [ nil ]) notNil
! !

!RFC822Scanner methodsFor:'converting'!

phraseAsString: phrase
	| stream |
	stream := (String new: 40) writeStream.
	self class printPhrase: phrase on: stream.
	^stream contents.
! !

!RFC822Scanner methodsFor:'multi-character scans'!

scanAtom
" atom  =  1*<any CHAR except specials, SPACE and CTLs> "

	token := self scanTokenMask: AtomMask.
	tokenType := #atom.
	^token.
!

scanComment
	"collect comment"

	| output |

	output := saveComments
				ifTrue: [(String new: 40) writeStream]
				ifFalse: [nil].

	self scanCommentOn: output.
	output notNil ifTrue: [
		currentComment isNil
			ifTrue: [currentComment := OrderedCollection with: output contents]
			ifFalse: [currentComment add: output contents]].
	^token
!

scanDate
	^self class readRFC822DateAndTimeFrom: self asStream
!

scanDomainText
	"dtext = <any CHAR excluding <[>, <]>, <\> & CR, & including linear-white-space> ; => may be folded"

	token := self
				scanToken: [self scanQuotedChar; matchCharacterType: DomainTextMask]
				delimitedBy: '[]'
				notify: 'Malformed domain literal'.
	tokenType := #domainText.
	^token
!

scanEndOfLine
	"Note: this will work only for RFC822 but not for HTTP. Needs more design work"

	(self matchCharacterType: CRLFMask)
		ifTrue:
			[(hereChar == Character return)
				ifTrue:
					[self shouldFoldLine ifTrue: [
						self hereChar: Character space.
						^self]].

			"Otherwise we have an end-of-line condition -- set appropriate masks"
			classificationMask := (classificationMask maskClear: WhiteSpaceMask)
						maskSet: EndOfLineMask]
!

scanFieldName
	" RFC822, p.9: field-name = 1*<any CHAR excluding CTLs, SPACE and ':'> "
	^self scanTokenMask: HeaderNameMask
!

scanPhrase
	"RFC822: phrase = 1*word ; Sequence of words. At the end of scan the scanner has read the first token after phrase "

	^self tokenizeWhile: [#(#quotedText #atom) includes: tokenType]
!

scanQuotedChar
	"Scan possible quoted character. If the current char is $\, read in next character and make it a quoted
	string character"

	(hereChar == QuotedPairChar)
		ifTrue:
			[self step.
			classificationMask := QuotedPairMask.
			^true]
		ifFalse: [^false]
!

scanQuotedText
" quoted-string = <""> *(qtext/quoted-pair) <"">; Regular qtext or quoted chars.
  qtext    =  <any CHAR excepting <"">, <\> & CR, and including linear-white-space>  ; => may be folded"

	" We are positioned at the first double quote character "
	token := self scanToken: [ self scanQuotedChar; matchCharacterType: QuotedTextMask ] delimitedBy: '""' notify: 'Unmatched quoted text'.
	tokenType := #quotedText.
	^token.
!

scanText
	"RFC822: text = <Any CHAR, including bare CR & bare LF, but not including CRLF. This is a 'catchall' category and cannot be tokenized. Text is used only to read values of unstructured fields"

	^self scanUntil: [ self matchCharacterType: CRLFMask ].
!

scanWord
	self nextToken.
	(#(#quotedText #atom) includes: tokenType)
			ifFalse: [self error: 'Expecting word'].
	^token
!

skipWhiteSpace
	"It is inefficient because intermediate stream is created. Perhaps refactoring scanWhile: can help"

	self scanWhile: [hereChar == $(
			ifTrue:
				[self stepBack; scanComment.
				true]
			ifFalse: [self matchCharacterType: WhiteSpaceMask]]
! !

!RFC822Scanner methodsFor:'private'!

nextRFC822Token
	| char |
	self skipWhiteSpace.
	char := self peek.
	char isNil                              "end of input"
		ifTrue: [tokenType := #doIt.
			^token := nil].
	char == $( ifTrue: [^self scanComment; nextToken].
	char == $" ifTrue: [^self scanQuotedText].
	(self specials includes: char)
		ifTrue: [tokenType := #special.         " Special character. Make it token value and set token type "
				^token := self next.].
	(self matchCharacterType: AtomMask)
		ifTrue: [^self scanAtom].
	tokenType := #doIt.
	token := char.
	^token
!

scanCommentOn: streamOrNil
	"scan comment copying on specified stream"

	self step ~~ $( ifTrue: [self error: 'Unmatched comment' ].             " Should never be the case "
	token := self
				scanUntil: [
					((self scanQuotedChar; matchCharacterType: CommentMask)
						ifTrue:
							[streamOrNil notNil ifTrue: [streamOrNil nextPut: hereChar].
							true]
						ifFalse: [hereChar == $(
								ifTrue:
									[streamOrNil notNil ifTrue: [streamOrNil space].
									self stepBack; scanCommentOn: streamOrNil.
									streamOrNil notNil ifTrue: [streamOrNil space].
									true]
								ifFalse: [false]]) not].
	hereChar ~~ $) ifTrue: [ self error: 'Unmatched comment' ].
	^token
!

shouldFoldLine
	"Answers true if next line is to be folded in, that is, if CRLF is followed by at least one white space"
	| char |

	self atEnd ifTrue: [ ^false ].
	char := source peek.
	^((self classificationMaskFor: char) anyMask: WhiteSpaceMask)
		ifTrue: [self sourceTrailNextPut: source next. true]
		ifFalse: [false]
!

step
	super step.
	self scanEndOfLine.
	^hereChar
! !

!RFC822Scanner methodsFor:'testing'!

isRFC822Scanner
	^true
! !

!RFC822Scanner methodsFor:'tokenization'!

nextToken
	^self nextRFC822Token
!

specials
" This method is provided to encapsulate lexical differences between RFC822 on one side, and MIME, HTTP on the other side. MIME definiton of 'tspecials' is the same as the RFC 822 definition of ''specials' with the addition of the three characters </>, <?>, and <=>, and the removal of <.>. To present uniform tokenization interface, this method is overridden in Mime scanner "

	^self class specials
! !

!RFC822Scanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__RFC822Scanner.st,v 1.4 2004/11/18 10:15:40 cg Exp $'
! !

RFC822Scanner initialize!
