"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

MimeMessageBody subclass:#SimpleBody
	instanceVariableNames:''
	classVariableNames:'DefaultCharsetEncoder'
	poolDictionaries:''
	category:'NetClients-MIME'
!

SimpleBody comment:'This represents simple message bodies. Simple message body''s value is its contents, which is
a string. Simple message bodies make no attempt to decode or transform its contents, it is
responsibility of higher-level code.
Parsing and value initialization is done lazily. Source stream is remembered so that parsing
can be done at any time.'
!


!SimpleBody class methodsFor:'defaults'!

defaultCharsetEncoder
	^DefaultCharsetEncoder isNil
		ifTrue: [ DefaultCharsetEncoder := Locale current defaultEncoding ]
		ifFalse: [ DefaultCharsetEncoder ]
!

defaultCharsetEncoder: aString
	DefaultCharsetEncoder := aString
! !

!SimpleBody methodsFor:'accessing'!

contentLength
" Answer content length in octets, nil if not known "
	^self parent contentLength
!

contentTypeEncoding
	" Answer symbolic name of the character encoding for this body "

	^self parent notNil
		ifTrue: [self parent contentTypeEncoding]
		ifFalse: [#default]
!

contentTypeSpecies
	^self parent contentTypeSpecies
!

contentsSpecies
	^self parent contentsSpecies
!

decodedValue
	^self decodedValueWith:  self class defaultCharsetEncoder.
!

decodedValueWith: aSymbol
	self parent hasContentTransferEncoding
		ifTrue:
			[ ^self value ]
		ifFalse:
			[^ (self source encodedContents asByteArray withEncoding: aSymbol )
				readStream
					lineEndConvention: self source lineEndConvention;
					contents]
!

encoding
	" Answer symbolic name of the character encoding for this body "

	^self parent notNil
		ifTrue: [self parent encoding]
		ifFalse: [#default]
!

partAt: anInteger
	self shouldNotImplement
!

parts
	^OrderedCollection new
!

removePart: aMimeEntity
	self shouldNotImplement
!

removePartAt: anInteger
	self shouldNotImplement
!

sourceLineEndConvention
	^self source isNil
		ifTrue: [nil]
		ifFalse:
			[self source "isInternal stx" isExternalStream not
				ifTrue: [ LineEndCR ]
				ifFalse: [ self source lineEndConvention]]
!

value: anArray
	self source: anArray readStream
!

valueAsStream
	" Answer stream representing value. If value has been set, make stream out of it. Otherwise just use source stream. Reset the stream before returning.
	If the source contains non-ASCII chars convert it to an EncodedString."
	| encoding contents |

	self source == nil
		ifTrue: [^ String new readStream]
		ifFalse:
			[ self source "isInternal stx" isExternalStream not
				ifTrue:
					[contents := self source reset contents.
					encoding := parent charsetIfNone: [MimeEncodedWordCoDec findEncodingFor: contents ].
					encoding ~~ nil
						ifTrue: [ Smalltalk isSmalltalkX ifTrue:[
							    contents := CharacterEncoder encodeString:contents from:encoding into:#unicode
							  ] ifFalse:[
							    contents := (contents asByteArrayEncoding: encoding ) asString
							  ]
							].
					^contents readStream]
				ifFalse:
					[" Attachment "
					^ self source reset ]]
! !

!SimpleBody methodsFor:'defaults'!

defaultContentType
	^'text/plain'
! !

!SimpleBody methodsFor:'mutating'!

addPart: part
	(MimeEntityBot new
		makeMultipart: self parent)
		addPart: part;
		yourself
! !

!SimpleBody methodsFor:'printing'!

printForDisplayOn: aStream
	self isBinary
		ifTrue: [self printOn: aStream]
		ifFalse: [self writeOn: aStream]
! !

!SimpleBody methodsFor:'private'!

getValue
	^source isNil
		ifTrue: [nil]
		ifFalse: [[self source reset; contents] ensure: [self source close]]
! !

!SimpleBody methodsFor:'private -- parsing'!

doParse: rfc822Scanner
	^rfc822Scanner contents
! !

!SimpleBody methodsFor:'transfer encoding'!

decodeAs: encodingName
        | outStream inStream output lineEndConvention |

        (#('base64' 'quoted-printable') includes: encodingName) ifFalse: [ ^self].

        inStream := self source reset.
        " This code is messy because actual encoding is done differently now because of
        a few glitches. Should be consolidated shortly "
        self source "isInternal stx" isExternalStream 
                ifTrue: [ lineEndConvention := self source lineEndConvention].

        [encodingName = 'base64'
                ifTrue:
                        [outStream := (ByteArray new: inStream size * 2 // 3 + 1) writeStream.
                        self class scannerType decodeBase64From: inStream to: outStream.
                        self source: ((StreamEncoder new: self parent charset) == nil
                                ifTrue: [ outStream contents readStream ]
                                ifFalse: [ (outStream contents withEncoding: self parent charset ) readStream ]) ].

        encodingName = 'quoted-printable'
                ifTrue:
                        [output := (self class scannerType decodeQuotedPrintableFrom: inStream).
                        self source: ((StreamEncoder new: self encoding) == nil
                                ifTrue: [ output readStream ]
                                ifFalse: [ (output asByteArray withEncoding: self parent charset ) readStream]) ]

        ] ensure: [inStream close].

        lineEndConvention ~~ nil
                ifTrue: [ self source lineEndConvention: lineEndConvention]
                ifFalse: [ self source lineEndCR ].
!

encodeAs: encodingName
        | outStream lineEndConvention |

        (#('base64' 'quoted-printable') includes: encodingName) ifFalse: [^self].

        self source "isInternal stx" isExternalStream
                ifTrue: [ lineEndConvention := self source lineEndConvention].
        outStream := (ByteArray new
                                        withEncoding: self parent charset )
                                        readWriteStream.
        lineEndConvention ~~ nil
                ifTrue: [outStream lineEndConvention: lineEndConvention].

        self encodeAs: encodingName on: outStream.
        outStream reset.
        self source: outStream
!

encodeAs: encodingName on: outStream
	| inStream |

	[inStream := self valueAsStream.
	inStream isNil ifTrue: [^self].
	encodingName = 'base64'
		ifTrue: [self class scannerType encode: inStream asBase64To: outStream].
	encodingName = 'quoted-printable'
		ifTrue: [self class scannerType encode: inStream asQuotedPrintableTo: outStream]        ]
		ensure: [inStream close].
!

needsContentTransferEncoding
	^self class scannerType
			needsTransferEncoding: self valueAsStream mimeType: self mimeType
! !

!SimpleBody methodsFor:'visiting'!

acceptVisit: aVisitor with: argument
	^aVisitor acceptSimpleBody: self with: argument
! !

!SimpleBody methodsFor:'writing'!

writeContentsOn: aStream
	| sourceStream |

	sourceStream := self valueAsStream.
	[[sourceStream atEnd]
		whileFalse:
			[aStream nextPut: sourceStream next]]
		ensure: [sourceStream close].
!

writeOn: aStream
" Writing can be different from printing. Printing may be indented, abbreviated
and otherwise formatted for readability. Writing is used to actually send message.
Default implementation is the same "

	self writeContentsOn: aStream
! !

!SimpleBody class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__SimpleBody.st,v 1.5 2010/03/09 12:14:18 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__SimpleBody.st,v 1.5 2010/03/09 12:14:18 cg Exp $'
! !
