"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#SimpleScanner
	instanceVariableNames:'source lineEndCRLF hereChar token tokenType saveComments
		currentComment buffer classificationMask sourceTrailStream
		lookahead lookahead2'
	classVariableNames:'CRLFMask WhiteSpaceMask EndOfLineMask Cr CRLF DigitMask Lf
		NilMask AlphabeticMask'
	poolDictionaries:''
	category:'NetClients-Base'
!

SimpleScanner class instanceVariableNames:'classificationTable'

"
 No other class instance variables are inherited by this class.
"
!

SimpleScanner comment:'This is an abstract class that is common superclass for all scanners in Internet Connectivity package. Specific scanners are used to scan and parse RFC822- and Mime messages, IMAP commands and responses, etc. Each scanner scans source stream and is a stream wrapper that republishes most of stream interface, so the code using scanner may view it also as a stream.
Source consists of tokens. nextToken reads and answers next lexical token and stores token lexical type into tokenType variable. Each scanner has its own character classification table. This has an advantage that same code may work across different protocols. For example, RFC822 and Mime have common lexical structures but slightly different sets of special characters. Therefore, MimeScanner uses its own character classification table and mostly the same code across the trwo scanners.
All scanners provide methods to scan specific token types (scanText, scanWord, etc.).
This class also provides methods to scan token sequences (see category ''extersiion types'':)
	scanUntil:
	scanWhile:, etc.
There methods look one character ahead and require stream to be positionable 1 position back.
There are also methods that look one or more tokens ahead and require fully positionable stream (see category ''tokenization'')

Subclasses must implement the following messages:
	Instance protocol:
		tokenization
			nextToken
			scanXXX methods for all their token types
	Class protocol:
		printing
			defaultTokenType
			printToken:tokenType:on:


Instance Variables:
	source  <Stream>  Source stream
	hereChar        <Character>  Last read character
	token   <String>  Last token
	tokenType       <Symbol>  Token type
	saveComments    <Boolean>  Preserve and accumulate comments
	currentComment  <String>  Current comment
	buffer  <?type?>  comment
	classificationMask      <Integer>  Classification mask of hereChar
	sourceTrailStream       <PeekableStream>  Source trail. NOt currently used


Shared Variables:
	AlphabeticMask  <Integer>  Mask for alpha characters
	Cr      <Character>  Character cr
	CRLF    <Array of: ?type?>  String with: Character cr with: Character lf
	CRLFMask        <Integer>  Mask for either cr or lf.
	DigitMask       <Integer>  Mask for 0-9
	EndOfLineMask   <Integer>  Line terminator mask. A cr character is not always line terminator. rfc822 uses line folding, so cr followed by white space is treated as linear white space (lwsp), not line terminator
	Lf      <Character>  Character lf
	NilMask <Integer>  No mask flags
	WhiteSpaceMask  <Integer>  Mask for white space


Class Instance Variables:
	classificationTable     <WordArray of: Integer>  Classification table indexed by characters integer value. Each entry is a bit mask comprised of mask defined above
'
!


!SimpleScanner class methodsFor:'instance creation'!

new
	^super new initialize
!

on: stream
	^self new on: stream
!

on: stream from: startPositionNumber to: sizeNumber
	^self new
		on: stream
		from: startPositionNumber
		to: sizeNumber
! !

!SimpleScanner class methodsFor:'accessing'!

classificationTable

	^classificationTable isNil
		ifTrue: [ self superclass classificationTable ]
		ifFalse: [ classificationTable ]
!

classificationTable: aValue

	classificationTable := aValue
!

cr

	^Cr
!

crlf

	^CRLF
!

lf

	^Lf
! !

!SimpleScanner class methodsFor:'character classification'!

whiteSpace
	^String with: Character space with: Character tab
! !

!SimpleScanner class methodsFor:'class initialization'!

initClassificationTable
	classificationTable := WordArray new: 256.
	self initClassificationTableWith: AlphabeticMask when:
		[:c | ($a <= c and: [c <= $z]) or: [$A <= c and: [c <= $Z]]].
	self initClassificationTableWith: DigitMask when:
		[:c | c >= $0 and: [c <= $9]].
	self initClassificationTableWith: WhiteSpaceMask when:
		[:c | #(32 "space" 9  "tab") includes: c asInteger].
	self initClassificationTableWith: CRLFMask when:
		[:c | c == Character return or: [ c == Character lf ] ].
	self initClassificationTableWith: EndOfLineMask when: [:c | c == Character return ].
!

initClassificationTableWith: mask when: aBlock
	"Set the mask in all entries of the classificationTable for which
	aBlock answers true."

	0 to: classificationTable size-1 do:
		[:i |
		(aBlock value: (Character value: i)) ifTrue:
			[classificationTable
				at: i + 1 put: ((classificationTable at: i+1) maskSet: mask)]]
!

initialize
	"SimpleScanner initialize"

	self initializeConstants; initClassificationTable
!

initializeConstants
	AlphabeticMask := 1.
	DigitMask := 2.
	WhiteSpaceMask := 4.
	CRLFMask := 8.
	EndOfLineMask := 16.
	NilMask := 0.
	Cr := Character return.
	Lf := Character lf.
	CRLF := Array with: Cr with: Lf.
! !

!SimpleScanner class methodsFor:'printing'!

defaultTokenType
	self subclassResponsibility
!

printToken: assocOrValue on: stream
	| tokenType token |
	(assocOrValue isKindOf: Association)
		ifTrue:
			[tokenType := assocOrValue key.
			token := assocOrValue value]
		ifFalse:
			[tokenType := self defaultTokenType.
			token := assocOrValue].
	self printToken: token tokenType: tokenType on: stream
!

printToken: value tokenType: aSymbol on: stream
	self subclassResponsibility
! !

!SimpleScanner methodsFor:'accessing'!

classificationMask
	^classificationMask
!

currentComment
	^currentComment
!

hereChar
	^hereChar
!

hereChar: char
	hereChar := char.
	classificationMask := self classificationMaskFor: hereChar
!

saveComments

	^saveComments
!

saveComments: aValue

	saveComments := aValue
!

size

	^source size
!

source
	^source
!

source: aStreamSegment
	source := aStreamSegment
!

stream
	^source stream
!

token
	^token
!

tokenType
	^tokenType
! !

!SimpleScanner methodsFor:'error handling'!

expected: aString
	"Notify that there is a problem at current token."

	^ self notify: ((#expected << #dialogs >> '<1s> expected')expandMacrosWith: aString)
!

notify: string
	"Subclasses may wish to override this"
	self error: string
!

offEnd: aString
	"Parser overrides this"

	^self notify: aString
! !

!SimpleScanner methodsFor:'expression types'!

classificationMaskFor: charOrNil
	charOrNil isNil
		ifTrue: [^NilMask]
		ifFalse: [^self class classificationTable at: charOrNil asInteger + 1]
!

matchCharacterType: mask
	^self classificationMask anyMask: mask
!

mustMatch: char
	^self mustMatch: char notify: [self expected: (String with: char)]
!

mustMatch: char notify: message

	self skipWhiteSpace.
	(self next == char)
		ifFalse: [self notify: message]
!

scanTokenMask: tokenMask
	"Scan token based on character mask. Answers token's value. Stream is positioned before trhe character that terminated scan"

	^self scanWhile: [self matchCharacterType: tokenMask]
!

scanUntil: aNiladicBlock
	^self scanUntil: aNiladicBlock do: [:stream :char | stream nextPut: char]
!

scanUntil: aNiladicBlock do: actionBlock
	"Scan token using a block until match is found. At the end of scan the stream is positioned after the
	matching character. Answers token value"

	| stream |
	stream := (String new: 40) writeStream.

	[self atEnd
		ifTrue:
			[self hereChar: nil.
			^stream contents].
	self step.
	aNiladicBlock value]
		whileFalse: [actionBlock value: stream value: hereChar].
	^stream contents
!

scanWhile: aNiladicBlock
	"Scan token using a block. At the end of scan the stream is positioned at the first character that does not match. hereChar is nil. Answers token value"

	| str |
	str := self scanUntil: [ aNiladicBlock value not ].
	hereChar notNil ifTrue: [ self stepBack ].
	^str
!

scanWhile: aNiladicBlock do: actionBlock
	"Scan token using a block. At the end of scan the stream is positioned at the first character that does not match. hereChar is nil. Answers token value"

	| str |
	str := self scanUntil: [ aNiladicBlock value not ] do: actionBlock.
	hereChar notNil ifTrue: [ self stepBack ].
	^str
!

step
	^self next.
!

stepBack
	self skip: -1.
! !

!SimpleScanner methodsFor:'initialize-release'!

initialize
	buffer := WriteStream on: (String new: 40).
	saveComments := true.
	self hereChar: nil
!

on: inputStream
	"Bind the input stream"
	self hereChar: nil.
	self source: inputStream asStream
!

on: inputStream from: startPositionNumber to: sizeNumber
	self on: (StreamSegment
			on: inputStream asStream
			offset: startPositionNumber
			size: sizeNumber)
!

scan: inputStream
	"Bind the input stream, fill the character buffers and first token buffer"
	self on: inputStream.
	^self nextToken
! !

!SimpleScanner methodsFor:'instance creation'!

asStream
	^source
!

from: start to: end
	^self newSize: end - start startingAt: start
!

newSize: size
	^self species on: (self asStream newSize: size )
!

newSize: sizeNumber startingAt: aNumber
	^self species on: (self asStream newSize: sizeNumber startingAt: aNumber)
! !

!SimpleScanner methodsFor:'multi-character scans'!

skipWhiteSpace
	"It is inefficient because intermediate stream is created. Perhaps refactoring scanWhile: can help"

	self scanWhile: [self matchCharacterType: WhiteSpaceMask]
! !

!SimpleScanner methodsFor:'positioning'!

close
	source close
!

reset
        "Set the receiver's position to start position."

        lookahead := lookahead2 := nil.
        source reset
! !

!SimpleScanner methodsFor:'printing'!

printToken: assoc on: stream
	self class printToken: assoc on: stream
!

printToken: value tokenType: aSymbol on: stream
	self class printToken: value tokenType: aSymbol on: stream
! !

!SimpleScanner methodsFor:'private'!

resetToken
	token := tokenType := nil
! !

!SimpleScanner methodsFor:'source trail'!

sourceTrail
	| res |
	sourceTrailStream notNil ifTrue: [res := sourceTrailStream contents].
	sourceTrailStream := nil.
	^res
!

sourceTrailNextPut: char
	(sourceTrailStream notNil and: [char notNil])
		ifTrue: [sourceTrailStream nextPut: char]
!

sourceTrailNextPutAll: string
	(sourceTrailStream notNil and: [string notNil])
		ifTrue: [sourceTrailStream nextPutAll: string]
!

sourceTrailOff
	sourceTrailStream := nil
!

sourceTrailOn
	sourceTrailStream := (String new: 64) writeStream.
!

sourceTrailSkip: integer
	sourceTrailStream notNil
		ifTrue: [sourceTrailStream skip: integer]
! !

!SimpleScanner methodsFor:'stream interface -- reading'!

atEnd
	|atEnd|

	lookahead notNil ifTrue:[^ false].
	source isExternalStream ifFalse:[ ^ source atEnd].
(source readWaitWithTimeout:3) ifFalse:[^ false].
^ true.
source blocking:false.
	atEnd := source atEnd.
source blocking:true.
	^ atEnd.
!

basicBackstep
        lookahead notNil ifTrue:[
            lookahead2 notNil ifTrue:[
                self error:'can only backstep twice'.
            ].
        ].
        hereChar isNil ifTrue:[
            self error:'should not happen'.
        ].
        lookahead2 := lookahead.
        lookahead := hereChar.

"/ original: source skip: -1.
!

basicNext
        |ch|

        (ch := lookahead) notNil ifTrue:[
            lookahead := lookahead2.
            lookahead2 := nil.
        ] ifFalse:[
            ch := source next.
        ].
        ch == Character return ifTrue:[
            ch := source next.
            ch ~~ Character nl ifTrue:[
                lookahead2 := lookahead.
                lookahead := ch.
            ].
            ^ Character nl
        ].
        ^ ch

"/ original: ^ source next.
!

binary
	source binary
!

contents
        |lS|

        lookahead notNil ifTrue:[
            lS := lookahead asString.
            lookahead := lookahead2.
            lookahead2 := nil.
            ^ lS , self contents
        ].
        ^ source contents
!

contentsSpecies

	^((source respondsTo: #isBinary) and:[source isBinary])
		ifTrue: [ByteArray]
		ifFalse: [String]
!

next
	self hereChar: self basicNext.
	self sourceTrailNextPut: hereChar.
	^hereChar
!

next: anInteger
	"Answer the next anInteger elements of the receiver."
	| newCollection res |
	newCollection := self contentsSpecies new: anInteger.
	res := self next: anInteger into: newCollection startingAt: 1.
	self sourceTrailNextPutAll: res.
	^res
!

next: anInteger into: aSequenceableCollection startingAt: startIndex
	"Store the next anInteger elements of the receiver into aSequenceableCollection
	starting at startIndex in aSequenceableCollection. Answer aSequenceableCollection."

	| index stopIndex |
	index := startIndex.
	stopIndex := index + anInteger.
	[[index < stopIndex and: [self atEnd not ]]
				whileTrue: [aSequenceableCollection at: index put: self basicNext.
							index := index + 1]]
		on: Stream endOfStreamSignal
		do:
			[:ex | ex retryUsing:
					[(Stream incompleteNextCountSignal newExceptionFrom: self)
						parameter: index - startIndex;
						raiseRequest]].
	^aSequenceableCollection
!

nextLine
	^self scanUntil: [ self matchCharacterType: CRLFMask ]
!

nextLineWithoutTerminator
" Answer next line. Do not inlude cr. Stream is positioned before terminating cr "
	^self scanWhile: [ (self matchCharacterType: CRLFMask) not ].
!

peek
        "Answer what would be returned with a self next, without
        changing position.  If the receiver is at the end, answer nil."

        | nextObject |

        lookahead notNil ifTrue:[^ lookahead].

        self atEnd ifTrue: [^nil].
        nextObject := self basicNext.
        lookahead2 := lookahead.
        lookahead := nextObject.
        ^nextObject
!

peekFor: anObject
	"Answer false and do not move the position if self next ~= anObject or if the
	receiver is at the end. Answer true and increment position if self next = anObject."

	| nextObject |
	self atEnd ifTrue: [^false].
	nextObject := self next.
	"peek for matching element"
	anObject = nextObject ifTrue: [^true].
	"gobble it if found"
	self stepBack.
	^false
!

position
	^source position
!

position: anInt
	source position: anInt
!

skip: integer
	integer == -1 ifTrue:[
	    self basicBackstep.
	] ifFalse:[
	    self error:'can only backStep once'
"/ original: source skip: integer.
	].
	self sourceTrailSkip: integer.
!

text
	^source text
!

throughAll: pattern
	| str |
	str := source throughAll: pattern.
	self sourceTrailNextPutAll: str.
	^str
!

upTo: anObject
	"Answer a subcollection from position to the occurrence (if any, exclusive) of anObject.
	 The stream is left positioned after anObject.
	If anObject is not found answer everything."

	| newStream element |
	newStream := (self contentsSpecies new: 64) writeStream.
	[self atEnd]
		whileFalse:
			[element := self next.
			element = anObject
				ifTrue: [^newStream contents].
			newStream nextPut: element.].
	^newStream contents
!

upToAll: pattern
	| str |
	str := source upToAll: pattern.
	self sourceTrailNextPutAll: str.
	^str
!

upToEnd
	| str |
	str := source upToEnd.
	self sourceTrailNextPutAll: str.
	^str
! !

!SimpleScanner methodsFor:'sunit test helpers'!

testScanTokens
	| s st |
	s := WriteStream on: (Array new: 16).
	st  := WriteStream on: (Array new: 16).
	[tokenType = #doIt]
		whileFalse:
			[s nextPut: token. st nextPut: tokenType.
			self nextToken].
	^Array with: s contents with: st contents
!

testScanTokens: textOrString
	"Answer with an Array which has been tokenized"

	self scan: (ReadStream on: textOrString asString).
	^self testScanTokens.
! !

!SimpleScanner methodsFor:'tokenization'!

nextToken
	self subclassResponsibility
!

nextTokenAsAssociation
" Read next token and and answer tokenType->token "
	self nextToken.
	^tokenType->token
!

scanToken: aNiladicBlock delimitedBy: anArray notify: errorMessageString
" Scan next lexical token based on the criteria defined by NiladicBlock. The block is evaluated for every character read from input stream until it yields false. Stream is positioned before character that terminated scan"

"Example: self scanToken: [ self scanQuotedChar; matchCharacterType: DomainTextMask ]
	delimitedBy: '[]' notify: 'Malformed domain text'."

	| string |
	self mustMatch: anArray first.
	string := self scanWhile: aNiladicBlock.
	self mustMatch: anArray last notify: errorMessageString.
	^string
!

scanTokens: textOrString
	"Answer with an Array which has been tokenized"

	^self on: (ReadStream on: textOrString asString); tokenize.
!

tokenize
	| s |
	s := WriteStream on: (Array new: 16).
	[self nextToken. tokenType = #doIt]
		whileFalse: [s nextPut: token].
	^s contents
!

tokenizeList: aBlock separatedBy: comparisonBlock
" list = token *( separator token) "
	| stream block v |

	stream := (Array new: 4) writeStream.
	block := [(v := aBlock value) notNil ifTrue: [stream nextPut: v]].
	block value.                                                                    " Evaluate for the first element "
	self tokenizeWhile: [comparisonBlock value ] do: block.
	^stream contents
!

tokenizeUntil: aBlock do: actionBlock
	[self skipWhiteSpace. self position. self nextToken. tokenType == #doIt or: aBlock]
		whileFalse:
			[actionBlock value].
!

tokenizeWhile: aBlock
	| s |
	s := WriteStream on: (Array new: 16).
	self tokenizeWhile: [aBlock value]
		do: [s nextPut: token].
	^s contents
!

tokenizeWhile: aBlock do: actionBlock
        | pos oldLookahead oldLookahead2|

        [
            self skipWhiteSpace.
            pos := self position. 
            oldLookahead := lookahead.
            oldLookahead2 := lookahead2.
            self nextToken.
            (tokenType ~= #doIt and:[aBlock value])
        ] whileTrue: [actionBlock value].
        self position: pos.      "Reset position to the beginning of the token that did not match"
        lookahead := oldLookahead.
        lookahead2 := oldLookahead2.
! !

!SimpleScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__SimpleScanner.st,v 1.4 2009/07/27 10:55:16 cg Exp $'
! !

SimpleScanner initialize!
