"{ Package: 'stx:goodies/net' }"

"{ NameSpace: Net }"

Object subclass:#ValueWithParams
	instanceVariableNames:'params separators value'
	classVariableNames:''
	poolDictionaries:''
	category:'NetClients-MIME'
!

ValueWithParams comment:'ValueWithParams parses, composes and holds values for header fields that have value and parameters.

Instance Variables:
	params  <Dictionary>    filed parameters
	separators      <SequenceableCollection>        field separators
	value   <String>        parameter  value

For example, Content-Type field

  content := "Content-Type" ":" type "/" subtype
	     *(";" parameter)
	     ; Matching of media type and subtype
	     ; is ALWAYS case-insensitive.

  parameter := attribute "=" value
  value := token / quoted-string

If while parsing parameter value non-ASCII characters would be detected the parser is going to raise NonASCIIFieldParamException. If there is no handler for this exception the parameter value will be set to stream bytes.
'
!


!ValueWithParams class methodsFor:'instance creation'!

newWithSeparators: anArray
	^self new
		separators: anArray;
		yourself
! !

!ValueWithParams class methodsFor:'parsing'!

scannerOn: aStream
	^(aStream isKindOf: self scannerType)
		ifTrue: [aStream]
		ifFalse: [self scannerType on: aStream asStream]
!

scannerType
	^MimeScanner
! !

!ValueWithParams methodsFor:'accessing'!

groupSeparator
	^separators last
!

paramAt: aString
	^self paramAt: aString asLowercase ifAbsent: [ nil ]
!

paramAt: aString ifAbsent: aBlock
	^self params at: aString ifAbsent: [ aBlock value ]
!

paramAt: aName put: aValue
	^self params at: aName put: aValue
!

parametersSeparator
	^separators at: 2
!

params
	params isNil ifTrue: [ params := Dictionary new].
	^params
!

separators: anArray
	separators := anArray
!

value
	^value
!

value: aString
	value := aString
!

valueSeparator
	^separators first
! !

!ValueWithParams methodsFor:'composing'!

writeOn: aStream encoding: aString
	self
		writeValueOn: aStream;
		writeParamsOn: aStream encoding: aString.
!

writeParameter: assoc on: aStream encoding: aString

	| rfcString |
	self class scannerType
			printWord: assoc key on: aStream;
			print: $= on: aStream.
	assoc value ~~ nil
		ifTrue:
			[(MimeEncodedWordCoDec shouldBeEncoded: assoc value)
				ifTrue:
					[rfcString := MimeEncodedWordCoDec
									encodeText: assoc value
									asBase64Using: aString.
					aStream nextPut: $"; nextPutAll:  rfcString; nextPut: $" ]
				ifFalse: [ self class scannerType printWord: assoc value on: aStream ]]
!

writeParamsOn: aStream encoding: aString

	(params ~~ nil and: [ params notEmpty])
		ifTrue:
			[aStream nextPut: self valueSeparator.
			params associations
				do: [ :assoc | self writeParameter: assoc on: aStream encoding: aString ]
				separatedBy: [ aStream nextPut: self parametersSeparator ]]
!

writeValueOn: aStream

	self printValueOn: aStream
! !

!ValueWithParams methodsFor:'parsing'!

doParse: scanner
	" Canonical value removes all white space and comments from the source "

	| str |
	str := scanner scanUntil:
			[ scanner hereChar == self valueSeparator
				or: [ scanner hereChar == self groupSeparator] ].
	^self  tokenizedValueFrom: (self scannerOn: str readStream).
!

parse: scanner
	| paramName |
	scanner skipWhiteSpace.
	value := self doParse: scanner.
	scanner atEnd ifTrue: [ ^self].
	scanner hereChar ~~ self valueSeparator
		ifTrue: [ scanner stepBack. ^self ]
		ifFalse:
			[ scanner skipWhiteSpace.
			scanner atEnd
				ifFalse:
					[scanner
						tokenizeList:
							[paramName := scanner nextToken.
							scanner skipWhiteSpace.
							(scanner next == $=)
								ifFalse: [Transcript cr; show: (#DetectedMalformedField << #net >> 'Detected malformed field:'); cr; show: scanner contents.  ^self ].
							self params
								at: paramName asLowercase
								put: (self parseParamValueFrom: scanner).]
						separatedBy: [scanner token == self parametersSeparator ]]]
!

parseParamValueFrom: scanner
"Trying to handle non-ASCII characters that some mail clients can send unencoded "
	| startPos |
	^[
		[startPos := scanner position.
		MimeEncodedWordCoDec decodeText: scanner nextToken]
			on: Error
			do: [ :ex |  | string |
				scanner position: startPos.
				string := scanner upToEnd.
				NonASCIIFieldParamException raiseRequestFrom: string].
	] on: NonASCIIFieldParamException
	do: [ :ex |
		ex isNested
			ifTrue: [ ex outer]
			ifFalse: [ ex parameter ]].
!

readFrom: aStream
	^self parse: (self scannerOn: aStream)
!

scannerOn: aStream
	^self class scannerOn: aStream
! !

!ValueWithParams methodsFor:'printing'!

printOn: aStream

	self printValueOn: aStream.
	self params notEmpty
		ifTrue: [       self printParamsOn: aStream ]
!

printParameter: assoc on: aStream
	self class scannerType
			printWord: assoc key on: aStream;
			print: $= on: aStream.

	assoc value ~~ nil
		ifTrue:
			[[self class scannerType printWord: assoc value on: aStream]
			on: NonASCIICharException
			do: [ :ex | aStream nextPut: $"; nextPutAll: assoc value; nextPut: $"] ].
!

printParamsOn: aStream
	params notNil
		ifTrue:
			[aStream nextPut: self valueSeparator.
			params associations
				do: [ :assoc | self printParameter: assoc on: aStream ]
				separatedBy: [ aStream nextPut: self parametersSeparator ]]
!

printValueOn: aStream

	value ~~ nil
		ifTrue: [aStream nextPutAll: self value]
! !

!ValueWithParams methodsFor:'private-utility'!

tokenize: rfc822Stream
	"Scan field value token by token. Answer an array of tokens"

	| result token |
	result := (Array new: 2) writeStream.
	[rfc822Stream atEnd or: [rfc822Stream peek == Character cr or: [(token := rfc822Stream nextToken) isNil]]]
		whileFalse: [result nextPut: token].
	^result contents
!

tokenizedValueFrom: rfc822Stream
	"Scan field value token by token. Answer a string that is a concatenation of all elements in the array. One can view this as a canonicalized field value because this operation eliminates all white space and comments "

	| result tokens |
	result := (String new: 20) writeStream.
	tokens := self tokenize: rfc822Stream.
	tokens
		do: [:token | token isString
				ifTrue: [result nextPutAll: token]
				ifFalse: [result nextPut: token]
].
	^result contents
! !

!ValueWithParams class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/Net__ValueWithParams.st,v 1.1 2004/11/12 10:44:44 penk Exp $'
! !
