"{ Package: 'stx:goodies/net/ldap' }"

"{ NameSpace: LDAP }"

Object subclass:#LDAPConnection
	instanceVariableNames:'worker workerThread'
	classVariableNames:''
	poolDictionaries:''
	category:'LDAP-Main'
!

!LDAPConnection class methodsFor:'documentation'!

examples
"
PASSWORD CHECKING / an exception will be raised, if the password does no match
[exBegin]
    |ldap|

    ldap := LDAP::LDAPConnection to:'ldap.bh.exept.de'.
    LDAP::LDAPSecurityProblemException handle:[:ex |
        ldap disconnect.
        ^ false. 'wrong password'
    ] do:[
        ldap bindAs:'uid=sr, ou=People, dc=eXept, dc=de' credentials:'take_your_user_and_password'.
    ].
    ldap disconnect.

    ^ true. 'correct password'       
[exEnd]

Using SSL:
[exBegin]
    |ldap|

    ldap := LDAP::LDAPConnection to:'ldap.bh.exept.de' onTLSPort:nil.
    LDAP::LDAPSecurityProblemException handle:[:ex |
        ldap disconnect.
        ^ false. 'wrong password'
    ] do:[   
        ldap bindAs:'uid=stefan, ou=People, dc=eXept, dc=de' credentials:'stefans password here'.
    ].
    ldap disconnect.

    ^ true. 'correct password'       
[exEnd]


[exBegin]
        |ldap searchReq|

        ldap := LDAP::LDAPConnection to:'ldap.bh.exept.de'.
        'no bind is an anonymous bind'.
        searchReq := ldap 
                        newSearch:'dc=exept,dc=de' 
                        scope:(LDAPConnection wholeSubtree)
                        deref: (LDAPConnection derefNever) 
                        filter: nil
                        attrs: nil
                        wantAttrsOnly: false.
        searchReq result inspect.

        ldap disconnect.
[exEnd]

[exBegin]
        |ldap searchReq|

        ldap := LDAP::LDAPConnection to:'ldap.bh.exept.de'.
        ldap bindAs:'uid=stefan, ou=People, dc=exept, dc=de' credentials:'Stefans Secret'.
        searchReq := ldap 
                        newSearch:'dc=exept,dc=de' 
                        scope:(LDAPConnection wholeSubtree)
                        deref: (LDAPConnection derefNever) 
                        filter: (LDAPFilter with: 'uid' equalTo: 'stefan')
                        attrs: #('cn' 'uid')
                        wantAttrsOnly: false.

        [searchReq isDone] whileFalse:[
              searchReq next inspect
        ].
        ldap disconnect.
[exEnd]
"
!

missingRFC2251
	self notYetImplemented
	
"The following parts of RFC 2251 have not been implemented:

SOON 
- When Message ID is over the maximum, we should rebind to reset the counter. Although practically it would be enough to just zero the counter when it overflows, since we can reuse the IDs when we have alroady gotten a response
- When rebinding we must terminate all pending operations, as the server will get rid of them; note we will have to notify waiters.
- Distinction between binary attributes (as is <AttributeType>:binary -- do we really need something implemented?
- UTF-8 encoding
- Extended Request
EVENTUALLY
- Controls / Search constraints
- Automatic referral following
- SearchResultReference (Continuation References)
- Filter extensibleMatch and MatchingRuleAssertion
- Would be nice to have a getSchema (by using subschemaSubentry)
WONT
- SASL Bind
"
! !

!LDAPConnection class methodsFor:'*Cobalt-Router-CoLocalSecretAuthService'!

to: aHostname onTLSPort: aPort
	| obj |
	obj := self basicNew initializeWithHost: aHostname onTLSPort: aPort.
	^ obj

! !

!LDAPConnection class methodsFor:'as yet unclassified'!

collectionizeAttrs: someAttrs
	| collectionized |
	collectionized := someAttrs clone.
	collectionized keys do: [:eachKey | | val |
		val := someAttrs at: eachKey.
		(val isKindOf: String)
			ifTrue: [
				someAttrs at: eachKey put: (OrderedCollection new add: val; yourself) ] ].

	^ collectionized
!

to: aHostname
	^ self to: aHostname port: 389
!

to: aHostname port: aPort
	| obj |
	obj := self basicNew initializeWithHost: aHostname port: aPort.
	^ obj
! !

!LDAPConnection class methodsFor:'interface'!

checkPassword:aPlainTextPassword 
    distinguishedName:aDistinguishedName
    server:aServer 
    useSSL:useSSL 
    onPasswordFailDo:passwordFailBlock
    onConnectionFailDo:connectionFailBlock
    "avoid code duplication, 
     provide a common method"

    |ldap|

    NameLookupError, OpenError, LDAP::LDAPException handle:[:ex |
        "Cannot connect to LDAP server"
        ldap notNil ifTrue:[
            ldap disconnect.
        ].

        connectionFailBlock notNil ifTrue:[
            connectionFailBlock value.         
        ] ifFalse:[
            ex reject.
        ].

        ^ self
    ] do:[
        useSSL ifTrue:[
            ldap := self to:aServer onTLSPort:nil.
        ] ifFalse:[
            ldap := self to:aServer.
        ].

        LDAP::LDAPSecurityProblemException handle:[:ex |
            "LDAP server has rejected the password"
            ldap disconnect.

            passwordFailBlock notNil ifTrue:[
                passwordFailBlock value.
            ] ifFalse:[
                ex reject.
            ].

            ^ self
        ] do:[
            ldap bindAs:aDistinguishedName credentials:aPlainTextPassword.
        ].

        ldap disconnect.
    ].

    "Created: / 25-11-2011 / 13:24:36 / sr"
! !

!LDAPConnection class methodsFor:'search deref constants'!

derefAlways
	^ 3
!

derefFindingBaseObj
	^ 2
!

derefInSearching
	^ 1
!

derefNever
	^ 0
! !

!LDAPConnection class methodsFor:'search scope constants'!

baseObject
	^ 0
!

singleLevel
	^ 1
!

wholeSubtree
	^ 2
! !

!LDAPConnection methodsFor:'*Cobalt-Router-CoLocalSecretAuthService'!

initializeWithHost: aHostname onTLSPort: aPort
        worker := LDAPWorker newTo: aHostname port: aPort ? 636.  "636 is the default for ldaps (without STARTSSL)"
        worker connectTLS.
        workerThread := worker start.
! !

!LDAPConnection methodsFor:'as yet unclassified'!

addEntry: aDN attrs: someAttrs
	^ LDAPAddRequest new initializeWith: worker dn: aDN attrs: (self class collectionizeAttrs: someAttrs).
!

bindAs: aUsername credentials: aPassword
	^ self bindAs: aUsername credentials: aPassword method: nil
!

bindAs: aUsername credentials: aPassword method: anAuthMethod
    "bind as aUsername. If sUsernam and aPassword are both empty strings,
     this will be an anonymous bind.
     We wait for the result here..."

    |bindReq|

    bindReq := LDAPBindRequest new 
            initializeWith: worker username: aUsername credentials: aPassword method: anAuthMethod;
            wait;
            yourself.

    ^ bindReq.
!

compare: aDN using: anEqualityFilter
	^ LDAPCompareRequest new initializeWith: worker dn: aDN comparing: anEqualityFilter
!

delEntry: aDN
	^ LDAPDelRequest new initializeWith: worker dn: aDN
!

disconnect
        self unbind.            "unbind is not the opposite of bind - the server closes the connection"
        worker disconnect.
        workerThread terminate.
        self unregisterForFinalization.
        worker := workerThread := nil.
!

isValid
	"Answers whether the connection looks still valid"
	^worker running
!

modify: aDN with: someLDAPAttrModifiers
	^ LDAPModifyRequest new initializeWith: worker dn: aDN ops: someLDAPAttrModifiers
!

modifyDN: aDN toRDN: aRDN moveUnderDN: aUnderDN wantToClearAttrs: aWantsToClearAttrs
	^ LDAPModifyDNRequest new initializeWith: worker modifyDN: aDN toRDN: aRDN underDN: aUnderDN wantToClearAttrs: aWantsToClearAttrs
!

modifyDN: aDN toRDN: aRDN wantToClearAttrs: aWantsToClearAttrs
	^ self modifyDN: aDN toRDN: aRDN moveUnderDN: nil wantToClearAttrs: aWantsToClearAttrs
!

newSearch: aBase scope: aSearchScope deref: aDerefBehaviour filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly
	| nonNilFilter nonNilAttrs |
	nonNilFilter := aFilter
		ifNil: [ nonNilFilter := LDAPFilter with: 'objectclass' ]
		ifNotNil: [ aFilter ].
	
	nonNilAttrs := someAttrs
		ifNil: [ nonNilAttrs := OrderedCollection new ]
		ifNotNil: [ someAttrs ].
	
	^ LDAPSearchRequest new initializeWith: worker base: aBase scope: aSearchScope deref: aDerefBehaviour filter: nonNilFilter attrs: nonNilAttrs wantAttrsOnly: wantsAttrsOnly
!

unbind
    "Unbind is not the opposite of bind.
     Tell the server that we are finished - it will close the connection."

    LDAPUnbindRequest new 
            initializeWith: worker.
! !

!LDAPConnection methodsFor:'initialization & release'!

finalize
    worker disconnect.
    workerThread terminate.
!

initializeWithHost: aHostname port: aPort
"/        isBound := false.
        worker := LDAPWorker newTo: aHostname port: aPort.
        worker connect.
        workerThread := worker start.
        self registerForFinalization.

    "Modified: / 03-11-2011 / 12:36:12 / sr"
! !

!LDAPConnection class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/LDAP__LDAPConnection.st,v 1.15 2011/11/25 12:26:15 sr Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/net/ldap/LDAP__LDAPConnection.st,v 1.15 2011/11/25 12:26:15 sr Exp $'
! !
