"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

DecodeLine subclass:#ITUxT6Line
	instanceVariableNames:'referenceLine changingElements a0 b1 position'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!ITUxT6Line class methodsFor:'instance creation'!

asReferenceFor: aClient
	"^an ITUxT6Line
I return an instance of myself which may be used as the first 'blank' reference line for ITU T.6 decoding."

	^self new asReferenceFor: aClient

! !

!ITUxT6Line class methodsFor:'decoding'!

following: aLine for: aClient
	^self new following: aLine for: aClient

! !

!ITUxT6Line methodsFor:'accessing'!

a0
	"^a ITUxT6ChangingElement
I return the current a0 changing element.  This can't be easily deduced from my collection of changing elements because some values of a0 don't go in the collection (in the case of Pass codes), and some changing elements are never the current a0 (in the case of the first of the Horizontal change pairs).
See the ITU T6 spec section 2.2.2 for details."

	^a0

!

a0: aChangingElement
	"^self
I make aChangingElement my new a0 changing element."

	a0 := aChangingElement.
	^self

!

b1
	"^a ITUxT6ChangingElement
I return the current b1 changing element.  This is a changing element from the elements of my reference line.  b1 is the element in the reference line to the right of my a0, and of an oposite color.
See the ITU T6 spec section 2.2.2 for details."

	b1 isNil ifTrue: [self resetReferenceChanges].
	b1 := b1 priorLink.  "I step back here because b1 can move backwards sometimes"
	[b1 offset > self a0 offset and: [b1 color == self a0 oppositeColor]]
		whileFalse: [self b1: b1 nextLink].
	^b1

!

b1: aChangingElement
	"^self
I make aChangingElement my new b1 changing element."

	b1 := aChangingElement.
	^self

!

changingElements
	"^an OrderedCollection
I return my collection of chaging elements. See self>>initialize for the initialization description."

	^changingElements

!

incrementPositionBy: anInteger
	"^self
I'm being notified that anInteger pels have just been decoded and written.  I update my position accordingly."

	self position: self position + anInteger.
	^self

!

nextPosition
	"^an Integer
I the position just after my current position."


	^self position + 1

!

position
	"^an Integer
I return mu current position along the line.  The first pel in the line is at position 1"

	position isNil ifTrue: [self position: 0].
	^position

!

position: aValue

	position := aValue

!

referenceLine

	^referenceLine

!

referenceLine: aValue

	referenceLine := aValue

! !

!ITUxT6Line methodsFor:'decoding'!

decodeTo: outputStream for: aClient
	"^self
I read the codes from inputStream and write the decoded bits to outputStream.  I follow the encoding flowchart in the ITU T.6 spec (para 2.2.4), but with the opposite objective."

	| modeCode |
	[self atEnd] whileFalse:
			[modeCode := self xT6DecodeTree nextEntryFrom: aClient bitStream.
			modeCode decodeTo: outputStream for: aClient].
	self terminateClosingElements.
	^self

! !

!ITUxT6Line methodsFor:'initialize-release'!

asReferenceFor: aClient
	"^an ITUxT6Line
I initilalize myself  as the first 'blank' reference line for ITU T.6 decoding.  I need the length of the line to create my terminal changing element which changes the color just past the end of the line (@ length + 1)."

	self initialize.
	self client: aClient.
	self addChangingElement: (ITUxT6ChangingElement color: self blackColorCode
				offset: aClient rowWidth + 1).
	self terminateClosingElements.
	^self

!

following: aLine for: aClient
	"^self
I initialize myself to represent a coding line following aLine."

	self initialize.
	self referenceLine: aLine.
	self client: aClient.
	^self

!

initialize
	"^self
I initialize myself.  I set the first element of my changing elements to a white change at offset zero, and I set a0 to this new changing element."

	| initialA0 |
	changingElements := NativeLinkedList new.
	initialA0 := ITUxT6ChangingElement color: self whiteColorCode offset: 0.
	self addChangingElement: initialA0.
	self a0: initialA0.
	^self

!

resetReferenceChanges
"^self
I'm being asked to reset myself as a reference line.  This means determining the first b1 in my >>changingElements.  Basicly, this means the second element becomes b1."

	self b1: (self referenceLine changingElements at: 2).
	^self

!

terminateClosingElements
	"^self
I add a final changing element to my collection of changing elements.  I add one with the same offset but opposite color to the current last one.  This ensures that there will always be an appropriate b1 element if I'm used as a reference line."

	| roleModel |
	roleModel := self changingElements last.
	self changingElements
		add: (ITUxT6ChangingElement color: roleModel oppositeColor
				offset: roleModel offset).
	^self

! !

!ITUxT6Line methodsFor:'private'!

addChangingElement: aChangingElement
	"^aChangingElement
I add aChangingElement to my collection of changing elements.  I return the element I just added."

	self changingElements add: aChangingElement.
	^aChangingElement

!

blackColorCode
	"^an Integer
I return the number which is used to represent black in the decoded image."

	^0

!

whiteColorCode
	"^an Integer
I return the number which is used to represent white in the decoded image."

	^1

! !

!ITUxT6Line methodsFor:'services'!

a1
	"^a ITUxT6ChangingElement
I return the current a1 changing element.  I take this from my colletion of changing elements using self>>a0Index + 1.
See the ITU T6 spec section 2.2.2 for details."

nil halt: 'this should never be needed for decoding.'.
	^self changingElements after: (self a0)

!

a2
	"^a ITUxT6ChangingElement
I return the current a2 changing element.  I take this from my colletion of changing elements using self>>a0Index + 2.
See the ITU T6 spec section 2.2.2 for details."

	nil halt: 'this should never be needed for decoding.'.
	^self changingElements after: self a1

!

b2
	"^a ITUxT6ChangingElement
I return the current b2 changing element.  I take this from the colletion of changing elements of my reference line.  b2 is the changing element in the reference line to the right of self>>b1.
See the ITU T6 spec section 2.2.2 for details."

	^self b1 nextLink isNil ifTrue: [self b1] ifFalse: [self b1 nextLink]

!

xT6DecodeTree
	"^an ITUDecodeTree
I return the decode tree which is a ITUDecodeTree populated with the pre-defined T.6 mode codes."

	^CCITTDecodeTree forITUxT6Decoding

! !

!ITUxT6Line methodsFor:'testing'!

atEnd
	"^a Boolean
I return true if my a0 is beyond my width."

	^self a0 offset >= self referenceLine changingElements last offset

! !

!ITUxT6Line class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__ITUxT6Line.st,v 1.2 2006/09/07 11:36:08 sr Exp $'
! !
