"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFFramePagePlacement
	instanceVariableNames:'frame absoluteRectangle currentInsertPoint yAxisGrowth reportPage
		closed'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFFramePagePlacement class methodsFor:'instance creation'!

for: aFrame on: aPage 
	^self new for: aFrame on: aPage
			
! !

!PDFFramePagePlacement methodsFor:'accessing'!

absoluteRectangle
	"^a PDFRectangle 
	Returns the rectangle which defines the current absolute position of my frame 
	on the page. In not initialized, the rectangle will be computed."

	self hasAbsoluteRectangle ifFalse: [self initializeAbsoluteRectangle].
	^self basicAbsoluteRectangle
			
!

currentInsertPoint
	"^PDFPoint 
	I return my current insert point.

      Here we should be establishing the absolute top left.  We can't know the rectangle
      at this stage (or may not, anyway).  Right - we can't insist on knowing."

	currentInsertPoint isNil 
		ifTrue: [self currentInsertPoint: self initialInsertPoint].
	^currentInsertPoint
			
!

currentInsertPoint: aPDFPoint 
	"^self 
	I record my current insert point."

	currentInsertPoint := aPDFPoint.
	^self
			
!

frame
	"^a Frame 
	I return my frame."

	^frame
			
!

frame: aFrame 
	"^self 
	I record my frame."

	frame := aFrame.
	frame addPlacement: self.
	^self
			
!

reportPage
	"^a PDFReportPage"

	^reportPage
			
!

reportPage: aPDFReportPage 
	reportPage := aPDFReportPage
			
!

yAxisGrowth
	"^an Integer 
	I return my Y axis growth."

	yAxisGrowth isNil ifTrue: [yAxisGrowth := 0].
	^yAxisGrowth
			
!

yAxisGrowth: anInteger 
	"^self 
	I record my Y axis growth."

	yAxisGrowth := anInteger.
	^self
			
! !

!PDFFramePagePlacement methodsFor:'emiting'!

writeBorder
	"^self 
	Writes the border of this frame placement to the current page, if my frame 
	required a border."

	| pageDescription pathObject path |
	self frame hasBorder 
		ifTrue: 
			[pageDescription := self pageDescription.
			pathObject := PDFPathObject for: pageDescription.
			pageDescription addGraphicObject: pathObject.
			path := PDFPathRectangle for: pathObject.
			path rectangle: self absoluteRectangle.
			pathObject addOperator: path].
	^self
			
! !

!PDFFramePagePlacement methodsFor:'initialize-release'!

for: aFrame on: aPage
	"^self 
	I initialize and return myself."

	self frame: aFrame.
	self reportPage: aPage.
	^self
			
! !

!PDFFramePagePlacement methodsFor:'private'!

basicAbsoluteRectangle
	"^a Rectangle or nil 
	Returns the value in the absolute rectangle iv."

	^absoluteRectangle
			
!

completeAbsoluteRectangle
	"^self
I make sure that all my coordinates are complete - no nils.  My top left must have been known by this stage, so the only thing I may have to fix up is the extent.  I make my width swell to the width of my parent, and my hight will be just large enough to accomodate any content frames that were written to me.
The extra 5 by which the extent is increased just makes sure that no decenders are chopped off when the frame closes."

	| extent newX newY |
	self absoluteRectangle isComplete ifTrue: [^self].
	extent := self absoluteRectangle extent.
	extent x isNil 
		ifTrue: [newX := self parentPlacement absoluteRectangle width]
		ifFalse: [newX := extent x].
	extent y isNil 
		ifTrue: 
			[newY := self absoluteRectangle topLeft y - self currentInsertPoint y + 5]
		ifFalse: [newY := extent y].
	absoluteRectangle := PDFRectangle topLeft: self absoluteRectangle topLeft
				extent: (PDFPoint x: newX y: newY).
	^self
			
!

derivedTopLeft
	"^a PDFPoint
I get my parent placement to figure out where my top left is given my frame."

	^self parentPlacement topLeftPositionForFrame: self frame
			
!

hasAbsoluteRectangle
	"^aBoolean 
	Returns true if there is an absolute rectangle, otherwise false."

	^absoluteRectangle notNil
			
!

initializeAbsoluteRectangle
	"^an PDFRectangle 
	The initial absolute rectangle must have a concrete top left,  but may have an undefined
extent.  We must know where the top left is in order to write out content.  The extent can remain open until the placement is explicitly closed, or if the extent reaches the extent of my parent placement."

	self frame isMediaFrame 
		ifTrue: [absoluteRectangle := self frame rectangle]
		ifFalse: 
			[absoluteRectangle := PDFRectangle topLeft: self derivedTopLeft
						extent: self frame rectangle extent].
	^absoluteRectangle
			
!

topLeftPositionForFrame: aFrame 
	"^a PDFPoint
Return the top left point of a placement of aFrame within myself.  I take the rectangle of a frame to be offset from my bottom left or positioned at my current insert point.  If I don't have an absolute rectangle, I throw an exception."

	| myInsertPoint myBottomLeft frameTopLeft newX newY |
	myInsertPoint := self currentInsertPoint.
	myBottomLeft := self absoluteRectangle bottomLeft.
	frameTopLeft := aFrame rectangle topLeft.
	newX := frameTopLeft x isNil 
				ifTrue: [myInsertPoint x]
				ifFalse: [myBottomLeft x + frameTopLeft x].
	newY := frameTopLeft y isNil 
				ifTrue: [myInsertPoint y]
				ifFalse: [myBottomLeft y + frameTopLeft y].
	^PDFPoint x: newX y: newY
			
! !

!PDFFramePagePlacement methodsFor:'services'!

close
	"^self 
	Calculate the final absolute rectangle, write out my border and mar myself as closed."

	self isClosed 
		ifFalse: 
			[self completeAbsoluteRectangle.
			self writeBorder.
			self parentPlacement updateCurrentInsertPoint: self absoluteRectangle.
			closed := true].
	^self
			
!

ensureSpaceFor: aFrame 
	"^a PDFFramePagePlacement
	Ensure sufficient space for a frame by closing myself if necessary, and
	opening a new placement for my frame on a new page.
	*** For now, I just worry about height."

	| requiredHeight availableHeight |
	requiredHeight := aFrame rectangle height.
	availableHeight := self currentInsertPoint y - self absoluteExtentLimit y.
	requiredHeight <= availableHeight ifTrue: [^self].
	requiredHeight < self maximumExtent y 
		ifTrue: 
			[self currentInsertPoint y = self initialInsertPoint y 
				ifFalse: [self close].
			self report newPage.
			^self frame currentPlacement]
		ifFalse: 
			[NativeError 
				raiseSignal: 'Not enough room to write given frame into meta-frame.']
			
!

initialInsertPoint
	"^a PDFPoint
	I return my initial insert point, which is the top left hand corner of my absolute rectangle."

	^self absoluteRectangle topLeft
			
!

moveToNewLine
	"^self 
	Moves the insert point to the extreme left of the frame, and moves 
	the y axis of the insert point down by the yAxisGrowth."

	| newYCoordinate newXCoordinate |
	newYCoordinate := self currentInsertPoint y - self yAxisGrowth.
	newXCoordinate := self absoluteRectangle topLeft x.
	self currentInsertPoint: (PDFPoint x: newXCoordinate y: newYCoordinate).
	self yAxisGrowth: 0.
	^self
			
!

parentPlacement
	"^a PDFFramePagePlacement
	I return the placement within which I am placed.  The root placement is the page itself."

	^self frame parent placementFor: self reportPage
			
!

updateCurrentInsertPoint: aRectangle 
	"^self 
	Updates the point at which text will be written. The point is updated given 
	aRectangle which represents the space taken up by some text this method assumes 
	has just been written. The insertion point is moved along the x axis by the width of the 
	rectangle, and the maximum text depth is updated is this is the deepest rectangle so far. 
	Note, the growth is noted as a positive number, even though growth really means moving 
	down the page, which means that growth might be better represented as a negative."

	self currentInsertPoint: (PDFPoint x: self currentInsertPoint x + aRectangle width y: self currentInsertPoint y).
	self yAxisGrowth < aRectangle height ifTrue: [self yAxisGrowth: aRectangle height].
	^self
			
! !

!PDFFramePagePlacement methodsFor:'services-queries'!

absoluteExtentLimit
	"^a PDFPoint
	I return the point which is the maximim extent available for my content."

	^self absoluteRectangle isComplete 
		ifTrue: [self absoluteRectangle bottomRight]
		ifFalse: [self parentPlacement absoluteExtentLimit]
			
!

filePage
	"^a PDFPage
	I return the page in the PDFFile my content will appear on."

	^self reportPage page
			
!

maximumExtent
	"^a PDFPoint
	I return a PDFPoint which represents the theoretical maximum extent possible for
	me.  e.g. the extent I could expand to on a blank page."

	^self frame maximumExtent
			
!

pageDescription
	"^a PDFPageDescription 
	Returns the current pageDescription within the report. That is, the pageDescription 
	that will be the target of any write commands. (It's simply the pageDescription 
	of the my page)."

	^self filePage pageDescription
			
!

report
	^self frame report
			
!

textObject
	"^a PDFTextObject
	I return the text object which is the target for text written to me."

	^self filePage pageDescription currentTextObject
			
! !

!PDFFramePagePlacement methodsFor:'testing'!

isClosed
	closed isNil ifTrue: [closed := false].
	^closed
			
! !

!PDFFramePagePlacement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFFramePagePlacement.st,v 1.1 2006/09/07 11:18:19 sr Exp $'
! !
