"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

Object subclass:#PDFReport
	instanceVariableNames:'body writer pageHeader currentLineNumber pages mediaFrame'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Reporting'
!


!PDFReport class methodsFor:'instance creation'!

on: aStream 
	"^a PDFReport 
	I create and initialize a new instance of myself."
 
	^self new on: (PDFWriter on: aStream)
			
! !

!PDFReport methodsFor:'accessing'!

body
	"^aPDFReportBody 
	I return my current body. I lazily create one if I don't already 
	have one."

	body isNil ifTrue: [body := self buildNewBody].
	^body
			
!

currentLineNumber
	"^an Integer 
	I return the current line number."

	currentLineNumber isNil ifTrue: [currentLineNumber := 0].
	^currentLineNumber
			
!

currentLineNumber: anInteger 
	"^self 
	I record the current line number."

	currentLineNumber := anInteger.
	^self
			
!

mediaFrame
	"^a PDFMediaFrame 
	I return the media frame.  This is the root of all frames for this report."

	mediaFrame isNil ifTrue: [mediaFrame := PDFMediaFrame within: self].
	^mediaFrame
			
!

pages
	"^a PDFReportPages 
	Returns the PDFReportPages which manages the pages (current 
	page etc) plus information needed by the frames for each page."

	pages isNil ifTrue: [pages := PDFReportPages for: self].
	^pages
			
!

writer
	"^a PDFWriter 
	I return the PDFWriter that I am to write to."

	^writer
			
!

writer: aWriter 
	"^self 
	I record the PDFWriter that I am to write to."

	writer := aWriter.
	^self
			
! !

!PDFReport methodsFor:'initialize-release'!

on: aWriter 
	"^self 
	I initialize and return myself."

	self writer: aWriter.
	^self
			
! !

!PDFReport methodsFor:'private'!

buildNewBody
	"^a PDFReportBody 
	I construct a new report body and return it. This involves 
	the creation of a new page in the underlying reports."

	self pages newPage.
	^PDFReportBody for: self
			
!

resetBody
	"^self 
	The current body is releasesd. The current line number is reset."

	body := nil.
	self resetCurrentLineNumber.
	^self
			
!

resetCurrentLineNumber
	"^self 
	Resets the current line number to 1."

	self currentLineNumber: 1.
	^self
			
! !

!PDFReport methodsFor:'services'!

absoluteRectangle
	"^a PDFRectangle 
	I return my absolute rectangle, which is the size of the paper I'm writing 
	to, which is defined by my media box."

	^self mediaBox
			
!

close
	"^self 
	Flushes the remaining text to the PDF document, and then closes the 
	PDFWriter."

	self writer close.
	^self
			
!

currentInsertPoint
	"^a PDFPoint 
	Returns the top left of the media box for the page. 
	.... well, for now anyway."

	^self absoluteRectangle topLeft
			
!

currentPage
	"^a PDFReportPage 
	I return the current page, that is the page that will be written to by default."

	^self pages current
			
!

incrementCurrentLineNumber
	"^self 
	Increments the current line number."

	self currentLineNumber: self currentLineNumber + 1.
	^self
			
!

mediaBox
	"^a PDFRectangle 
	I return the rectangle which defines the dimensions of the 
	media I'll be written to."

	^mediaFrame rectangle
			
!

mediaBox: aRectangle 
	"^self 
	I record the rectangle which defines the dimensions of the 
	media I'll be written to.  My media frame looks after this."

	self mediaFrame rectangle: aRectangle.
	^self
			
!

pageHeader
	"^a PDFReportPageHeader 
	I return my PDFReportPageHeader. I have a single page heder which I 
	use at the start of every new page in the target document."

	pageHeader isNil ifTrue: [pageHeader := PDFReportPageHeader for: self].
	^pageHeader
			
!

parent
	"^self
	This job should really be done by a media frame rather than by me. For me, this is a no-op."

	^self
			
!

placementFor: aReportPage 
	"^a PDFFramePagePlacement
	Returns my placement on aPage.  This job should really be done by a media frame rather than by me."

	^aReportPage framePlacementFor: self
			
!

report
	"^aPDFReport 
	I return my parent report. I don't have one.  This is me."

	^self
			
!

setOrientationToLandscape
	"^self 
	Sets the media for the report to landscape letter."

	self mediaBox: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 792 612)).
	^self
			
!

setOrientationToPortrait
	"^self 
	Sets the media for the report to portrait letter."

	self mediaBox: (PDFRectangle fromPDFStandardIntegerArray: #(0 0 612 792)).
	^self
			
!

throwPageIfNecessary
	"^self 
	Throws a page and prints out the header if the end of page has been reached. 
	NOTE: For now, the page is set to hold 42 lines."

	self currentLineNumber > 42 ifTrue: [self newPage].
	^self
			
!

updateCurrentInsertPoint: aRectangle 
	"^self 
	Updates the point at which the next frame will be written. 
	
	NOTE Currently, report does nothing about this."

	^self
			
! !

!PDFReport methodsFor:'writing'!

newLine
	"^self 
	Moves to a new line in the body of the target PDF document. This 
	may cause a page throw."

	self body newLine.
	^self
			
!

newPage
	"^self 
	Moves to the start of a new page in the target PDF Document and prints 
	the page header."

	self body flush.
	self resetBody.
	^self
			
!

write: aStringLikeThing 
	"^self 
	Writes aStringLikeThing to the current body of the PDF document. Note that there 
	is no new line or anything like that. The string like thing will either be a String or a 
	PDFCharacterFrame"

	self throwPageIfNecessary.
	self body write: aStringLikeThing asString.
	^self
			
!

writeHeader
	"^self 
	I get the header to write itself."

	self pageHeader writeOn: self writer.
	^self
			
!

writeLine: aString 
	"^self 
	Writes aString to the current page of the PDF document and then move to the 
	next line. This may cause a page throw."

	self write: aString.
	self newLine.
	^self
			
! !

!PDFReport class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__PDFReport.st,v 1.1 2006/09/07 11:08:09 sr Exp $'
! !
