"{ Package: 'stx:goodies/pdf' }"

"{ NameSpace: Squeak }"

TIFFObject subclass:#TIFFImageFileDirectory
	instanceVariableNames:'fields nextIFDOffset nextIFD'
	classVariableNames:''
	poolDictionaries:''
	category:'SPDF-Images'
!


!TIFFImageFileDirectory methodsFor:'accessing'!

fields
	"^an OrderedCollection
I return the field entries that make up the IDF I represent.  Each field entry
is 12 bytes long."

	| fieldOffset |
	fields isNil 
		ifTrue: 
			[fields := OrderedCollection new.
			1 to: self numberOfEntries
				do: 
					[:fieldNumber | 
					fieldOffset := self offset + (2 + ((fieldNumber - 1) * 12)).
					fields add: (TIFFField ofAppropriateClassWithin: self image at: fieldOffset)]].
	^fields
			
!

nextImageFileDirectory
	"^a TIFFImageFileDirectory or nil
	I return the next IFD if there is one, otherwise I return nil.  This is the IFD which follows me in the 
	TIFF file.  The offset of the next IFD follows all my field entries, and is a 4 byte value.
	We now have a really ugly fix for truncated documents where the next IFD pointer points
	beyond the end of the file."

	| startOffset |
	nextIFDOffset isNil 
		ifTrue: 
			[startOffset := self offset + 2 + (self numberOfEntries * 12).
			nextIFDOffset := self bytes readIntegerFrom: startOffset
						to: startOffset + 3.
			nextIFDOffset > self image fileSizeInBytes 
				ifTrue: [1 halt: 'We seem to have a truncated file, here']
				ifFalse: 
					[nextIFDOffset isZero 
						ifFalse: [nextIFD := self class within: self image at: nextIFDOffset]]].
	^nextIFD
			
! !

!TIFFImageFileDirectory methodsFor:'compression'!

decompressFrom: inputStream to: outputStream stripIndex: stripIndex 
	"^self
	I return a decompressed byte array.  I check the compression method used
	and use the appropriate decoder."

	| compressionType |
	compressionType := self compression value.
	compressionType == 1 
		ifTrue: 
			[self 
				decompressUncompressedBytesFrom: inputStream
				to: outputStream
				stripIndex: stripIndex. ^self].
	compressionType == 2 
		ifTrue: 
			[self decompressModifiedHuffmanBytesFrom: inputStream to: outputStream. ^self].
	compressionType == 3 
		ifTrue: [self decompressITUxT4BytesFrom: inputStream to: outputStream. ^self].
	compressionType == 4 
		ifTrue: 
			[self 
				decompressITUxT6BytesFrom: inputStream
				to: outputStream
				stripIndex: stripIndex. ^self].
	compressionType == 5 
		ifTrue: [self decompressLZWBytesFrom: inputStream to: outputStream. ^self].
	TIFFImageException raiseSignal: 'I can''t handle decompression of type ' 
				, compressionType printString
			
!

decompressITUxT4BytesFrom: inputStream to: outputStream 
	"^self
	I write the decompressed form of the data in inputStream to outputStream.  The bytes in inputStream are encoded using the (CCITT/)ITU T4 compression scheme."

	^(ITUxT4CodeReadStream onByteStream: inputStream) decodeTo: outputStream
			
!

decompressITUxT6BytesFrom: inputStream to: outputStream stripIndex: stripIndex
	"^self
	I write decompressed bytes to outputStream given that inputStream are encoded
	using the (CCITT/)ITU T6 compression scheme."

	| readStream stripHeight |
	stripHeight := self rowsPerStrip min: (self height - (self rowsPerStrip * (stripIndex - 1))).
	readStream := ITUxT6CodeReadStream 
				onByteStream: inputStream
				rowWidth: self width
				numberOfRows: stripHeight.
	self fillOrder = 2 ifTrue: [readStream bitStream setLowOrderBitFirst].
	readStream decodeTo: outputStream.
	^self
			
!

decompressLZWBytes: compressedBytes to: aStream 
	"^a ByteArray
	I return a decompressed byte given that compressedBytes are encoded
	using the TIFF 6.0 LZW compression scheme."

	^(LZWByteArray onStream: compressedBytes readStream) decompressedBytes
			
!

decompressModifiedHuffmanBytesFrom: inputStream to: outputStream 
	"^a ByteArray
	I write the decompressed form of the data in inputStream to outputStream.  The bytes in inputStream are encoded using the Modified Huffman compression scheme."

	^(ModifiedHuffmanCodeReadStream onByteStream: inputStream
		lineLength: self width) decodeTo: outputStream
			
!

decompressUncompressedBytesFrom: inputStream to: outputStream stripIndex: stripIndex 
	"^self
While no decompression is required, some processing may be.  For example, the PhotometricInterpretation field value may indicate that 1's and 0's have to be flipped to avoid a 'negative' image."

	self photometricInterpretation = 0 
		ifTrue: 
			[[inputStream atEnd] 
				whileFalse: [outputStream nextPut: (inputStream next bitXor: 255)]]
		ifFalse: [outputStream nextPutAll: inputStream contents].
	^self
			
! !

!TIFFImageFileDirectory methodsFor:'converting'!

asNativeByteArray
	"^a ByteArray
	I return a byte array which contains the image in a form suitable for an Image.  I have
	to increase the padding.  TIFF uses whole numbers of bytes, and Image uses whole
	numbers of 'words' (four bytes - 32 bits)."

	| outputStream rawBytes bytesPerRow bytesPastWordBoundary paddingBytesNeededPerRow numberOfRows rawBytesStream rawBytesWriteStream imageByteSize |
	bytesPerRow := (self width * self depth / 8) ceiling.
	imageByteSize := self height * bytesPerRow.
	rawBytesWriteStream := (ByteArray new: imageByteSize) writeStream.
	self writeRawImageBytesTo: rawBytesWriteStream.
	rawBytes := rawBytesWriteStream contents.
	numberOfRows := self height.
	bytesPastWordBoundary := bytesPerRow \\ 4.
	bytesPastWordBoundary == 0 
		ifTrue: [paddingBytesNeededPerRow := 0]
		ifFalse: [paddingBytesNeededPerRow := 4 - bytesPastWordBoundary].
	rawBytesStream := rawBytes readStream.
	outputStream := ByteArray new writeStream.
	"outputStream int32: imageByteSize."
	numberOfRows timesRepeat: 
			[outputStream nextPutAll: (rawBytesStream nextAvailable: bytesPerRow).
			paddingBytesNeededPerRow timesRepeat: [outputStream nextPut: 0]].
	^outputStream contents
!

asNativeImage
	"^an Image
	I return an Image which is a conversion of the image I represent."

	| form |
	form := Form 
		extent: self width @ self height 
		depth: self depth.
	form bits
		copyFromByteArray: self asNativeByteArray.
	^form
!

decodedBitReadStream
	"^a BitReadStream
	I return a bitstream on my decoded (uncompressed) image."

	| sourceByteArray |
	sourceByteArray := ByteArray 
				new: (self width * self depth * self height / 8) ceiling.
	self writeRawImageBytesTo: sourceByteArray writeStream.
	^BitReadStream on: sourceByteArray
			
!

numberOfNativeImageBytesPerRow
	"^an Integer
I return the number of bytes needed for each row when I am converted to a native byte image array.  This is the number of bytes needed per row in my representation, rounded up to 32 bits (a word)."

	| wordsPerRow |
	wordsPerRow := self width * self depth / 32.
	^wordsPerRow ceiling * 4
			
! !

!TIFFImageFileDirectory methodsFor:'services'!

bitsPerSample
	"^an Integer
The number of bits per pixel."

	| bitsPerSampleField |
	bitsPerSampleField := self fields detect: [:field | field tag = 258].
	^bitsPerSampleField value
			
!

compression
	"^an Integer
The compression code which may be 1, 2 or 32773."

	| compressionField |
	compressionField := self fields detect: [:field | field tag = 259].
	^compressionField value value
			
!

depth
	"^an Integer
The number of bits per pixel.  In TIFF terms, this is the sum of the bits per sample
for each of the samples per pixel.  In practice, I just sum all of the bits per pixel."

	| bitsPerSampleField totalBitsPerPixel |
	bitsPerSampleField := self fields detect: [:field | field tag = 258].
	totalBitsPerPixel := 0.
	bitsPerSampleField values 
		do: [:aTIFFValue | totalBitsPerPixel := totalBitsPerPixel + aTIFFValue value].
	^totalBitsPerPixel
			
!

fieldWithTag: anInteger 
	"^a TIFFField
	I return the TIFF field with a tag == an integer, or nil if no such
	field exists in this directory."

	^self fields detect: [:field | field tag == anInteger] ifNone: [nil]
			
!

fillOrder
	"^an Integer
I return the code which indicated the fill order of the bits within bytes in the image data.  If there is no fill order field, I simply return 1 - the defalt according to the spec."

	| fillOrderField |
	fillOrderField := self fields detect: [:field | field tag = 266] ifNone: [^1].
	^fillOrderField value value
			
!

height
	"^an Integer
The number of rows (sometimes described as scanlines) in the image."

	| heightField |
	heightField := self fields detect: [:field | field tag = 257].
	^heightField value value
			
!

numberOfEntries
	"^an Integer
	I return the number of field entries present in the IFD I represent."

	^self bytes readIntegerFrom: self offset to: self offset + 1
			
!

photometricInterpretation
	"^an Integer
The compression code which may be 0 or 1."

	| field |
	field := self fields detect: [:aField | aField tag = 262] ifNone: [^0].
	^field value value
			
!

rowsPerStrip
	"^an Integer
The number of rows per strip."

	| field |
	field := self fields detect: [:aField | aField tag = 278].
	^field value value
			
!

width
	"^an Integer
The number of columns in the image, i.e., the number of pixels per scanline."

	| widthField |
	widthField := self fields detect: [:field | field tag = 256].
	^widthField value value
			
!

writeRawImageBytesTo: aStream 
	"^self
	I write my decompressed and concatenated strips to aStream."

	| stripOffsets stripByteCounts offsetStream |
	stripOffsets := (self fieldWithTag: 273) values.
	stripByteCounts := (self fieldWithTag: 279) values.
	stripOffsets inject: 1
		into: 
			[:index :stripOffset | 
			offsetStream := OffsetReadStream 
						onCollection: self image bytes
						offset: stripOffset value + 1
						length: (stripByteCounts at: index) value.
			self decompressFrom: offsetStream to: aStream stripIndex: index.
			index + 1].
	^self
			
! !

!TIFFImageFileDirectory methodsFor:'testing'!

writeSummaryTo: aStream 
	"^self
I write a textual summary of myself to aStream."

	aStream
		nextPutAll: '*** Directory ***';
		cr;
		nextPutAll: 'Starting offset:  ';
		nextPutAll: self offset printString;
		cr;
		nextPutAll: 'There are ';
		nextPutAll: self fields size printString;
		nextPutAll: ' entries in this directory';
		cr.
	self fields do: [:field | field writeSummaryTo: aStream].
	^self
			
! !

!TIFFImageFileDirectory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/pdf/Squeak__TIFFImageFileDirectory.st,v 1.1 2006/09/07 11:22:40 sr Exp $'
! !
