"{ Package: 'stx:goodies/prototypeBehavior' }"

Object subclass:#PrototypeObject
	instanceVariableNames:'slots delegates'
	classVariableNames:''
	poolDictionaries:''
	category:'System-Prototypes'
!

PrototypeObject comment:'PrototypeObjects are objects with instance-specific behavior and structure. You can add and remove slots at any time with the messages

	PrototypeObject addSlot:

	PrototypeObject addSlot:withValue:

	PrototypeObject removeSlot:

Added slots are accessible with standard getter/setter messages.

Methods can be added and removed with

	PrototypeObject addMethod:

	PrototypeObject removeMethod:

PrototypeObjects have a class which is an instance of PrototypeBehavior. Look there for the internals of behavior sharing.



An example structure of PrototypeObjects: execute PrototypeObject exampleFamily.'
!


!PrototypeObject class methodsFor:'instance creation'!

new

	"Objects are instantiated with no slots"

	^self newPrototypeBehavior basicNew slots: #(); delegates: #()
!

newPrototypeBehavior

	"Create a new PrototypeBehavior for a cloned object"

	^PrototypeBehavior new fromSuperclass: self
! !

!PrototypeObject class methodsFor:'examples'!

exampleFamily

	"PrototypeObject exampleFamily"

	| person child parent dad mom son daughter jimmy patty |

	person _ self new.

	person addSlot: 'name'.

	parent _ person clone.

	parent addSlot: 'children'.



	dad _ parent clone.

	dad name: 'Daddy'.

	mom _ parent clone.

	mom name: 'Mommy'.

	dad addSlot: 'wife' withValue: mom.

	mom addSlot: 'husband' withValue: dad.



	child _ person clone.

	child addSlot: 'dad' withValue: dad.

	child addSlot: 'mom' withValue: mom.

	child addMethod: 'childSpec ^''child'''.

	child addMethod: 'printOn: aStream

					aStream

						nextPutAll: self name;

						nextPutAll: '', '', self childSpec, '' of '', self dad name;

						nextPutAll: '' and '', self mom name'.

	son _ child clone.

	son addMethod: 'childSpec ^''son'''.

	daughter _ child clone.

	daughter addMethod: 'childSpec ^''daughter'''.



	jimmy _ son clone.

	jimmy name: 'Jimmy'.

	patty _ daughter clone.

	patty name: 'Patty'.



	dad children: (OrderedCollection with: jimmy with: patty).

	mom children: dad children copy.



	dad inspect
! !

!PrototypeObject methodsFor:'accessing'!

delegateNames

	^self class delegateNames
!

methodNames

	^self class methodNames
!

methodSourceAt: aSymbol

	^self class sourceCodeAt: aSymbol
!

name

	^'a Protoype Object'
!

slotNames

	^self class slotNames
! !

!PrototypeObject methodsFor:'cloning'!

clone

	self class beShared.

	^super clone
		privatePostClone;
		postClone;
		yourself
!

postClone

	"instances can define something else"
! !

!PrototypeObject methodsFor:'delegation'!

addDelegate: aString withValue: anObject

	"Add a new slot with the given value. New clones of this object will also have that slot"

	delegates _ delegates copyWith: anObject.

	self ownClass compileDelegate: aString index: delegates size
!

removeDelegate: aString

	"The storage for the slot is not actually removed. This is left as an exercise for the reader"

	| setter getter |

	setter _ (aString, ':') asSymbol.
	getter _ aString asSymbol.
	self perform: setter with: nil.

	self ownClass
		removeSelector: setter;
		removeSelector: getter.

	
! !

!PrototypeObject methodsFor:'inspecting'!

defaultLabelForInspector

	^self name
!

inspect
	"Create and schedule an Inspector in which the user can examine the receiver's variables."

	PrototypeInspector openOn: self withEvalPane: true
! !

!PrototypeObject methodsFor:'printing'!

printOn: aStream

	aStream nextPutAll: self name
! !

!PrototypeObject methodsFor:'private'!

delegates: anArray

	delegates _ anArray
!

doesNotUnderstand: aMessage 
	| method result selector |
	selector _ aMessage selector.
	delegates do: [:each | each
		ifNotNil: [(method _ each getCompiledMethod: selector)
				ifNotNil: [method last = 250 | (method last = 251)
						ifTrue: [^ each perform: selector withArguments: aMessage arguments]
						ifFalse: [self class addSelector: selector withMethod: method.
							result _ self perform: selector withArguments: aMessage arguments.
							self class removeSelector: selector.
							^ result]]]].
	^ super doesNotUnderstand: aMessage
!

getCompiledMethod: aSelector
	| temp |
	^self class compiledMethodAt: aSelector ifAbsent: [
		delegates do: [:each |
			(temp _ each getCompiledMethod: aSelector) ifNotNil: [^temp]].
		^ nil]
!

ownClass

	| newClass |

	self class isShared ifTrue: [

		newClass _ self class cloneForModifiedObject.

		self become: (newClass basicNew slots: slots; delegates: delegates)].

	^self class
!

privatePostClone

	slots _ slots clone.
	delegates _ delegates clone
!

slots: anArray

	slots _ anArray
! !

!PrototypeObject methodsFor:'slot adding/removing'!

addMethod: aString
	"Add a new method to the receiver, or overwrite a method of the same name.
	Returns the selector of the new method"

	^self ownClass compile: aString
!

addSlot: aString

	^self addSlot: aString withValue: nil
!

addSlot: aString withValue: anObject
	"Add a new slot with the given value. New clones of this object will also have that slot"

	slots _ slots copyWith: anObject.
	self ownClass compileSlot: aString index: slots size
!

removeMethod: aSymbol

	self ownClass removeSelector: aSymbol
!

removeSlot: aString
	"The storage for the slot is not actually removed. This is left as an exercise for the reader"
	| setter getter |

	setter _ (aString, ':') asSymbol.
	getter _ aString asSymbol.

	self perform: setter with: nil.

	self ownClass
		removeSelector: setter;
		removeSelector: getter
! !

!PrototypeObject class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/prototypeBehavior/PrototypeObject.st,v 1.1 2003/09/09 09:45:25 cg Exp $'
! !
