"{ Package: 'stx:goodies/rdoit' }"

Object subclass:#RDoItServer
	instanceVariableNames:'socket myPortNumberOrPath connectProcess openErrorHandler
		udSocketFilename restartFromImage'
	classVariableNames:'LoggingAction ErrorLoggingAction Logging ErrorLogging
		RemoteExecutionQuerySignal ServersRunning
		DefaultExecutionPriority DefaultServerLoopPriority ErrorCatching
		ErrorWalkback StandardObjectQuerySignal RestartFromImage
		LocalConnectionsOnly'
	poolDictionaries:''
	category:'Net-Communication-Smalltalk'
!

RDoItServer class instanceVariableNames:'ConfirmNewHosts BadHosts OkHosts DefaultPortNrOrPath AllowAllHosts'

"
 No other class instance variables are inherited by this class.
"
!

ProtoObject subclass:#Multiplexer
	instanceVariableNames:'defaultStream defaultObject'
	classVariableNames:''
	poolDictionaries:''
	privateIn:RDoItServer
!

!RDoItServer class methodsFor:'documentation'!

documentation
"
    First, how to use:
    
    RDoit sits on a server socket, waiting for incoming requests.
    The socket is either a tcp socket defined by a numeric portNr, 
    or a unix domain socket defined by a path-String.

    Smalltalk side (see also the Launchers-Settings-Communications Dialog):

        RDoItServer initialize
                        To reset the authorized/unauthorized host sets.

        RDoItServer allowHost:'aHostname'
                        To add a host to the list of allowed hosts.
                        If a request comes in from an unknown host, a Dialog
                        pops up asking if this host is allowed. If it is,
                        future requests are OK without further Dialogs to pop up.

        RDoItServer denyHost:'aHostname'
                        To add a host to the list of denied hosts.
                        No requests are allowed from this host.        

        RDoItServer start 
                        To start a server on the default port.
                        The port is defined by any of (in the order given): 
                            the classVariable: 'DefaultPortNrOrPath' (numeric or string)
                            the environmentVariable 'STX_RDOIT_PORT' (numeric or string)
                            8004

        RDoItServer startAt:8004
                        To start a server on tcp port 8004.

        RDoItServer startAt:'/foo/bar/baz'
                        To start a server on unix domain socket '/foo/bar/baz'.

        RDoItServer logging:true
                        To turn on command logging. A false argument turns it off.

        RDoItServer killAll              
                        To kill all servers.

    or, in the Launchers-Settings-Communications Dialog, simply enable rdoit execution.


    Unix command (shell level) side:
        silent evaluation (doIt):
            rdoit ``50 factorial''
            rdoit ``FileBrowser open''
            rdoit ``FileBrowser openOnXScreenNamed:'foo:0'''

        printing the returned value (printIt):
            rdoit -print ``50 factorial''

        Notice: on the unix command side, the ST-expression has to be quoted in
        double quotes (in the above, these are shown as `` and '').

        The unix-commands C-source is found in goodies/rdoit.
        The complete command syntax is:
            rdoit [-print] [hostname [portNr]] quotedExpression

    ----------------------------------------------------------
    Original comment (by Bill Voss):


    This is a quick hack to allow an external C program to execute a Smalltalk doIt.

       I built this so I could put the Launcher options on my X11 root menu.
       I also wanted a tiny project to help me get back up to speed after
       not writing any Smalltalk code for a long time.

    The basic idea is that we start up a Smalltalk process which creates a socket,
    and sits there listening for connections.  When a connection comes in, it forks
    a subprocess to do the actual work.  This subprocess reads from the socket
    until it sees a CR, and then evaluates what it has read (simillar to DoIt in a workspace).
    The subprocess then closes its connection and dies, while the main process
    continues to wait for connections.   
    The server also catches the #returnFromSnapshot update: message, and restarts itself.

    The external C program 'rdoit hostname portnumber message' is similar to 'rsh' in operation.

    CAUTION: The doits are NOT logged in the change log.

    DANGER: Security is done on a host by host basis.  When a connection
            comes in from a previously unrecognized host, a dialog box
            pops up asking the user to authorize the host.
                Select yes, and all future requests from that host will suceed.
                Select no, and all future attempts will be denied.
                (This is done with the Sets BadHosts and OkHosts.)
            This means that while the server is running, ANY user on any
            authorized host could have your st80 process do anything.
            Including run any program on the system SO BE CAREFUL !!

    FUTURE WORK:  
            Fix security problems.
                (There are only two people with accounts on my system,
                 so I personally am not going to 'fix' this problem anytime soon.)
            Cleanup Code.
                (Especially consider logging the doits.)
            Find and Fix the Bugs.
                (I haven''t used the UnixSockets fileIn before, so I almost
                 certainly goofed somewhere.)


    [author:]
        Bill Voss               <voss@cs.uiuc.edu>     June 14th, 1991
        Claus Gittinger         changes for ST/X; added -print (result return)
                                minor additions: logging, cleanup on shutdown etc.

    [start with:]
        RDoItServer allowHost:'localhost'
        RDoItServer denyHost:'untrustedHost'
        RDoItServer start
        RDoItServer killAll              

    [see also:]
        Socket
"
!

examples
"
    rdoit: hostName return: aBoolean port: aPortNumber command: aSmalltalkCommand
        'send aSmalltalkCommand command to hostName using port aPortNumber'

        |sock gotLine sendMSG|

        sock := Socket newTCPclientToHost: hostName port:aPortNumber.
        sock isNil ifTrue:[
            ^nil
        ].
        Socket streamErrorSignal handle: [:ex|
            'oops' printCR.
            ex return
        ] do:[
            sock buffered:false.
            'can now do transfer via sock'.
            sendMSG := aSmalltalkCommand.
            aBoolean ifTrue:[
               sendMSG := '?',aSmalltalkCommand.
            ].
            sock nextPutLine: sendMSG.
            sock accept.
            sock readWait.
            sock buffered: false.
            gotLine := sock nextLine.
            gotLine isNil ifTrue:[
                sock close.
                ^nil
            ].
            (((gotLine withoutSpaces startsWith: 'nil') 
                      or: [gotLine withoutSpaces startsWith: '-Smalltalk'])
                      or: [gotLine withoutSpaces startsWith: 'Error']) ifTrue:[
               sock close.
               ^nil
            ].
            sock close.
        ].
        ^gotLine
"
! !

!RDoItServer class methodsFor:'instance creation'!

start
    "Start an rdoit server on the default port."

    ^ self new startServer

    "
     RDoItServer start.
    "


!

startAt: aPortNumber 
    "Start an rdoit server on a specified port."

    ^ self new startServerAt: aPortNumber

    "
     RDoItServer startAt:8004.

     RDoItServer startAt:9104.
    "
! !

!RDoItServer class methodsFor:'Signal constants'!

remoteExecutionQuerySignal
    ^ RemoteExecutionQuerySignal
!

standardObjectQuerySignal
    ^ StandardObjectQuerySignal
! !

!RDoItServer class methodsFor:'accessing'!

allowAllHosts
    "if set to true, all hosts are allowed.
     Watch out: no security anymore"

    ^ AllowAllHosts ? false
!

allowAllHosts:something
    "if set to true, all hosts are allowed.
     Watch out: no security anymore"

    AllowAllHosts := something.
!

defaultExecutionPriority
    ^ DefaultExecutionPriority
!

defaultExecutionPriority:something
    DefaultExecutionPriority := something.
!

defaultServerLoopPriority
    ^ DefaultServerLoopPriority
!

defaultServerLoopPriority:something
    DefaultServerLoopPriority := something.
! !

!RDoItServer class methodsFor:'class initialization'!

initialize
    "Initialize class variables."

    ServersRunning := 0.
    DefaultServerLoopPriority := DefaultExecutionPriority := Processor userBackgroundPriority.

    ConfirmNewHosts := true.
    BadHosts := Set new.
    OkHosts := Set new.
    RemoteExecutionQuerySignal := QuerySignal new.
    StandardObjectQuerySignal := QuerySignal new.

    self initializeLoggingActions.
    Logging := false.
    ErrorLogging := true.

    ErrorWalkback := false.
    ErrorCatching := true.

    "
     RDoItServer initialize.

     RDoItServer allowHost:'localhost'.
    "
! !

!RDoItServer class methodsFor:'defaults'!

defaultNumericPortNr
    ^ 8004
!

defaultPortNameInEnvironment
    ^ 'STX_RDOIT_PORT'
!

defaultPortNumber
    "the port we are listening on (by default).
     However, a server may also be started on another port"

    ^ self defaultPortNumberOrPath

    "Created: / 15.7.1998 / 16:35:28 / cg"
!

defaultPortNumber:aPortNrOrPath
    "set the default port we are listening on"

    self defaultPortNumberOrPath:aPortNrOrPath
!

defaultPortNumberOrPath
    "return the port we are listening on (by default).
     However, a server may also be started on another port"

    |envString port|

    DefaultPortNrOrPath notNil ifTrue:[^ DefaultPortNrOrPath].

    envString := OperatingSystem getEnvironment:(self defaultPortNameInEnvironment).
    envString notNil ifTrue:[
        port := Integer readFrom:envString onError:nil.
        port notNil ifTrue:[^ port].
        ^ envString
    ].
    ^ self defaultNumericPortNr

    "Created: / 15.7.1998 / 16:35:28 / cg"
!

defaultPortNumberOrPath:aPortNrOrPath
    "set the default port we are listening on.
     A numeric argument defines a tcp port; a string argument defines a unix domain sockets name"

    DefaultPortNrOrPath := aPortNrOrPath

    "
     RDoItServer defaultPortNumberOrPath:8005  
    "
!

restartFromImage
    ^ RestartFromImage ? true
!

restartFromImage:aBoolean
    "can be used to suppress a restart when running from an image"

    RestartFromImage := aBoolean
! !

!RDoItServer class methodsFor:'host management'!

allowHost:aHostname
    "allow a host"

    RemoteExecutionQuerySignal query == true ifTrue:[
        self log:'tried to allowHost from remote command'.
        self error:'allowHost is not allowed from remote command'.
    ].

    OkHosts add: aHostname.
    BadHosts remove:aHostname ifAbsent:[].

    "
     RDoItServer allowHost:'localhost'
     RDoItServer allowHost:'193.12.12.0'
     RDoItServer allowHost:'fooHost'
    "

    "Modified: / 20.5.1998 / 18:34:49 / cg"
!

badHosts

    ^BadHosts
!

confirmNewHost: hostName 
    "A previously unauthorized host is trying to connect.
     Ask the user if we can authorize this new host."

    | answer |

    ConfirmNewHosts ifFalse:[^ false].

    answer := OptionBox 
                request:('RDoit ok from host ' , 
                                   hostName printString , 
                                   ' ?')
                label:'Host confirmation' 
                buttonLabels:#('Allow Always' 'Deny Always' 'Allow' 'Deny')
                values:#(#allowAlways #denyAlways #allow #deny) 
                default:#allow
                onCancel:#deny.


    answer == #allowAlways ifTrue:[
        self allowHost:hostName.
        ^ true
    ].
    answer == #denyAlways ifTrue:[
        self denyHost:hostName.
        ^ false
    ].
    answer == #allow ifTrue:[
        ^ true
    ].
    ^ false.

"/    DialogView confirm: ('RDoit ok from host ' , 
"/                                   hostName printString , 
"/                                   ' ?') 
"/                         "initialAnswer: true".
"/    answer 
"/        ifTrue: [OkHosts add: hostName]
"/        ifFalse: [BadHosts add: hostName].
"/    ^ answer

    "Modified: / 20.5.1998 / 18:35:05 / cg"
!

confirmNewHosts:aBoolean
    ConfirmNewHosts := aBoolean
!

denyHost:aHostname
    "disallow a host"

    RemoteExecutionQuerySignal raise == true ifTrue:[
        self log:'tried to denyHost from remote command'.
        self error:'denyHost is not allowed from remote command'.
    ].

    BadHosts add: aHostname.
    OkHosts remove:aHostname ifAbsent:[].


    "
     RDoItServer denyHost:'untrustedHost'
    "

    "Modified: / 20.5.1998 / 18:34:55 / cg"
!

isHostAllowed:aHostName
    |isInBadHosts isInOkHosts ok realHostName|

    aHostName = 'localhost' ifTrue:[
        realHostName := OperatingSystem getHostName.
"/    ] ifFalse:[
"/        realHostName := aHostName
    ] ifFalse:[
        (self localConnectionsOnly ? true) ifTrue:[
            ^ false
        ].
    ].

    AllowAllHosts == true ifTrue:[^ true].

    ok := false. 
    isInBadHosts := (BadHosts includes:aHostName) or:[BadHosts includes:realHostName].
    isInBadHosts ifFalse:[ 
        isInOkHosts := (OkHosts includes:aHostName) or:[OkHosts includes:realHostName].
        isInOkHosts ifTrue: [
            ok := true.
        ] ifFalse: [
            (self confirmNewHost:aHostName) ifTrue: [
                ok := true
            ]
        ]
    ].
    ^ ok

    "Modified: / 20-05-2010 / 14:53:22 / cg"
!

localConnectionsOnly
    "if true (the default), only local connections are accepted"

    ^ LocalConnectionsOnly ? true

    "Modified: / 20-05-2010 / 14:50:58 / cg"
!

localConnectionsOnly:aBoolean
    "if true only local connections are accepted"

    LocalConnectionsOnly := aBoolean

    "Created: / 20-05-2010 / 14:51:13 / cg"
!

okHosts

    ^OkHosts
!

okHosts: aSet

    OkHosts := aSet
!

showHosts:aStream 
    "Output the current hosts, normally to stdout."

    self log: 'Authorized Hosts: ' , OkHosts printString.
    self log: 'Unauthorized Hosts: ' , BadHosts printString.

    "Modified: / 20.5.1998 / 18:08:51 / cg"
! !

!RDoItServer class methodsFor:'logging'!

errorCatching:aBoolean
    "turn on/off logging of errors"

    ErrorCatching := aBoolean

    "
     RDoItServer errorCatching:false
    "
!

errorLogging:aBoolean
    "turn on/off logging of erronous commands"

    ErrorLogging := aBoolean

    "
     RDoItServer errorLogging:false
    "
!

errorLoggingAction:aOneArgBlock
    ErrorLoggingAction := aOneArgBlock
!

errorWalkback:aBoolean
    "turn on/off printing of an error walkback"

    ErrorWalkback := aBoolean

    "
     RDoItServer errorWalkback:false
     RDoItServer errorWalkback:true
    "
!

initializeLoggingActions
    LoggingAction      := [:msg | ('RDoItServer [info]: ' , msg) infoPrintCR. ].
    ErrorLoggingAction := [:msg | ('RDoItServer [error]: ' , msg) errorPrintCR. ].
!

isErrorCatching
    "return true, if catching of errors is enabled"

    ^ ErrorCatching ? false
!

isErrorLogging
    "return true, if logging of erronous commands is enabled"

    ^ ErrorLogging ? false
!

isLogging
    "return true, if logging of commands is enabled"

    ^ Logging ? false
!

log:aMessage
    Logging == true ifTrue:[
        LoggingAction valueWithOptionalArgument:aMessage and:nil
    ].
!

log:aMessage detail:detail
    Logging == true ifTrue:[
        LoggingAction valueWithOptionalArgument:aMessage and:detail
    ].
!

logError:aMessage
    ((Logging == true) or:[ErrorLogging == true]) ifTrue:[
        ErrorLoggingAction valueWithOptionalArgument:aMessage and:nil
    ].
!

logError:aMessage detail:detail
    ((Logging == true) or:[ErrorLogging == true]) ifTrue:[
        ErrorLoggingAction valueWithOptionalArgument:aMessage and:detail
    ].
!

logging:aBoolean
    "turn on/off logging of commands"

    Logging := aBoolean
!

loggingAction:aOneArgBlock
    LoggingAction := aOneArgBlock
! !

!RDoItServer class methodsFor:'queries'!

runningServers
    "return a colelction of active servers"

    ^ self allInstances select:[:server | server isActive]
!

serverRunning
    "return true, if any server is running"

    ^ ServersRunning ~~ 0
! !

!RDoItServer class methodsFor:'server destruction'!

killAll
    "Destroy all instances of the server. (overkill)"

    RemoteExecutionQuerySignal query == true ifTrue:[
        self log:'tried to killAll from remote command'.
        self error:'killAll is not allowed from remote command'.
    ].

    self allInstances do: [:a | a release].

"claus: ORIGINAL
    RDoItServer allInstances do: [:a | a become: String new].
"
    self allInstances do: [:a | a becomeNil].
    ServersRunning := 0.

    ^nil

    "
     RDoItServer killAll.
    "

    "Modified: / 20.5.1998 / 18:23:29 / cg"
!

stop
    "Destroy all instances of the server. (overkill)"

    ^ self killAll

    "
     RDoItServer stop.
    "
! !

!RDoItServer methodsFor:'accessing'!

openErrorHandler
    "answer the error handler block for socket creation/bind errors"

    ^ openErrorHandler
!

openErrorHandler:aOneArgBlock
    "allow a handler to be called when socket creation/bind fails.
     This is only needed, when the socket is recreated at snapshotv restart.
     Otherwise you can handle the error at server creation time"

    openErrorHandler := aOneArgBlock.
!

restartFromImage
    "should the server be restarted from a snapshot?
     If nil, ask the class"

    ^ restartFromImage ? self class restartFromImage
!

restartFromImage:aBooleanOrNil
    restartFromImage := aBooleanOrNil.
! !

!RDoItServer methodsFor:'constants'!

handlePriority
    "The priority a handler process should run at."

    ^ DefaultExecutionPriority
!

myName
    ^ 'rdoit'
!

serverPriority
    "The priority the server process should run at."

    ^ DefaultServerLoopPriority.
! !

!RDoItServer methodsFor:'handle client'!

forkClientOn:aSocket
    |clientProcess|

    clientProcess := [
                        self handleClient:aSocket
                     ] forkAt:self handlePriority. 
    ^ clientProcess
!

handleClient:aSocket 
    "This method is forked off when a new connection arrives.
     It checks the host (allow/deny), possibly asking via a Dialog,
     then calls #handleOKClient: to do the real work."

    |hostName peerAddress|

    peerAddress := aSocket getPeer.
    peerAddress isLocal ifTrue:[
        hostName := 'localhost'
    ] ifFalse:[
        hostName := peerAddress hostNameOrAddressString.
    ].

    Processor activeProcess name:self myName, ' (from: ' , hostName , ')'.

    (self class isHostAllowed:hostName) ifTrue:[
        [
            Error handle:[:ex |
                ErrorCatching ifFalse:[ex reject].
                self logError:('cought error: ' , ex description). 
                ErrorWalkback ifTrue:[ thisContext fullPrintAll ].
            ] do:[
                self handleOkClient: aSocket.
            ].
            Delay waitForSeconds:0.1.
        ] ensure:[
            aSocket close.
        ]
    ] ifFalse:[
        self logError:('Denied rdoit attempt from: ' , hostName). 
        aSocket shutDown
    ].

    "Modified: / 20.5.1998 / 18:09:02 / cg"
!

handleOkClient:aSocket 
    "Called when a good client request arrives;
     Read the socket through cr, for the command, and then doit.
     new:
        If the first character of the command is a ?-character, strip it off
        and send the printString of the result back for rdoit to print it.
        If the first character of the command is a #-character, strip it off,
        read a numBytes number, fileIn that number of bytes,
        and enter pass-mode, where input arrives from rdoit and output is sent back
        to rdoit."

    |commandLine value exitCodeMsgString returnResult stxRunMode numBytesToFileIn code
     peekChar displayName myDisplay mustCloseDisplay|

    returnResult := false.
    stxRunMode := false.

    aSocket readWait.
        
    "/claus:
    "/ if commandLine starts with a (-character, 
    "/   read everything upTo $) and treat text in-between as
    "/   display connection to use.
    "/ if commandLine starts with a ?-character, 
    "/   send return value back as printString.
    "/ if commandLine starts with a #-character, 
    "/   go into command-mode
    "/
    peekChar := aSocket peek.
    peekChar == $( ifTrue:[
        aSocket next.
        displayName := aSocket upTo:$).
    ].
    peekChar == $? ifTrue:[
        aSocket next.
        returnResult := true.
    ] ifFalse:[
        "/ if command starts with a #-character, go into command-mode
        peekChar == $# ifTrue:[
            aSocket next.
            stxRunMode := true.
        ]
    ].

    commandLine := aSocket nextLine.
    commandLine isNil ifTrue:[
        "maybe socket has been closed"
        self logError:'connection with no command'.
        ^ #Error.
    ].

    stxRunMode ifTrue:[
        "/ script mode - load the code and run it
        numBytesToFileIn := Integer readFrom:commandLine onError:0.
        code := aSocket next:numBytesToFileIn.

        self log:('script from ' , aSocket getPeerName , ':').
        self log:('--> ' , code).

        exitCodeMsgString := '#0'.               "/ exitCode

        value := self 
                    handleStxRunClient:code display:displayName on:aSocket 
                    onError:[:eMsg | exitCodeMsgString := '-Smalltalk error: ' , eMsg].

        self log:('return to ' , aSocket getPeerName , ':').
        self log:('<-- exitCode:' , exitCodeMsgString).

        aSocket nextPutLine:exitCodeMsgString.
        ^ value
    ].

    "/ doit mode - execute the expression

"
'got request <' print. commandLine print. '>' printNL.
"

    self log:('request from ' , aSocket getPeerName) detail:commandLine.
    self log:('--> ' , commandLine).

    mustCloseDisplay := false.
    displayName notNil ifTrue:[
        [
            myDisplay := XWorkstation newDispatchingFor:displayName.
            mustCloseDisplay := true.
        ] on:Screen deviceOpenErrorSignal do:[:ex|
            self logError:('Smalltalk error: Could not open display:' , displayName).
            ^ #Error.
        ]
    ] ifFalse:[
        Screen notNil ifTrue:[
            myDisplay := Screen default.
        ]
    ].

    [
        "/ claus:
        "/  I catch all errors - and send some diagnostics to stderr instead
        "/
        SignalSet anySignal handle:[:ex |
            (HaltInterrupt accepts:ex signal) ifTrue:[
                self log:('halt/breakpoint ignored').
                ErrorCatching == true ifFalse:[ex reject].
                ex proceed
            ].

            self logError:('error during rdoit evaluation:').
            self logError:('  commandLine: ' , commandLine).
            self logError:('  exception  : ' , ex errorString).

            ErrorCatching == true ifFalse:[ex reject].

            value := nil.
            exitCodeMsgString := '-Smalltalk error: ' , ex errorString.
            ex return
        ] do:[
            |evalAction|

            evalAction :=
                [
                    value := Compiler evaluate:(commandLine readStream) notifying:self.
                    exitCodeMsgString := ' ' , value printString.
                ].

            RemoteExecutionQuerySignal answer:true do:[
                Screen isNil ifTrue:[
                    evalAction value.
                ] ifFalse:[
                    Screen currentScreenQuerySignal answer:myDisplay do:evalAction
                ].
            ]
        ].
    ] ensure:[
        mustCloseDisplay ifTrue:[
            myDisplay close
        ].
    ].

    returnResult ifTrue:[

        self log:('return to ' , aSocket getPeerName , ':').
        self log:('<-- "' , exitCodeMsgString , '"').

        aSocket nextPutLine:exitCodeMsgString.
    ].

    ^ value

    "Modified: / 11.1.1998 / 15:28:55 / stefan"
    "Modified: / 4.7.1999 / 19:26:52 / cg"
!

handleStxRunClient:code display:displayName on:aSocket onError:errAction
    |rslt simulatedObjects 
     sStdin sStdout sStderr sTranscript sDisplay
     myStdin myStdout myStderr myTranscript myDisplay mustCloseMyDisplay
     execContext ns|

    myStdin := ActorStream new.
    myStdin nextBlock:[' '].
    myStdin atEndBlock:[true].
                
    myStdout := ActorStream new.
    myStdout nextPutBlock:[:arg |    aSocket nextPutAll:'11 '; nextPut:arg].
    myStdout nextPutAllBlock:[:arg | aSocket nextPutAll:'1'; nextPutAll:(arg size printString); space; nextPutAll:arg].

    myStderr := ActorStream new.
    myStderr nextPutBlock:[:arg |    aSocket nextPutAll:'21 '; nextPut:arg].
    myStderr nextPutAllBlock:[:arg | aSocket nextPutAll:'2'; nextPutAll:(arg size printString); space; nextPutAll:arg].

    myTranscript := ActorStream new.
    myTranscript nextPutBlock:[:arg |    aSocket nextPutAll:'21 '; nextPut:arg].
    myTranscript nextPutAllBlock:[:arg | aSocket nextPutAll:'2'; nextPutAll:(arg size printString); space; nextPutAll:arg].

    simulatedObjects := IdentityDictionary new.

    simulatedObjects at:(sStdin := Multiplexer new defaultObject:Stdin) put:myStdin.
    simulatedObjects at:(sStdout := Multiplexer new defaultObject:Stdout) put:myStdout.
    simulatedObjects at:(sStderr := Multiplexer new defaultObject:Stderr) put:myStderr.
    simulatedObjects at:(sTranscript := Multiplexer new defaultObject:Transcript) put:myTranscript.

    mustCloseMyDisplay := false.
    displayName notNil ifTrue:[
        [
            myDisplay := XWorkstation newDispatchingFor:displayName.
            mustCloseMyDisplay := true.
        ] on:Screen deviceOpenErrorSignal do:[:ex|
            self logError:('Smalltalk error: Could not open display: ' , displayName).
            errAction value:('Could not open display: ' , displayName).
            ^ nil.
        ].
        simulatedObjects at:(sDisplay := Multiplexer new defaultObject:Display) put:myDisplay.
    ].

    [
        Stdin := sStdin.
        Stdout := sStdout.
        Stderr := sStderr.
        Transcript := sTranscript.
        myDisplay notNil ifTrue:[
            Display := sDisplay.
        ].

        Class withoutUpdatingChangesDo:[ 
            ns := NameSpace name:('Script' , Processor activeProcess id printString).
        ].

        SignalSet anySignal handle:[:ex |
            |con|

            (HaltInterrupt accepts:ex signal) ifTrue:[
                ErrorCatching == true ifFalse:[ex reject].
                self logError:('Smalltalk warning: halt/breakpoint ignored').
                ex proceed
            ].

            "/ if the connection is broken, cannot send a backtrace ...
            (Stderr errorSignal , AbortAllOperationRequest) handle:[:ex |
            ] do:[
                self logError:('Smalltalk error: ' , ex errorString).
                self logError:('Smalltalk backtrace: ').
                con := ex suspendedContext.
                [con notNil and:[con ~~ execContext]] whileTrue:[
                    self logError:(con fullPrintString).
                    con := con sender.
                ].
            ].
            errAction value:(ex description).
            rslt := nil.
            ex return
        ] do:[
            Screen currentScreenQuerySignal answer:myDisplay 
            do:[
                RemoteExecutionQuerySignal answer:true 
                do:[
                    Class nameSpaceQuerySignal answer:ns 
                    do:[
                        StandardObjectQuerySignal handle:[:ex |
                            ex proceedWith:(simulatedObjects at:ex parameter).
                        ] do:[
                            |s|

                            execContext := thisContext.

                            "/ rslt := code readStream fileIn.

                            s := code readStream.
                            [s atEnd] whileFalse:[
                                Class withoutUpdatingChangesDo:[ 
                                    rslt := s 
                                                fileInNextChunkNotifying:nil 
                                                passChunk:false silent:true.
                                ]
                            ].
                        ].
                    ].
                ].
            ]
        ].
    ] ensure:[
        Stdin := Stdin defaultObject.
        Stdout := Stdout defaultObject.
        Stderr := Stderr defaultObject.
        Transcript := Transcript defaultObject.

        (mustCloseMyDisplay and:[myDisplay notNil]) ifTrue:[
            myDisplay close.
            Display := Display defaultObject.
        ].

        "/ clean up
        Class withoutUpdatingChangesDo:[ 
            ns allClassesDo:[:eachClass |
                eachClass removeFromSystem 
            ].
            Smalltalk removeClass:ns.
        ].
    ].
    self log:('script result for ' , aSocket getPeerName , ':').
    self log:('--> ' , rslt printString).
    ^ rslt
!

serverLoop
    "Called when snapshot is started, and at instance creation.
     When a new connection arrives on my server socket,
     fork another subprocess to handle the request (in #handleClient:).

     Note: Someone else has already forked me, so I run as a subprocess
           and can block.

     SEE ALSO: 
        UnixSocketAccessor class howToImplementAServer."

    |thisSocket thisUdSocketFilename|

    thisSocket := socket.
    thisUdSocketFilename := udSocketFilename.

    [
        |stayInLoop|

        stayInLoop := true.
        [socket == thisSocket and:[stayInLoop]] whileTrue:[
            "/ don't move newskt out of the block,
            "/ it is local for each thread!!
            |newskt clientProcess|

            newskt := socket accept.
            newskt notNil ifTrue: [
                "/
                "/ 'got a connection ...' printNL.
                "/
                clientProcess := self forkClientOn:newskt.
            ] ifFalse:[
                stayInLoop := false.
                self logError:'socket accept failed'.
            ].
        ].
    ] ensure:[
        ServersRunning := (ServersRunning - 1) max:0.
        thisSocket notNil ifTrue: [
            thisSocket shutDown "close".
            thisUdSocketFilename notNil ifTrue:[
                thisUdSocketFilename remove.
            ].
        ].
        self release.
    ].

    "Modified: / 15.7.1997 / 12:20:04 / stefan"
    "Modified: / 4.7.1999 / 19:38:28 / cg"
! !

!RDoItServer methodsFor:'initialize-release'!

release
"/ claus:ORIGINAL
"/    (socket isKindOf: UnixSocketAccessor) ifTrue: [socket close].
"/

    (connectProcess notNil or:[socket notNil]) ifTrue:[
        self log:'shutting down.'.
    ].

    socket notNil ifTrue:[socket shutDown. "close"].
    socket := nil.
    super release.

    connectProcess notNil ifTrue:[
        connectProcess isDead ifFalse:[ connectProcess terminateGroup ].
        connectProcess := nil
    ]

    "Modified: / 20.5.1998 / 18:10:35 / cg"
! !

!RDoItServer methodsFor:'logging'!

log:aMessage
    self class log:aMessage.
!

log:aMessage detail:detail
    self class log:aMessage detail:detail.
!

logError:aMessage
    self class logError:aMessage.
! !

!RDoItServer methodsFor:'parser callbacks'!

correctableError:aString position:relPos to:relEndPos from:aCompiler
    "callBack from the compiler - ignored here"

    ^ #continue

    "Created: / 20.5.1998 / 18:44:37 / cg"
    "Modified: / 20.5.1998 / 18:48:14 / cg"
!

correctableSelectorWarning:aString position:pos1 to:pos2 from:aCompiler
    "callBack from the compiler - ignored here"

    ^ self warning:aString position:pos1 to:pos2 from:aCompiler
!

error:aString position:relPos to:relEndPos from:aCompiler
    "callBack from the compiler - ignored here"

    self logError:('evaluation error: ' , aString).    
    Error raiseErrorString:('Syntax: ' , aString).

    "Modified: / 20.5.1998 / 18:45:53 / cg"
!

warning:aString position:relPos to:relEndPos from:aCompiler
    "callBack from the compiler - ignored here"

    "Modified: / 20.5.1998 / 18:48:08 / cg"
! !

!RDoItServer methodsFor:'private'!

portNumber
    "Return my port number (tcp socket)  or path name (unix domain socket)."
    "Default st80 R4 == 8004 (totally arbitrary)."

    ^ self portNumberOrPath.
!

portNumberOrPath
    "Return my port number (tcp socket)  or path name (unix domain socket)."
    "Default st80 R4 == 8004 (totally arbitrary)."

    myPortNumberOrPath isNil ifTrue: [^ self class defaultPortNumberOrPath].
    ^myPortNumberOrPath

    "Modified: / 15.7.1998 / 16:36:01 / cg"
! !

!RDoItServer methodsFor:'queries'!

connectProcess
    "return the connectProcess"

    ^ connectProcess
!

isActive
    "return true, if I am active"

    ^ connectProcess notNil and:[connectProcess isDead not]
! !

!RDoItServer methodsFor:'restart-update'!

restart
    "Called when snapshot is restarted, and at instance creation.
     Raise an error if socket creation/bind fails."

    |portOrPath thisSocket|

    Logging == true ifTrue:[
        "/ self log:'started with:'.
        self class showHosts:Stderr.
    ].

    portOrPath := self portNumberOrPath.
    [
        portOrPath isNumber ifTrue:[
            thisSocket := Socket newTCPserverAtPort:portOrPath.
            udSocketFilename := nil.
        ] ifFalse:[
            udSocketFilename := portOrPath asFilename.
            [
                udSocketFilename remove.
            ] on:OperatingSystem accessDeniedErrorSignal do:[:ex| ].
            thisSocket := Socket newUNIXserverAt: portOrPath.
        ].
        thisSocket listenFor:5.
    ] on:Socket openErrorSignal do:[:ex|
        openErrorHandler notNil ifTrue:[
            openErrorHandler value:ex.
        ] ifFalse:[
            ex reject.
        ].
        ^ self.
    ].

    socket := thisSocket.

    connectProcess := [
                        self serverLoop
                      ] newProcess.

    connectProcess priority:self serverPriority.
    connectProcess beGroupLeader.
    connectProcess name:self myName,' server (port: ' , self portNumber printString , ')'.
    connectProcess resume.

    self log:'(re)started.'.

    "Modified: / 20.5.1998 / 18:08:07 / cg"
!

startServer
    "This is only run by SERVER instances at startup."

    ObjectMemory addDependent: self.

    ServersRunning isNil ifTrue:[
        ServersRunning := 1
    ] ifFalse:[
        ServersRunning := ServersRunning + 1.
    ].

    self restart.

    "
     RDoItServer killAll.
     RDoItServer start.
    "

    "Modified: 9.1.1997 / 15:22:53 / cg"
!

startServerAt: aPortNumberOrPath 
    "This is only run by SERVER instances at startup."

    myPortNumberOrPath := aPortNumberOrPath.
    self startServer
!

update:aSymbol 
    "Dependents of ObjectMemory are sent update: 
     #returnFromSnapshot when a snapshot is started."

    aSymbol == #returnFromSnapshot ifTrue: [
        self restartFromImage ifTrue:[
            "do not raise an error if socket creation fails"
            [
                self restart
            ] on:Socket openErrorSignal do:[:ex|
                self logError:( 'socket creation failed (port:' , self portNumber printString , ')' ). 
            ].
        ]
    ].
    super update: aSymbol.
    ^self

    "Modified: 9.1.1997 / 15:22:22 / cg"
! !

!RDoItServer::Multiplexer methodsFor:'accessing'!

defaultObject
    ^ defaultObject
!

defaultObject:something
    defaultObject := something.
! !

!RDoItServer::Multiplexer methodsFor:'error handling'!

doesNotUnderstand:aMessage
    |realObject|

    realObject := RDoItServer standardObjectQuerySignal raiseRequestWith:self.
    realObject isNil ifTrue:[
	realObject := defaultObject.
    ].
    ^ aMessage sendTo:realObject.
! !

!RDoItServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/rdoit/RDoItServer.st,v 1.78 2010/05/20 12:54:06 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/rdoit/RDoItServer.st,v 1.78 2010/05/20 12:54:06 cg Exp $'
! !

RDoItServer initialize!
