"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

RBProgramNode subclass:#RBMethodNode
	instanceVariableNames:'selector selectorParts body source arguments tags'
	classVariableNames:''
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBMethodNode comment:'BRMethodNode is the AST that represents a Smalltalk method.

Instance Variables:
	arguments	<SequenceableCollection of: BRVariableNode>	the arguments to the method
	body	<BRSequenceNode>	the body/statements of the method
	selector	<Symbol | nil>	the method name (cached)
	selectorParts	<SequenceableCollection of: BRValueToken>	the tokens for the selector keywords
	source	<String>	the source we compiled
	tag	<Interval | nil>	the source location of any resource/primitive tags

'
!


!RBMethodNode class methodsFor:'instance creation'!

selector: aSymbol arguments: variableNodes body: aSequenceNode 
	^(self new)
		arguments: variableNodes;
		selector: aSymbol;
		body: aSequenceNode;
		yourself
!

selector: aSymbol body: aSequenceNode 
	^self 
		selector: aSymbol
		arguments: #()
		body: aSequenceNode
!

selectorParts: tokenCollection arguments: variableNodes 
	^((tokenCollection detect: [:each | each isPatternVariable] ifNone: [nil]) 
		notNil ifTrue: [RBPatternMethodNode] ifFalse: [RBMethodNode]) 
		new selectorParts: tokenCollection arguments: variableNodes
! !

!RBMethodNode methodsFor:'accessing'!

addNode: aNode 
	^body addNode: aNode
!

addReturn
	body addReturn
!

addSelfReturn
	^body addSelfReturn
!

allArgumentVariables
	^(self argumentNames asOrderedCollection)
		addAll: super allArgumentVariables;
		yourself
!

allDefinedVariables
	^(self argumentNames asOrderedCollection) addAll: super allDefinedVariables;
		yourself
!

allVariablesOnScope
    ^ self argumentNames "allArgumentVariables" , (super allVariablesOnScope)

    "Created: / 15-10-2006 / 14:05:08 / cg"
!

argumentNames
        ^ arguments collect: [:each | each name]
!

arguments
	^arguments
!

arguments: variableNodes 
	arguments := variableNodes.
	arguments do: [:each | each parent: self]
!

body
	^body
!

body: stmtsNode 
	body := stmtsNode.
	body parent: self
!

children
    body isNil ifTrue:[ ^ arguments ].
    ^ arguments copyWith: body
!

primitiveSources
	^self tags 
		collect: [:each | self source copyFrom: each first to: each last]
!

selector
	^selector isNil
		ifTrue: [selector := self buildSelector]
		ifFalse: [selector]
!

selector: aSelector 
	| keywords numArgs |
	keywords := aSelector keywords.
	numArgs := aSelector numArgs.
	numArgs == arguments size 
		ifFalse: 
			[self 
				error: 'Attempting to assign selector with wrong number of arguments.'].
	selectorParts := numArgs == 0 
				ifTrue: [Array with: (RBIdentifierToken value: keywords first start: nil)]
				ifFalse: 
					[keywords first last == $: 
						ifTrue: [keywords collect: [:each | RBKeywordToken value: each start: nil]]
						ifFalse: [Array with: (RBBinarySelectorToken value: aSelector start: nil)]].
	selector := aSelector
!

source
	^source
!

source: anObject
	source := anObject
!

start
	^1
!

stop
	^source size
!

tags
	^tags isNil ifTrue: [#()] ifFalse: [tags]
!

tags: aCollectionOfIntervals 
	tags := aCollectionOfIntervals
!

temporaries
    body isNil ifTrue:[^ #()].
    ^ body temporaries
! !

!RBMethodNode methodsFor:'comparing'!

= anObject 
        self == anObject ifTrue: [^true].
        self class = anObject class ifFalse: [^false].
        (self selector = anObject selector) ifFalse: [^false].
        (body = anObject body) ifFalse: [^false].
        1 to: arguments size
                do: [:i | (arguments at: i) = (anObject arguments at: i) ifFalse: [^false]].
        ^true
!

equalTo: anObject withMapping: aDictionary 
        self class = anObject class ifFalse: [^false].
        (self selector = anObject selector 
                and: [body equalTo: anObject body withMapping: aDictionary]) 
                        ifFalse: [^false].
        1 to: arguments size
                do: 
                        [:i | 
                        ((arguments at: i) equalTo: (anObject arguments at: i)
                                withMapping: aDictionary) ifFalse: [^false].
                        aDictionary removeKey: (arguments at: i) name].
        ^self primitiveSources = anObject primitiveSources
!

hash
        ^(self selector hash bitXor: body hash)
                bitXor: arguments hash
! !

!RBMethodNode methodsFor:'copying'!

postCopy
	super postCopy.
	body := body copy.
	arguments := arguments collect: [:each | each copy]
! !

!RBMethodNode methodsFor:'initialization & release'!

selectorParts: tokenCollection arguments: variableNodes 
	selectorParts := tokenCollection.
	self arguments: variableNodes
! !

!RBMethodNode methodsFor:'matching'!

copyInContext: aDictionary 
	^(self class new)
		selectorParts: (selectorParts collect: [:each | each removePositions]);
		arguments: (arguments collect: [:each | each copyInContext: aDictionary]);
		body: (body copyInContext: aDictionary);
		source: (aDictionary at: '-source-');
		yourself
!

match: aNode inContext: aDictionary 
	self class == aNode class ifFalse: [^false].
	aDictionary at: '-source-' put: aNode source.
	self selector == aNode selector ifFalse: [^false].
	^(self matchList: arguments
		against: aNode arguments
		inContext: aDictionary)
			and: [body match: aNode body inContext: aDictionary]
! !

!RBMethodNode methodsFor:'printing'!

printOn: aStream
	aStream nextPutAll: self formattedCode
! !

!RBMethodNode methodsFor:'private'!

buildSelector
	| selectorStream |
	selectorStream := WriteStream on: (String new: 50).
	selectorParts do: [:each | selectorStream nextPutAll: each value].
	^selectorStream contents asSymbol
!

selectorParts
	^selectorParts
!

selectorParts: tokenCollection 
	selectorParts := tokenCollection
! !

!RBMethodNode methodsFor:'replacing'!

replaceNode: aNode withNode: anotherNode 
	aNode == body ifTrue: [self body: anotherNode].
	self arguments: (arguments 
				collect: [:each | each == aNode ifTrue: [anotherNode] ifFalse: [each]])
! !

!RBMethodNode methodsFor:'testing'!

defines: aName 
	^(arguments detect: [:each | each name = aName] ifNone: [nil]) notNil
!

isLast: aNode 
	^body isLast: aNode
!

isMethod
	^true
!

isPrimitive
        RBParser isSmalltalkX ifTrue:[
            (body notNil and:[body statements contains:[:node | node isSTXPrimitiveCCode]]) ifTrue:[
                ^ true
            ].
        ].
        ^tags notNil and: 
                        [tags isEmpty not and: 
                                        [(self primitiveSources detect: [:each | '*primitive*' match: each]
                                                ifNone: [nil]) notNil]]
!

lastIsReturn
	^body lastIsReturn
!

references: aVariableName 
	^body references: aVariableName
!

uses: aNode 
	^body == aNode and: [aNode lastIsReturn]
! !

!RBMethodNode methodsFor:'visitor'!

acceptCommentsAfterVisitor:aVisitor
    aVisitor formatMethodCommentsAfter:(self commentsAfter) for:self
!

acceptCommentsBeforeVisitor:aVisitor
    aVisitor formatMethodCommentsBefore:(self commentsBefore) for:self
!

acceptVisitor: aProgramNodeVisitor 
	^aProgramNodeVisitor acceptMethodNode: self
! !

!RBMethodNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBMethodNode.st,v 1.17 2008/07/12 15:49:14 cg Exp $'
! !
