"{ Package: 'stx:goodies/refactoryBrowser/parser' }"

Object subclass:#RBProgramNode
	instanceVariableNames:'parent commentsBefore commentsAfter lineNumber'
	classVariableNames:'FormatterClass CascadeListCharacter IgnoreInMatchCharacter
		ListCharacter LiteralCharacter RecurseIntoCharacter
		StatementCharacter'
	poolDictionaries:''
	category:'Refactory-AST-Nodes'
!

RBProgramNode comment:'BRProgramNode is an abstract class that represents an abstract syntax tree node in a Smalltalk program.

Subclasses must implement the following messages:
	accessing
		start
		stop
	visitor
		acceptVisitor:

The #start and #stop methods are used to find the source that corresponds to this node. "source copyFrom: self start to: self stop" should return the source for this node.

The #acceptVisitor: method is used by BRProgramNodeVisitors (the visitor pattern). This will also require updating all the BRProgramNodeVisitors so that they know of the new node.

Subclasses might also want to redefine match:inContext: and copyInContext: to do parse tree searching and replacing.

Subclasses that contain other nodes should override equalTo:withMapping: to compare nodes while ignoring renaming temporary variables, and children that returns a collection of our children nodes.

Instance Variables:
	comments        <Collection of: Interval>       the intervals in the source that have comments for this node
	parent  <BRProgramNode> the node we''re contained in

'
!


!RBProgramNode class methodsFor:'initialization'!

initialize
    CascadeListCharacter := $;.
    ListCharacter := $@.
    LiteralCharacter := $#.
    RecurseIntoCharacter := $`.
    StatementCharacter := $..
    IgnoreInMatchCharacter := $?.
! !

!RBProgramNode class methodsFor:'accessing'!

formatterClass
	^FormatterClass isNil ifTrue: [RBFormatter] ifFalse: [FormatterClass]
!

formatterClass: aClass 
	FormatterClass := aClass
!

optimizedSelectors
	^#(#== #ifTrue: #ifTrue:ifFalse: #ifFalse: #ifFalse:ifTrue: #whileTrue: #whileTrue #whileFalse: #whileFalse #to:do: #yourself #and: #or:)
! !

!RBProgramNode methodsFor:'accessing'!

asReturn
	"Change the current node to a return node."

	parent isNil ifTrue: [self error: 'Cannot change to a return without a parent node.'].
	parent isSequence ifFalse: [self error: 'Parent node must be a sequence node.'].
	(parent isLast: self)
		ifFalse: [self error: 'Return node must be last.'].
	^parent addReturn
!

children
	^#()
!

comments
	^ self commentsAfter
!

comments: aCollection
	^ self commentsAfter:aCollection
!

commentsAfter
	^commentsAfter isNil
		ifTrue: [#()]
		ifFalse: [commentsAfter]
!

commentsAfter: aCollection
	commentsAfter notEmptyOrNil ifTrue:[
	    commentsAfter := aCollection copy , commentsAfter.
	    commentsAfter sort:[:a :b | a start < b start].
	    ^ self.
	].
	commentsAfter := aCollection
!

commentsBefore
	^commentsBefore isNil
		ifTrue: [#()]
		ifFalse: [commentsBefore]
!

commentsBefore: aCollection
"/ commentsBefore notEmptyOrNil ifTrue:[self halt:'oops - loosing comments'].
        commentsBefore := aCollection
!

formattedCode
	^self formatterClass new format: self
!

formatterClass
	^self class formatterClass
!

lineNumber
"/    lineNumber isNil ifTrue:[
"/        self breakPoint:#cg info:'shouldn't this always be non-nil?'
"/    ].
    ^ lineNumber

    "Modified: / 06-03-2012 / 11:23:47 / cg"
!

lineNumber:anInteger
    lineNumber := anInteger.

    "Modified (format): / 06-03-2012 / 11:13:19 / cg"
!

originalCode
	^ (self source copyFrom:self start to:self stop) 
!

parent
	^parent
!

parent: anObject
	parent := anObject
!

precedence
	^6
!

setCommentsAfter: aCollection
	commentsAfter := aCollection
!

source
        ^parent notNil ifTrue: [parent source] ifFalse: [nil]
!

sourceInterval
    |start stop|

    start := self start.
    stop := self stop.
    start > stop ifTrue:[^ 1 to: 0].
    ^ start to: stop
!

start
	self subclassResponsibility
!

stop
	self subclassResponsibility
! !

!RBProgramNode methodsFor:'comparing'!

equalTo: aNode exceptForVariables: variableNameCollection 
	| dictionary |
	dictionary := Dictionary new.
	(self equalTo: aNode withMapping: dictionary) ifFalse: [^false].
	dictionary keysAndValuesDo: 
			[:key :value | 
			(key = value or: [variableNameCollection includes: key]) ifFalse: [^false]].
	^true
!

equalTo: aNode withMapping: aDictionary 
	^self = aNode
! !

!RBProgramNode methodsFor:'copying'!

copy
	"This is redefined for IBM Smalltalk which doesn't have postCopy."

	^self shallowCopy postCopy
!

copyCommentsFrom: aNode 
	"Add all comments from aNode to us. If we already have the comment, then don't add it."

	| newComments |
	newComments := OrderedCollection new.
	aNode nodesDo: [:each | newComments addAll: each comments].
	self nodesDo: 
			[:each | 
			each comments do: [:comment | newComments remove: comment ifAbsent: []]].
	newComments isEmpty ifTrue: [^self].
	newComments := newComments asSortedCollection: [:a :b | a first < b first].
	self comments: newComments
!

postCopy
! !

!RBProgramNode methodsFor:'enumeration'!

collect: aBlock 
	"Hacked to fit collection protocols"

	^aBlock value: self
!

do: aBlock 
	"Hacked to fit collection protocols"

	aBlock value: self
!

size
	"Hacked to fit collection protocols"

	^1
! !

!RBProgramNode methodsFor:'iteration'!

nodesDo: aBlock 
	aBlock value: self.
	self children do: [:each | each nodesDo: aBlock]
! !

!RBProgramNode methodsFor:'matching'!

copyInContext: aDictionary
	^self copy
!

copyList: matchNodes inContext: aDictionary 
	| newNodes |
	newNodes := OrderedCollection new.
	matchNodes do: 
			[:each | 
			| object |
			object := each copyInContext: aDictionary.
			newNodes addAll: object].
	^newNodes
!

match: aNode inContext: aDictionary 
	^self = aNode
!

matchList: matchNodes against: programNodes inContext: aDictionary 
	^self
		matchList: matchNodes
		index: 1
		against: programNodes
		index: 1
		inContext: aDictionary
!

matchList: matchNodes index: matchIndex against: programNodes index: programIndex inContext: aDictionary 
	| node currentIndex currentDictionary nodes |
	matchNodes size < matchIndex ifTrue: [^programNodes size < programIndex].
	node := matchNodes at: matchIndex.
	node isList 
		ifTrue: 
			[currentIndex := programIndex - 1.
                        
			[currentDictionary := aDictionary copy.
			programNodes size < currentIndex or: 
					[nodes := programNodes copyFrom: programIndex to: currentIndex.
					(currentDictionary at: node ifAbsentPut: [nodes]) = nodes and: 
							[(self 
								matchList: matchNodes
								index: matchIndex + 1
								against: programNodes
								index: currentIndex + 1
								inContext: currentDictionary) 
									ifTrue: 
										[currentDictionary keysAndValuesDo: [:key :value | aDictionary at: key put: value].
										^true].
							false]]] 
					whileFalse: [currentIndex := currentIndex + 1].
			^false].
	programNodes size < programIndex ifTrue: [^false].
	(node match: (programNodes at: programIndex) inContext: aDictionary) 
		ifFalse: [^false].
	^self 
		matchList: matchNodes
		index: matchIndex + 1
		against: programNodes
		index: programIndex + 1
		inContext: aDictionary
! !

!RBProgramNode methodsFor:'meta variable-accessing'!

cascadeListCharacter
	^$;
!

ignoreInMatchCharacter
        ^$?
!

listCharacter
	^$@
!

literalCharacter
	^$#
!

recurseIntoCharacter
	^$`
!

statementCharacter
	^$.
! !

!RBProgramNode methodsFor:'printing'!

printOn: aStream 
	aStream nextPutAll: self class name;
		nextPut: $(;
		nextPutAll: self formattedCode;
		nextPut: $)
! !

!RBProgramNode methodsFor:'querying'!

bestNodeFor: anInterval 
        | selectedChildren |
        (self intersectsInterval: anInterval) ifFalse: [^nil].
        (self containedBy: anInterval) ifTrue: [^self].
        selectedChildren := self children 
                                select: [:each | each intersectsInterval: anInterval].
        ^selectedChildren size >= 1 
                ifTrue: [selectedChildren first bestNodeFor: anInterval]
                ifFalse: [self]
!

firstLineNumber
    lineNumber isNil ifTrue:[
        self breakPoint:#cg info:'lineNumber should always be non-nil'.
        lineNumber := 1.
    ].
    ^ lineNumber
!

lastLineNumber
"/ lineNumber isNil ifTrue:[self halt:'should always be non-nil'].
    ^ lineNumber
!

selfMessages
	| searcher |
	searcher := ParseTreeSearcher new.
	searcher matches: 'self `@msg: ``@args'
		do: 
			[:aNode :answer | 
			answer
				add: aNode selector;
				yourself].
	^searcher executeTree: self initialAnswer: Set new
!

statementNode
	"Return your topmost node that is contained by a sequence node."

	(parent isNil or: [parent isSequence]) ifTrue: [^self].
	^parent statementNode
!

superMessages
	| searcher |
	searcher := ParseTreeSearcher new.
	searcher matches: 'super `@msg: ``@args'
		do: 
			[:aNode :answer | 
			answer
				add: aNode selector;
				yourself].
	^searcher executeTree: self initialAnswer: Set new
! !

!RBProgramNode methodsFor:'replacing'!

removeDeadCode
	self children do: [:each | each removeDeadCode]
!

replaceNode: aNode withNode: anotherNode 
	self error: 'I don''t store other nodes'
!

replaceWith: aNode 
	parent isNil ifTrue: [self error: 'This node doesn''t have a parent'].
	parent replaceNode: self withNode: aNode
! !

!RBProgramNode methodsFor:'searching'!

whichNodeIntersects: anInterval 
        | selectedChildren nChildren |

        (self intersectsInterval: anInterval) ifFalse: [^nil].

        selectedChildren := self children 
                                select: [:each | each intersectsInterval: anInterval].

        nChildren := selectedChildren size.
        nChildren == 0 ifTrue:[
            ^ self "/ I intersect
        ].
        nChildren == 1 ifTrue:[
            ^ selectedChildren first whichNodeIntersects: anInterval
        ].
"/ self halt:'should this happen ?'.
        ^ self "/ I intersect
!

whichNodeIsContainedBy: anInterval 
	| selectedChildren |
	(self intersectsInterval: anInterval) ifFalse: [^nil].
	(self containedBy: anInterval) ifTrue: [^self].
	selectedChildren := self children 
				select: [:each | each intersectsInterval: anInterval].
	^selectedChildren size == 1 
		ifTrue: [selectedChildren first whichNodeIsContainedBy: anInterval]
		ifFalse: [nil]
!

whoDefines: aName 
	^(self defines: aName)
		ifTrue: [self]
		ifFalse: [parent notNil
				ifTrue: [parent whoDefines: aName]
				ifFalse: [nil]]
! !

!RBProgramNode methodsFor:'special queries'!

addAllCommentsTo:aCollection
    aCollection addAll:(self commentsBefore).
    aCollection addAll:(self commentsAfter).
    self children do:[:eachChild |
        eachChild addAllCommentsTo:aCollection.
    ].

    "Created: / 30-06-2011 / 12:32:17 / cg"
!

addReferencedVariablesTo:aCollection
    self children do: [:each | each addReferencedVariablesTo:aCollection].
!

addUsedLiteralsTo:aCollection
    self children do: [:eachChild | eachChild addUsedLiteralsTo:aCollection].

    "Created: / 12-10-2006 / 21:24:46 / cg"
!

addUsedSymbolsTo:aCollection
    self children do: [:eachChild | eachChild addUsedSymbolsTo:aCollection].
!

allArgumentVariables
        | children |
        children := self children.
        children isEmpty ifTrue: [^#()].
        ^children 
                inject: OrderedCollection new
                into: 
                        [:vars :each |
                        each notNil ifTrue:[
                            vars
                                addAll: each allArgumentVariables.
                        ].
                        vars]
!

allComments
    |comments|

    comments := OrderedCollection new.
    self addAllCommentsTo:comments.
    ^ comments

    "Created: / 30-06-2011 / 12:32:43 / cg"
!

allDefinedVariables
	| children |
	children := self children.
	children isEmpty ifTrue: [^#()].
	^children inject: OrderedCollection new
		into: 
			[:vars :each | 
			vars addAll: each allDefinedVariables;
				yourself]
!

allTemporaryVariables
	| children |
	children := self children.
	children isEmpty ifTrue: [^#()].
	^children inject: OrderedCollection new
		into: 
			[:vars :each | 
			vars
				addAll: each allTemporaryVariables;
				yourself]
!

allVariablesOnScope
	^parent isNil
		ifTrue: [#()]
		ifFalse: [parent allVariablesOnScope]
!

blockVariables
	^parent isNil
		ifTrue: [#()]
		ifFalse: [parent blockVariables]
!

referencedVariables
        | variables |
        variables := Set new.
        self addReferencedVariablesTo:variables.
        ^variables
!

sentMessages
    | messages |
    messages := Set new.
    self children do: [:each | messages addAll: each sentMessages].
    ^messages

    "Modified: / 12-10-2006 / 21:26:32 / cg"
!

temporaryVariables
	^parent isNil
		ifTrue: [#()]
		ifFalse: [parent temporaryVariables]
!

usedLiterals
    | literals |

    literals := Set new.
    self addUsedLiteralsTo:literals.
    ^literals

    "Created: / 12-10-2006 / 21:24:34 / cg"
!

usedSymbols
    | symbols |

    symbols := IdentitySet new.
    self addUsedSymbolsTo:symbols.
    ^symbols

    "Modified: / 12-10-2006 / 21:26:26 / cg"
! !

!RBProgramNode methodsFor:'stx:libtool support'!

highlightInCodeView: aCodeView

    aCodeView highlightClear

    "Created: / 18-02-2008 / 17:48:12 / janfrog"
!

leftClickMenuInCodeView: aCodeView 
    ^ nil

    "Created: / 18-02-2008 / 17:49:43 / janfrog"
!

middleClickMenuInCodeView: aCodeView 
    ^ nil

    "Created: / 18-02-2008 / 17:49:57 / janfrog"
!

rightClickMenuInCodeView: aCodeView 
    ^ nil

    "Created: / 18-02-2008 / 17:50:10 / janfrog"
! !

!RBProgramNode methodsFor:'testing'!

assigns: aVariableName 
	^(self children detect: [:each | each assigns: aVariableName] ifNone: [nil]) 
		notNil
!

containedBy: anInterval 
	^anInterval first <= self start and: [anInterval last >= self stop]
!

containsReturn
	^(self children detect: [:each | each containsReturn] ifNone: [nil]) 
		notNil
!

defines: aName
	^false
!

directlyUses: aNode
	^true
!

evaluatedFirst: aNode 
	self children do: 
			[:each | 
			each == aNode ifTrue: [^true].
			each isImmediate ifFalse: [^false]].
	^false
!

intersectsInterval: anInterval 
        |myStart myStop ivStart ivStop|

        myStart := self start.
        myStop := self stop.
        ivStart := anInterval first.
        ivStop := anInterval last.
        ivStop < myStart ifTrue:[^ false].
        ivStart > myStop ifTrue:[^ false].
        ^ true
"/        ^(anInterval first between:myStart and:myStop) 
"/        or: [myStart between: anInterval first and: anInterval last]
!

isAssignment
	^false
!

isBlock
	^false
!

isCascade
	^false
!

isChildOf:anotherNode
    |nd p|

    nd := self.
    [nd notNil] whileTrue:[
	p := nd parent.
	p == anotherNode ifTrue:[^ true].
	nd := p
    ].
    ^ false
!

isDirectlyUsed
	"This node is directly used as an argument, receiver, or part of an assignment."

	^parent isNil
		ifTrue: [false]
		ifFalse: [parent directlyUses: self]
!

isEvaluatedFirst
	"Return true if we are the first thing evaluated in this statement."

	^parent isNil or: [parent isSequence or: [parent evaluatedFirst: self]]
!

isImmediate
	^false
!

isLast: aNode 
	| children |
	children := self children.
	^children isEmpty not and: [children last == aNode]
!

isLiteral
	^false
!

isLiteralArray
    ^false

    "Created: / 28-12-2011 / 16:05:40 / cg"
!

isLiteralByteArray
    ^false

    "Created: / 28-12-2011 / 16:05:33 / cg"
!

isLiteralNumber

    ^false

    "Created: / 31-10-2011 / 11:54:16 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

isLiteralString

    ^false

    "Created: / 28-12-2011 / 16:05:27 / cg"
!

isMessage
	^false
!

isMethod
	^false
!

isReturn
	^false
!

isSTXPrimitiveCCode
	^false
!

isSelf
    ^ false

    "Created: / 31-07-2011 / 01:00:41 / cg"
!

isSequence
	^false
!

isSqueakArray
        ^false
!

isUsed
	"Answer true if this node could be used as part of another expression. For example, you could use the 
	result of this node as a receiver of a message, an argument, the right part of an assignment, or the 
	return value of a block. This differs from isDirectlyUsed in that it is conservative since it also includes 
	return values of blocks."

	^parent isNil
		ifTrue: [false]
		ifFalse: [parent uses: self]
!

isValue
	^false
!

isVariable
	^false
!

isVariableNotInLeftHandSideOfAssignment
    "Returns true iff receiver is variable node but not
     in left-hand side of assignment"    
    ^false

    "Created: / 28-03-2012 / 17:14:56 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

lastIsReturn
	^self isReturn
!

references: aVariableName 
	^(self children detect: [:each | each references: aVariableName]
		ifNone: [nil]) notNil
!

uses: aNode
	^true
! !

!RBProgramNode methodsFor:'testing-matching'!

canMatchMethod: aCompiledMethod 
	self sentMessages do: 
			[:each | 
			((self class optimizedSelectors includes: each) 
				or: [aCompiledMethod refersToLiteral: each]) ifFalse: [^false]].
	^true
!

isList
	^false
!

isPatternNode
	^false
!

recurseInto
	^false
! !

!RBProgramNode methodsFor:'visitor'!

acceptCommentsAfterVisitor:aVisitor
    |comments|

    comments := self commentsAfter.
    comments notEmptyOrNil ifTrue:[
	aVisitor formatCommentsAfter:comments for:self
    ]
!

acceptCommentsBeforeVisitor:aVisitor
    |comments|

    comments := self commentsBefore.
    comments notEmptyOrNil ifTrue:[
	aVisitor formatCommentsBefore:comments for:self
    ]
!

acceptVisitor: aProgramNodeVisitor 
	self subclassResponsibility
! !

!RBProgramNode class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/refactoryBrowser/parser/RBProgramNode.st,v 1.47 2012/04/02 10:27:25 vrany Exp $'
! !

RBProgramNode initialize!
