'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:41'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WACache
	instanceVariableNames:'mutex keysByObject objectsByKey expiryPolicy reapingStrategy
		missStrategy removalAction'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-Cache'
!

WACache comment:'A WACache represents a collection of arbitrary objects referenced by a key (an instance of String).
Methods are provided to add and retrieve and remove objects from the cache. The behaviour of the cache is configurable through four different plugins:
+ expiry policy: determines when objects are considered expired
+ cache miss strategy: determines what do to if a requested object is not found in the cache
+ cache reap action: determines what to do when an expired object is eventually reaped from the cache
+ cache reaping strategy: determines when expired objects should be reaped from the cache
Access to the two dictionaries which store the objects and keys is protected by a mutex. Calls to the plugins are currently not protected by the mutex, but possibly should be to simplify implementation of the plugins.'
!

!WACache methodsFor:'configuration'!

keySize
	^ 16
! !

!WACache methodsFor:'copying'!

copy
	"Some of our operations replace the collections sotred in our instance variables. We don't
	want a copy to be made in the middle of this process (or it might have an unmatched pair)
	so we need to copy within our mutex."
	^ mutex critical: [ super copy ]
!

copyEmpty
	"Create a copy with no values."
	
	^ self species new
		setExpiryPolicy: self expiryPolicy copy;
		setMissStrategy: self missStrategy copy;
		setRemovalAction: self removalAction copy;
		setReapingStrategy: self reapingStrategy copy;
		yourself
!

postCopy
	super postCopy.
	"At this point we are sharing the mutex with the original object. This means
	we can safely copy our dictionaries and our plugins."
	mutex critical: [
		keysByObject := keysByObject copy.
		objectsByKey := objectsByKey copy.
		
		self setExpiryPolicy: self expiryPolicy copy.
		self setMissStrategy: self missStrategy copy.
		self setRemovalAction: self removalAction copy.
		self setReapingStrategy: self reapingStrategy copy ].
	"Now we're done and have all our own data so we want our own mutex."
	self initializeMutex
! !

!WACache methodsFor:'enumerating'!

keysAndValuesDo: aBlock

	objectsByKey keysAndValuesDo: aBlock
! !

!WACache methodsFor:'initialization'!

initialize
	super initialize.
	self initializeCollections.
	self initializeMutex.
	self setExpiryPolicy: WALastAccessExpiryPolicy new.
	self setReapingStrategy: WAAccessIntervalReapingStrategy new.
	self setMissStrategy: WACacheMissStrategy new.
	self setRemovalAction: WANotifyRemovalAction new
!

initializeCollections
	objectsByKey := GRPlatform current reducedConflictDictionary new.
	keysByObject := GRPlatform current reducedConflictDictionary new
!

initializeMutex
	mutex := WAMutex new
!

setExpiryPolicy: anExpiryPolicy
	expiryPolicy := anExpiryPolicy setCache: self; yourself
!

setMissStrategy: aMissStrategy
	missStrategy := aMissStrategy setCache: self; yourself
!

setReapingStrategy: aReapingStrategy
	reapingStrategy := aReapingStrategy setCache: self; yourself
!

setRemovalAction: aRemovalAction
	removalAction := aRemovalAction setCache: self; yourself
! !

!WACache methodsFor:'private'!

notifyRemoved: anObject key: aString
	self pluginsDo: [ :each | each removed: anObject key: aString ]
!

notifyRetrieved: anObject key: aString
	self pluginsDo: [ :each | each retrieved: anObject key: aString ]
!

notifyStored: anObject key: aString
	self pluginsDo: [ :each | each stored: anObject key: aString ]
!

pluginsDo: aBlock
	aBlock value: self expiryPolicy.
	aBlock value: self reapingStrategy.
	aBlock value: self missStrategy.
	aBlock value: self removalAction.
! !

!WACache methodsFor:'private-accessing'!

expiryPolicy
	^ expiryPolicy
!

missStrategy
	^ missStrategy
!

reapingStrategy
	^ reapingStrategy
!

removalAction
	^ removalAction
! !

!WACache methodsFor:'public'!

at: aString
	^ self at: aString ifAbsent: [ nil ]
!

at: aString ifAbsent: absentBlock
	"Return the object stored at the key aString. If the key is not found in the
	cache, return the result of evaluating absentBlock.
	The block should not be evaluated within the mutex since it could take a long time."
	
	| object |
	mutex critical: [
		object := objectsByKey at: aString ifAbsent: [ self missStrategy missed: aString ].
		object isNil ifFalse: [
			(self expiryPolicy isExpired: object key: aString)
				ifTrue: [
					"Should we remove the object here?"
					object := nil ]
				ifFalse: [ self notifyRetrieved: object key: aString ] ] ].
	
	^ object ifNil: [ absentBlock value ]
!

clear
	"Remove all objects from the cache. The removal action will be triggered for each
	object that is removed."
	
	| objects |
	mutex critical: [
		objects := objectsByKey.
		self initializeCollections.
		objects isNil ifFalse: [
			objects keysAndValuesDo: [ :key :object | self notifyRemoved: object key: key ] ] ]
!

keyAtValue: anObject
	^ self keyAtValue: anObject ifAbsent: [ nil ]
!

keyAtValue: anObject ifAbsent: absentBlock
	"Return the key at which anObject is stored. If anObject is not found in the
	cache, return the result of evaluating absentBlock.
	The block should not be evaluated within the mutex since it could take a long time."
	
	| key |
	key := mutex critical: [ keysByObject at: anObject ifAbsent: [ nil ] ].
	
	^ key ifNil: [ absentBlock value ]
!

reap
	"Iterate through the cache and remove objects that have expired."
	| newObjectsByKey newKeysByObject |
	newObjectsByKey := GRPlatform current reducedConflictDictionary new.
	newKeysByObject := GRPlatform current reducedConflictDictionary new.
	mutex critical:  [
		objectsByKey keysAndValuesDo: [ :key :object | 
			(self expiryPolicy isExpired: object key: key) 
				ifTrue: [
					self 
						notifyRemoved: object
						key: key ]
				ifFalse: [
					newObjectsByKey 
						at: key
						put: object.
					newKeysByObject 
						at: object
						put: key ] ].
		objectsByKey := newObjectsByKey.
		keysByObject := newKeysByObject ]
!

remove: anObject
	^ self remove: anObject ifAbsent: [ nil ]
!

remove: anObject ifAbsent: absentBlock
	"Remove anObject from the cache, returning the removed object. If the object is not
	found in the cache,	return the result of evaluating absentBlock.
	The block should not be evaluated within the mutex since it could take a long time."
	
	| key |
	mutex critical: [
		key := keysByObject at: anObject ifAbsent: [ nil ].
		key isNil ifFalse: [
			keysByObject removeKey: anObject ifAbsent: [ nil ].
			objectsByKey removeKey: key ifAbsent: [ nil ].
			self notifyRemoved: anObject key: key ] ].
		
	^ key isNil ifTrue: [ absentBlock value ] ifFalse: [ anObject ]
!

removeKey: aString
	^ self removeKey: aString ifAbsent: [ nil ]
!

removeKey: aString ifAbsent: absentBlock
	"Remove the object stored at the key aString and return the removed object. If the key
	is not found in the cache, return the result of evaluating absentBlock.
	The block should not be evaluated within the mutex since it could take a long time."
	
	| object |
	mutex critical: [
		object := objectsByKey removeKey: aString ifAbsent: [ nil ].
		keysByObject removeKey: object ifAbsent: [ nil ].
		object isNil ifFalse: [ self notifyRemoved: object key: aString ] ].
	
	^ object ifNil: [ absentBlock value ]
!

size
	"Retun the number of objects currently in the cache. This number includes objects that
	are expired but have not yet been removed in a cache reap."

	^ mutex critical: [ objectsByKey size ].
!

store: anObject
	"Store anObject in the cache, generating a new random key. Return the key."
	
	| key |
	mutex critical: [
		[ objectsByKey includesKey:
			(key := WAKeyGenerator current keyOfLength: self keySize) ] whileTrue.
		objectsByKey at: key put: anObject.
		keysByObject at: anObject put: key.
		self notifyStored: anObject key: key ].
	
	^ key
! !

!WACache methodsFor:'testing'!

isEmpty
	^ self size = 0
! !

