'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:44'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WARequestCookie subclass:#WACookie
	instanceVariableNames:'expiry secure discard comment commentUrl httpOnly maxAge'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WACookie comment:'I represent a cookie, a piece of information that is stored on the client and read and writable by the server. I am basically a key/value pair of strings.
You can never trust information in a cookie, the client is free to edit it.
I model only a part of the full cookie specification.
Browser support:
http://www.mnot.net/blog/2006/10/27/cookie_fun
Netscape spec
http://cgi.netscape.com/newsref/std/cookie_spec.html
Cookie spec
http://tools.ietf.org/html/rfc2109
Cookie 2 spec
http://tools.ietf.org/html/rfc2965
HttpOnly
http://msdn2.microsoft.com/en-us/library/ms533046.aspx
https://bugzilla.mozilla.org/show_bug.cgi?id=178993
Compared to WARequestCookie I represent the information that is sent to the user agent.'
!

!WACookie class methodsFor:'convenience'!

combine: aCollectionOfCookies using: aOneArgumentBlock
	"combines several cookies into one HTTP header line
	aOneArgumentBlock takes a cookie as an argument and returns a string,
	it is supposed to send one of #oldNetscapeString #rfc2109String #rfc2965String to the argument"
	^ String streamContents: [ :stream |
		aCollectionOfCookies
			do: [ :each |
				stream nextPutAll: (aOneArgumentBlock value: each) ]
			separatedBy: [ stream nextPutAll: ', ' ] ]
! !

!WACookie methodsFor:'accessing'!

comment
	^ comment
!

comment: aString
	comment := aString
!

commentUrl
	^ commentUrl
!

commentUrl: aUrl
	commentUrl := aUrl
!

discard
	^ discard
!

discard: aBoolean
	discard := aBoolean
!

expiry
	^ expiry
!

expiry: aDateTime
	expiry := aDateTime asUTC
!

httpOnly
	^ httpOnly
!

httpOnly: anObject
	httpOnly := anObject
!

maxAge
	^ maxAge
!

maxAge: anInteger
	"Optional.  The Max-Age attribute defines the lifetime of the
	cookie, in seconds.  The delta-seconds value is a decimal non-
	negative integer.  After delta-seconds seconds elapse, the client
	should discard the cookie.  A value of zero means the cookie
	should be discarded immediately."
	maxAge := anInteger
!

secure
	^ secure
!

secure: aBoolean
	secure := aBoolean
! !

!WACookie methodsFor:'api'!

expireIn: aDuration
	"sets the lifetime of the receiver mesaured in a duration"
	self expiry: DateAndTime now + aDuration.
	self maxAge: aDuration asSeconds
!

expireInPast
	self maxAge: 0.
	self expiry: (DateAndTime now - (Duration days: 10 hours: 0 minutes: 0 seconds: 0)).
!

expireInSeconds: anInteger
	"sets the lifetime of the receiver mesaured in seconds"
	self expiry: (Duration days: 0 hours: 0 minutes: 0 seconds: anInteger).
	self maxAge: anInteger
! !

!WACookie methodsFor:'initialization'!

initialize
	super initialize.
	self secure: false.
	self discard: false.
	self httpOnly: false.
	self version: 1
! !

!WACookie methodsFor:'private'!

expiryString
	^ GRPrinter cookieTimestamp print: expiry
! !

!WACookie methodsFor:'writing'!

oldNetscapeString
	"Serializes the receiver accroding to the orginal Netscape spec.
	This has the broadest browser support but limited features.
	It can not be used with HTTP header combining.
	See class comment for a link to the spec."
	^ String streamContents: [ :stream |
		self writeKeyValueQuoted: false on: stream.
		self writeExpiresOn: stream.
		self writePathQuoted: false on: stream.
		self writeDomainQuoted: false on: stream.
		self writeSecureOn: stream.
		self writeHttpOnlyOn: stream ]
!

oldNetscapeStringWithoutKey
	"same as oldNetscapeString except that the key is not included, used for Kom hacks"
	^ String streamContents: [ :stream |
		self value isNil ifFalse: [
			stream nextPutAll: self value ].
		self writeExpiresOn: stream.
		self writePathQuoted: false on: stream.
		self writeDomainQuoted: false on: stream.
		self writeSecureOn: stream.
		self writeHttpOnlyOn: stream ]
!

rfc2109String
	"Serializes the receiver accroding to RFC 2109.
	Almost no browser support.
	It can be used with HTTP header combining.
	See class comment for a link to the spec."
	^ String streamContents: [ :stream |
		self writeKeyValueQuoted: true on: stream.
		self writeCommentOn: stream.
		self writeMaxAgeOn: stream.
		self writePathQuoted: true on: stream.
		self writeDomainQuoted: true on: stream.
		self writeSecureOn: stream.
		self writeHttpOnlyOn: stream.
		self writeVersionOn: stream ]
!

rfc2965String
	"Serializes the receiver accroding to RFC 2965.
	This has the most features but is only supported in Opera.
	It can be used together with other serialization forms.
	It can be used with HTTP header combining.
	See class comment for a link to the spec."
	^ String streamContents: [ :stream |
		self writeKeyValueQuoted: true on: stream.
		self writeCommentOn: stream.
		self writeCommentUrlOn: stream.
		self writeMaxAgeOn: stream.
		self writePathQuoted: true on: stream.
		self writePortsOn: stream.
		self writeDomainQuoted: true on: stream.
		self writeSecureOn: stream.
		self writeHttpOnlyOn: stream.
		self writeDiscardOn: stream.
		self writeVersionOn: stream ]
!

writeOn: aStream
	aStream nextPutAll: self oldNetscapeString
! !

!WACookie methodsFor:'writing-private'!

write: aString asQuotedStringOn: aStream

	aStream nextPut: $".
	aString do: [ :each |
		each = $" 
			ifTrue: [ aStream nextPutAll: '\"']
			ifFalse: [ aStream nextPut: each ] ].
	aStream nextPut: $"
!

write: aString quoted: aBoolean on: aStream
	aBoolean
		ifTrue: [
			self
				write: aString
				asQuotedStringOn: aStream ]
		ifFalse: [ aStream nextPutAll: aString ]
!

writeCommentOn: aStream
	self comment isNil
		ifTrue: [ ^ self ].
	aStream nextPutAll: '; Comment='.
	self write: self comment asQuotedStringOn: aStream
!

writeCommentUrlOn: aStream 
	self commentUrl isNil ifTrue: [ ^ self ].
	aStream nextPutAll: '; CommentURL='.
	self 
		write: self commentUrl greaseString
		asQuotedStringOn: aStream
!

writeDiscardOn: aStream
	(self discard notNil and: [ self discard ])
		ifTrue: [ aStream nextPutAll: '; Discard' ]
!

writeDomainQuoted: aBoolean on: aStream
	domain isNil
		ifTrue: [ ^ self ].
	aStream nextPutAll: '; domain='.
	self write: self domain quoted: aBoolean on: aStream
!

writeExpiresOn: aStream
	expiry isNil
		ifTrue: [ ^ self ].
	aStream
		nextPutAll: '; expires=';
		nextPutAll: self expiryString
!

writeHttpOnlyOn: aStream
	self httpOnly
		ifTrue: [ aStream nextPutAll: '; HttpOnly' ]
!

writeKeyValueQuoted: aBoolean on: aStream
	aStream nextPutAll: self key.
	aStream nextPut: $=.
	self
		write: (self value ifNil: [ '' ])
		quoted: aBoolean
		on: aStream
!

writeMaxAgeOn: aStream
	maxAge isNil
		ifTrue: [ ^ self ].
	aStream nextPutAll: '; Max-Age="'.
	aStream print: maxAge.
	aStream nextPut: $"
!

writePathQuoted: aBoolean on: aStream
	path isNil
		ifTrue: [ ^ self ].
	aStream nextPutAll: '; path='.
	self write: self path quoted: aBoolean on: aStream
!

writePortsOn: aStream
	ports isEmptyOrNil
		ifTrue: [ ^ self ].
	aStream nextPutAll: '; Port="'.
	self ports do: [ :each | aStream print: each ] separatedBy: [ aStream nextPut: $, ].
	aStream nextPut: $"
!

writeSecureOn: aStream
	self secure
		ifTrue: [ aStream nextPutAll: '; Secure' ]
!

writeVersionOn: aStream
	aStream nextPutAll: '; Version='.
	aStream print: self version
! !

