'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:43'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WARequestHandler subclass:#WARegistry
	instanceVariableNames:'cache'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-RequestHandling'
!

WARegistry comment:'WARegistry maintains a set of handlers indexed by a key which it assigns when the handler is registerd. WARegistry checks incoming request URLs for a key and looks for a matching active request handler. If one exists, the request is sent to the proper handler. If not, the request is either a new request (in which case #handleDefaultRequest: is called) or a request to a now-inactive handler (in which case #handleExpiredRequest: is called). These two methods allow subclasses to properly handle these requests.
Subclasses must implement the following messages:
	handleDefaultRequest:
		Handle a request without a session key, ie a new request.
	handlerField
		The URL parameter in which to store the request handler key.
Instance Variables:
	cache - an instance of WACache to hold the stored request handlers
'
!

!WARegistry class methodsFor:'testing'!

isAbstract
	^ self name = #WARegistry
! !

!WARegistry methodsFor:'accessing'!

cache
	^ cache
!

cache: aCache
	cache := aCache
!

urlFor: aRequestHandler
	^ self
		urlFor: aRequestHandler
		ifAbsent: [ WAUnregisteredHandlerError signal ]
!

urlFor: aRequestHandler ifAbsent: absentBlock
	"Return the URL for the given request handler. If the handler is not registered, return
	the result of absentBlock.
	This URL will include the handler's key; if you don't want this, use #url instead."

	| key url |
	key := self keyFor: aRequestHandler ifAbsent: [ ^ absentBlock value ].
	url := self url.
	"Another option would be to have the Registry cache the cookie in the request context."
	(self hasCookieInContext: self requestContext withValue: key)
		"add the url field if the cookie is pointing to a different one"
		ifFalse: [ url addField: self keyField value: key ].
		
	^ url
! !

!WARegistry methodsFor:'configuration'!

defaultConfiguration
	^ super defaultConfiguration
		removeParent: WARequestHandlingConfiguration instance;
		addParent: WARegistryConfiguration instance;
		yourself
!

useCookies
	^ self configuration at: #useCookies
! !

!WARegistry methodsFor:'copying'!

postCopy
	super postCopy.
	"Since our handlers are dynamically registered and temporary, it makes
	sense not to copy them."
	cache := cache species new
! !

!WARegistry methodsFor:'deprecated'!

clearHandlers
	self 
		greaseDeprecatedApi: 'WARegistry>>#clearHandlers'
		details: 'Use #clear.'.
	^ self clear
!

ensureKeyForHandler: aRequestHandler 
	self 
		greaseDeprecatedApi: 'WARegistry>>#ensureKeyForHandler:'
		details: 'Use #ensureKeyFor:.'.
	^ self ensureKeyFor: aRequestHandler
!

keyOrNilForHandler: aRequestHandler 
	self 
		greaseDeprecatedApi: 'WARegistry>>#keyOrNilForHandler:'
		details: 'Use #keyFor:ifAbsent:.'.
	^ self 
		keyFor: aRequestHandler
		ifAbsent: [ nil ]
!

registerRequestHandler: aRequestHandler 
	self 
		greaseDeprecatedApi: 'WARegistry>>#registerRequestHandler:'
		details: 'Use register:.'.
	^ self register: aRequestHandler
!

unregisterRequestHandler: aRequestHandler 
	self 
		greaseDeprecatedApi: 'WARegistry>>#unregisterRequestHandler:'
		details: 'Use #unregister:.'.
	^ self unregister: aRequestHandler
!

urlForRequestHandler: aRequestHandler addHandlerField: aBoolean 
	| key url |
	self 
		greaseDeprecatedApi: 'WARegistry>>#urlForRequestHandler:addHandlerField:'
		details: 'Use #urlFor: or #urlFor:ifAbsent:. In any case where you do not want the handler field, simply use #url.'.
	key := self ensureKeyForHandler: aRequestHandler.
	url := self url.
	aBoolean ifTrue: 
		[ url 
			addField: self keyField
			value: key ].
	^ url
! !

!WARegistry methodsFor:'enumerating'!

handlersDo: aBlock
	self keysAndHandlersDo: [ :key :value | aBlock value: value ]
!

keysAndHandlersDo: aBlock

	self cache keysAndValuesDo: aBlock
! !

!WARegistry methodsFor:'handling'!

handleDefault: aRequestContext 
	"This method is called whenever a request is received with no key specified. This method can be overridden by subclasses to register a new handler to handle the request."
	
	aRequestContext responseGenerator
		notFound;
		respond
!

handleExpired: aRequestContext 
	"This method is called whenever a request is received with a key that does not match a registered handler."
	
	aRequestContext responseGenerator
		expiredRegistryKey;
		respond
!

handleFiltered: aRequestContext
	| cookie key handler urlKey |
	urlKey := self urlKeyFromContext: aRequestContext ifAbsent: [ nil ].
	cookie := self useCookies
		ifFalse: [ nil ]
		ifTrue: [ self cookieFromContext: aRequestContext ifAbsent: [ nil ] ].
	key := self keyFromFieldValue: urlKey cookie: cookie.
	key isNil
		ifTrue: [ self handleDefault: aRequestContext ]
		ifFalse: [
			handler := self cache at: key ifAbsent: [ nil ].
			handler isNil
				ifTrue: [
					cookie notNil ifTrue: [ self addDeletionOfCookie: cookie to: aRequestContext ].
					self handleExpired: aRequestContext ]
				ifFalse: [
					((self doesHandlerSupportCookies: handler) and: [ (cookie isNil or: [ (cookie value = key) not ]) and: [ self useCookies ] ])
						ifTrue: [ self addCookieForHandler: handler to: aRequestContext ] ].
					self handleKeyed: aRequestContext with: handler ]
!

handleKeyed: aRequestContext with: aRequestHandler
	"This method is called whenever a request is received with a key that matches
	a registered handler."
	
	self dispatch: aRequestContext to: aRequestHandler
! !

!WARegistry methodsFor:'handling-helpers'!

addCookieForHandler: aRequestHandler to: aRequestContext
	"Create a new handler key cookie and att it to the response."
	
	aRequestContext response addCookie: 
		(aRequestContext newCookie
			key: self keyCookieName;
			value: (self keyFor: aRequestHandler);
			discard: true;
			comment: 'the Seaside Session Cookie';
			commentUrl: 'http://www.seaside.st/documentation/sessionCookie';
			yourself)
!

addDeletionOfCookie: aCookie to: aRequestContext
	"Request the deletion of aCookie in the response."

	| baseUrl |
	baseUrl := self url.
	aRequestContext response
		deleteCookie: aCookie
		path: baseUrl pathString
		domain: baseUrl host
!

cookieFromContext: aRequestContext ifAbsent: absentBlock
	"Answer the handler key cookie. If none is found, return the result of evaluating absentBlock.
	There can be several cookies with the correct key so we try to find the one with the most specific path."

	| candidates requestUrl |
	candidates := aRequestContext request cookiesAt: self keyCookieName.
	
	candidates isEmptyOrNil
		ifTrue: [ ^ absentBlock value ].
	candidates size = 1 
		ifTrue: [ ^ candidates first ].
		
	requestUrl := aRequestContext request url.
	candidates := candidates collect: [ :each |
		each -> (each path seasideUrl relativeTo: requestUrl) ].
	
	candidates := candidates reject: [ :each |
		| path |
		path := each value.
		path notEmpty and: [ path first = '..' ] ].

	candidates isEmpty
		ifTrue: [ ^ absentBlock value ].
		
	candidates size > 1
		ifTrue: [
			candidates sort: [ :a :b |
				 a value size <= b value size ] ].
	
	^ candidates first key
!

dispatch: aRequestContext to: aRequestHandler
	"This method is used by the handling methods to trigger the dispatch of a request
	to the specified handler."
	
	aRequestHandler handle: aRequestContext
!

handle: aRequestContext registering: aRequestHandler
	"Register a new handler and then dispatch the request to it. A cookie may need
	to be added to the response with the handler's key."
	
	self register: aRequestHandler.
	self useCookies ifTrue: [ self addCookieForHandler: aRequestHandler to: aRequestContext ].
	self dispatch: aRequestContext to: aRequestHandler.
!

keyCookieName
	"Returns the key (name) of the key cookie."
	
	^ self keyField
!

keyField
	"Returns the name of the URL query field used to store the key."
	
	^ '_key'
!

keyFromFieldValue: aStringOrNil cookie: aCookieOrNil
	"Combine the keys from the URL and cookie.
	If we have a URL key we always use that. If not, we use the cookie if it is present."
	
	aStringOrNil isNil ifFalse: [ ^ aStringOrNil ].
	aCookieOrNil isNil ifFalse: [ ^ aCookieOrNil value ].
	^ nil
	
!

urlKeyFromContext: aRequestContext ifAbsent: absentBlock
	"Answer the key found in the request URL. If none is found, return the
	result of evaluating absentBlock."
	
	^ aRequestContext request at: self keyField ifAbsent: absentBlock
! !

!WARegistry methodsFor:'initialization'!

initialize
	super initialize.
	cache := WACache new
! !

!WARegistry methodsFor:'parent/child'!

childrenDo: aBlock
	self cache keysAndValuesDo: [ :key :value | aBlock value: value ]
!

clear
	"Remove all registered handlers."

	self cache clear
!

ensureKeyFor: aRequestHandler
	"Ensure the request handler is registered and returns its key."
	
	^ self keyFor: aRequestHandler ifAbsent: [ self register: aRequestHandler ]
!

keyFor: aRequestHandler
	^ self
		keyFor: aRequestHandler
		ifAbsent: [ WAUnregisteredHandlerError signal ]
!

keyFor: aRequestHandler ifAbsent: absentBlock
	^ (self cache keyAtValue: aRequestHandler) ifNil: [ absentBlock value ]
!

register: aRequestHandler
	^ self cache store: (aRequestHandler setParent: self; yourself)
!

unregister: aRequestHandler
	^ self cache remove: aRequestHandler
!

unregistered
	super unregistered.
	self clear
! !

!WARegistry methodsFor:'private'!

doesHandlerSupportCookies: aRequestHandler
	^ true
! !

!WARegistry methodsFor:'testing'!

hasCookieInContext: aRequestContext
	self cookieFromContext: aRequestContext ifAbsent: [ ^ false ].
	^ true
!

hasCookieInContext: aRequestContext withValue: aString 
	| cookie |
	cookie := self cookieFromContext: aRequestContext ifAbsent: [ ^ false ].
	^ cookie value = aString
!

isRegistry
	^ true
! !

