'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:45'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WARequest
	instanceVariableNames:'method uri version remoteAddress headers cookies body postFields'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WARequest comment:'I am a server independent http request object. Instance of me can be aquired through WAObject >> #currentRequest.
Instance Variables
	url:				<WAUrl>
	fields:			<Dictionary<String, Object>>
	headers:		<Dictionary<String, String>>
	nativeRequest:	<Object>
	cookies:			<Collection<WARequestCookie>>
	method:			<String>
	remoteAddress:	<String>
	body:				<String>
url
	- The request url without parameters. For example if the client requested ''http://www.google.com/search?q=seaside'' then the contents of url would be ''/search''. To get the parameters use #fields. This url is fully decoded. Use the #host method to get the host name. Dependening on the server adapter the #scheme may be ''http'' or ''https'' if the original request was HTTPS.
headers
	- The header of the HTTP request. This is a Dictionary mapping lowercase strings to other strings.
	
fields
	- The HTTP request parameters. These are the GET and POST parameters in combined into a single dictionary. In general this is a dictionary mapping Strings to Strings. In the case of multivalued paramters or parameters specified in both GET and POST it maps Strings to a Collection of Strings.
	
cookies
	- The collection of cookies (instance of WARequestCookie) the client sent. Note not all clients support all fields. E.g. you might send a path but the client might not return it. Note there can be several cookies with the same key but a different domain or path. See the #cookiesAt: method.
	
nativeRequest
	- The underlying request object of the web server (Kom, Swazoo, OpenTalk, FastCGI, ...). Accessing this makes your code unportable.
	
method
	- the HTTP method, should be upper case. In general only ''GET'' and ''POST'' are encountered in Seaside. SqueakSource also supports ''PUT''.
	
remoteAddress
	- The IP address of the client. If the server is behind a reverse proxy then this is ''127.0.0.1''. This could in theory also be an IPv6 address.
	
body
	- The undecoded, raw request body as a String, may be nil. See the "accessing-body" protocol for accessing it.'
!

!WARequest class methodsFor:'instance creation'!

method: methodString uri: aUrlOrString
	^ self method: methodString uri: aUrlOrString version: self defaultVersion
!

method: methodString uri: aUrlOrString version: aVersion
	^ self basicNew
		initializeWithMethod: methodString
			uri: aUrlOrString
			version: aVersion;
		yourself
!

new
	^ self
		method: self defaultMethod
		uri: self defaultUri
		version: self defaultVersion
! !

!WARequest class methodsFor:'configuration'!

defaultMethod
	^ 'GET'
!

defaultUri
	^ '/' seasideUrl
!

defaultVersion
	^ WAHttpVersion http11
! !

!WARequest class methodsFor:'deprecated'!

method: methodString url: urlOrString headers: headDict fields: fieldDict cookies: cookieDict 
	self 
		greaseDeprecatedApi: 'WARequest class>>#method:url:headers:fields:cookies:'
		details: 'Create Requests with #method:uri:version: or #method:uri: and use accessors to set other values.'.
	^ (self 
		method: methodString
		uri: urlOrString)
		setHeaders: headDict;
		setPostFields: fieldDict;
		setCookies: cookieDict;
		yourself
! !

!WARequest methodsFor:'accessing'!

at: key
	^ self fields at: key
!

at: key ifAbsent: aBlock
	^ self fields at: key ifAbsent: aBlock
!

at: key ifPresent: aBlock
	^ self fields at: key ifPresent: aBlock
!

fields
	^ WAMergedRequestFields on: (Array with: self queryFields with: self postFields)
!

headerAt: aKey
	^ self headerAt: aKey ifAbsent: [ nil ]
!

headerAt: aKey ifAbsent: aBlock
	^ headers at: aKey ifAbsent: aBlock
!

headers
	^ headers
!

method
	^ method
!

password
	"Answer the password from basic authentication."
	
	| authorization |
	^ (authorization := self authorization) isNil
		ifFalse: [ authorization copyAfter: $: ]
!

queryFields
	^ self uri queryFields
!

remoteAddress
	"answer the IP address of the client (String)"
	^ remoteAddress
!

uri
	^ uri
!

url
	^ self uri
!

user
	"Answer the username from basic authentication."
	
	| authorization |
	^ (authorization := self authorization) isNil
		ifFalse: [ authorization copyUpTo: $: ]
!

version
	^ version
!

version: aVersion
	version := aVersion
! !

!WARequest methodsFor:'accessing-body'!

bodyDecodeUsing: aCharSetName 
	"Answer the body decoded using the given character set name. Answer nil if there's no body."
	^ body isNil ifFalse: [ (GRCodec forEncoding: aCharSetName) decode: body ]
!

bodyDecoded
	"Answer the body decoded using the character set in the request header. Answer nil if no body is present. Signal an error if not character set is present in the request header."
	| contentType charSet |
	contentType := self contentType.
	charSet := contentType isNil ifFalse: [ contentType charSet ].
	^ charSet isNil
		ifTrue: [ WAIllegalStateException signal: 'no character set of request body can not be determined' ]
		ifFalse: [ self bodyDecodeUsing: charSet ]
!

postFields
	^ postFields
!

rawBody
	"Answer the raw, undecoded body. May be nil.
	
	Use #bodyDecoded or #bodyDecodeUsing: to access the decoded contents."

	^ body
! !

!WARequest methodsFor:'accessing-cookies'!

cookieAt: aKey
	"Answer the first found cookie with the given key, there might be several cookies with the name key but different paths, in this case use #cookiesAt:."

	^ self cookies
		detect: [ :each | each key = aKey ]
		ifNone: [ nil ]
!

cookies
	^ cookies ifNil: [ #() ]
!

cookiesAt: aKey
	"Answer all the cookies with the given key."
	
	^ self cookies
		select: [ :each | each key = aKey ]
! !

!WARequest methodsFor:'accessing-headers'!

accept
	"The Accept request-header field can be used to specify certain media types which are acceptable for the response. Accept headers can be used to indicate that the request is specifically limited to a small set of desired types, as in the case of a request for an in-line image.
	
	Answers an OrderedCollection of WAAccept with the values in the order the client sent them."

	^ WAAccept fromString: (self headerAt: 'accept' ifAbsent: [ ^ nil ])
!

acceptCharset
	"The Accept-Charset request-header field can be used to indicate what character sets are acceptable for the response. This field allows clients capable of understanding more comprehensive or special- purpose character sets to signal that capability to a server which is capable of representing documents in those character sets.
	
	Answers an OrderedCollection of WAAcceptCharset with the values in the order the client sent them."

	^ WAAcceptCharset fromString: (self headerAt: 'accept-charset' ifAbsent: [ ^ nil ])
!

acceptEncoding
	"The Accept-Encoding request-header field is similar to Accept, but restricts the content-codings (section 3.5) that are acceptable in the response.
	
	Answers an OrderedCollection of WAAcceptEncoding with the values in the order the client sent them."

	^ WAAcceptEncoding fromString: (self headerAt: 'accept-encoding' ifAbsent: [ ^ nil ])
!

acceptLanguage
	"The Accept-Language request-header field is similar to Accept, but restricts the set of natural languages that are preferred as a response to the request.
	
	Answers an OrderedCollection of WAAcceptLanguage with the values in the order the client sent them."

	^ WAAcceptLanguage fromString: (self headerAt: 'accept-language' ifAbsent: [ ^ nil ])
!

authorization
	"Answer the basic authorization string from the request. This is the username and the password separated by a colon."
	
	| authorization |
	authorization := self headerAt: 'authorization'.
	^ authorization isNil ifFalse: [ self decodeAuthorization: authorization ]
!

contentType
	"The Content-Type entity-header field indicates the media type of the entity-body sent to the recipient or, in the case of the HEAD method, the media type that would have been sent had the request been a GET.
	
	Answer an instance of WAMimeType or nil."
	
	| contentType |
	contentType := self headerAt: 'content-type'.
	^ contentType isNil ifFalse: [ WAMimeType fromString: contentType ]
!

host
	^ self headerAt: 'host'
!

referer
	^ self headerAt: 'referer'
!

userAgent
	^ self headerAt: 'user-agent'
! !

!WARequest methodsFor:'initialization'!

destroy
	method := uri := version := postFields := remoteAddress := headers := cookies := nil
!

initializeWithMethod: methodString uri: aUrlOrString version: aVersion
	self initialize.
	method := methodString asUppercase.
	uri := aUrlOrString seasideUrl.
	version := aVersion.
	headers := WAHeaderFields new.
	postFields := Dictionary new.
	cookies := OrderedCollection new.
	remoteAddress := nil
!

setBody: aString
	body := aString
!

setCookies: cookieCollection
	cookies := cookieCollection
!

setHeaders: headerDictionary
	headers := headerDictionary
!

setPostFields: aRequestFields
	postFields := aRequestFields seasideRequestFields
!

setRemoteAddress: addressString
	remoteAddress := addressString
! !

!WARequest methodsFor:'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream
		nextPutAll: ' method: ';
		nextPutAll: self method.
	aStream
		nextPutAll: ' uri: ';
		nextPutAll: self uri greaseString.
	aStream nextPutAll: ' version: '.
	self version printOn: aStream
! !

!WARequest methodsFor:'private'!

decodeAuthorization: aString 
	^ GRPlatform current base64Decode: (aString subStrings: ' ') last
! !

!WARequest methodsFor:'testing'!

includesKey: key
	^ self fields includesKey: key
!

isGet
	^ self method = 'GET'
!

isHttp10
	^ version isHttp10
!

isHttp11
	^ version isHttp11
!

isPost
	^ self method = 'POST'
!

isPrefetch
	"Link prefetching is a browser mechanism, which utilizes browser idle time to download or prefetch documents that the user might visit in the near future. When the user visits one of the prefetched documents, it can be served up quickly out of the browser's cache."
	
	^ (self headerAt: 'x-moz') = 'prefetch'
!

isSsl
	^ self uri isSsl
!

isXmlHttpRequest
	^ (self headerAt: 'x-requested-with') = 'XMLHttpRequest'
! !

