'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:41'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WARequestHandler
	instanceVariableNames:'filter parent configuration'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-RequestHandling'
!

WARequestHandler comment:'WARequestHandler is an abstract class whose subclasses handle http requests. Most of the methods are either empty or return a default value. 
Subclasses must implement the following messages:
	handleFiltered:	process the request
'
!

!WARequestHandler class methodsFor:'accessing'!

concreteSubclasses
	^ self allSubclasses reject: [ :each | each isAbstract ]
!

description
	^ self name greaseString
! !

!WARequestHandler class methodsFor:'testing'!

isAbstract
	^ self name = #WARequestHandler
! !

!WARequestHandler methodsFor:'accessing'!

name
	^ self isRoot
		ifTrue: [ '' ]
		ifFalse: [ self parent nameOfHandler: self ]
!

parent
	^ parent
!

url
	| url |
	
	url := self basicUrl.
	self serverPath isNil ifFalse: [ url parsePath: self serverPath ].
	self serverProtocol isNil ifFalse: [ url scheme: self serverProtocol ].
	self serverHostname isNil ifFalse: [ url host: self serverHostname ].
	self serverPort isNil ifFalse: [ url port: self serverPort ].

	^ url
!

urlFor: aRequestHandler
	"Return the URL for the given request handler, which should be a
	child of the receiver. The default implementation simply returns the
	receiver's URL, but subclasses should override to add a relevant
	paramter or path segment that identifies the particular chid."

	^ self url
! !

!WARequestHandler methodsFor:'authentication'!

verifyPassword: password forUser: username 
	"This method is called by WAAuthMain to verify the username provided
	in the HTTP request. The default implementation compares them against
	the configuration values provided by WAAuthConfiguration. WAAuthMain
	can also be used by itself if you implement this method in your
	own Application subclass to do something different."
	| loginPreference hashPreference |
	loginPreference := self preferenceAt: #login.
	hashPreference := self preferenceAt: #passwordHash.
	loginPreference isNil ifTrue: [
		self authConfigurationError: 'Login not defined. The default authentication method requires a WAAuthConfiguration' ].
	hashPreference isNil ifTrue: [
		self authConfigurationError: 'Password not defined. The default authentication method requires a WAAuthConfiguration' ].
	^ username isEmptyOrNil not
		and: [ password isEmptyOrNil not
		and: [ loginPreference = username
		and: [ hashPreference = (GRPlatform current secureHashFor: password) ] ] ]
! !

!WARequestHandler methodsFor:'configuration'!

charSet
	^ self preferenceAt: #charSet
!

configuration
	^ configuration
!

configuration: aConfiguration
	configuration := aConfiguration
!

defaultConfiguration
	^ WAUserConfiguration new
		addParent: WARequestHandlingConfiguration instance;
		yourself
!

documentClass
	^ self preferenceAt: #documentClass
!

preferenceAt: aSymbol
	^ self configuration at: aSymbol
!

preferenceAt: aSymbol put: anObject
	anObject isNil
		ifTrue: [ self configuration removeExpressionAt: aSymbol ]
		ifFalse: [ self configuration at: aSymbol put: anObject ].
	
	^ anObject
!

responseGenerator
	^ self preferenceAt: #responseGenerator
!

serverHostname
	^ self preferenceAt: #serverHostname
!

serverHostname: aString
	self preferenceAt: #serverHostname put: aString
!

serverPath
	^ self preferenceAt: #serverPath
!

serverPath: aString
	self preferenceAt: #serverPath put: aString
!

serverPort
	^ self preferenceAt: #serverPort
!

serverPort: anInteger
	self preferenceAt: #serverPort put: anInteger
!

serverProtocol
	^ self preferenceAt: #serverProtocol
!

serverProtocol: aSymbol
	self preferenceAt: #serverProtocol put: aSymbol
! !

!WARequestHandler methodsFor:'copying'!

copyTo: aRequestHandler
	^ aRequestHandler
!

postCopy
	super postCopy.
	filter := filter copy.
	self setFilter: (self filter copyTo: self).
	configuration := configuration copy.
	parent := nil
! !

!WARequestHandler methodsFor:'deprecated'!

basePath
	self greaseDeprecatedApi: 'WARequestHandler>>#basePath'
		details: 'Use WARequestHandler>>#url to get a WAUrl object and then #pathString if you want th path.'.
	^ self url pathString
!

baseUrl
	self greaseDeprecatedApi: 'WARequestHandler>>#baseUrl'
		details: 'Use WARequestHandler>>#url.'.
	
	^ self url
!

description
	self 
		greaseDeprecatedApi: 'WARequestHandler>>#description'
		details: 'Use WARequestHandler class>>#description.'.
	^ self class description
! !

!WARequestHandler methodsFor:'enumerating'!

parentChildPairsDo: aTwoArgumentBlock
	"Evaluate the given two argument block for all the direct parent-child handler pairs in the handler chain up the receiver."
	self parent isNil ifFalse: [
		self parent parentChildPairsDo: aTwoArgumentBlock.
		aTwoArgumentBlock value: self parent value: self ].
! !

!WARequestHandler methodsFor:'filters'!

addFilter: aFilter
	"Add aFilter to the receiver. Answer the filter added."

	^ self addFilterLast: aFilter
!

addFilterFirst: aFilter
	"Add aFilter as the first one in the filter chain. Answer the filter added."

	aFilter setNext: self filter.
	self setFilter: aFilter.
	^ aFilter
!

addFilterLast: aFilter
	"Add aFilter as the last one in the filter chain. Answer the filter added."

	| previous current |
	previous := nil.
	current := self filter.
	[ current = self ] whileFalse: [
		previous := current.
		current := current next ].
	aFilter setNext: current.
	previous isNil
		ifTrue: [ self setFilter: aFilter ]
		ifFalse: [ previous setNext: aFilter ].
	^ aFilter
!

filters
	"Answer an ordered colleciton of all filters active in the receiver."
	
	| current result |
	current := self filter.
	result := OrderedCollection new.
	[ current = self ] whileFalse: [
		result add: current.
		current := current next ].
	^ result
!

handler
	^ self
!

removeFilter: aFilter
	"Remove aFilter from the receiver. Answer the removed filter or nil."

	| previous current |
	previous := nil.
	current := self filter.
	[ current = self ] whileFalse: [
		current = aFilter ifTrue: [
			previous isNil
				ifTrue: [ self setFilter: current next ]
				ifFalse: [ previous setNext: current next ].
			^ aFilter setNext: nil ].
		previous := current.
		current := current next ].
	^ nil
! !

!WARequestHandler methodsFor:'filters-convenience'!

exceptionHandler: anExceptionHandlerClass
	| theFilter |
	theFilter := self filters
		detect: [ :ea | ea isExceptionFilter ]
		ifNone: [ self addFilter: WAExceptionFilter new ].
	
	theFilter configuration at: #exceptionHandler put: anExceptionHandlerClass
! !

!WARequestHandler methodsFor:'handling'!

handle: aRequestContext
	"Handle a request described by aRequestContext. Do not override this method; instead subclass #handleFiltered: to allow filters to process the request first."

	aRequestContext push: self during: [ self filter handleFiltered: aRequestContext ]
!

handleFiltered: aRequestContext 
	"Handle aRequestContext by either dispatching to a different request handler of by producing a response and singalling the result."

	self subclassResponsibility
! !

!WARequestHandler methodsFor:'initialization'!

initialize
	super initialize.
	configuration := self defaultConfiguration.
	filter := WAValueHolder with: self
! !

!WARequestHandler methodsFor:'parent/child'!

childrenDo: aBlock
	"Request handlers that act as containers should implement this method to
	iterate over all child handlers and pass each into aBlock."
!

setParent: aRequestHandler
	parent notNil ifTrue: [ parent unregister: self ].
	parent := aRequestHandler
!

unregister
	"Unregister this request handler from its parent handler."
	
	parent notNil ifTrue: [ parent unregister: self ]
!

unregister: aRequestHandler
	"Request handlers that act as containers should implement this method to
	remove aRequestHandler from their collections and send #unregistered to
	aRequestHandler. This method is also called by #setParent:."
!

unregistered
	"This method is called to notify a RequestHandler that it has been removed
	from its container. Subclasses should override to perfrom cleanup actions."
	parent := nil
!

withDescendantsDo: aBlock
	"Evaluate aBlock for this request handler and recursively for each of
	its child handlers."
	
	aBlock value: self.
	self childrenDo: [ :each | each withDescendantsDo: aBlock ]
! !

!WARequestHandler methodsFor:'private'!

authConfigurationError: aString
	WAAuthConfigurationError signal: aString
!

basicUrl
	^ self isRoot
		ifTrue: [ WAUrl new ]
		ifFalse: [ self parent urlFor: self ]
! !

!WARequestHandler methodsFor:'private-accessing'!

filter
	^ filter contents
!

setFilter: aFilter
	filter contents: aFilter
! !

!WARequestHandler methodsFor:'testing'!

isDispatcher
	^ false
!

isRegistry
	^ false
!

isRoot
	^ self parent isNil
! !

