'From Smalltalk/X, Version:6.2.1 on 05-09-2011 at 23:04:44'                     !

"{ Package: 'stx:goodies/seaside30/core' }"

WAObject subclass:#WAUrl
	instanceVariableNames:'scheme user password host port path slash queryFields fragment
		pathParameters'
	classVariableNames:''
	poolDictionaries:''
	category:'Seaside-Core-HTTP'
!

WAUrl comment:'I represent all portions of an URL as described by the RFC 1738 and updated by RFC 3986. I include scheme, userinfo, host, port, path, query, and fragment.
Instance Variables
	scheme:			<String> or nil
	user:		<String> or nil
	password:		<String> or nil
	host:		<String> or nil
	port:			<Integer> or nil
	path:			<OrderedCollection> or nil
	slash:			<Boolean>
	queryFields:		<WARequestFields> or nil
	fragment:		<String> or nil
	pathParameters:		<GROrderedMultiMap> or nil'
!

!WAUrl class methodsFor:'decoding'!

decodePercent: aString
	"percent decodes the given String"
	| input output char |
	input := aString readStream.
	output := WriteStream on: (String new: aString size).
	[ input atEnd ] whileFalse: [
		char := input next.
		output nextPut: (char = $+
			ifTrue: [ Character space ]
			ifFalse: [
				char = $%
					ifTrue: [
						Character codePoint: input next asUppercase digitValue * 16
							+ input next asUppercase digitValue ]
					ifFalse: [ char ] ]) ].
	^ output contents
! !

!WAUrl class methodsFor:'parsing'!

absolute: aString
	"Parse aString as an absolute URL."

	^ self basicNew initializeFromString: aString
!

absolute: aString relative: aRelativeString
	"Take absolute URL aString and combine it with a relative path aRelativeString."

	^ (self absolute: aString)
		parsePath: aRelativeString;
		yourself
! !

!WAUrl methodsFor:'accessing'!

fragment
	"Answer the fragment part of the URL."

	^ fragment
!

fragment: aString
	fragment := aString
!

host
	"Answer the host part of the URL."

	^ host
!

host: aString
	host := aString isNil
		ifFalse: [ aString greaseString asLowercase ]
!

password
	"Answer the password part of the URL userinfo. Note that the use of the format
	'user:password' for userinfo has been deprecated in RFC 3986 (3.2.1)."

	^ password
!

password: aString
	password := aString
!

path
	"Answer the path part of the URL."

	^ path ifNil: [ path := OrderedCollection new ]
!

path: aCollection
	"Set the path part of the URL to aCollection."

	path := aCollection
!

pathParameterAllAt: anIndex key: aKey ifAbsent: aZeroArgumentBlock
	"Answer all the paramters at path index anIndex with key aKey evaluation aZeroArgumentBlock if none is found."
	^ self
		privatePathParameterMapAt: anIndex
		ifPresentDo: [ :parameters |
			parameters allAt: aKey ifAbsent: aZeroArgumentBlock ]
		ifAbsent: aZeroArgumentBlock
!

pathParameterAt: anIndex key: aKey ifAbsent: aZeroArgumentBlock
	"Answer the first paramter at path index anIndex with key aKey evaluation aZeroArgumentBlock if none is found."
	^ self
		privatePathParameterMapAt: anIndex
		ifPresentDo: [ :parameters |
			parameters at: aKey ifAbsent: aZeroArgumentBlock ]
		ifAbsent: aZeroArgumentBlock
!

port
	"Answer the port number part of the URL."

	^ port
!

port: aNumber
	port := aNumber
!

queryFields
	"Answer a dictionary with the query fields."
	^ queryFields ifNil: [ queryFields := WARequestFields new ]
!

queryFields: aDictionary
	queryFields := aDictionary
!

scheme
	"Answer the URL's scheme."

	^ scheme ifNil: [ 'http' ]
!

scheme: aString
	scheme := aString isNil ifFalse: [ aString asLowercase ]
!

slash
	"Answer if the path has a trailing slash."

	^ slash
!

slash: aBoolean
	slash := aBoolean
!

user
	"Answer the user part of the URL userinfo. Note that the use of the format
	'user:password' for userinfo has been deprecated in RFC 3986 (3.2.1)."

	^ user
!

user: aString
	user := aString
!

userinfo
	"Answer the userinfo part of the URL. We store user and password seperately,
	even though the format 'user:password' is deprecated for userinfo in RFC 3986,
	because otherwise it is impossible to figure out correct encoding when userinfo
	data has a colon in it."

	^ password isNil
		ifTrue: [ user ]
		ifFalse: [ String streamContents: [ :stream |
			stream
				nextPutAll: user;
				nextPut: $:;
				nextPutAll: password ] ]
!

userinfo: aString
	"Colons in userinfo must be encoded. The only exception (now deprecated by RFC 3986)
	is a single colon separating a password from the rest of the user information. Since
	we can't tell the difference here, we assume that any colons should be encoded. If
	you want to specify a (deprecated) password, use #user: and #password:."
	
	user := aString.
	password := nil
! !

!WAUrl methodsFor:'adding'!

addAllToPath: aCollectionOfStrings
	"Add a collection of strings to the path."
	
	aCollectionOfStrings do: [ :each | self addToPath: each ]
!

addField: aKey
	"Add the query field aKey to the receiving url."

	self addField: aKey value: nil
!

addField: aKey value: aValue
	"Add the query field aKey with aValue to the receiving url."
	
	aKey isEmpty 
		ifFalse: [ self queryFields at: aKey add: aValue ]
!

addPathParameterAt: anIndex key: aKey
	"Add the path parameter aKey to the path element at anIndex."
	(self privatePathParameterMapAt: anIndex) at: aKey put: nil 
!

addPathParameterAt: anIndex key: aKey value: aValue
	"Add the path parameter aKey with aKey to the path element at anIndex."
	(self privatePathParameterMapAt: anIndex) at: aKey add: aValue 
!

addToPath: aString
	"Adds the argument as a path element. If the argument contains $/ it is split into multiple elements."

	| p |
	p := self path.
	self pathElementsIn: aString do: [ :each |
		p add: each ].
	self slash: (aString notEmpty and: [ aString last = $/ ])
! !

!WAUrl methodsFor:'comparing'!

= aUrl
	^ self class = aUrl class and: [ self printString = aUrl printString ]
!

hash
	^ self host hash bitXor: self path hash
! !

!WAUrl methodsFor:'converting'!

decodedWith: aCodec
	"answers a copy of the receiver decoded with the given codec
	http://www.ietf.org/rfc/rfc3987.txt"
	| url |
	url := self copy.
	
	"scheme, host, port cannot include escaped characters (%FF) and so should not
	be able to include characters that need decoding"
	
	"userinfo (username + password) can be percent encoded"
	self user isNil ifFalse: [
		url user: (aCodec url decode: self user) ].
	self password isNil ifFalse: [
		url password: (aCodec url decode: self password) ].
	
	self path isEmpty ifFalse: [
		url path: (self path collect: [ :each | aCodec url decode: each ]) ].
	url queryFields: nil.
	self queryFields keysAndValuesDo: [ :key :value |
		url
			addField: (aCodec url decode: key)
			value: (value isNil ifFalse: [ aCodec url decode: value ]) ].
	self fragment isNil ifFalse: [
		url fragment: (aCodec url decode: self fragment) ].
	
	^ url
!

pathString
	"Answer the path converted to a string."

	| document |
	^ String streamContents: [ :stream | 
		document := WAHtmlDocument on: stream.
		self encodePathOn: document ]
!

queryString
	"Answer the full query part of the URL. It would probably be better to
	store this as a string somewhere so we can support queries that are not
	key/value field lists."
	
	^ String streamContents: [ :stream |
		| urlEncoder |
		urlEncoder := WAUrlEncoder on: stream.
		self queryFields associations
			do: [ :each |
				urlEncoder nextPutAll: each key.
				each value notNil ifTrue: [
					stream nextPut: $=.
					urlEncoder nextPutAll: each value greaseString ] ]
			separatedBy: [ stream nextPut: $& ] ]
!

relativeTo: aUrl
	"Answer a path element collection relative from the receiver to aUrl."
	
	| currentPath argumentPath resultingPath |
	currentPath := self path copy.
	argumentPath := aUrl path copy.
	[ currentPath notEmpty and: [ argumentPath notEmpty and: [ currentPath first = argumentPath first ] ] ] whileTrue: [
		currentPath removeFirst.
		argumentPath removeFirst ].
	resultingPath := OrderedCollection new.
	currentPath size 
		timesRepeat: [ resultingPath add: '..' ].
	resultingPath addAll: argumentPath.
	^ resultingPath
!

seasideUrl
	^ self
! !

!WAUrl methodsFor:'copying'!

postCopy
	super postCopy.
	path := path copy.
	queryFields := queryFields copy.
	pathParameters := pathParameters copy
!

with: pathString
	^ self copy
		addToPath: pathString;
		yourself
!

withField: aString
	^ self copy
		addField: aString;
		yourself
!

withField: aString value: valueString
	^ self copy
		addField: aString value: valueString;
		yourself
!

withoutFragment
	^ self copy
		fragment: nil;
		yourself
!

withoutQuery
	^ self copy
		queryFields: nil;
		yourself
! !

!WAUrl methodsFor:'deprecated'!

addParameter: aKey
	self greaseDeprecatedApi: 'WAUrl>>#addParameter:'
		details: 'Use WAUrl>>#addField:.'.
	
	^ self addField: aKey
!

addParameter: aKey value: aValue
	self greaseDeprecatedApi: 'WAUrl>>#addParameter:value:'
		details: 'Use WAUrl>>#addField:value:.'.
	
	^ self addField: aKey value: aValue
!

hostname
	self greaseDeprecatedApi: 'WAUrl>>#hostname'
		details: 'Use WAUrl>>#host.'.

	^ self host
!

hostname: aString
	self greaseDeprecatedApi: 'WAUrl>>#hostname:'
		details: 'Use WAUrl>>#host:.'.
	
	^ self host: aString
!

parameters
	self greaseDeprecatedApi: 'WAUrl>>#parameters'
		details: 'Use WAUrl>>#queryFields.'.

	^ self queryFields
!

parameters: aDictionary
	self greaseDeprecatedApi: 'WAUrl>>#parameters:'
		details: 'Use WAUrl>>#queryFields:.'.

	^ self queryFields: aDictionary
!

parseHost: aString
	"This was probably only used internally anyway, but what the heck."
	
	self greaseDeprecatedApi: 'WAUrl>>#parseHost:'
		details: 'Use WAUrl>>#parseAuthority:.'.
		
	^ self parseAuthority: aString
!

parseParameters: aString
	"This was probably only used internally anyway, but what the heck."
	
	self greaseDeprecatedApi: 'WAUrl>>#parseParameters:'
		details: 'Use WAUrl>>#parseQuery:.'.
		
	^ self parseQuery: aString
!

username
	self greaseDeprecatedApi: 'WAUrl>>#username'
		details: 'Use WAUrl>>#user.'.

	^ self user
!

username: aString
	self greaseDeprecatedApi: 'WAUrl>>#username:'
		details: 'Use WAUrl>>#user:.'.
	
	^ self user: aString
!

withParameter: aKey
	self greaseDeprecatedApi: 'WAUrl>>#withParameter:'
		details: 'Use WAUrl>>#withField:.'.
	
	^ self withField: aKey
!

withParameter: aKey value: aValue
	self greaseDeprecatedApi: 'WAUrl>>#withParameter:value:'
		details: 'Use WAUrl>>#withField:value:.'.
	
	^ self withField: aKey value: aValue
!

withoutParameters
	self greaseDeprecatedApi: 'WAUrl>>#withoutParameters'
		details: 'Use WAUrl>>#withoutQuery.'.
	
	^ self withoutQuery
! !

!WAUrl methodsFor:'encoding'!

encodeFragmentOn: aDocument
	aDocument urlEncoder nextPutAll: fragment
!

encodeOn: aDocument
	self encodeSchemeAndAuthorityOn: aDocument.
	self encodePathOn: aDocument.
	(queryFields isNil or: [ queryFields isEmpty ]) ifFalse: [
		aDocument xmlEncoder nextPut: $?.
		self encodeQueryOn: aDocument ].
	fragment isNil ifFalse: [
		aDocument xmlEncoder nextPut: $#.
		self encodeFragmentOn: aDocument ]
!

encodePathOn: aDocument
	path isNil ifFalse: [
		path keysAndValuesDo: [ :index :each |
			aDocument nextPut: $/.
			aDocument urlEncoder nextPutAll: each.
			pathParameters isNil ifFalse: [
				self encodePathParametersAt: index on: aDocument ] ] ].
	(path isNil or: [ path isEmpty or: [ slash ] ])
		ifTrue: [ aDocument nextPut: $/ ]
!

encodePathParametersAt: anIndex on: aDocument
	self
		privatePathParameterMapAt: anIndex
		ifPresentDo: [ :parameters |
			parameters keys asSet do: [ :key |
				| values |
				aDocument nextPut: $;.
				aDocument urlEncoder nextPutAll: key.
				values := parameters allAt: key.
				(values size = 1 and: [ values first isNil ]) ifFalse: [
					| first |
					first := true.
					aDocument nextPut: $=.
					values do: [ :value |
						first ifFalse: [
							aDocument nextPut: $, ].
						aDocument urlEncoder nextPutAll: value greaseString.
						first := false ] ] ] ]
		ifAbsent: [ "ignore" ] 
!

encodeQueryOn: aDocument 
	| first |
	first := true.
	self queryFields keysAndValuesDo: [ :key :value |
		first ifFalse: [ aDocument xmlEncoder nextPut: $& ].
		first := false.
		aDocument urlEncoder nextPutAll: key.
		value isNil ifFalse: [
			aDocument nextPut: $=.
			aDocument urlEncoder nextPutAll: value greaseString ] ]
!

encodeSchemeAndAuthorityOn: aDocument
	host ifNil: [ ^ self ].
	aDocument nextPutAll: self scheme; nextPutAll: '://'.
	user notNil ifTrue: [
		aDocument urlEncoder nextPutAll: user.
		password notNil ifTrue: [
			aDocument nextPut: $:.
			aDocument urlEncoder nextPutAll: password ].
		aDocument nextPut: $@ ].
	aDocument urlEncoder nextPutAll: host.
	port notNil ifTrue: [
		((self scheme = 'http' and: [ port = 80 ])
			or: [ self scheme = 'https' and: [ port = 443 ] ])
			ifFalse: [ aDocument nextPut: $:; print: port ] ]
! !

!WAUrl methodsFor:'initialization'!

initialize
	super initialize.
	slash := false
!

initializeFromString: aString
	| string index |
	self initialize.
	string := aString.
	index := string indexOf: $#.
	index > 0 ifTrue: [
		self parseFragment: (string copyFrom: index + 1 to: string size).
		string := string copyFrom: 1 to: index - 1 ].
	index := string indexOf: $?.
	index > 0 ifTrue: [
		self parseQuery: (string copyFrom: index + 1 to: string size).
		string := string copyFrom: 1 to: index - 1 ].
	index := string indexOfSubCollection: '://'.
	index > 0 ifTrue: [ 
		self scheme: (string copyFrom: 1 to: index - 1).
		string := string copyFrom: index + 3 to: string size.
		index := string indexOf: $/.
		string := index > 0
			ifFalse: [ 
				self parseAuthority: string.
				'' ]
			ifTrue: [
				self parseAuthority: (string copyFrom: 1 to: index - 1).
				string copyFrom: index + 1 to: string size ] ].
	self parsePath: string
! !

!WAUrl methodsFor:'parsing'!

parseAuthority: aString 
	| string temp index |
	aString isEmpty ifTrue: [ ^ self ].
	string := aString.
	index := string indexOf: $@.
	index > 1 ifTrue: 
		[ temp := string 
			copyFrom: 1
			to: index - 1.
		string := string 
			copyFrom: index + 1
			to: string size.
		index := temp indexOf: $:.
		index > 0 
			ifFalse: [ self user: (self decode: temp) ]
			ifTrue: 
				[ self user: (self decode: (temp 
							copyFrom: 1
							to: index - 1)).
				self password: (self decode: (temp 
							copyFrom: index + 1
							to: temp size)) ] ].
	index := string lastIndexOf: $:.
	index > 0 ifTrue: 
		[ temp := string 
			copyFrom: index + 1
			to: string size.
		string := string 
			copyFrom: 1
			to: index - 1.
		self port: ((temp allSatisfy: [ :each | each isDigit ]) ifTrue: [ temp greaseInteger ]) ].
	self host: (self decode: string)
!

parseFragment: aString
	aString isEmpty ifTrue: [ ^ self ].
	self fragment: (self decode: aString)
!

parsePath: aString
	| p index |
	(aString notEmpty and: [ aString first = $/ ]) ifTrue: [
		path := nil.
		pathParameters := nil ].
	p := self path.
	index := p size.
	self pathElementsIn: aString do: [ :each |
		index := index + 1.
		each = '..'
			ifTrue: [
				p isEmpty ifFalse: [
					p removeLast ] ]
			ifFalse: [
				each = '.' ifFalse: [
					| semiColonIndex |
					semiColonIndex := each indexOf: $;.
					semiColonIndex = 0
						ifTrue: [
							"we don't have path parameters (default case), add the whole element"
							p addLast: (self decode: each) ]
						ifFalse: [
							"we do have path parameters, only add up to $; and the parse the parameters"
							p addLast: (self decode: (each copyFrom: 1 to: semiColonIndex - 1)).
							self parsePathParametersAt: index in: (each copyFrom: semiColonIndex to: each size) ] ] ] ].
	slash := aString notEmpty and: [ aString last = $/ ]
!

parsePathParametersAt: anIndex in: aString
	self subStringsIn: aString splitBy: $; do: [ :parameter |
		| equalIndex |
		equalIndex := parameter indexOf: $=.
		equalIndex = 0
			ifTrue: [ self addPathParameterAt: anIndex key: parameter ]
			ifFalse: [
				| key |
				key := parameter copyFrom: 1 to: equalIndex - 1.
				self
					subStringsIn: (parameter copyFrom: equalIndex + 1 to: parameter size)
					splitBy: $,
					do: [ :each |
						self addPathParameterAt: anIndex key: key value: each ] ] ]
!

parsePathParametersAt: anIndex in: aString startingAt: startIndex
	| p |
	(aString notEmpty and: [ aString first = $/ ])
		ifTrue: [ path := nil ].
	p := self path.
	self pathElementsIn: aString do: [ :each |
		each = '..'
			ifTrue: [
				p isEmpty ifFalse: [
					p removeLast ] ]
			ifFalse: [
				each = '.' ifFalse: [
					| semiColonIndex |
					semiColonIndex := each indexOf: $;.
					semiColonIndex = 0
						ifTrue: [ p addLast: (self decode: each) ]
						ifFalse: [
							p addLast: (self decode: (each copyFrom: 1 to: semiColonIndex)) ] ] ] ].
	slash := aString notEmpty and: [ aString last = $/ ]
!

parseQuery: aString
	| input string index |
	input := aString readStream.
	[ input atEnd ] whileFalse: [
		string := input upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ self addField: (self decode: string) ]
			ifTrue: [
				self
					addField: (self decode: (string copyFrom: 1 to: index - 1))
					value: (self decode: (string copyFrom: index + 1 to: string size)) ] ]
! !

!WAUrl methodsFor:'printing'!

printOn: aStream 
	| document codec |
	codec := [ self requestContext codec ] 
		on: WARequestContextNotFound
		do: [ :e | GRNullCodec new ].
	document := WAUnescapedDocument 
		on: aStream
		codec: codec.
	self encodeOn: document
! !

!WAUrl methodsFor:'private'!

decode: aString
	"percent decodes the given String"

	^ self class decodePercent: aString
!

isSeasideField: aString
	^ aString notEmpty
		and: [ aString first = $_
		or: [ aString allSatisfy: [ :each | each isDigit ] ] ]
!

pathElementsIn: aString do: aOneArgumentBlock
	self
		subStringsIn: aString
		splitBy: $/
		do: aOneArgumentBlock
!

privatePathParameterMapAt: anIndex
	pathParameters isNil ifTrue: [
		pathParameters := OrderedCollection new: self path size ].
	^ pathParameters at: anIndex ifAbsentPut: [ GROrderedMultiMap new ]
!

privatePathParameterMapAt: anIndex ifPresentDo: aOneArgumentBloack ifAbsent: aZeroArgumentBlock
	| parameters |
	pathParameters isNil ifTrue: [
		^ aZeroArgumentBlock value ].
	parameters := pathParameters at: anIndex ifAbsent: [
		^ aZeroArgumentBlock value ].
	parameters isNil ifTrue: [
		^ aZeroArgumentBlock value ].
	^ aOneArgumentBloack value: parameters
!

subStringsIn: aString splitBy: aCharacter do: aOneArgumentBlock
	"#subStrings: can be slow on Pharo because it uses a lot of allocation so we avoid it
	see issue Issue 609"
	| startIndex |
	startIndex := aString indexOf: aCharacter.
	startIndex = 0
		ifTrue: [
			aString isEmpty ifFalse: [
				aOneArgumentBlock value: aString ] ]
		ifFalse: [
			| endIndex stringSize |
			stringSize := aString size.
			startIndex = 1 ifFalse: [
				"case where we have a path not starting with aCharacter"
				aOneArgumentBlock value: (aString copyFrom: 1 to: startIndex - 1) ].
			endIndex := aString indexOf: aCharacter startingAt: startIndex + 1.
			[ startIndex ~= 0 ] whileTrue: [
				startIndex + 1 = endIndex
					ifTrue: [
						"skip, avoid adding empty strings"
						startIndex := endIndex.
						endIndex := aString indexOf: aCharacter startingAt: startIndex + 1]
					ifFalse: [
						endIndex = 0
							ifTrue: [
								startIndex < stringSize ifTrue: [
									"avoid adding empty path if aString ends with aCharacter"
									aOneArgumentBlock value: (aString copyFrom: startIndex + 1 to: stringSize) ].
								startIndex := 0 ]
							ifFalse: [
								aOneArgumentBlock value: (aString copyFrom: startIndex + 1 to: endIndex  - 1).
								startIndex := endIndex.
								endIndex := aString indexOf: aCharacter startingAt: startIndex + 1 ] ] ] ]
!

takeServerParametersFromRequest: aRequest
	"Retrieves scheme, hostname and basepath from the request unless already set."

	scheme isNil
		ifTrue: [ self scheme: aRequest url scheme ].
	host isNil 
		ifTrue: [ aRequest headers at: 'host' ifPresent: [ :value | self parseAuthority: value ] ].
	path isNil
		ifTrue: [ aRequest headers at: 'base' ifPresent: [ :value | self parsePath: value ] ]
! !

!WAUrl methodsFor:'testing'!

isSsl
	^ self scheme isEmptyOrNil not
		and: [ self scheme last = $s ]
! !

