"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRPlatform
	instanceVariableNames:''
	classVariableNames:'Current'
	poolDictionaries:''
	category:'Grease-Core'
!

GRPlatform comment:'The abstract platform implementation. Each platform should provide a subclass implementing any abstract methods and overriding any other methods as necessary.
Default implementations should be provided here when possible/useful but default implementations MUST be valid on ALL PLATFORMS so it is rarely practical. VA Smalltalk flags sends of uknown messages so even these must be known to exist on all platforms.
Common cases where default implementations *are* appropriate are where there is a standard implementation that is valid on all platforms but one or more platforms have an additional, optimized implementation that should be used instead.
All classes and methods used by methods of this class should be either:
  + included in the Seaside-Platform package;
  + defined by the ANSI Smalltalk standard; or
  + (not ideal) referenced via ''Smalltalk at: #ClassName''.'
!


!GRPlatform class methodsFor:'registration'!

current

	^ Current
!

current: aPlatform
	Current := aPlatform
!

select
	GRPlatform current: self new
!

unselect
	GRPlatform current class = self ifTrue: [ GRPlatform current: nil ]
! !

!GRPlatform methodsFor:'cryptography'!

secureHashFor: aString
	self subclassResponsibility
! !

!GRPlatform methodsFor:'encoding'!

base64Decode: aString
	"Base64 decode the given String and answer the result as a String."
	self subclassResponsibility
! !

!GRPlatform methodsFor:'exceptions'!

openDebuggerOn: anError
	self subclassResponsibility
!

stackDepth
	self subclassResponsibility
! !

!GRPlatform methodsFor:'factory'!

newRandom
	"Answers the random number generator to be used to create session and continuation keys. Make sure it is seeded. They only methods that will be sent to it are:
	  #nextInt: - should answer a random integer in the interval [1, anInteger]
	  #randomFrom: - should answer a random element from the given collection
	
	Make sure that both methods are safe under heavy concurrent load.
	
	Used by Gemstone/S traditional Randoms which cannot be persisted.
	Used by Squeak to use a secure random when avaiable."
	self subclassResponsibility
!

readWriteByteStream
	"Return a ReadWriteStream on a ByteArray that stores integers 0..255
		^ReadWriteStream on: ByteArray new
	"

	^ self subclassResponsibility
!

readWriteCharacterStream
	"Return a ReadWriteStream on a String that stores characters
		^ReadWriteStream on: String new
	"

	^ self subclassResponsibility
!

reducedConflictDictionary
	"used by Gemstone/S reduced conflict classes that can be used to avoid transaction conflicts"
	^ Dictionary
!

semaphoreClass
	"used by Gemstone/S traditional Semaphores which cannot be persisted"
	self subclassResponsibility
!

weakDictionaryOfSize: aNumber
	self subclassResponsibility
! !

!GRPlatform methodsFor:'file library'!

asMethodReturningByteArray: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a ByteArray"
	self subclassResponsibility
!

compile: aString into: aClass classified: aSymbol
	"The trick here is to be as silently a possible so that the package is not marked dirty when running WAFileLibrary test.
	This also makes running tests much faster."
	self subclassResponsibility
!

contentsOfFile: aString binary: aBoolean
	self subclassResponsibility
!

convertToSmalltalkNewlines: aString
	"Convert any line endings (CR, CRLF, LF) to the default platform newline."
	
	aString isNil 
		ifTrue: [ ^ nil ].
	^ String streamContents: [ :writeStream |
		| readStream |
		readStream := aString readStream.
		[ readStream atEnd ] whileFalse: [
			| next |
			next := readStream next.
			next = Character cr
				ifTrue: [
					readStream peek = Character lf
						ifTrue: [ readStream skip: 1 ].
					writeStream nextPutAll: self newline ]
				ifFalse: [
					next = Character lf
						ifTrue: [ writeStream nextPutAll: self newline ]
						ifFalse: [ writeStream nextPut: next ] ] ] ]
!

ensureExistenceOfFolder: aString
	"Create a folder named aString in the image directory."
	self subclassResponsibility
!

filesIn: aPathString
	"Answer a collection of absolute paths for all the files (no directories) in the directory given by aPathString
	must not include file names that start with ."
	self subclassResponsibility
!

localNameOf: aFilename
	"Answer the local name of a file indentified by an absolute file path.
	
	Eg.
	If the platform is Windwos and aFilename is 'C:\Windows\win32.dll' then it would answer 'win32.dll'.
	If the platform is Unix and aFilename is '/usr/bin/vim' then it would answer 'vim'."
	self subclassResponsibility
!

newline
	"Answer a String with the default newline character of this platform."
	
	self subclassResponsibility
!

pathSeparator
	"Answer the path separator as a String, eg. '/' on Unix and '\' on Windows."
	self subclassResponsibility
!

removeSelector: aSymbol from: aClass
	self subclassResponsibility
!

write: aStringOrByteArray toFile: aFileNameString inFolder: aFolderString
	"Write aStringOrByteArray to a file named aFilenameString in the folder aFolderString."
	self subclassResponsibility
! !

!GRPlatform methodsFor:'processes'!

isProcessTerminated: aProcess
	"Return a boolean indicating whether aProcess has been terminated."
	
	self subclassResponsibility
!

terminateProcess: aProcess
	"Permanently terminate the process, unwinding first to execute #ensure: and #ifCurtailed: blocks."
	
	self subclassResponsibility
! !

!GRPlatform methodsFor:'startup'!

addToShutDownList: anObject
	"Add anObject to the shutdown-list of the system. On shutdown the message #shutDown will be sent to anObject."
	self subclassResponsibility
!

addToStartUpList: anObject
	"Add anObject to the startup-list of the system. On startup the message #startUp will be sent to anObject."
	self subclassResponsibility
!

removeFromShutDownList: anObject
	"Remove anObject from the shutdown list in the system."
	self subclassResponsibility
!

removeFromStartUpList: anObject
	"Remove anObject from the startup list in the system."
	self subclassResponsibility
! !

!GRPlatform methodsFor:'testing'!

includesUnsafeUrlCharacter: aString
	"Answer whether the given string contains a character that is not URL safe.
	May answer true even if only safe charaters are in the string.
	
	Platforms may override this for performance."
	^ true
!

includesUnsafeXmlCharacter: aString
	"Answer whether the given string contains a character that needs to be escapted for XML.
	May answer true even if no character that needs to be escaped in the the XML.
	
	Platforms may override this for performance."
	^ true
! !

!GRPlatform methodsFor:'transactions'!

doTransaction: aBlock
	"for Gemstone/S compatibility
	http://gemstonesoup.wordpress.com/2007/05/10/porting-application-specific-seaside-threads-to-gemstone/
	use when modifying an object from an outside thread"

	^ aBlock value
! !

!GRPlatform methodsFor:'version info'!

label
	"Answer a descriptive label string for the platform implementation"
	
	self subclassResponsibility
!

version
	"Answer the Grease version"

	^ (GRVersion major: 1 minor: 0 revision: 6)
		yourself
!

versionString
	^ String streamContents: [ :stream |
		stream
			nextPutAll: (self version greaseString);
			nextPutAll: ' (';
			nextPutAll: (self label);
			nextPut: $) ]
! !

!GRPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPlatform.st,v 1.1 2011/09/05 09:07:11 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRPlatform.st,v 1.1 2011/09/05 09:07:11 cg Exp $'
! !
