"{ Package: 'stx:goodies/seaside30/grease' }"

GRPlatform subclass:#GRSTXPlatform
	instanceVariableNames:''
	classVariableNames:'UrlTable XmlTable'
	poolDictionaries:''
	category:'Grease-Pharo-Core'
!


!GRSTXPlatform class methodsFor:'class initialization'!

initialize
	self initializeXmlTable.
	self initializeUrlTable.
	self select
!

initializeUrlTable
	UrlTable := ByteArray new: 256.
	1 to: 256 do: [ :index |
		('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_.~' includes: (Character codePoint: index - 1))
			ifTrue: [ UrlTable at: index put: 0 ]
			ifFalse: [ UrlTable at: index put: 1 ] ]
!

initializeXmlTable
	XmlTable := ByteArray new: 256.
	1 to: 256 do: [ :index |
		('"<&>' includes: (Character codePoint: index - 1))
			ifTrue: [ XmlTable at: index put: 1 ]
			ifFalse: [ XmlTable at: index put: 0 ] ]
!

unload
	self unselect
! !

!GRSTXPlatform methodsFor:'cryptography'!

secureHashFor: aString
	^ SecureHashAlgorithm new hashMessage: aString
! !

!GRSTXPlatform methodsFor:'encoding'!

base64Decode: aString
	^ (Base64MimeConverter mimeDecodeToChars: aString readStream) contents
! !

!GRSTXPlatform methodsFor:'exceptions'!

openDebuggerOn: anError
	| process |
	process := Processor activeProcess.
	"If we are running in the UI process, we don't want to suspend the active process. The
	error was presumably triggered while stepping in the Debugger. If we simply immediately
	signal an UnhandledError, the debugger will catch this and display the signaling context.
	It isn't perfect or pretty but it works."
	(ProcessBrowser isUIProcess: process)
		ifTrue: [
			UnhandledError signalForException: anError ]
		ifFalse: [
			WorldState addDeferredUIMessage:
				[process
					debug: anError signalerContext
					title: anError description
					full: true].
			process suspend ]
!

stackDepth

	| depth current |
	depth := 0.
	current := thisContext.
	[ current isNil ] whileFalse: [
		current := current sender.
		depth := depth + 1 ].
	^ depth - 1
! !

!GRSTXPlatform methodsFor:'factory'!

newRandom
	"Answers the random number generator to be used to create session and continuation keys. Make sure it is seeded. They only methods that will be sent to it are:
	  #nextInt: - should answer a random integer in the interval [1, anInteger]
	  #randomFrom: - should answer a random element from the given collection
	
	Make sure that both methods are safe under heavy concurrent load.
	
	Used by Gemstone/S traditional Randoms which cannot be persisted.."
	^ GRPharoRandomProvider
!

readWriteByteStream
	"ByteArray based read write stream"
	
	^ ReadWriteStream on: (ByteArray new: 4096)
!

readWriteCharacterStream
	"String based read write stream"

	^ ReadWriteStream on: (String new: 4096)
!

semaphoreClass
	"used by Gemstone/S traditional Semaphores which cannot be persisted"
	^ Semaphore
!

weakDictionaryOfSize: aNumber
	^ IdentityDictionary new: aNumber
! !

!GRSTXPlatform methodsFor:'file library'!

asMethodReturningByteArray: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a ByteArray"
	^ self useByteArrayLiterals
		ifTrue: [ self asMethodReturningByteArrayLiteral: aByteArrayOrString named: aSymbol ]
		ifFalse: [ self asMethodReturningByteArrayWithCache: aByteArrayOrString named: aSymbol ]
!

compile: aString into: aClass classified: aSymbol 
	"The trick here is to be as silently a possible so that the package is not marked dirty when running WAFileLibrary test.
	This also makes running tests much faster."
	| methodNode compiledMethod selector methodAndNode |
	methodNode := aClass compilerClass new 
		compile: aString
		in: aClass
		notifying: nil
		ifFail: [ GRError signal: 'syntax error' ].
	selector := methodNode selector.
	methodAndNode := CompiledMethodWithNode 
		generateMethodFromNode: methodNode
		trailer: aClass defaultMethodTrailer.
	compiledMethod := methodAndNode method.
	compiledMethod 
		putSource: aString
		fromParseNode: methodAndNode node
		inFile: 2
		withPreamble: [ :file | 
			aClass
				printCategoryChunk: aSymbol asString
				on: file priorMethod: (aClass compiledMethodAt: selector ifAbsent: [ nil ]).
			file cr ].
	aClass 
		addSelectorSilently: selector
		withMethod: compiledMethod.
	SystemChangeNotifier uniqueInstance doSilently:  [
		aClass organization 
			classify: selector
			under: aSymbol ]
!

contentsOfFile: aString binary: aBoolean
	| stream |
	stream := aBoolean
		ifTrue: [ (FileDirectory default oldFileNamed: aString)
			binary;
			yourself ]
		ifFalse: [ (MultiByteFileStream oldFileNamed: aString)
			ascii;
			wantsLineEndConversion: true;
			yourself ].
	^ [ stream contents ] ensure: [ stream close ]
!

ensureExistenceOfFolder: aString
	"creates a folder named aString in the image directory"
	FileDirectory default assureExistenceOfPath: aString
!

filesIn: aPathString
	"Return a collection of absolute paths for all the files (no directories) in the directory given by aPathString
	must not include file names that start with ."
	| directory |
	directory := FileDirectory default directoryNamed: aPathString.
	^ (directory fileNames
		reject: [ :each | each first = $. ])
		collect: [ :each | directory fullNameFor: each ]
!

localNameOf: aFilename
	^ (FileDirectory on: aFilename) localName
!

newline
	^ String cr
!

pathSeparator
	^ FileDirectory pathNameDelimiter asString
!

removeSelector: aSymbol from: aClass
	aClass removeSelectorSilently: aSymbol
!

write: aStringOrByteArray toFile: aFileNameString inFolder: aFolderString
	"writes aStringOrByteArray to a file named aFilenameString in the folder aFolderString"
	| folder stream fullFilePath |
	folder := FileDirectory default directoryNamed: aFolderString.
	fullFilePath := folder fullNameFor: aFileNameString.
	stream := aStringOrByteArray isString
		ifTrue: [
			(CrLfFileStream forceNewFileNamed: fullFilePath)
				ascii;
				wantsLineEndConversion: true;
				yourself ]
		ifFalse: [ (FileStream forceNewFileNamed: fullFilePath) binary ].
	[ stream nextPutAll: aStringOrByteArray ]
		ensure: [ stream close ]
! !

!GRSTXPlatform methodsFor:'private-file library'!

asMethodReturningByteArrayLiteral: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a byte array using VW/NewCompiler byte array literal syntax."
	^ String streamContents: [ :stream |
		stream nextPutAll: aSymbol; nextPut: Character cr.
		stream tab; nextPutAll: '^ #['.
		aByteArrayOrString asByteArray
			do: [ :each | each printOn: stream ]
			separatedBy: [ stream space ].
		stream nextPutAll: ']' ]
!

asMethodReturningByteArrayWithCache: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a byte array and caching this array in a literal array of size 1."
	^ String streamContents: [ :stream |
		stream nextPutAll: aSymbol; nextPut: Character cr.
		stream tab; nextPutAll: '^ #('.
		aByteArrayOrString asByteArray
			do: [ :each | each printOn: stream ]
			separatedBy: [ stream space ].
		stream nextPutAll: ') asByteArray' ]
!

useByteArrayLiterals
	"whether ByteArray literals can/should be used"
	| hasSettings |
	hasSettings := Smalltalk at: #PragmaSetting ifAbsent: [ nil ].
	^ hasSettings isNil
		ifTrue: [
			(Smalltalk at: #Preferences)
				valueOfFlag: #compileUseNewCompiler
				ifAbsent: [ false ] ]
		ifFalse: [ false ]
! !

!GRSTXPlatform methodsFor:'processes'!

isProcessTerminated: aProcess
	"Return a boolean indicating whether aProcess has been terminated."
	^ aProcess isTerminated
!

terminateProcess: aProcess
	"Permanently terminate the process, unwinding first to execute #ensure: and #ifCurtailed: blocks."
	aProcess terminate
! !

!GRSTXPlatform methodsFor:'startup'!

addToShutDownList: anObject
	"Add anObject to the shutdown-list of the system. On shutdown the message #shutDown will be sent to anObject."

	Smalltalk addToShutDownList: anObject
!

addToStartUpList: anObject
	"Add anObject to the startup-list of the system. On startup the message #startUp will be sent to anObject."

	Smalltalk addToStartUpList: anObject
!

removeFromShutDownList: anObject
	"Remove anObject from the shutdown list in the system."

	Smalltalk removeFromShutDownList: anObject
!

removeFromStartUpList: anObject
	"Remove anObject from the startup list in the system."

	Smalltalk removeFromStartUpList: anObject
! !

!GRSTXPlatform methodsFor:'testing'!

includesUnsafeUrlCharacter: aString
	^ aString isWideString "use slow fallback for WideStrings"
		or: [ (ByteString findFirstInString: aString inSet: UrlTable startingAt: 1) ~= 0 ]
!

includesUnsafeXmlCharacter: aString
	^ aString isWideString "use slow fallback for WideStrings"
		or: [ (ByteString findFirstInString: aString inSet: XmlTable startingAt: 1) ~= 0 ]
!

indexOfFirstUnsafeUrlCharacterIn: aString startingAt: startIndex
	^ aString isWideString "use slow fallback for WideStrings"
		ifTrue: [ startIndex ]
		ifFalse: [ ByteString findFirstInString: aString inSet: UrlTable startingAt: startIndex ]
! !

!GRSTXPlatform methodsFor:'version info'!

label
	^ 'Pharo'
! !

!GRSTXPlatform class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRSTXPlatform.st,v 1.1 2011/09/05 09:13:26 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRSTXPlatform.st,v 1.1 2011/09/05 09:13:26 cg Exp $'
! !

GRSTXPlatform initialize!
