"{ Package: 'stx:goodies/seaside30/grease' }"

GRObject subclass:#GRSmallDictionary
	instanceVariableNames:'size keys values'
	classVariableNames:''
	poolDictionaries:''
	category:'Grease-Core-Collections'
!

GRSmallDictionary comment:'I am an implementation of a dictionary. Compared to other dictionaries I am very efficient for small sizes, speed- and space-wise. I also mantain the order in which elements are added when iterating. My implementation features some ideas from the RefactoringBrowser.'
!


!GRSmallDictionary class methodsFor:'instance creation'!

new
	^ self new: 3
!

new: anInteger
	^ self basicNew initialize: anInteger; yourself
!

withAll: aDictionary
	^ (self new: aDictionary size)
		addAll: aDictionary;
		yourself
! !

!GRSmallDictionary methodsFor:'accessing'!

add: anAssociation
	self at: anAssociation key put: anAssociation value.
	^ anAssociation
!

addAll: aDictionary
	aDictionary keysAndValuesDo: [ :key :value | self add: key -> value ].
	^ aDictionary
!

associations
	"Answer a Collection containing the receiver's associations."

	| result |
	result := WriteStream on: (Array new: self size).
	self associationsDo: [ :assoc | result nextPut: assoc ].
	^ result contents
!

at: aKey
	"Answer the value associated with aKey. Raise an exception, if no such key is defined."

	^ self at: aKey ifAbsent: [ self errorKeyNotFound ]
!

at: aKey ifAbsent: aBlock
	"Answer the value associated with aKey. Evaluate aBlock, if no such key is defined."

	| index |
	index := self findIndexFor: aKey.
	^ index = 0
		ifFalse: [ values at: index ]
		ifTrue: [ aBlock value ]
!

at: aKey ifAbsentPut: aBlock
	"Answer the value associated with aKey. Evaluate aBlock, if no such key is defined and store the return value."

	| index |
	index := self findIndexFor: aKey.
	^ index = 0
		ifFalse: [ values at: index ]
		ifTrue: [ self privateAt: aKey put: aBlock value ]
!

at: aKey ifPresent: aBlock
	"Lookup aKey in the receiver. If it is present, answer the value of evaluating the given block with the value associated with the key. Otherwise, answer nil."

	| index |
	index := self findIndexFor: aKey.
	^ index = 0 ifFalse: [ aBlock value: (values at: index) ]
!

at: aKey put: aValue
	"Set the value of aKey to be aValue."

	| index |
	index := self findIndexFor: aKey.
	^ index = 0
		ifFalse: [ values at: index put: aValue ]
		ifTrue: [ self privateAt: aKey put: aValue ]
!

removeKey: aKey
	"Remove aKey from the receiver, raise an exception if the element is missing."

	^ self removeKey: aKey ifAbsent: [ self errorKeyNotFound ]
!

removeKey: aKey ifAbsent: aBlock
	"Remove aKey from the receiver, evaluate aBlock if the element is missing."

	| index |
	index := self findIndexFor: aKey.
	index = 0 ifTrue: [ ^ aBlock value ].
	^ self removeIndex: index
!

size
	^ size
! !

!GRSmallDictionary methodsFor:'copying'!

postCopy
	super postCopy.
	keys := keys copy.
	values := values copy
! !

!GRSmallDictionary methodsFor:'enumerating'!

associationsDo: aBlock
	self keysAndValuesDo: [ :key :value | aBlock value: key -> value ]
!

do: aBlock
	1 to: size do: [ :index | aBlock value: (values at: index) ]
!

keys
	^ keys copyFrom: 1 to: size
!

keysAndValuesDo: aBlock
	1 to: size do: [ :index | aBlock value: (keys at: index) value: (values at: index) ]
!

keysDo: aBlock
	1 to: size do: [ :each | aBlock value: (keys at: each) ]
!

values
	^ values copyFrom: 1 to: size
! !

!GRSmallDictionary methodsFor:'initialization'!

initialize: anInteger
	self initialize.
	size := 0.
	keys := Array new: anInteger.
	values := Array new: anInteger
! !

!GRSmallDictionary methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	
	aStream nextPut: $(.
	self size <= 100
		ifTrue: [
			| first |
			first := true.
			self keysAndValuesDo: [ :key :value |
				"keysAndValuesDo:separatedBy: would be nice"
				first
					ifTrue: [ first := false ]
					ifFalse: [ aStream space ].
				aStream
					print: key;
					nextPutAll: '->';				
					print: value] ]
		ifFalse: [
			aStream
				nextPutAll: 'size ';
				print: self size ].
	aStream nextPut: $)	
! !

!GRSmallDictionary methodsFor:'private'!

errorKeyNotFound
	self error: 'Key not found'
!

findIndexFor: aKey
	1 to: size do: [ :index |
		(keys at: index) = aKey
			ifTrue: [ ^ index ] ].
	^ 0
!

grow
	| newKeys newValues |
	newKeys := Array new: 2 * size.
	newValues := Array new: 2 * size.
	1 to: size do: [ :index |
		newKeys at: index put: (keys at: index).
		newValues at: index put: (values at: index) ].
	keys := newKeys.
	values := newValues
!

privateAt: aKey put: aValue
	size = keys size ifTrue: [ self grow ].
	keys at: (size := size + 1) put: aKey.
	^ values at: size put: aValue
!

removeIndex: index
	| value |
	value := values at: index.
	index to: size - 1 do:
			[ :i | 
			keys at: i put: (keys at: i + 1).
			values at: i put: (values at: i + 1) ].
	keys at: size put: nil.
	values at: size put: nil.
	size := size - 1.
	^ value
! !

!GRSmallDictionary methodsFor:'testing'!

includesKey: aKey
	"Answer whether the receiver has a key equal to aKey."

	^ (self findIndexFor: aKey) ~= 0
!

isCollection
	^ true
!

isEmpty
	^ size = 0
! !

!GRSmallDictionary class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRSmallDictionary.st,v 1.2 2011/09/05 17:58:13 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/seaside30/grease/GRSmallDictionary.st,v 1.2 2011/09/05 17:58:13 cg Exp $'
! !
