"{ Package: 'stx:goodies/soap/splash' }"

"{ NameSpace: SOAP }"

SplashRequest subclass:#SplashRPCRequest
	instanceVariableNames:'rpcNode writeMap flags'
	classVariableNames:'IsIDHref IsSoapEnc'
	poolDictionaries:''
	category:'SOAP-Splash'
!

SplashRPCRequest comment:'I am a rpc request soap envelope.

I am constructed as a RPC envelope (my soap:Body has one child element, which is an RPC wrapper element).

See showHelp for examples.

Instance Variables:
	rpcNode	<XeNode>	
	writeMap	<XeNodeWriteMap> Used to preserve object identity
	flags	<Integer>'
!


!SplashRPCRequest class methodsFor:'instance creation'!

method: aMethodNameString ns: aMethodNSSymbol 
	"Answer an instance which is initialized as an RPC/Encoded soap request envelope.
	-The method name and namespace are used to construct the soap 'rpc wrapper element' ie:
		<soap:Envelope ... >
		  <soap:Body>
		    <m:aMethodNameString xmlns:m='aMethodNSSymbol'/>
		  </soap:Body>
		</soap:Envelope>
	-Parameters to the request are added as children of the rpc wrapper element (ie: <m:aMethodNameString>)"

	^self new 
		addRPCWrapperNamed: (XeQName prefix: 'm' local: aMethodNameString ns: aMethodNSSymbol asSymbol);
		yourself
! !

!SplashRPCRequest class methodsFor:'initializing'!

initialize
	"
	self initialize
	"

	IsIDHref := 1.
	IsSoapEnc := 2
! !

!SplashRPCRequest class methodsFor:'must strip'!

exampleQueue
        <resource: #example>

        |request exchanges queue|

        request := SplashRPCRequest method: 'echoBoolean' ns: #'http://soapinterop.org/'.
        request parameterAt: #inputBoolean put: true.

        exchanges := (1 to: 20) collect: [ :i | request exchangeTo: 'http://www.dolphinharbor.org/services/interop2001' soapAction: nil].
        queue := SptExchangeQueue new.
        exchanges do: [ :each | queue queueLast: each].

        exchanges do: [ :each | each assertSuccessState].
        exchanges do: [ :each | self assert: [(SplashResponse exchange: each) return]]

        "
         Transcript showCR:
            self exampleQueue
        "
! !

!SplashRPCRequest class methodsFor:'spray examples'!

exampleArray
        "Arrays can require more type information than other objects, and have a separate helper method. 
         A WebService will typically require an Array of a type of item."

        | request |
        request := SplashRPCRequest method: 'ArrayTest' ns: #'http://ArrayTest.org'.
        request 
                parameterAt: #anArray
                putAll: #(1 2 3)
                itemType: 'xsd:int'.
        ^request xmlFormatted

        "
         Transcript showCR:
            self exampleArray
        "
!

exampleInlineType
        "Splash takes the approach of only adding inline type attributes where absolutely necessary. 
         It does give the user the ability to manually add them, but since Splash does not have knowledge of the interface 
         the best strategy is to provide no type information, and let the server interpret the parameter.
        If it becomes necessary to add a type attribute to a parameter use the method #parameterAt:put:type: "

        | request |
        request := SplashRPCRequest method: 'Multiply' ns: #'http://soapinterop.org'.
        request 
                parameterAt: #a
                put: 5
                type: 'xsd:int'.
        request 
                parameterAt: #b
                put: 8
                type: ('int' ns: #'http://www.w3.org/2001/XMLSchema').
        ^request xmlFormatted

        "
         Transcript showCR:
            self exampleInlineType
        "
!

exampleMultiref
        "Splash maintains object identity when encoding objects, and will encode the same object twice as a multireference."


        | request sameString |
        sameString := 'hello'.
        request := SplashRPCRequest method: 'stringTest' ns: #'http://stringTest.org'.
        request parameterAt: #first put: sameString.
        request parameterAt: #firstAgain put: sameString.
        request parameterAt: #firstCopy put: sameString copy.
        ^request xmlFormatted

        "
         Transcript showCR:
            self exampleMultiref
        "
!

exampleRaw
        "This can be used to add any text content as a parameter. 
         Splash does not check whether the string is well formed xml or not."


        | request |
        request := SplashRPCRequest method: 'test' ns: #'http://test.org'.
        request parameterAt: #foo putXml: 'simple'.
        request parameterAt: #bar putXml: '<abc>compex</abc>'.
        request parameterAt: #fubar putXml: '<oops>but if you really want to</ok>'.
        ^request xmlFormatted

        "
         Transcript showCR:
            self exampleRaw
        "
!

exampleSend
        "SplashRequest contains a number of helper methods that will send the request using defaults, check for Faults and answer a SplashResponse containing the response from the server.
        For example to send a request to the 'Barnes and Noble Price Quote' service <http://www.xmethods.net/detail.html?id=7>
        The response is an object that gives you a number of ways to access to the soap envelope."

        | request response |
        request := SplashRPCRequest method: 'getPrice' ns: #'urn:xmethods-BNPriceCheck'.
        request parameterAt: #isbn put: '0201737930'.
        ^response := request 
                        responseFrom: 'http://services.xmethods.net:80/soap/servlet/rpcrouter'
                        soapAction: nil

        "
         Transcript showCR:
            self exampleSend
        "
!

exampleSendReturn
	"If you are sending an RPC request there is an additional helper method that will answer the 'return' value. The 'return' value is defined by the SOAP 1.1 specification as the first element of the RPC response element. The actual name of the element is not significant."

	| request return |
	request := SplashRPCRequest method: 'getPrice' ns: #'urn:xmethods-BNPriceCheck'.
	request parameterAt: #isbn put: '0201737930'.
	^return := request returnFrom: 'http://services.xmethods.net:80/soap/servlet/rpcrouter'
				soapAction: nil
!

exampleStruct
	| request struct |
	(struct := XeQStruct new)
		at: #varString put: 'abc';
		at: #varInt put: 123;
		at: #varFloat put: 3.14.
	request := SplashRPCRequest method: 'echoStruct' ns: #'http://soapinterop.org/'.
	request parameterAt: #inputStruct put: struct.
	^request xmlFormatted
! !

!SplashRPCRequest methodsFor:'accessing'!

nodeAtId: aString 
	"Private - Answer the node with the id 'aString' or nil.
	-See #valueNodeFor:
	-This is not efficient, but fast enough for Splashes intended use"

	^node nodeAtId: aString
!

parameterAt: aString
	"Answer the XeNode that has encoded the parameter named aString"

	^self parameterAt: aString ifAbsent: [self errorNotFound: aString]
!

parameterAt: aString ifAbsent: operation
	self assertHasRPCNode.
	^rpcNode detectChildNode: aString in: nil ifNone: operation 
!

valueNodeFor: aNodeOrNil
	"Answer the node that encodes the value of aNodeOrNil. 
	-This method is necessary so that the encoding methods (#parameterAt:put: etc) answer the node the value was encoded into. 	
	--This allows senders of those messages to alter the encoding if necessary.
	-If no multireferencing the answer == aNodeOrNil
	-If a multireference is used the answer will be different. ie
		<root>
		  <foo href='#1'/>
		  <bar id='1'>123</bar>
		</root>
	--If aNodeOrNil is 'foo' this method will answer 'bar'"

	| href derefNode |
	(aNodeOrNil isNil or: [(href := aNodeOrNil getAttribute: 'href') isNil]) 
		ifTrue: [^aNodeOrNil].
	(derefNode := self nodeAtId: (href copyFrom: 2 to: href size)) isNil 
		ifTrue: [self error: 'Could not resolve the href ' , href displayString].
	^self valueNodeFor: derefNode
! !

!SplashRPCRequest methodsFor:'accessing-private'!

isIDHref
	^flags allMask: IsIDHref
!

isIDHref: aBoolean
	flags := flags mask: IsIDHref set: aBoolean
!

isSoapEnc
	^flags allMask: IsSoapEnc
!

isSoapEnc: aBoolean
	flags := flags mask: IsSoapEnc set: aBoolean
! !

!SplashRPCRequest methodsFor:'asserting-private'!

assertHasRPCNode
	self assert: [rpcNode notNil]
! !

!SplashRPCRequest methodsFor:'encoding-helpers'!

useReferenceOf: anObject on: aXeNode
	"Answer true if we encode object as a ref into node, otherwise false.
	Side effects of this method are;
	-if anObject can encode as a multiref, it is encoded
	-we remember anObject and aNode, so that if anObject is encoded again, we multireference it then"

	^self 
		useReferenceOf: anObject
		on: aXeNode
		special: false
!

useReferenceOf: anObject on: aXeNode special: aBool
	"See #useReferenceOf:on: comment.
	-If special is true (should only be true if anObject is a string or array of bytes), we use the SOAP1.1 id/href 'special rule' to leave the multireference value in place."

	^self isIDHref 
		ifTrue: 
			[writeMap 
				useReferenceOf: anObject
				on: aXeNode
				special: aBool]
		ifFalse: [false]
! !

!SplashRPCRequest methodsFor:'encoding-helpers-private'!

putAll: aCollection on: pNode itemType: aXeName
	"Private - Helper method to encode an Array of a certain type of items"

	| typeQName |
	typeQName := self qualify: aXeName.
	node fixupPrefixGloballyFor: typeQName.
	pNode setQAttribute: (XeQName soapEnc: 'arrayType')
		value: ('%1[%2]' formatWith: typeQName qString with: aCollection size displayString).
	aCollection do: [:each | each putSplashOn: (pNode addElement: 'item') context: self].
	^pNode
! !

!SplashRPCRequest methodsFor:'encoding-parameters'!

parameterAt: aString put: anObject 
	"Add a parameter named 'aString' to the rpc element, and encode a representation of anObject.
	Answer the node the value encodes into (may not be the parameter node ... see #valueNodeFor:)
	-Notes:
	--Parameter names are significant, order is not. (for best interop you should add parameters in the 'expected' order).
	--Missing parameters are interpreted as nil. (xsi:nil can also be used, however omitting is the recommend practice)
	--IMPORTANT: If anObject isKindOf: String, Splash assumes you are encoding an xsd:string. 
	----If you just want to add some text, use parameterAt:putXml: (which will not assume aString is an xsd:string. This is important because xsd:strings have special id/href encoding rules which are not valid for other types "
	
	| pNode |
	anObject putSplashOn: (pNode := self addParameterNode: aString) context: self.
	^self valueNodeFor: pNode
!

parameterAt: aString put: anObject type: aName 
	"Add a parameter named 'aString' to the rpc element, and encode a representation of anObject.
	-add an xsi:type attribute with the value aName
	-aName can be either
	--a String like 'xsd:string' (late qualifies using receiver as a Qualifying Context)
	--a XePName like XePName xsd: 'string' (late qualifies using receiver as a Qualifying Context)
	--a XeQName like XeQName xsd2001: 'string' (fully qualified) "

	| vNode |
	vNode := self parameterAt: aString put: anObject.
	self putXSIType: aName asXeName on: vNode 
!

parameterAt: aString putAll: aCollection itemType: aName
	"Add a parameter named 'aString' to the rpc element, encoding aCollection as an Arrayof  aName type.
	-This is the recommended method to encode an array parameter"
 
	| pNode |
	pNode := self addParameterNode: aString.
	^self 
		putAll: aCollection
		on: pNode
		itemType: aName
!

parameterAt: aString putXml: xmlString
	"Add a parameter, and make its contents anXmlString.
	-Note: the XML in xmlString does not become part of the 'infoset' of the envelope as XeEncoding uses a special class for this functionality.
	-Use with caution ... nothing will stop you from creating non-well formed xml with this method
	-Can either be simple content ie if aString='foo' and xmlString ='abc'
		<m:method ...>
		  <foo>abc</foo>
		</m:method>
	-Or complex content ie if aString='foo' and xmlString ='<bar>abc</bar>'
		<m:method ...>
		  <foo><bar>abc</bar></foo>
		</m:method>"

	| pNode |
	(pNode := self addParameterNode: aString) addInsert: xmlString.
	^pNode
! !

!SplashRPCRequest methodsFor:'encoding-parameters-private'!

addParameterNode: aString
	"Private - Answer an XeNode. 
	Add a node to the RPCWrapper node and answer it"

	self assertHasRPCNode.
	^rpcNode addElement: aString
! !

!SplashRPCRequest methodsFor:'encoding-rpc-private'!

addRPCWrapperNamed: aXeQName
	"Private -  Answer a XeNode 
	Add  the RPCWrapper node to the soap:Body node and answer it.
	-ie after this method we have: <soap:Body><m:localName xmlns:m='namespace'/><soap:Body>"

	rpcNode notNil ifTrue: [self error: 'RPC Wrapper has already been added to the envelope'].
	^rpcNode := bodyNode addElementNamed: aXeQName rootDeclare: false
! !

!SplashRPCRequest methodsFor:'initializing-private'!

createEnvelope
	"Private - create the soap envelope adding some standard namespace declarations to it"

	super createEnvelope.
	writeMap := XeNodeWriteMap new initializeFor: nil node: bodyNode 
!

initialize
        "Private - initialize the request.
        -We default to using soap-enc,  the 2001 schema, and IDHref encoding"

        super initialize.
        flags := IsIDHref | IsSoapEnc.
        self createEncodedEnvelope.
        prefixMap := Dolphin::LookupTable new.
        self beSchema2001.
! !

!SplashRPCRequest methodsFor:'modes'!

beLiteralEnc
	"Use with caution ... only soap-enc is tested, literalEnc is Alpha.
	-When an object encodes itself, it can test whether the encoding is soap-enc or not, and can make interop encoding decisions based on that. Typical decisions would be to not add an inline type attribute.
	-Very rare to see WebServices that combine RPC with Literal"

	self isSoapEnc: false

!

beNotIdHref
	"Dont encode identical objects using multireferences.
	-IDHref's are only valid in soap-enc."

	self isIDHref: false

!

beSoapEnc
	"Set mode to soap-enc (this is the default)"

	self isSoapEnc: true

! !

!SplashRPCRequest methodsFor:'testing'!

hasParameter: aString
	"Answer whether the RPCNode has a parameter with the name aString."

	self parameterAt: aString ifAbsent: [^false].
	^true
! !

!SplashRPCRequest methodsFor:'transporting'!

returnFrom: aURLString soapAction: aStringOrNil
	"Answer the object decoded from the 'return' parameter of a response.
	-Send the receiver to the server at aURLString. (with the HTTP Header SOAPAction set to aString2).
	--If the message exchange does not end in the Success state, signal an error
	--If the response received is a soap:Fault, signal an error"

	^(self responseFrom: aURLString soapAction: aStringOrNil) return
! !

!SplashRPCRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashRPCRequest.st,v 1.5 2008/09/05 17:48:26 cg Exp $'
! !

SplashRPCRequest initialize!
