"{ Package: 'stx:goodies/soap/splash' }"

"{ NameSpace: SOAP }"

Object subclass:#SplashResponse
	instanceVariableNames:'envelope prefixMap textDecoder flags'
	classVariableNames:'IsConvertText IsSoapEnc'
	poolDictionaries:''
	category:'SOAP-Splash'
!

SplashResponse comment:'I am a soap message response. I wrap a SpwSoapInEnvelope and provide access to the contents of a received soap envelope

Although Splash has no knowledge of an operations interface, it can always determine whether the encoded value is an Array, a Struct or simple content. By default, it will use inline type attributes to convert a nodes text into an object, however a server is not required to add inline type attributes and Splash will answer the text.

Note: both #return and #returnAs: can
-answer nil 
-signal an exception if the response from the server was a Fault. 

For consistency across servers recommended practice is to either:
-Use the #returnAs: method to force a conversion. 
-Send the response the message #beDontConvertText (which will force inline typing to be ignored) and do the conversions from text yourself. 

==========
Instance Variables:
	envelope		<SpwSoapInEnvelope>
	prefixMap		<Dictionary>
	textDecoder	<SplashTextDecoder>
	flags		<Integer>



'
!


!SplashResponse class methodsFor:'instance creation'!

envelope: aSpwSoapEnvelope 
	^self new envelope: aSpwSoapEnvelope 
!

exchange: aSptRequestingExchange
	^self envelope: aSptRequestingExchange in
!

new
	^self basicNew initialize
! !

!SplashResponse class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'objectIn.ico'
    ]
! !

!SplashResponse class methodsFor:'initializing'!

initialize
	"
	self initialize
	"

	IsConvertText := 1.
	IsSoapEnc := 2
! !

!SplashResponse class methodsFor:'must strip'!

exampleOutParameters
        <resource: #example>

        "This example uses the SoapBuilders interop service. See <http://www.whitemesa.com/interop/proposalB.html>."

        | request response struct |
        (struct := XeQStruct new)
                at: #varString put: 'abc';
                at: #varInt put: 123;
                at: #varFloat put: 3.14.
        request := SplashRPCRequest method: 'echoStructAsSimpleTypes'
                                ns: #'http://soapinterop.org/'.
        request parameterAt: #inputStruct put: struct.
        response := request responseFrom: 'http://www.dolphinharbor.org/services/interopB2001'
                                soapAction: nil.
        response return.
        response outParameters
!

exampleTyping
        <resource: #example>

        "This example uses the SoapBuilders interop service. See <http://www.whitemesa.com/interop/proposal2.html>."

        | request response |
        request := SplashRPCRequest method: 'echoBoolean' ns: #'http://soapinterop.org/'.
        request parameterAt: #inputBoolean put: true.

        "The Spray servers on dolphinharbor always inline type"
        response := request responseFrom: 'http://www.dolphinharbor.org/services/interop2001'
                                soapAction: nil.
        response return.
        response returnAs: 'xsd:boolean'.

        "The SOAP4R server does not inline type"
        response := request 
                                responseFrom: 'http://www.jin.gr.jp/~nahi/Ruby/SOAP4R/SOAPBuildersInterop/'
                                soapAction: nil.
        response return.
        response returnAs: 'xsd:boolean'
! !

!SplashResponse methodsFor:'accessing'!

addPrefix: aString namespace: aSymbol 
	prefixMap at: aString put: aSymbol 
!

envelope: aSoapInEnvelope 
	envelope := aSoapInEnvelope 
! !

!SplashResponse methodsFor:'accessing-decoded'!

outParameters
	"Answer the outParameters as a XeQStruct.
		<soap:Body>
		<m:echoStructAsSimpleTypesResponse ...>
		  <outputString>abc</outputString>
		  <outputInteger>123</outputInteger>
		  <outputFloat>3.14</outputFloat>
		</m:echoStructAsSimpleTypesResponse>
		</soap:Body>
	This would answer an XeQStruct with the accessors (keys) #outputString, #outputInteger, #outputFloat
	(Note: the return for this envelope: outputString is still a valid return parameter)"

	^self decodeNode: self wrapperNode as: nil
!

return
	"Answer the return parameter as defined by the SOAP section 7 rules.
		<SOAP-ENV:Envelope>
		  <SOAP-ENV:Body>
  		    <ns1:theGetPriceResponse  ... >
		      <theReturn>39.99</theReturn>
		    </ns1:theGetPriceResponse>
		  </SOAP-ENV:Body>
		</SOAP-ENV:Envelope>
	-The node named 'theGetPriceResponse is the RPC Response wrapper.
	-The node named 'theReturn' is the RPC Response return parameter.
	-The names of both of these nodes are not significant, it is their position which is significant"

	^self decodeNode: self returnNode as: nil 
	
!

returnAs: aTypeName 
	"(See comment for #return)
	Answer the return parameter, converting it to an object using the converter for aTypeName.
	As examples, aTypeName could be 'xsd:string' 'xsd:int' 'xsd:float' etc .
	-Ignore the isConvertText mode"

	| converting answer|
	converting := self isConvertText.
	self beConvertText.
	answer := self decodeNode: self returnNode as: aTypeName.
	self isConvertText: converting.
	^answer

! !

!SplashResponse methodsFor:'accessing-nodes-private'!

returnNode
	^envelope firstBodyBlock firstChild
!

wrapperNode
	^envelope firstBodyBlock
! !

!SplashResponse methodsFor:'accessing-private'!

isConvertText: aBoolean
	flags := flags mask: IsConvertText set: aBoolean
!

isSoapEnc: aBoolean
	flags := flags mask: IsSoapEnc set: aBoolean
!

namespacePrefixed: aString 
	"Private - Answer a NSSymbol"

	^prefixMap at: aString ifAbsent: [self error: 'The typespace does not contain the prefix: ' , aString displayString]
!

textDecoder
	"Private - answer a SplashTextDecoder"

	^textDecoder
! !

!SplashResponse methodsFor:'accessing-xml'!

iXml
        ^Dolphin::IXMLDOMDocument new loadText: self xml
!

xml
	"Answer a String, the raw xml string of the response envelope"

	^envelope xml
!

xmlFormatted
	"Answer a String, the raw xml string of the response envelope (formatted with line breaks)"

	^envelope xmlFormatted
! !

!SplashResponse methodsFor:'asserting-private'!

assertNotFault
        self isFault ifTrue: [
            Smalltalk isSmalltalkX ifTrue:[
                envelope faultException raiseSignal
            ] ifFalse:[
                envelope faultException signal
            ]
        ]
! !

!SplashResponse methodsFor:'initializing-private'!

initialize
        super initialize.
        prefixMap := Dolphin::LookupTable new.
        self addPrefix: 'xsd' namespace: XeNamespace schema2001.
        self addPrefix: 'soap' namespace: XeNamespace soapEnc.
        self addPrefix: 'SOAP-ENC' namespace: XeNamespace soapEnc.
        flags := IsSoapEnc | IsConvertText.
        textDecoder := SplashTextDecoder new
! !

!SplashResponse methodsFor:'modes'!

beConvertText
	"This mode will convert text into an object if the node contains an inline type attribute"

	self isConvertText: true
!

beDontConvertText
	"This mode will ignore inline type attributes and always answer the text contents of a node(s)"

	self isConvertText: false
!

beLiteralEnc
	"This mode assumes the xml is literal.
	Main difference is that complexContent in literal can have attributes"

	self isSoapEnc: false
!

beSoapEnc
	"Default soap-enc mode"

	self isSoapEnc: true
! !

!SplashResponse methodsFor:'operations'!

convertText: aTextString as: aTypeName 
	"Helper method.
	-Answer the object that is converted from aTextString using the converter for the type named aTypeName.
	-The guard clauses give this method similar semantics to standard soap-enc decoding"

	aTextString isNil ifTrue: [^nil].
	aTypeName isNil ifTrue: [^aTextString].
	^textDecoder convert: aTextString typeName: (self qualify: aTypeName)
!

convertTextArray: anArray as: aName
	"Helper method. - does not preserve identity"

	anArray isNil ifTrue: [^nil].
	^anArray collect: [:each | self convertText: each as: aName]
!

convertTextStruct: aStruct asAll: aColl 
	"Helper method. - does not preserve identity
	aColl is a collection of two element arrays
	-first element = parameter name
	-second element = parameter type"

	| struct |
	aStruct isNil ifTrue: [^nil].
	struct := XeQStruct new.
	aColl do: 
			[:each | 
			struct at: each first put: (self convertText: (aStruct at: each first ifAbsent: []) as: each second)].
	^struct
! !

!SplashResponse methodsFor:'operations-private'!

decodeNode: aNode as: aNameOrNil 
	"Private - Answer the object decoded from aNode
	-If the envelope is a soap:Fault, signal a SoapSendFault  exception.
	-If aNameOrNil is notNil, manually convert the object using the converter for aNameOrNil"

	| reader |
	self assertNotFault.
	aNode isNil ifTrue: [^nil].
	reader := self readerFor: aNode.
	aNameOrNil notNil ifTrue: [reader typeName: (self qualify: aNameOrNil )].
	reader decode.
	^reader nodeObject
!

qualify: aTypeName
	^aTypeName asXeName qualifiedNameIn: self
!

readerFor: aNode
	| reader |
	reader := SplashNodeReader onNode: aNode.
	self isConvertText 
		ifTrue: [reader textDecoder: textDecoder]
		ifFalse: [reader textDecoder: nil].
	self isLiteralEnc ifTrue: [reader beLiteralEnc].
	^reader
! !

!SplashResponse methodsFor:'testing'!

isFault
	^envelope isFault
! !

!SplashResponse methodsFor:'testing-private'!

isConvertText
	^flags allMask: IsConvertText
!

isLiteralEnc
	^self isSoapEnc not
!

isSoapEnc
	^flags allMask: IsSoapEnc
! !

!SplashResponse class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashResponse.st,v 1.8 2010/02/26 12:02:22 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashResponse.st,v 1.8 2010/02/26 12:02:22 cg Exp $'
! !

SplashResponse initialize!
