"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpdSourceAnalysis
	instanceVariableNames:'global selector methodSource pairs'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Development'
!

SpdSourceAnalysis comment:'I analyze the source of a method and provide access to the metadata required to construct the interface for a RPC/Encoded operation.

I do two things:
	-I locate the first comment starting with "Spd" and assume that it will evaluate to an instance of <SpdOperationCreator>
	-I supply defaults to the SpdOperationCreator where they have not been specified.

The second function is deprecated. Best practice is to always use a SpdOperationCreator. 
Future versions will:
	-only supply a default name and selector to the SpdOperationCreator. 
	-require that types are always specified using XMLSchema QName literal form.
	(All the current examples and samples should follow these practices)

Instance Variables:
	global		<Class>
	selector		<Symbol>
	methodSource	<String>
	pairs		<Collection>
'
!


!SpdSourceAnalysis methodsFor:'accessing'!

answerTypeString
        "Private - locate a line starting with answer, and extract the remainder of that line"

        | ansC |

        Smalltalk isSmalltalkX ifTrue:[
            ansC := self firstComment lines 
                                    detect: [:each | '*answer*' match: each trimBlanks readStream upToSeparator asLowercase]
                                    ifNone: [self errorNoComment].
        ] ifFalse:[
            ansC := self firstComment lines 
                                    detect: [:each | '*answer*' match: each trimBlanks readStream nextWord asLowercase]
                                    ifNone: [self errorNoComment].
        ].
        ^self answerTypeStringFrom: ansC
!

commentArray
	^self separateSource second asArray
!

commentStrippedSource
	^self separateSource first trimBlanks
!

firstComment
	| commentArray |
	(commentArray := self commentArray) isEmpty ifTrue: [self errorNoComment].
	^commentArray first
!

global: aClass
	global := aClass
!

inputParts
	self isUnaryMethod ifTrue: [^OrderedCollection new].
	^self pairs 
		collect: [:each | SpmdTypePart partName: each key typeName: (self typePNameFor: each value)]
!

methodSource
	^methodSource
!

methodSource: aString
	methodSource := aString
!

operationCreator
        | evalString opCreator |
        evalString := self commentArray detect: [:each | each beginsWith: 'Spd'] ifNone: [].
        opCreator := evalString isNil 
                                ifTrue: [SpdOperationCreator new]
                                ifFalse: [Compiler new setNameSpace:SOAP; evaluate: evalString logged: false].
        opCreator isNil ifTrue: [Error signal: 'Compiler failed to evaluate SpdOperationCreator string'].
        opCreator sourceAnalysis: self.
        ^opCreator
!

outputParts
	^self outputPartsWithType: (self outputTypePNameFor: self answerTypeString)
!

pairs
	^pairs
!

selector
	^selector
!

selector: aSelectorSymbol
	selector := aSelectorSymbol
!

setPairsFromSource: aString
        "Private - 'Borrowed idea(errors are mine) from IDB's MethodOf"

        |stream  word |
        pairs := OrderedCollection new.
        stream := aString readStream.
        Smalltalk isSmalltalkX ifTrue:[
            word := stream upToSeparator.
        ] ifFalse:[
            word := stream nextWord.
        ].
        (word includes: $:)
                ifTrue: [
                        [word notNil and: [word includes: $:]] whileTrue: [ | pair |
                                pairs add: (pair := Association new).
                                pair key: (word copyFrom: 1 to: (word indexOf: $:) - 1).
                                word last = $: 
                                        ifTrue: [pair value: stream nextWord]
                                        ifFalse: [pair value: (word copyFrom: (word indexOf: $:) + 1 to: word size)].
                                word := stream nextWord]]
                ifFalse: [pairs add: word -> nil].

        ^pairs

!

suggestedOperationName
	^self pairs first key
! !

!SpdSourceAnalysis methodsFor:'asserting'!

assertNotImpliedSelfReturn
	self isImpliedSelfReturn ifTrue: [self errorImpliedSelfReturn]
! !

!SpdSourceAnalysis methodsFor:'exceptions'!

errorCantDetermineType: aString
	Error signal: ('Warning %1>>%2 unable to convert %3.' 
				formatWith: global name
				with: selector
				with: aString displayString)
!

errorCommentFormat
	self error: 'Unexpected comment format'
!

errorImpliedSelfReturn
	Error 
		signal: ('Warning %1>>%2 has implied self return' formatWith: global name with: selector)
!

errorNoComment
	Error 
		signal: ('Warning %1>>%2 has no answer comment for type analysis.' formatWith: global name
				with: selector)
! !

!SpdSourceAnalysis methodsFor:'helpers'!

outputTypePNameFor: string
	"Private - Firstly check for a void answer.
	  If not, check to see if the type includes a colon which indicates that the string should be treated as a schema QName literal
	  Otherwise try and convert a the answer into a schema typeString"

	(#('nil' 'null' 'void') includes: string) ifTrue: [^nil].
	^self typePNameFor: string
!

trimQNameLiteral: aString
	self assert: [aString includes: $:].
	^aString last = $. ifTrue: [aString copyFrom: 1 to: aString size - 1] ifFalse: [aString]
!

typePNameFor: string
	"Private - Check to see if the type includes a colon which indicates that the string has the XMLSchema QName literal form.
		If not, assume the string has the typical form of a Smalltalk argument and use a typeSuggestor 
			Note: Using the typeSuggestor is deprecated in Spray Beta 2"

	^(string includes: $:) 
		ifTrue: [self trimQNameLiteral: string]
		ifFalse: 
			[SpdTypeSuggestor new suggestFor: string ifNone: [self errorCantDetermineType: string]]
! !

!SpdSourceAnalysis methodsFor:'initializing'!

setGlobal: aClass selector: aSymbol
	global := aClass.
	selector := aSymbol.
	methodSource := (aClass lookupMethod: aSymbol) getSource.
	self setPairs
!

setPairs
	self setPairsFromSource: self methodSource
! !

!SpdSourceAnalysis methodsFor:'operations'!

answerTypeStringFrom: aString 
        "Private - aString is of the form [a|A]nswer [a|an] type.
        Answer the string 'type'"

        | word commentStream |
        commentStream := aString readStream.
        Smalltalk isSmalltalkX ifTrue:[
            word := commentStream upToSeparator.
        ] ifFalse:[
            word := commentStream nextWord.
        ].
        (word isNil or: [('*nswer*' match: word) not]) ifTrue: [self errorCommentFormat].
        Smalltalk isSmalltalkX ifTrue:[
            word := commentStream upToSeparator.
        ] ifFalse:[
            word := commentStream nextWord.
        ].
        word isNil ifTrue: [self errorCommentFormat].
        (#('a' 'an') includes: word) 
                ifTrue: [
                    Smalltalk isSmalltalkX ifTrue:[
                        word := commentStream upToSeparator.
                    ] ifFalse:[
                        word := commentStream nextWord.
                    ].
                    word isNil ifTrue: [self errorCommentFormat]].
        ^word
!

outputPartsWithType: prefixedTypeString
	^prefixedTypeString isNil 
		ifTrue: [OrderedCollection new]
		ifFalse: 
			[OrderedCollection 
				with: (SpmdTypePart partName: self returnPartName typeName: prefixedTypeString)]
!

returnPartName
	^'Result'
!

separateSource
        | sourceStream commentStream commentArray isSource firstLoop prev|
        sourceStream := String writeStream.
        sourceStream nextPut: self methodSource first.
        commentStream := String writeStream.
        commentArray := OrderedCollection new.
        isSource := true.

        firstLoop := true.
        self methodSource do:[:next |
            firstLoop ifTrue:[
                firstLoop := false.
            ] ifFalse:[

                        isSource 
                                ifTrue: 
                                        [(next = $" and: [prev ~= $$]) 
                                                ifTrue: 
                                                        [isSource := false.
                                                        sourceStream space]]
                                ifFalse: 
                                        [next = $" 
                                                ifTrue: 
                                                        [isSource := true.
                                                        commentArray add: commentStream contents.
                                                        commentStream := String writeStream]].
                        (next = $" and: [prev ~= $$]) 
                                ifFalse: [(isSource ifTrue: [sourceStream] ifFalse: [commentStream]) nextPut: next]].
            prev := next.
        ].
        ^Array with: sourceStream contents with: commentArray




















! !

!SpdSourceAnalysis methodsFor:'testing'!

isImpliedSelfReturn
	^(self commentStrippedSource includes: $^) not
!

isUnaryMethod
	^self pairs first value isNil
! !

!SpdSourceAnalysis class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SOAP__SpdSourceAnalysis.st,v 1.2 2008/09/07 17:29:54 cg Exp $'
! !
