"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdMessage
	instanceVariableNames:'name parts flags'
	classVariableNames:'IsNoReturnPart'
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdMessage comment:'I am a web service message. I am the metadata that describes a collection of parameters contained in a soap message.

I am  used to create a "wire" object that decodes a soapBlock (either in the soap:Body or soap:Header) to an object, and encodes an object to a soapBlock (Note: a soapBlock is a string of xml)

Instance Variables:
	name	<String>
	parts	<Collection> of <SpmdParts>
	flags	<Integer>

My name only has significance to a WSDL''s internal referencing'
!


!SpmdMessage class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	IsNoReturnPart := 1
! !

!SpmdMessage class methodsFor:'instance creation'!

forSingleElement: aXeQName
	"Helper construction method for a message to be used with literal encoding"

	^self onPart: (SpmdElementPart partName: 'part1' elementName: aXeQName)
!

forSingleTypeName: aXeQName partName: aString 
	"Helper construction method for a message to be used with literal encoding"

	^self onPart: (SpmdTypePart partName: aString typeName: aXeQName)
!

fromWSDLMessage: aWSDLMessage
	^self new setFromWSDLMessage: aWSDLMessage
!

name: aString parts: anOrderedCollection
	self assert: [aString notNil and: [aString notEmpty]].
	^self new name: aString parts: anOrderedCollection
!

onPart: aPart
	^self onParts: (OrderedCollection with: aPart)
!

onParts: anOrderedCollection
	"This is the typical construction method. The operation will sebsequently name the message."

	^self new parts: anOrderedCollection
! !

!SpmdMessage class methodsFor:'constants'!

icon
	^SpmdPort icon
! !

!SpmdMessage methodsFor:'accessing'!

firstPartName
	^self parts first partName
!

sourceFormat
	"answer a representation of the message parts similar to that used in the SpdOperationCreator comments"

	^(self parts collect: [ :each | each sourceFormat]) asArray
! !

!SpmdMessage methodsFor:'accessing-private'!

assertAllHaveSelector: partNames
	"partNames is necessary because the receiver's owning <SpmdBlockBinding> may specify that it only uses certain parts 
		(This is (IMO an ugly) implementation detail of WSDL)."

	^(self selectParts: partNames) do: [:each | self assert: [each selector notNil]]
!

defaultFlags
	^0
!

docSelector: aSymbol 
	(self parts size ~~ 1) ifTrue: [self error: 'This message has multiple parts. Selectors must be set into the parts individually'].
	self parts first selector: aSymbol
!

docSelectors: partNames
	"Answer the selectors for the parts named by partNames
	-The WSDL allows 'extra' parts to be added to a message (to be used by header bindings), and
		a partNames attribute to select the parts from the message"

	self assertAllHaveSelector: partNames.
	^(self selectParts: partNames) collect: [:each | each selector].
!

flags
	flags isNil ifTrue: [flags := self defaultFlags].
	^flags
!

flags: anInteger
	flags := anInteger
!

isNoReturnPart
	^self flags allMask: IsNoReturnPart
!

isNoReturnPart: aBool
	flags := self flags mask: IsNoReturnPart set: aBool
!

messageName
	self assert: [name notNil and: [name notEmpty]].
	^name
!

name
	^name
!

name: aString 
	name := aString
!

name: aString parts: anOrderedCollection
	name := aString.
	parts := anOrderedCollection. 
!

parts
	^parts
!

parts: anArray 
	parts := anArray 
!

setFromWSDLMessage: aWSDLMessage
	name := aWSDLMessage messageName.
	parts := aWSDLMessage createParts
! !

!SpmdMessage methodsFor:'operations-private'!

docSpecs: partNames namespace: aNSSymbolOrNil
	"Answer a collection of <XeElementSpec>s.
	partNames is necessary because the receiver's owning <SpmdBlockBinding> may specify that it only uses certain parts 
		(This is (IMO an ugly) implementation detail of WSDL)."

	^(self selectParts: partNames) 
		collect: [:each | each docSpecQualifyingIn: aNSSymbolOrNil]
!

rpcSpecs: partNames
	"Answer a collection of <XeElementSpec>s.
	partNames is necessary because the receiver's owning <SpmdBlockBinding> may specify that it only uses certain parts 
		(This is (IMO an ugly) implementation detail of WSDL)."

	^(self selectParts: partNames) collect: [:each | each rpcSpec]
!

selectParts: anArrayOrNil 
	"Private - Answer a collection of parts. 
	-If anArrayOrNil isNil answer all parts, otherwise answer only the named parts."

	anArrayOrNil isNil ifTrue: [^self parts].
	^anArrayOrNil collect: [ :each | self parts detect: [ :eachP | eachP partName = each]]
! !

!SpmdMessage methodsFor:'printing'!

printOn: aStream
    super printOn:aStream.
    aStream nextPut:$(.
    name printOn:aStream.
    aStream nextPut:$).
!

putSpraySignatureOn: aWriteStream partNames: anArrayOrNil
	"Put a readable description string on the stream for documentation purposes"

	aWriteStream display: $(.
	(self selectParts: anArrayOrNil) do: [:each | each putSpraySignatureOn: aWriteStream]
		separatedBy: [aWriteStream space].
	aWriteStream display: $)
! !

!SpmdMessage methodsFor:'testing'!

hasNoneOrOneParts
	^self parts size <= 0
! !

!SpmdMessage methodsFor:'wsdl-private'!

defineMessage: aDefinitionNode typespace: aTypespace
	"Add the receiver to the WSDL definition <XeNode>"

	| messageNode |
	messageNode := aDefinitionNode addElement: 'message'.
	messageNode setAttribute: 'name' value: self messageName.
	self parts do: [:each | each definePart: messageNode in: aTypespace]
!

putWSDLNameOn: aNode
	aNode setAttribute: 'message' value: 'tns:' , self messageName.
	^aNode
! !

!SpmdMessage class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdMessage.st,v 1.3 2008/09/03 08:52:43 cg Exp $'
! !

SpmdMessage initialize!
