"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdOperation
	instanceVariableNames:'name inputBody outputBody inputHeaders outputHeaders soapAction
		flags documentation'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdOperation comment:'I am a web service operation. I am the metadata that describes two soap envelopes, an input and an output that form a request/response pair.

I am  used to create the "wire" objects that process soap envelopes and their content for both servers and clients.

Note: the labels ''Input'' and ''Output'' reflect their use in the WSDL specification. 
	From a server''s perspective:
		Input == incoming soap request message
		Output == outgoing soap response message
	From a client''s perspective:
		Input ==outgoing soap request message
		Output == incoming soap response message

Instance Variables:
	name		<String>
	inputBody		<SpmdSoapBody>
	outputBody	<SpmdSoapBody>
	inputHeaders	<SpmdSoapHeader>
	outputHeaders	<SpmdSoapHeader>
	soapAction	<String>
	flags		<Integer>
	documentation	<String>

'
!


!SpmdOperation class methodsFor:'instance creation'!

fromWSDLOperation: aWSDLOperation
	^(self named: aWSDLOperation name)
		setFromWSDLOperation: aWSDLOperation
!

named: aString 
	^self new
		name: aString;
		yourself
! !

!SpmdOperation class methodsFor:'constants'!

icon
	^SpmdPort icon
! !

!SpmdOperation methodsFor:'accessing'!

documentation: aString
	documentation := aString
!

name
	^name
!

name: aString
	"Answer the name of the operation.
	This describes the name of the request SOAP RPC Wrapper.
		 <soap:Body>
			<m:operationName xmlns:m='?'/>
		  </soap:Body>
	"

	name := aString
!

rpcSelector: aSymbol 
	self inputBody selector: aSymbol 
!

selectors
	"Answer a collection of selectors (document style theoretically could have multiple selectors)"

	^self subclassResponsibility
!

soapAction
	"Answer the SOAPAction <String> to include as an HTTP Header in requests"

	soapAction isNil ifTrue: [soapAction := self defaultSoapAction].
	^soapAction
!

soapAction: aString
	"Set the SOAPAction <String> to include as an HTTP Header in requests"

	soapAction := aString
!

spraySignature
	"Answer a readable description string for documentation purposes"

	| stream |
	stream := String writeStream.
	self putSpraySignatureOn: stream.
	^stream contents
!

style
	^self subclassResponsibility
! !

!SpmdOperation methodsFor:'accessing-private'!

allMessages
	"Private - Answer all of the receiver's <SpmdMessage>s "

	| coll |
	coll := OrderedCollection new.
	coll 
		add: inputBody message;
		add: outputBody message.
	self inputHeaders do: [ :each | coll add: each message].
	self outputHeaders do: [ :each | coll add: each message].
	^coll

!

defaultFlags
	^0
!

documentation
	documentation isNil ifTrue: [documentation := ''].
	^documentation
!

flags
	flags isNil ifTrue: [flags := self defaultFlags].
	^flags
!

flags: anInteger
	flags := anInteger
!

inEnvelopeClass
	^self subclassResponsibility
!

inputBody
	"Answer a <SpmdSoapBody>"

	^inputBody
!

inputHeaders
	"Answer a collection of <SpmdSoapHeader>s"

	inputHeaders isNil ifTrue: [inputHeaders := OrderedCollection new].
	^inputHeaders
!

outEnvelopeClass
	^self subclassResponsibility
!

outputBody
	"Answer a <SpmdSoapBody>"

	^outputBody
!

outputHeaders
	"Answer a collection of <SpmdSoapHeader>s"

	outputHeaders isNil ifTrue: [outputHeaders := OrderedCollection new].
	^outputHeaders
!

setFromWSDLOperation: aWSDLOperation
	soapAction := aWSDLOperation soapAction.
	self inputBody: aWSDLOperation createInputBody.
	self outputBody: aWSDLOperation createOutputBody.
	self inputHeaders: aWSDLOperation createInputHeaders.
	self outputHeaders: aWSDLOperation createOutputHeaders
! !

!SpmdOperation methodsFor:'asserting'!

assertValidIn: typespace 
	self inEnvelopeIn: typespace.
	self outEnvelopeIn: typespace
! !

!SpmdOperation methodsFor:'construction'!

addInputHeader: aHeaderBinding
	"Add the <SpmdSoapHeader>
	Note:
		soap:header SpmdMessages can be shared by multiple operations (isOwned == false),
		or belong to a single operation (isOwned == true) "

	self addBinding: aHeaderBinding to: self inputHeaders.
	^aHeaderBinding
!

addInputHeaderDeclared: aXeQName
	"Add and answer a <SpmdSoapHeader> 
	-Helper construction method, the receiver owns the header, and will be allowed to change its name."

	^self addInputHeader: (SpmdSoapHeader literalDeclared: aXeQName) beOwned
!

addOutputHeader: aHeaderBinding
	"Add the <SpmdSoapHeader>
	Note:
		soap:header SpmdMessages can be shared by multiple operations (isOwned == false),
		or belong to a single operation (isOwned == true) "

	self addBinding: aHeaderBinding to: self outputHeaders.
	^aHeaderBinding
!

addOutputHeaderDeclared: aXeQName
	"Add and answer a <SpmdSoapHeader> 
	-Helper construction method, the receiver owns the header, and will be allowed to change its name."

	^self addOutputHeader: (SpmdSoapHeader literalDeclared: aXeQName) beOwned
!

inputBody: aSpmdSoapBody
	aSpmdSoapBody operationName: self name.
	inputBody := aSpmdSoapBody
!

linkHeader: anInputSoapHeader to: anOutputSoapHeader selector: aSymbol
	"Add an input and output <SpmdSoapHeader> and link them.
	-The handlers that will be created (for use by a server) will use the input to create a Smalltalk MessageSend, and will encode the answer using the output"

	self addBinding: anInputSoapHeader to: self inputHeaders.
	self addBinding: anOutputSoapHeader to: self outputHeaders.
	anInputSoapHeader linkOutput: anOutputSoapHeader.
	anInputSoapHeader docSelector: aSymbol
!

outputBody: aSpmdSoapBody
	aSpmdSoapBody operationName: self name.
	outputBody := aSpmdSoapBody
! !

!SpmdOperation methodsFor:'construction-private'!

addBinding: aBinding to: aCollection
	aBinding operationName: self name.
	aCollection add: aBinding.
	self renameMessages
!

inputHeaders: aCollection
	inputHeaders := OrderedCollection new.
	aCollection do: [:each | self addInputHeader: each]
!

outputHeaders: aCollection
	outputHeaders := OrderedCollection new.
	aCollection do: [:each | self addOutputHeader: each]
! !

!SpmdOperation methodsFor:'defaults-private'!

defaultSoapAction
        Smalltalk isSmalltalkX ifTrue:[
            ^ self name printString
        ].
        ^self name displayString
! !

!SpmdOperation methodsFor:'enumerating-private'!

allBlockBindingsDo: aBlockClosure
	aBlockClosure value: inputBody.
	aBlockClosure value: outputBody.
	self inputHeaders do: aBlockClosure.
	self outputHeaders do: aBlockClosure
! !

!SpmdOperation methodsFor:'operations-private'!

renameMessages
        "Private - Only requirement is to generate unique names"

        self allBlockBindingsDo: [ :each | each operationName: self name].
        self inputBody renameMessageAsInput.
        self outputBody renameMessageAsOutput.
        self inputHeaders , self outputHeaders keysAndValuesDo: [ :i :each |
            Smalltalk isSmalltalkX ifTrue:[
                each renameMessageAppending: 'Header' , i printString
            ] ifFalse:[
                each renameMessageAppending: 'Header' , i displayString
            ]
        ]
! !

!SpmdOperation methodsFor:'printing'!

printOn:aStream
    super printOn:aStream.
    aStream nextPut:$(.
    name printOn:aStream.
    aStream nextPut:$).
!

putSpraySignatureOn: aWriteStream
	"Put a readable description string on the stream for documentation purposes"

	aWriteStream
		display: name;
		space.
	inputBody putSpraySignatureOn: aWriteStream
! !

!SpmdOperation methodsFor:'testing'!

isDOC
	^false
!

isEncoded
	^inputBody isEncoded
!

isRPC
	^false
! !

!SpmdOperation methodsFor:'wire creation-coder-private'!

bodyInputCodersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBCoder>s from the receivers description of the soap:Body
	Note: in rpc style, (and typically in document style) there is only 1 	<SpwSoapBCoder>.
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	^self subclassResponsibility
!

bodyOutputCodersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBCoder>s from the receivers description of the soap:Body
	Note: in rpc style, (and typically in document style) there is only 1 <SpwSoapBCoder>
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml"

	^self subclassResponsibility
!

headerInputCodersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBCoder>s from the receivers description of the soap:Header
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml.
	Note - Headers are always document style."

	^self inputHeaders collect: [:each | each docCoderIn: aXeTypespace]
!

headerOutputCodersIn: aXeTypespace
	"Create and answer a collection of <SpwSoapBCoder>s from the receivers description of the soap:Header
	The <SpwSoapBCoder> is used by both clients and servers to encode/decode objects/xml,
	Note - Headers are always document style."

	^self outputHeaders collect: [:each | each docCoderIn: aXeTypespace]
! !

!SpmdOperation methodsFor:'wire creation-envelopes'!

inEnvelopeOf: anInEnvelopeClass in: aXeTypespace
	"Answer a <SpwInEnvelope>.
	Note: Instead of using the style of operation to determine the <SpwInEnvelope> subclass, 
		it has become necessary to allow the <SprayClient> to override the style, so that DOTNET services can be accessed using RPC semantics.
	The <SprayClient> methods #send:/#send:with: etc construct  'rpc' envelopes, while the #sendDoc:/#sendDoc:with:  construct 'document' envelopes"

	^(anInEnvelopeClass newIn: aXeTypespace) 
		bodyCoders: (self bodyOutputCodersIn: aXeTypespace)
		headerCoders: (self headerOutputCodersIn: aXeTypespace)
!

outEnvelopeOf: anOutEnvelopeClass in: aXeTypespace
	"Answer a <SpwOutEnvelope>
	Note: Instead of using the style of operation to determine the <SpwInEnvelope> subclass, 
		it has become necessary to allow the <SprayClient> to override the style, so that DOTNET services can be accessed using RPC semantics.
	The <SprayClient> methods #send:/#send:with: etc construct  'rpc' envelopes, while the #sendDoc:/#sendDoc:with:  construct 'document' envelopes"

	^(anOutEnvelopeClass newIn: aXeTypespace) 
		bodyCoders: (self bodyInputCodersIn: aXeTypespace)
		headerCoders: (self headerInputCodersIn: aXeTypespace)
! !

!SpmdOperation methodsFor:'wire creation-envelopes-private'!

inEnvelopeIn: aXeTypespace
	"Private - Answer a <SpwInEnvelope>."

	^self inEnvelopeOf: self inEnvelopeClass in: aXeTypespace
!

outEnvelopeIn: aXeTypespace
	"Private - Answer a <SpwOutEnvelope>"

	^self outEnvelopeOf: self outEnvelopeClass in: aXeTypespace
! !

!SpmdOperation methodsFor:'wire creation-handler'!

allHandlersIn: aXeTypespace
	"Create and answer all the <SpwSoapBHandler>s described by the receiver"

	| handlers |
	handlers := OrderedCollection new.
	handlers addAll: (self bodyHandlersIn: aXeTypespace).
	handlers addAll: (self headerHandlersIn: aXeTypespace).
	^handlers
! !

!SpmdOperation methodsFor:'wire creation-handler-private'!

bodyHandlersIn: aXeTypespace
	"Create and answer all the <SpwSoapBHandler>s from the receiver's description of the soap:Body,
	Notes: 
		-Handlers are used by servers.
		-In rpc style, (and typically in document style)  there is 1 body handler for each operation."

	^self subclassResponsibility
!

headerHandlersIn: aXeTypespace
	"Create and answer all the <SpwSoapBHandler>s from the receiver's description of the soap:Header
	Note: handlers are used by servers"

	^self inputHeaders collect: [:each | each handlerIn: aXeTypespace]
! !

!SpmdOperation methodsFor:'wsdl-private'!

defineBinding: aBindingNode
	"Add the receiver to the WSDL binding <XeNode>
	Private - WSDL Spec: 
		An operation element within a binding specifies binding information for the operation 
		with the same name within the binding's portType. Since operation names are not 
		required to be unique (for example, in the case of overloading of method names), the 
		name attribute in the operation binding element might not be enough to uniquely identify 
		an operation. In that case, the correct operation should be identified by providing the name 
		attributes of the corresponding wsdl:input and wsdl:output elements"

	| opNode soapOpNode inputNode outputNode |
	opNode := aBindingNode addElement: 'operation'.
	opNode setAttribute: 'name' value: self name.
	"The soap:operation element provides information for the operation as a whole"
	soapOpNode := opNode addElement: 'soap:operation'.
	self soapAction notNil 
		ifTrue: [soapOpNode setAttribute: 'soapAction' value: self soapAction].
	soapOpNode setAttribute: 'style' value: self style.
	inputNode := opNode addElement: 'input'.
	self inputBody defineBinding: inputNode.
	self inputHeaders do: [:each | each defineBinding: inputNode].
	outputNode := opNode addElement: 'output'.
	self outputBody defineBinding: outputNode.
	self outputHeaders do: [:each | each defineBinding: outputNode]
!

definePortType: aNode
	"Add the receiver to the WSDL portType <XeNode>

	Private - WSDLSpec1.1 Sec2.4"

	| opNode |
	self assert: [self name notEmpty].
	opNode := aNode addElement: 'operation'.
	opNode setAttribute: 'name' value: self name.
	self inputBody message putWSDLNameOn: (opNode addElement: 'input').
	self outputBody message putWSDLNameOn: (opNode addElement: 'output').
	self documentation notEmpty 
		ifTrue: [(opNode addElement: 'documentation') addTextNode: self documentation]
! !

!SpmdOperation class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdOperation.st,v 1.3 2005/01/26 16:18:26 stefan Exp $'
! !
