"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdPort
	instanceVariableNames:'name documentation operations transportUri'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Meta'
!

SpmdPort comment:'I am a web service port. I am the metadata that describes a collection of operations that make up a web service.

My <SpmdOperation>s (which describe webservice operations) are used to create the "wire" objects that process soap envelopes and their content.

I am created either by:
	-Analyzing source code of a <SprayOperationModel>
	-Parsing a WSDL (Using the WSDL* classes)
	-Manually

Instance Variables:
	name		<String>
	documentation	<String>
	operations	<Collection> of <SpmdOperation>s
	transportUri	<String>

'
!


!SpmdPort class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'i1.ico'
    ]
! !

!SpmdPort methodsFor:'accessing'!

documentation: aString
	documentation := aString
!

name: aString
	name := aString
!

numberEncoded
	"Answer the number of encoded operations in the receiver"

	^(self operations select: [:each | each isEncoded]) size
!

numberOfOperations
	"Answer the number of operations in the receiver"

	^self operations size
!

numberRPC
	"Answer the number of rpc operations in the receiver"

	^(self operations select: [:each | each isRPC]) size
!

operationNamed: aString
	"Answer the <SpmdOperation> named aString.
		Note: There is no requirement that operations within a port are uniquely named,
		Spray answers the first"

	^self operations detect: [:each | each name = aString]
		ifNone: [self error: 'Operation not found: ' , aString]
!

operationNames
	^self operations collect: [ :each | each name]
!

transportUri: aString
	transportUri := aString
! !

!SpmdPort methodsFor:'accessing-private'!

allMessages
	^self allMessagesWithPrivate: #()

!

allMessagesWithPrivate: aCollOfOpNames
	^(self operationsWithPrivate: aCollOfOpNames)
		inject: IdentitySet new
		into: [ :acc :each | 
			acc 
				addAll: each allMessages;
				yourself]

!

bindingName
	^self name , 'Binding'
!

documentation
	documentation isNil ifTrue: [documentation := ''].
	^documentation
!

name
	"Private - Answer the name of the receiver. This has no significance except to be used in a WSDL's internal name referencing"

	^name
!

operations
	operations isNil ifTrue: [operations := OrderedCollection new].
	^operations
!

operations: aCollection
	operations := aCollection
!

operationsWithPrivate: aCollOfOpNames
	^self operations reject: [ :each | aCollOfOpNames notNil and: [aCollOfOpNames includes: each name]]
!

transportUri
	transportUri isNil ifTrue: [transportUri := 'http://schemas.xmlsoap.org/soap/http'].
	^transportUri
! !

!SpmdPort methodsFor:'adding'!

addOperation: anOperation 
	self assertUniqueName: anOperation.
	^self operations add: anOperation
! !

!SpmdPort methodsFor:'asserting'!

assertValidIn: typespace
	self operations do: [ :each | each assertValidIn: typespace].
	self assert: [(self allMessages asOrderedCollection collect: [:each | each name]) asSet size = self allMessages size].

! !

!SpmdPort methodsFor:'asserting-private'!

assertUniqueName: anOperation
	self assert: [(self operations anySatisfy: [:each | each name = anOperation name]) not]
! !

!SpmdPort methodsFor:'printing & storing'!

printOn: aStream
    super printOn:aStream.
    aStream nextPut:$(.
    name printOn:aStream.
    aStream nextPut:$).
! !

!SpmdPort methodsFor:'removing'!

keepOperationsNamed: anArray 
	operations := operations select: [ :each | anArray includes: each name ].
!

removeOperation: anOperation
	self operations remove: anOperation
!

removeOperationNamed: aString 
	| remove |
	remove := self operations select: [ :each | each name = aString].
	self operations removeAll: remove.
	
!

removeOperationWithSelector: aSymbol
	| remove |
	remove := self operations select: [ :each | each selector == aSymbol].
	self operations removeAll: remove.
	
! !

!SpmdPort methodsFor:'soap action'!

formatSoapActionWith: aWin32FormatString
	"Enumerate over the receiver's operations, setting into each a SOAPAction constructed by the formatString and the operation's name"

	self operations do: [:each | each soapAction: (aWin32FormatString formatWith: each name)]
!

soapAction: aString
	"Set the same SOAPAction into all the operations"

	self operations do: [:each | each soapAction: aString]
! !

!SpmdPort methodsFor:'testing'!

includesOperationNamed: aString
	^self operations anySatisfy: [ :each | each name = aString]
! !

!SpmdPort methodsFor:'wire creation-handler'!

allHandlersIn: aXeTypespace
	"Create and answer all the <SpwSoapBHandler>s described by the receiver.
	(Used by servers only)"

	^self operations inject: OrderedCollection new
		into: 
			[:acc :each | 
			acc
				addAll: (each allHandlersIn: aXeTypespace);
				yourself]
! !

!SpmdPort methodsFor:'wsdl'!

defineBinding: aDefinitionNode private: aCollOfOpNames
	"WSDL Spec:
		A binding defines message format and protocol details for operations and messages defined 
		by a particular portType. There may be any number of bindings for a given portType
			-The name attribute provides a unique name among all bindings defined within in the 
			enclosing WSDL document.
			-A binding references the portType that it binds using the type attribute
			-The value of the style attribute is the default for the style attribute for each contained operation. 
			If the style attribute is omitted, it is assumed to be document. See section 3.4 
			<http://www.w3.org/TR/wsdl> for more information on the semantics of style.
			-The value of the required transport attribute indicates which transport of SOAP this binding 
			corresponds to. The URI value http://schemas.xmlsoap.org/soap/http corresponds to the HTTP binding 
			in the SOAP specification. Other URIs may be used here to indicate other transports 
			(such as SMTP, FTP, etc.)."

	| bindingNode |
	bindingNode := aDefinitionNode addElement: 'binding'.
	bindingNode
		setAttribute: 'name' value: self bindingName;
		setAttribute: 'type' value: 'tns:' , self name , 'PortType'.
	(bindingNode addElement: 'soap:binding') setAttribute: 'transport'
		value: self transportUri.
	"setAttribute: 'style' value: 'rpc';"	"style=rpc|document?>"	"A binding MUST specify exactly one protocol"
	(self operationsWithPrivate: aCollOfOpNames) 
		do: [:each | each defineBinding: bindingNode]
!

defineMessages: aDefinitionNode typespace: aTypespace private: aCollOfOpNames
	"Add the receivers messages to the WSDL definitions element"

	(self allMessagesWithPrivate: aCollOfOpNames) 
		do: [:each | each defineMessage: aDefinitionNode typespace: aTypespace]
!

definePort: aServiceNode at: anAddressLocation 
	"Add the receivers information to the WSDL definitions/service element"

	| portNode |
	portNode := aServiceNode addElement: 'port'.
	portNode
		setAttribute: 'name' value: self name;
		setAttribute: 'binding' value: 'tns:' , self bindingName.
	(portNode addElement: 'soap:address') setAttribute: 'location' value: anAddressLocation.
	self documentation notEmpty 
		ifTrue: [(portNode addElement: 'documentation') addTextNode: self documentation]
!

definePortType: aDefinitionNode private: aCollOfOpNames
	"WSDLSpec1.1
		-A port type is a named set of abstract operations and the abstract messages involved
		-The port type name attribute provides a unique name among all port types 
		defined within in the enclosing WSDL document."

	| portTypeNode |
	self assert: [self name notEmpty].
	portTypeNode := aDefinitionNode addElement: 'portType'.
	portTypeNode setAttribute: 'name' value: self name , 'PortType'.
	(self operationsWithPrivate: aCollOfOpNames) 
		do: [:each | each definePortType: portTypeNode]
! !

!SpmdPort class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdPort.st,v 1.4 2010/02/26 12:02:51 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdPort.st,v 1.4 2010/02/26 12:02:51 cg Exp $'
! !
