"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpmdWSDLBuilder
	instanceVariableNames:'service ports addressLocation xeContents xeElements'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SpmdWSDLBuilder comment:'I am used to build a WSDL for a service.

==========
Instance Variables
	service		<SprayService>
	ports		<Collection> of <SpmdPort>
	addressLocation	<String> (the url of the service ... not determined until a server is running)
	xeContents	<Collection> of <XeContent>
	xeElements	<Collection> of <XeElements>

==========
Notes
Building a WSDL consists of two operations. 
-The first is to ask the SpmdPort (and its SpmdOperations etc) to define themselves.
-The second is to enumerate over all the xeContents and xeElements used by the service, These XeComponents may be from a namespace we dont have to define (like the XMLSchemas namespace), or they may be in the same namespace, or in multiple namespaces. We write a schema for each namespace (ie targeting)


'
!


!SpmdWSDLBuilder class methodsFor:'must strip'!

example
        <resource: #example>

        "Requires the 'SW BigDog' package"

        ^(self environment at: #BigDogService) new localWSDL
! !

!SpmdWSDLBuilder methodsFor:'accessing'!

addressLocation: aString
	addressLocation := aString
!

ports: anObject
	ports := anObject
!

wsdl
	"build and answer an XeDoc"

	| node |
	self ports isEmpty 
		ifTrue: [self error: 'A Service must have a least one port for a WSDL to be created'].
	node := self newDefinitionsNode.
	self
		defineTypes: node;
		definePorts: node;
		defineService: node.
	^node
!

wsdlXmlString
	"Build and answer the WSDL as an xml string"

	^self wsdl ownerDocument xml
!

xeContents: aCollOfXeContent
	xeContents := aCollOfXeContent
!

xeElements: anOrderedCollection 
	xeElements := anOrderedCollection
! !

!SpmdWSDLBuilder methodsFor:'accessing-private'!

addressLocation
	^addressLocation
!

elementsIn: aSymbol 
	"Private - select all the global XeElements used by this service in aNamespace"

	^(xeElements select: [:each | each namespace == aSymbol]) asSet asOrderedCollection
!

ports
	^ports
!

privateOperations
	"Private - answer a collection of operations that should not be added to the WSDL"

	^service privateOperations
!

service: aSprayService
	service := aSprayService
!

serviceDocumentation
	^service documentation
!

serviceName 
	^service name
!

serviceNamespace
	^service namespace
!

targetNamespaces
	"Private - Answer a collection of all namespaces that we must write a schema.
	-sort for unitTest purposes"

	^(self xeElementNamespaces , self xeContentNamespaces) asSet asSortedCollection asOrderedCollection
		select: [:each | (self isNamespaceImported: each) not]
!

typesIn: aNamespace
	"Private - Enumerate all XeContent and extract the set of namespaces for this port.
	-sort then ordered to give tests some stability."

	^(xeContents select: [:each | each isTargetNS: aNamespace]) asSet asOrderedCollection
!

typespace
	^service typespace
!

xeContentNamespaces
	"Private - Enumerate all XeContent instances and extract the set of namespaces for this port.
	-sort then ordered to give tests some stability."

	^(xeContents collect: [:each | each isNamed ifTrue: [each namespace] ifFalse: []]) asSet 
		asSortedCollection asOrderedCollection
!

xeElementNamespaces
	"Private - Enumerate all XeElements and extract the set of namespaces for this port.
	sort then ordered to give tests some stability."

	^(xeElements collect: 
			[:each | 
			(each isFormQualified and: [each qName isNullNamespace not]) ifTrue: [each namespace] ifFalse: []]) 
		asSet asSortedCollection 
		asOrderedCollection
! !

!SpmdWSDLBuilder methodsFor:'helpers-private'!

putXSDDefaultNSOn: node
	^self typespace putXSDDefaultNSOn: node
!

putXSINamespaceOn: node
	^self typespace putXSINamespaceOn: node
! !

!SpmdWSDLBuilder methodsFor:'testing-private'!

isNamespaceImported: aSymbol
	"Private - answer if the namespace should be imported, ie we do not need to provide a schema targeting it"

	^service importOnlySchemaNamespaces  includes: aSymbol
! !

!SpmdWSDLBuilder methodsFor:'wsdl-private'!

definePorts: aDefinitionNode 
	"Private - write the WSDL into aDefinitionNode
	-definition order is important"

	self ports do: [:eachP | eachP defineMessages: aDefinitionNode typespace: self typespace private: self privateOperations].
	self ports do: [:each | each definePortType: aDefinitionNode private: self privateOperations].
	self ports do: [:each | each defineBinding: aDefinitionNode private: self privateOperations]
!

defineSchema: typesNode for: aNSSymbol 
	"Private - write the schema targeting aNSSymbol"

	| externalTypes externalElements schemaWriter |
	externalTypes := self typesIn: aNSSymbol .
	externalElements := self elementsIn: aNSSymbol .
	externalTypes isEmpty & externalElements isEmpty 
		ifFalse: 
			[schemaWriter := XeSchemaWriter  
				targeting: aNSSymbol
				node: typesNode
				schemaNS: self typespace xsdNamespace.
			service useEFDQSchemas ifTrue: [schemaWriter beElementFormQualified].
			externalTypes do: [:eachType | schemaWriter define: eachType].
			externalElements do: [:eachElement | schemaWriter declare: eachElement]].
!

defineService: aDefinitionXeNode
	| serviceNode |
	serviceNode := aDefinitionXeNode addElement: 'service'.
	serviceNode setAttribute: 'name' value: self serviceName.
	self serviceDocumentation notEmpty 
		ifTrue: [(serviceNode addElement: 'documentation') addTextNode: self serviceDocumentation].
	self ports do: [:each | each definePort: serviceNode at: addressLocation].
	^serviceNode
!

defineTypes: aXeNode
	"Private - write the schemas for our namespaces"

	| typesNode |
	typesNode := aXeNode addElement: 'types'.
	self targetNamespaces do: [:each | self defineSchema: typesNode for: each]
!

newDefinitionsNode
        "Private - The Spmd* instances rely on the prefix declarations made in this method"

        | node |
        node := YAXO::XeDoc new addElement: 'definitions'.
        node
                setAttribute: 'name' value: self serviceName;
                setAttribute: 'targetNamespace' value: self serviceNamespace;
                defaultNS: #'http://schemas.xmlsoap.org/wsdl/';
                addNSPrefix: 'soap' to: #'http://schemas.xmlsoap.org/wsdl/soap/';
                addNSPrefix: 'tns' to: self serviceNamespace.
        self putXSINamespaceOn: node.
        ^node
! !

!SpmdWSDLBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpmdWSDLBuilder.st,v 1.3 2008/09/08 10:35:58 cg Exp $'
! !
