"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SprayExchangeFactory subclass:#SprayClient
	instanceVariableNames:'typespace operationPort messageQPs'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayClient comment:'I am a factory to create exchanges <SpraySoapSend>s. I have access to the metadata of a service, and I use that information to construct  the objects that will send a soap request and decode the soap response.

Qualified Properties can be added to me, and I will add them to each <SpraySoapSend> that I create. See the SptPropertyScope class comments for more information about qualified properties.

See examples 
See #showHelp

'
!


!SprayClient class methodsFor:'instance creation'!

new
	^self shouldNotImplement
!

newIn: aTypespace
	^super new initializeIn: aTypespace
! !

!SprayClient class methodsFor:'constants'!

defaultMessageQProperties
	"This can be used to create subclasses that will always add certain qualified properties to the <SpwRequestingExchange>s it creates"

	^#()
! !

!SprayClient class methodsFor:'must strip'!

exampleCollection
        "The following example is an operation from the base SoapBuilders interop-lab service. For the full list of implementations see <http://www.whitemesa.com/interop.htm>."
        <resource: #example>

        | exchange client |
        client := (SprayWSDLService 
                                onUrl: 'http://www.dolphinharbor.org/services/interop2001/service.wsdl') createClient.
        exchange := client send: 'echoIntegerArray' with: #(1 2 3).
        ^exchange value
!

exampleCreateFromLocal
        "Requires 'Steve Waring\Spray WebServices\Interop\ServerInterop\Camp Smalltalk3\SW Interop CS3.pac' package.
        In the following example, a local service is used to create the client. "

        <resource: #example>

        | service client |
        service := (self environment at: #CS3InteropService) new.
        client := service createClientTo: 'http://www.dolphinharbor.org/services/cs3'.
        ^(client send: 'pingService' with: 'Hi From Spray!!') value
!

exampleCreateFromWSDL
        "This service is deployed on dolphinharbor and can be accessed with a client created from the dolphinharbor WSDL. This example does not require the 'SW Interop CS3' package to be installed. "

        <resource: #example>

        | service client |
        service := SprayWSDLService 
                                onUrl: 'http://www.dolphinharbor.org/services/cs3/service.wsdl'.
        client := service createClient.
        ^(client send: 'pingService' with: 'Hi again!!') value
!

exampleFault
        "Requires 'Steve Waring\Spray WebServices\Services\Temperatures\SW WS Temperatures.pac' package.
        A SoapSendFault exception is signaled when a server returns an Envelope with a fault message. The exception can be accessed for the faultCode faultDetails and faultString. The faultString is the description of the exception.

        The Temperatures service has an operation that will generate a fault if nil is sent as an argument. "

        <resource: #example>

        | service client soapSend |
        service := (self environment at: #TemperaturesService) new.
        client := service createClientTo: 'http://www.dolphinharbor.org/services/temper'.
        soapSend := client 
                                send: 'compare'
                                with: 21.3
                                with: nil.
        [soapSend value] on: SoapSendFault
                do: [:e | self halt:'debugHalt'. '%1, %2' formatWith: e faultCode displayString with: e faultString]
!

exampleInfo
        "The SprayClient can be asked for information about operations. While the WSDL is the best source of information, this can serve as a quick guide. ."

        <resource: #example>

        | client |
        client := (SprayWSDLService 
                                onUrl: 'http://www.dolphinharbor.org/services/cs3/service.wsdl') createClient.
        client operationNames.  "an OrderedCollection('pingService')"
        client signatureFor: 'pingService'.     "pingService ((inMsgText xsd:string))"
        client := (SprayWSDLService 
                                onUrl: 'http://www.mssoapinterop.org/remoting/ServiceA.soap?wsdl') createClient.
        client operationNames.  "an OrderedCollection('echoString' 'echoBoolean' 'echoInteger' 'echoFloat' 'echoStruct' 'echoVoid' 'echoStringArray' 'echoIntegerArray' 'echoFloatArray' 'echoStructArray' 'echoDate' 'echoBase64' 'echoDecimal')"
        client signatureFor: 'echoStruct'.      "echoStruct ((inputStruct ns2:SOAPStruct))"
        client := (SprayWSDLService onUrl: 'http://ws.xara.com/graphicrender/render3d.wsdl') 
                                createClient.
        client operationNames.  "an OrderedCollection('RenderURL' 'GetTemplates' 'GetFonts' 'GetExportTypes')"
        client signatureFor: 'RenderURL'        "'RenderURL ((Template xsd:string) (Text xsd:string) (TextColor xsd:string) (BGColor xsd:string) (Font xsd:string) (FontSize xsd:string) (ExportType xsd:string))'"
!

exampleSOAPAction
        "SOAPAction is an HTTP header sent in an HTTPRequest. It is possible that it's use will either be deprecated or eliminated from future Soap specifications.
        The client created from a service uses the soapAction defined by the service's interface, therefore the correct soapAction will always be sent. 
        It is possible to override the WSDL soapAction by setting a nonNil soapAction into the client. 
        In the following workspace, the MS Server will probably respond with a fault, while a Spray server will ignore the soapAction. "

        <resource: #example>

        | service client |
        service := SprayWSDLService onUrl: 'http://mssoapinterop.org/stk/wsdl/EmptySA.wsdl'.
        client := service createClient.
        client soapAction: 'notEmpty'.
        (client send: 'echoString' with: 'abc') value   "Should signal Fault"
!

exampleStruct
        "The following example is an operation from the base SoapBuilders interop-lab service. For the full list of implementations see <http://www.whitemesa.com/interop.htm>."

        <resource: #example>

        | exchange client struct |
        client := (SprayWSDLService 
                                onUrl: 'http://www.dolphinharbor.org/services/interop2001/service.wsdl') createClient.
        (struct := XeQStruct new)
                at: #varInt put: 123;
                at: #varFloat put: 3.14;
                at: #varString put: 'abc'.
        exchange := client send: 'echoStruct' with: struct.
        ^exchange value
! !

!SprayClient methodsFor:'accessing'!

atMessageQP: aName put: anObject
	"Set the qualified property (which will be added to each <SpwSoapSend> created"

	messageQPs at: aName put: anObject
!

httpRequestSoapBinding: aBinding
	"Set the <SptSoapBinding> to be used to process <SpwSoapSends> created.
	Note: the SptEnvironment is typically configured to use  a SptHTTPBinding, setting this will override the default only for <SpwSoapSends> created from this receiver"

	self atMessageQP: (SptPropertyScope sprayTransQ: 'httpRequestSoapBinding') put: aBinding
!

immediateDestination
	^self atMessageQP: #'transport:ImmediateDestination'
!

immediateDestination: aUrlString
	"Set the immediateDestination (ie the endPoint url). 
	-This will be used for all <SpwSoapSends> created.
	-This is automatically set from a service's interface (WSDL), but this can be used to override and send the messages to an alternate url"

	self atMessageQP: #'transport:ImmediateDestination' put: aUrlString
!

operationNames
	^operationPort operationNames
!

soapAction: aString
	"Set the soapAction to aStringOrNil.
	-If nil (or not set), the operations default soapAction will be used. 
	-If not nil, aStringOrNil will override the default"

	self atMessageQP: 'action:SOAPActionURI' put: aString
! !

!SprayClient methodsFor:'accessing-private'!

atMessageQP: aName
	^messageQPs at: aName
!

operationNamed: aString 
	"Private - Answer a SpmdOperation
	Note: there is no requirement for operations to be uniquely named. 
	Future versions will need to locate the correct operation by name and arguments"

	^operationPort operationNamed: aString
!

operationPort
	"Private - Answer an SpmdPort"

	^operationPort
!

port: anOperationPort 
	operationPort := anOperationPort.
!

typespace
	^typespace
! !

!SprayClient methodsFor:'initializing-private'!

initialize
	super initialize.
	messageQPs := SptRequestingExchange newQProperties.
!

initializeIn: aXeTypespace
	typespace := aXeTypespace.
	self initialize.
! !

!SprayClient methodsFor:'modes'!

beLongTimeouts
	"Note: the SptHTTPRequest only uses the last timeout (and the actual time to timeout is between 1-2 times the set value)"

	 self setTimeouts: #(300000 300000 60000 30000)
!

beNormalTimeouts
	"Note: the SptHTTPRequest only uses the last timeout (and the actual time to timeout is between 1-2 times the set value)"

	 self setTimeouts: #(30000 30000 60000 30000)
!

beQuickTimeouts
	"Note: the SptHTTPRequest only uses the last timeout (and the actual time to timeout is between 1-2 times the set value)"

	 self setTimeouts: #(10000 10000 10000 10000)
!

setTimeouts: anArray
	self atMessageQP: (SptPropertyScope sprayHttpQ: 'timeoutReceive') put: anArray first.
	self atMessageQP: (SptPropertyScope sprayHttpQ: 'timeoutSend') put: anArray second.
	self atMessageQP: (SptPropertyScope sprayHttpQ: 'timeoutConnect') put: anArray third.
	self atMessageQP: (SptPropertyScope sprayHttpQ: 'timeoutResolve') put: anArray fourth.
! !

!SprayClient methodsFor:'operations'!

send: opNameString
	"Answer a SpwSoapSend.
	Note: this does not  actually 'send' anything, but answers an <SpwRequestingExchange> that  can be processed to 'send' and 'receive' soap messages"

	^self createRPCSoapSendFor: (self operationNamed: opNameString)
!

send: opNameString withArguments: aCollection 
	"Answer a SpwSoapSend which can be used to invoke the operation opNameString.
	The arguments are encoded as a parameters. (the parameter names are found in the operation's XMLElement)"

	^(self send: opNameString) arguments: aCollection
!

sendDoc: opNameString
	"Answer a SpwSoapSend that contains  'document' style SpwEnvelopes.
	-Each argument is encoded directly as a child of soap:Body
	Note: This is not used for most DOTNET document operations because DOTNET's default document operations have the same form as RPC operations. "

	^self createDOCSoapSendFor: (self operationNamed: opNameString)
!

sendDoc: opNameString arguments: aCollection
	"See #sendDoc: comment"

	^(self sendDoc: opNameString) arguments: aCollection
!

sendDoc: opNameString with: anObject
	"See #sendDoc: comment"

	^self sendDoc: opNameString arguments: (Array with: anObject)
!

sendDoc: opNameString with: anObject with: anObject2
	"See #sendDoc: comment"

	^self sendDoc: opNameString arguments: (Array with: anObject with: anObject2)
!

signatureFor: anOpName
	^(self operationNamed: anOpName) spraySignature
! !

!SprayClient methodsFor:'operations-private'!

createDOCSoapSendFor: aSpmdOperation 
	"Private - 
	NOTE: This method overrides the style of aSpmdOperation and creates a send with DOC semantics."

	^(self newSoapSendFor: aSpmdOperation)
		outEnvelope: (aSpmdOperation outEnvelopeOf: SpwOutDocument in: typespace);  
		inEnvelope: (aSpmdOperation inEnvelopeOf: SpwInDocument in: typespace); 
		yourself
!

createRPCSoapSendFor: aSpmdOperation 
	"Private - Answer a SpwSoapSend.
	NOTE: This method overrides the style of aSpmdOperation and creates a send with RPC semantics.
	-the outEnvelope allows objects to be collected (which may be encoded in the body or header) before creating a SoapOutEnvelope and sending it. This object is not strictly necessary for simple RPC soap, but is needed to construct more complex envelopes
	-the inEnvelope allows all objects that are decoded from an incoming message to be accessed. In a typical RPC operation the first parameters is answered as the 'return'. The objectInEnvelope allows further access to other out parameters and objects decoded from the headers"

	^(self newSoapSendFor: aSpmdOperation)
		outEnvelope: (aSpmdOperation outEnvelopeOf: SpwRPC in: typespace);  
		inEnvelope: (aSpmdOperation inEnvelopeOf: SpwRPCReturn in: typespace); 
		yourself
!

newSoapSendFor: anOperation
	"Private - answer a <SpwSoapSend>"

	| soapSend |
	soapSend := SpraySoapSend new.
	soapSend soapAction: anOperation soapAction.
	soapSend addAllQPs: messageQPs.
	^soapSend
! !

!SprayClient methodsFor:'testing'!

includesOperationNamed: aString 
	"Answer a Boolean
	Note: there is no requirement for operations to be uniquely named. 
	Future versions will need to locate the correct operation by name and arguments"

	^operationPort includesOperationNamed: aString
! !

!SprayClient class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayClient.st,v 1.4 2008/09/05 17:47:01 cg Exp $'
! !
