"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SprayService
	instanceVariableNames:'typespace'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayService comment:'I am the interface of a web service.

I can construct <SprayClient>s that are used to access operations defined by my interface.

==========
Example (using a local service)

	service := CS3InteropService new.
	client :=  service createClientTo: ''http://www.dolphinharbor.org/services/cs3''.
	(client send: ''ping'' with: ''hi dolphinharbor'') value

==========
Instance Variables:
	typespace		<XeTypespace>

'
!


!SprayService class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'service.ico'
    ]
! !

!SprayService methodsFor:'accessing'!

typespace
	"Answer the <XeTypespace> of the service"

	typespace isNil ifTrue: [self setTypespace].
	^typespace
!

typespaceOptions
	"Answer the typespace's <XeTypespaceOptions>"

	^typespace options
! !

!SprayService methodsFor:'accessing-port-private'!

operationPort
	"Answer a SpmdPort. The operationPort contains the objects that model the interface of the service" 

	^self subclassResponsibility
! !

!SprayService methodsFor:'accessing-service'!

destinationUrl
	"Answer a <String>. (This is used by clients only as servers are informed of their location by their HTTPBinding)
	A service can implement this so that clients have a default immediateDestination"

	^self subclassResponsibility
! !

!SprayService methodsFor:'client server creation'!

createClient
    "Answer a SprayClient which can be used to construct soap exchanges <SpwSoapSend>s"
    
    |destinationURL|

    destinationURL := self destinationUrl.
    (destinationURL isEmptyOrNil) ifTrue:[
        self error:'No default immediateDestination, use #createClientTo:'
    ].
    ^ self createClientTo:destinationURL

    "Modified: / 13-07-2011 / 10:28:22 / cg"
    "Modified (format): / 13-07-2011 / 20:01:29 / cg"
!

createClientTo: aDestinationUrl
	"Answer a SprayClient which can be used to construct soap exchanges <SpraySoapSend>s"

	^(SprayClient newIn: self typespace)
		port: self operationPort;
		immediateDestination: aDestinationUrl;
		yourself
! !

!SprayService methodsFor:'enumerating'!

doAdditionalTypes: operation
	"Used in creation of WSDL. If a service has additional types it wants to define in a Schema, then it should implement this method and enumerate them"
	
! !

!SprayService methodsFor:'event handling-private'!

onTypespaceCreated
	"Private - subclasses should implement to modify the typespace.
	Typical modifications:"

	"This modifies the typespace so that the service's classes encode/decode into the schemaNS using a <XeSprayClassFactory>
	(by default  'XeNamespace sprayClass' is used) "

	"This modifies the typespace by mapping the complexType typeNames to alternate classNames. 
	( by default the class name is assumed to be the same as the local part of the typeName)"

	"This modifies the typespace by including the XMLSchema created from the xml string, at it's targetNamespace in the typespace. 
	-The definitions and declarations can then be used in the service (ie in the operationModelClasses methods, or as part of other definitions)"

	"This modifies the typespace by adding a new <XeSprayClassFactory> at the targetNamespace.
	The XMLSchema types in this namespace will encode/decode into instances of classes named by the type's local name"

	"This modifies the typespace by telling the <XeSchema> targeting the namespace to use an alternate <XeContentFactory> for all the localNames.
	This is useful for selectively choosing types in the schema to encode/decode as instances of pre-prepared classes"

	"This modifies the typespace by adding element declarations for the typeNames.
	(by default an element will be declared with the same name as the typeName ... this is useful when implementing dotnet defined services)"

	"This modifies the typespace by adding a prefix to namespace declaration (to be used by Spray's late-binding naming system).
	This enables the convenience of being able to use the prefixes in an OperationModel's methods"

	"This modifies the typespace by changing a typespace option <XeTypespaceOption>. This options resumes XeValueRangeError exceptions,
	and will decode the value as an instance of <XeAnyValue> (typically only signalled  for  NaN,INF and-INF"

	"This modifies the typespace by telling the <XeSchema> targeting the namespace, to use a <XeSprayClassFactory> for  the listed localNames only.
	-This is useful for selectively choosing types in the schema to encode/decode as instances of pre-prepared classes"

	"This modifies the typespace by extracting the schema's from the external WSDL, and adding them to the typespace.
	-The definitions and declarations can then be used in the service (ie in the OperationClasses methods, or as part of other definitions)"

	"This modifies the typespace by changing the namespace used to identify classes that a <XeSprayClassFactory> is responsible for.
	-(It changes the prefix-to-namespace declaration of 'd' (which is effective because of Spray's late-binding naming system))."

	"This modifies the classFactory to be elementFormDefault='qualified' (which is equivalent to the XMLSchema attribute switch). 
	-This tells the factory that local elements should be qualified. 
	-Not used in soap-enc, but common for DOTNET services"

	
! !

!SprayService methodsFor:'helpers-private'!

schemaQ: aString
	"Helper method"

	^XeQName local: aString ns: self schemaNS
	
! !

!SprayService methodsFor:'typespace creation-private'!

createTypespace
	"Create and answer the typespace of the service.
	-This answers the default soap-enc/spray typespace.
	(subclasses can implement this method if they need a 1999 schema based typespace)"

	^XeTypespace defaultSoapEnc2001
!

setTypespace
	"Private - subclasses should implement #onTypespaceCreated  if they need to modify the typespace"

	typespace := self createTypespace.
	self onTypespaceCreated
! !

!SprayService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayService.st,v 1.4 2011/07/13 18:01:41 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SprayService.st,v 1.4 2011/07/13 18:01:41 cg Exp $'
! !
