"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptRequestingExchange subclass:#SpraySoapSend
	instanceVariableNames:'outEnvelope inEnvelope'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Client'
!

SpraySoapSend comment:'I am a requesting message exchange. I also contain objects <SpwEnvelope>s that can be used to build and decode my in and out soapEnvelopes.

I am constructed by a <SprayClient> which in turn constructs my <SpwEnvelope>s from the WSDL description of the operation.

See #showHelp for examples
See SprayClient class examples

==========
Exceptions

Most errors during encoding and decoding, are signaled as Errors, however there are a number of exceptions that can be resumed. 

If a response is received from a server, and an error is signaled during the decoding of the response, there are a number of ways a value can be extracted. 

	Sending #extractResuming to a soapSend will decode the SoapInEnvelope, resuming 
		the following exceptions; XeTypeMismatch, XeSchemaNotFound, 
		SoapBlockNameMismatch. This is effective if the form of the xml received 
		is similar to what was expected. 
	Sending #extractForcing to a soapSend will decode the SoapInEnvelope in a new empty
		typespace which will result in <XeGuessContent>s being used to decode each 
		element. All simple content will be converted to strings, complexContent will be
		converted to XeQStructs. 
	Sending #inXmlOrEmpty  will answer the xml string of the response. 
'
!


!SpraySoapSend class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'soapSend.ico'
    ]
! !

!SpraySoapSend methodsFor:'accessing'!

out
	"Answer a SpwSoapOutEnvelope"

	out isNil ifTrue: [out := self soapOutEnvelope].
	^out
! !

!SpraySoapSend methodsFor:'accessing-object envelopes'!

inEnvelope
	"Answer a <SpwInEnvelope>, which contains the incoming decoded objects.
	When a SoapInEnvelope arrives, the <SpwInEnvelope> is used to convert the xml into objects.
	-The receiver contains a number of helper methods to access these objects, 
		but the inEnvelope can be accessed directly for full access"

	^inEnvelope
!

outEnvelope
	"Answer a <SpwOutEnvelope>, which contains the outgoing objects.
	a <SpwOutEnvelope>allows objects to be collected (which may be encoded in the body or header) before creating a SoapOutEnvelope and sending it. 
	(This object is not strictly necessary for simple RPC soap, but is needed to construct more complex envelopes)"

	^outEnvelope
! !

!SpraySoapSend methodsFor:'accessing-object envelopes-private'!

inEnvelope: aSpwInEnvelope
	inEnvelope := aSpwInEnvelope
!

outEnvelope: aSpwOutEnvelope
	outEnvelope := aSpwOutEnvelope
! !

!SpraySoapSend methodsFor:'accessing-soap envelopes'!

soapOutEnvelope
	^self outEnvelope soapOutEnvelope
! !

!SpraySoapSend methodsFor:'accessing-values'!

arguments: anArray
	"Sets the arguments for the soapSend. Arguments are matched to parameter names.
		-For RPC the arguments become the parameters of the RPCWrapper .
		-For DOC the arguments are encoded directly below the soap:Body node "

	outEnvelope arguments: anArray
!

outParameters
	"Answer a keyed collection of all out parameters received. (convention is to answer first as the 'return')."

	^inEnvelope outParameters
! !

!SpraySoapSend methodsFor:'evaluating'!

extractForcing
	"If an error occurred during the decoding of a SoapInEnvelope, this method can be used to attempt to
	extract the object from it. First attempt should be made with #valueGuessing which uses the existing <XeElement>. 
	This method creates a new typespace and decodes the received XML in that.
	-As a last resort, the raw XML can be accessed with #inXmlOrEmpty"

	in isNil ifTrue: [self error: 'Can not use extractForcing;  Response not received'].
	inEnvelope forceInSoap: in.
	^inEnvelope extractReturn
!

extractResuming
	"If an error occurred during the decoding of a SoapInEnvelope, this method can be used to attempt to
	extract the object from it.
	-it will decode the SoapInEnvelope, resuming the following exceptions; XeTypeMismatch, XeSchemaNotFound, SoapBlockNameMismatch. This is effective if the form of the xml received is similar to what was expected. "

	in isNil ifTrue: [self error: 'Can not use extractResuming;  Response not received'].
	inEnvelope resumeInSoap: in.
	^inEnvelope extractReturn
!

extractReturn
	^inEnvelope extractReturn
!

return
	^inEnvelope extractReturn
!

value
	"Send the request, and answer  the 'return' value. 
	(If the operation has multiple out parameters, they can be accessed using  #outParameters).
	-A SoapSendFault exception is signaled if the server responds with a soap:Fault"

	^self valueForOut: self soapOutEnvelope
!

valueForXml: xmlString
	^self valueForOut: (SpwSoapInEnvelope fromXmlString: xmlString)
!

valueForcing
	"Name changed to reflect that unlike #value, the server is not contacted during the processing of this message"

	Notification deprecated.
	^self extractForcing
!

valueResuming
	"Name changed to reflect that unlike #value, the server is not contacted during the processing of this message"

	Notification deprecated.
	^self extractResuming
! !

!SpraySoapSend methodsFor:'evaluating-private'!

decode
	self assertSuccessState.
	self assertNotFault.
	self decodeSoapInEnvelope
!

valueForOut: aSoapOutEnvelope
	self out: aSoapOutEnvelope.
	self processUsingDefault.
	self decode.
	^self extractReturn
! !

!SpraySoapSend methodsFor:'operations'!

decodeSoapInHeaders
	"Decode the headers only. 
	-Use this if the response body is a Fault, but you still need to access the headers"

	inEnvelope decodeHeaders: in
! !

!SpraySoapSend methodsFor:'operations-private'!

decodeSoapInEnvelope
	inEnvelope decodeSoap: in
! !

!SpraySoapSend class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpraySoapSend.st,v 1.4 2010/02/26 12:02:24 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpraySoapSend.st,v 1.4 2010/02/26 12:02:24 cg Exp $'
! !
