"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SptExchangeQueue
	instanceVariableNames:'binding queue processing processingAvailable queueLoop'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptExchangeQueue comment:'I am a queue of exchanges waiting to be processed. I process ''waitingSize'' exchanges simultaneously. When an exchange has been processed I trigger the #processed: event with the exchange as the argument.

Exchanges can be added at the end of the queue with #queueLast: or at the front with: #queueFirst:

See SplashRPCRequest(class)>>exampleQueue in the "SW Splash" package
'
!


!SptExchangeQueue class methodsFor:'instance creation'!

new
	^self newProcessing: self defaultProcessing
!

newProcessing: anInteger 
	"Answer an instance that will concurrently process anInteger exchanges (and queue the rest)"

	^self basicNew initializeFor: anInteger
! !

!SptExchangeQueue class methodsFor:'constants'!

defaultProcessing
	"To be effective, I think we need to search the queue for requests to other hosts"

	^3
!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'queue.ico'
    ]
! !

!SptExchangeQueue methodsFor:'accessing'!

binding: anSptSoapBinding
	binding := anSptSoapBinding
!

processingSize
	"for status information only
	-processing exchanges have started processing but are waiting for a response from the server"

	^processing size
!

queueSize
	"For status information only.
	-The queueSize is the number of exchanges that have not entered into the 'processing' state"

	^queue size
!

totalSize
	"for status information only.
	-queued and processing"

	^self queueSize + self processingSize
! !

!SptExchangeQueue methodsFor:'accessing-private'!

binding
	binding isNil ifTrue: [binding := SptEnvironment current httpRequestSoapBinding].
	^binding
!

queue
	^queue
! !

!SptExchangeQueue methodsFor:'asserting'!

assertQueueLoopActive
	queueLoop isNil ifTrue: [self error: 'Queue has been terminated']
! !

!SptExchangeQueue methodsFor:'event handling-private'!

onProcessedExchange: anExchange 
	SessionManager current inputState queueDeferredAction: [self trigger: #processed: with: anExchange]
! !

!SptExchangeQueue methodsFor:'initializing-private'!

initializeFor: maxProcessing
        "Private - Note: this is an experimental new class ... use with caution.
        -See other comments for ideas"

        processing := Dolphin::SharedSet new.
        processingAvailable := Semaphore new.
        maxProcessing timesRepeat: [processingAvailable signal].
        queue := SharedQueue new.
        queueLoop := [[self processQueue] repeat] forkAt: Processor userBackgroundPriority
! !

!SptExchangeQueue methodsFor:'operations'!

close
        queueLoop notNil ifTrue: [queueLoop terminate].
        queueLoop := nil.
        processing do: [:each | each notNil ifTrue: [each terminate]].
        processing := Dolphin::SharedSet new
! !

!SptExchangeQueue methodsFor:'operations-private'!

processExchange: anExchange
	Processor yield.
	"should never signal an exception"
	self binding processExchange: anExchange.
	processing remove: Processor activeProcess.
	"other options? another queue?"
	[self onProcessedExchange: anExchange] ensure: [processingAvailable signal]
!

processQueue
	"This would be much more efficient if it could detect in the queue exchanges to servers 
	other than in the processing collection, and do them out of turn"

	| nextExchange |
	processingAvailable wait.
	nextExchange := queue next.
	processing 
		add: ([self processExchange: nextExchange] forkAt: Processor userBackgroundPriority)
! !

!SptExchangeQueue methodsFor:'queueing'!

queueFirst: anExchange 
        self assertQueueLoopActive.
        Smalltalk isSmalltalkX ifTrue:[
            queue nextPutFirst: anExchange.
        ] ifFalse:[
            queue queueFirst: anExchange.
        ].
        self trigger: #exchangeQueued: with: anExchange
!

queueLast: anExchange 
	self assertQueueLoopActive.
	queue nextPut: anExchange.
	self trigger: #exchangeQueued: with: anExchange
! !

!SptExchangeQueue class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptExchangeQueue.st,v 1.5 2010/02/26 12:02:48 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptExchangeQueue.st,v 1.5 2010/02/26 12:02:48 cg Exp $'
! !
