"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SptHTTPRequest
	instanceVariableNames:'socket url method headers statusLine responseHeaders responseBody
		deferredSend timeoutState timeout proxySettings'
	classVariableNames:'Verbose'
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptHTTPRequest comment:'I am a single request and response http message send in a background process.

See class example.

I will follow redirects, replacing url with the redirect Location.

Instance Variables:
	url	<String> HTTP url (absolute).
	method	<String>. HTTP method
	headers	<Dictionary> HTTP request headers.
	statusLine	<String>. Response statusLine
	responseHeaders	<Dictionary> HTTP response headers.
	responseBody	<String> response headers
	timeout		<Integer> or nil. (milliseconds)
	timeoutState 	<Symbol> #off, #on or #waiting.
	proxySettings	<SptHTTPProxySettings> or nil. If proxySettings useProxy, the proxySetting''s host and port value will be used.

==========
Timeout

The timeout is implemented to handle the common case of being able to connect and send the request, but no response
is returned. Directly monitoring the Socket messages in WinAsyncSocket>>wsaEvent:wParam:lParam: shows that no events are received from Windows, and setting timeout values into the Socket has no effect on the length of the timeout.

If the timeout instance variable is notNil, and the message #waitOnReady is sent, a timeout loop will be created that checks whether the receiver canTimeout. See #waitTiming comment.

Currently, timeoutState will only be #on after the request has been sent until the status line has been received. If the #checkTimeout message is sent while the timeoutState is #on, it will change it to #waiting and after another timeout period, close the socket.






'
!


!SptHTTPRequest class methodsFor:'instance creation'!

new
        ^self basicNew initialize
! !

!SptHTTPRequest class methodsFor:'constants'!

icon
	^Icon fromId: 179 in: ShellLibrary default
!

userAgent
	^'Spray/0.55'
! !

!SptHTTPRequest class methodsFor:'must strip'!

example
        <resource: #example>

        | request |
        request := SptHTTPRequest new.
        request openGetTo: 'http://www.dolphinharbor.org/'.
        request addHeader: 'Accept' value: '*'.
        request send.
        request waitOnReady.    "Will block until response received"
        request responseBody.   "Inspect It"
        request url.    "Inspect It ... the request was redirected"
        request responseHeaders.        "Inspect It "
        request status.         "Inspect It "
        request statusText      "Inspect It "
! !

!SptHTTPRequest methodsFor:'accessing'!

addHeader: aHeaderString value: aHeaderValue
	"Add the HTTP header to the request. 
	Note: do *not* include the colon in aHeaderString, (it will be added)"

	headers add: aHeaderString -> aHeaderValue
!

open: aMethodString bstrUrl: aUrl varAsync: aBool bstrUser: s1 bstrPassword: s2
	"MSXML IXMLHTTPRequest compatibility method"

	((s1 isNil or: [s1 isEmpty]) and: [s2 isNil or: [s2 isEmpty]]) ifFalse: [self error: 'Not yet implemented'].
	url := aUrl.
	method := aMethodString 
!

open: aMethodString to: aUrl
	"Set the receiver to send a request using the HTTP method (typically GET or POST), to the url"

	url := aUrl.
	method := aMethodString 
!

openGetTo: aUrl
	"Set the receiver to send a GET request to the url"

	self open: 'GET' to: aUrl
!

openPostTo: aString 
	"Set the receiver to send a POST request to the url"

	self open: 'POST' to: aString 

!

proxySettings: aSptHTTPProxySettings 
	proxySettings := aSptHTTPProxySettings 
!

responseBody
	"Answer a the response entity <String>"

	^responseBody
!

responseHeaders
	"Answer a table of response headers"

	^responseHeaders
!

setRequestHeader: aHeaderString bstrValue: aHeaderValue
	"MSXML IXMLHTTPRequest compatibility method"

	self addHeader: aHeaderString value: aHeaderValue
!

setTimeouts: t1 connectTimeout: t2 sendTimeout: t3 receiveTimeout: t4
	"On the receive timeout is used. See class comments."

	timeout := t4
!

status
        "Answer the status code of the response"

        | stream |

        "/ cg: addition begin
        statusLine isNil ifTrue:[^ 902]. "/ unknown host.
        "/ cg: addition end

        stream := statusLine readStream.
        stream upTo: Character space.
        ^(stream upTo: Character space) asNumber
!

statusLine
	"Answer the response status line"

	^statusLine
!

statusText
        "Answer the Status text of the response"

        | stream |

        "/ cg: addition begin
        statusLine isNil ifTrue:[
            deferredSend isLazyValue ifFalse:[
                deferredSend value isException ifTrue:[
                    ^ deferredSend value description 
                ].
            ].
        ].
        "/ cg: addition end

        stream := statusLine readStream.
        stream upTo: Character space.
        stream upTo: Character space.
        ^stream upToEnd
!

timeout: millisecondsOrNil
	"Set the timeout value. 
	Note: The actual time before timing out will be somewhere between milliseconds and milliseconds * 2.
	nil disables the receiver's timeouts and relies on win socket timeouts"

	timeout := millisecondsOrNil
!

url
	"Answer the url.
	If a redirect was followed, this will be the actual url of the resource."

	^url
!

url: aString
	"Must be of the form 'http://host/resource'"
	url := aString
! !

!SptHTTPRequest methodsFor:'accessing-private'!

addResponseHeaderLine: aString
	| stream key value|
	stream := aString readStream.
	key := stream upTo: $:.
	value := stream upToEnd.
	responseHeaders at: key put: value trimBlanks
	
	
	
	
	
!

host
	^self useProxy
		ifTrue: [proxySettings proxyHost]
		ifFalse: [self urlAuthority readStream upTo: $:]
!

log: aString
    (Verbose == true or:[ UserPreferences current logHTTPRequests]) ifTrue:[
        Transcript showCR: aString.
    ].
!

logWire: aString
    (Verbose == true or:[ UserPreferences current logHTTPRequests]) ifTrue:[
        Transcript showCR: aString asStringCollection asString
    ]
!

logWireLine: aString
    (Verbose == true or:[ UserPreferences current logHTTPRequests]) ifTrue:[
        self logWire: aString , String lineDelimiter
    ].
!

method
        (url startsWith:'http://') ifTrue:[
            ^ #http
        ].
        (url startsWith:'https://') ifTrue:[
            ^ #https
        ].

        self error: 'Expected http or https url. ' , url displayString.
        ^nil 
!

port
	^self useProxy
		ifTrue: [proxySettings proxyPort]
		ifFalse: [self urlPort]
!

proxySettings
	^proxySettings
!

requestURI
	^self useProxy
		ifTrue: [url]
		ifFalse: [self urlPath]
!

responseContentLength
	^(responseHeaders at: 'Content-Length') asNumber
!

urlAuthority
	^self urlStream upTo: $/
!

urlPath
	| stream |
	stream := self urlStream.
	stream upTo: $/.
	^'/' , (stream atEnd
		ifTrue: ['']
		ifFalse: [stream upToEnd])
!

urlPort
        | stream |
        stream := self urlAuthority readStream.
        stream upTo: $:.
        ^stream atEnd                                        
                ifTrue: [ (self method == #https) ifTrue:[443] ifFalse:[80] ]
                ifFalse: [stream upToEnd asNumber]
!

urlStream
        (url startsWith:'http://') ifTrue:[
            ^ url readStream skip:'http://' size
        ].
        (url startsWith:'https://') ifTrue:[
            ^ url readStream skip:'https://' size
        ].

        self error: 'Expected http or https url. ' , url displayString.
        ^nil 
! !

!SptHTTPRequest methodsFor:'helpers-private'!

nextLineFrom: stream
        | line lineString nextChar |
        stream isBinary ifTrue:[
            line := stream upTo:Character return codePoint.
            nextChar := stream peek.
            nextChar = Character nl codePoint 
                ifTrue:[ stream next ]
                ifFalse:[ self breakPoint:#cg ].
        ] ifFalse:[
            line := stream upTo:Character return.
            nextChar := stream peek.
            nextChar = Character nl 
                ifTrue:[ stream next ]
                ifFalse:[ self breakPoint:#cg ].
        ].
        self logWireLine: (lineString := line asString).
        ^lineString 
!

putHeader: anAssoc on: aStream
	aStream
		display: anAssoc key;
		display: ': ';	
		display: anAssoc value;
		cr.
!

readStream:stream for: nBytes
    |buffer n|

    Smalltalk isSmalltalkX ifTrue:[
        buffer := ByteArray new:nBytes.
        n := stream nextBytes:nBytes into:buffer startingAt:1.
        self assert:(n == nBytes).
        ^ buffer
    ] ifFalse:[
        ^ stream next: nBytes
    ].














!

readStreamToEnd: inStream
        |outStream socketError|

        Smalltalk isSmalltalkX ifTrue:[
            ^ inStream upToEnd
        ].

        socketError := SocketClosed.

        outStream := ByteArray writeStream.
        [
            inStream do: [:each | outStream nextPut: each]
        ] on: socketError do: [:e |
            UserPreferences current soapErrorDebugging ifTrue:[e reject].
            self breakPoint:#cg.
        ].
        ^ outStream contents

    "Modified: / 12-07-2011 / 17:46:10 / cg"
! !

!SptHTTPRequest methodsFor:'initializing-private'!

initialize
        super initialize.
        headers := OrderedCollection new.
        Smalltalk isSmalltalkX ifTrue:[
            responseHeaders := Dolphin::PluggableLookupTable new searchPolicy: Dolphin::SearchPolicy caseInsensitive.
        ] ifFalse:[
            responseHeaders := PluggableLookupTable new searchPolicy: SearchPolicy caseInsensitive.
        ].
        timeoutState := #off
! !

!SptHTTPRequest methodsFor:'operations'!

close
        "Close the socket.
        -If  the DeferredValue send is blocked reading, an error will be signalled in that process (which will cause the socket to be closed)
        -An error will also be signalled in any client processes blocking on #waitOnReady"

        socket notNil ifTrue: [socket close].
!

send
	"Send the request with no entity (typical for a GET Request).
	This method returns immediately.
	You can either:
		-test with #isReady (and when true access the #responseBody).
		-wait (block) on #waitOnReady (and then access #responseBody)"

	self send: ''
!

send:aString 
    "Send the request with the aString entity (typical for a POST Request).
     See #send comment"
    
    deferredSend := 
            [
                [ 
                    self doSend:aString. 
                ] 
                on:Error do:[:e | 
                    UserPreferences current soapErrorDebugging ifTrue:[e reject].
                    self breakPoint:#cg. 
                    e 
                ]
            ] deferredValue

    "Modified: / 12-07-2011 / 17:46:16 / cg"
!

waitOnReady
        "Block the active process until the request has been processed. 
        -This will signal an error in the active process if one occurs in the background process.
        -See #waitTiming comment"

        #todo.  "Should timeout of 0 return immediately?"
        "Necessary if sent from the main process"
        Smalltalk isSmalltalkX ifFalse:[
            Processor forkMainIfMain.
        ].
        ^(timeout isNil or: [timeout = 0]) ifTrue: [self waitNoTiming] ifFalse: [self waitTiming]
! !

!SptHTTPRequest methodsFor:'operations-private'!

basicSend: aString
        |response|

"/        Smalltalk isSmalltalkX ifTrue:[
"/            response := HTTPInterface get:'https://jira.neckermann.com/rpc/soap'.
"/            
"/        ].

        self connect.
        
        [
            self sendRequestOn: socket "writeStream" with: aString.
            self log: 'SOAP-HTTP: -> Sent'.
            timeoutState := #on.
            self readResponseFrom: socket "readStream"
        ] ensure: [
            socket close
        ]

    "Modified: / 20-03-2012 / 21:47:33 / cg"
!

checkTimeout
        "If the timeoutState is #wait, then we activate the timeout by closing the socket.
        If the timeoutState is #on, we move it to #wait (so the next time we check, we may timeout"

        timeoutState == #off ifTrue: [^nil].
        timeoutState == #wait 
                ifTrue: 
                        [timeoutState := #off.
                        self close.
                        self log: '-> Timeout']
                ifFalse: [timeoutState := #wait]
!

connect
        |if address retry method connected|

        Smalltalk isSmalltalkX ifTrue:[
            (method := self method) == #http ifTrue:[
                socket := Socket newTCP.
                (socket connectTo:(self host) port:(self port)) ifFalse:[
                    Error raiseWith:(self host printString,':',self port printString) errorString:'cannot connect'.
                ].
            ] ifFalse:[
                method == #https ifTrue:[
                    Error handle:[:ex |
self halt
                    ] do:[
                        if := HTTPInterface connectTo:(self host) port:(self port) useSSL:true.
                    ].
                    connected := if notNil and:[if socket notNil].
                    socket := if socket.
                    self log: '-> Connected '.
                    ^socket.

                    socket := SSL::HttpsSocket new.
                    socket connectTo:(self host) port:(self port).
                    connected := socket waitForConnectionUntil: (timeout milliseconds ).
                    connected ifFalse: [
                        Error raiseWith:(self host printString,':',self port printString) errorString:'cannot connect'.
                    ].
"/                    socket := SSLSocket newTCP.
"/                    (socket connectTo:address) ifFalse:[
"/                        Error raiseErrorString:'cannot connect'.
"/                    ].
                ] ifFalse:[
                    self error:'unexpected method'
                ].
            ].

        ] ifFalse:[
            address := InternetAddress fromString: self host.
            socket := nil.
            "The retry is necessary to handle what I understand to be a WinSock bug.
            -It will only show if many small requests are made quickly. ( I can reproduce on my local network by forking 10 processes each sending 200 requests to the same address).
            -See <http://www.cygwin.com/ml/cygwin-developers/2001-11/msg00080.html> for a discussion of the problem"
            retry := 5.
            [socket isNil and: [retry > 0]] whileTrue: 
                            [[(socket := Socket port: self port address: address) connect] on: SocketError
                                    do: 
                                            [:e | 
                                            e errorCode = 10048 
                                                    ifTrue: 
                                                            [socket close.
                                                            socket := nil.
                                                            self log: '-> connect  WSAEADDRINUSE'.
                                                            (retry := retry - 1) = 0 ifTrue: [e pass]]
                                                    ifFalse: [e pass]]].
        ].
        self log: 'SOAP-HTTP: -> Connected '.
        ^socket

    "Modified: / 20-03-2012 / 21:43:04 / cg"
!

doSend: aString 
        | visited |
        visited := OrderedCollection new.
        [
            visited add: url.
            self basicSend: aString.
            self prepareRedirect: visited
        ] whileTrue
!

prepareRedirect: visited 
	| redirect |
	(redirect := self isRedirect) 
		ifTrue: 
			[self setUrlToLocation.
			(visited includes: url) ifTrue: [self error: 'Redirection loop'].
			self resetResponse].
	^redirect
!

readResponseFrom: stream
        | line responseBytes |

        [(line := self nextLineFrom: stream) isEmpty] 
                whileTrue: [line := self nextLineFrom: stream].

        statusLine := line.
        self log: 'SOAP-HTTP:-> Read Status'.
        timeoutState := #off.
        [(line := self nextLineFrom: stream) isEmpty] 
                whileFalse: [self addResponseHeaderLine: line].

        responseBytes := self hasResponseContentLength 
                                ifTrue: [ self readStream:stream for: self responseContentLength]
                                ifFalse: [self readStreamToEnd: stream].

        responseBody := responseBytes asString.
"/ self breakPoint:#soap.
        self logWire: responseBody

    "Modified: / 20-03-2012 / 21:47:48 / cg"
!

requestStringWith: anEntityString 
	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: method;
		space;
		nextPutAll: self requestURI;
		space;
		nextPutAll: 'HTTP/1.0';
		cr.
	self putHeader: 'Host' -> self urlAuthority on: stream.
	self putHeader: 'User-Agent' -> self class userAgent on: stream.
	headers do: [:each | self putHeader: each on: stream].
	(anEntityString notNil and: [anEntityString notEmpty]) 
		ifTrue: [self putHeader: 'Content-Length' -> anEntityString size on: stream].
	stream cr.
	^stream contents
!

resetResponse
        "Private - statusLine := nil. "
        responseHeaders := Dolphin::PluggableLookupTable new searchPolicy: Dolphin::SearchPolicy caseInsensitive.
        responseBody := nil             
                
!

sendRequestOn: aWriteStream with: aString
        | headerString fullString|

"/        headerString := self requestStringWith: aString.
"/        aWriteStream nextPutAll: headerString.
"/        self logWire: headerString.
"/        (aString notNil and: [aString notEmpty]) ifTrue: [
"/                aWriteStream nextPutAll: aString.
"/                self logWire: aString].
"/        aWriteStream flush.
"/        self logWire: String lineDelimiter.

        headerString := self requestStringWith: aString.
        (aString notNil and: [aString notEmpty]) ifTrue: [
            fullString := headerString , aString
        ] ifFalse:[
            fullString := headerString
        ].

        aWriteStream nextPutBytes:fullString size from:fullString startingAt:1.
        self logWire: 'SOAP-HTTP: -> Send:'.
        self logWire: '-----------------------'.
        self logWire: fullString.
        aWriteStream flush.
        self logWire: String lineDelimiter.
        self logWire: '-----------------------'.

    "Modified: / 20-03-2012 / 21:45:47 / cg"
!

setUrlToLocation
	url := responseHeaders at: 'Location'		
		
!

waitNoTiming
	"Block the active process until the request has been processed. 
	-Rely on OS timeouts if there is a network problem"

	^deferredSend value.
	
		
		
!

waitTiming
        "Block the active process until the request has been processed. 
        -Fork a second process that repeats until terminated. It sends the receiver the #checkTimeout message. 
         This method firstly checks whether the receiver is in a state that can timeout (See #checkTimeout comment) , and if so closes the socket. (See #close comment)"

        "Early out check to avoid overhead of forking new process"

        | delayLoop |

        deferredSend hasValue ifTrue: [^deferredSend value].
        delayLoop := [
                        [
                            (Delay forMilliseconds: timeout) wait.
                            self checkTimeout
                        ] repeat
                     ] fork.
        ^ [deferredSend value] ensure: [delayLoop terminate]
! !

!SptHTTPRequest methodsFor:'test accessing-private'!

deferredSend: anObject
	"Unit Test helper method"

	deferredSend := anObject
!

responseBody: aString
	responseBody := aString
!

socket: anObject
	"Unit Test helper method"

	socket := anObject
!

statusLine: anObject
	statusLine := anObject
!

timeoutState
	"Unit Test helper method"
	^timeoutState
!

timeoutState: anObject
	"Unit Test helper method"
	timeoutState := anObject
! !

!SptHTTPRequest methodsFor:'testing'!

isReady
	"Answer whether the response has been fully read."

	^deferredSend hasValue
		
		
! !

!SptHTTPRequest methodsFor:'testing-private'!

hasResponseContentLength
	^responseHeaders includesKey: 'Content-Length'
!

isRedirect
	^self status >=300 
		and: [self status < 400
		and: [responseHeaders includesKey: 'Location']]
!

useProxy
	proxySettings isNil ifTrue: [^false].
	^proxySettings useProxy
! !

!SptHTTPRequest class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptHTTPRequest.st,v 1.21 2012/03/20 20:49:56 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptHTTPRequest.st,v 1.21 2012/03/20 20:49:56 cg Exp $'
! !
