"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptMEContext subclass:#SptRequestingExchange
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-SoapCore'
!

SptRequestingExchange comment:'I am a requesting message exchange. I take an outgoing soap request and process it (using a binding typically a <SptHTTPBinding>) into an incoming soap response. (I am used by clients)

All exceptions are handled during the processing of an exchange by the  binding (typically a <SptHTTPBinding>). If there is an error, the exchange will finish in the #fail state, and the exception message and failureReason are stored as qualified properties.

=========
Scope
See superclass comment

==========
Qualified Properties
	#''transport:ImmediateDestination''	<String> (ie for HTTP a url)
	#''transport:FailureReason'' <String> reason if end state is #fail
	#sprayTrans:exceptionMessage <String> messageText of any exception handled during the exchange
	#action:SOAPActionURI <String> HTTP SOAPAction header value

==========
Processing

In Splash, the process of sending and receiving a soap message via HTTP is done by the helper methods in SplashRequest envelope class. In Spray this is done by sending the exchange the #value message (which also converts arguments into xml, and the return into an object answer)

The underlying model is:

-The client application (in Splash this is whoever is constructing the SplashRequest, in Spray it is an instance of SprayClient) constructs a soap envelope. At this point it is acting as the SOAP Sender. 
-The client application constructs a SptRequestingExchange instance, and sets the soap envelope into it. The client application, knowing that it is going to send the envelope over HTTP, needs to set into the SptRequestingExchange the extra information that the HTTP binding requires; an immediateDestination URL and a SOAPAction. 
-The client application then asks the binding (ie the class SptHTTPBinding or SptXMLHTTPBinding), to #processExchange:. This blocks until the state of the exchange is either "Success" or "Fail". 
-If "Success" the client application can then access the response envelope. It is then responsible for checking for a Fault response, and decoding the contents. 

'
!


!SptRequestingExchange class methodsFor:'instance creation'!

out: aSpwSoapEnvelope to: aURLString 
	"Answer an instance which will send the output envelope to the server at the url.
	Note: typically aSpwSoapEnvelope is an instance of SpwSoapOutEnvelope which has been built either by Spray or Splash, but it could be an instance of SpwSoapInEnvelope if you wanted to send an envelope created by parsing an xml string"

	^self new
		out: aSpwSoapEnvelope ;
		immediateDestination: aURLString;
		yourself
!

out: aSpwSoapEnvelope to: aURLString soapAction: aString2 
	^(self out: aSpwSoapEnvelope to: aURLString)
		soapAction: aString2;
		yourself
!

outXml: xmlString to: aURLString soapAction: aString2
	^self 
		out: (SpwSoapInEnvelope fromXmlString: xmlString)
		to: aURLString
		soapAction: aString2
! !

!SptRequestingExchange methodsFor:'accessing'!

errorNotSuccess
	self error: self failedString
!

httpRequestSoapBinding: aBinding
	self atQP: (self class sprayTransQ: 'httpRequestSoapBinding') put: aBinding.
!

immediateDestination
	^self atQP: #'transport:ImmediateDestination'
!

immediateDestination: aString 
	self atQP: #'transport:ImmediateDestination' put: aString
!

inXmlOrEmpty
	in isNil ifTrue: [^''].
	^in xml
!

outXml
	^self out xml
!

outXmlOrEmpty
	out isNil ifTrue: [^''].
	^out xml
!

soapAction
	^self atQP: #'action:SOAPActionURI' ifAbsent: ['""']
!

soapAction: aString 
	self atQP: #'action:SOAPActionURI' put: aString
! !

!SptRequestingExchange methodsFor:'accessing-private'!

failedString
	^'Message exchange failed: %1 %2' 
		formatWith: (self atQP: #'transport:FailureReason' ifAbsent: [''])
		with: (self atQP: (self class sprayTransQ: 'exceptionMessage') ifAbsent: [''])
!

in: aSoapEnvelope
	in := aSoapEnvelope
!

out: aSoapEnvelope
	out := aSoapEnvelope
! !

!SptRequestingExchange methodsFor:'asserting'!

assertNotFault
        "If the in envelope is a soap:Fault envelope, signal an exception"

        #cg.

        self assert: [in notNil].
        in isFault ifTrue: [in faultException raiseRequest "signal"].
!

assertSuccessState
	"If processing the exchange has not finished in the #success state, signal an exception"

	self isSuccessState ifFalse: [self errorNotSuccess]
! !

!SptRequestingExchange methodsFor:'initializing-private'!

initialize
	super initialize.
	self setInitialState
!

reset
	"Private - qProperties := self class newQProperties." "?"
	self setInitialState
!

setInitialState
	self atQP: #'single:Role' put: 'RequestingSOAPNode'.
	self atQP: #'single:State' put: 'Requesting'.
	self atQP: #'transport:FailureReason' put: 'None'
! !

!SptRequestingExchange methodsFor:'operations'!

processUsingDefault
	(self atScopedQP: (self class sprayTransQ: 'httpRequestSoapBinding')) processExchange: self
! !

!SptRequestingExchange methodsFor:'state-private'!

setTransportFailureReason
	| dict |
	(dict := IdentityDictionary new)
		at: 'Requesting' put: 'transmissionFailure';
		at: 'Waiting' put: 'NoResponse';
		at: 'Receiving' put: 'ReceptionFailure'.
	self atQP: #'transport:FailureReason' put: (dict at: self state ifAbsent: ['transmissionFailure'])
! !

!SptRequestingExchange methodsFor:'test accessing-private'!

inXml: anXmlString
	"Test helper method"

	in := SpwSoapInEnvelope fromXmlString: anXmlString
! !

!SptRequestingExchange class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptRequestingExchange.st,v 1.2 2002/11/26 10:59:33 cg Exp $'
! !
