"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

SptMEContext subclass:#SptRespondingExchange
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Server'
!

SptRespondingExchange comment:'I am a responding message exchange. I take an incoming soap request and process it into an outgoing soap response. (I am used by servers)

A Spray webService operation model method can be configured to receive me as one of its arguments so that it has full access to the exchange.

-I am multiple process safe because I am created (and effectively destroyed) in a single process. 

=========
Scope
See superclass comment

==========
Qualified Properties
	-A Spray WebService should use the ''user'' namespace to pass objects from a soap:Header operation to a soap:Body operation. An example of this can be found in SpAuthServiceAbstract services. See SpAuthOpAuthorizer where a soap:Header operation performs the authentication and stores a token in the exchange.'
!


!SptRespondingExchange class methodsFor:'instance creation'!

onXmlString: xmlString 
	"Answer an instance.
	An exception will be signaled if the xmlString is malformed"

	^self onXmlString: xmlString xsiNS: XeNamespace schema2001i
!

onXmlString: xmlString xsiNS: aNSSymbol 
	"Answer an instance.
	An exception will be signaled if the xmlString is malformed"

	^self new onXmlString: xmlString xsiNS: aNSSymbol
! !

!SptRespondingExchange methodsFor:'accessing'!

firstInBodyBlock
	"Answer the first 'root' block in the Body of the SoapMessage. The specification is not fully clear on this,
		and a common practice has been defined by Andrew Layman on the SoapBuilders list to address this"

	^in firstBodyBlock
!

outBodyNode
	"Answer the XeNode which is the soap:Body of the out SpwSoapOutEnvelope"

	^out bodyNode
!

outHeaderNode
	"Answer the XeNode which is the soap:Header of the out SpwSoapOutEnvelope"

	^out headerNode
!

xsiNS
	^self atQP: #'sprayTrans:xsiNS'
!

xsiNS: aSymbol 
	self atQP: #'sprayTrans:xsiNS' put: aSymbol
! !

!SptRespondingExchange methodsFor:'constants'!

faultEnvelopeClass
	^SpwFaultEnvelope
! !

!SptRespondingExchange methodsFor:'enquiries'!

inHeaderBlocksFor: aSoapNode
	"Answer a collection of XeNodes which are the header soapBlocks targeted at us"

	^self in headerBlocks select: [:each | self isHeaderBlock: each targetedAt: aSoapNode]
!

mustUnderstandsFor: aSoapNode
	"Answer a collection of XeNodes which are the header soapBlocks we mustUnderstand"

	^in mustUnderstandNodes select: [:each | self isHeaderBlock: each targetedAt: aSoapNode]
		thenCollect: [:each | each qName]
! !

!SptRespondingExchange methodsFor:'initializing-private'!

createOut
	"Private - create the out SpwSoapOutEnvelope "

	self assert: [in notNil].
	"self assert: [xsiNS notNil]."
	out := SpwSoapOutEnvelope new createEnvelope.
	out addXSINamespace: self xsiNS 
!

initialize
	super initialize.
	self atQP: #'single:Role' put: 'RespondingSOAPNode'.
	self atQP: #'single:State' put: 'Receiving'.
	self atQP: #'transport:FailureReason' put: 'None'.
	self atQP: #'transport:ExchangePatternName'
		put: 'http://www.example.org/2001/12/soap/transport-mep/single-request-response/'
!

onXmlString: xmlString xsiNS: aNSSymbol
	in := SpwSoapInEnvelope fromXmlString: xmlString.
	self xsiNS: aNSSymbol.
	self createOut
! !

!SptRespondingExchange methodsFor:'state'!

beOutFaultCode: aString string: aString2 
	"Use with caution. Typically exceptions are automatically handled by creating a SoapFaultEnvelope. 
	-This manually forces a Fault to be returned. (typically only necessary for soap:header processing"

	self beFailState.
	out beFaultCode: aString string: aString2 
!

failMisunderstanding: anArray 
	self beFailState.
	out := self faultEnvelopeClass misunderstand: anArray
!

failWithException: exception
	self beFailState.
	out := SpwFaultEnvelope fromException: exception	
	
!

failWithFaultException: exception
	self beFailState.
	out := SpwFaultEnvelope fromFaultException: exception		
	
!

isContinueState
	^self state ~= 'Fail'
!

startProcessingBy: aSoapNode
	self beProcessingState.
	
	
	
! !

!SptRespondingExchange methodsFor:'state-private'!

setTransportFailureReason
	| dict |
	(dict := LookupTable new)
		at: 'Receiving' put: 'ReceptionFailure';
		at: 'Processing' put: 'NoResponse';
		at: 'Responding' put: 'transmissionFailure'.
	self atQP: #'transport:FailureReason' put: (dict at: self state ifAbsent: ['ReceptionFailure'])
! !

!SptRespondingExchange methodsFor:'testing-private'!

isHeaderBlock: aXeNode targetedAt: aSpSoapReceiver
	"Private - Is the block targeted at the soapReceiver?
	If it is, and we mustUnderstand it, and we dont, we MUST return a Fault"

	| actor actorAtt |
	(actorAtt := aXeNode 
				getAttribute: 'actor'
				in: XeNamespace soapEnv
				ifNone: []) isNil 
		ifTrue: [^true].
	actor := actorAtt asSymbol.
	^actor == #'http://schemas.xmlsoap.org/soap/actor/next' 
		or: [aSpSoapReceiver actors includes: actor]
! !

!SptRespondingExchange class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SptRespondingExchange.st,v 1.1 2002/11/18 19:39:52 cg Exp $'
! !
