"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpwSoapBCoder
	instanceVariableNames:'element controlAttributes'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Client'
!

SpwSoapBCoder comment:'I wrap an XeElement, and can be used to: 
	-decode xml nodes into objects
	-encode objects into xml.

Servers use me (in SoapBHandlers) and clients use me (in Envelopes).

Once created, I am multiple process safe.

I understand the semantics of the SOAP1.1 specification RPC Section7. I can be asked to encode and decode RPC answers/parameters and will construct and access the  RPC Wrapper element as defined by Section7.

Control attributes are attributes that do not effect the data contents of an envelope, but are intended for the soap processing layer. 
	Examples are soap-env:mustUnderstand and soap-env:actor.

==========
Instance Variables:
	element		<XeElement>
	controlAttributes	<Collection>'
!


!SpwSoapBCoder class methodsFor:'instance creation'!

onElement: aXeElement
	^self new onElement: aXeElement
! !

!SpwSoapBCoder methodsFor:'accessing'!

elementQName
	"Answer a <XeQName>"

	^element qName
!

numberOfRPCArgs
	^element childElements size
!

setControlAttribute: aXeQName value: anObject
	"Add the control attribute to the element. Unlike XeAttributes, control attributes do
	not encode/decode data, and therefore are not part of an elements content.
	From a Soap perspective, control attributes are from the soap-env layer and 
	typically used in soap:Headers"

	self controlAttributes add: aXeQName -> anObject
! !

!SpwSoapBCoder methodsFor:'accessing-private'!

controlAttributes
	controlAttributes isNil ifTrue: [controlAttributes := OrderedCollection new].
	^controlAttributes
!

controlAttributes: aCollection
	controlAttributes := aCollection
!

element
	^element
!

element: aXeElement
	element := aXeElement
!

elementContent
	^self element content 
!

onElement: aXeElement
	aXeElement encodingStyle isSoapEnc 
		ifTrue: 
			[self setControlAttribute: (XeQName 
						prefix: 'SOAP-ENV'
						local: 'encodingStyle'
						ns: XeNamespace soapEnv)
				value: XeNamespace soapEnc].
	self element: aXeElement
!

rpcContent
	^self element content
! !

!SpwSoapBCoder methodsFor:'asserting-private'!

assertCorrectCoder: aXeNode
        aXeNode qName = self elementQName 
                ifFalse: 
                        [SoapBlockNameMismatch signal: ('Expected SoapBlock named: %1 received: %2' 
                                                formatWith: self elementQName printString "displayString"
                                                with: aXeNode qName printString "displayString")]
! !

!SpwSoapBCoder methodsFor:'converting-private'!

parametersFrom: anArray
	"Answer the RPC Wrapper struct for the parameter values of anArray"
	| accessors |
	accessors := self rpcContent accessorKeys.
	^XeQStruct params: accessors values: anArray
! !

!SpwSoapBCoder methodsFor:'operations'!

decodeRPCSoapB: aXeNode
	"Decode the aXeNode soapBlock and answer an Array of parameters
		Since this is RPC we know the soapBlock is a SOAP Section 7 RPC 
		Wrapper element, and what we are interested in are it child parameter objects.

	Missing accessors can indicate a 'default' parameter -we detect missing parameters, and use nil (common practice).
	Void block decodes as empty array, which is appropriate for a unary selector.
	We do not check the name of the block; 
		-either the soapReceiver has used the name to locate the receiver,
		 or, as per section 7, we MUST ignore the name of an RPC response block"

	| struct |
	(struct := self basicDecodeSoapB: aXeNode) isNil ifTrue: [^#()].
	^struct values asArray
!

doXeContents: aBlockClosure
	"Evaluate the blockClosure for all the XeContent instances used by the receiver."

	^element doXeContents: aBlockClosure
!

doXeElements: operation
	"We only have to declare elements we use when the encodingStyle is literal"

	element encodingStyle isLiteralStyle ifTrue: [operation value: element]
!

encodeObject: anObject into: aXeNode
        "Encode anObject into aXeNode.
                This will create a new childNode node (aXeNode is either the soap:Body or soap:Header node), 
                which we then add our control attributes to (typically used in soap:Headers)"

        | writer newNode |
        writer := XeSoapXMLWriter onTopNode: aXeNode.
        newNode := element encode: anObject writer: writer.
        Smalltalk isSmalltalkX ifTrue:[
            self controlAttributes 
                    do: [:each | newNode setQAttribute: each key value: each value printString].
        ] ifFalse:[
            self controlAttributes 
                    do: [:each | newNode setQAttribute: each key value: each value displayString].
        ].
        ^newNode
!

encodeRPCAnswer: answer into: aXeNode
	"Create a RPC Wrapper for the answer and encode it into the XeNode (aXeNode is always the soap:Body node), 
		Since this is RPC the answer can either be 
			-an array of out parameters
			-the 'return' object"

	self isMultipleRPCParameters 
		ifTrue: 
			[self assert: [answer isKindOf: Collection].
			self encodeRPCOuts: answer into: aXeNode]
		ifFalse: [self encodeRPCResult: answer into: aXeNode]
! !

!SpwSoapBCoder methodsFor:'operations-private'!

basicDecodeSoapB: aXeNode
	^(XeSoapXMLReader read: aXeNode element: element) nodeObject
!

decodeSoapB: aXeNode
	"Answer the <Object> decoded from aXeNode"

	self assertCorrectCoder: aXeNode.
	^self basicDecodeSoapB: aXeNode
!

encodeRPCOuts: aCollection into: aXeNode
	"Create a RPC Wrapper for the multiple out parameters and encode it into the XeNode 
		(aXeNode is always the soap:Body node),"

	| rpcWrapperStruct |
	rpcWrapperStruct := XeQStruct new.
	self rpcContent accessorKeys with: aCollection do: [:k :v | rpcWrapperStruct at: k put: v].
	self encodeObject: rpcWrapperStruct into: aXeNode
!

encodeRPCResult: anObject into: aXeNode
	"Create a RPC Wrapper for the multiple out parameters and encode it into the XeNode 
		(aXeNode is always the soap:Body node),"

	| rpcWrapperStruct |
	rpcWrapperStruct := XeQStruct new.
	self rpcContent isVoid 
		ifFalse: [rpcWrapperStruct at: self elementContent firstAccessorKey put: anObject].
	self encodeObject: rpcWrapperStruct into: aXeNode
! !

!SpwSoapBCoder methodsFor:'testing-private'!

isMultipleRPCParameters
	^self elementContent isMultipleAccessors
! !

!SpwSoapBCoder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapBCoder.st,v 1.3 2008/09/07 18:30:05 cg Exp $'
! !
