"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpwSoapBReceiver
	instanceVariableNames:'namespace object flags guards'
	classVariableNames:'AllowNilArguments IsShared'
	poolDictionaries:''
	category:'SOAP-Spray-Core-Server'
!

SpwSoapBReceiver comment:'I hold the receiver instance for SoapBlocks of a specific namespace. I am used by servers. A SoapResponder contains a number of me, one for each namespace it responds to.

-I am multiple process safe because I do not change state once created. **My receiver object must be responsible for its own process safety**

The following envelope contains one body SoapBlock named "getPrice" in the namespace "urn:thisNamespace". A SoapResponder receiving this soapBlock would first convert it to a Smalltalk Message (using a SpwSoapBHandler). The SoapResponder would then locate me by the namespace "urn:thisNamespace", and ask me to send the message. I return the answer.

<SOAP-ENV:Envelope>
  <SOAP-ENV:Body>
    <m:getPrice xmlns:m="urn:thisNamespace">
      <param>IBM</param>
    </ns1:getPrice>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>

Instance Variables:
	namespace	<Symbol> My identifying namespace
	object		<Object> The object I wrap (and send messages to)
			Typically this is a <SprayOperationModel>
	flags		<Integer>
	guards		<Collection> of <SpwBlockGuard>s

==========
Notes
-If I contain any SpwBlockGuards, then each must answer true when sent #assertAndPass: with the SptRespondingExchange argument, before I will send the message to my object
-The soapSpec allows for any parameter to be nil, however the service has the right to Fault that request. If the #beDisallowNilArguments mode is on, I will Fault the requests on behalf of the model (this feature is a conveniece so that models do not have to guard for nil)





'
!


!SpwSoapBReceiver class methodsFor:'instance creation'!

new
        ^self basicNew initialize
!

receiveFor: anObject at: aNamespace
	"Answer an instance that will act as a <SpwSoapBReceiver>. Any incoming SoapBlocks with aNamespace, will have their Smalltalk message (construct by a Handler) sent to anObject"

	^self new receiveFor: anObject at: aNamespace
! !

!SpwSoapBReceiver class methodsFor:'initializing'!

initialize
	"
	self initialize
	"

	AllowNilArguments := 1.
	IsShared := 2
! !

!SpwSoapBReceiver methodsFor:'accessing'!

addGuard: aSpwBlockGuard
	^guards add: aSpwBlockGuard
!

namespace
	^namespace
!

receiveFor: anObject at: aNSSymbol
	"anObject can be any object. It will receive Smalltalk Messages decoded from SoapBlocks.
	Each SoapBlock must have a namespace, and that is used to locate the correct <SpwSoapBReceiver>"

	object := anObject.
	namespace := aNSSymbol
! !

!SpwSoapBReceiver methodsFor:'accessing-private'!

allowNilArguments
	^self flags allMask: AllowNilArguments
!

allowNilArguments: aBool
	flags := self flags mask: AllowNilArguments set: aBool
!

defaultFlags
	^AllowNilArguments
!

flags
	flags isNil ifTrue: [flags := self defaultFlags].
	^flags
!

guards
	^guards
!

object
	^object
!

object: anObject
	"Private - anObject can be any object. It will receive Smalltalk Messages decoded from SoapMessages"

	object := anObject
! !

!SpwSoapBReceiver methodsFor:'asserting-private'!

assertNoNilArguments: aMessage
	(aMessage arguments anySatisfy: [:each | each isNil]) 
		ifTrue: 
			[SoapServerFault new beClient 
				signal: 'Soap Block receiver is configured to not accept null parameters']
! !

!SpwSoapBReceiver methodsFor:'initializing-private'!

initialize
	super initialize.
	guards := OrderedCollection new
! !

!SpwSoapBReceiver methodsFor:'modes'!

beAllowNilArguments
	self allowNilArguments: true
!

beDisallowNilArguments
	self allowNilArguments: false
! !

!SpwSoapBReceiver methodsFor:'operations'!

sendMessage: aMessage exchange: aSptRespondingExchange
	"Send the Smalltalk message to the object.
	-First check with the guards, which must answer true to keep processing"

	(guards allSatisfy: [:each | each assertAndPass: aSptRespondingExchange]) 
		ifFalse: 
			[aSptRespondingExchange beFailState.
			^nil].
	self allowNilArguments ifFalse: [self assertNoNilArguments: aMessage].
	^self sendMessage: aMessage
! !

!SpwSoapBReceiver methodsFor:'operations-private'!

sendMessage: message
    
        ^message sendTo:object "/ forwardTo: object
! !

!SpwSoapBReceiver class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapBReceiver.st,v 1.5 2010/06/14 10:55:51 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapBReceiver.st,v 1.5 2010/06/14 10:55:51 cg Exp $'
! !

SpwSoapBReceiver initialize!
