"{ Package: 'stx:goodies/soap/spray' }"

"{ NameSpace: SOAP }"

Object subclass:#SpwSoapResponder
	instanceVariableNames:'handlerTable actors receiverTable voidProcessor'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Server'
!

SpwSoapResponder comment:'I am a SOAP responder node. I am used by servers. I receive incoming soap request messages and process them by matching the soapBlocks they contain to my soapBHandlers and soapBReceivers. My soapBHandlers are responsible for converting xml to/from objects, and my soapBReceivers are responsible for providing the "answer" object for the soap request. (a typical simple service has only 1soapBReceiver)

I am created from a <SprayLocalService>. During the operation of the server, a <SptReceiveBinding> is used to convert soap messages for a specific transport (typically HTTP POSTs) into a <SptRespondingExchange> which I then process. I am multiple process safe because I do not change state once created.

For each soapBlock (Note: a typical soap message has a single soapBlock which is the child of soap:Body) 
	-A handler is used to convert it to a Smalltalk Message. 
	-The namespace of the soapBlock is then used to locate the correct soapBReceiver
	-The Smalltalk Message is then sent to the saopBReceivers object (typically a <SprayOperationModel>).
	-The handler is used to encode the answer into the <SptRespondingExchange>s outEnvelope"

My #process: method handles all errors, and will change the state of the SptRespondingExchange to a soap Fault if an error occurs in the processing of the exchange.

Instance Variables:
	handlerTable	<LookupTable>
	receiverTable	<LookupTable>
	actors		<Collection>
	voidProcessor	<SoapBProcessor> or nil  (rarely required)




'
!


!SpwSoapResponder class methodsFor:'instance creation'!

new
        ^self basicNew initialize
! !

!SpwSoapResponder class methodsFor:'constants'!

icon
    "/ dolphin only
    Smalltalk isSmalltalkX ifTrue:[
        ^ nil
    ] ifFalse:[
        ^Dolphin::SWEnvironment spIconNamed: 'rec.ico'
    ]
! !

!SpwSoapResponder methodsFor:'accessing'!

actors
	actors isNil ifTrue: [actors := OrderedCollection new].
	^actors
!

actors: aCollection
	actors := aCollection
! !

!SpwSoapResponder methodsFor:'accessing-private'!

handlerFor: aXeQName ifAbsent: operation
	"Private - Answer the <SpwSoapBHandler> registered for the XeName. 
		-Handlers are identified by the full XeName ie localName and namespace.
		-The answered handler will be used to decode a soapBlock"

	^handlerTable at: aXeQName ifAbsent: operation
!

receiverAt: aNSSymbol
	"Private - Answer the <SpwSoapBReceiver> which receives messages for the aNamespace."

	^receiverTable at: aNSSymbol
		ifAbsent: 
			[SoapServerFault new beClient 
				signal: 'Can not locate the handler for: ' , aNSSymbol displayString]
!

receiverAt: aNSSymbol ifAbsent: operation
	"Private - Answer the <SpwSoapBReceiver> at the namespace, (which holds the object that will be the receiver of the smalltalk message created from the Soap Envelope"

	^receiverTable at: aNSSymbol ifAbsent: operation
!

receiverTable
	^receiverTable
!

selectCantHandle: aCollOfQNames
	"Private - Answer a collection of XeQNames that we dont have handlers for"

	^aCollOfQNames reject: [ :each | 
		(handlerTable includesKey: each) 
			and: [ (receiverTable includesKey: each namespace)]]
!

voidProcessor: aSpwVoidProcessor
	voidProcessor := aSpwVoidProcessor
! !

!SpwSoapResponder methodsFor:'asserting-private'!

assertVersion: aSptRespondingExchange
	
! !

!SpwSoapResponder methodsFor:'error handling-private'!

notFoundBody: aXeNode
	SoapServerFault new beClient 
		signal: 'Can not locate the handler or receiver for: ' , aXeNode qName displayString
! !

!SpwSoapResponder methodsFor:'initializing'!

addHandlers: anOrderedCollection
	"Add the <SpwSoapBHandler>s. 
	-A handler is accessed by its qName, which is used to identify the correct handler for an incoming soapBlock 
		(A soapBlock is a childNode of either soap:Header or soap:Body).
	-The handler will decode the xml soapBlocks into Smalltalk Messages and encode the answer into an outgoing soapBlock"

	anOrderedCollection do: 
			[:each | 
			self assert: [(handlerTable includesKey: each) not].	"error?"
			handlerTable at: each identificationQName put: each].
	^anOrderedCollection
!

addSoapBReceivers: anOrderedCollection
	"Add the <SpwSoapBReceiver>s. 
		A soapBReceiver is accessed by its namespace. It wraps an object that is the 
	receiver of Smalltalk messages decoded from soapBlocks in its namespace"

	anOrderedCollection do: [:each | self addSoapBReceiver: each].
	^anOrderedCollection
! !

!SpwSoapResponder methodsFor:'initializing-private'!

addSoapBReceiver: aSpBlockReceiver 
	self assert: [aSpBlockReceiver namespace notNil].
	^self receiverTable at: aSpBlockReceiver namespace put: aSpBlockReceiver
!

initialize
	super initialize.
	handlerTable := LookupTable new.
	receiverTable := IdentityDictionary new
! !

!SpwSoapResponder methodsFor:'operations'!

process: aSptRespondingExchange
	"Process the exchange, handling all errors by changing the state of the exchange to #fail"

	
	[aSptRespondingExchange startProcessingBy: self.
	self processAll: aSptRespondingExchange] 
			on: SoapServerFault
			do: [:e | aSptRespondingExchange failWithFaultException: e]
			on: Error
			do: [:e | aSptRespondingExchange failWithException: e]
! !

!SpwSoapResponder methodsFor:'operations-private'!

createSoapBProcessorFor: aXeNode ifAbsent: operation
	| blockName |
	blockName := aXeNode qName.
	^SpwSoapBProcessor onHandler: (self handlerFor: blockName ifAbsent: [^operation value])
		receiver: (self receiverAt: blockName namespace ifAbsent: [^operation value])
!

processAll: aSptRespondingExchange
	"Private - Process the exchange
		-Check for any headers that we mustUnderstand. If we dont, we MUST fault.
		-Process the headers (if any).
		-Process the body.
	-A header receiver can set the beDontContinue mode into the exchange which will stop body processing"

	(self canUnderstand: aSptRespondingExchange) 
		ifTrue: 
			[self processHeaders: aSptRespondingExchange.
			aSptRespondingExchange isContinueState 
				ifTrue: [self processBodies: aSptRespondingExchange]]
		ifFalse: [self processMisunderstand: aSptRespondingExchange]
!

processBodies: aSptRespondingExchange
	"Private - Process the body. Unlike headers, we must Fault if we cant understand the body"

	| firstBlock spBlockProcessor |
	(firstBlock := aSptRespondingExchange firstInBodyBlock) isNil 
		ifTrue: [^self processVoidBody: aSptRespondingExchange].
	spBlockProcessor := self createSoapBProcessorFor: firstBlock
				ifAbsent: [self notFoundBody: firstBlock].
	spBlockProcessor processSoapB: firstBlock exchange: aSptRespondingExchange
!

processHeader: aXeNode exchange: aSpEnvelopeExchange
	"Private - Process the header <aXeNode > soapBlock. 
	Unless we are told we 'mustUnderstand', there is no requirement to understand nor expect a header. ie we MUST ignore it"

	| spBlockProcessor |
	spBlockProcessor := self createSoapBProcessorFor: aXeNode ifAbsent: [^nil].
	spBlockProcessor processSoapB: aXeNode exchange: aSpEnvelopeExchange
!

processHeaders: aSptRespondingExchange
	"Private - Process the header blocks. At this point we know that we have handlers and receivers for the 
		headers we must understand, so if we can not locate a handler/receiver, we do nothing and keep 
		processing the rest of the blocks (This is a SOAP1.1 MUST)"

	(aSptRespondingExchange inHeaderBlocksFor: self) 
		do: [:each | self processHeader: each exchange: aSptRespondingExchange]
!

processMisunderstand: aSptRespondingExchange
	"Private - Are there are any soap:Headers, marked mustUnderstand and targeted at us?
		Do we expect them?
			If not we must Fault"

	| misunderstands |
	misunderstands := self 
				selectCantHandle: (aSptRespondingExchange mustUnderstandsFor: self).
	^aSptRespondingExchange failMisunderstanding: misunderstands
!

processVoidBody: aSptRespondingExchange
	"Private - Process the boundary case of an empty soap:Body"

	self hasVoidProcessor ifFalse: [self error: 'No Body soap blocks received'].
	voidProcessor processSoapB: nil exchange: aSptRespondingExchange
! !

!SpwSoapResponder methodsFor:'test accessing-private'!

allSoapBReceivers
	^receiverTable values asOrderedCollection
!

numberOfHandlers
	^handlerTable size
!

numberOfReceivers
	^receiverTable size
! !

!SpwSoapResponder methodsFor:'testing-private'!

canUnderstand: aSptRespondingExchange
	"Private - Answer a Bool, can we handle all the mustUnderstand headers targeted at us?"

	self assertVersion: aSptRespondingExchange.
	^(self selectCantHandle: (aSptRespondingExchange mustUnderstandsFor: self)) isEmpty
!

hasVoidProcessor
	^voidProcessor notNil
! !

!SpwSoapResponder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapResponder.st,v 1.5 2010/06/14 10:56:04 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/SpwSoapResponder.st,v 1.5 2010/06/14 10:56:04 cg Exp $'
! !
