"{ Package: 'stx:goodies/soap/spray/auth' }"

"{ NameSpace: SOAP }"

Object subclass:#SpAuthClientAuth
	instanceVariableNames:'nonce auth userID realm cNonce authorizer answerStruct'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth'
!

SpAuthClientAuth comment:'Instances of this class are deserialized from soap:Header blocks in messages to authorized web services.
'
!


!SpAuthClientAuth class methodsFor:'instance creation'!

onAuthorizer: aSpAuthDigestAuthorizer userID: aString nonce: aByteArray
	^self new 
		authorizer: aSpAuthDigestAuthorizer
		userID: aString
		nonce: aByteArray
! !

!SpAuthClientAuth class methodsFor:'constants'!

icon
	^SWEnvironment spIconNamed: 'key.ico'
! !

!SpAuthClientAuth class methodsFor:'spray factory'!

sprayElements
	^(OrderedCollection new)
		add: (XeElementSpec named: #Nonce xsd: 'hexBinary');
		add: (XeElementSpec named: #Auth xsd: 'hexBinary');
		add: (XeElementSpec named: #UserID xsd: 'string');
		add: (XeElementSpec named: #Realm xsd: 'string');
		add: (XeElementSpec named: #CNonce xsd: 'hexBinary');
		yourself
! !

!SpAuthClientAuth methodsFor:'accessing'!

answerStruct
	^answerStruct
!

auth
	^auth
!

auth: aByteArray
	auth := aByteArray
!

cNonce
	^cNonce isNil
		ifTrue: []
		ifFalse: [ByteArray fromHexString: cNonce]
!

cNonce: aByteArray
	aByteArray notNil ifTrue: [cNonce := aByteArray printSchemaHexString]
!

nonce
	^nonce isNil
		ifTrue: []
		ifFalse: [ByteArray fromHexString: nonce ]
!

nonce: aByteArray

	nonce := aByteArray printSchemaHexString
!

realm

	^realm
!

realm: aString
	realm:= aString
!

userID

	^userID
!

userID: aString
	userID := aString
! !

!SpAuthClientAuth methodsFor:'accessing-private'!

authorizer: aSpAuthDigestAuthorizer
	authorizer := aSpAuthDigestAuthorizer
!

authorizer: anAuthorizer userID: aString nonce: aByteArray
	"Private - Answer the receiver.
	Instances are both decoded from soap:headers, and encode into a soap:header. 
	-This accessor is used to construct an instance to encode"

	authorizer := anAuthorizer .
	realm := authorizer realm.
	userID := aString.
	self nonce: aByteArray . 
	cNonce := authorizer generateNonce.
	auth := self calcAuth
! !

!SpAuthClientAuth methodsFor:'asserting-private'!

assertAuthSC: aXeStruct 
	"Private - aXeStruct is an AuthSC structure extracted from a header. Assert it is valid"

	| rAuth |
	rAuth := authorizer 
		authOf: userID 
		sn: (aXeStruct at: #Nonce) printSchemaHexString
		cn: (aXeStruct at: #CNonce) printSchemaHexString.
	self assert: [rAuth = (aXeStruct at: #RAuth)]
! !

!SpAuthClientAuth methodsFor:'constants'!

faultTypeName
	^'AuthSC_Fault_t' ns: self authNamespace
!

passTypeName
	^'AuthSC_t' ns: self authNamespace
! !

!SpAuthClientAuth methodsFor:'constants-private'!

authNamespace
	"Private - Answer the namespace of the soapBlock this instance encodes and decodes into"

	^#'http://soap-authentication.org/digest/2001/08/'
! !

!SpAuthClientAuth methodsFor:'helpers'!

newAuthSCFStruct
	"Private - Answer a struct which when encoded into a soap:header, informs the sender that its credentials where either missing or invalid"

	^(XeQStruct new)
		typeName: self faultTypeName;
		yourself
!

newAuthSCStruct
	"Private - Answer a struct which when encoded into a soap:header, informs the sender that its credentials where valid.
	-The struct also contains;
	--information the sender needs send another message (Nonce), 
	--if the client is challenging the server, it will also contain an Auth (RAuth)"

	^(XeQStruct new)
		typeName: self passTypeName;
		yourself
! !

!SpAuthClientAuth methodsFor:'modes-private'!

beAnswerExpired
	answerStruct := self challengeWithStatus: 'E03'
!

beAnswerIncorrect
	answerStruct := self challengeWithStatus: 'E02'
!

beAnswerInvalidID
	answerStruct := self challengeWithStatus: 'E04'
!

beAnswerMissing
	answerStruct := self challengeWithStatus: 'E01'
!

beAnswerNextChallenge
	| newNonce |
	newNonce := authorizer generateNonce.
	answerStruct := self newAuthSCStruct.
	answerStruct
		at: #Status put: 'S01';
		at: #Nonce put: (ByteArray fromHexString: newNonce);
		at: #CNonce put: self cNonce;
		at: #RAuth
			put: (cNonce isNil 
					ifTrue: []
					ifFalse: 
						[authorizer 
							authOf: userID
							sn: newNonce
							cn: cNonce])
! !

!SpAuthClientAuth methodsFor:'operations'!

authorize
	"The receiver has been decoded from a soap:Header.
	-check the auth received, and set the appropriate answerStruct (will be encoded into a response header)"

	self isExpired ifTrue: [^self beAnswerExpired].
	self isWrongRealm ifTrue: [^self beAnswerInvalidID].
	^self checkAuth
! !

!SpAuthClientAuth methodsFor:'operations-private'!

calcAuth
	"Private - see #calcAuthIfAbsent:"

	^self calcAuthIfAbsent: [self error: 'User not found']
	

	
	


	
!

calcAuthIfAbsent: operation
	"Private - answer the Auth which is MD5( concat(secret, ':', sn, ':', cn) ) 
	  where secret is MD5( concat(uid, ':', realm, ':', pwd) ) "

	^authorizer 
		authOf: userID
		sn: nonce
		cn: cNonce 
		ifAbsent: operation.
	

	
	


	
!

challengeWithStatus: aStatusString
	^(self newAuthSCFStruct)
		at: #Status put: aStatusString;
		at: #Nonce put: (ByteArray fromHexString: authorizer generateNonce);
		at: #Realm put: authorizer realm;
		yourself
!

checkAuth
	"Private - The receiver has been decoded from a soap:Header.
	-check the auth received, and set the appropriate answerStruct (will be encoded into a response header)"

	| actualAuth |
	actualAuth := self calcAuthIfAbsent: [^self beAnswerInvalidID].
	^auth = actualAuth ifTrue: [self beAnswerNextChallenge] ifFalse: [self beAnswerIncorrect]
! !

!SpAuthClientAuth methodsFor:'testing-private'!

isExpired
	^authorizer isExpiredNonce: nonce
!

isFail
	^self answerStruct typeName = self faultTypeName
!

isWrongRealm
	^realm ~= authorizer realm
! !

!SpAuthClientAuth class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/auth/SOAP__SpAuthClientAuth.st,v 1.2 2009/10/04 17:38:23 cg Exp $'
! !
