"{ Package: 'stx:goodies/soap/spray/auth' }"

"{ NameSpace: SOAP }"

Object subclass:#SpAuthSoapSend
	instanceVariableNames:'authorizer username lastNonce'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Auth'
!

SpAuthSoapSend comment:'I send exchanges <SpwRequestingExchange>s to an authorized webservice.

Each time I process an exchange, I extract and store authorization information for the next send.

See the SoapInteropAuthService example in the "SW Interop Auth" package.

Instance Variables:
	authorizer		<SpAuthDigestAuthorizer>
	username	<String>
	lastNonce		<ByteArray>'
!


!SpAuthSoapSend class methodsFor:'instance creation'!

onAuthorizer: aSpAuthDigestAuthorizer
	^self new authorizer: aSpAuthDigestAuthorizer
!

onAuthorizer: aSpAuthDigestAuthorizer username: aString
	^(self new)
		authorizer: aSpAuthDigestAuthorizer;
		username: aString;
		yourself
! !

!SpAuthSoapSend class methodsFor:'constants'!

icon
	^SWEnvironment spIconNamed: 'soapSend.ico'
! !

!SpAuthSoapSend methodsFor:'accessing-private'!

authorizer
	^authorizer
!

authorizer: aSpAuthDigestAuthorizer
	authorizer := aSpAuthDigestAuthorizer
!

username: aString
	username := aString
! !

!SpAuthSoapSend methodsFor:'asserting-private'!

assertAuthorizedHeader: objectEnv
	self assert: [objectEnv headerObjects size = 1].
	self assert: [objectEnv firstHeaderObject typeName localName = 'AuthSC_t']
!

assertExpectedFaultHeader: objectEnv
	self assert: [objectEnv headerObjects size = 1].
	self assert: [objectEnv firstHeaderObject typeName localName = 'AuthSC_Fault_t'].
	self assert: [(objectEnv firstHeaderObject at: #Realm) = authorizer realm]
!

assertValidConfiguration
	username isNil ifTrue: [self error: 'No username has been set'].
	authorizer isNil ifTrue: [self error: 'No authorizer has been set']
! !

!SpAuthSoapSend methodsFor:'operations'!

send: anExchange
        "Send the <SptRequestingExchange> for username who's credentials are found in the authorizer.
        Note: this may involve sending twice. The first send typically results in a fault which contains a Nonce that can then be used to construct valid credentials"

        self assertValidConfiguration.
        ^[self firstSend: anExchange] on: SoapSendFault
                do: 
                        [:e | 
                        UserPreferences current soapErrorDebugging ifTrue:[e reject].
                        self extractFaultNonce: anExchange.
                        self secondSend: anExchange]

    "Modified: / 12-07-2011 / 17:44:20 / cg"
! !

!SpAuthSoapSend methodsFor:'operations-private'!

createClientAuth
	"Private - Answer an instance of SpAuthClientAuth,
	This instance will contain information for username from authorizer. It's auth will be based on the last received nonce"

	^authorizer clientAuthFor: username nonce: lastNonce
!

extractFaultNonce: anExchange
	"Private - extract the Nonce to use in the next <SptRequestingExchange>"

	| objectEnv |
	anExchange decodeSoapInHeaders.
	objectEnv := anExchange inEnvelope.
	self assertExpectedFaultHeader: objectEnv.
	lastNonce := objectEnv firstHeaderObject at: #Nonce
!

extractNonce: anExchange
	"Private - extract the Nonce to use in the next <SptRequestingExchange>"

	| objectEnv |
	objectEnv := anExchange inEnvelope.
	self assertAuthorizedHeader: objectEnv.
	lastNonce := objectEnv firstHeaderObject at: #Nonce
!

firstSend: anExchange
	"Private - Send the <SptRequestingExchange>. If we dont have a Nonce, we expect a fault from which we can extract a Nonce to use
	 (If we do have a Nonce, the send may still result in a fault since the Server may use a technique to limit the life of a Nonce"

	^self hasNonce 
		ifTrue: [self sendWithClientAuth: anExchange]
		ifFalse: [self sendWithoutClientAuth: anExchange]
!

secondSend: anExchange
	^self sendWithClientAuth: anExchange
!

sendWithClientAuth: anExchange
	"Private - Send the <SptRequestingExchange> storing the nonce for later use"

	| answer clientAuth |
	clientAuth := self createClientAuth.
	anExchange outEnvelope firstHeaderObject: clientAuth.
	answer := anExchange value.
	clientAuth assertAuthSC: anExchange inEnvelope firstHeaderObject.
	self extractNonce: anExchange.
	^answer
!

sendWithoutClientAuth: anExchange
	"Private - Send the <SptRequestingExchange> storing the nonce for later use"

	| answer |
	answer := anExchange value.
	self extractNonce: anExchange.
	^answer
! !

!SpAuthSoapSend methodsFor:'testing-private'!

hasNonce
	^lastNonce notNil
! !

!SpAuthSoapSend class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/spray/auth/SOAP__SpAuthSoapSend.st,v 1.3 2011/07/12 16:20:57 cg Exp $'
! !
