"{ Package: 'stx:goodies/soap/wsdl' }"

"{ NameSpace: SOAP }"

SprayService subclass:#SprayWSDLService
	instanceVariableNames:'wsdlDefinitions destinationUrl invalidOperations operationPort
		report'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-Core-Service'
!

SprayWSDLService comment:'I am the interface of a web service that is dynamically created from downloading a WSDL.

Most WSDLs have only one soap port, so most of the time you do not need to specify the portName.

I will download the WSDL from its url, and I will also download its imports.

If I encounter errors, I will place the problem operations in the invalidOperation collection, and answer true to #hasInvalidOperations

See #example

'
!

!SprayWSDLService class methodsFor:'documentation'!

documentation
"
    Well, the problem with webServices: you depend upon someone else to keep the service up.
    Most of the example services which used to work in 2002 are no longer working in 2010.
    The only one willing to still serve us is the xara 3D image service (exampleXara).
    I start to hate web services....
"
! !

!SprayWSDLService class methodsFor:'instance creation'!

definitionNode: aXeDoc
	"Answer an instance of the receiver created from the WSDL XeDoc.
	-The first port in the WSDL will be used. (Typically the first port is the SOAP port).
	-If the WSDL contains imports, the defaultTransport will be used to download the imports"

	^self
		definitionNode: aXeDoc
		portName: nil
		transport: self defaultTransport
!

definitionNode: aXeDoc portName: aString
	"Answer an instance of the receiver created from the WSDL XeDoc.
	-The port named aString will be used (if nil, the first port will be used (Typically the first port is the SOAP port))
	-If the WSDL contains imports, the defaultTransport will be used to download the imports"

	^self
		definitionNode: aXeDoc
		portName: aString
		transport: self defaultTransport
!

definitionNode: aXeDoc portName: aString transport: aTransport
	"Answer an instance of the receiver created from the WSDL XeDoc.
	-The portNamed aString will be used (if nil, the first port will be used (Typically the first port is the SOAP port))
	-If the WSDL contains imports, aTransport will be used to download the imports"

	^(self
		onDefinitions: (WSDLDefinitions onDocument: aXeDoc asParserNode transport: aTransport))
			setForPortNamed: aString
!

onDefinitions: aWSDLDefinitions
	"Answer an instance of the receiver, created from the <WSDLDefinitions>.
	-The instance must be told which port to use, as a WSDL may contain multiple ports. (Often the additional ports are for alternate bindings to the same service) Common practice is to list the soap port first, and the receiver defaults to using the first port. "

	^self new setWSDLDefinitions: aWSDLDefinitions
!

onExternalWSDL: xmlString portName: aStringOrNil
	"Answer an instance of the receiver created from the WSDL xml string.
	-The port named aString will be used (if nil, the first port will be used (Typically the first port is the SOAP port)).
	-No validation is performed.
	-This constructor should be used by SprayServices that are implemented by accessing a 3rd party WSDL"

	| wsdlDefs service |
	wsdlDefs := WSDLDefinitions onDocument: (XeEnvironment rootNodeFor: xmlString)
				transport: nil.
	service := self onDefinitions: wsdlDefs.
	service setPortNamed: aStringOrNil.
	^service
!

onUrl: aURLString
	"Answer an instance of the receiver, created by downloading the WSDL document from the url.
	-The first port in the WSDL will be used. (Typically the first port is the SOAP port)."

	^self onUrl: aURLString portName: nil
!

onUrl: aURLString portName: aString
	"Answer an instance of the receiver, created by downloading the WSDL document from the url.
	-The port named aString will be used (if nil, the first port will be used (Typically the first port is the SOAP port))"

	^(self
		onDefinitions: (WSDLDefinitions onUrl: aURLString transport: self defaultTransport))
			setForPortNamed: aString
!

onXmlString: xmlString
	"Answer an instance of the receiver created from the WSDL xml string.
	-The first port in the WSDL will be used. (Typically the first port is the SOAP port)."

	^self onXmlString: xmlString portName: nil
!

onXmlString: xmlString portName: aStringOrNil
	"Answer an instance of the receiver created from the WSDL xml string.
	-The port named aString will be used (if nil, the first port will be used (Typically the first port is the SOAP port))"

	^self definitionNode: (XeEnvironment rootNodeFor: xmlString) portName: aStringOrNil
! !

!SprayWSDLService class methodsFor:'helpers'!

defaultTransport
	^SOAP::SptEnvironment current httpClient
! !

!SprayWSDLService class methodsFor:'must strip'!

example
        <resource: #example>

        "Example"

        | wsdlUrl service client exchange |

        wsdlUrl := 'http://webservice.webserviceshare.com/currencyconverter/rates.asmx?WSDL'.
        service := SprayWSDLService onUrl: wsdlUrl.

        "Accessing"
        service hasInvalidOperations.
        service report. "Inspect it"

        "Create client and use it"
        client := service createClient.

        "/ ^ (exchange := client send: 'GetSupportedCurrencies') value
        ^ (exchange := client send: 'GetRates' withArguments:#( '[HIDDEN]' 'USD' 'EUR')) value
!

exampleBabelFish
	<resource: #example>

	"Translation service example"

	|client exchange|

	client := (SprayWSDLService onUrl: 'http://www.xmethods.com/sd/2001/BabelFishService.wsdl')
				createClient.

	"A Spray client can be asked for information about its operations:"
	client operationNames.
	client signatureFor: 'BabelFish'.

	exchange := client send: 'BabelFish' with: 'en_de' with:'Hello World'.
	^ exchange value
!

exampleBook
	<resource: #example>

	"Book price based on ISBN http://www.xmethods.net/ve2/ViewListing.po?serviceid=7"

	"Example"

	| client exchange |
	client := (SprayWSDLService onUrl: 'http://www.xmethods.net/sd/2001/BNQuoteService.wsdl')
				createClient.

	"A Spray client can be asked for information about its operations:"
	client operationNames.  "an OrderedCollection('getPrice')"
	client signatureFor: 'getPrice'.         "'getPrice ((isbn xsd:string))"

	exchange := client send: 'getPrice' with: '0471117090'.
	^ exchange value
!

exampleCurrency
	<resource: #example>

	"Currency Exchange rates <http://www.xmethods.net/ve2/ViewListing.po?serviceid=5>"

	"The 'Steve Waring\Spray WebServices\Clients\Currency\TB Soap Currency.pac' package contains a UI for this service"

	| client exchange |
	client := (SprayWSDLService
				onUrl: 'http://www.xmethods.net/sd/2001/CurrencyExchangeService.wsdl') createClient.
	exchange := client
				send: 'getRate'
				with: 'England'
				with: 'Australia'.
	^'Currency rate for England/Australia is %1' formatWith: exchange value displayString
!

exampleDOTNet
	<resource: #example>

	"The #send:with: messages create a SoapSend which uses RPC semantics. This works for .NET DOC/Literal operations because .NET seems to be designed such that the xml form of its DOC/Literal operations is the same as an RPC operation."

	| service client exchange |
	service := SprayWSDLService onUrl: 'http://services.pagedownweb.com/ZipCodes.asmx?WSDL'.
	client := service createClient.
	^(exchange := client
				send: 'ZipToState'
				with: '98052'
				with: 'WA') value
!

exampleTemperature
	<resource: #example>

	"Currency Exchange rates <http://www.xmethods.net/ve2/ViewListing.po?serviceid=5>"

	"The 'Steve Waring\Spray WebServices\Clients\Currency\TB Soap Currency.pac' package contains a UI for this service"

	| client temperature |
	client := (SprayWSDLService
				onUrl: 'http://www.xmethods.net/sd/2001/TemperatureService.wsdl') createClient.
	temperature := client
				send: 'getTemp'
				with: '12345'.
	^'Temperature for ZIP 12345 is %1' formatWith: temperature value displayString
!

exampleTicker
        <resource: #example>

        "Example"

        | wsdlUrl service client exchange |

        wsdlUrl := 'http://www.flash-db.com/services/ws/companyInfo.wsdl'.
        service := SprayWSDLService onUrl: wsdlUrl.

        "Accessing"
        service hasInvalidOperations.
        service report. "Inspect it"

        "Create client and use it"
        client := service createClient.

        ^ (exchange := client send: 'doCompanyInfo' withArguments:#( 'dummy' 'dummy' 'MACR')) value

        "
         self exampleTicker
        "
!

exampleXara
        <resource: #example>

        "Xara 3d Graphic generator http://www.xmethods.net/ve2/ViewListing.po?serviceid=159 by Xara. "

        |service client args exchange result|

        service := SprayWSDLService onUrl: 'http://ws.xara.com/graphicrender/render3d.wsdl'.
        client := service createClient.

"/        (client send:'GetFonts') value.
"/        (client send:'GetTemplates') value.
"/        (client send:'GetExportTypes') value.

        "Color green hex
         Color yellow hex"

        args := #('Xtreme3' 'SOAP and Spray under Smalltalk/X!!' '00FF00' 'FFFF00' 'Times New Roman' '50' 'gif').
        exchange := client send: 'RenderURL' withArguments: args.

        exchange value asURI readStreamDo:[:stream :headerInfo | |reader|
            reader := MIMETypes imageReaderForType:(headerInfo at:'content-type').
            reader isNil ifTrue:[self error:'unknown image type'].
            result := reader fromStream:stream.
        ].
        ^ result.

        "
         self exampleXara
        "
! !

!SprayWSDLService methodsFor:'accessing'!

invalidOperationReport
	"Answer a string detailing any operations deemed to be invalid"

	| stream |
	stream := String writeStream.
	self invalidOperations do: [ :each |
		stream
			display: $(;
			display: each key name;
			display: '->';
			display: each value;
			display: ') '].
	^stream contents
!

invalidOperations
	invalidOperations isNil ifTrue: [invalidOperations := OrderedCollection new].
	^invalidOperations
!

report
	^report
! !

!SprayWSDLService methodsFor:'accessing-port-private'!

operationPort
	"Answer a SpmdPort"

	^operationPort
! !

!SprayWSDLService methodsFor:'accessing-service'!

destinationUrl
	^destinationUrl
! !

!SprayWSDLService methodsFor:'event handling-private'!

onTypespaceCreated
	"This modifies the typespace by extracting the schema's from the external WSDL, and adding them to the typespace."

	super onTypespaceCreated.
	wsdlDefinitions extractSchemas do: [:each | typespace addSchemaNode: each]
! !

!SprayWSDLService methodsFor:'operations'!

assertValidOperations
	"Helper method, if the receiver has invalid operation, use this method to get a walkback on the problem"

	operationPort operations
		do: [:each | each assertValidIn: self typespace].
	self invalidOperations do: [:each | each key assertValidIn: self typespace].
!

setForPortNamed: aStringOrNil
	"Answer the receiver.
	-Set the port named aStringOrNil, (or the first port if nil) into the receiver, from wsdlDefinitions.
	--In #checkForInvalidOperations, we go through a process similar to creating a Server/Client for each operation, and if an error is signaled, mark the operation as invalid "

	self setPortNamed: aStringOrNil.
	self checkForInvalidOperations
!

setPortNamed: aStringOrNil
	"Answer the receiver.
	-Set the port named aStringOrNil, (or the first port if nil) into the receiver, from wsdlDefinitions.
	--An instance of WSDLPortData is used,  we extract the operationPort and other information from it, and then discard it."

	| wsdlPort |
	self resetPort.
	wsdlPort := WSDLPortData onDefinitions: wsdlDefinitions portName: aStringOrNil.
	operationPort := wsdlPort createPort.
	destinationUrl := wsdlPort addressLocation.
	report := wsdlPort report
! !

!SprayWSDLService methodsFor:'operations-private'!

checkForInvalidOperations
        "Used during portSetting.
        -Enumerate through the operations, and move any operations found to be invalid to the invalidOperations collection"

        invalidOperations := OrderedCollection new.
        operationPort operations do:
                        [:each |
                        [each assertValidIn: self typespace] 
                                on: Error
                                do: [:e | 
                                    UserPreferences current soapErrorDebugging ifTrue:[e reject].
                                    invalidOperations add: each -> e description
                                ]
                        ].
        invalidOperations do: [:each | operationPort removeOperation: each key].
        self hasInvalidOperations
                ifTrue:
                        [report warnInvalid.
                        report atInfo: 'Invalid' put: self invalidOperationReport]

    "Modified: / 12-07-2011 / 17:44:58 / cg"
!

resetPort
	destinationUrl := nil.
	operationPort := nil.
	invalidOperations := nil.
	report := nil
!

setWSDLDefinitions: aWSDLDefinitions
	"Answer the receiver."

	self resetPort.
	wsdlDefinitions := aWSDLDefinitions
!

wsdlDefinitions
	^ wsdlDefinitions
! !

!SprayWSDLService methodsFor:'testing'!

hasInvalidOperations
	^self invalidOperations notEmpty
!

hasWarnings
	^report hasWarnings
! !

!SprayWSDLService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/SprayWSDLService.st,v 1.14 2011/07/12 16:20:50 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/wsdl/SprayWSDLService.st,v 1.14 2011/07/12 16:20:50 cg Exp $'
! !
