"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeNamedParticle subclass:#XeElementAbstract
	instanceVariableNames:''
	classVariableNames:'IsEncodeBelow IsMultipleOccurs IsRootDeclare IsTyped'
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeElementAbstract comment:'I am an element component, I model the information in a XMLSchema element declaration

Flags:
	IsEncodeBelow   When asked to encode an object, should I encode into the node or create a new child?
	IsMultipleOccurs        Can this component be used to encode multiple nodes? (ie an array)
	IsRootDeclare   If a prefix needs declaring, should I add the declaration to the root node?
	IsTyped         Should I add an inline type attribute?

==========
Definition
From Section 2.2 at http://www.w3.org/TR/xmlschema-1/ 

"2.2.2.1 "An element declaration is an association of a name with a type definition, either simple or complex ..." 
'
!


!XeElementAbstract class methodsFor:'initialization'!

initialize
	"
	self initialize
	"

	IsRootDeclare := 16.
	IsMultipleOccurs := 32.
	IsTyped := 64.
	IsEncodeBelow := 128
! !

!XeElementAbstract class methodsFor:'instance creation'!

name: aString 
        
	^self new 
		qName: aString asQName
! !

!XeElementAbstract methodsFor:'accessing'!

allContent
	^Array with: content
!

allQNames
	^OrderedCollection with: self qName
!

childElements
	"Answer a collection of <XeElements>
	(This is the childElements of the receivers content)"

	^self subclassResponsibility
!

elementName
	^qName
!

isEncodeBelow: aBool
	"See #beEncodeBelow/#beEncodeWithin comments"

	flags := self flags mask: IsEncodeBelow set: aBool
!

isMultipleOccurs: aBool
	"See #beMultipleOccurs comments"

	flags := self flags mask: IsMultipleOccurs set: aBool
! !

!XeElementAbstract methodsFor:'accessing-private'!

isTyped: aBoolean
	"Private - See #beTyped comment"

	flags := self flags mask: IsTyped set: aBoolean
! !

!XeElementAbstract methodsFor:'modes'!

beAllNotTyped
	"Turn off inline typing for the receiver and all it's children."

	self beNotTyped.
	self content beAllNotTyped
!

beAllTyped
	"Turn on inline typing for the receiver and all it's children."

	self beTyped.
	self content beAllTyped
!

beEncodeBelow
	"Set the receiver to create a new child node for itself to encode into"

	self isEncodeBelow: true
!

beEncodeWithin
	"Set the receiver to not encode into an existing node (vs creating its own child node"

	self isEncodeBelow: false
!

beLocalDeclare
	"Set the receiver to declare (if needed) it's namespace prefix on the node it encodes into.
	This is cosmetic"

	flags := self flags mask: IsRootDeclare set: false
!

beMultipleOccurs
	"Enable the receiver to can encode/decode multiple (child)nodes.
	-Effectively the schema occurence constraint maxOccurs>1"

	flags := self flags mask: IsMultipleOccurs set: true
!

beNotNillable
	"Set the receiver to be nillable='false', and disallow nil instances    (should always be true in soap-enc)"

	self isSoapEnc ifTrue: [self error: 'Elements must be nillable in soapEnc'].
	super beNotNillable
!

beNotTyped
	"Turn off inline typing for the receiver"

	self isTyped: false
!

beRootDeclare
	flags := self flags mask: IsRootDeclare set: true
!

beSoapRoot
	self 
		beEncodeBelow;
		beAllTyped;
		beFormQualified
!

beTyped
	"Turn on inline typing for the receiver"

	self isTyped: true
! !

!XeElementAbstract methodsFor:'operations'!

createEncodeNode: aNode
	"Answer a <XeNode> for the receiver to encode a value into"

	^self isEncodeBelow ifTrue: [self addNewChildNodeTo: aNode] ifFalse: [aNode]
!

doXeContents: aBlockClosure
	"Evaluate the block for the receivers XeContent (and all descendant <XeContent>)"

	aBlockClosure value: self content.
	self childElements do: [:each | each doXeContents: aBlockClosure]
! !

!XeElementAbstract methodsFor:'operations-private'!

addNewChildNodeTo: aXeNode
	"Private - Add a new child <XeNode> to aXeNode with the receivers name"

	^self isFormQualified 
		ifTrue: [aXeNode addElementNamed: qName rootDeclare: self isRootDeclare]
		ifFalse: [aXeNode addElement: self localName]
! !

!XeElementAbstract methodsFor:'printing'!

printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' {'.
	qName displayOn: aStream.
	aStream nextPutAll: '}'.
! !

!XeElementAbstract methodsFor:'test accessing-private'!

firstChildContent
	^self firstChildElement content
!

firstChildElement
	^self childElements first
! !

!XeElementAbstract methodsFor:'testing'!

isDeclaredIn: schemaNode
	"Answer if the receiver has been declared in the <XeNode> schema"

	^self isDeclaredIn: schemaNode space: 'element'
!

isEncodeBelow
	"See #beEncodeBelow/#beEncodeWithin comments"

	^self flags allMask: IsEncodeBelow
!

isMultipleOccurs
	"See #beMultipleOccurs comments"

	^self flags allMask: IsMultipleOccurs
!

isNillable
	"In soap-enc elements MUST be nillable."

	^self isSoapEnc or: [super isNillable]
!

isRootDeclare
	"See #beLocalDeclare comment"

	^self flags allMask: IsRootDeclare
!

isTyped
	"See #beTyped comment"

	^self flags allMask: IsTyped
! !

!XeElementAbstract methodsFor:'visitor'!

declareGloballyUsing: aXeSchemaWriter
	"Tell the <XeSchemaWriter> to add a global element declaration for the receiver"

	self content isNamed 
		ifTrue: [aXeSchemaWriter globalDeclareTyped: self]
		ifFalse: [aXeSchemaWriter globalDeclareInline: self]
!

declareLocallyUsing: aXeSchemaWriter
	"Tell the <XeSchemaWriter> to add a local element declaration for the receiver"

	self content isNamed 
		ifTrue: [aXeSchemaWriter localDeclareTyped: self]
		ifFalse: [aXeSchemaWriter localDeclareInline: self]
! !

!XeElementAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeElementAbstract.st,v 1.5 2008/09/03 08:52:25 cg Exp $'
! !

XeElementAbstract initialize!
