"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

Object subclass:#XeNodeWriteMap
	instanceVariableNames:'topNode idCounter singleRefs multiRefs idBase'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Core'
!

XeNodeWriteMap comment:'I am a table of objects and their encoded nodes. I am necessary to encode identical objects using references (soap-enc id/href attributes)

-I am not multiple process safe. I must be used in a single process.

Instance Variables: 
	topNode		<XeNode>
	idCounter		<Integer>
	singleRefs	<IdentityDictionary>
	multiRefs		<IdentityDictionary>
	idBase		<String>
'
!


!XeNodeWriteMap class methodsFor:'helpers'!

isValueNoRef: anObject
	"The soap spec does not mention numbers nor booleans, but I suspect that some other implementations would not understand the concept of a multi reference to a number
	-Empty strings are also stupid to reference encode (ah well, the things we do to pass tests :)"

	^(anObject isKindOf: Number) 
		or: [(anObject isKindOf: Boolean)
		"or: [(object isKindOf: String) and: [object isEmpty]]"]
! !

!XeNodeWriteMap methodsFor:'accessing-private'!

multiRefAt: anObject ifPresent: operation 
	"Private - If multiRefs has anObject as a key, evaluate operation"

	^multiRefs at: anObject ifPresent: operation
!

singleRefAt: anObject ifAbsent: operation 
	"Private - answer the node that anObject has been encoded into "

	^singleRefs at: anObject ifAbsent: operation
!

singleRefAt: anObject put: aXeNode
	^singleRefs at: anObject put: aXeNode
!

topNode
	"Private - Answer the 'topNode' ... has meaning in that any multiple reference nodes will be moved to be children of the 'topNode'.
	-Soap1.1 was unclear on exactly what the 'topNode' should be."

	^topNode
!

topNode: aXeNode
	topNode := aXeNode
! !

!XeNodeWriteMap methodsFor:'constants-private'!

qNameEncodingStyle
	^XeQName 
		prefix: 'SOAP-ENV'
		local: 'encodingStyle'
		ns: XeNamespace soapEnv
!

qNameRoot
	^XeQName 
		prefix: 'SOAP-ENC'
		local: 'root'
		ns: XeNamespace soapEnc
! !

!XeNodeWriteMap methodsFor:'initializing'!

initializeFor: anElement node: aNode 

	self topNode: aNode.
	idBase := 'id'.
	singleRefs := IdentityDictionary new.
	multiRefs := IdentityDictionary new
! !

!XeNodeWriteMap methodsFor:'operations'!

makeNormalMultiRef: aNode 
	"Private - Answer a String which is the id of the multiref node.
	This was written was I was using MSXML directly, it could now be simplified. 
	There is constant rumbling about the id/href rules being changed. I am waiting for the changes "

	| accessorNode valueNode parent id |
	self assert: [topNode ~~ aNode].
	parent := aNode parentNode.
	accessorNode := aNode cloneNode: false.	"shallow (no content) copy"
	parent replaceChild: accessorNode oldChild: aNode.
	id := self nextID.
	valueNode := self topNode addNode: aNode.
	self assert: [valueNode == aNode].
	valueNode nodeName: id.
	valueNode setAttribute: 'id' value: id.
	accessorNode setAttribute: 'href' value: '#' , id.
	self fixupAccessorAttributes: accessorNode.
	self fixupValueAttributes: valueNode.
	self putSoapRootAttributeOn: valueNode.
	^id
!

putSoapRootAttributeOn: aNode 
	"Private - Soap Spec 1.1
5.6 SOAP root Attribute
The SOAP root attribute can be used to label serialization roots that are not true roots of an object graph so that the object graph can be deserialized. The attribute can have one of two values, either 1 or 0. True roots of an object graph have the implied attribute value of 1. Serialization roots that are not true roots can be labeled as serialization roots with an attribute value of 1 An element can explicitly be labeled as not being a serialization root with a value of 0.
The SOAP root attribute MAY appear on any subelement within the SOAP Header and SOAP Body elements. The attribute does not have a default value.
"

	aNode setQAttribute: self qNameRoot value: '0'.
	aNode setQAttribute: self qNameEncodingStyle value: XeNamespace soapEnc
!

useReferenceOf: object on: aNode special: aBool
	"Answer true if we encode object as a multireference.
	Side effects of this method are;
	-if anObject can encode as a multiref, it is encoded
	-we remember object and aNode, so that if object is encoded again, we can multireference it"

	| isReference |
	(self class isValueNoRef: object) ifTrue: [^false].
	isReference := (self useMultiRefOf: object on: aNode) or: 
					[self 
						useSingleRefOf: object
						on: aNode
						special: aBool].
	isReference ifFalse: [self singleRefAt: object put: aNode].
	^isReference
! !

!XeNodeWriteMap methodsFor:'operations-private'!

fixupAccessorAttributes: aNode
	"Private - SoapBuilders Andrew Layman 20010505
> Certain attributes will be on the accessor, those that modify the accessor.
> These are the href, soap-enc:position, soap-enc:offset. Other attributes
> appear on the element containing the value, those that modify the value.
> These include xsi:type and soap-enc:arrayType."

	self removeAttributes: #('type' 'arrayType') from: aNode
	
!

fixupValueAttributes: aNode
	"Private - remove attributes that shouldnt be on the value"

	self removeAttributes: #('offset' 'position' 'offset' 'mustUnderstand' 'actor') from: aNode
	
!

makeSpecialMultiRef: aNode
	"Private - SoapSpecv1.1 sec5.1
Strings and byte arrays are represented as multi-reference simple types, but special rules allow them to be represented efficiently for common cases (see also section 5.2.1  and 5.2.3 ). An accessor to a string or byte-array value MAY have an attribute named id and of type ID per the XML Specification [7] . If so, all other accessors to the same value are encoded as empty elements having a local, unqualified attribute named href and of type uri-reference per the XML Schema Specification [11] , with a href attribute value of a URI fragment identifier referencing the single element containing the value. "

	| id|
	aNode setAttribute: 'id' value: (id := self nextID).
	^id	

	
!

moveObject: object href: anHRefString 
	"Private - move the object to the multiRefs map. (If the identical object is again encoded, we just use an href) "

	singleRefs removeKey: object.
	multiRefs at: object put: anHRefString
!

nextID
	"Private - note: this will allow the same id to be encoded twice in the same document.
	-this is a bug and needs to be fixed.
	-Soap with attachment support would involve initializing a NodeWriteMap with the attachments, and would
		be the logical place to set a base id"

	idCounter isNil ifTrue: [ idCounter := 0].
	idCounter := idCounter + 1.
	^idBase , idCounter printString
	
!

removeAttributes: anArray from: aNode
	| removeKeys |
	removeKeys := aNode attributes keys select: [ :each | anArray includes: each xmlBase].
	removeKeys do: [ :each | aNode removeAttribute: each].
!

useMultiRefOf: object on: node
	"Answer true if we encode object as a multiref into node, otherwise false.
	NOTE: this is not just a testing method; if anObject can encode as a multiref, it is encoded"

	^(self 
		multiRefAt: object 
		ifPresent: [ :href | node setAttribute: 'href' value: href. true]) == true
!

useSingleRefOf: object on: node special: aBool
	"Private- Answer a boolean. Has this object already been encoded once, but not yet multireferenced? 
	-If so 
	--make the original encoding a reference, and move it to be a child of the top node.
	--move the object from the singleRefMap to the multiRefMap.
	-A Bool indicates the value (either a string or byteArray) should use the 'special' rules (id attribute can be added inline) "

	| originalNode id href |
	originalNode := self singleRefAt: object ifAbsent: [^false].
	id := aBool 
		ifTrue: [self makeSpecialMultiRef: originalNode]
		ifFalse: [self makeNormalMultiRef: originalNode].

	node setAttribute: 'href' value: (href := '#' , id).
	self moveObject: object href: href.
	^true

! !

!XeNodeWriteMap class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeNodeWriteMap.st,v 1.4 2003/10/08 17:31:36 cg Exp $'
! !
