"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComponentVisitor subclass:#XeSoapXMLReader
	instanceVariableNames:'topNode currentDecoding readMap isVisitingSuper'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapXMLReader comment:'I am a xml reader. I take an xml document (instance) and a XeComponent tree, and decode the xml to an object based on the XeComponent tree.

Instance Variables:
	topNode         <XeNode>
	currentDecoding <XeNodeDecoding>
	readMap         <XeNodeReadMap> 
	isVisitingSuper <Boolean>


==========
Implementation

-As I work down a (component) tree branch, I will create a new instance of myself which shares some of my state (topNode/readMap), and the new instance will decode the branch''s object/component.

The currentDecoding is created for each node/branch, and is responsible for decoding identical objects (and nil)

==========
Q: How is object identity maintained?
-Spray treats soap encoding as an external interfacing issue and pre-builds a component tree for each interface. (a typical service will have component trees for each rpc operations'' request and response wrapper elements). 
-The encoding/decoding process involves walking the component tree. As part of that process, pointers to objects that have been already serialized/deserialized are stored in a map. 
-This map can efficiently be queried as to whether it already contains an object. 
-While objects in Smalltalk are strongly typed (ie every object belongs to a class), variables can only be pointers, and can point to an object of any class. 
-Circular references are handled by storing a proxy object into the map prior to decoding a complex content element. If necessary, I can then switch all pointer references from the proxy to the newly created object. 
'
!


!XeSoapXMLReader class methodsFor:'instance creation'!

onNode: aXeNode
	^self new initializeOn: aXeNode
!

read: aXeNode element: aXeElement
	| instance |
	instance := self onNode: aXeNode.
	aXeElement decode: instance.
	^instance
!

read: aXeNode in: aXeTypespace
	"Answer an instance of the receiver, with the aXeNode decoded into its nodeObject.
	-Firstly create an XeElement, based on the information in aXeNode.
	-Then use the XeElement to decode aXeNode"

	^self read: aXeNode element: (aXeTypespace defaultEncodingStyle elementForNode: aXeNode)
!

readNode: aXeNode
	"Answer an instance of the receiver, with the aXeNode decoded into its nodeObject.
	-This is a helper method, and will first construct an XeElement for the decoding to use. This XeElement will be created in a default soapEnc typespace"

	^self read: aXeNode in: XeTypespace defaultSoapEnc2001
! !

!XeSoapXMLReader class methodsFor:'decoding'!

guess: aXeNode in: aXeTypespace
        ^[(self read: aXeNode in: aXeTypespace) nodeObject] 
                on: XeTypeMismatch , XeSchemaNotFound
                do: [:e | e resume]

    "Modified (format): / 12-07-2011 / 17:54:23 / cg"
! !

!XeSoapXMLReader class methodsFor:'must strip'!

example
        <resource: #example>

        | node |
        node := XeEnvironment rootNodeFor: 
'<root
  xsi:type="xsd:string"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema">abc</root>'.

        ^(XeSoapXMLReader readNode: node) nodeObject. 
! !

!XeSoapXMLReader methodsFor:'accessing'!

nodeObject
	^currentDecoding object
!

nodeObject: anObject
	currentDecoding object: anObject
!

readMap
	^readMap
!

topNode
	^topNode
! !

!XeSoapXMLReader methodsFor:'accessing-private'!

currentChildren
	^self currentNode childNodes
!

currentContent
	^self currentDecoding content
!

currentDecoding
	^currentDecoding
!

currentNode
	^currentDecoding isNil ifTrue: [topNode] ifFalse: [currentDecoding node]
!

makeCurrentNode: aXeNode
	currentDecoding := XeNodeDecoding node: aXeNode reader: self
! !

!XeSoapXMLReader methodsFor:'branching-private'!

branchWithCurrentNode: aXeNode
	^self branch makeCurrentNode: aXeNode
!

postCopy
	super postCopy.
	isVisitingSuper := false
! !

!XeSoapXMLReader methodsFor:'helpers-private'!

assertType: aXeContent
	isVisitingSuper ifFalse: [aXeContent checkTypeMismatch: self currentNode]
! !

!XeSoapXMLReader methodsFor:'initializing-private'!

initializeOn: aXeNode
	topNode := aXeNode.
	readMap := XeNodeReadMap on: aXeNode.
	self makeCurrentNode: aXeNode.
	isVisitingSuper := false
! !

!XeSoapXMLReader methodsFor:'operations'!

doDecoding
	self currentContent acceptVisitor: self
! !

!XeSoapXMLReader methodsFor:'operations-private'!

doAsSuper: operation
	isVisitingSuper := true.
	operation value.
	isVisitingSuper := false
!

newArrayReader: anArrayContent 
	| arrayReader |
	arrayReader := anArrayContent readerClass on: self currentNode.
	arrayReader assertValidArrayNode.
	self assertType: anArrayContent.
	^arrayReader
! !

!XeSoapXMLReader methodsFor:'reading-private'!

guessArray: aGuessContent
	self 
		nodeObject: (self currentChildren collect: 
					[:each | 
					self visitChildElement: (aGuessContent elementForNode: self currentNode) node: each])
!

guessComplex: aGuessContent
	self isCurrentNodeArray 
		ifTrue: [self guessArray: aGuessContent]
		ifFalse: [self guessStruct: aGuessContent]
!

guessLiteralStruct: aGuessContent
	| struct |
	struct := XeQStruct new.
	self currentChildren do: 
			[:each | 
			struct at: each qName
				put: (self visitChildElement: (aGuessContent elementForNode: each) node: each)].
	self currentNode getQAttributes 
		do: [:each | struct attributes at: each key put: each value].
	self nodeObject: struct
!

guessSimple: aGuessContent
	self nodeObject: self currentNode text
!

guessSoapStruct: aGuessContent
	| struct |
	struct := XeQStruct new.
	self currentChildren do: 
			[:each | 
			struct at: each localName asSymbol
				put: (self visitChildElement: (aGuessContent elementForNode: each) node: each)].
	self nodeObject: struct
!

guessStruct: aGuessContent
	aGuessContent encodingStyle isSoapEnc 
		ifTrue: [self guessSoapStruct: aGuessContent]
		ifFalse: [self guessLiteralStruct: aGuessContent]
!

readArray: anArrayContent
	self nodeObject: (self currentChildren 
				collect: [:each | self visitChildElement: anArrayContent childElement node: each])
!

readAttributes: aXeContent
        isVisitingSuper ifTrue: [^nil].
        aXeContent attributes do: [:each | each isFixed ifFalse: [self visitAttribute: each]]
!

readBase: aXeSimpleContent
	self nodeObject: (aXeSimpleContent convertNode: self currentNode)
!

readComplex: aComplexContent table: complexNodeTable
	self nodeObject: aComplexContent newContentObject.
	complexNodeTable with: aComplexContent
		doContent: 
			[:each :childNode :accessorObj | 
			accessorObj setValue: (childNode isNil 
						ifTrue: []
						ifFalse: [self visitChildElement: each node: childNode])
				into: self nodeObject]
!

readSoapArray: aSoapArrayContent reader: aSoapArrayReader
	self nodeObject: (aSoapArrayReader childNodes 
				collect: [:each | self visitChildElement: aSoapArrayContent childElement node: each])
!

readSoapArrayKeyed: anArrayContent reader: aSoapArrayReader
	| index maxSize |
	anArrayContent assertAllowKeyedArrays.
	index := aSoapArrayReader offsetAttribute + 1.
	self nodeObject: IdentityDictionary new.
	aSoapArrayReader childNodes do: 
			[:each | 
			| att |
			att := each getAttribute: 'position' in: XeNamespace soapEnc.
			att notNil ifTrue: [index := (aSoapArrayReader extractSize: att) + 1].
			self nodeObject at: index
				put: (self visitChildElement: anArrayContent childElement node: each).
			index := index + 1].
	(self nodeObject includesKey: (maxSize := aSoapArrayReader aSize)) 
		ifFalse: [self nodeObject at: maxSize put: nil]
!

readSoapArrayMD: anArrayContent reader: aSoapArrayReader
        | nodeASize offset childStream x y|
        "/ cg: changed from LookupTable
        self nodeObject: Dictionary new.
        nodeASize := aSoapArrayReader aSize.
        offset := aSoapArrayReader offsetAttribute.
        childStream := aSoapArrayReader childNodes readStream.

        x := offset x.
        [x < nodeASize x] whileTrue:[
            x := x + 1.
        "/ offset x + 1 to: nodeASize x do:[:xA | |x| x := xA.
            y := offset y.
            [y < nodeASize y] whileTrue:[ | nextNode position item|
                y := y + 1.
            "/ offset y + 1 to: nodeASize y do:[:yA | | nextNode position y | y := yA.
                childStream atEnd 
                        ifTrue: [nextNode := nil]
                        ifFalse: 
                                [nextNode := childStream next.
                                position := nextNode 
                                                        getAttribute: 'position'
                                                        in: XeNamespace soapEnc
                                                        ifNone: [].
                                position notNil 
                                        ifTrue: 
                                                [
                                                position := aSoapArrayReader extractSize: position.
                                                x := position x + 1.
                                                y := position y + 1]].
                item := (nextNode isNil 
                                        ifTrue: []
                                        ifFalse: [self visitChildElement: anArrayContent childElement node: nextNode]).
                self nodeObject at: x @ y put: item
            ]
        ]
! !

!XeSoapXMLReader methodsFor:'testing-private'!

isCurrentNodeArray
	^currentDecoding isNodeSoapArray
! !

!XeSoapXMLReader methodsFor:'visiting-private'!

visitAttribute: aXeAttribute
	^aXeAttribute decode: self
!

visitChildElement: aXeElement node: aXeNode
	^aXeElement decode: (self branchWithCurrentNode: aXeNode)
! !

!XeSoapXMLReader methodsFor:'visitor'!

acceptArray: anArrayContent
	"Not a soap-enc Array, but a complexType with a single unbounded element"

	self assertType: anArrayContent.
	self readArray: anArrayContent.
	self readAttributes: anArrayContent

        
!

acceptComplex: aComplexContent
	| complexNodeTable |
	self assertType: aComplexContent.
	complexNodeTable := aComplexContent readerClass onComplexContent: aComplexContent
				node: self currentNode.
	self readComplex: aComplexContent table: complexNodeTable.
	self readAttributes: aComplexContent
!

acceptGuess: aGuessContent 
	self currentNode isSimpleElement ifTrue: [self guessSimple: aGuessContent ] ifFalse: [self guessComplex: aGuessContent ]
!

acceptNodeContent: aXeNodeContent 
	self nodeObject: self currentNode cloneNodeDeep 
!

acceptSimple: aSimpleContent 
	self assertType: aSimpleContent.
	self readBase: aSimpleContent.
	self readAttributes: aSimpleContent
!

acceptSimpleExtended: aXeSimpleExtendedContent 
	self assertType: aXeSimpleExtendedContent.
	self doAsSuper: [
		aXeSimpleExtendedContent baseContent acceptVisitor: self.
		self nodeObject: (aXeSimpleExtendedContent newContentObject value: self nodeObject)].
	self readAttributes: aXeSimpleExtendedContent 
!

acceptSimpleRestricted: aRestrictedContent 
	self assertType: aRestrictedContent.
	self doAsSuper: [
		aRestrictedContent baseContent acceptVisitor: self.
		(aRestrictedContent shouldntValidate: self nodeObject) 
			ifFalse: [aRestrictedContent validate: self nodeObject]].
	self readAttributes: aRestrictedContent 
!

acceptSoapArray: anArrayContent 
	"If the node includes a non-zero offset, or the items use the position attribute,
	for safety reasons, read the array into a KeyedCollection"

	| arrayReader |
	arrayReader := self newArrayReader: anArrayContent.
	arrayReader isKeyedNode 
		ifTrue: [self readSoapArrayKeyed: anArrayContent reader: arrayReader]
		ifFalse: [self readSoapArray: anArrayContent reader: arrayReader]
!

acceptSoapMultiDimArray: anArrayContent
	| arrayReader |
	arrayReader := self newArrayReader: anArrayContent.
	self readSoapArrayMD: anArrayContent reader: arrayReader
!

decodeAnyAttribute: aXeAnyAttribute 
	#todo. "this will only decode into XeComplexValues (can it work with the SprayClassFactory?)
		-maybe need to store objectAccessor
		-see XeAnyAttribute>>encodeChildValueFrom:writer: for reverse problem.
		-Also need to remove any other owned attributes"

	self currentNode getQAttributes keysAndValuesDo: [ :k :v |
		self nodeObject attributes at: k put: v]
!

decodeAnyTypeElement: aXeAnyTypeElement 
	^(aXeAnyTypeElement elementForNode: self currentNode) decode: self
!

decodeAttribute: aXeAttribute
	aXeAttribute objectAccessor setValue: (aXeAttribute extractValueFrom: self currentNode)
		into: self nodeObject
!

decodeChoiceElement: aXeChoiceElement 
	^(aXeChoiceElement selectElementForNode: self currentNode) decode: self
!

decodeElement: aXeElement
	"Answer the object from the current decoding.
	Use the XeElement to read the node in the currentDecoding.
	Note: the currentDecoding is responsible for maintaining soap-enc 'id/href' references and answering identical objects for a multireference. If there is no object previously created for the currentNode, the currentDecoding sends the receiver the #doDecoding message"

	^(self currentDecoding element: aXeElement) readNode object
! !

!XeSoapXMLReader class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapXMLReader.st,v 1.9 2011/07/12 16:20:21 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapXMLReader.st,v 1.9 2011/07/12 16:20:21 cg Exp $'
! !
