"{ Package: 'stx:goodies/soap/xe' }"

"{ NameSpace: SOAP }"

XeComponentVisitor subclass:#XeSoapXMLWriter
	instanceVariableNames:'writeMap topNode node element object isIDHref'
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Spray-XeEncoding-Schema'
!

XeSoapXMLWriter comment:'I am a xml writer. I take an object and a XeComponent tree, and encode the object to xml based on the XeComponent tree.

Instance Variables:
	writeMap        <XeNodeWriteMap> Used to track object identity and encode using id/href.
	topNode <XeNode>        TopNode of the encoding
	node    <XeNode>        CurrentNode of the encoding
	element <XeElement>     The component
	object  <Object>                The object
	isIDHref        <Boolean>       Should I encoding using id/href?

==========
Implementation

-As I work down a (component) tree branch, I will create a new instance of myself which shares some of my state (topNode/writeMap/isIDHref), and the new instance will encode the branch''s object/component.'
!


!XeSoapXMLWriter class methodsFor:'instance creation'!

onTopNode: aXeNode
	"Answer an instance that will encode into aXeNode.
	'topNode' is significant in that ' not true roots of an object graph' are encoded as its children (in a soap envelope this is the soap:Body node) ."

	^self new initializeOnTopNode: aXeNode
!

write: anObject
	"Answer an instance.
	Create a default typespace, create an XeElement component for anObject, and then encode anObject."

	| typespace |
	typespace := XeTypespace defaultSoapEnc2001.
	^self write: anObject element: (typespace defaultEncodingStyle elementForObject: anObject)
!

write: anObject element: aXeElement 
	^self 
		write: anObject
		element: aXeElement
		topNode: (aXeElement typespace newNode: 'root')
!

write: anObject element: aXeElement topNode: aXeNode
	"Answer an instance of the receiver.
	-write an encoding of anObject, using aXeElement, into aXeNode"

	| instance |
	instance := self onTopNode: aXeNode.
	aXeElement encode: anObject writer: instance.
	^instance
!

writeSprayReadOnly: anObject
	"Create an instance of the receiver, and write anObject into it. Answer the instance.
	-see #xmlSprayReadOnly: for more information"

	| typespace |
	typespace := (XeTypespace defaultSoapEnc2001)
				addSchema: (XeSchema onFactory: XeSprayReadClassFactory targetNS: XeNamespace sprayClass);
				yourself.
	typespace addEncodingStyle: XeSoapEncodingNoRefStyle for: XeNamespace soapEnc.
	^self write: anObject
		element: (typespace defaultEncodingStyle elementForObject: anObject) beEncodeWithin
! !

!XeSoapXMLWriter class methodsFor:'encoding'!

xmlSprayReadOnly: anObject
	"Answer a string of xml.
	An XeElement is constructed for anObject in a spray read only typespace. This typespace differs from the default soap-enc typespace in that;
	  1/ identical objects do not encode using the soap-enc id/href attributes,
	  2/ Any objects not specifically configured to be encoded, will encode their displayString, rather than signal an error
	As a result of these differences, an xmlString produced by this method will not *not* necessarily decode"

	^(self writeSprayReadOnly: anObject) xml
! !

!XeSoapXMLWriter class methodsFor:'must strip'!

example1
        <resource: #example>

        "Create a typespace (can have many elements)"

        | typespace element writer |
        typespace := XeTypespace defaultSoapEnc2001.
        typespace addEncodingStyle: XeSoapEncodingNoRefStyle for: XeNamespace soapEnc.
        "Create an element (can do many encodings multiple process safe)"
        element := (XeElementSpec named: #root arrayOfXSD: 'string') 
                                xeParticleWith: typespace soapEncStyle.
        element beSoapRoot.

        "Do the writing ... must do one encoding only in a single process"
        writer := XeSoapXMLWriter write: #('abc' 'def') element: element.       "Debug it"
        writer xml      "Inspect it"
!

example2
        <resource: #example>

        "Stepping through the code above you will see an encoding using XeAnyElement. "
        | writer |
        writer := XeSoapXMLWriter write: #('abc' 123 (3.14)). "Debug it"
        ^writer xml 
! !

!XeSoapXMLWriter methodsFor:'accessing'!

isIDHref: aBool
	isIDHref := aBool
!

nodeObject
	^object
!

nodeObject: anObject
"/ anObject = '' ifTrue:[self halt].
        object := anObject
!

topNode: anObject
	topNode := anObject
!

writeMap
	^writeMap
!

xml
	^topNode xml
! !

!XeSoapXMLWriter methodsFor:'accessing-private'!

currentContent
	^element content
!

currentElement
	^element
!

currentNode
	^node isNil ifTrue: [topNode] ifFalse: [node]
!

topNode
	^topNode
! !

!XeSoapXMLWriter methodsFor:'branching-private'!

branchWithNodeObject: anObject
	^self branch nodeObject: anObject
!

branchWithNodeObject: anObject element: aXeElement node: aXeNode
	^self branch 
		nodeObject: anObject
		element: aXeElement
		node: aXeNode
!

nodeObject: anObject element: anElement node: aNode
        object := anObject.
"/ anObject = '' ifTrue:[self halt].
        element := anElement.
        node := aNode
! !

!XeSoapXMLWriter methodsFor:'initializing-private'!

initializeOnTopNode: aXeNode
	writeMap := XeNodeWriteMap new initializeFor: nil node: aXeNode.
	topNode := aXeNode
! !

!XeSoapXMLWriter methodsFor:'operations'!

doEncode
	object isNil ifTrue: [^self writeNullAttribute].
	"give object the opportunity to do its own encoding"
	(object xmlEncodeUsing: self) ifTrue: [^self].
	(self isIDHref not or: [self doEncodeReferencing not]) 
		ifTrue: 
			[self writeInlineType.
			self visitContent: self currentContent]
! !

!XeSoapXMLWriter methodsFor:'operations-private'!

doEncodeReferencing
	"Answer true if the writeMap was able to encode object as a multireference."

	^writeMap 
		useReferenceOf: object
		on: node
		special: self isSpecial
! !

!XeSoapXMLWriter methodsFor:'testing'!

isIDHref
	"Answer if we should use id/href encoding.
	-(Allows object identity to be encoded)
	-Only valid in soap-enc"

	^isIDHref isNil 
		ifTrue: [element isIDHref]
		ifFalse: [isIDHref]
!

isSpecial
	"See SoapSpecv1.1 sec5.1 rule6
	-Strings and Arrays of Bytes can be 'id'ed inline"

	^element content isSoapSpecialMultiRef 
! !

!XeSoapXMLWriter methodsFor:'visitor'!

acceptAny: anAnyContent
	self writeAttributes: anAnyContent.
	(anAnyContent encodingStyle elementForObject: self nodeObject) content acceptVisitor: self
!

acceptArray: anArrayContent 
	self writeAttributes: anArrayContent .
	self writeArray: anArrayContent
!

acceptComplex: aComplexContent
	"Note: the nodeObject can be either a aXeQStruct or an instance of class which defines #sprayElements.
		-each childElement's objectAccessor is used to extract it's value from the nodeObject
        
	Note: Correct practice of encoding nil is controversial
	-Spray currently encodes a nil object using xsi:nil attribute. 
	-This may be changed in the future to omitting the element"

	self writeAttributes: aComplexContent.
	aComplexContent childElements 
		do: [:each | each encodeChildValueFrom: self nodeObject writer: self]
!

acceptSimple: aSimpleContent 
	"Note: the nodeObject is typically an immutable instance like a String, Integer etc."

	self writeAttributes: aSimpleContent.
	self writeText: (aSimpleContent convertValue: self nodeObject)
!

acceptSimpleExtended: aXeSimpleExtendedContent 
	"Note: the nodeObject should be an instance of XeSimpleValue.
	It will encode as; <elementName attributes>value</elementName>"

	self writeAttributes: aXeSimpleExtendedContent.
	aXeSimpleExtendedContent baseContent acceptVisitor: (self branchWithNodeObject: self nodeObject value).
        

        
!

acceptSimpleRestricted: aRestrictedContent 
	"Note: the nodeObject is typically an immutable instance.
	Most common restricted content nodeObject is an Integer or Float"

	self writeAttributes: aRestrictedContent.
	aRestrictedContent validate: self nodeObject.
	aRestrictedContent baseContent acceptVisitor: self
!

acceptSoapArray: anArrayContent
	"Note: the nodeObject should be a Collection.
	If the nodeObject is a Dictionary, we encode it as a soap-enc sparse Array. 
	-Other collection classes will encode as a standard soap-enc Array"

	self writeAttributes: anArrayContent.
	(self nodeObject isKindOf: Dictionary) 
		ifTrue: [self writeSoapArrayKeyed: anArrayContent]
		ifFalse: [self writeSoapArray: anArrayContent]
!

acceptSoapMultiDimArray: anArrayContent 
	| keys |
	self writeAttributes: anArrayContent.
	keys := self nodeObject keys 
				asSortedCollection: [:p1 :p2 | p1 x < p2 x or: [p1 x = p2 x and: [p1 y < p2 y]]].
	self writeSoapArrayKeyed: anArrayContent keys: keys
!

encode: anAssociation inAnyAttribute: aXeAnyAttribute
	self currentNode setQAttribute: (aXeAnyAttribute typespace qualify: anAssociation key)
		value: anAssociation value schemaDisplayString
!

encode: anObject inAnyTypeElement: aXeAnyElement 
	| valueElement encodeNode |
	valueElement := anObject isNil 
				ifTrue: [aXeAnyElement]
				ifFalse: [aXeAnyElement elementForObject: anObject].
	encodeNode := aXeAnyElement createEncodeNode: self currentNode.
	^self 
		encode: anObject
		element: valueElement
		node: encodeNode
!

encode: anObject inAttribute: aXeAttribute 
	aXeAttribute createAttribute: self currentNode value: anObject.
	^self currentNode
!

encode: anObject inChoiceElement: aXeChoiceElement 
	^(aXeChoiceElement selectChoiceForObject: anObject)
		encode: anObject writer: self 
!

encode: anObject inElement: aXeElement
	"Answer a XeNode with an encoding of anObject written into it.
	-the XeElement controls whether a new child node is created for the encoding (ie typically complex content), or whether the current node is used (ie typically simple content)"

	(anObject isNil and: [aXeElement isOmitable]) ifTrue: [^self currentNode].
	^self 
		encode: anObject
		element: aXeElement
		node: (aXeElement createEncodeNode: self currentNode)
! !

!XeSoapXMLWriter methodsFor:'visitor-private'!

encode: anObject element: aXeElement node: aXeNode
	"Private - Answer a XeNode with an encoding of anObject written into it."

	(self 
		branchWithNodeObject: anObject
		element: aXeElement
		node: aXeNode) doEncode.
	^aXeNode
! !

!XeSoapXMLWriter methodsFor:'writing-private'!

assertNillable
	"Private - are we allowed to encode nil? 
	-In soap-enc we can always encode nil, 
	-In Literal the element must have a nillable=true attribute"

	element isNillable 
		ifFalse: 
			[self 
				error: 'Error encoding nil into an element with nillable="false": ' , element qName displayString]
!

writeArray: aXeArrayContent
	self nodeObject do: [:value | aXeArrayContent childElement encode: value writer: self]
!

writeAttributes: aXeContent
	aXeContent attributes 
		do: [:attribute | attribute encodeChildValueFrom: self nodeObject writer: self]
!

writeInlineType
	element putInlineTypeIfNeededOn: node
!

writeNullAttribute
	"Private - The correct form of the null attribute depends on the schema being used
	-Find the xsi namespace for this node, and set the appropriate attribute.
	-If the xsi namespace is not declared, declare it to  the 2001 schema"

	self assertNillable.
	node addNullAttributeFor: element xsiNamespace
!

writeSoapArray: aXeSoapArrayContent
	aXeSoapArrayContent putArrayTypeSize: self nodeObject size on: self currentNode.
	self writeArray: aXeSoapArrayContent
!

writeSoapArrayKeyed: aXeSoapArrayContent
	self writeSoapArrayKeyed: aXeSoapArrayContent
		keys: self nodeObject keys asSortedCollection
!

writeSoapArrayKeyed: anArrayContent keys: keys
	| position |
	position := anArrayContent positionAttribute.
	anArrayContent 
		putArrayTypeSize: (keys isEmpty ifTrue: [anArrayContent zeroKey] ifFalse: [keys last])
		on: self currentNode.
	keys do: 
			[:p | 
			| child value |
			value := self nodeObject at: p.
			value notNil 
				ifTrue: 
					[child := anArrayContent childElement encode: value writer: self.
					child setQAttribute: position
						value: (anArrayContent aSizeStringFor: p - anArrayContent oneKey)]]
!

writeText: aString 
	node addTextNode: aString
! !

!XeSoapXMLWriter class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/xe/XeSoapXMLWriter.st,v 1.6 2008/09/05 17:45:35 cg Exp $'
! !
