Object subclass:#TCPIPServer
	instanceVariableNames:'processList serverPort acceptingSocketAccessor acceptingProcess
		serverBlock connectionCount maxConnections totalConnections
		exceptionLogQueue exceptionLogProcess exceptionLogFilename
		tcpipServerProcessClass ioType'
	classVariableNames:''
	poolDictionaries:''
	category:'TCPIP'
!

TCPIPServer comment:'I am a tcpip server.
I accept incoming client connections on a port.
To use me take a look at the example helloClientExample.
Just use my instance creation method port:block to create an instance of me.

Note that block must accept a single argument, namely an instance of TCPIPServerProcess
which I will create for the block.  The instance of TCPIPServerProcess contains the socketAccessor
which the block will use to communicate with the client.
You can also use me by creating subclasses.
In this case the subclass will define the block using lazy initialization.
'!


!TCPIPServer class methodsFor:'initialization'!

new
    ^ super basicNew initialize
! !

!TCPIPServer class methodsFor:'instance creation'!

port:aServerPort block:aServerBlock 
    ^ self port:aServerPort block:aServerBlock ioType:nil
!

port:aServerPort block:aBlock ioType:iot

    |aTCPIPServer|

    aTCPIPServer := self new.
    aTCPIPServer serverPort: aServerPort.
    aTCPIPServer ioType:iot.
    aTCPIPServer serverBlock: aBlock.
    aTCPIPServer start.
  ^ aTCPIPServer
!

port:aServerPort block:aServerBlock  tcpipServerProcessClass:aTCPIPServerProcessClass

    |aTCPIPServer|

    aTCPIPServer := self new.

    aTCPIPServer serverPort:aServerPort.
    aTCPIPServer serverBlock:aServerBlock.
    aTCPIPServer tcpipServerProcessClass:aTCPIPServerProcessClass.
    aTCPIPServer start.
  ^ aTCPIPServer
! !

!TCPIPServer class methodsFor:'examples'!

continuousMessageServer

"doit!!" "self continuousMessageServer"

| tcpipServer |

tcpipServer := TCPIPServer

	port: 1284

	block:

		[ :tcpipServerProcess |

			[       (tcpipServerProcess stream) nextPutAll: 'Hello Roger!!'; cr; nextPut: (Character lf); commit.

				(Delay forSeconds: 5) wait.

			] repeat

		].

self halt.

tcpipServer stopAll.!

echoServer

"doit!!" "self echoServer"

| tcpipServer |

tcpipServer := TCPIPServer

	port: 1284

	block:

		[ :tcpipServerProcess |

			[tcpipServerProcess stream atEnd ] whileFalse:

			[ tcpipServerProcess stream nextPut:  tcpipServerProcess stream next ].

		].

self halt.

tcpipServer stopAll.!

helloClientExample

"doit!!" "TCPIPServer helloClientExample"

| tcpipServer |

tcpipServer := TCPIPServer

	port: 1286

	block:

		[ :tcpipServerProcess |

			(tcpipServerProcess stream) nextPutAll: 'Hello Roger!!'; cr; nextPut: (Character lf); nextPutAll: 'Hello Roger!!'; commit.

			(Delay forSeconds: 5) wait.

		].

self halt.

tcpipServer stopAll.! !

!TCPIPServer methodsFor:'accessing'!

acceptingProcess

    |sock|

    acceptingProcess isNil ifTrue:[ 
        acceptingProcess := [
            [true] whileTrue:[ 
                sock := self acceptingSocketAccessor accept.
                self newServerSessionWithSocket:sock.

                self maxConnections notNil ifTrue:[
                    (self connectionCount) >= (self maxConnections) ifTrue:[
                        self stop
                    ]
                ]
            ]
        ] newProcess.

        acceptingProcess resume.
    ].

    ^ acceptingProcess
!

acceptingSocketAccessor

    acceptingSocketAccessor isNil ifTrue:[
        acceptingSocketAccessor := Socket newTCPserverAtPort:self serverPort.
        acceptingSocketAccessor listenFor: 2.
    ].
    ^ acceptingSocketAccessor
!

connectionCount
    ^ connectionCount
!

exceptionLogFilename
    ^ exceptionLogFilename
!

exceptionLogFilename:aFileName
    exceptionLogFilename := aFileName
!

exceptionLogProcess
    ^ exceptionLogProcess
!

exceptionLogQueue
    exceptionLogQueue isNil ifTrue:[
        exceptionLogQueue := SharedQueue new
    ].
    ^ exceptionLogQueue
!

ioType
    "supported types are: #stream, #polling or #blocking
    "
   ^ioType
!

ioType:anIOtype
    ioType := anIOtype ? #stream
!

maxConnections
    ^ maxConnections
!

maxConnections:aNumber
    maxConnections := aNumber
!

processList
    ^ processList
!

serverBlock
    ^ serverBlock
!

serverBlock:aBlock
    serverBlock := aBlock
!

serverPort
    ^ serverPort
!

serverPort:aPort
    serverPort := aPort
!

tcpipServerProcessClass
    ^ tcpipServerProcessClass
!

tcpipServerProcessClass:aClassName
    tcpipServerProcessClass := aClassName
!

totalConnections
    ^ totalConnections
! !

!TCPIPServer methodsFor:'control'!

restart

    self stop.
    self start.
!

start

    self serverPort notNil ifTrue:[
        self startExceptionLogProcess.
        self acceptingProcess.
    ].
!

startExceptionLogProcess

    exceptionLogProcess := [ self exceptionLogLoop 
                           ] newProcess.

    exceptionLogProcess resume.
!

stop

    acceptingProcess notNil ifTrue:[
        acceptingProcess terminate.
        acceptingProcess := nil.
    ].

    acceptingSocketAccessor notNil ifTrue:[
        acceptingSocketAccessor close.
        acceptingSocketAccessor := nil.
    ].
    self stopExceptionLogProcess.
!

stopAll

    self stop.

    self processList do:[:aProc|
        aProc notNil ifTrue:[
            aProc stop
        ]
    ].
!

stopExceptionLogProcess

    self exceptionLogProcess notNil ifTrue:[
        self exceptionLogQueue nextPut:nil 
    ].
! !

!TCPIPServer methodsFor:'default'!

defaultExceptionLogFilename
    ^ 'server.log'
! !

!TCPIPServer methodsFor:'initialization'!

initialize
    super initialize.
    connectionCount         := 0.
    totalConnections        := 0.
    processList             := List new.
    exceptionLogFilename    := self defaultExceptionLogFilename.
    tcpipServerProcessClass := TCPIPServerProcess.
    self ioType:nil.
! !

!TCPIPServer methodsFor:'logging'!

exceptionLogLoop

    |ex|

    [ (ex := self exceptionLogQueue next) notNil ] whileTrue:[
        self processException:ex
    ]
!

logException:anException
    self exceptionLogQueue nextPut:anException
!

processException:anException

    |filename stream|

    filename := self exceptionLogFilename asFilename.

    filename isDirectory ifFalse:[
        (stream := filename appendStream) notNil ifTrue:[
            [  stream nextPutAll:AbsoluteTime now printString;
                      nextPutAll:' - Exception: ';
                      nextPutAll:anException errorString;
                      cr.
            ] valueNowOrOnUnwindDo:[
                stream close
            ]
        ]
    ]
! !

!TCPIPServer methodsFor:'session'!

newServerSessionWithSocket:aSocket

    |serverProcess|

    serverProcess := self tcpipServerProcessClass socket:aSocket
                                             tcpipServer:self
                                                  ioType:self ioType.

    self processList add:serverProcess.

    connectionCount  := connectionCount  + 1.
    totalConnections := totalConnections + 1.
! !

!TCPIPServer methodsFor:'statistics'!

currentConnections
    ^ self processList size
! !

!TCPIPServer methodsFor:'updating'!

closeClientProcessFor:aClientProcess
    self processList remove:aClientProcess ifAbsent:nil
! !

!TCPIPServer class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tcpip/TCPIPServer.st,v 1.3 1999/04/15 09:21:13 cg Exp $'
! !
