"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

AbstractScanner subclass:#FSABasedScanner
	instanceVariableNames:'fsa'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Scanners'
!

FSABasedScanner class instanceVariableNames:'fsa'

"
 No other class instance variables are inherited by this class.
"
!

FSABasedScanner comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an abstract class of scanner that scans a source string and breaks it up into tokens using a minimal deterministic finite-state automata (FSA).  Each token is also given a type by its associated final state in the FSA.  Specific FSAs are stored in class instance variables of my concrete subclasses.

Instance Variables:
	fsa                             <FSAState> - a local reference to the token recognizer, in minimal deterministic form, for this class of scanner.
'
!


!FSABasedScanner class methodsFor:'class initialization'!

initialize
	"Concrete subclasses must somehow provide a fsa. Subclasses created by 
	automatic means may simply 'plug-in' a dynamically computed fsa. However, if a 
	class that can be filed-out is desired then it is worthwhile to override this 
	initialization method with one that can build the appropriate fsa directly."
	"FSABasedScanner initialize"

	self fsa: nil
! !

!FSABasedScanner class methodsFor:'state accessing'!

fsa

	^fsa
!

fsa: argument 

	fsa := argument
! !

!FSABasedScanner methodsFor:'accessing'!

endOfInputToken
	"Answer a token representing the end of the input."

	^Character endOfInput
!

endOfInputTokenType
	"Answer the token type representing the end of the input."

	^self endOfInputToken
!

myFsa

	^self class fsa
!

startState

	^self fsa
! !

!FSABasedScanner methodsFor:'converting'!

fastScanner

	^self defaultOptimizedScannerClass buildFrom: self
! !

!FSABasedScanner methodsFor:'initialization'!

init

	super init.
	self fsa: self myFsa
! !

!FSABasedScanner methodsFor:'private'!

at: state tokenTypeAndActionFor: tok 

	^state tokenTypeAndActionFor: tok
!

at: state transitionFor: char 

	^state transitionFor: char
! !

!FSABasedScanner methodsFor:'scanner generation'!

classInitializationMethodTextForClassNamed: name spec: tokenSpec 

	^self subclassResponsibility
!

createScannerClassNamed: name category: category spec: tokenSpec 

	| scannerClass |
	scannerClass := self defaultScannerClass
				subclass: name asSymbol
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: category.
	scannerClass comment: self generatedScannerClassComment.
	scannerClass class compile: (self classInitializationMethodTextForClassNamed: name spec: tokenSpec)
		classified: 'class initialization'.
	scannerClass initialize.
	^scannerClass
!

defaultOptimizedScannerClass

	^OptimizedScanner
!

defaultScannerClass

	^self class
!

generatedScannerClassComment

	^'This scanner class was automatically generated by ', TranslatorGenerator versionName , '.'
!

newStreamForMethodRendering
 | ws |
 ws := WriteStream on: (String new: 2048).
 Smalltalk isSmalltalkX ifFalse:[
    ws policy printCharactersLiterally: true.
 ].
 ^ws
! !

!FSABasedScanner methodsFor:'scanning'!

scanToken
	"Scan the next token and compute its token type."

	| state nextState tok typeAction |
	self atEnd
		ifTrue: [self signalEndOfInput]
		ifFalse: 
			[state := self startState.
			[(nextState := self at: state transitionFor: self nextChar) isNil]
				whileFalse: 
					[state := nextState.
					self getNextChar].
			tok := self buffer contents.
			typeAction := self at: state tokenTypeAndActionFor: tok.
			self tokenType: typeAction type.
			self token: tok.
			self buffer reset.
			typeAction action notNil ifTrue: [self perform: typeAction action]]
! !

!FSABasedScanner methodsFor:'scanning directives'!

compactDoubleApostrophes
	"Compact all two apostrophe sequences in my current token into a single 
	apostrophe."

	| readStream writeStream ch nextCh |
	readStream := ReadStream on: self token.
	writeStream := WriteStream on: (String new: 20).
	[readStream atEnd]
		whileFalse: 
			[writeStream nextPut: (ch := readStream next).
			(ch = $' and: [(nextCh := readStream peek) notNil and: [nextCh = $']])
				ifTrue: [readStream skip: 1]].
	self token: writeStream contents
!

ignoreComment

	self scanToken
!

ignoreDelimiter

	self scanToken
! !

!FSABasedScanner methodsFor:'state accessing'!

fsa

	^fsa
!

fsa: argument 

	fsa := argument
! !

!FSABasedScanner class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/FSABasedScanner.st,v 1.7 2006/03/31 09:15:42 cg Exp $'
! !

FSABasedScanner initialize!
