"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

EdgeLabeledDigraphNode subclass:#FSAState
	instanceVariableNames:'stateID'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Scanning/Parsing'
!

FSAState class instanceVariableNames:'noTransitionSignal'

"
 No other class instance variables are inherited by this class.
"
!

FSAState comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am a general state in a finite state automata.'
!


!FSAState class methodsFor:'instance creation'!

new

	^super new init
! !

!FSAState class methodsFor:'class initialization'!

initialize
	"FSAState initialize"

	self noTransitionSignal: (Signal new nameClass: self message: #noTransitionSymbol)
! !

!FSAState class methodsFor:'state accessing'!

noTransitionSignal

	^noTransitionSignal
!

noTransitionSignal: argument 

	noTransitionSignal := argument
! !

!FSAState methodsFor:'accessing'!

states
	"Answer the Set states reachable from here. 
	If I am the start state this is all my states."

	| states |
	states := self stateSetClass new.
	self collectStatesIn: states.
	^states
! !

!FSAState methodsFor:'building'!

goto: aState on: transitionSymbol 

	self addSuccessor: aState withEdgeLabeled: transitionSymbol
! !

!FSAState methodsFor:'converting'!

spaceOptimize

	self states do: [:state | state spaceOptimizeMap]
! !

!FSAState methodsFor:'exception handling'!

endOfInputErrorString

	^'end of input encountered'
!

raiseNoTransitionExceptionErrorString: aString 

	self class noTransitionSignal raiseErrorString: aString
!

standardErrorString

	^'illegal character encountered:  '
! !

!FSAState methodsFor:'minimizing'!

asMinimalDFSA
	"Answer a new minimal deterministic version of myself. 
	NOTE: the recipient of the DFSA should send the spaceOptimize 
	message to the DFSA.
        
	Based on Algorithm 3.3 from 'Principles of Compiler Design', 
	by Aho and Ullman, 1977."

	| dfsa states statePartitionMap oldPartition newPartition |
	dfsa := self asDeterministicFSA.
	states := dfsa states.
	newPartition := self computeInitialPartitionFor: states.
	oldPartition := Set new.
	[newPartition size = oldPartition size]
		whileFalse: 
			[oldPartition := newPartition.
			statePartitionMap := self computeStatePartitionMapFor: states using: oldPartition.
			self computePartitionTransitionsFor: states using: statePartitionMap.
			newPartition := self computeNewPartitionFor: oldPartition using: statePartitionMap].
	^self
		computeNewDFSAFor: oldPartition
		using: statePartitionMap
		startState: dfsa
!

asNearMinimalDFSAWithUniqueTokenClasses
	"Answer a new almost minimal deterministic version of myself. The result is not always 
	minimal due to the extra constraint that final state partitions containing final states for two 
	different token classes must be split. This allows the DFSA to properly handle overlapping 
	token classes.  NOTE: the recipient of the DFSA should send the spaceOptimize 
	message to the DFSA.
        
	Based on Algorithm 3.3 from 'Principles of Compiler Design', 
	by Aho and Ullman, 1977."

	| dfsa states statePartitionMap oldPartition newPartition |
	dfsa := self asDeterministicFSA.
	states := dfsa states.
	newPartition := self computeNearMinimalInitialPartitionFor: states.
	oldPartition := Set new.
	[newPartition size = oldPartition size]
		whileFalse: 
			[oldPartition := newPartition.
			statePartitionMap := self computeStatePartitionMapFor: states using: oldPartition.
			self computePartitionTransitionsFor: states using: statePartitionMap.
			newPartition := self computeNewPartitionFor: oldPartition using: statePartitionMap].
	^self
		computeNewDFSAFor: oldPartition
		using: statePartitionMap
		startState: dfsa
!

computeInitialPartitionFor: states 
	"Partition states into final and nonfinal states."

	| finalStates nonFinalStates |
	finalStates := states select: [:state | state isFSAFinalState].
	nonFinalStates := states reject: [:state | state isFSAFinalState].
	^nonFinalStates isEmpty
		ifTrue: [Set with: finalStates]
		ifFalse: [Set with: nonFinalStates with: finalStates]
!

computeNearMinimalInitialPartitionFor: states 
	"Partition states into nonfinal, literal final, and common token class final state partitions."

	| finalStates nonFinalStates partition tokenClasses literalTokens tc |
	finalStates := states select: [:state | state isFSAFinalState].
	nonFinalStates := states reject: [:state | state isFSAFinalState].
	partition := nonFinalStates isEmpty
				ifTrue: [Set new]
				ifFalse: [Set with: nonFinalStates].
	tokenClasses := SetDictionary new.
	literalTokens := Set new.
	finalStates do: 
		[:finalState | 
		(tc := finalState tokenClasses) size > 1 ifTrue: [self error: 'multiple token class states are not currently supported'].
		tc size = 0
			ifTrue: [literalTokens add: finalState]
			ifFalse: [tokenClasses at: tc first tokenType add: finalState]].
	literalTokens isEmpty ifFalse: [partition add: literalTokens].
	tokenClasses isEmpty ifFalse: [partition addAll: tokenClasses].
	^partition
!

computeNewDFSAFor: partition using: statePartitionMap startState: startState 
	"Answer a new dfsa whose states represent partitions and whose transitions are 
	computed from the statePartitionMap. The state for the partition containing 
	startState is the new start state.  NOTE: the recipient of the DFSA should send
	the spaceOptimize message to the DFSA."

	| newStateMap partitionRepresentativeState newState ch st newStartState |
	newStateMap := IdentityDictionary new.
	partition do: [:part | newStateMap at: part put: (self newDFSAStateFor: part)].
	partition do: 
		[:part | 
		partitionRepresentativeState := part first.
		newState := newStateMap at: part.
		(statePartitionMap at: partitionRepresentativeState) transitionMap
			associationsDo: 
				[:assoc | 
				ch := assoc key.
				st := newStateMap at: assoc value.
				newState goto: st on: ch]].
	newStartState := newStateMap at: (statePartitionMap at: startState) partition.
	^newStartState
!

computeNewPartitionFor: oldPartition using: statePartitionMap 
	"Answer a new state partition that is a refinement of oldPartition based on 
	partition transitions. An old partition is split into partitions of states with 
	equivalent partition transition maps."

	| newPartition partCopy initialState newPart |
	newPartition := Set new.
	oldPartition do: 
		[:part | 
		partCopy := part copy.
		[partCopy isEmpty]
			whileFalse: 
				[initialState := partCopy removeFirst.
				newPart := self stateSetClass with: initialState.
				partCopy copy do: [:state | ((statePartitionMap at: initialState)
						hasSameTransitionMapAs: (statePartitionMap at: state))
						ifTrue: 
							[partCopy remove: state.
							newPart add: state]].
				newPartition add: newPart]].
	^newPartition
!

computePartitionTransitionsFor: states using: statePartitionMap 
	"For each state in states compute its partition-based transition map, 
	i.e. a transition map from characters to partitions."

	| char targetPartition |
	states do: [:state | state edgeLabelMap
			associationsDo: 
				[:assoc | 
				char := assoc key.
				targetPartition := (statePartitionMap at: (state transitionFor: char)) partition.
				(statePartitionMap at: state)
					goto: targetPartition on: char]]
!

computeStatePartitionMapFor: states using: partition 
	"Answer a Dictionary mapping each state to an object containing its 
	corresponding partition and a partition-based transition map for the state."

	| statePartitionMap |
	statePartitionMap := Dictionary new.
	states do: [:state | statePartitionMap at: state put: (self partitionTransitionMapClass forPartition: (partition detect: [:par | par includes: state]))].
	^statePartitionMap
! !

!FSAState methodsFor:'private'!

collectStatesIn: stateSet 
	"Add myself and all states reachable from me to stateSet. 
	If I'm the start state of an fsa then all my states are added."

	(stateSet includes: self)
		ifFalse: 
			[stateSet add: self.
			self successorsExceptSelfDo: [:succ | succ collectStatesIn: stateSet]]
!

dfsaFinalStateClass

	^FSAFinalState
!

dfsaStateClass

	^FSAState
!

endOfInputToken
	"Answer a token representing the end of the input."

	^Character endOfInput
!

epsilon
	"Answer an object used to represent the empty string (epsilon)."

	^EpsilonNode epsilon
!

newDFSAStateFor: multiState 
	"Answer a new dfsa state that will represent the argument, a collection of states. 
	Make sure to transfer any final state information to the new state."

	| newFinalState finalStates |
	(finalStates := multiState select: [:state | state isFSAFinalState]) isEmpty
		ifTrue: [^self dfsaStateClass new]
		ifFalse: 
			[newFinalState := self dfsaFinalStateClass new.
			finalStates do: 
				[:fs | 
				fs literalTokens do: [:lit | newFinalState addLiteralToken: lit].
				fs tokenClasses do: [:tc | newFinalState addTokenClass: tc]].
			^newFinalState]
!

nilOutStateIDs
	"Set my stateID to nil, likewise with all my successors."

	self stateID notNil
		ifTrue:
			[self stateID: nil.
			self successorsDo: [:succ | succ nilOutStateIDs]]
!

partitionTransitionMapClass

	^PartitionTransitionMap
!

stateSetClass

	^ItemSet
! !

!FSAState methodsFor:'removing nondeterminism'!

asDeterministicFSA
	"Answer a new deterministic version of myself. 
        
	Based on Algorithm 3.1 from 'Principles of Compiler Design', 
	by Aho and Ullman, 1977."

	| multiStateMap unprocessedStates newStartState currState ch transitStates multiState epsilonClosures newMultiState newState |
	epsilonClosures := self computeEpsilonClosures.
	multiStateMap := Dictionary new.
	unprocessedStates := Set new.
	newStartState := self newDFSAStateFor: (epsilonClosures at: self).
	multiStateMap at: (epsilonClosures at: self)
		put: newStartState.
	unprocessedStates add: newStartState.
	[unprocessedStates isEmpty]
		whileFalse: 
			[currState := unprocessedStates removeFirst.
			multiState := multiStateMap keyAtValue: currState.
			(self computeTransitionMapFor: multiState)
				associationsDo: 
					[:assoc | 
					ch := assoc key.
					transitStates := assoc value.
					newMultiState := self stateSetClass new.
					transitStates do: [:ts | newMultiState addAll: (epsilonClosures at: ts)].
					(multiStateMap includesKey: newMultiState)
						ifTrue: 
							["previously encountered state"
							newState := multiStateMap at: newMultiState]
						ifFalse: 
							["make a new state"
							newState := self newDFSAStateFor: newMultiState.
							multiStateMap at: newMultiState put: newState.
							unprocessedStates add: newState].
					currState goto: newState on: ch]].
	^newStartState spaceOptimize
!

computeEpsilonClosureOf: stateSet 
	"Answer the set of states that can be reached from those in stateSet by epsilon 
	transitions alone."

	(stateSet includes: self)
		ifFalse: 
			[stateSet add: self.
			(self edgeLabelMap at: self epsilon ifAbsent: [^self])
				do: [:state | state computeEpsilonClosureOf: stateSet]]
!

computeEpsilonClosures
	"Answer a Dictionary from states to their corresponding closures."

	| closures |
	closures := Dictionary new.
	self states do: [:state | closures at: state put: state epsilonClosure].
	^closures
!

computeTransitionMapFor: multiState 
	"Answer a transition map (minus any epsilon transitons) for multiState, 
	a collection of states."

	| newMap |
	newMap := SetDictionary new.
	multiState do: [:state | state copyTransitionsTo: newMap].
	newMap removeKey: self epsilon ifAbsent: [].
	^newMap
!

epsilonClosure
	"Answer the set of states that can be reached from me by epsilon transitions 
	alone."

	| states |
	states := self stateSetClass new.
	self computeEpsilonClosureOf: states.
	^states
! !

!FSAState methodsFor:'state accessing'!

stateID

	^stateID
!

stateID: id 

	stateID := id
! !

!FSAState methodsFor:'state transitions'!

copyTransitionsTo: transitionMap 

	self edgeLabelMap associationsDo: [:assoc | transitionMap at: assoc key addAll: assoc value]
!

transitionFor: aSymbol 

	^self transitionFor: aSymbol ifNone: [self raiseNoTransitionExceptionErrorString: (aSymbol = self endOfInputToken
				ifTrue: [self endOfInputErrorString]
				ifFalse: [self standardErrorString , '''' , aSymbol printString , ''''])]
!

transitionFor: aSymbol ifNone: aBlock 

	^self edgeLabelMap at: aSymbol ifAbsent: [^aBlock value]
! !

!FSAState methodsFor:'testing'!

hasStateID

	^self stateID notNil
! !

!FSAState class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/FSAState.st,v 1.5 2002/11/20 13:50:22 cg Exp $'
! !

FSAState initialize!
