"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

OptimizedLookaheadScanner subclass:#OptimizedScannerWithTwoTokenLookahead
	instanceVariableNames:'stateStack saveState saveChar'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Scanners'
!

OptimizedScannerWithTwoTokenLookahead comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

This class provides a table-based optimized scanner with simple two-token lookahead.

Instance Variables:
	stateStack      <Stack> - primary state stack for scanning tokens.
	saveState       <Integer> - pointer into input source for error notification.
	saveChar        <Character> - pointer into input source for error notification.'
!


!OptimizedScannerWithTwoTokenLookahead methodsFor:'initialization'!

reset
	"Reset the initial state of the scanner before scanning a new source."

	super reset.
	self stateStack: Stack new
!

scanSource: aString 
	"Convert the input string to a read stream and scan the first token."

	self reset.
	self source: ((RetractableReadStream on: aString) signalAtEnd:true).
	self nextChar: self source next.
	self checkForTokenIn: self stateStack buffer: self buffer.
	self scanToken

    "Created: 5.2.1996 / 21:36:54 / stefan"
    "Modified: 5.2.1996 / 21:44:18 / stefan"
! !

!OptimizedScannerWithTwoTokenLookahead methodsFor:'scanning'!

checkForTokenIn: newStateStack buffer: charBuffer 
	"Scan the input using the arguments. Answer true if a legal token (or no illegal token) was 
	found and false otherwise."

	| nextState |
	self atEnd
		ifFalse: 
			[newStateStack push: self startState.
			"look for longest possible token"
			[(nextState := (fsa at: newStateStack top) at: self nextChar asInteger) isNil]
				whileFalse: 
					[newStateStack push: nextState.
					"getNextChar for local vars"
					charBuffer nextPut: self nextChar.
					self nextChar: self source next].
			"save the current position for error notification"
			self savePosition: self position + (self atEnd ifTrue: [1] ifFalse: [0]).
			(self isFSAFinalState: newStateStack top)
				ifFalse: 
					["save the current position for error notification"
					saveChar := self nextChar.
					saveState := newStateStack top.
					"backup to the previous final state or to the start state"
					[newStateStack size = 1 or: [self isFSAFinalState: newStateStack top]]
						whileFalse: 
							[newStateStack pop.
							"putBackChar for local vars"
							charBuffer backspace.
							self backspaceSource].
					newStateStack size = 1 ifTrue: 
						["backed up to the start state"
						self stateStack == newStateStack
							ifTrue: 
								["this is the first token, so signal an error (abort and return)"
								self at: saveState transitionFor: saveChar]
							ifFalse: 
								["we may be able to backup in the previous token"
								^false]]]].
	^true
!

scanToken
	"Scan the next token and compute its token type."

	| tok typeAction newStateStack charBuffer |
	newStateStack := Stack new.
	charBuffer := RetractableWriteStream on: (String new: 32).
	(self checkForTokenIn: newStateStack buffer: charBuffer)
		ifTrue: 
			["either a legal token or the end on input was found"
			self stateStack isEmpty ifTrue: [self atEnd
					ifTrue: [^self signalEndOfInput]
					ifFalse: [self error: 'no more vaild tokens']].
			tok := self buffer contents.
			typeAction := self at: stateStack top tokenTypeAndActionFor: tok.
			self tokenType: typeAction type.
			self token: tok.
			self buffer: charBuffer.
			self stateStack: newStateStack.
			typeAction action notNil ifTrue: [self perform: typeAction action]]
		ifFalse: 
			["an illegal token was found, try to look for earlier final state in current token buffers"
			charBuffer size timesRepeat: 
				["put back illegal token chars"
				self backspaceSource].
			"backup in current token to next smallest legal token"
			[self stateStack size = 1
				or: 
					[self stateStack pop.
					self putBackChar.
					self isFSAFinalState: stateStack top]] whileFalse.
			self stateStack size = 1
				ifTrue: 
					["no smaller legal token so signal error"
					self at: saveState transitionFor: saveChar]
				ifFalse: 
					["try again"
					self scanToken]]
! !

!OptimizedScannerWithTwoTokenLookahead methodsFor:'state accessing'!

saveChar

	^saveChar
!

saveChar: argument 

	saveChar := argument
!

saveState

	^saveState
!

saveState: argument 

	saveState := argument
!

stateStack

	^stateStack
!

stateStack: argument 

	stateStack := argument
! !

!OptimizedScannerWithTwoTokenLookahead class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/OptimizedScannerWithTwoTokenLookahead.st,v 1.6 2002/11/20 13:50:03 cg Exp $'
! !
