"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#ParseTreeBuilder
	instanceVariableNames:'stack'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Parse Trees'
!

ParseTreeBuilder comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

This is an abstract class that provides a framework for building parse trees during parsing.  Parse trees are built in a bottom-up fashion during parsing by processing key productions, and with the help of a stack.  In general, a key production has the form:

	A -> N1 N2 ... Nk => symbol

where A and the Ni are nonterminals (terminals may be interspersed freely in the right-hand side) and symbol is the production directive (or translation symbol).  Since trees are built bottom-up, the information flow in a production is from the right-hand side to the left-hand side.  When a production is ready to be processed, the top of the stack contains objects (parse trees) associated with the right-hand-side nonterminals of the production.  Processing a production involves replacing these objects with a single object representing (associated with) the left-hand-side nonterminal.  This can be thought of as computing a value for A as a function of the values of the Ni''s, i.e. value(A) = fcn(value(N1), value(N2), ..., value(Nk)).  Default functions are defined in my concrete subclasses but users may define their own production processing functions by creating a new subclass and implementing appropriate messages.  This enables users to have direct control over exactly how parse trees are built. 

Instance Variables:
	stack   <Stack> - holds intermediate node values during production processing.'
!


!ParseTreeBuilder class methodsFor:'instance creation'!

new

	^super new init
! !

!ParseTreeBuilder methodsFor:'accessing'!

popStack

	^self stack pop
!

pushStack: anObject 

	^self stack push: anObject
!

result
	"Answer the root of the tree build by this tree builder."

	self stack size = 1 ifFalse: [self error: 'incorrectly built tree'].
	^self popStack
! !

!ParseTreeBuilder methodsFor:'initialization'!

init

	self stack: Stack new
! !

!ParseTreeBuilder methodsFor:'production processing'!

addChildrenFirst: children to: aNode 
	"Add children, as the new first children, to aNode and answer aNode."

	aNode addChildrenFirst: children.
	^aNode
!

addChildrenLast: children to: aNode 
	"Add children, as the new last children, to aNode and answer aNode."

	aNode addChildrenLast: children.
	^aNode
!

answerArgument: arg 

	^arg
!

answerNil

	^nil
!

makeNewNode: stringOrSymbol 
	"Answer a new parse tree node representing the argument."

	self subclassResponsibility
!

makeNewNode: stringOrSymbol withAttribute: value 
	"Answer a new parse tree node and initialize its attribute value using the 
	setAttribute: message."

	| newNode |
	newNode := self makeNewNode: stringOrSymbol.
	newNode setAttribute: value.
	^newNode
!

makeNewNode: stringOrSymbol withChildren: children 
	"Answer a new parse tree node and initialize its children using the 
	addChildrenInitial: message."

	| newNode |
	newNode := self makeNewNode: stringOrSymbol.
	newNode addChildrenInitial: children.
	^newNode
! !

!ParseTreeBuilder methodsFor:'state accessing'!

stack

	^stack
!

stack: argument 

	stack := argument
! !

!ParseTreeBuilder methodsFor:'tree building'!

popArgNodesForProduction: grammarProd fromParser: parser 
	"Answer a collection of nodes from my stack required for processing 
	grammarProd. The order for collecting nodes is parser dependent."

	| nodes |
	nodes := OrderedCollection new.
	grammarProd numberOfRhsNonterminals timesRepeat: (parser performsLeftmostDerivation
			ifTrue: [[nodes add: self popStack]]
			ifFalse: [[nodes addFirst: self popStack]]).
	^nodes
!

processProduction: grammarProd forParser: parser 
	"This is the main driver for production processing. The actual production 
	processing messages are sent indirectly by grammarProd."

	self pushStack: (grammarProd hasSingleTokenClassRhs
			ifTrue: [grammarProd computeResultNodeFor: self withTokenClassValue: parser prevToken]
			ifFalse: [grammarProd computeResultNodeFor: self withArgNodes: (self popArgNodesForProduction: grammarProd fromParser: parser)])
! !

!ParseTreeBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/ParseTreeBuilder.st,v 1.5 2002/11/20 13:47:33 cg Exp $'
! !
