"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

AbstractParser subclass:#RecursiveDescentParser
	instanceVariableNames:'here hereType hereMark prevMark class encoder parseNode
		lastTempMark correctionDelta'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

RecursiveDescentParser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an abstract class that provides the framework for creating objects from textual representations using a recursive descent parse.
This class is what used to be called ''NewCompiler'' in old TS implementations.  It has not been rewritten to reflect its new place in the compiler framework in order to maintain compatibility with the old TS subclasses.  When they are rewritten (when the Tektronix implementation is abandoned) this class should be also.

Instance Variables:
	here                    <Object> the current token
	hereType                <Symbol> the "type" of the current token 
	hereMark                <Integer> position in source stream (mark) where this token began
	prevToken*      <Integer> size in chars of the previous token parsed
	prevMark                <Integer> mark of previous token
	class                   <Class> provides a context for the text being parsed
	encoder         <Encoder> which uses tables to decode tokens
	parseNode       <ParseNode> intermediate result of current parse (for use by subclasses)
	lastTempMark <Integer> mark of last temp;
						points to vert bar, or last char of pattern if no temps declared
	correctionDelta <Integer> offset of corrected code relative to source stream
						owing to interactive corrections so far.

* inherited from AbstractParser, but with new semantics.'
!


!RecursiveDescentParser class methodsFor:'accessing'!

preferredScannerClass
	"Answer with a scanner class which is appropiate for scanning tokens used 
	by this compiler class. Should be overwritten by subclasses."

	self subclassResponsibility
! !

!RecursiveDescentParser methodsFor:'comparing'!

match: type 
	"Answer with true if next tokens type matches"

	hereType == type
		ifTrue: 
			[self advance.
			^true].
	^false
!

matchToken: thing 
	"matches the token, not its type"

	here = thing
		ifTrue: 
			[self advance.
			^true].
	^false
! !

!RecursiveDescentParser methodsFor:'error handling'!

abort

	| exitBlock |
	encoder == nil
		ifFalse: 
			[encoder release.
			encoder := nil].                "break cycle"
	exitBlock := failBlock.
	failBlock := nil.
	^exitBlock value
!

editor

	^requestor
!

expected: aString 
	"Notify a problem at token 'here'"

	scanner atEnd ifTrue: [hereMark := hereMark + 1].
	hereType == #doIt ifTrue: [hereMark := hereMark + 1].
	^self notify: aString , ' expected ->' at: hereMark + correctionDelta
!

notify: aString 
	"Notify problem at token before 'here'"

	^self notify: aString , ' ->' at: prevMark + correctionDelta
!

notify: aString at: position 
	"If the editor is nil, pop up a SyntaxError, otherwise have the editor insert 
	aString."

	| editor |
	editor := self editor.
	Cursor normal show.
	editor == nil
		ifTrue: [SyntaxError
				errorInClass: class
				withCode: (scanner contents
						copyReplaceFrom: position
						to: position - 1
						with: aString)
				errorString: aString]
		ifFalse: [editor insertAndSelect: aString at: (position max: 1)].
	self abort
!

offEnd: aString 
	"notify a problem beyond 'here' (in lookAhead token). Don't be offEnded!!"

	^self notify: aString at: scanner mark + correctionDelta
! !

!RecursiveDescentParser methodsFor:'parsing'!

parse
	"This is the top level method that controls the (recursive descent) parse."

	self subclassResponsibility
! !

!RecursiveDescentParser methodsFor:'private'!

init: sourceString notifying: req failBlock: aBlock 

	requestor := req.
	failBlock := aBlock.
	correctionDelta := 0.
	scanner := self preferredScannerClass new.
	scanner scan: sourceString notifying: self.
	prevMark := hereMark := scanner mark.
	self advance
!

initEncoder

	self subclassResponsibility
!

initEncoder: anEncoder 

	encoder := anEncoder
!

preferredScannerClass

	^self class preferredScannerClass
! !

!RecursiveDescentParser methodsFor:'public access'!

compile: textOrStream encodeIn: anEncoder notifying: aRequestor ifFail: aBlock 
	"Answer with the result of the compilation. NOTE: information may be added 
	to the argument anEncoder during the course of this compilation."

	| result |
	self
		init: textOrStream
		notifying: aRequestor
		failBlock: aBlock.
	class isNil ifTrue: [class := Object].          "some methods rely on class being non-nil"
	self initEncoder: anEncoder.
	result := self parse.
	encoder := failBlock := requestor := parseNode := nil.          "break cycles & mitigate refct overflow"
	^result
!

compile: textOrStream in: aClass encodeIn: anEncoder notifying: aRequestor ifFail: aBlock 
	"Answer the result of compiling the text in the context of aClass. NOTE: 
	information 
	may be added to the argument anEncoder during the course of this compilation."

	class := aClass.
	^self
		compile: textOrStream
		encodeIn: anEncoder
		notifying: aRequestor
		ifFail: aBlock
!

compile: textOrStream in: aClass notifying: aRequestor ifFail: aBlock 
	"Answer the result of compiling the text in the context of aClass."

	class := aClass.
	^self
		compile: textOrStream
		notifying: aRequestor
		ifFail: aBlock
!

compile: textOrStream notifying: aRequestor ifFail: aBlock 
	"Answer with the result of the compilation."

	| result |
	self
		init: textOrStream
		notifying: aRequestor
		failBlock: aBlock.
	class isNil ifTrue: [class := Object].          "some methods rely on class being non-nil"
	self initEncoder.
	result := self parse.
	encoder := failBlock := requestor := parseNode := nil.          "break cycles & mitigate refct overflow"
	^result
! !

!RecursiveDescentParser methodsFor:'scanning'!

advance

	| this |
	prevMark := hereMark.           "Now means prev size"
	prevToken := hereType == #number | (hereType == #string)
				ifTrue: [scanner mark - prevMark]
				ifFalse: [here size].
	this := here.
	here := scanner nextToken.
	hereType := scanner nextTokenType.
	hereMark := scanner mark.
	scanner scanToken.
	^this
!

bareEndOfLastToken

	^prevMark + prevToken - 1 + correctionDelta max: 0
!

endOfInput
	"Use the eof token."

	^self endOfInputToken
!

endOfLastToken

	hereType == #doIt ifTrue: [^prevMark + prevToken + 1 + correctionDelta].
	scanner atEnd ifTrue: [^prevMark + prevToken + correctionDelta].
	^prevMark + prevToken - 1 + correctionDelta
!

reset
	"Reinitialize the scanner and the parse."

	scanner reset.
	prevMark := hereMark := scanner mark.
	self advance
!

startOfNextToken
	"return starting position in source of next token"

	hereType == #doIt ifTrue: [^scanner position + 1 + correctionDelta].
	^hereMark + correctionDelta
! !

!RecursiveDescentParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/RecursiveDescentParser.st,v 1.6 2002/11/20 13:47:13 cg Exp $'
! !
