"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

AbstractParser subclass:#TableDrivenParser
	instanceVariableNames:'parseTable transcript treeBuilder'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Compilers-Parsers'
!

TableDrivenParser class instanceVariableNames:'parseTable'

"
 No other class instance variables are inherited by this class.
"
!

TableDrivenParser comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I am an abstract class representing table (FSA) driven parsers.


Instance Variables:
	parseTable      <LL1ParserTable | LRParserState> - basic parsing mechanism.
	transcript              <TranslatorGenerator | UndefinedObject> - status messages get sent here.
	treeBuilder     <ParseTreeBuilder> - used in the construction of abstract syntax trees.'
!


!TableDrivenParser class methodsFor:'instance creation'!

new

	^super new init
! !

!TableDrivenParser class methodsFor:'class initialization'!

initialize
	"Concrete subclasses must somehow provide a parse table. Subclasses created 
	by automatic means may simply 'plug-in' a dynamically computed parse table. 
	However, if a class that can be filed-out is desired then it is worthwhile to 
	override this initialization method with one that can build the appropriate parse 
	table directly."
	"TableDrivenParser initialize"

	self parseTable: nil
! !

!TableDrivenParser class methodsFor:'state accessing'!

parseTable

	^parseTable
!

parseTable: argument 

	parseTable := argument
! !

!TableDrivenParser methodsFor:'converting'!

spaceOptimize

	self parseTable spaceOptimize
! !

!TableDrivenParser methodsFor:'exception handling'!

abort

	| block |
	block := self failBlock.
	self failBlock: nil.
	^block value
!

exceptionHandlers
        "Answer a HandlerCollection that will catch and handle scanner and parser errors."

        | handlers |
        handlers := HandlerCollection new.
        handlers on: self scannerErrorSignal
                handle: 
                        [:ex |  
                        self breakPoint:#tgen.
                        self requestor notNil
                                ifTrue: [self requestor insertAndSelect: 'SCANNER ERROR: ' , ex description , ' ->' at: self scanner errorPosition.
                                         "self requestor activate"].
                        self abort].
        handlers on: self parserErrorSignal
                handle: 
                        [:ex | 
                        self breakPoint:#tgen.
                        self requestor notNil
                                ifTrue: [self requestor insertAndSelect: '<- PARSER ERROR: ' , ex description at: self scanner errorPosition.
                                         "self requestor activate"].
                        self abort].
        ^handlers
!

parserErrorSignal

	self subclassResponsibility
!

scannerErrorSignal

	^FSAState noTransitionSignal
! !

!TableDrivenParser methodsFor:'initialization'!

init

	super init.
	self parseTable: self myParseTable.
	self treeBuilder: self treeBuilderClass new
! !

!TableDrivenParser methodsFor:'parsing'!

parse

	self subclassResponsibility
!

parseForAST

	| builder |
	builder := self treeBuilder reset.
	^self parseWithTreeBuilder: builder
!

parseForDerivationTree

	^self parseWithTreeBuilder: self derivationTreeBuilderClass new
!

parseForShamAST

	| builder |
	builder := self treeBuilder reset.
	builder setShamMode.
	^self parseWithTreeBuilder: builder
!

traceParse

	self subclassResponsibility
! !

!TableDrivenParser methodsFor:'private'!

derivationTreeBuilderClass

	^DerivationTreeBuilder
!

myParseTable

	^self class parseTable
!

scannerClass
	"Translator generator tools may initially create an 'abstract' parser and 'plug-in' 
	a scanner. This allows instances of these abstract parsers to be used in this 
	fashion. Ultimately, the tools will create concrete scanner and parser classes 
	with the proper links established."

	^Object
!

treeBuilderClass
	"Different tree builders can either be plugged in or subclasses can override this 
	method."

	^AbstractSyntaxTreeBuilder
! !

!TableDrivenParser methodsFor:'public access'!

parse: aString ifFail: aBlock 

	self failBlock: aBlock.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self parse]
!

parseAndTrace: aString ifFail: aBlock 

	self failBlock: aBlock.
	"Make sure we don't accidently write to someone else's window."
	self transcript: nil.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self traceParse]
!

parseAndTrace: aString on: aTranscript ifFail: aBlock 

	self failBlock: aBlock.
	self transcript: aTranscript.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self traceParse]
!

parseForAST: aString ifFail: aBlock 

	self failBlock: aBlock.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self parseForAST]
!

parseForDerivationTree: aString ifFail: aBlock 

	self failBlock: aBlock.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self parseForDerivationTree]
!

parseForShamAST: aString ifFail: aBlock 

	self failBlock: aBlock.
	self exceptionHandlers
		handleDo: 
			[self initScannerSource: aString.
			^self parseForShamAST]
! !

!TableDrivenParser methodsFor:'scanner/parser generation'!

classInitializationMethodTextForClassNamed: name spec: grammarSpec 

	^self subclassResponsibility
!

createParserClassNamed: name category: category spec: grammarSpec 

	| parserClass |
	parserClass := self defaultParserClass
				subclass: name asSymbol
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: category.
	parserClass comment: self generatedParserClassComment.
	parserClass class compile: (self classInitializationMethodTextForClassNamed: name spec: grammarSpec)
		classified: 'class initialization'.
	parserClass initialize.
	^parserClass
!

createScannerClassNamed: name category: category spec: tokenSpec 

	^self scanner
		createScannerClassNamed: name
		category: category
		spec: tokenSpec
!

createScannerParserClassesNamed: namePrefix category: category tokenSpec: tokenSpec grammarSpec: grammarSpec 
	| parserClass |
	self
		createScannerClassNamed: namePrefix , 'Scanner'
		category: category
		spec: tokenSpec.
	parserClass := self
				createParserClassNamed: namePrefix , 'Parser'
				category: category
				spec: grammarSpec.
	parserClass compile: 'scannerClass
	^' , namePrefix , 'Scanner' classified: 'private'.
"/        parserClass compile: 'treeBuilderClass
"/        ^' , self treeBuilder class printString classified: 'private'
	parserClass compile: 'treeBuilderClass
	^' , self treeBuilder class name classified: 'private'

    "Created: 5.2.1996 / 21:36:33 / stefan"
!

defaultParserClass

	^self class
!

generatedParserClassComment

	^'This parser class was automatically generated by ', TranslatorGenerator versionName , '.'
!

newStreamForMethodRendering
 | ws |
 ws := WriteStream on: (String new: 2048).
 Smalltalk isSmalltalkX ifFalse:[
   ws policy printCharactersLiterally: true.
 ].
 ^ws
! !

!TableDrivenParser methodsFor:'scanning'!

endOfInput
	"Use the eof token type."

	^self endOfInputTokenType
!

endOfInputTokenType
	"Answer the token type used by my scanner to represent the end of the input."

	^self scanner endOfInputTokenType
! !

!TableDrivenParser methodsFor:'state accessing'!

parseTable

	^parseTable
!

parseTable: argument 

	parseTable := argument
!

requestor

	^requestor
!

requestor: argument 

	requestor := argument
!

transcript

	^transcript
!

transcript: argument 

	transcript := argument
!

treeBuilder

	^treeBuilder
!

treeBuilder: argument 

	treeBuilder := argument
! !

!TableDrivenParser methodsFor:'testing'!

performsLeftmostDerivation
	"This is the default, let subclasses override."

	^false
!

performsRightmostDerivation
	"This is the default, let subclasses override."

	^false
! !

!TableDrivenParser methodsFor:'tracing'!

cr

	self show: '
'
!

defaultTranscript

	^Transcript
!

show: aString 

	(self transcript isNil
		ifTrue: [self defaultTranscript]
		ifFalse: [self transcript])
		show: aString
!

showCR: aString 

	self show: aString , '
'
! !

!TableDrivenParser class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TableDrivenParser.st,v 1.9 2006/04/06 10:52:46 stefan Exp $'
! !

TableDrivenParser initialize!
