"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

GrammarProduction subclass:#TransductionGrammarProduction
	instanceVariableNames:'translationSymbol'
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Scanning/Parsing'
!

TransductionGrammarProduction comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Tgen::ReadMe tgenCopyright."
=================================================

I add a translation attribute to context-free grammar productions so that I can be used to build simple transduction grammars (or syntax-directed translation scheme).  Transduction grammars are used to build abstract syntax trees rather than derivation trees during parsing.  For more information, refer to Chapter 7. ("Syntax-Directed Translation") in {\em Compiler Construction:  Theory and Practice} by Barrett, Bates, Gustafson, and Couch.

Instance Variables:
	translationSymbol <String> - used as basis for translation node when parsing.'
!


!TransductionGrammarProduction class methodsFor:'instance creation'!

leftHandSide: arg1 rightHandSide: arg2 translationSymbol: arg3 

	| newMe |
	newMe := self new.
	newMe leftHandSide: arg1.
	newMe rightHandSide: arg2.
	newMe translationSymbol: arg3.
	^newMe
! !

!TransductionGrammarProduction methodsFor:'conversion'!

asInitialLR0Item

	^self lr0ItemClass
		leftHandSide: self leftHandSide
		preDotSymbols: OrderedCollection new
		postDotSymbols: self rightHandSide copy
		translationSymbol: self translationSymbol
!

asInitialLR1ItemWithLookahead: terminal 

	^self lr1ItemClass
		leftHandSide: self leftHandSide
		preDotSymbols: OrderedCollection new
		postDotSymbols: self rightHandSide copy
		lookahead: terminal
		translationSymbol: self translationSymbol
!

asNonLalrSuffixedProduction
	"Assuming I am of the form 'A.<stuff1>* -> B.<stuff2>* C.<stuff3>*', 
	answer the prefix production 'A -> B C'."

	| separator lhs rhs |
	separator := self symbolSuffixSeparatorChar.
	lhs := self leftHandSide copyUpTo: separator.
	rhs := self rightHandSide collect: [:sym | sym copyUpTo: separator].
	^self species
		leftHandSide: lhs
		rightHandSide: rhs
		translationSymbol: self translationSymbol
! !

!TransductionGrammarProduction methodsFor:'parse tree building'!

computeResultNodeFor: builder withArgNodes: nodes 
	"Three kinds of translation symbols are currently supported: node names, special 
	directives, and arbitrary message selectors. For a node name, a new instance of 
	the specified node is created and given nodes, if any, as its children. The special 
	directive 'nil' simply returns nil. The directive liftRightChild adds any nodes 
	preceeding the right-most node as children to the right-most node, and returns 
	the right-most node. The directive liftLeftChild works in an analogous fashion. 
	Arbitrary message selectors must take the number of arguments in nodes and 
	are invoked as a builder message, thus allowing users to define their own 
	tree-building messages."

	| symbol node |
	symbol := self translationSymbol asSymbol.
	symbol first isUppercase ifTrue: [^nodes isEmpty
			ifTrue: [builder makeNewNode: symbol]
			ifFalse: [builder makeNewNode: symbol withChildren: nodes]].
	symbol = self epsilonSymbol ifTrue: [^builder answerNil].
	symbol = self rightLiftSymbol
		ifTrue: 
			[nodes size < 2 ifTrue: [self error: 'Only use liftRightChild when there are at least two right-hand-side nonterminals.'].
			"special case for building lists ending with epsilon"
			(nodes size = 2 and: [nodes last isNil])
				ifTrue: [^builder answerArgument: nodes first].
			node := nodes removeLast.
			^builder addChildrenFirst: nodes to: node].
	symbol = self leftLiftSymbol
		ifTrue: 
			[nodes size < 2 ifTrue: [self error: 'Only use liftLeftChild when there are at least two right-hand-side nonterminals.'].
			"special case for building lists beginning with epsilon"
			(nodes size = 2 and: [nodes first isNil])
				ifTrue: [^builder answerArgument: nodes last].
			node := nodes removeFirst.
			^builder addChildrenLast: nodes to: node].
	symbol numArgs = nodes size ifFalse: [self error: 'Translation message selectors must have the same number of arguments as right-hand-side nonterminals.'].
	nodes isEmpty ifTrue: [^builder perform: symbol].
	"It may be more efficient to check the number of arguments and use 
	perform:with:, etc., but probably not."
	^builder perform: symbol withArguments: nodes asArray
!

computeResultNodeFor: builder withTokenClassValue: value 
	"I am assumed to be a production of the form 'A -> <tc> => symbol'. 
	The symbol can be either a node name or a one-argument message selector. 
	If it is a node name then create a new instance of that node with the specified 
	attribute value. If it is a message selector then invoke the corresponding 
	operation on the builder with the specified value."

	| symbol |
	symbol := self translationSymbol asSymbol.
	symbol first isUppercase
		ifTrue: [^builder makeNewNode: symbol withAttribute: value]
		ifFalse: [symbol numArgs = 1
				ifTrue: [^builder perform: symbol with: value]
				ifFalse: [self error: 'Expected either a node name or a one argument
message selector as a translation symbol.']]
! !

!TransductionGrammarProduction methodsFor:'printing'!

printOn: aStream 

	super printOn: aStream.
	aStream nextPutAll: ' {'.
	self printSymbol: self translationSymbol asSymbol on: aStream.
	aStream nextPutAll: '} '
! !

!TransductionGrammarProduction methodsFor:'private'!

epsilonSymbol

	^#nil
!

leftLiftSymbol

	^#liftLeftChild
!

rightLiftSymbol

	^#liftRightChild
! !

!TransductionGrammarProduction methodsFor:'reconstructing'!

constructItsContentOn: aStream using: tokenTable 
	"Emit  lhs , #( rhs ) and translationSymbol on aStream"

	super constructItsContentOn: aStream using: tokenTable.
	(tokenTable indexOf: self translationSymbol)
		reconstructOn: aStream
! !

!TransductionGrammarProduction methodsFor:'state accessing'!

translationSymbol

	^translationSymbol
!

translationSymbol: argument 

	translationSymbol := argument
! !

!TransductionGrammarProduction methodsFor:'testing'!

hasTranslation

	^true
! !

!TransductionGrammarProduction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TransductionGrammarProduction.st,v 1.5 2002/11/20 13:48:36 cg Exp $'
! !
