"{ Package: 'stx:goodies/tgen' }"

"{ NameSpace: Tgen }"

Object subclass:#TreeNode
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Tgen-Collections-Graph Nodes'
!

TreeNode comment:'=================================================
    Copyright (c) 1992 by Justin O. Graver.
    All rights reserved (with exceptions).
    For complete information evaluate "Object tgenCopyright."
=================================================

This abstract class provides the framework for both destructive and non-destructive tree traversals in which references to locally global objects are available without being explicitly passed as arguments.

Concrete subclasses must implement methods for traversing

	childrenDo:
		"Evaluate the argument block with each of my children."

	updateChildrenUsing:
		"Replace my children with the result of evaluating the argument block with the corresponding child."'
!


!TreeNode methodsFor:'copying'!

copyTree
	"Answer a copy of this tree."

	^self copy updateChildrenUsing: [:child | child copyTree]
! !

!TreeNode methodsFor:'enumerating'!

postorderDo: aBlock
	"Perform a postorder traversal on myself and my children.
	This message may be used for examining the nodes of a tree
	for the purpose of gathering data or altering data fields.
	To alter the structure of the tree see traverseDo:.  One of
	the main advantages of this message is that it allows all nodes
	of the tree 'global' access to objects referenced in aBlock.
	Before, such arguments had to be passed explitely as arguments.
	This message may be used as follows.

	aMethodNode postorderDo: [:node | node enc: encoder root: self]"

	self childrenDo: [:child | child postorderDo: aBlock].
	aBlock value: self
!

preorderDo: preBlock postorderDo: postBlock 
	"Perform a traversal on myself and my children.  The preBlock is 
	evaluated when entering a node and postBlock is evaluated just before 
	leaving.  See comment in postorderDo:."

	preBlock value: self.
	self childrenDo: [:child | child preorderDo: preBlock postorderDo: postBlock].
	postBlock value: self
! !

!TreeNode methodsFor:'traversing'!

childrenDo: aBlock
	"Evaluate aBlock for each of my children.
	This message should be reimplemented by my subclasses."

	^self           "default"
!

preorderDo: preBlock updateUsing: postBlock 
	"Perform a traversal on myself and my children.  The preBlock is
	evaluated when first entering a node.  My children are replaced
	with the results of the traversal.  Thus, this message can be used
	to generate objects or alter my structure, whereas postorderDo:
	can only be used to examine my structure.  This message may be
	used in the following manner. 
         
	a := aMethodNode
		preorderDo: [:node | node msg1]
		updateUsing: [:node | node msg2: globalRef]"

	preBlock value: self.
	self updateChildrenUsing: [:child | child preorderDo: preBlock updateUsing: postBlock].
	^postBlock value: self
!

updateChildrenUsing: aBlock
	"Replace my children according to the value of aBlock.
	This message should be reimplemented by my subclasses."

	^self           "default"
!

updateCopyUsing: aBlock 
	"Perform a postorder traversal on a copy of myself and
	my children, replacing my children with the results of the traversal. 
	Thus, this message can be used to generate objects or alter 
	my structure, whereas postorderDo: can only be used to examine 
	my structure.  This message may be used in the following manner. 
         
	a := aMethodNode updateCopyUsing: [:node | node msg: globalRef]"

	| newNode |
	newNode := self copy.
	newNode updateChildrenUsing: [:child | child updateCopyUsing: aBlock].
	^aBlock value: newNode
!

updateUsing: aBlock 
	"Perform a postorder traversal on myself and my children, 
	replacing my children with the results of the traversal. 
	Thus, this message can be used to generate objects or alter 
	my structure, whereas postorderDo: can only be used to examine 
	my structure.  This message may be used in the following manner. 
         
	a := aMethodNode updateUsing: [:node | node msg: globalRef]"

	self updateChildrenUsing: [:child | child updateUsing: aBlock].
	^aBlock value: self
! !

!TreeNode class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tgen/TreeNode.st,v 1.5 2002/11/20 13:48:53 cg Exp $'
! !
