"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailDBFile variableSubclass:#CategoriesFile
	instanceVariableNames:'categories'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

CategoriesFile comment:'I represent the organization of the mail database into set of message lists called "categories".  Each category contains a collection of message ID''s.  The same message may be cross-filed quite cheaply by storing it''s ID in multiple categories.

The categories information is kept in a binary file on the disk.  It is read into memory in its entirety when the mail database is opened.  To make changes persist, the categories information must be saved out to disk.  This should be done after fetching new mail and when the mail database is closed.  It could also be done periodically by some sort of background process.

Note that the categories file, unlike the index file, cannot be re-created from the messages file.
'
!


!CategoriesFile methodsFor:'categories access'!

addCategory: categoryName 
	"Add a new category, if it doesn't already exist."
	(self categories includes: categoryName)
		ifFalse: [categories at: categoryName put: PluggableSet integerSet]
!

categories
	"Answer a collection of my categories, including the pseudo-categories '.unclassified.' and '.all.'. '.unclassified.' contains the orphaned messages that would otherwise not appear in any category. '.all.' contains all the messages in the database. Since these pseudo-categories are computed on the fly, there may be a noticable delay when one of them is selected."

	^(categories keys)
		add: '.all.';
		add: '.unclassified.';
		yourself
!

file: messageID inCategory: categoryName 
	"Add the given message ID to the given category. The target category  
	must be a real category, not a pseudo-category."

	(categoryName = '.unclassified.') | (categoryName = '.all.') ifTrue: [^ self].
	self addCategory: categoryName.
	(categories at: categoryName)
		add: messageID
!

isUnclassified: messageID
	"Answer true if the given message ID does not appear in any of my real (not pseudo) categories."

	categories do:
		[: category | (category includes: messageID) ifTrue: [^false]].
	^true
!

messagesIn: category
	"Answer a collection of message ID's for the messages in the given category. The pseudo-categories are dynamically computed and so they cannot be accessed in this manner."

	^categories at: category ifAbsent: [#()]
!

remove: messageID fromCategory: categoryName
	"Remove the given message ID from the given category."

	| msgList |
	msgList _ categories at: categoryName ifAbsent: [^self].
	msgList remove: messageID ifAbsent: [].
!

removeCategory: categoryName
	"Remove the given category, if it exists."

	categories removeKey: categoryName ifAbsent: [].
!

removeMessagesInCategory: categoryName butNotIn: indexFile
	"Used to clean the dead wood out of a category."

	| oldMsgs newMsgs |
	oldMsgs _ categories at: categoryName ifAbsent: [^self].
	newMsgs _ oldMsgs copy.
	oldMsgs do:
		[: msgID |
		 (indexFile includesKey: msgID) ifFalse:
			[newMsgs remove: msgID]].
	categories at: categoryName put: newMsgs.
!

renameCategory: oldName to: newName 
	"Rename the given category."

	| oldEntry |

	"can't rename a special category or overwrite an existing one"
	(oldName = '.all.') | (oldName = '.unclassified.') | (self categories includes: newName)
		ifTrue: [^ self].

	oldEntry _ categories removeKey: oldName ifAbsent: [PluggableSet integerSet].
	categories at: newName put: oldEntry
!

unclassifiedFrom: messageIDs
	"Answer the subset of the given set of message ID's that do not appear in any category."

	^messageIDs select:
		[: msgID | self isUnclassified: msgID]
! !

!CategoriesFile methodsFor:'read-write'!

readFrom: aFileStream 
	"Read the categories from the given FileStream."
	| name categorySize messageIDs |
	categories _ Dictionary new: 64.
	aFileStream binary; position: 0.
	[aFileStream atEnd]
		whileFalse: 
			[name _ aFileStream ascii; nextString.
			categorySize _ aFileStream binary; nextWord.
			messageIDs _ PluggableSet integerSet.
			categorySize timesRepeat: [messageIDs add: aFileStream nextInt32].
			categories at: name put: messageIDs]
!

writeOn: aFileStream
	"Write the categories to the given FileStream. The categories data is stored in binary (as opposed to a human-readable form) to save space."

	aFileStream binary; position: 0.
	categories associationsDo:
		[: category |
		 "(category key) is the category name"
		 "(category value) is the set of message ID's in that category"
		 aFileStream nextStringPut: (category key).
		 aFileStream nextWordPut: (category value) size.
		 (category value) do:
			[: messageID |
			 aFileStream nextInt32Put: messageID]].
! !

!CategoriesFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/CategoriesFile.st,v 1.1 2003/05/09 21:40:34 cg Exp $'
! !
