"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

Model subclass:#Celeste
	instanceVariableNames:'mailDB currentCategory currentMessages currentMsgID lastCategory
		subjectFilter participantFilter customFilterBlock messageTextView
		userPassword status tocLists'
	classVariableNames:'CCList CustomFilters CustomFiltersCompiled
		DeleteInboxAfterFetching FormatWhenFetching MessageCountLimit
		PopServer PopUserName SuppressWorthlessHeaderFields TimeZone'
	poolDictionaries:''
	category:'Network-Mail Reader'
!

Celeste comment:'I am the core of a mail reading and organizing program. The name "Celeste" is a reference to an earlier mail reader named "Babar", which was written at Xerox PARC by Steve Putz and John Maloney.

This object provides a user interface and some higher-level functionality for the application. The foundation of of the mail reader is really the mail database, implemented by the class MailDB.

Implementation note: the ''mailDB'' instance variable can be nil.  This way, it is possible to have a Celeste window with no DB currently loaded.  All user-accessible functions that access mailDB should be careful to quietly do nothing if mailDB is nil.'
!


!Celeste class methodsFor:'instance creation'!

current
	"Answer the currently active Celeste (assuming that there's
	only one Celeste open at a given time) or open a new one."

	Smalltalk garbageCollect.
	^Celeste allInstances detect: [:e | e isActive] ifNone: [self open].
!

defaultDBName
	"return the default database name to access"
	^'EMAIL'
!

onDatabase: aMailDB
	"open a Celeste on the given MailDB"
	^super new openOnDatabase: aMailDB
!

open
	"Open a MailReader on the default mail database."

	^ self openOn: self defaultDBName
!

openOn: rootFilename
	"Open a MailReader on the mail database with the given root filename."
	|database |

	(MailDB isADBNamed: rootFilename)
		ifTrue: [ database _  MailDB openOn: rootFilename. ]
		ifFalse: [ database _ nil.  "open an empty Celeste with a welcome message" ].

	^self openOnDatabase: database
! !

!Celeste class methodsFor:'accessing'!

messageCountLimit

	^ MessageCountLimit
!

messageCountLimit: anInteger
	MessageCountLimit _ anInteger
! !

!Celeste class methodsFor:'build-common'!

defaultWindowTitle
	^ 'Celeste email browser'
! !

!Celeste class methodsFor:'build-morphic'!

addLowerMorphicViews: views andButtons: buttons to: topWindow offset: offset 
	| row verticalOffset innerFractions |
	row _ AlignmentMorph newColumn hResizing: #spaceFill;
				 vResizing: #spaceFill;
				 layoutInset: 0;
				 borderWidth: 1;
				 borderColor: Color black;
				 layoutPolicy: ProportionalLayout new.
	verticalOffset _ 0.
	innerFractions _ 0 @ 0 corner: 1 @ 0.
	verticalOffset _ self
				addMorphicButtons: buttons
				to: row
				at: innerFractions
				plus: verticalOffset.
	self includeStatusPane ifTrue: [
	verticalOffset _ self
				addMorphicStatusPaneTo: row
				from: views
				at: innerFractions
				plus: verticalOffset].
	self
		addMorphicTextPaneTo: row
		from: views
		at: innerFractions
		plus: verticalOffset.
	topWindow
		addMorph: row
		frame: (0 @ offset extent: 1 @ (1 - offset)).
	row
		on: #mouseEnter
		send: #paneTransition:
		to: topWindow.
	row
		on: #mouseLeave
		send: #paneTransition:
		to: topWindow
!

addMorphicButtons: buttons to: row at: innerFractions plus: verticalOffset 
	| delta buttonRow |
	delta _ 25.
	buttonRow _ self morphicButtonRowFrom: buttons.
	buttonRow 
				color: (Color gray alpha: 0.2);
				 borderWidth: 1;
				 borderColor: Color lightGray.
	row addMorph: buttonRow
				fullFrame: (LayoutFrame
						fractions: innerFractions
						offsets: (0@verticalOffset corner: 0@(verticalOffset + delta))).
	^ verticalOffset + delta
!

addMorphicStatusPaneTo: row from: views at: innerFractions plus: verticalOffset 
	| delta statusFractions outputBoxFractions |
	delta _ 20.
	statusFractions _ innerFractions withRight: 0.5.
	outputBoxFractions _ (statusFractions withLeft: 0.5)
				withRight: 1.
	row
		addMorph: (views at: #status)
		fullFrame: (LayoutFrame
				fractions: statusFractions
				offsets: (0 @ verticalOffset corner: 0 @ (verticalOffset + delta))).
	row
		addMorph: (views at: #outBoxStatus)
		fullFrame: (LayoutFrame
				fractions: outputBoxFractions
				offsets: (0 @ verticalOffset corner: 0 @ (verticalOffset + delta))).
	^ verticalOffset + delta
!

addMorphicTextPaneTo: row from: views at: innerFractions plus: verticalOffset 
	row
		addMorph: (views at: #messageText)
		fullFrame: (LayoutFrame
				fractions: (innerFractions withBottom: 1)
				offsets: (0 @ verticalOffset corner: 0@0)).
	^ verticalOffset

!

addMorphicViews: views andButtons: buttons to: topWindow 
	topWindow
		addMorph: (views at: #categoryList)
		frame: (0.0 @ 0.0 extent: 0.2 @ 0.25).
	topWindow
		addMorph: (views at: #tocEntryList)
		frame: (0.2 @ 0.0 extent: 0.8 @ 0.25).
	self
		addLowerMorphicViews: views
		andButtons: buttons
		to: topWindow
		offset: 0.25 
!

buildMorphicCategoryListFor: model
	^(PluggableListMorphByItem
				on: model
				list: #categoryList
				selected: #category
				changeSelected: #setCategory:
				menu: #categoryMenu:
				keystroke: #categoriesKeystroke:) enableDragNDrop: true.
!

buildMorphicMessageTextPaneFor: model 
	^ PluggableTextMorph new
				on: model
				text: #messageText
				accept: #messageText:
				readSelection: nil
				menu: #messageMenu:shifted:
!

buildMorphicOutBoxStatusPaneFor: model 
	^ PluggableTextMorph new
		on: model
		text: #outBoxStatus
		accept: nil
		readSelection: nil
		menu: nil
!

buildMorphicStatusPaneFor: model 
	^ PluggableTextMorph new
				on: model
				text: #status
				accept: nil
				readSelection: nil
				menu: nil.
!

buildMorphicTocEntryListFor: model 
	^ (PluggableMultiColumnListMorphByItem
				on: model
				list: #tocEntryList
				selected: #tocEntry
				changeSelected: #setTOCEntry:
				menu: #tocMenu:
				keystroke: #tocKeystroke:
) enableDragNDrop: true.
!

buildMorphicViewsFor: model 
	"Answer a dictionary of window panes for the Celeste user interface."
	| listFont views v |
	listFont _ StrikeFont allSubInstances
				detect: [:f | (f name beginsWith: 'CourierFixed')
						and: [f height = 11]]
				ifNone: [TextStyle defaultFont].
	views _ Dictionary new.
	v _ self buildMorphicCategoryListFor: model.
	views at: #categoryList put: v.
	v _ self buildMorphicTocEntryListFor: model.
	v font: listFont.
	views at: #tocEntryList put: v.
	v _ self buildMorphicMessageTextPaneFor: model.
	v borderWidth: 1.
	model messageTextView: v.
	views at: #messageText put: v.
	v _ self buildMorphicStatusPaneFor: model.
	v borderWidth: 1.
	views at: #status put: v.
	v _ self buildMorphicOutBoxStatusPaneFor: model.
	v borderWidth: 1.
	views at: #outBoxStatus put: v.
	^ views
!

buildTopMorphicWindowTitled: title model: model 
	| topWindow views buttons |
	topWindow _ (SystemWindow labelled: title)
				model: model.
	buttons _ self buildButtonsFor: model.
	views _ self buildMorphicViewsFor: model.
	self addMorphicViews: views andButtons: buttons to: topWindow .
	buttons
		do: [:b | b onColor: Color lightGray offColor: Color white].
	^ topWindow
!

morphicButtonRowFrom: buttons 
	| aRow |
	aRow _ AlignmentMorph newRow.
	aRow setNameTo: 'buttonPane'.
	aRow beSticky.
	aRow hResizing: #spaceFill.
	aRow wrapCentering: #center;
		 cellPositioning: #leftCenter.
	aRow clipSubmorphs: true.
	aRow addTransparentSpacerOfSize: 5 @ 0.
	buttons
		do: [:btn | 
			btn useRoundedCorners;
			 hResizing: #spaceFill;
						 vResizing: #spaceFill.
			aRow addMorphBack: btn.
			aRow addTransparentSpacerOfSize: 3 @ 0].
	^ aRow
!

morphicButtonsClass

	^PluggableButtonMorph
! !

!Celeste class methodsFor:'build-mvc'!

addMVCViews: views andButtons: buttons to: topWindow 
	(views at: 1)
		window: (0 @ 0 extent: 20 @ 25).
	(views at: 2)
		window: (0 @ 0 extent: 80 @ 25).
	(views at: 3)
		window: (0 @ 0 extent: 100 @ 70).
	(buttons at: 1)
		window: (0 @ 0 extent: 12 @ 5).
	(buttons at: 2)
		window: (0 @ 0 extent: 12 @ 5).
	(buttons at: 3)
		window: (0 @ 0 extent: 12 @ 5).
	(buttons at: 4)
		window: (0 @ 0 extent: 10 @ 5).
	(buttons at: 5)
		window: (0 @ 0 extent: 13 @ 5).
	(buttons at: 6)
		window: (0 @ 0 extent: 13 @ 5).
	(buttons at: 7)
		window: (0 @ 0 extent: 15 @ 5).
	(buttons at: 8)
		window: (0 @ 0 extent: 13 @ 5).
	topWindow
		addSubView: (buttons at: 1);
		
		addSubView: (buttons at: 2)
		toRightOf: (buttons at: 1);
		
		addSubView: (buttons at: 3)
		toRightOf: (buttons at: 2);
		
		addSubView: (buttons at: 4)
		toRightOf: (buttons at: 3);
		
		addSubView: (buttons at: 5)
		toRightOf: (buttons at: 4);
		
		addSubView: (buttons at: 6)
		toRightOf: (buttons at: 5);
		
		addSubView: (buttons at: 7)
		toRightOf: (buttons at: 6);
		
		addSubView: (buttons at: 8)
		toRightOf: (buttons at: 7);
		
		addSubView: (views at: 1)
		below: (buttons at: 1);
		
		addSubView: (views at: 2)
		toRightOf: (views at: 1);
		
		addSubView: (views at: 3)
		below: (views at: 1)
!

buildTopMVCWindowTitled: title model: model 
	| topWindow views buttons |
	topWindow _ StandardSystemView new model: model;
				 label: title;
				 minimumSize: 400 @ 250.
	views _ self buildViewsFor: model.
	buttons _ self buildButtonsFor: model.
	self
		addMVCViews: views
		andButtons: buttons
		to: topWindow.
	^ topWindow
!

buildViewsFor: model 
	"Answer a collection of window panes for the Celeste user interface."
	| listViewClass textViewClass listFont views v multiListViewClass |
	listViewClass _ PluggableListViewByItem.
	multiListViewClass _ PluggableListViewByItem.
	textViewClass _ PluggableTextView.
	listFont _ StrikeFont allSubInstances
				detect: [:f | (f name beginsWith: 'CourierFixed')
						and: [f height = 11]]
				ifNone: [TextStyle defaultFont].
	views _ OrderedCollection new.
	v _ listViewClass
				on: model
				list: #categoryList
				selected: #category
				changeSelected: #setCategory:
				menu: #categoryMenu:
				keystroke: #categoriesKeystroke:.
	views add: v.
	v _ multiListViewClass
				on: model
				list: #tocEntryList
				selected: #tocEntry
				changeSelected: #setTOCEntry:
				menu: #tocMenu:
				keystroke: #tocKeystroke:.
	v font: listFont.
	views add: v.
	v _ textViewClass new
				on: model
				text: #messageText
				accept: #messageText:
				readSelection: nil
				menu: #messageMenu:shifted:.
	v borderWidth: 1.
	model messageTextView: v.
	views add: v.
	v _ textViewClass new
				on: model
				text: #status
				accept: nil
				readSelection: nil
				menu: nil.
	v borderWidth: 1.
	model messageTextView: v.
	views add: v.
	^ views
! !

!Celeste class methodsFor:'button specs'!

specForComposeButton
	"action label helpText"
	^ #(#turnOn 'New' 'Compose a new message' )
!

specForCustomFilterButton
	"getState action label helpText"
	^ #(#isCustomFilterOn #customFilterOn 'Custom F.' 'Select messages with a general custom filter' )
!

specForCustomFilterMoveButton
	"getState action label helpText"
	^ #(#nil #customFilterMove 'Custom F. Move' 'Move messages which match a custom filter to appropriate category' )
!

specForDeleteButton
	"action label helpText"
	^ #(#turnOn 'Delete' 'Delete the selected message' )
!

specForForwardButton
	"action label helpText"
	^ #(#turnOn 'Forward' 'Forward the selected message' )
!

specForMoveAgainButton
	"action label helpText"
	^ #(#turnOn 'Move Again' 'Move the selected message to the same category as previously' )
!

specForParticipantFilterButton
	"getState action label helpText"
	^ #(#isParticipantFilterOn #participantFilterOn 'Part. F.' 'Select messages by their From:, To:, and Cc: lines' )
!

specForReplyButton
	"action label helpText"
	^ #(#turnOn 'Reply' 'Reply to the selected message' )
!

specForSubjectFilterButton
	"getState action label helpText"
	^ #(#isSubjectFilterOn #subjectFilterOn 'Subj. F.' 'Select messages by their subject' )
! !

!Celeste class methodsFor:'class initialization'!

initialize
        "Celeste initialize"
        "user preferences"
        CCList _ nil.
        DeleteInboxAfterFetching _ false.
        PopServer _ nil.
        PopUserName _ nil.
        SuppressWorthlessHeaderFields _ true.

        "options with no UI; just set their values directly"
        FormatWhenFetching _ false.

        "dictionary of custom filters"
        CustomFilters _ Dictionary new.

        MessageCountLimit _ 200.

        Smalltalk isSmalltalkX ifFalse:[
            "Add global preferences"
            Preferences addPreferenceForOptionalCelesteStatusPane.
            Preferences addPreferenceForCelesteShowingAttachmentsFlag.
        ]
!

registerInOpenMenu
	"Register the receiver in the system's open menu"

	TheWorldMenu registerOpenCommand: 
		{ 'email reader' . { Celeste . #open }. '"Celeste", an e-mail client' }
	
!

unload
	"Unload the receiver from global registries"

	TheWorldMenu unregisterOpenCommandWithReceiver: self
!

versionString
	"Answer a short string describing this version of Celeste."

	| highestChangeSet versionAddendum |

	"the changeset number should probably be removed whenever Celeste settles down"
	highestChangeSet _ SystemVersion current highestUpdate.
	versionAddendum _ highestChangeSet
		ifNil: ['.x']
		ifNotNil: ['.' , highestChangeSet name initialIntegerOrNil printString].
	^ 'Celeste 2.0' , versionAddendum
! !

!Celeste class methodsFor:'common build'!

buildButtonFromSpec: spec forModel: model 
	| buttonViewClass b |
	Smalltalk isMorphic
		ifTrue: [buttonViewClass _ self morphicButtonsClass]
		ifFalse: [buttonViewClass _ PluggableButtonView].
	b _ buttonViewClass
				on: model
				getState: (self specificationFromList: spec at: 1)
				action: (self specificationFromList: spec at: 2).
	b
		label: (self specificationFromList: spec at: 3);
		 borderWidth: 1.
	b
		setBalloonText: (self specificationFromList: spec at: 4).
	^ b
!

buildButtonFromSpec: spec withBlock: aBlock 
	| b buttonViewClass |
	Smalltalk isMorphic
		ifTrue: [buttonViewClass _ self morphicButtonsClass]
		ifFalse: [buttonViewClass _ PluggableButtonView].
	b _ buttonViewClass new
				model: (Button new onAction: aBlock);
				
				action: (self specificationFromList: spec at: 1);
				
				label: (self specificationFromList: spec at: 2);
				 borderWidth: 1.
	b
		setBalloonText: (self specificationFromList: spec at: 3).
	^ b
!

buildButtonsFor: model 
	"Answer a collection of handy buttons for the Celeste user interface."
	| buttons b |
	buttons _ OrderedCollection new.
	b _ self buildButtonFromSpec: self specForSubjectFilterButton forModel: model.
	buttons add: b.
	b _ self buildButtonFromSpec: self specForParticipantFilterButton forModel: model.
	buttons add: b.
	b _ self buildButtonFromSpec: self specForCustomFilterButton forModel: model.
	buttons add: b.
	true
		ifFalse: ["Skip these buttons..."
	b _ self buildButtonFromSpec: self specForCustomFilterMoveButton forModel: model.
	buttons add: b].
	b _ self
				buildButtonFromSpec: self specForComposeButton
				withBlock: [model compose].
	buttons add: b.
	b _ self
				buildButtonFromSpec: self specForReplyButton
				withBlock: [model reply].
	buttons add: b.
	b _ self
				buildButtonFromSpec: self specForForwardButton
				withBlock: [model forward].
	buttons add: b.
	b _ self
				buildButtonFromSpec: self specForMoveAgainButton
				withBlock: [model moveAgain].
	buttons add: b.
	b _ self
				buildButtonFromSpec: self specForDeleteButton
				withBlock: [model deleteMessage].
	buttons add: b.
	^ buttons
!

newOpenableMorph
	"Answer window for a MailReader on a blank database."

	^ (self buildTopMorphicWindowTitled: self defaultWindowTitle model: (self onDatabase: nil)) applyModelExtent
!

openOnDatabase: aMailDB 
	"Open a MailReader on the given mail database."
	| model topWindow title |
	model _ self onDatabase: aMailDB.
	title _ self defaultWindowTitle.
	Smalltalk isMorphic
		ifTrue: [topWindow _ self
						buildTopMorphicWindowTitled: title
						model: model.
			topWindow openInWorld]
		ifFalse: [topWindow _ self buildTopMVCWindowTitled: title model: model.
			topWindow controller open].
	"in case the sender wants to know"
	^ model
!

specificationFromList: list at: index 
	| value |
	value _ list at: index.
	value = #nil
		ifTrue: [value _ nil].
	^ value
! !

!Celeste class methodsFor:'filters'!

makeFilterFor: filterExpr
	"compile a given custom filter"
	^Compiler evaluate: '[ :m | ', filterExpr, ']'.

! !

!Celeste class methodsFor:'options'!

includeStatusPane

	^Preferences celesteHasStatusPane
!

showAttachmentsFlag
	^ Preferences celesteShowsAttachmentsFlag
! !

!Celeste class methodsFor:'sending'!

eudoraSeparator
	"Return a Eudora-style message separator string."

	| s today dateString |
	s _ WriteStream on: (String new: 50).
	today _ Date today.
	dateString _ today printFormat: #(2 1 3 32 2 1).
	dateString _ dateString copyFrom: 1 to: dateString size - 4.
	s nextPutAll: 'From ???@??? '.
	s nextPutAll: (today weekday copyFrom: 1 to: 3); space.
	s nextPutAll: dateString.
	Time now print24: true on: s.
	s space.
	s print: today year; cr.
	^s contents
! !

!Celeste class methodsFor:'user preferences'!

ccList
	"Answer the default cc list to be used in composing messages."

	CCList isNil ifTrue: [CCList _ ''].
	^CCList
!

popServer
	"Answer the server for downloading email via POP"

	(PopServer isNil or: [PopServer isEmpty])
		ifTrue: [self setPopServer].

	PopServer isEmpty ifTrue: [
		self error: 'POP server not specified' ].

	^PopServer
!

popUserName
	"Answer the user's username on the POP server"

	(PopUserName isNil or: [PopUserName isEmpty])
		ifTrue: [self setPopUserName].

	PopUserName isEmpty ifTrue: [ 
		self error: 'no POP user name specified' ].

	^PopUserName
!

setCCList
	"Change the list of names used in the default cc list. Items in the list should be valid mail addresses and should be separated by commas."

	| newList |
	(CCList isNil) ifTrue: [CCList _ ''].
	newList _ FillInTheBlank
		request: 'addresses to automatically add to CC: fields?'
		initialAnswer: CCList.
	CCList _ newList.
!

setPopServer
	"Change the user's email name for use in composing messages."

	(PopServer isNil) ifTrue: [PopServer _ ''].
	PopServer _ FillInTheBlank
		request: 'What is your POP server''s hostname?'
		initialAnswer: PopServer.
!

setPopUserName
	"set the POP server used for downloading email"

	(PopUserName isNil) ifTrue: [PopUserName _ ''].
	PopUserName _ FillInTheBlank
		request: 'What is your username on your POP server?'
		initialAnswer: PopUserName.

	"be kind, if they include the host name here"
	(PopUserName includes: $@) ifTrue: [
		PopUserName _ PopUserName copyFrom: 1 to: (PopUserName indexOf: $@)-1 ].
!

setTimeZone
	TimeZone _ FillInTheBlank request: 'What is your time zone ?' initialAnswer: '+0300'
!

userTimeZone
	"Answer the user's timezone string to be used when sending messages."

	TimeZone isEmptyOrNil ifTrue: [self setTimeZone].
	^ TimeZone ifNil: ['']
! !

!Celeste methodsFor:'accessing'!

tocLists
	"Generated - Return the value of tocLists."

	tocLists == nil ifTrue: [self initializeTocLists].
	^tocLists
! !

!Celeste methodsFor:'categories pane'!

addCategory
	"Create a new category with the user-specified name. This does nothing if the category already exists."

	| newCatName |
	mailDB ifNil: [ ^self ].
	newCatName _ FillInTheBlank request: 'Name for new category?'.
	(newCatName isEmpty) ifTrue: [^self].	"user aborted"

	self requiredCategory: newCatName.
	self setCategory: newCatName.
!

cacheTOC
	"Caches a version of the TOC"
	| tocStringColumns |
	self initializeTocLists.

	'Processing ' , currentMessages size printString , ' messages.'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: currentMessages size
		during: [:bar | 1
				to: currentMessages size
				do: [:i | 
					bar value: i.

					(self tocLists at: 1)
						add: i printString.

					"columns from the database are 5"
					tocStringColumns _ mailDB
								getTOCstringAsColumns: (currentMessages at: i).

					(self tocLists at: 2)
						add: ((tocStringColumns at: 5)
								ifTrue: ['@']
								ifFalse: [' ']).
					(self tocLists at: 3)
						add: (tocStringColumns at: 1).
					(self tocLists at: 4)
						add: (tocStringColumns at: 2).
					(self tocLists at: 5)
						add: (tocStringColumns at: 4).
					(self tocLists at: 6)
						add: (tocStringColumns at: 3)]].

	(currentMessages includes: currentMsgID)
		ifFalse: [currentMsgID _ nil]
!

categoriesKeystroke: aCharacter

	aCharacter asciiValue = 30 ifTrue: [self previousCategory].
	aCharacter asciiValue = 31 ifTrue: [self nextCategory].
!

category
	"Answer the currently selected category or nil."

	^currentCategory
!

categoryList
	"Answer a list of categories for the categories pane."
	mailDB ifNil: [ ^#() ].
	^ mailDB allCategories
!

categoryMenu: aMenu 
	"Answer the menu for the categories pane."
	aMenu add: 'save' action: #save.
	aMenu balloonTextForLastItem: 'Save the database'.
	aMenu addLine.
	aMenu add: 'fetch mail' action: #fetchMail.
	aMenu balloonTextForLastItem: 'Fetch new mail from the server'.
	aMenu add: 'send queued mail' action: #sendQueuedMail.
	aMenu balloonTextForLastItem: 'Send newly written mail'.
	aMenu addLine.
	aMenu add: 'add category' action: #addCategory.
	aMenu balloonTextForLastItem: 'Add a new organizational category'.
	currentCategory notNil
		ifTrue: [aMenu add: 'view all messages' action: #viewAllMessages.
			aMenu balloonTextForLastItem: 'View all the messages'].
	"add extra commands if a normal category is selected"
	(currentCategory notNil
			and: [currentCategory ~= '.all.' & (currentCategory ~= '.unclassified.')])
		ifTrue: [aMenu add: 'edit category filter' action: #editCategoryFilter.
			aMenu balloonTextForLastItem: 'Edit a custom filter for this category'.
			aMenu add: 'rename category' action: #renameCategory.
			aMenu balloonTextForLastItem: 'Rename this organizational category'.
			aMenu add: 'remove category' action: #removeCategory.
			aMenu balloonTextForLastItem: 'Remove this organizational category
(NB: all messages will be safely available in other categories)'.
			aMenu addLine.
			aMenu add: 'import into category' action: #importIntoCategory.
			aMenu balloonTextForLastItem: 'Import messages from a Unix/Eudora file into this category'.
			aMenu add: 'export category (Celeste)' action: #exportCategory.
			aMenu balloonTextForLastItem: 'Copy all messages from this category to another Celeste database'.
			aMenu add: 'export category (Unix/Eudora)' action: #exportCategoryUnix.
			aMenu balloonTextForLastItem: 'Write a copy of all messages from this category to a Unix/Eudora file'].
	aMenu addLine.
	aMenu add: 'empty trash' action: #emptyTrash.
	aMenu balloonTextForLastItem: 'Completely remove all messages in the category .trash. from Celeste'.
	aMenu add: 'salvage & compact' action: #compact.
	aMenu balloonTextForLastItem: 'Salvage any work done since the last database save & recover space used by old deleted messages.
(This may be a bit slow)'.
	aMenu add: 'find duplicates' action: #findDuplicates.
	aMenu balloonTextForLastItem: 'Find messages which are exact duplicates'.
	aMenu addLine.
	aMenu addUpdating: #suppressingHeadersString action: #toggleSuppressHeaders.
	aMenu balloonTextForLastItem: 'Show either a complete or an easy-to-read message header'.
	aMenu add: 'change max current messages (' , self class messageCountLimit printString , ')' action: #changeMaxMessageCount.
	aMenu addLine.
	aMenu add: 'set user name' action: #setUserName.
	aMenu balloonTextForLastItem: 'Specify the ''From:'' user name for new messages'.
	aMenu add: 'set cc: list' action: #setCCList.
	aMenu balloonTextForLastItem: 'Specify a cc: list that is added to each new message'.
	aMenu add: 'set POP server' action: #setPopServer.
	aMenu balloonTextForLastItem: 'Specify which (POP3) server to check for new messages'.
	aMenu add: 'set POP username' action: #setPopUserName.
	aMenu balloonTextForLastItem: 'Specify the username to use when checking for new messages'.
	aMenu add: 'set SMTP server' action: #setSmtpServer.
	aMenu balloonTextForLastItem: 'Specify which (SMTP) server to use when sending messages'.
	aMenu addLine.
	aMenu addUpdating: #messagesOnServerString action: #toggleKeepMessagesOnServer.
	aMenu balloonTextForLastItem: 'When true, messages are not deleted from the server when you retreive them (typically used for testing only).  When false, messages are deleted from the server after you retreive them'.
	^ aMenu
!

compact
	"Salvage and Compact the messages file."

	| stats |
	mailDB ifNil: [ ^self ].
	Transcript cr; show: 'Compacting message file...'.
	Cursor execute showWhile: [stats _ mailDB compact].
	Transcript show: 'Done.'; cr.
	Transcript show:
		'Recovered ',
		(stats at: 1) printString, ' message',
		(((stats at: 1) = 1) ifTrue: [', '] ifFalse: ['s, ']),
		(stats at: 2) printString, ' bytes.  ',
		(stats at: 3) printString, ' active messages remain.'; cr.

	self setCategory: self category.		"Flush all the displayed state"
!

emptyTrash
	"Delete all messages in the '.trash.' category.
	WARNING: The messages will be totally removed from the Celeste index, and the .messages file will be marked so that the message contents are removed when it is next compressed."

	| msgList |
	mailDB ifNil: [ ^self ].
	self requiredCategory: '.trash.'.

	msgList _ mailDB messagesIn: '.trash.'.			"Look at ALL messages in the trash"
	"Remove from the list messages which are also in other categories"
	msgList _ msgList select: [ :msgID | (mailDB categoriesThatInclude: msgID) size = 1].

	mailDB deleteAll: msgList.
	mailDB cleanUpCategories.
	self updateTOC.

	(mailDB messagesIn: '.trash.') isEmpty ifFalse:
		[self inform: 'Some messages were not removed because they are also filed in other categories'].
!

exportCategory
	"Store the filtered message list of the current category to another mail database. The user is prompted for the name of the other database."

	| destDBName destDB |
	mailDB ifNil: [ ^self ].
	currentCategory ifNil: [ ^self ].

	destDBName _ FillInTheBlank
		request: 'Destination mail database?'
		initialAnswer: ''.
	(destDBName isEmpty) ifTrue: [^self].
	destDB _ MailDB openOn: destDBName.
	(destDB isNil) ifTrue: [^self].
	destDB mergeMessages: (self filteredMessagesIn: currentCategory) from: mailDB.
	destDB saveDB.
!

exportCategoryUnix
	"Store the filtered message list of the current category into a Eudora/Unix database"
	| destFileName destFile messageIds count |
	mailDB ifNil: [ ^self ].
	currentCategory ifNil: [ ^self ].

	destFileName _ FillInTheBlank
		request: 'Destination mail file?'
		initialAnswer: ''.
	(destFileName isEmpty) ifTrue: [^self].
	destFile _ FileStream fileNamed: destFileName.
	destFile ifNil: [ ^self error: 'could not open file' ].
	destFile setToEnd.

	messageIds _ self filteredMessagesIn: currentCategory.

	('exporting ', messageIds size printString, ' messages')
		displayProgressAt: Sensor mousePoint
		from: 0
		to: messageIds size
		during: [ :bar |
			count _ 0.
			messageIds do: [ :messageId |
				destFile nextPutAll: Celeste eudoraSeparator.
				(mailDB getMessage: messageId) text linesDo: [ :line |
					(line beginsWith: 'From ') ifTrue: [ destFile nextPut: $> ].
					destFile nextPutAll: line.
					destFile cr ].
				count _ count + 1.
				bar value: count. ].
		].

	destFile close.
!

fetchMail
	"Append messages from the user's mailbox to this mail database."

	| server password msgCount |
	mailDB ifNil: [ ^self ].
	server _ self class popServer.
	password _ self popPassword.
	password ifNil: [^ self].

	self requiredCategory: 'new'.

	msgCount _ mailDB fetchMailFromPOP: server
		userName: self class popUserName
		password: password
		doFormatting: FormatWhenFetching
		deleteFromServer: DeleteInboxAfterFetching.
	msgCount < 0
		ifTrue: [self inform: 'could not connect to the mail server']
		ifFalse: [self inform: msgCount printString, ' messages fetched'].
	msgCount <= 0 ifTrue: [^ self].

	self setCategory: 'new'.

!

findDuplicates
	"Find duplicate messages, and move the redundant copies to a given category."

	| duplicatesCategory |
	mailDB ifNil: [ ^self ].
	duplicatesCategory _ FillInTheBlank
		request: 'File duplicates in category?'
		initialAnswer: '.duplicates.'.
	duplicatesCategory isEmpty ifTrue:[^ self].

	self requiredCategory: duplicatesCategory.

	Utilities informUser: 'Searching for duplicates...'
		during: [mailDB fileDuplicatesIn: duplicatesCategory].

	self setCategory: duplicatesCategory.
!

importIntoCategory
	"Add the messages from a Unix or Eudora format file into this category"

	| inboxPath count |
	mailDB ifNil: [ ^self ].
	currentCategory ifNil: [ ^self ].

	"get the file to import from"
	inboxPath _ ''.
	[	inboxPath _ FillInTheBlank request: 'file to import from?\(should be Eudora or Unix format)' withCRs.
		inboxPath isEmpty ifTrue: [ ^self ].
		FileStream isAFileNamed: inboxPath 
	] whileFalse: [
		self inform: 'file does not exist' ].

	Utilities informUser: 'Fetching mail from ', inboxPath during: [
		count _ mailDB importMailFrom: inboxPath  intoCategory: currentCategory. ].
	self inform: count printString, ' messages imported'.

	self updateTOC.
!

maxMessagesToDisplay
	"return the maximum number of mesasges to display at one time; the  
	motivation is that huge lists of messages are impractical"
	^ self class messageCountLimit
!

messages: actuallyShown from: possible 
	self class includeStatusPane ifTrue: [
	self status: 'Showing ' , actuallyShown printString , ' of ' , possible printString , ' messages in "' , self category , '"']
!

messagesOnServerString
	| string |
	string _ 'leave messages on server'.
	^ DeleteInboxAfterFetching 
		ifTrue: ['<no>' , string]
		ifFalse: ['<yes>' , string]
!

nextCategory
	"Select the next category."

	| catList i |
	mailDB ifNil: [ ^self ].
	catList _ self categoryList.
	(currentCategory isNil) ifTrue: [currentCategory _ catList last].
	i _ catList indexOf: currentCategory.
	i < catList size
		ifTrue: [self setCategory: (catList at: i + 1)]
		ifFalse: [self setCategory: (catList at: 1)].
!

previousCategory
	"Select the next category."

	| catList i |
	mailDB ifNil: [ ^self ].
	catList _ self categoryList.
	(currentCategory isNil) ifTrue: [currentCategory _ catList last].
	i _ catList indexOf: currentCategory.
	i > 1
		ifTrue: [self setCategory: (catList at: i - 1)]
		ifFalse: [self setCategory: (catList at: catList size)].
!

removeCategory
	"Remove the existing category with the user-specified name."

	| msgList |
	mailDB ifNil: [ ^self ].
	currentCategory ifNil: [ ^self ].

	msgList _ mailDB messagesIn: currentCategory.
	(mailDB messagesIn: '.trash.') do: [: id |
		msgList remove: id ifAbsent: []].
	msgList isEmpty ifFalse: [
		(self confirm:
'This category is not empty. Are
you sure you wish to remove it?') ifFalse: [^self]].
	mailDB removeCategory: currentCategory.

	self changed: #categoryList.
	self setCategory: nil.
!

renameCategory
	"Rename the category with the user-specified name."

	| newCatName |
	mailDB ifNil: [ ^self ].
	currentCategory ifNil: [ ^self ].

	newCatName _ FillInTheBlank
		request: 'New name?'
		initialAnswer: currentCategory.
	(newCatName isEmpty) ifTrue: [^self].	"user aborted"
	mailDB renameCategory: currentCategory to: newCatName.
	currentCategory _ newCatName.
	self changed: #categoryList.
!

save
	"Snapshot the database to disk."
	mailDB ifNil: [ ^self ].
	mailDB saveDB.
!

setCCList
	"Change the default cc: list for use in composing messages."
	mailDB ifNil: [ ^self ].
	self class setCCList.
!

setCategory: newCategory 
	"Change the currently selected category. We must also compute the table  
	of contents and message list for the new category."
	| messageCount |
	currentCategory _ newCategory.
	newCategory isNil
		ifTrue: [currentMessages _ currentMsgID _ nil.
			self class includeStatusPane
				ifTrue: [status _ nil]]
		ifFalse: [currentMessages _ self filteredMessagesIn: newCategory.
			messageCount _ currentMessages size.
			messageCount > self maxMessagesToDisplay
				ifTrue: [self messages: self maxMessagesToDisplay from: messageCount.
					currentMessages _ currentMessages copyLast: self maxMessagesToDisplay]
				ifFalse: [self messages: messageCount from: messageCount].
			self cacheTOC].
	self changed: #category.
	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.
	self changed: #status.
	self changed: #outBoxStatus
!

setPopServer
	mailDB ifNil: [ ^self ].
	^self class setPopServer
!

setPopUserName
	mailDB ifNil: [ ^self ].
	userPassword _ nil.   "Clear the password when a new username is set"
	^self class setPopUserName
!

setSmtpServer
	mailDB ifNil: [ ^self ].
	^MailSender setSmtpServer
!

setUserName
	"Change the user's email name for use in composing messages."
	mailDB ifNil: [ ^self ].
	MailSender setUserName.
!

suppressingHeadersString
	| string |
	string _ 'suppress header'.
	^ SuppressWorthlessHeaderFields
		ifTrue: ['<yes>' , string]
		ifFalse: ['<no>' , string]
!

toggleKeepMessagesOnServer
	DeleteInboxAfterFetching _ DeleteInboxAfterFetching not
!

toggleSuppressHeaders

	SuppressWorthlessHeaderFields _ SuppressWorthlessHeaderFields not.
	self changed: #messageText.
!

viewAllMessages
	mailDB ifNil: [ ^self ].
	currentMessages _ self filteredMessagesIn: self category.
	self messages: currentMessages size from:  currentMessages size.
	self cacheTOC.
	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText
! !

!Celeste methodsFor:'drag and drop'!

acceptDroppingMorph: transferMorph event: evt inMorph: dstListMorph 
	"Accept messageIDs from the tocEntryList.  Move the indicated message to the destination category."

	| srcType moveID destCategory savedCurrentMsgID |

	srcType _ transferMorph dragTransferType.
	srcType == #tocEntryList ifFalse: [^false].

	"Get the message ID and the destination category"
	moveID _ transferMorph passenger initialIntegerOrNil.
	destCategory _ dstListMorph potentialDropMorph contents.
	[moveID isKindOf: Integer] assert.
	[self categoryList includes: destCategory] assert.

	"Don't do anything if the message was dropped into some particular categories"
	(destCategory = currentCategory) ifTrue: [^false].	"the current category"
	(destCategory = '.all.') ifTrue: [^false].				"the computed category .all."

	"Quickly remove the message from those displayed using removeMessage:.  And a bit of fiddling to ensure we display the original message or something similar"
	savedCurrentMsgID _ currentMsgID.
	mailDB file: moveID inCategory: destCategory.
	self removeMessage: moveID.
	savedCurrentMsgID = moveID ifFalse: [self displayMessage: savedCurrentMsgID].

	^true
!

dragPassengerFor: item inMorph: dragSource
	"Create a information string representing the message to drag (and display while dragging)"

	| msgID |

	(dragSource isKindOf: PluggableListMorph) ifFalse: [^item].

	dragSource getListSelector == #tocEntryList
		ifTrue: [ msgID _ self msgIDFromTOCEntry: item contents.
				^ msgID printString, ' ', (mailDB getMessage: msgID) from].

	"Give them nil if they try to drag a category for instance"
	^nil
!

dragTransferTypeForMorph: dragSource

	^(dragSource isKindOf: PluggableListMorph)
		ifTrue: [dragSource getListSelector]
!

wantsDroppedMorph: transferMorph event: anEvent inMorph: destinationLM 
	"We are only interested in TransferMorphs as wrappers for             
	informations. If their content is really interesting for us, will             
	determined later in >>acceptDroppingMorph:event:."

	| srcType dstType |

	"only want drops on lists (not, for example, on pluggable texts)"
	(destinationLM isKindOf: PluggableListMorph) ifFalse: [^ false].

	srcType _ transferMorph dragTransferType.
	dstType _ destinationLM getListSelector.

	(srcType = #tocEntryList) ifFalse: [^false].	"Only messages from TOC"
	(dstType = #categoryList) ifFalse: [^false].	"Only drop into category list"

	^true
! !

!Celeste methodsFor:'filtering'!

chooseFilterFor: msgID from: filterNames
	| res |
	res _ self filtersFor: msgID from: filterNames.
	res isEmpty ifTrue:[^nil].
	res size = 1 ifTrue: [^res anyOne].
	^self selectFilterFrom: res 
!

chooseFilterForCurrentMessage
	"automatically choose a filter to move the selected message.  Returns nil if there isn't a message selected, or if there isn't exactly 1 matching filter"
	| matchingFilters |
	currentMsgID ifNil: [ ^nil ].
	matchingFilters := self filtersFor: currentMsgID from: self filterNames.
	matchingFilters size = 1
		ifTrue: [ ^matchingFilters anyOne ]
		ifFalse: [ ^nil ]
!

customFilterMove
	"Select or define and activate a custom filter."
	| filterList filterName msgList |
	filterList _ CustomFilters keys select: [:e | self categoryList includes: e].

	filterName _ currentMsgID ifNil: [self selectFilterFrom: filterList] ifNotNil: [self chooseFilterFor: currentMsgID from: filterList].
	((filterName isNil or: [filterName isEmpty]) or: [filterName = 'none']) ifTrue: [^ self].
	customFilterBlock _ self customFilterNamed: filterName.
	msgList _ self filteredMessagesIn: currentCategory.
	mailDB removeAll: msgList fromCategory: currentCategory.
	mailDB fileAll: msgList inCategory: filterName.
	customFilterBlock _ nil.
	self updateTOC
!

customFilterNamed: filterName
 	^CustomFiltersCompiled at: filterName
!

customFilterOff
	"Cancel custom filtering."
	mailDB ifNil: [ ^self ].
	customFilterBlock ifNil: [
		"it's already turned off"
		^self ].

	customFilterBlock _ nil.
	self updateTOC.
	self changed: #isCustomFilterOn.
!

customFilterOn
	"Select or define and activate a custom filter."

	| filterName filterMenu |
	mailDB ifNil: [ ^self ].
	filterMenu := CustomMenu new.

	currentMsgID ifNotNil: [
		(self filtersFor: currentMsgID from: self filterNames) do: [ :name |
			filterMenu add: name action: name ].
		filterMenu addLine.].

	filterMenu add: '(none)' action: #none.
	filterMenu add: '<define new filter...>' action: #define.
	filterMenu add: '<edit exising filter...>' action: #edit.
	filterMenu add: '<delete a filter...>' action: #delete.
	filterMenu addLine.

	self filterNames do: [ :name |
		filterMenu add: name action: name ].

	filterName _ filterMenu startUpWithCaption: 'Select a filter:'.

	filterName ifNil: [ ^self ].
	filterName = #none ifTrue: [^self customFilterOff ].
	filterName = #delete ifTrue: [ ^self deleteFilter].
	filterName = #edit
		ifTrue: [filterName _ self editFilter]
		ifFalse: [
			filterName = #define
				ifTrue: [filterName _ self defineFilter] ].
	filterName ifNil: [ ^self ].
	filterName isEmpty ifTrue: [^self].

	customFilterBlock _ CustomFiltersCompiled at: filterName.

	self updateTOC.
	self changed: #isCustomFilterOn.
!

defineFilter
	| filterName |
	mailDB ifNil: [ ^self ].
	filterName _ FillInTheBlank request: 'Filter name?'.
	filterName isEmpty ifTrue: [^ ''].
	^self editFilterNamed: filterName 
!

deleteFilter

	| filterName |
	mailDB ifNil: [ ^self ].
	CustomFilters isEmpty ifTrue: [^''].

	filterName _ (CustomMenu selections: self filterNames)
		startUpWithCaption: 'Filter to delete?'.
	filterName = nil ifTrue: [^''].
	CustomFilters removeKey: filterName ifAbsent: [].
	CustomFiltersCompiled removeKey: filterName ifAbsent: [].
!

editCategoryFilter
	mailDB ifNil: [ ^self ].
	self editFilterNamed: currentCategory
!

editFilter

	| filterName |
	mailDB ifNil: [ ^self ].
	CustomFilters isEmpty ifTrue: [^''].

	filterName _ (CustomMenu selections: self filterNames)
		startUpWithCaption: 'Filter to edit?'.
	filterName = nil ifTrue: [^''].
	^self editFilterNamed: filterName filterExpr: (CustomFilters at: filterName)
!

editFilterNamed: filterName
	| expr |
	(CustomFilters includesKey: filterName)
		ifTrue: [expr _ CustomFilters at: filterName]
		ifFalse: [expr _ ''].
	^ self editFilterNamed: filterName filterExpr: expr
!

editFilterNamed: filterName filterExpr: oldExpr

	| newDefinition |
	newDefinition _ FillInTheBlank
		request:
'Enter a filter definition where "m" is the message being testing. The expression can send
"fromHas:", "toHas:", "ccHas:", "subjectHas:", "participantHas:", or "textHas:" to m to test for
inclusion of a string--or one of an array of strings--in a field. It can also test m''s time
and/or date, the textLength and can combine several tests with logical operators. Examples:
 
     m fromHas: ''johnm''                       -- messages from johnm
     m participantHas: ''johnm''                -- messages from, to, or cc-ing johnm
     m textHas: #(squeak smalltalk java)      -- messages with any of these words
     m subjectHas: #(0 1 2 3 4 5 6 7 8 9)       -- numbers in lists treated as strings
     m textLength > 50000                        -- message bodies larger than 50000 characters 
 
NOTE: "textHas:" is very slow, since it must read the message from disk.'
		initialAnswer: oldExpr.
	newDefinition isEmpty ifTrue: [^''].
	CustomFilters at: filterName put: newDefinition.
	CustomFiltersCompiled at: filterName put: (self class makeFilterFor: newDefinition).
	^filterName
!

filterNames
	"return a sorted list of custom filter names"
	^CustomFilters keys asSortedArray
!

filteredMessagesIn: categoryName

	| msgList |
	mailDB ifNil: [ ^#() ].

	msgList _ mailDB messagesIn: categoryName.

	(participantFilter size > 0) ifTrue:
		[msgList _ msgList select:
			[: id | (mailDB getTOCentry: id) participantHas: participantFilter ]].

	(subjectFilter size > 0) ifTrue:
		[msgList _ msgList select:
			[: id | (mailDB getTOCentry: id) subject includesSubstring: subjectFilter caseSensitive: false]].

	"We do custom filters last so they have to apply to the smallest number of messages"
	(customFilterBlock notNil) ifTrue:
		[msgList _ msgList select:
			[: id | customFilterBlock value: (mailDB getTOCentry: id) ]].

	^msgList
!

filtersFor: msgID from: filterNames
	| currentTocEntry |
	currentTocEntry := mailDB getTOCentry: msgID.
	^filterNames select: [:e | (self customFilterNamed: e) value: currentTocEntry].
!

isCustomFilterOn
	"whether there is a custom filter in effect"
	^customFilterBlock ~~ nil
!

isParticipantFilterOn
	"return whether a non-trivial participant filter is installed"
	^participantFilter notNil and: [ participantFilter isEmpty not ]
!

isSubjectFilterOn
	"whether there is a subject filter in effect"
	^ subjectFilter isEmptyOrNil not
!

makeFilterFor: filterExpr
 ^Compiler evaluate: '[ :m | ', filterExpr, ']'.
!

normalizedSubject: srcString
	"Turn the raw subject line into a decent possible subject filter"

	| res |
	res _ srcString.

	"Remove leading Re:s"
	[res asLowercase beginsWith: 're:']
		whileTrue: [res _ (res copyFrom: 4 to: res size) withBlanksTrimmed].

	^ res
!

participantFilterOn
	"Show only those messages where a specified user is either the sender or a receiver."
	mailDB ifNil: [ ^self ].
	participantFilter _
		(currentMsgID isNil)
			ifTrue: ['']
			ifFalse: [(mailDB getTOCentry: currentMsgID) from].
	participantFilter _ FillInTheBlank
		request: '''Participant:'' filter pattern?'
		initialAnswer: participantFilter.

	participantFilter _ participantFilter withBlanksTrimmed.
	self updateTOC.
	self changed: #isParticipantFilterOn.
!

selectFilterFrom: filters
	| filterName filterList |
	filterList _ filters asSortedCollection asOrderedCollection.
	filterList addFirst: '(none)'.
	filterList addLast: '<define new filter...>'.
	filterList addLast: '<edit exising filter...>'.
	filterList addLast: '<delete a filter...>'.
	filterName _ (CustomMenu selections: filterList)
			startUpWithCaption: 'Select a filter:'.
	(filterName isNil or: [filterName isEmpty]) ifTrue: [ ^nil ].
	filterName = '(none)' ifTrue: [^'none'].
	filterName = '<delete a filter...>' ifTrue: [ ^self deleteFilter].
	filterName = '<edit exising filter...>'
		ifTrue: [filterName _ self editFilter]
		ifFalse: [
			filterName = '<define new filter...>'
				ifTrue: [filterName _ self defineFilter]].
	^filterName
!

subjectFilterOn
	"Show only those messages whose subject matches the currently selected  
	message. The user is given a chance to edit the pattern string used to match  
	'Subject:' fields."
	mailDB ifNil: [ ^self ].
	subjectFilter _ currentMsgID isNil
				ifTrue: ['']
				ifFalse: [(mailDB getTOCentry: currentMsgID) subject].
	subjectFilter _ self normalizedSubject: subjectFilter.
	subjectFilter _ FillInTheBlank request: '''Subject:'' filter pattern?' initialAnswer: subjectFilter.
	self updateTOC.
	self changed: #isSubjectFilterOn
! !

!Celeste methodsFor:'initialize variables'!

initializeTocLists
	tocLists _ Array new: 6.
	1
		to: tocLists size
		do: [:index | tocLists at: index put: OrderedCollection new]
! !

!Celeste methodsFor:'message text pane'!

compose
	"Make a MailSendTool for composing a new message."
	mailDB ifNil: [ ^self ].
	self openSender: self composeText.
!

doItContext
	^nil
!

doItReceiver
	^self
!

format
	mailDB ifNil: [ ^self ].
	messageTextView
		editString: self formatedMessageText;
		hasUnacceptedEdits: true.
!

formatedMessageText
	"Answer a string that is my formatted mail message."
	| message header body bodyText |
	currentMsgID isNil ifTrue: [^ ''].
	message _ self currentMessage.
	header _ message cleanedHeader.
	body _ message body.
	body contentType = 'text/html'
		ifTrue: [bodyText _ (HtmlParser parse: (ReadStream on: body content)) formattedText]
		ifFalse: [bodyText _ body content].
	^ header asText , String cr , bodyText
!

forward
	"Make a MailSendTool for forwarding the current message."

	(currentMsgID notNil) ifTrue:
		[self openSender: (self forwardTextFor: currentMsgID)].
!

message
	"Answer the text of the currently selected message or nil if there isn't one."
self isThisEverCalled .
	(currentMsgID isNil)
		ifTrue: [^'']
		ifFalse: [^(mailDB getText: currentMsgID) asText]
!

messageMenu: aMenu shifted: shifted
	"Use the standard text menu."

	^ StringHolder basicNew codePaneMenu: aMenu shifted: shifted
!

messageText
	"Answer the text which makes up the complete message (header+body)"
	mailDB ifNil: [ ^self messageTextIfNoDB ].
	(currentMsgID isNil) ifTrue: [^''].

	"Always show the full message header for messages in the category .tosend. so that all special header lines are preserved, shown and can be edited."
	(currentCategory = '.tosend.')
		ifTrue: [^ mailDB getText: currentMsgID].

	SuppressWorthlessHeaderFields
		ifTrue: [^ self currentMessage formattedText]
		ifFalse: [^ mailDB getText: currentMsgID].
!

messageText: aStringOrText

	currentCategory isNil | currentMsgID isNil ifTrue: [^ self].
	mailDB newText: aStringOrText asString squeakToIso for: currentMsgID.
	self updateTOC.  "in case the message header was changed"
	messageTextView hasUnacceptedEdits: false.
	self changed: #messageText.
	^ true
!

messageTextIfNoDB
	"return text to display to the user if there is no DB opened"
	| openCommand |
	openCommand := 'OPEN' asText.
	openCommand addAttribute: (PluggableTextAttribute evalBlock: [ self openDefaultDatabase ]).
	openCommand addAttribute: (TextColor blue).


	^'No DB is currently open.  Press ' asText, openCommand, ' to open the default database.'
!

messageTextView: aView

	messageTextView _ aView.
!

reply
	"Make a MailSendTool for replying to the current message."

	(currentMsgID notNil) ifTrue:
		[self openSender: (self replyTextFor: currentMsgID)].
! !

!Celeste methodsFor:'open-close'!

close
	"Close the mail database."

	userPassword _ nil.
	mailDB ifNotNil: [
		mailDB close; release.
		mailDB _ nil].
!

isActive
	^mailDB notNil
!

openDefaultDatabase
	"open the default database, creating it if it isn't present"
	self openOnDatabase: (MailDB openOn: Celeste defaultDBName).
	self changed: #categoryList.
!

openOnDatabase: aMailDB
	"Initialize myself for the mail database with the given root filename."

	mailDB _ aMailDB.
	currentCategory _ 'new'.
	lastCategory _ ''.
	subjectFilter _ ''.
	participantFilter _ ''.
	self setCategory: currentCategory.
!

windowIsClosing
	"Close the mail database when my window is closed."

	self close.
! !

!Celeste methodsFor:'other'!

changeMaxMessageCount
	| countString count |
	mailDB ifNil: [ ^self ].
	countString _ FillInTheBlank request: 'Maximum number of messages displayed?' initialAnswer: self class messageCountLimit printString.
	countString isEmpty
		ifTrue: [^ self].
	count _ countString asInteger max: 1.
	self class messageCountLimit: count.
	self setCategory: currentCategory
!

clearUserEditFlag

	messageTextView hasUnacceptedEdits: false.
!

currentMessage
	^mailDB getMessage: currentMsgID
!

okToChange
	"This message is sent when changing the selection in either the message cateory or message list panes. Eventually, this should ask the user if it is okay to throw away and unaccepted edits of the current message. For now, it always gives permission."

	currentCategory isNil | currentMsgID isNil ifTrue: [
		"no message selected; discard edits in message pane silently"
		messageTextView hasUnacceptedEdits: false.
		^ true].

	messageTextView hasUnacceptedEdits ifFalse: [^ true].
	(CustomMenu confirm: 'Discard changes to currently selected message?')
		ifTrue: [messageTextView hasUnacceptedEdits: false. ^ true]
		ifFalse: [^ false].
!

outBoxStatus
	| outgoing |
	outgoing _ mailDB ifNil: [ #() ] ifNotNil: [ mailDB messagesIn: '.tosend.' ].
	outgoing isEmpty
		ifTrue: [^ 'no mail to be sent'].
	^ 'messages in queue: ' , outgoing size printString
!

perform: selector orSendTo: otherTarget
	"Celeste handles all menu commands."

	selector = #format ifTrue: [^ self perform: selector].

	((#(yellowButtonActivity shiftedYellowButtonActivity) includes: selector) or:
		[(ParagraphEditor yellowButtonMessages includes: selector) or:
		[ParagraphEditor shiftedYellowButtonMenu selections includes: selector]])
			ifTrue: [otherTarget perform: selector]
			ifFalse: [self perform: selector].
!

requiredCategory: catName
	"catName is a required category.  If it does not exist in the database, then create it, and update the category list to reflect that it now exists."

	(self categoryList includes: catName)
		ifFalse:
			[mailDB addCategory: catName.
			self changed: #categoryList.]

!

status
	^ status
!

status: aStringOrNil
	status _ aStringOrNil.
	self changed: #status
! !

!Celeste methodsFor:'sending mail'!

composeText
	"Answer the template for a new message."

	^ String streamContents: [:str |
		str nextPutAll: 'From: '.
		str nextPutAll: MailSender userName; cr.

		str nextPutAll: 'To: '; cr.
		str nextPutAll: 'Subject: '; cr.

		Celeste ccList isEmpty ifFalse: [
			str nextPutAll: 'Cc: '.
			str nextPutAll: Celeste ccList; cr].
		str cr].
!

forwardTextFor: msgID
	"Answer the template for forwarding the message with the given ID."
	| msg separator |
	msg := self currentMessage.

	^String streamContents: [ :str |
		"From header"
		str nextPutAll: 'From: ';
		nextPutAll: MailSender userName; cr.

		"Put a blank To"
		str nextPutAll: 'To: '; cr.

		"Add a subject modified from the original"
		str nextPutAll: 'Subject: (fwd) '.
		str nextPutAll: msg subject.
		str cr.


		"Add auto-cc if it's been set"
		Celeste ccList isEmpty ifFalse: [
			str nextPutAll: 'Cc: '.
			str nextPutAll: Celeste ccList; cr].

		"add the mime headers to make it multi-part"
		separator := MailMessage generateSeparator.
		str nextPutAll: 'MIME-Version: 1.0'; cr.
		str nextPutAll: 'Content-type: multipart/mixed; boundary="'.
		str nextPutAll: separator; nextPut: $".
		str cr.

		"skip down to the main part of the message"
		str cr.
		str nextPutAll: '--'; nextPutAll: separator; cr.
		str nextPutAll: 'Content-type: text/plain'; cr; cr.

		"insert the forwarded message"
		str cr; cr; nextPutAll: '====forwarded===='; cr; cr.
		str nextPutAll: '--'; nextPutAll: separator; cr.
		str nextPutAll: 'Content-type: message/rfc822'; cr; cr.
		str nextPutAll: msg text; cr.

		"final separator"
		str nextPutAll: '--'; nextPutAll: separator; nextPutAll: '--'; cr.
	].
!

openSender: initialText
	CelesteComposition openForCeleste: self  initialText: initialText.
!

popPassword
	"Answer the password to use when retrieving mail via POP3. The password is stored in an instance variable, which disappears when you close the Celeste window."

	userPassword ifNotNil: [^ userPassword].
	userPassword _ FillInTheBlank requestPassword: 'POP password'.
	userPassword isEmpty ifTrue: [userPassword _ nil].

	^ userPassword
!

preSendAuthentication
	"Where required, authenticate ourselves to the SMTP server before sending mail"

	"This is a placeholder for any required authentication"
!

queueMessageWithText: aStringOrText 
	"Queue a message to be sent later. The message is added to the database  
	and filed in the '.tosend.' category."

	| messageText id msg |

	messageText _ 'X-Mailer: ' , Celeste versionString , String cr , 'Date: ' , MailMessage dateStampNow , ' ' , self timeZoneString , ' ' , String cr.
	messageText _ messageText , aStringOrText asString.

	msg _ MailMessage from: messageText.

	"Check now that the addresses are well formed email addresses.  This prevents
	runtime errors when actually transmitting the mail"
	[MailAddressParser addressesIn: msg from] ifError:
		[ :err :rcvr | self inform: 'From: in message header', String cr, err. ^nil].
	[MailAddressParser addressesIn: msg to] ifError:
		[ :err :rcvr | self inform: 'To: in message header', String cr, err. ^nil].
	[MailAddressParser addressesIn: msg cc] ifError:
		[:err :rcvr | self inform: 'CC: in message header', String cr, err. ^nil].

	"queue the message"
	self requiredCategory: '.tosend.'.
	id _ mailDB addNewMessage: msg.
	mailDB file: id inCategory: '.tosend.'.
	self category = '.tosend.'
		ifTrue: [self updateTOC].
	self changed: #outBoxStatus.
	^ id
!

replyTextFor: msgID
	"Answer the template for a reply to the message with the given ID."

	| msg s anyCCs replyaddress |
	msg _ mailDB getMessage: msgID.
	s _ WriteStream on: (String new: 500).

	"add From:"
	s nextPutAll: 'From: ', MailSender userName; cr.


	"add Subject:"
	((msg subject asLowercase indexOfSubCollection: 're:' startingAt: 1) ~= 0)
		ifTrue: [s nextPutAll: 'Subject: ', msg subject]
		ifFalse: [s nextPutAll: 'Subject: Re: ', msg subject].
	s cr.


	"add To:"
	"Use the Reply-To: address if there is one, otherwise the From: address"
	replyaddress _ msg from.
	msg headerFieldsNamed: 'reply-to' do: [ :destAdd | replyaddress _ destAdd ].
	s nextPutAll: 'To: ', replyaddress; cr.
	

	"add CC:s from the message and from the user's CC list"
	s nextPutAll: 'CC: '.
	anyCCs _ false.
	(msg to isEmpty) ifFalse: [
		anyCCs ifTrue:[ s nextPutAll: ', '] ifFalse: [ anyCCs _ true ].
		s nextPutAll: msg to ].
	(msg cc isEmpty) ifFalse: [
		anyCCs ifTrue: [ s nextPutAll: ', ' ] ifFalse: [ anyCCs _ true ].
		s nextPutAll: msg cc ].
	(Celeste ccList isEmpty) ifFalse: [
		anyCCs ifTrue: [ s nextPutAll: ', ' ] ifFalse: [ anyCCs _ true ].	
		s nextPutAll: Celeste ccList ].
	s cr.


	"add contents of previous message"
	s cr.
	s nextPutAll: msg from; nextPutAll: ' wrote:'; cr.
	msg bodyText linesDo: [ :line |
		s nextPutAll: '> '.
		s nextPutAll: line.
		s cr ].
	s cr.

	^s contents
!

sendMail: aCollectionOfMessages
	"Send <aCollectionOfMessages> to the SMTP server."

	| sender n message recipients socket |

	self requiredCategory: '.sent.'.

	self preSendAuthentication.

	sender _ (MailAddressParser addressesIn: MailSender userName) first.

	[socket _ SMTPSocket usingServer: MailSender smtpServer]
		ifError: [ :a :b | self error: 'error opening connection to mail server'].

	('sending ', aCollectionOfMessages size printString, ' messages...')
		displayProgressAt: Sensor mousePoint
		from: 1
		to: aCollectionOfMessages size
		during: [:progressBar |
			n _ 0.
			aCollectionOfMessages do: [:id |
				progressBar value: (n _ n + 1).
				message _ mailDB getMessage: id.

				recipients _ Set new.
				recipients addAll: (MailAddressParser addressesIn: message to).
				recipients addAll: (MailAddressParser addressesIn: message cc).

				[socket 
					mailFrom: sender
					to: recipients 
					text: message text.	"send this one message on the stream"

				mailDB remove: id fromCategory: '.tosend.'.
				mailDB file: id inCategory: '.sent.'
				] ifError: [ :a :b | self error: 'error posting mail']
		]].

	socket quit; close.
	mailDB saveDB.

	(self category = '.tosend.') |  (self category = '.sent.') ifTrue: [self updateTOC].

!

sendQueuedMail
	"Post queued messages to the SMTP server."
	| outgoing sender n message recipients socket |
	mailDB ifNil: [ ^self ].
	outgoing _ mailDB messagesIn: '.tosend.'.
	outgoing isEmpty
		ifTrue: [^ self inform: 'no mail to be sent'].
	self requiredCategory: '.sent.'.
	self preSendAuthentication.
	sender _ (MailAddressParser addressesIn: MailSender userName) first.
	[socket _ SMTPSocket usingServer: MailSender smtpServer]
		ifError: [:a :b | self error: 'error opening connection to mail server'].
	'sending ' , outgoing size printString , ' messages...'
		displayProgressAt: Sensor mousePoint
		from: 1
		to: outgoing size
		during: [:progressBar | 
			n _ 0.
			outgoing
				do: [:id | 
					progressBar value: (n _ n + 1).
					message _ mailDB getMessage: id.
					recipients _ Set new.
					recipients
						addAll: (MailAddressParser addressesIn: message to).
					recipients
						addAll: (MailAddressParser addressesIn: message cc).
					[socket
						mailFrom: sender
						to: recipients
						text: message text.
					"send this one message on the stream"
					mailDB remove: id fromCategory: '.tosend.'.
					mailDB file: id inCategory: '.sent.']
						ifError: [:a :b | self error: 'error posting mail']]].
	socket quit; close.
	mailDB saveDB.
	self category = '.tosend.' | (self category = '.sent.')
		ifTrue: [self updateTOC].
	self changed: #outBoxStatus
!

timeZoneString
	^Celeste userTimeZone
! !

!Celeste methodsFor:'table of contents pane'!

autoFile
	"automatically pick a folder for the current message, and file the current message there"
	| folder |
	mailDB ifNil: [ ^self ].
	folder := self chooseFilterForCurrentMessage.
	folder ifNil: [ ^self].
	lastCategory := folder.
	mailDB file: currentMsgID inCategory: folder.
!

autoMove
	"automatically pick a folder for the current message, and move the message there"
	| folder |
	mailDB ifNil: [ ^self ].
	folder := self chooseFilterForCurrentMessage.
	folder ifNil: [ ^self].
	lastCategory := folder.
	mailDB file: currentMsgID inCategory: folder.
	self removeMessage.
!

deleteAll
	"Move all visible messages in the current category to '.trash.'."

	| |
	mailDB ifNil: [ ^self ].

	self requiredCategory: '.trash.'.
	mailDB fileAll: currentMessages inCategory: '.trash.'.
	self removeAll.
!

deleteMessage
	"Move the current message to the '.trash.' category and select the next  
	message. Deleted messages can later purged by the 'empty trash' menu  
	item"
	currentMsgID isNil
		ifTrue: [^ self].
	self requiredCategory: '.trash.'.
	mailDB file: currentMsgID inCategory: '.trash.'.
	self removeMessage
!

displayMessage: msgID
	"If the message is in our displayed category, show it"

	(currentMessages notNil and: [currentMessages includes: msgID])
		ifTrue: [currentMsgID _ msgID]
		ifFalse: [currentMsgID _ nil].

	self changed: #tocEntry.
	self changed: #messageText.


	"Celeste someInstance displayMessage: 671458061"
!

fileAgain
	"File the current message in the same category as last time."

	| newCatName |
	mailDB ifNil: [ ^self ].
	(lastCategory isEmpty not)
		ifTrue: [newCatName _ lastCategory]
		ifFalse: [newCatName _ self getCategoryNameIfNone: [^self]].
	mailDB file: currentMsgID inCategory: newCatName.
!

fileAll
	"File all visible messages in the current category in some other category as well."

	| newCatName msgList |
	mailDB ifNil: [ ^self ].
	newCatName _ self getCategoryNameIfNone: [^self].
	msgList _ self filteredMessagesIn: currentCategory.
	mailDB fileAll: msgList inCategory: newCatName.
	self updateTOC.
!

fileMessage
	"File the current message in another category."

	| newCatName |
	mailDB ifNil: [ ^self ].
	newCatName _ self getCategoryNameIfNone: [^self].
	mailDB file: currentMsgID inCategory: newCatName.
!

getCategoryNameIfNone: aBlock
	"Prompt the user for a category name"

	| catList categoryName categoryMenu |
	catList _ self categoryList.
	catList remove: '.all.' ifAbsent: [].
	catList remove: '.trash.' ifAbsent: [].
	catList remove: '.unclassified.' ifAbsent: [].
	catList add: '<new category>'.
	categoryMenu _ CustomMenu selections: catList.
	categoryName _ categoryMenu startUp.
	categoryName = nil ifTrue: [^aBlock value].
	categoryName = '<new category>' ifTrue: [
		categoryName _ FillInTheBlank
			request: 'New category name?'
			initialAnswer: ''.
		(categoryName isEmpty) ifTrue: [^aBlock value].
		mailDB addCategory: categoryName.
		self changed: #categoryList.
	].
	^lastCategory _ categoryName
!

moveAgain
	"Move the current message to the same category as last time."

	| newCatName |

	currentMsgID ifNil: [ ^self ].
	(lastCategory isEmpty not)
		ifTrue: [newCatName _ lastCategory]
		ifFalse: [newCatName _ self getCategoryNameIfNone: [^self]].
	newCatName = currentCategory ifTrue: [ ^self ].

	mailDB file: currentMsgID inCategory: newCatName.
	self removeMessage.
!

moveAll
	"Move all visible messages in the current category to another category."

	| newCatName |
	mailDB ifNil: [ ^self ].
	newCatName _ self getCategoryNameIfNone: [^self].
	newCatName = currentCategory ifTrue:[ ^self ].
	mailDB fileAll: currentMessages inCategory: newCatName.

	self removeAll.
!

moveMessage
	"Move the current message to another category; this consists of filing it in the new category, and then removing it from the current category"

	| newCatName |
	mailDB ifNil: [ ^self ].
	newCatName _ self getCategoryNameIfNone: [^self].
	newCatName = currentCategory ifTrue: [ ^self ].
	mailDB file: currentMsgID inCategory: newCatName.

	self removeMessage.
!

msgIDFromTOCEntry: newTOCentry 
	"Given an entry index number from the TOC pane, find the corresponding msgID"

	^currentMessages at: ((self tocLists at: 1) indexOf: newTOCentry ifAbsent: [^nil])
!

nextMessage
	"Select the next message."

	| index |
	mailDB ifNil: [ ^self ].
	(currentCategory isNil | currentMsgID isNil) ifTrue: [^ self].
	index _ currentMessages indexOf: currentMsgID.
	index < currentMessages size
		ifTrue: [self setTOCEntry: ((self tocLists at: 1) at: index + 1)]
		ifFalse: [self setTOCEntry: ((self tocLists at: 1) at: 1)].
!

otherCategories
	"Prompt the user with a menu of all other categories in which the currently selected message appears. If the user chooses a category from this menu, go to that category."

	| otherCategories choice |
	mailDB ifNil: [ ^self ].
	otherCategories _
		(mailDB categoriesThatInclude: currentMsgID) asOrderedCollection.
	otherCategories remove: currentCategory ifAbsent: [].
	(otherCategories isEmpty) ifTrue: [^self].
	choice _ (CustomMenu selections: otherCategories) startUp.
	choice = nil ifFalse: [self setCategory: choice].
!

partsMenu
	"Show a menu listing all the parts of this message, and let the user save the chosen part to a file"

	| menu currMessage part |
	currentMsgID ifNil: [ ^self ].
	menu _ CustomMenu new.
	currMessage _ self currentMessage.
	currMessage atomicParts do: [:e | menu add: 'save ' , e printString action: e].
	part _ menu startUp.
	part ifNotNil: [part save]
!

previousMessage
	"Select the previous message."

	| index |
	mailDB ifNil: [ ^self ].
	(currentCategory isNil | currentMsgID isNil) ifTrue: [^ self].
	index _ currentMessages indexOf: currentMsgID.
	index > 1
		ifTrue: [self setTOCEntry: ((self tocLists at: 1)  at: index - 1)]
		ifFalse: [self setTOCEntry: ((self tocLists at: 1)  at: currentMessages size)].

!

removeAll
	"Remove all presently listed messages from the current category."
	mailDB ifNil: [ ^self ].
	mailDB removeAll: currentMessages fromCategory: currentCategory.
	currentMsgID _ nil.

	"Regenerate the (possibly empty) TOC for this category"
	self setCategory: currentCategory.
!

removeMessage
	self removeMessage: currentMsgID
!

removeMessage: msgID
	"Remove the message from the current category and update the display.  This is done in a way that avoids having to regenerate the TOC list and thus is pretty fast"

	| currentMessageIndex |

	msgID ifNil: [^ self].
	[currentMessages includes: msgID] assert.

	mailDB remove: msgID fromCategory: currentCategory.

	"remove the message from the listing"
	currentMessageIndex _ currentMessages indexOf: msgID.
	currentMessages _ currentMessages copyWithout: msgID.

	2
		to: self tocLists size
		do: [:index | (tocLists at: index) removeAt: currentMessageIndex].
	tocLists first removeLast.

	"update the message index and message ID"
	msgID = currentMsgID ifTrue: [
		currentMessages isEmpty
			ifTrue: [currentMsgID _ nil]
			ifFalse: [currentMsgID _ currentMessages
						at: (currentMessageIndex min: currentMessages size)]].
	currentMsgID ifNotNil: [[currentMessages includes: currentMsgID] assert].

	self changed: #tocEntryList.
	self changed: #tocEntry.
	self changed: #messageText.
	self changed: #status.
	self changed: #outBoxStatus
!

saveMessage
	"save the currently selected message to a file"
	| fileName file |
	currentMsgID ifNil: [^ self].
	fileName _ FillInTheBlank request: 'file to save in'.
	fileName isEmpty ifTrue: [^ self].
	file _ FileStream fileNamed: fileName.
	file nextPutAll: (self currentMessage) text.
	file close
!

search
	"Search the text of all messages in the present category"

	| destCat matchString msgText |
	mailDB ifNil: [ ^self ].
	destCat _ FillInTheBlank
		request: 'In what category should the search results be filed?'
		initialAnswer: '.search results.'.
	(destCat isEmpty) ifTrue: [^self].
	matchString _ FillInTheBlank
		request: 'String sought in message text?'
		initialAnswer: ''.
	(matchString isEmpty) ifTrue: [^self].

	self requiredCategory: destCat.

	(self filteredMessagesIn: currentCategory) do:
		[: msgID |
		 msgText _ mailDB getText: msgID.
		 ((msgText findString: matchString startingAt: 1) > 0) ifTrue:
			[mailDB file: msgID inCategory: destCat]].

	self setCategory: destCat.

!

setTOCEntry: newTOCentry 
	"Change the currently selected message. This is done by finding the  
	message ID corresponding to the selected table of contents entry."

	currentMsgID _ self msgIDFromTOCEntry: newTOCentry. 
	self changed: #tocEntry.
	Cursor read
		showWhile: [self changed: #messageText]
!

tocEntry
	"Answer the table of contents entry for the currently selected message  
	or nil."
	currentMsgID isNil
		ifTrue: [^ nil]
		ifFalse: [^ (self tocLists at: 1)
				at: (currentMessages indexOf: currentMsgID)]
!

tocEntryList
	currentCategory isNil
		ifTrue: [self initializeTocLists].
	self tocLists == nil
		ifTrue: [self initializeTocLists].
	^ self tocLists
!

tocKeystroke: aCharacter 
	aCharacter = Character backspace
		ifTrue: [self deleteMessage].
	aCharacter asciiValue = 30
		ifTrue: [self previousMessage].
	aCharacter asciiValue = 31
		ifTrue: [self nextMessage].
	aCharacter = $c
		ifTrue: [self customFilterOn].
	aCharacter = $m
		ifTrue: [self customFilterMove].
	aCharacter = $s
		ifTrue: [self subjectFilterOn]
!

tocMenu: aMenu 
	"Answer the menu for the table of contents pane."
	| messageSelected autoFolder |
	currentCategory ifNil: [^ nil].
	messageSelected _ currentMsgID isNil not.
	messageSelected
		ifTrue: 
			[aMenu add: 'delete' action: #deleteMessage.
			aMenu balloonTextForLastItem: 'Move this message to the .trash. category'.
			aMenu addLine.
			aMenu add: 'compose' action: #compose.
			aMenu balloonTextForLastItem: 'Compose a new message'.
			aMenu add: 'reply' action: #reply.
			aMenu balloonTextForLastItem: 'Reply to this message'.
			aMenu add: 'forward' action: #forward.
			aMenu balloonTextForLastItem: 'Forward this message'.
			self currentMessage body isMultipart
				ifTrue: 
					[aMenu add: 'parts...' action: #partsMenu.
					aMenu balloonTextForLastItem: 'Forward this message'].
			aMenu addLine.
			lastCategory isEmpty
				ifFalse: 
					[aMenu add: 'file -> ' , lastCategory action: #fileAgain.
					aMenu balloonTextForLastItem: 'Add this message also to the category ' , lastCategory.
					aMenu add: 'move -> ' , lastCategory action: #moveAgain.
					aMenu balloonTextForLastItem: 'Move this message to the category ' , lastCategory.
					aMenu addLine].
			autoFolder := self chooseFilterForCurrentMessage.
			autoFolder ifNotNil: [
				aMenu add: ('file -> ', autoFolder) action: #autoFile.
				aMenu balloonTextForLastItem: 'Add this message also to the (automatically selected) category ' , autoFolder.
				aMenu add: ('move -> ', autoFolder) action: #autoMove.
				aMenu balloonTextForLastItem: 'Move this message to the (automatically selected) category ' , autoFolder.
				aMenu addLine ].

			aMenu add: 'file' action: #fileMessage.
			aMenu balloonTextForLastItem: 'Add this message also to a different category'.
			aMenu add: 'move' action: #moveMessage.
			aMenu balloonTextForLastItem: 'Move this message to a different category'.
			aMenu add: 'remove' action: #removeMessage.
			aMenu balloonTextForLastItem: 'Remove this message from this category
(NB: the message will be safely available in another category)'.
			aMenu addLine]
		ifFalse: 
			[aMenu add: 'compose' action: #compose.
			aMenu balloonTextForLastItem: 'Compose a new message'.
			aMenu addLine].

	"The following are common for all menus"
	aMenu add: 'file all' action: #fileAll.
	aMenu balloonTextForLastItem: 'Add all messages also to another category'.
	aMenu add: 'move all' action: #moveAll.
	aMenu balloonTextForLastItem: 'Move all messages to another category'.
	aMenu add: 'remove all' action: #removeAll.
	aMenu balloonTextForLastItem: 'Remove all messages from this catgegory
(NB: each message will be safely available in other categories)'.
	aMenu add: 'delete all' action: #deleteAll.
	aMenu balloonTextForLastItem: 'Move all messages to the .trash. category'.
	aMenu addLine.
	messageSelected
		ifTrue: 
			[aMenu add: 'other categories' action: #otherCategories.
			aMenu balloonTextForLastItem: 'Check which other categories also contain this message'.
			aMenu add: 'save message' action: #saveMessage.
			aMenu balloonTextForLastItem: 'Save this message'.
			aMenu addLine].
	self tocEntryList size = self maxMessagesToDisplay ifTrue: [
		"the test above is not exactly correct, but is usually correct."
		aMenu add: 'view all messages' action: #viewAllMessages.
		aMenu balloonTextForLastItem: 'View all messages that match the current filters,	even if there are many thousands of such messages'. ].

	aMenu add: 'search' action: #search.
	aMenu balloonTextForLastItem: 'Search all messages in this category for something'.
	^ aMenu
!

updateTOC
	"Update the table of contents after a moving, removing, or deleting a message.
	Select a message near the removed message in the table of contents if possible."

	| savedMsgID |
	mailDB ifNil: [ ^self ].
	savedMsgID _ currentMsgID.

	"update the TOC listing without displaying any particular message"
	currentMsgID _ nil.
	self setCategory: currentCategory.

	self displayMessage: savedMsgID.

	"NB: self changed: #tocEntryList is already done above by setCategory: and can be slow"
	self changed: #outBoxStatus
! !

!Celeste class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/Celeste.st,v 1.1 2003/05/09 21:40:29 cg Exp $'
! !

Celeste initialize!
