"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailDBFile subclass:#IndexFile
	instanceVariableNames:'msgDictionary timeSortedEntries logfile'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

IndexFile comment:'I represent an index for the messages in a mail database. I acts like a dictionary that maps unique message ID''s to IndexFileEntry objects.

The index file is read into memory in its entirety and kept there for the duration of a mail reading session. It should be stored back to disk at the end of the session, or after a major change, such as fetching new mail.

If necessary, it can be completely recreated by scanning the messages file!!'
!


!IndexFile class methodsFor:'instance creation'!

openOn: fileName messageFile: messageFile readLogFlag: readLogFlag 
	"Answer a new instance of me for the given message file, backed by the 
	file with the given name."
	^ super new
		openOn: fileName
		messageFile: messageFile
		readLogFlag: readLogFlag
! !

!IndexFile methodsFor:'dictionary access'!

at: msgID
	"Answer the IndexFileEntry for the message with the given ID."

	^msgDictionary at: msgID ifAbsent: [self reportInconsistency]
!

at: msgID ifAbsent: aBlock
	"Answer the IndexFileEntry for the message with the given ID. Evaluate the given block if there is no entry for the given ID."

	^msgDictionary at: msgID ifAbsent: aBlock
!

at: msgID put: anIndexFileEntry 
	"Associate the given IndexFileEntry with the given message ID."
	self privateAt: msgID put: anIndexFileEntry.
	self logStream print: msgID;
	 cr.
	"message ID"
	anIndexFileEntry writeOn: self logStream.
	self logStream close
!

includesKey: msgID
	"Answer true if my message dictionary contains an entry for the message with the given ID."

	^msgDictionary includesKey: msgID
!

keys
	"Answer a collection of message IDs for the messages in this IndexFile, sorted in ascending timestamp order. Because sorting is expensive, the sorted key list is cached."

	| keys |
	keys _ OrderedCollection new: timeSortedEntries size * 2.
	timeSortedEntries do: [: assoc | keys addLast: assoc key].
	^keys
!

keysCount
	^msgDictionary size
!

logStream

	logfile ifNil: [
		^logfile _ (StandardFileStream fileNamed: filename,'.log') setToEnd
	].
	logfile closed ifTrue: [logfile open; setToEnd].
	^ logfile
!

privateAt: msgID put: anIndexFileEntry 
	"Associate the given IndexFileEntry with the given message ID."
	timeSortedEntries removeAllSuchThat: [:assoc | assoc key = msgID].
	"don't duplicate the entry!!"
	msgDictionary at: msgID put: anIndexFileEntry.
	timeSortedEntries add: (Association key: msgID value: anIndexFileEntry).
	
!

privateRemove: msgID 
	"Remove the entry with the given ID from my Dictionary."
	timeSortedEntries removeAllSuchThat: [:assoc | assoc key = msgID].
	msgDictionary removeKey: msgID ifAbsent: [].
	
!

remove: msgID 
	"Remove the entry with the given ID from my Dictionary."
	self privateRemove: msgID.
	self logStream nextPutAll: 'remove';
	 cr;
	 print: msgID;
	 cr;
	 close
! !

!IndexFile methodsFor:'file operations'!

delete
	| logFilename |
	"delete logfile"
	logFilename _ filename , '.log'.
	FileDirectory
		splitName: logFilename
		to: [:dirPath :name | (FileDirectory forFileName: logFilename)
				deleteFileNamed: name
				ifAbsent: []].
	"Delete this file."
	super delete
!

openOn: aFileName messageFile: messageFile readLogFlag: readLogFlag
	"Initialize myself from the file with the given name."
	| fileStream |
	filename _ aFileName.
	fileStream _ FileStream fileNamed: aFileName.
	self
		readFrom: (ReadStream on: fileStream contentsOfEntireFile)
		messageFile: messageFile.
	readLogFlag ifTrue: [self readLogMessageFile: messageFile].
	"close and release the file stream"
	fileStream close.
	fileStream _ nil
!

readLogMessageFile: messageFile 
	| file msgID entry |
	file _ self logStream position: 0.
	[file atEnd]
		whileFalse: [(file peekFor: $r)
				ifTrue: [MailDB readStringLineFrom: file.
					self
						privateRemove: (MailDB readIntegerLineFrom: file)]
				ifFalse: [msgID _ MailDB readIntegerLineFrom: file.
					entry _ IndexFileEntry
								readFrom: file
								messageFile: messageFile
								msgID: msgID.
					self privateAt: msgID put: entry]].
	self logStream close
!

rename: newFileName 
	"rename log file"
	| logFilename newLogFilename |
	logFilename _ filename , '.log'.
	newLogFilename _ newFileName , '.log'.
	FileDirectory
		splitName: logFilename
		to: [:dirPath :oldFileName | 
			| fd | 
			fd _ FileDirectory forFileName: logFilename.
			(fd fileExists: logFilename)
				ifTrue: [fd rename: oldFileName toBe: newLogFilename]].
	"at last, it changes filename!!"
	super rename: newFileName
! !

!IndexFile methodsFor:'read-write'!

readFrom: aStream messageFile: messageFile 
	"Initialize myself from the given text stream. It is assumed that the 
	.index file was written in order of ascending message timestamps, 
	although this method is only less efficient, not incorrect, if this is not 
	the case."
	| sorted lastTime msgID entry |
	msgDictionary _ PluggableDictionary integerDictionary.
	timeSortedEntries _ (SortedCollection new: 1000)
				sortBlock: [:m1 :m2 | m1 value time <= m2 value time].
	sorted _ true.
	lastTime _ nil.
	[aStream atEnd]
		whileFalse: 
			[msgID _ MailDB readIntegerLineFrom: aStream.
			entry _ IndexFileEntry
						readFrom: aStream
						messageFile: messageFile
						msgID: msgID.
			msgDictionary at: msgID put: entry.
			timeSortedEntries addLast: (Association key: msgID value: entry).
			(sorted & lastTime notNil and: [lastTime > entry time])
				ifTrue: [sorted _ false].
			lastTime _ entry time].
	sorted ifFalse: [timeSortedEntries reSort]
!

save
	| dir shortName |
	super save. 
	self logStream close.
	shortName _ FileDirectory localNameFor: filename.
	dir _ FileDirectory forFileName: filename.
	dir deleteFileNamed: shortName , '.log' ifAbsent: []
!

writeOn: aStream
	"Write my index entries to the given text stream in human-readable form."
	"Note: For efficiency, this is done in order of increasing message timestamps, to save the cost of sorting when we read it back in. It is assumed that timeSortedEntries should contains exactly the same message ID's as msgDictionary."

	timeSortedEntries do:
		[: assoc |
		 (assoc key) printOn: aStream.		"message ID"
		 aStream cr.
		 (assoc value) writeOn: aStream].	"index entry"
! !

!IndexFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/IndexFile.st,v 1.1 2003/05/09 21:40:54 cg Exp $'
! !
