"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

Object subclass:#IndexFileEntry
	instanceVariableNames:'messageFile msgID location textLength time from to cc subject
		tocLineCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

IndexFileEntry comment:'An IndexFileEntry contains enough information to present a table-of-contents entry for the message and to support simple filtering based on a subset of the mail header information. The IndexFile maps unique message ID''s to IndexFileEntries.

'
!


!IndexFileEntry class methodsFor:'instance creation'!

message: aMailMessage location: location messageFile: aMessageFile msgID: msgID
	"Answer a new instance of me for the given message and message file location."

	^self new
		messageFile: aMessageFile;
		msgID: msgID;
		location: location;
		textLength: aMailMessage text size;
		time: aMailMessage time;
		from: aMailMessage from;
		to: aMailMessage to;
		cc: aMailMessage cc;
		subject: aMailMessage subject
!

readFrom: aStream messageFile: aMessageFile msgID: msgID
	"Answer a new instance of me initialized from the given text stream."

	^(self new readFrom: aStream)
		messageFile: aMessageFile;
		msgID: msgID
! !

!IndexFileEntry methodsFor:'access'!

cc

	^cc
!

cc: aString

	cc _ aString.
!

date
	"Answer a date string for this index entry."

	^Date fromDays: (time + (Date newDay: 1 year: 1980) asSeconds) // 86400
!

from

	^from
!

from: aString

	from _ aString.
!

likelyEqual: otherIndexFileEntry
	"return true if the two toc entries seem to represent the same message"
	^(self textLength = otherIndexFileEntry textLength and: [self subject = otherIndexFileEntry subject]) and: [self from = otherIndexFileEntry from]
!

location

	^location
!

location: anInteger

	location _ anInteger.
!

messageFile

	^messageFile
!

messageFile: aMessageFile

	messageFile _ aMessageFile.
!

msgID

	^msgID
!

msgID: anID

	msgID _ anID.
!

subject

	^subject
!

subject: aString

	subject _ aString.
!

textLength

	^textLength
!

textLength: anInteger

	textLength _ anInteger.
!

time

	^time
!

time: anInteger

	time _ anInteger.
!

to

	^to
!

to: aString

	to _ aString.
! !

!IndexFileEntry methodsFor:'filtering support'!

ccHas: stringOrList

	^ self field: cc has: stringOrList
!

field: field has: stringOrList
	"Return true if either the given field contains the argument string or, if the argument is a collection, return true if the given field contains any of the strings in that collection."

	| s |
	(stringOrList isKindOf: String) ifTrue: [
		^ field includesSubstring: stringOrList caseSensitive: false
	] ifFalse: [
		1 to: stringOrList size do: [ :i |
			s _ stringOrList at: i.
			s isNumber ifTrue: [s _ s printString].
			(field includesSubstring: s caseSensitive: false) ifTrue: [^ true].
		].
		^ false
	].
!

fromHas: stringOrList

	^ self field: from has: stringOrList
!

getMessage
	"Answer the MailMessage for this index file entry."
	^ MailMessage from: (self rawText)
!

participantHas: stringOrList

	^ (self field: from has: stringOrList) or:
	   [(self field: self to has: stringOrList) or:
	   [self field: self cc has: stringOrList]]
!

rawText
	"Answer the unparsed text for this entry."
	^ messageFile
			getMessage: msgID
			at: location
			textLength: textLength
!

subjectHas: stringOrList

	^ self field: subject has: stringOrList
!

text
	^self getMessage text
!

textHas: stringOrList 
	^ self field: self text has: stringOrList
!

toHas: stringOrList

	^ self field: to has: stringOrList
! !

!IndexFileEntry methodsFor:'printing'!

computeTOCString
	"Answer a string for the table of contents."
	"IndexFileEntry allInstancesDo: [: e | e flushTOCCache]"

	| fromFieldSize s |
	fromFieldSize _ 18.
	s _ WriteStream on: (String new: 200).
	s nextPutAll: self dateString.
	[s position < 9] whileTrue: [s space].
	s nextPutAll: (self fromStringLimit: fromFieldSize).
	[s position <= (9 + fromFieldSize + 2)] whileTrue: [s space].
	s nextPutAll: subject decodeMimeHeader.
	^ s contents
!

computeTOCStringAsColumns
	"Answer a string for the table of contents."
	"IndexFileEntry allInstancesDo: [: e | e flushTOCCache]"
	| fromFieldSize array attachFlag |
	fromFieldSize _ 18.
	attachFlag _ Celeste showAttachmentsFlag
		ifTrue: [self getMessage body isMultipart]
		ifFalse: [false].
	array _ Array new: 5.
	array at: 1 put: self dateString.
	array
		at: 2
		put: (self fromStringLimit: fromFieldSize).
	array at: 3 put: subject decodeMimeHeader.
	array at: 4 put: self textLength asStringWithCommas.
	array at: 5 put: attachFlag.
	^ array
!

dateString
	"Answer a date string for this index entry."

	^self date printFormat: #(2 1 3 47 1 2)
!

flushTOCCache
	"Flush my cached table-of-contents entry string."
	"IndexFileEntry allInstancesDo: [: e | e flushTOCCache]"

	tocLineCache _ nil.
!

fromStringLimit: limit
	"Answer a cleaned up 'from' field for the table of contents."

	| editedFrom s ch i |
	editedFrom _ WriteStream on: (String new: limit + 1).
	s _ ReadStream on: from decodeMimeHeader.
	s skipSeparators.
	('"<' includes: s peek) ifTrue: [s next].
	((i _ from indexOf: $() > 0) ifTrue: [s position: i].
	[s atEnd] whileFalse: [
		ch _ s next.
		(('@<>)$"' includes: ch) or: [editedFrom position >= limit])
			ifTrue: [^editedFrom contents]
			ifFalse: [editedFrom nextPut: ch]].
	^editedFrom contents
!

printOn: aStream

	aStream nextPutAll: self dateString; cr.
	aStream nextPutAll: from; cr.
	aStream nextPutAll: to; cr.
	aStream nextPutAll: cc; cr.
	aStream nextPutAll: subject; cr.
	aStream nextPut: $(; nextPutAll: location printString; space.
	aStream nextPutAll: textLength printString; nextPut: $).
	aStream cr.
!

tocString
	"Answer a string for the table of contents."

	(tocLineCache isNil) ifTrue:
		[tocLineCache _ self computeTOCString].
	^tocLineCache
!

tocStringAsColumns
	"Answer a string for the table of contents."
	^self computeTOCStringAsColumns
! !

!IndexFileEntry methodsFor:'read-write'!

readFrom: aStream
	"Initialize myself from the given text stream."

	location _ MailDB readIntegerLineFrom: aStream.
	textLength _ MailDB readIntegerLineFrom: aStream.
	time _ MailDB readIntegerLineFrom: aStream.
	from _ MailDB readStringLineFrom: aStream.
	to _ MailDB readStringLineFrom: aStream.
	cc _ MailDB readStringLineFrom: aStream.
	subject _ MailDB readStringLineFrom: aStream.
!

writeOn: aStream
	"Write a human-readable representation of myself on the given text stream."

	aStream
		nextPutAll: location printString; cr;
		nextPutAll: textLength printString; cr;
		nextPutAll: time printString; cr;
		nextPutAll: from; cr;
		nextPutAll: to; cr;
		nextPutAll: cc; cr;
		nextPutAll: subject; cr.
! !

!IndexFileEntry methodsFor:'testing'!

comparableString: aString
	"This is for Celeste testing purposes only"

	"Return exactly this string.  This is the most strict mode, meaning things must be exactly equal"
	^ aString  

	"Make consecutive white space into a single space (less strict but still pretty)"
	"^ (aString collect: [ :ch | ch isSeparator ifTrue: [ Character space ] ifFalse: [ ch ] ])
			withBlanksCondensed"

	"Look only at nonwhite characters (least strict)"
	"^ aString select: [ :ch | ch isSeparator not ]"	



	"(self comparableString: 'a  b	c')  = (self comparableString: 'a	b     c')"
!

selfTestEquals: anIndexFileEntry
	"For testing and debugging purposes only, test whether the two entries are equivalent.  If you expect that IndexFileEntries should be identical, the use strict equality here.  Otherwise use approximate comparisons."

	"These should be exactly equal"
	#(messageFile msgID location from) do:
		[ :sel | ((self perform: sel) = (anIndexFileEntry perform: sel)) ifFalse: [ 
			Transcript cr.
			Transcript show: msgID printString, ' ', sel printString, ': ', (self perform: sel); cr.
			Transcript show: msgID printString, 'n', sel printString, ': ', (anIndexFileEntry perform: sel); cr.
]].

	"These should be comparably equal :-), typically varying only by white space"
	#(cc to subject) do:
		[ :sel | ((self comparableString: (self perform: sel)) =
				(self comparableString: (anIndexFileEntry perform: sel))) ifFalse: [
			Transcript cr.
			Transcript show: msgID printString, ' ', sel printString, ': ', (self perform: sel); cr.
			Transcript show: msgID printString, 'n', sel printString, ': ', (anIndexFileEntry perform: sel); cr.
]].

	"It could be that these are not absolutely identical, though they should be close"
	#(date time) do:
		[ :sel | (self perform: sel) = (anIndexFileEntry perform: sel) ifFalse:
					[Transcript cr;
						show: msgID printString, ' ', sel printString, ':', (self perform: sel) printString; cr]].

	[(self textLength - anIndexFileEntry textLength) abs <= 2] assert. 
! !

!IndexFileEntry class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/IndexFileEntry.st,v 1.1 2003/05/09 21:40:48 cg Exp $'
! !
