"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

Model subclass:#MailDB
	instanceVariableNames:'rootFilename messageFile indexFile categoriesFile
		canRenumberMsgIDs lastIssuedMsgID'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

MailDB comment:'I am the in-memory representation of a Celeste mail database. A mail database named "current" consists of three files:

	current.messages	-- an append-only file containing the full content of all messages
	current.index		-- an index of all messages in the messages file that maps unique
						   message ID''s to index entries containing some header information
						   and the offset of the full message in the .messages file
	current.categories	-- maps category names to collections of message ID''s

Each open mail database has a MailDB object that manages these three files. Operations such as fetching a message given its unique ID and finding out which messages are in which category are supported by the MailDB object. It also supports incorporating new messages (reading mail), message editing and deletions, and message file compaction and recovery.

canRenumberMsgIDs is used only during compacting the message file.  It controls whether renumbering can happen for messages which have duplicate (and therefore not unique) messageIDs.  It is an instance variable so that it can control behaviour in subsequent compactions.  Please do not use it for other behaviour.

lastIssuedMsgID contains the most recently issued message ID.  It is mostly private to nextUnusedID, though it is also cleared in the compact routine.  Otherwise, please do not read or write it.'
!


!MailDB class methodsFor:'instance creation'!

isADBNamed: dbname
	"return whether there is a MailDB on disk with the specified name"
	| status |
	status := self dbStatusFor: dbname.
	^status ~~ #doesNotExist.
!

openOn: rootFilename
	"Open or create a mail database with the given root filename. If an instance of me exists with the given root filename, return a reference to that instance rather than creating a new one. This allows multiple MailReaders to be open on the same database without synchronization problems."

	| alreadyOpenDB |
	alreadyOpenDB _ self allSubInstances
		detect:
			[: db |
			 (db rootFilename notNil) and:
			 [db rootFilename = rootFilename]]
		ifNone: [nil].
	(alreadyOpenDB notNil)
		ifTrue: [^alreadyOpenDB]
		ifFalse: [^(self new) openOn: rootFilename].
! !

!MailDB class methodsFor:'private'!

dbStatusFor: rootFilename

	"See if the named databes exists. Since the database has several components, the answer is one of:
	#exists				all files exist, and were created in the right order
	#partialDatabase	only some of the files exist
	#doesNotExist		none of the files exist"

	| dir localName messageFileExists indexFileExists categoriesFileExists messageFileTime categoriesFileTime |

	dir _ FileDirectory forFileName: rootFilename.
	localName _ FileDirectory localNameFor: rootFilename.

	messageFileExists _ dir includesKey: localName, '.messages'.
	indexFileExists _ dir includesKey: localName, '.index'.
	categoriesFileExists _ dir includesKey: localName, '.categories'.

	"Check if no parts of the database exist"
	(messageFileExists | indexFileExists | categoriesFileExists) ifFalse: [^ #doesNotExist].

	"Check if the database was written in a normal sequence"
	(messageFileExists & indexFileExists & categoriesFileExists) ifTrue: [
		messageFileTime _ (dir entryAt: localName, '.messages') modificationTime.
		categoriesFileTime _ (dir entryAt: localName, '.categories') modificationTime.

		"Unfortunately the strongest thing we can say is that the message file should be the oldest file on disk, and the categories file the newest"
		(messageFileTime <= categoriesFileTime)
			ifTrue: [^ #exists].
		].

	^ #partialDatabase
! !

!MailDB class methodsFor:'shut down'!

shutDown
	"snapshot all mail databases to disk"

	self allSubInstancesDo: [:db | db saveDB]
! !

!MailDB class methodsFor:'utilities'!

readIntegerLineFrom: aStream
        "Read a positive integer from the given stream. Answer zero if there are no digits. Consume the stream through the next carriage return."

        | digit value |
        value _ 0.
        [aStream atEnd] whileFalse: 
                [digit _ aStream next digitValue.
                 ((digit >= 0) & (digit <= 9))
                        ifTrue: 
                                [value _ (value * 10) + digit]
                        ifFalse:
                                [(digit == Character cr digitValue) ifFalse:
                                        [self skipRestOfLine: aStream].
                                 ^value]].
        ^value
!

readStringLineFrom: aStream 
	"Read and answer the next line from the given stream. Consume the carriage return but do not append it to the string."

	| |

	^aStream upTo: Character cr
!

skipRestOfLine: aStream
	"Consume characters from the given stream through the next carriage return."

	| crValue |
	crValue _ Character cr asciiValue.
	[aStream atEnd or:
	 [aStream next asciiValue == crValue]] whileFalse:
		["consume until end of stream or a carriage return"].
! !

!MailDB methodsFor:'*sbceleste-operations'!

hasFile
	^messageFile notNil and: [ categoriesFile notNil ].
!

itemActionFor: aVisitor
	^aVisitor doMailDB: self
! !

!MailDB methodsFor:'categories'!

addCategory: categoryName
	"Create a category with the given name if one doesn't already exist."

	categoriesFile addCategory: categoryName.
!

allCategories
	"Answer a list of categories sorted in alphabetical order, including the special categories."

	^categoriesFile categories asSortedCollection asOrderedCollection
!

categoriesThatInclude: msgID
	"Answer a collection of names for real categories that include the message with the given ID. Pseudo-categories (such as '.unclassified.') are not considered real categories."

	^categoriesFile categories select:
		[: catName |
		 (categoriesFile messagesIn: catName) includes: msgID]
!

file: msgID inCategory: categoryName
	"File the message with the given ID in the given category."

	categoriesFile file: msgID inCategory: categoryName.
!

fileAll: msgIDList inCategory: categoryName
	"File all the messages with ID's in the given list in the given category."

	msgIDList do:
		[: msgID |
		 categoriesFile file: msgID inCategory: categoryName].
!

messagesIn: categoryName 
	"Answer a collection of message ID's for the messages in the given 
	category, sorted in ascending time order.  If the category does not exist, 
	answer an empty collection. The pseudo-categories '.all.' and 
	'.unclassified.' are computed dynamically, which may take a little time."
	| msgList category |
	categoryName = '.unclassified.'
		ifTrue: 
			[Cursor execute showWhile: [msgList _ categoriesFile unclassifiedFrom: indexFile keys].
			^ msgList].
	categoryName = '.all.' ifTrue: [^ indexFile keys].
	"otherwise, it is a real category"
	category _ categoriesFile messagesIn: categoryName.
	^self sortedKeysForMessages: category.
	
!

remove: msgID fromCategory: categoryName
	"Remove the message with the given ID from the given category."

	categoriesFile remove: msgID fromCategory: categoryName.
!

removeAll: msgIDList fromCategory: categoryName
	"Remove all the messages with ID's in the given list from the given category."

	msgIDList do:
		[: msgID |
		 categoriesFile remove: msgID fromCategory: categoryName].
!

removeCategory: categoryName
	"Remove the category with the given name. This does nothing if the category does not exist or if it is a pseudo-category."

	categoriesFile removeCategory: categoryName.
!

removeFromAllCategories: msgID
	"Remove the message with the given ID from all categories. The message will appear in 'unclassified'."

	categoriesFile categories do:
		[: categoryName |
		 categoriesFile remove: msgID fromCategory: categoryName].
!

renameCategory: oldName to: newName
	"Rename the given category. This does nothing if the category does not exist or if it is a special category ('.all.' or '.unclassified.')."

	categoriesFile renameCategory: oldName to: newName.
!

sortedKeysForMessages: aSet 
	"use a heuristic to choose method of obtaining sorted message list"
	^ aSet size * (aSet size log: 2) * 3 > indexFile keysCount
		ifTrue: [indexFile keys select: [:msgID | "indexFile keys is sorted"
				aSet includes: msgID]]
		ifFalse: [(aSet asArray
				mergeSortFrom: 1
				to: aSet size
				by: [:a :b | (self getTOCentry: a) time < (self getTOCentry: b) time]) asOrderedCollection]
! !

!MailDB methodsFor:'copying'!

veryDeepCopyWith: deepCopier
	"don't copy MailDB's -- they refer to external state in files, and the user almost certainly does not intend for a completely independent MailDB to be created"
	^self
! !

!MailDB methodsFor:'fetch-import-export'!

fetchMailFromPOP: server userName: userName password: password doFormatting: doFormatting deleteFromServer: deleteFromServer
	"Download mail from the given POP3 mail server and append it this mail database. Answer the number of messages fetched. If doFormatting is true, messages will be formatted as they are received. If deleteFromServer is true, then messages will be removed from the POP3 server after being successfully retrieved. (Note: If there is a failure while fetching mail, all messages will be left on the server.)"

	| popConnection msgCount |
	Socket initializeNetwork.
	popConnection _ POPSocket new
		serverName: server;	
		userName: userName;
		password: password;
		addProgressObserver: Transcript.
	Utilities
		informUser: 'connecting to ', server
		during: [popConnection connectToPOP].
	popConnection isConnected ifFalse: [^ -1].

	msgCount _ popConnection numMessages.
	msgCount > 0 ifTrue: [
		self fetchMessageCount: msgCount
			fromPOPConnection: popConnection
			doFormatting: doFormatting.

		deleteFromServer ifTrue: [
			self removeMessageCount: msgCount fromPOPConnection: popConnection]].

	popConnection disconnectFromPOP.
	^ msgCount
!

fetchMessageCount: msgCount fromPOPConnection: popConnection doFormatting: doFormatting
	"Download the given number of messages from the given open POP3 connection. If doFormatting is true, messages will be formatted as they are received."

	| nextID msgText msg location |
	messageFile beginAppend.
	('Downloading ', msgCount printString, ' messages...')
		displayProgressAt: Sensor mousePoint
		from: 0
		to: msgCount
		during: [:progressBar |
			1 to: msgCount do: [:messageNum |
				progressBar value: messageNum.
				popConnection isConnected ifFalse: [
					popConnection destroy.  "network error"
					messageFile endAppend.
					self saveDB.
					^ self inform: 'Server connection unexpectedly closed.'].

				"get a message"
				msgText _ popConnection retrieveMessage: messageNum.
				nextID _ self nextUnusedID.

				"save that message"
				msg _ MailMessage from: msgText.
				doFormatting ifTrue: [msg format].
				location _ messageFile basicAppend: msg text id: nextID.
				indexFile
					at: nextID
					put: (IndexFileEntry
						message: msg
						location: location
						messageFile: messageFile
						msgID: nextID).
				categoriesFile file: nextID inCategory: 'new'.
				]].

	messageFile endAppend.
	self saveDB.
!

fetchNewsFrom: inboxPathName doFormatting: doFormatting deleteInbox: deleteInbox
	"Append the messages from the given news inbox file to this mail database. Answer the number of messages fetched."

	| inbox nextID count msg location |
	"is there any news?"
	((FileDirectory on: inboxPathName) includesKey: 'news') ifFalse: [^ 0].

	inbox _ RNInboxFile openOn: inboxPathName, ':news'.
	count _ 0.
	messageFile beginAppend.
	inbox newsMessagesDo:
		[: newsgroup : msgText |
		 msg _ MailMessage from: msgText.
		 nextID _ self nextUnusedID.
		 doFormatting ifTrue: [msg format].
		 location _ messageFile basicAppend: msg text id: nextID.
		 indexFile
			at: nextID
			put: (IndexFileEntry
					message: msg
					location: location
					messageFile: messageFile
					msgID: nextID).
		 categoriesFile file: nextID inCategory: newsgroup.
		 categoriesFile file: nextID inCategory: 'new'.
		 count _ count + 1].
	messageFile endAppend.

	"snapshot the database and remove the inbox file"
	self saveDB.
	deleteInbox ifTrue: [inbox delete].
	^ count
!

importMailFrom: inboxFileName intoCategory: category
	"Append the messages from the given mail file to this mail database, and store them in the given category. Answer the number of messages imported."

	| inbox nextID count msg location |
	inbox _ MailInboxFile openOn: inboxFileName.
	count _ 0.
	messageFile beginAppend.
	inbox mailMessagesDo: [:msgText |
		 msg _ MailMessage from: msgText.
		 nextID _ self nextUnusedID.
		 location _ messageFile basicAppend: msg text id: nextID.
		 indexFile
			at: nextID
			put: (IndexFileEntry
					message: msg
					location: location
					messageFile: messageFile
					msgID: nextID).
		 categoriesFile file: nextID inCategory: category.
		 count _ count + 1].
	messageFile endAppend.
	self saveDB.
	^ count
!

mergeMessages: msgIDList from: sourceDB
	"Merge the given collection of messages from the source database into the receiver. When a message being added has the same message ID as an existing message, check to see if the two message texts are identical. If so, do not store the duplicate message. If the texts are different, make a new ID for the message being added. This operation will also copy the category information from the sourceDB, creating new catetories if necessary."

	| msgText newMsgID saveIt msg location entry |
	messageFile beginAppend.
	msgIDList do:
		[: oldMsgID |
		 msgText _ sourceDB getText: oldMsgID.

		 "resolve ID conflicts"
		 (indexFile includesKey: oldMsgID)
			ifFalse:	"no ID conflict"
				[newMsgID _ oldMsgID.
				 saveIt _ true]
			ifTrue:	"resolve an ID conflict"
				[(msgText = (self getText: oldMsgID))
					ifTrue:	"identical text; don't save again"
						[newMsgID _ oldMsgID.
						 saveIt _ false]
					ifFalse:	"different text; save with new ID"
						[newMsgID _ self nextUnusedID.
						 saveIt _ true]].

		 "save the message in the destination DB"
		 saveIt ifTrue:
			[msg _ MailMessage from: msgText.
			 location _ messageFile basicAppend: msg text id: newMsgID.
			 entry _ IndexFileEntry
						message: msg
						location: location 
						messageFile: messageFile
						msgID: newMsgID.
			 indexFile at: newMsgID put: entry].

		 "update the categories for the message in the destination DB"
		 (sourceDB categoriesThatInclude: oldMsgID) do:
			[: categoryName |
			 self file: newMsgID inCategory: categoryName]].
	messageFile endAppend.
!

removeMessageCount: msgCount fromPOPConnection: popConnection
	"Remove messages 1 through msgCount from the given POP3 server."

	('Removing ', msgCount printString, ' messages from the server...')
		displayProgressAt: Sensor mousePoint
		from: 0
		to: msgCount
		during: [:progressBar |
			1 to: msgCount do: [:messageNum |
				progressBar value: messageNum.
				popConnection isConnected ifFalse: [
					popConnection destroy.  "network error"
					^ self inform: 'Server connection unexpectedly closed.'].
				popConnection deleteMessage: messageNum]].
! !

!MailDB methodsFor:'housekeeping'!

appendMessages: msgBuffer messageFile: msgFile indexFile: idxFile
	"Append the given collection of messages to the message file. msgBuffer is a collection of (message ID, message text) pairs."

	| id msgText location entry |
	msgBuffer do:
		[: idAndText |
		 id _ idAndText at: 1.
		 msgText _ idAndText at: 2.
		 location _ msgFile basicAppend: msgText id: id.
		 entry _ indexFile
			at: id
			ifAbsent:
				[IndexFileEntry
					message: (MailMessage from: msgText)
					location: location
					messageFile: msgFile
					msgID: id].
		 entry _ (entry copy) location: location; textLength: msgText size.
		 idxFile at: id put: entry].
!

cleanUpCategories
	"Prune the dead wood out of all categories."

	categoriesFile categories do:
		[: category |
		 categoriesFile removeMessagesInCategory: category butNotIn: indexFile].
!

compact
	"Compact the message file and rebuild the index file. Answer an array  
	containing with the number of messages and the number of bytes  
	recovered."
	| newMessageFile newIndexFile stats |
	newMessageFile _ MessageFile openOn: rootFilename , '.messages.tmp'.
	"don't read log file here!!"
	newIndexFile _ IndexFile
				openOn: rootFilename , '.index.tmp'
				messageFile: newMessageFile
				readLogFlag: false.
	stats _ self copyUndeletedTo: newMessageFile indexFile: newIndexFile.
	newMessageFile save.
	newIndexFile save.
	messageFile rename: rootFilename , '.messages.bak'.
	indexFile rename: rootFilename , '.index.bak'.
	newMessageFile rename: rootFilename , '.messages'.
	newIndexFile rename: rootFilename , '.index'.
	indexFile delete.
	messageFile delete.
	messageFile _ MessageFile openOn: rootFilename , '.messages'.
	"update messageFile in IndexFile entries by clean reopen of indexFile"
	indexFile _ IndexFile
				openOn: rootFilename , '.index'
				messageFile: messageFile
				readLogFlag: true.
	self cleanUpCategories.
	categoriesFile save.
	^ stats
!

copyUndeletedTo: newMsgFile indexFile: newIndexFile
        "Copy all the undeleted messages in my current message file into the new message file, recording their locations in the new index file.  Also eliminates duplicate messageIDs.  Answer an array containing with the number of messages and the number of bytes recovered, as well as the number of messages still remaining."

        | bufferLimit msgBuffer bufferSize deletedCount deletedBytes msgIDlist keptCount duplicateMsgCount |

        msgIDlist _ Set new: 10000.     "Record of the msgIDs we have already processed"
        deletedCount _ deletedBytes _ 0.
        keptCount _ 0.
        duplicateMsgCount _ 0.

        "Note: To reduce disk seeks, messages are buffered and written in large batches.  You may wish to tune the amount of buffering if you have a particular shortage or abundance of physical memory.  bufferLimit is the approximate number of bytes of messages that will be accumulated before writing the buffered messages to disk."
        Smalltalk garbageCollect.
        msgBuffer _ OrderedCollection new: 1000.
        bufferLimit _ (Smalltalk primBytesLeft // 2) min: 2000000.
        bufferSize _ 0.

        newMsgFile beginAppend.
        messageFile messagesDo:
                [: deleted : msgIDArg : msgText | |msgID|

                 msgID := msgIDArg.
                 (deleted)
                        ifTrue:
                                [deletedCount _ deletedCount + 1.
                                 deletedBytes _ deletedBytes + msgText size]
                        ifFalse:
                                [
                                (msgIDlist includes: msgID) ifTrue: [   "We have a duplicate msgID"
                                        "We only renumber if we have previously salvaged"
                                        "and thus know all of the existing message IDs"
                                        (canRenumberMsgIDs = true)
                                                ifTrue: [msgID _ self nextUnusedID]
                                                ifFalse: [duplicateMsgCount _ duplicateMsgCount + 1]
                                        ].
                                msgIDlist add: msgID.

                                msgBuffer addLast: (Array with: msgID with: msgText).
                                keptCount _ keptCount + 1.
                                 bufferSize _ bufferSize + msgText size.
                                 (bufferSize >= bufferLimit) ifTrue:
                                        [self
                                                appendMessages: msgBuffer
                                                messageFile: newMsgFile
                                                indexFile: newIndexFile.
                                         msgBuffer _ OrderedCollection new: 1000.
                                         bufferSize _ 0]]].

        "flush remaining buffered messages"
        self
                appendMessages: msgBuffer
                messageFile: newMsgFile
                indexFile: newIndexFile.
        newMsgFile endAppend.

        (canRenumberMsgIDs = true) ifTrue: [
                canRenumberMsgIDs _ nil.        "We're now done with renumbering duplicate IDs"
                [duplicateMsgCount = 0] assert.
                ].

        duplicateMsgCount > 0 ifTrue: [
                canRenumberMsgIDs _ true.               "Allow renumbering duplicate IDs next time around"
                lastIssuedMsgID _ nil.                  "Ensure it is later initialized by nextUnusedID"
                self inform:
                        'Warning:  ', duplicateMsgCount printString, ' duplicate msgIDs were found.', String cr,
                        'Please use "salvage & compact" again to replace them with the correct unique IDs'.
                ].

        "return statistics"
        ^Array with: deletedCount with: deletedBytes with: keptCount
!

fileDuplicatesIn: categoryName
	"MailDB someInstance fileDuplicatesIn: '.duplicates.'"

	self fileAll: self findDuplicates inCategory: categoryName.
!

findDuplicates
	"MailDB someInstance findDuplicates"
	| msgsAtTime duplicates msgsBySize similarMessages textCache first second text1 text2 |
	duplicates _ PluggableSet integerSet.
	msgsAtTime _ ((self messagesIn: '.all.')
				collect: [:e | indexFile at: e])
				groupBy: [:e | e time]
				having: [:arr | arr size > 1].
	msgsAtTime
		associationsDo: 
			[:assoc | 
			msgsBySize _ assoc value groupBy: [:e | e textLength]
						having: [:arr | arr size > 1].
			msgsBySize
				associationsDo: 
					[:assoc2 | 
					similarMessages _ assoc2 value.
					textCache _ PluggableDictionary integerDictionary.
					similarMessages combinations: 2
						atATimeDo: 
							[:each | 
							first _ each first.
							second _ each second.
							((first likelyEqual: second)
								and: 
									[text1 _ textCache at: first msgID ifAbsentPut: [(self getMessage: first msgID) bodyText].
									text2 _ textCache at: second msgID ifAbsentPut: [(self getMessage: second msgID) bodyText].
									text1 = text2])
								ifTrue: [duplicates add: second msgID]]]].
	^ duplicates asArray
!

nextUnusedID
	"Answer the next unused message identifier number."

	"Each message needs to have a unique ID.  Message ID's are a monotonically increasing integers roughly related to the time that they were requested.  The last ID used is kept in lastIssuedMsgID, to guard against reuse (e.g. if the clock changes)."

	| id |
	(lastIssuedMsgID isNil) ifTrue: [ 		"find the largest msgID currently in this database"
		lastIssuedMsgID _ 0.
		indexFile keys do: [ :msgID | lastIssuedMsgID _ lastIssuedMsgID max: msgID].
		].

 	"message ID's are roughly the number of seconds since the beginning of 1980"
	id _ Date today asSeconds + Time now asSeconds -
		(Date newDay: 1 year: 1980) asSeconds.
	id _ id max: (lastIssuedMsgID + 1).  "never go backwards!!"
	lastIssuedMsgID _ id.
	^ id

	"MailDB someInstance nextUnusedID"
! !

!MailDB methodsFor:'initialize-release'!

close
	"Close up the database in preparation for closing celeste"

	self saveDB.
	indexFile ifNotNil: [indexFile save].
	messageFile ifNotNil: [messageFile close].

	rootFilename _ nil.
	messageFile _ indexFile _ categoriesFile _ nil.
!

openOn: rootNameString
	"Open a mail database with the given root file name."

	| status |
	rootFilename _ rootNameString.
	status _ self dbStatus.
	messageFile _ indexFile _ categoriesFile _ nil.
	(status = #exists) ifTrue: [^self openDB].
	(status = #partialDatabase) ifTrue: [^self recoverDB].
	(status = #doesNotExist) ifTrue: [^self createDB].
! !

!MailDB methodsFor:'messages'!

addNewMessage: message
	"Add the given message to the database, and answer its message id."

	| id location |
	id _ self nextUnusedID.
	location _ messageFile append: message text id: id.
	indexFile
		at: id
		put: (IndexFileEntry
				message: message
				location: location
				messageFile: messageFile
				msgID: id).
	^ id
!

deleteAll: msgIDList
	"Delete all the messages with ID's in the given list from the message file. This is permanent!!"

	msgIDList do:
		[: msgID |
		 messageFile
			deleteMessageAt: (indexFile at: msgID) location
			id: msgID.
		 indexFile remove: msgID].
	self cleanUpCategories.
!

getMessage: msgID
	"Answer the MailMessage with the given ID."

	^MailMessage from: (self getText: msgID)
!

getTOCentry: msgID
	"Answer the table-of-contents entry for the message with the given ID."

	^indexFile at: msgID
!

getTOCstring: msgID
	"Answer the table-of-contents string for the message with the given ID."

	^(indexFile at: msgID) tocString
!

getTOCstringAsColumns: msgID 
	^ (indexFile at: msgID) tocStringAsColumns
!

getText: msgID
	"Answer the text for the message with the given ID."

	| entry |
	entry _ indexFile at: msgID.
	^messageFile
		getMessage: msgID
		at: entry location
		textLength: entry textLength
!

newText: newText for: msgID
	"Replace the text for the message with the given ID."

	| oldLocation newLocation newEntry |
	oldLocation _ (indexFile at: msgID) location.
	newLocation _ messageFile update: newText at: oldLocation id: msgID.
	newEntry _ IndexFileEntry
		message: (MailMessage from: newText)
		location: newLocation
		messageFile: messageFile
		msgID: msgID.
	indexFile at: msgID put: newEntry.
! !

!MailDB methodsFor:'open-create-save'!

createDB
	"Create a new mail database."


	self openDB.							"creates new DB files"
	self saveDB.							"save the new mail database to disk"
!

dbStatus
	^self class dbStatusFor: rootFilename
!

openDB
	"Open an existing mail database."

	Transcript show: 'Opening mail database ''', rootFilename, '''...'.
	messageFile _ MessageFile openOn: rootFilename, '.messages'.
	indexFile _ IndexFile openOn: rootFilename, '.index' messageFile: messageFile readLogFlag: true.
	categoriesFile _ CategoriesFile openOn: rootFilename, '.categories'.
	Transcript show: 'Done.'; cr.
!

recoverDB
	"Open a mail database with the given root file name."

	(self confirm:
'The mail database named:
    ', rootFilename, '
appears to be damaged.
Shall I fix it?  (This might take some time)')
		ifFalse: [self release. ^nil].

	self openDB.
	Cursor execute showWhile: [self compact].
!

rootFilename
	"Answer my root filename."

	^rootFilename
!

saveDB
	"Write all database files to disk."

	"Return quietly if the database is no longer in use"
	rootFilename isNil & messageFile isNil & indexFile isNil & categoriesFile isNil
		ifTrue: [^self].

	Transcript show: 'Saving mail database ''' , (rootFilename ifNil: ['']) , '''...'.

	messageFile notNil ifTrue: [messageFile save].
	"indexFile notNil ifTrue: [indexFile save].    Omitted because of incremental log file saving"
	categoriesFile notNil ifTrue: [categoriesFile save].

	Transcript show: 'Done.'; cr
! !

!MailDB methodsFor:'printing'!

printOn: aStream 
	aStream nextPutAll: 'a MailDB on '.
	rootFilename ifNotNil: [aStream nextPutAll: '''' , rootFilename , '''']
! !

!MailDB methodsFor:'testing'!

selfTest
	"This is purely for testing purposes.  It checks out various things to make sure that everything is well formed and looks as it should.  This can be a bit slow, but is very useful because it tests much of Celeste using every message in the mail database"

"NOTE: The mechanism used to generate IndexFileEntries has changed significantly over time, especially as MIME support was added and bugs were fixed.  That means that entries generated two years ago can sometimes be different from what would be generated with a current system.  Part of the selfTest compares the actual entry in the index file to what would be generated now.  The differences highlight three things:
	(1) Changes in convention between then and now (e.g. how we handle white space in a header continuation lines)
	(2) Results of bugs that were in the system when the entry was created and are now fixed
	(3) Functional result of changes made to Celeste
(1) and (3) are particularly helpful to use as part of testing enhancements to Celeste."

	| msgIDlist delCount goodCount msg dupid msgTextFromID indexEntry testEntry |

	msgIDlist _ Set new: 10000.
	delCount _ goodCount _ 0.
	dupid _ 0.

	messageFile messagesDo: [ :deleted :msgID :msgBody |

		deleted
			ifTrue: [ delCount _ delCount + 1 ]
			ifFalse: [ goodCount _ goodCount + 1 ].

		(msgIDlist includes: msgID)
			ifTrue: [dupid _ dupid + 1]
			ifFalse: [deleted ifFalse: [msgIDlist add: msgID]].

		"Try creating a formated version of the message from it's raw text"
		msg _ MailMessage from: msgBody.
		msg selfTest.
		
		deleted ifFalse: [
			"Check the indexing information for this message"

			"Check that the contents of this message is the same as what the index provides"
			msgTextFromID _ self getText: msgID.
			[msgTextFromID = msgBody] assert.

			"Check that the index entry is equivalent to what would be produced now"
			indexEntry _ indexFile at: msgID.
			testEntry _ IndexFileEntry message: msg location: indexEntry location messageFile: messageFile msgID: msgID.
			indexEntry selfTestEquals: testEntry. 
		]].

	Transcript cr; show: 'Dup:', dupid asString, '  del:', delCount asString, '  good:', goodCount asString; cr.

	"MailDB someInstance selfTest"
! !

!MailDB class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/MailDB.st,v 1.1 2003/05/09 21:41:04 cg Exp $'
! !
