"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailDBFile subclass:#MailInboxFile
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

MailInboxFile comment:'I am a parser for mail inboxes in a form found in the Unix ''/var/spool'' inboxes. This is also the format used by Eudora 3.0 on the Macintosh (and perhaps by other versions of Eudora as well).
'
!


!MailInboxFile methodsFor:'read-write'!

readFrom: aStream
	"This operation is a noop for mail inboxes. Use 'messagesDo:' to enumerate the messages in the inbox."
!

writeOn: aStream
	"This operation is illegal for mail inboxes."

	self error: 'Mail inboxes are read only!!'
! !

!MailInboxFile methodsFor:'scanning'!

delimitersDo: aBlock
	"Invoke the given block for each message in the mail inbox. The block argument is the text of a new message."

	| fileStream stream msgStart msgSize msgText delim |
	fileStream _ FileStream fileNamed: filename.
	Smalltalk garbageCollect.
	(fileStream size < (Smalltalk bytesLeft - 200000))
		ifTrue: [
			"if possible, buffer the entire file in memory for speed"
			stream _ ReadStream on: (fileStream contentsOfEntireFile).
			fileStream _ nil]
		ifFalse: [
			"otherwise, use the actual file stream, reading from disk"
			stream _ fileStream].

	[self scanToNextMessageIn: stream] whileTrue: [
		aBlock value: (MailDB readStringLineFrom: stream)].

	fileStream = nil ifFalse: [fileStream close].
!

findPossibleMessageStart: aStream
	"Find the next line starting with the string 'From' followed by a space. Leave the input stream positioned at the character following the space."

	(self nextStringIs: 'From ' in: aStream) ifTrue: [^true].
	[true] whileTrue: [
		aStream skipTo: Character cr.
		[aStream peek = Character cr] whileTrue: [aStream next].
		(self nextStringIs: 'From ' in: aStream) ifTrue: [^true].
		aStream atEnd ifTrue: [^false].
	].
!

mailMessagesDo: aBlock
	"Invoke the given block for each message in the mail inbox. The block argument is the text of a new message."

	| fileStream stream msgStart msgSize msgText |
	fileStream _ CrLfFileStream readOnlyFileNamed: filename.
	Smalltalk garbageCollect.
	(fileStream size < (Smalltalk bytesLeft - 200000))
		ifTrue: [
			"if possible, buffer the entire file in memory for speed"
			stream _ ReadStream on: (fileStream contentsOfEntireFile).
			fileStream _ nil]
		ifFalse: [
			"otherwise, use the actual file stream, reading from disk"
			stream _ fileStream].

	self scanToNextMessageIn: stream.
	MailDB skipRestOfLine: stream.  "skip message delimiter"
	msgStart _ stream position.
	[self scanToNextMessageIn: stream] whileTrue: [
		msgSize _ stream position - msgStart.
		stream position: msgStart.
		msgText _ stream next: msgSize.
		MailDB skipRestOfLine: stream.  "skip message delimiter"
		msgStart _ stream position.
		aBlock value: msgText].

	"process final message"
	msgSize _ stream position - msgStart.
	msgSize > 0 ifTrue: [
		stream position: msgStart.
		msgText _ stream next: msgSize.
		aBlock value: msgText].

	fileStream = nil ifFalse: [fileStream close].
!

nextStringIs: aString in: aStream
	"If the next characters of the given stream form the given string, then advance the stream position by the size of the string and return true. Otherwise, leave the stream untouched and return false."

	| oldPosition |
	oldPosition _ aStream position.
	1 to: aString size do: [ :i |
		aStream next = (aString at: i) ifFalse: [
			aStream position: oldPosition.
			^false
		].
	].
	aStream position: oldPosition.
	^true
!

scanToNextMessageIn: aStream
	"Scan to the start of the next message in the given stream. Answer true if we find a message delimiter, false if we hit the end of the stream first. The stream is left positioned at the start of the next message or at the end of the stream."

	| msgStart line dayOfWeek year |
	[aStream atEnd] whileFalse: [
		(self findPossibleMessageStart: aStream) ifFalse: [^false].
		msgStart _ aStream position.
		aStream next: 5.  "skip 'From '"

		"skip address"
		[aStream peek isSeparator] whileFalse: [aStream next].
		[aStream peek = Character space] whileTrue: [aStream next].

		line _ MailDB readStringLineFrom: aStream.
		line size >= 7 ifTrue: [
			dayOfWeek _ (line copyFrom: 1 to: 3) asLowercase.
			year _ (line at: line size - 3) isDigit
					ifTrue: [(line copyFrom: line size - 3 to: line size) asNumber]
					ifFalse: [0].
			((#('sun' 'mon' 'tue' 'wed' 'thu' 'fri' 'sat') includes: dayOfWeek) and:
			[(year > 1900) and: [year < 2100]]) ifTrue: [
				aStream position: msgStart.
				^true  "found a message!!"
			].
		].
	].
	^false
! !

!MailInboxFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/MailInboxFile.st,v 1.1 2003/05/09 21:41:09 cg Exp $'
! !
