"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailDBFile subclass:#MessageFile
	instanceVariableNames:'file'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

MessageFile comment:'I represent the messages file of a mail database. This file is treated as (mostly) append-only. All new messages or edited messages are appended to the end of this file.

Messages are stored as ASCII text with seperator strings and some additional information between each message. If necessary, this file can be inspected or edited with a text editor; after editing, the index file should be recreated using the "compact" command. (This is necessary because the offsets kept in the index file will probably be different after the edits.)

When a message is deleted, its delimiter is changed to mark it as deleted, but it is left in the mail file until the next time "compact" command is executed. Generally, the reduction in disk fragmentation resulting from keeping the messages in a single file more than offsets the temporary space lost by having a few deleted messages hanging around between compactions. Compaction is done into a new copy of the file, followed by a renaming operation. Thus, if compaction fails, the original messages file will not be lost.
'
!


!MessageFile methodsFor:'file operations'!

close
	"Close the file."

	file ifNil: [^ self].
	file
		ensureOpen;
		setToEnd;
		close.
	file _ nil.
!

delete
	"I must close my file handle before the file can be deleted."

	self close.
	super delete.
!

ensureFileIsOpen
	"Make sure that my file is open. The file is automatically closed on snapshots."

	file
		ifNil: [file _ FileStream fileNamed: filename]
		ifNotNil: [file ensureOpen].
!

openOn: aFileName
	"Initialize myself for the message file with the given name."

	filename _ aFileName.
	file _ nil.
	self ensureFileIsOpen.
!

rename: newFileName
	"I must close my file handle before the file can be renamed."

	self close.
	super rename: newFileName.
!

save
	"Make sure the message file is flushed to disk. This is NOT atomic because MessageFiles can get large and there might not be enough disk space to save them atomically. Besides, it would be very slow."

	file ifNil: [^ self].
	file ensureOpen.
	file closed  "Will still be closed if no file present"
		ifFalse: [file setToEnd; close; reopen].
! !

!MessageFile methodsFor:'message operations'!

append: messageText id: messageID
	"Append the given message text with the given unique identifier. Answer the new location of the message."

	| location |
	self beginAppend.
	location _ self basicAppend: messageText id: messageID.
	self endAppend.
	^location
!

assertValidMessageAt: filePosition id: msgID
	"Verify that the given filePosition is, indeed, the start of a valid undeleted message with the given ID and raise an error if this assertion is false."

	| delimiter fileMsgID |
	"assume file is open"

	file position: filePosition.

	delimiter _ file next: 11.
	(delimiter = ('&&&&&start', String cr)) ifFalse:
		[^self reportInconsistency].

	fileMsgID _ MailDB readIntegerLineFrom: file.
	(msgID = fileMsgID) ifFalse:
		[^self reportInconsistency].

	^true.
!

basicAppend: messageText id: messageID
	"Append the given message text with the given message ID. Answer the new location of the message."
	"WARNING: This operation assumes:
		1. the sender positioned the stream to the end of the file (using beginAppend), and
		2. the sender will do an endAppend operation after all messages are appended to flush all file buffers to disk."

	| location |
	file setToEnd.
	location _ file position.
	file nextPutAll: '&&&&&start'.	"message delimiter"
	file cr.
	messageID printOn: file.		"message ID"
	file cr.
	file nextPutAll: messageText.
	^location
!

beginAppend
	"Set the file to the end prior to performing a sequence of basicAppend operations."

	self ensureFileIsOpen.
	file setToEnd.
!

deleteMessageAt: filePosition id: msgID
	"Mark as deleted the message with the given ID located at the given file position."

	self ensureFileIsOpen.
	(self assertValidMessageAt: filePosition id: msgID)
		ifFalse: [^false].		"Don't delete if it looks like we have a problem"

	file position: filePosition.
	file nextPutAll: '&&&&&XXXXX'.		"delimiter for deleted messages"
	file flush.
!

endAppend
	"Complete an append transaction by flushing the file to disk."

	self save.
!

getMessage: msgID at: start textLength: textSize
	"Retrieve the message with the given ID, location, and text size."

	self ensureFileIsOpen.
	self assertValidMessageAt: start id: msgID.
	^file next: textSize
!

scanToNextAndSigns: aStream
	"Scan the stream for 5 consecutive and-sign (&) characters.  If they are found, position the stream at the start of the and-signs and answer true.  Answer false if the end of the stream is reached"

	| chunk index blocksize target |

	target _ '&&&&&'.
	blocksize _ 4000.		"Must be more than target size :-)"

	"Quickly skip over sections that do not have and-signs."
	index _ 0.
	[index = 0]
		whileTrue:
			[chunk _ aStream next: blocksize.
			chunk size = 0 ifTrue: [^false].		"end of file"
			index _ chunk findString: target.

			"Handle the yucky case where the target might be split between
			this block and the next.  We back up a bit before continuing.
			We back up 4, since the whole target is clearly not there"
			(index = 0 and: [chunk size = blocksize]) ifTrue: [aStream skip: -4]. 
			].

	"We found some &s, so position the stream to read it"
	aStream skip: (chunk size - index + 1) negated.
	[aStream peek = $&] assert.
	^true
!

update: messageText at: oldMessagePosition id: msgID
	"Atomically update the message having the old location and ID with the given new text (e.g. when the user has edited a message). Answer the new location of the message."

	| newLocation |
	newLocation _ self append: messageText id: msgID.
	self deleteMessageAt: oldMessagePosition id: msgID.
	^newLocation
! !

!MessageFile methodsFor:'scanning'!

messagesDo: aBlock
	"Scan the message file and invoke the given block for each message in it. The block arguments are:
	deleted			true if this message is marked deleted
	msgID			the message ID
	msgBody			the message text
This operation is very expensive."

	| more deleted msgID textStart textSize msgBody |
	self ensureFileIsOpen.
	file position: 0.
	more _ self scanToNextMessageIn: file.
	[more] whileTrue:
		[deleted _								"deleted"
			(MailDB readStringLineFrom: file) = '&&&&&XXXXX'.
		 msgID _ MailDB readIntegerLineFrom: file.	"msgID"
		 textStart _ file position.
		 more _ self scanToNextMessageIn: file.
		 textSize _ file position - textStart.
		 file position: textStart.
		 msgBody _ file next: textSize.				"msgBody"
		 aBlock valueWithArguments:
			(Array
				with: deleted
				with: msgID
				with: msgBody)].
!

scanToNextMessageIn: aStream
	"Scan to the start of the next message. Answer true if we find a message delimiter, false if we hit the end of the file first. The stream is left positioned at the start of the next message (at the message delimiter) or at the end of the stream."

	| delimiter |
	[self scanToNextAndSigns: aStream] whileTrue:
		[delimiter _ aStream next: 10.
		 ((delimiter = '&&&&&start') or: [delimiter = '&&&&&XXXXX'])
			ifTrue: [aStream skip: -10. ^true]
			ifFalse: [aStream skip: -5]  "Keep going - it was't a delimiter"
		].
	^false	"end of file"
! !

!MessageFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/MessageFile.st,v 1.1 2003/05/09 21:40:59 cg Exp $'
! !
