"{ Package: 'stx:goodies/tools/mail/celeste' }"

"{ NameSpace: Tools }"

MailDBFile subclass:#ReadNewsInboxFile
	instanceVariableNames:'currentNewsgroup msgBuffer'
	classVariableNames:''
	poolDictionaries:''
	category:'Network-Mail Reader'
!

ReadNewsInboxFile comment:'I am a parser for news files in the format output by the Unix ''readnews'' program.
'
!


!ReadNewsInboxFile methodsFor:'read-write'!

readFrom: aStream
	"This operation is a noop for news inboxes. Use 'messagesDo:' to enumerate the messages in the inbox."
!

writeOn: aStream
	"This operation is illegal for news inboxes."

	self error: 'News inboxes are read only!!'
! !

!ReadNewsInboxFile methodsFor:'scanning'!

allDashes: aString
	"Answer true if the given string is not empty and consists entirely of dash characters."

	(aString isEmpty) ifTrue: [^false].
	aString detect: [: ch | ch ~= $-] ifNone: [^true].
	^false	"we must have detected a non-dash"
!

appendLine: aString
	"Append the given line to the buffer."

	msgBuffer nextPutAll: aString; cr.
!

endOfArticleDo: aBlock
	"We've just hit the end of an article. Evaluate the given block on the article we've been accumulating in the buffer (if any) and reset the buffer for the next article."

	| msgText end |
	"get text and remove trailing separators (blanks, cr's, etc)"
	msgText _ msgBuffer contents.
	end _ msgText size.
	[(end > 0) and: [(msgText at: end) isSeparator]] whileTrue: [end _ end - 1].
	(end > 1) ifTrue:
		[aBlock value: currentNewsgroup value: (msgText copyFrom: 1 to: end)].
	msgBuffer reset.	"reset the buffer for the next message"
!

newsMessagesDo: aBlock
	"Invoke the given block for each message in the news inbox file. The block arguments are the newsgroup name and the text of a new message."

	| fileStream stream |
	fileStream _ FileStream fileNamed: filename.
	(fileStream size < 50000)
		ifTrue:
			["for small inboxes, buffer the entire file in memory for speed"
			 stream _ ReadStream on: (fileStream contentsOfEntireFile)]
		ifFalse:
			["otherwise, use the actual file stream, reading from disk"
			 stream _ fileStream].
	self parse: stream do: aBlock.
	fileStream close.
!

parse: aStream do: aBlock
	"Parse the given stream into newsgroup articles, invoking the given block once for each article in the stream. The stream is divided into articles by two kinds of delimiters. The first kind indicates the start of a new newsgroup and includes the newsgroup name. The second kind indicates the start of a new article within a newsgroup."

	| done line nextLine |
	currentNewsgroup _ 'unknown newsgroup'.
	msgBuffer _ WriteStream on: (String new: 5000).
	done _ false.
	[done] whileFalse:
		[(aStream atEnd) ifTrue:
			["end of stream"
			 self endOfArticleDo: aBlock.
			 done _ true].
		 line _ MailDB readStringLineFrom: aStream.
		 (self allDashes: line) ifTrue:	"leading line of dashes"
			["could be a newsgroup header"
			 nextLine _ MailDB readStringLineFrom: aStream.
			 ((nextLine size >= 10) and:
			  [(nextLine copyFrom: 1 to: 10) = 'Newsgroup '])
				ifTrue:
					["yep, it is a newsgroup header"
					 self endOfArticleDo: aBlock.
					 self setNewsGroup: nextLine.
					 MailDB skipRestOfLine: aStream.	"skip trailing line of dashes"
					 MailDB skipRestOfLine: aStream.	"skip blank line"
					 MailDB skipRestOfLine: aStream.	"skip next article delimiter"
					 line _ MailDB readStringLineFrom: aStream]
				ifFalse:
					["nope, it's not a newsgroup header"
					 self appendLine: line.
					 line _ nextLine]].
		 (self startOfArticle: line) ifTrue:
			[self endOfArticleDo: aBlock.
			 line _ MailDB readStringLineFrom: aStream].
		 self appendLine: line].	"normal line: append it to the message buffer"
!

setNewsGroup: aLine
	"Set the current newsgroup name from the given line of text, which is of the form:
		Newsgroup comp.lang.smalltalk"

	(aLine size > 11)
		ifTrue: [currentNewsgroup _ aLine copyFrom: 11 to: aLine size]
		ifFalse: [currentNewsgroup _ 'unknown newsgroup'].
!

startOfArticle: aString
	"Answer true if the given string is the start of a new news article. That is, does it start with the string 'Article ' and end with a period?"

	^((aString size >= 8) and:
	   [((aString copyFrom: 1 to: 8) = 'Article ') & (aString last = $.)])
! !

!ReadNewsInboxFile class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/tools/mail/celeste/ReadNewsInboxFile.st,v 1.1 2003/05/09 21:41:14 cg Exp $'
! !
