"{ Package: 'stx:goodies/twoflower' }"

"{ NameSpace: Twoflower }"

TfResourceModel subclass:#TfDownloadModel
	instanceVariableNames:'stream filename status startTimestamp readStream downloadSize
		reportedProgress transferredSize progressProcess executeNeeded
		executeProcess'
	classVariableNames:''
	poolDictionaries:''
	category:'Twoflower-Base'
!


!TfDownloadModel class methodsFor:'accessing'!

handleDownload

	^ true
! !

!TfDownloadModel methodsFor:'accessing'!

downloadSize
	" number of bytes of the downloaded file "

	^ downloadSize
!

downloadSize: anInteger
	" number of bytes of the downloaded file "

	downloadSize :=anInteger
!

filename

	^ filename
!

filename: aFilename

	filename := aFilename
!

readStream
	" input stream (optional) "

	^ readStream
!

resource

	^ nil
!

startTimestamp

	^ startTimestamp
!

status

	^ status
!

stream
	" file stream "

	^ stream
!

transferredSize
	" number of transferred bytes "

	^ transferredSize
!

useCache

	^ false
! !

!TfDownloadModel methodsFor:'comparing'!

= anotherResourceModel

	^ self class = anotherResourceModel class
! !

!TfDownloadModel methodsFor:'downloading'!

download: aTfURLDescriptor to: aFilename
	" download the binary data from the URL to the file "

	self urlDescriptor: aTfURLDescriptor.
	self filename: aFilename.
	executeNeeded := true.
!

redirectEntity: aMimeEntity from: aTfURLDescriptor to: aFilename
	" redirect the entity stream to the file "

	| l |
	self urlDescriptor: aTfURLDescriptor.
	self filename: aFilename.
	(l := aMimeEntity contentLength) notNil
		ifTrue: [ self downloadSize: l ].
	executeNeeded := false.
	self initializeDownloadEntity: aMimeEntity.
!

startExecuteProcess

	executeNeeded
		ifFalse: [ ^ self ].
	self stopExecuteProcess.
	startTimestamp := Timestamp now.
	executeProcess := 
		[ 
			self execute.
			executeProcess := nil.
		]  forkAt: Processor userBackgroundPriority
! !

!TfDownloadModel methodsFor:'initialize-release'!

initialize 

	super initialize.
	reportedProgress := 0.
	transferredSize := 0.
	executeNeeded := false.
	startTimestamp := Timestamp now.
	self status: #Connecting.
!

shutDown

	self isActive
		ifTrue: [ self cancel ].
	self stopProgressProcess.
	self stopExecuteProcess.
! !

!TfDownloadModel methodsFor:'notification'!

onProgressSend: methodSelector to: anObject

	self when: #progress send: methodSelector to: anObject
!

onStatusChangeSend: methodSelector to: anObject

	self when: #changedStatus send: methodSelector to: anObject
! !

!TfDownloadModel methodsFor:'printing'!

sizeString

	^ self downloadSize notNil
		ifTrue: [ self downloadSize tfSizePrintString ]
		ifFalse: [ '(unknown)' ].
!

speedString

	| elapsed speed |
	elapsed := Timestamp now asSeconds - startTimestamp asSeconds.
	^ elapsed  > 0
		ifTrue: [ 
			speed := (self transferredSize / elapsed) rounded.
			speed tfSizePrintString, '/s'.
		] ifFalse: [
			''
		]
!

statusString

	| str prog |
	str := (String new: self status size) writeStream.
	str nextPutAll: self status asString.
	self isDownloading
		ifTrue: [
			str nextPutAll: ' ('.
			(prog := self downloadProgress) notNil
				ifTrue: [ 
					(prog * 100) rounded printOn: str. 
					str nextPut: $% ]
				ifFalse: [ 
					str nextPutAll: self transferredSize tfSizePrintString
				].
			str nextPut:$).
		].
	^ str contents
! !

!TfDownloadModel methodsFor:'private'!

chunkSize

	" todo "
	^ 1024
!

close

	self stream notNil
		ifTrue: [ self stream close ].
	self readStream notNil
		ifTrue: [ self readStream close ].
!

closeAndDelete

	self close.
	self filename exists
		ifTrue: [ self filename delete ]
!

execute

	| buffer |
	self initializeDownload.
	self startProgressProcess.
	buffer := ByteArray new: self chunkSize.
	[ readStream atEnd ] whileFalse: [ 
		| actualSize |
		actualSize := readStream nextAvailable: buffer size into: buffer startingAt: 1.
		stream next: actualSize putAll: buffer startingAt: 1.
	].
	self downloadSize: self stream position.
	self completed.
!

initializeDownload

	stream := self filename writeStream binary.
	readStream := self urlDescriptor url binaryReadStream.
	self status: #Downloading.
!

initializeDownloadEntity: aMimeEntity 

	stream := aMimeEntity isBinary 
				ifTrue: [self filename writeStream binary]
				ifFalse: [(self filename withEncoding: #ISO8859_1) writeStream lineEndCRLF].
	self status: #Downloading.
	self startProgressProcess.
!

progressLoopStep

	| prog |
	transferredSize := self stream position.
	prog := self downloadProgress.
	prog notNil
		ifTrue: [
			(prog = 1 or: [ (prog - reportedProgress) >= 0.01 ])
				ifTrue: [ 
					reportedProgress := prog.
					self triggerEvent: #progress with: transferredSize
				].
		]
		ifFalse: [
			(transferredSize - reportedProgress) >= self chunkSize
				ifTrue: [ 
					reportedProgress := transferredSize.
					self triggerEvent: #progress with: transferredSize.
				].
		].
	(Delay forSeconds: 1) wait.
!

startProgressProcess
	" instead of putting progress code at all places where the input stream is filled
	(which is not a good idea since some stuff belongs to the HTTP framework and is subject to change),
	start a polling process "

	self stopProgressProcess.
	progressProcess := 
		[
			[ self isActive ] whileTrue: [
				self progressLoopStep 
			].
			progressProcess := nil
		]  forkAt: Processor userBackgroundPriority
!

status: aSymbol

	status := aSymbol.
	self triggerEvent: #changedStatus with: status.
	Log debugDo: [
		self urlDescriptor notNil
			ifTrue: [ Log logDebug: [status asString, ' ', self urlDescriptor printString ]]].
!

stopExecuteProcess

	executeProcess notNil
		ifTrue: [ 
			executeProcess terminate.
			executeProcess := nil
		].
	^ executeProcess
!

stopProgressProcess

	progressProcess notNil
		ifTrue: [ 
			progressProcess terminate.
			progressProcess := nil
		].
	^ progressProcess
! !

!TfDownloadModel methodsFor:'state'!

cancel

	self stopExecuteProcess.
	self closeAndDelete.
	self status: #Canceled.
!

completed

	self close.
	self status: #Completed.
!

downloadProgress

	^ self downloadSize notNil
		ifTrue: [ self transferredSize / self downloadSize ]
		ifFalse: [ nil ].
!

failed

	self stopExecuteProcess.
	self closeAndDelete.
	self status: #Failed.
!

isActive

	^ #(Downloading Connecting) includes: self status
!

isDownloading

	^ self status = #Downloading
! !

!TfDownloadModel methodsFor:'views'!

createEmbeddedApp

	^ nil
! !

!TfDownloadModel class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/Twoflower__TfDownloadModel.st,v 1.1 2009/10/01 18:04:46 cg Exp $'
! !
