"{ Package: 'stx:goodies/twoflower/compat' }"

"{ NameSpace: XML }"

SAXDriver subclass:#SAXWriter
	instanceVariableNames:'output textMap attrMap hasOpenTag normalizeText notations
		newNamespaces'
	classVariableNames:''
	poolDictionaries:''
	category:'XML-VW-SAX'
!

!SAXWriter class methodsFor:'documentation'!

documentation
"
SAXWriter is a kind of SAXDriver that converts SAX events back into textutal XML documents.

Instance Variables:
	attrMap	&lt;Dictionary&gt;	Lists the characters that are not allowed to appear literally in attributes, and maps them to their quoted form.
	hasOpenTag	&lt;Boolean&gt;	True if we are inside an element that has no content as yet. This allows us to know when we reach the end tag whether we can write the element as &lt;x/&gt; rather than &lt;x&gt;&lt;/x&gt;.
	newNamespaces	&lt;nil | Dictionary&gt;	Maps namespace qualifiers to URIs, for namespaces which the current element defines differently than its parent.
	normalizeText	&lt;Boolean&gt;	Should be true in almost all cases--can be false only if the &quot;XML&quot; document being parsed is actually not XML, in which case it may be legal to not quote characters such as $&lt;.
	notations	&lt;Collection&gt;	List of notations defined in the document being parsed.
	output	&lt;Stream&gt;	Stream on which the XML is rendered as text.
	textMap	&lt;Dictionary&gt;	Lists the characters that are not allowed to appear literally in text, and maps them to their quoted form.

    [author:]
        cg (cg@CG-PC)

    [instance variables:]

    [class variables:]

    [see also:]

"
! !

!SAXWriter methodsFor:'DTD handler'!

notationDecl:nameString publicID:publicIDString systemID:systemIDString 
    notations == nil ifTrue:[
        notations := OrderedCollection new
    ].
    notations add:(Array 
                with:nameString
                with:publicIDString
                with:systemIDString)
! !

!SAXWriter methodsFor:'content handler'!

characters:aString from:start to:stop 
    |ch mapped|

    self closeOpenTag.
    normalizeText ifTrue:[
        start to:stop do:[:i | 
            ch := aString at:i.
            mapped := textMap at:ch ifAbsent:[ nil ].
            mapped == nil ifTrue:[
                output nextPut:ch
            ] ifFalse:[
                output nextPutAll:mapped
            ]
        ]
    ] ifFalse:[
        output 
            next:stop + 1 - start
            putAll:aString
            startingAt:start
    ]
!

comment:data from:start to:stop 
    self closeOpenTag.
    output
        nextPutAll:'<!!--';
        next:stop + 1 - start putAll:data startingAt:start;
        nextPutAll:'-->'
!

endElement:namespaceURI localName:localName qName:name 
    hasOpenTag == true ifTrue:[
        output nextPutAll:'/>'
    ] ifFalse:[
        output nextPutAll:'</' , name , '>'
    ].
    hasOpenTag := false.
!

processingInstruction:targetString data:dataString 
    self closeOpenTag.
    output
        nextPutAll:'<?';
        nextPutAll:targetString;
        space;
        nextPutAll:dataString;
        nextPutAll:'?>'
!

startDocument
    hasOpenTag := false
!

startDocumentFragment
    "Nonstandard extension to SAX"
    
    hasOpenTag := false
!

startElement:namespaceURI localName:localName qName:name attributes:attributes 
    |val|

    notations == nil ifFalse:[
        self emitDTD:name
    ].
    self closeOpenTag.
    output nextPutAll:'<'.
    output nextPutAll:name.
    (self sort:attributes) do:[:att | 
        output space.
        output nextPutAll:att tag asString.
        output nextPutAll:'="'.
        1 to:att value size do:[:i | 
            |ch mapped|

            ch := att value at:i.
            mapped := attrMap at:ch ifAbsent:[ nil ].
            mapped == nil ifTrue:[
                output nextPut:ch
            ] ifFalse:[
                output nextPutAll:mapped
            ]
        ].
        output nextPutAll:'"'
    ].
    newNamespaces == nil ifFalse:[
        newNamespaces keys asSortedCollection do:[:key | 
            output space.
            output 
                nextPutAll:(key isEmpty ifTrue:[ 'xmlns' ] ifFalse:[ 'xmlns:' , key ]).
            output nextPutAll:'="'.
            val := newNamespaces at:key.
            1 to:val size do:[:i | 
                |ch mapped|

                ch := val at:i.
                mapped := attrMap at:ch ifAbsent:[ nil ].
                mapped == nil ifTrue:[
                    output nextPut:ch
                ] ifFalse:[
                    output nextPutAll:mapped
                ]
            ].
            output nextPutAll:'"'
        ]
    ].
    newNamespaces := nil.
    hasOpenTag := true
!

startPrefixMapping:prefix uri:uri 
    newNamespaces == nil ifTrue:[
        newNamespaces := Dictionary new
    ].
    newNamespaces at:prefix put:uri.
! !

!SAXWriter methodsFor:'initialize'!

minimalCharacterMapping

	textMap := Dictionary new
		at: $< put: '&lt;';
		at: $& put: '&amp;';
			"Added > because in rare cases, like text that contains ]]>,
			the > must be quoted."
		at: $> put: '&gt;';
		yourself.
	attrMap := Dictionary new
		at: $< put: '&lt;';
		at: $& put: '&amp;';
		at: $" put: '&quot;';
		yourself.
!

normalizeText: aBoolean

	normalizeText := aBoolean
!

output: aStream

	output := aStream.
	normalizeText := true.
	notations := nil.
	textMap == nil ifTrue: [self minimalCharacterMapping].
! !

!SAXWriter methodsFor:'other'!

endCDATA

	normalizeText := true.
	output nextPutAll: ']]>'.
!

startCDATA

	normalizeText ifFalse: [self error: #NestedCDATANotPermitted << #xml >> 'nested CDATA sections are not permitted'].
	normalizeText := false.
	output nextPutAll: '<!![CDATA['.
! !

!SAXWriter methodsFor:'private'!

closeOpenTag

	hasOpenTag == true
		ifTrue:
			[output nextPutAll: '>'.
			hasOpenTag := false].
!

emitDTD: name

	| list |
	output nextPutAll: '<!!DOCTYPE ';
		nextPutAll: name;
		nextPutAll: ' [';
		cr.
	list := notations asSortedCollection: [:a1 :a2 | a1 first < a2 first].
	list do: [:notation |
		self emitNotation: notation.
		output cr].
	output nextPutAll: ']>'; cr.
	notations := nil.
!

emitNotation: array

	output nextPutAll: '<!!NOTATION ';
		nextPutAll: (array at: 1);
		space;
		nextPutAll: ((array at: 2) == nil ifTrue: ['SYSTEM'] ifFalse: ['PUBLIC']).
	(array at: 2) == nil
		ifFalse: [output nextPutAll: ' ''';
				nextPutAll: (array at: 2);
				nextPutAll: ''''].
	(array at: 3) == nil
		ifFalse: [output nextPutAll: ' ''';
				nextPutAll: (array at: 3);
				nextPutAll: ''''].
	output nextPutAll: '>'.
!

sort: attributes

	^attributes asSortedCollection: [:a1 :a2 | a1 tag asString < a2 tag asString]
! !

!SAXWriter class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/twoflower/compat/XML__SAXWriter.st,v 1.1 2009/10/02 00:57:20 cg Exp $'
! !
