"{ Package: 'stx:goodies/webServer/comanche' }"

HTTPService subclass:#ComancheService
	instanceVariableNames:'autoStart log module'
	classVariableNames:'Services'
	poolDictionaries:''
	category:'Comanche-Mimicry'
!

!ComancheService class methodsFor:'documentation'!

documentation
"
    This class provides a mimicry protocol for Comanche Services (which are subclasses of me).

    The protocol is compatible to the corresponding Squeak-ComanceService class.
    However, the implementation is for interaction with ST/X's HTTPServer instead of Comanche.
"
! !

!ComancheService class methodsFor:'initialization'!

initialize
    " ComancheService initialize  "
    
    super initialize.
    Services := Dictionary new.
    Smalltalk isSmalltalkX ifTrue:[
        Smalltalk addDependent:self.
    ] ifFalse:[
        Smalltalk addToStartUpList:self.
        Smalltalk addToShutDownList:self
    ].
! !

!ComancheService class methodsFor:'instance creation'!

named: aString
        ^self new registerAs: aString
! !

!ComancheService class methodsFor:'accessing'!

defaultPriority
        ^Processor highIOPriority
!

defaultSearchPriority
        ^self defaultPriority - 1
! !

!ComancheService class methodsFor:'change & update'!

update:something with:aParameter from:changedObject
    changedObject == ObjectMemory ifTrue:[
        something == #aboutToQuit ifTrue:[
            self shutDown.
            ^ self.
        ].
        something == #returnFromSnapshot ifTrue:[
            self startUp.
            ^ self.
        ].
    ].
    super update:something with:aParameter from:changedObject
! !

!ComancheService class methodsFor:'configuring - setup'!

loadPackage
    "raise an error: must be redefined in concrete subclass(es)"

    ^ self subclassResponsibility
! !

!ComancheService class methodsFor:'queries'!

isAbstract
    self == ComancheService ifTrue:[^ true].
    ^ super isAbstract
! !

!ComancheService class methodsFor:'registration'!

addService: aComancheService named: aString
        | tmp |
        tmp _ self services.
        (tmp includesKey: aString)
                ifTrue: [ self error: 'A service by the same name already exists' mayProceed:true ].
        tmp at: aString put: aComancheService
!

nameForService: aComancheService
        ^self services 
                keyAtIdentityValue: aComancheService 
                ifAbsent: [ '(unregisterd service)' ]
!

removeService: aComancheService
        | tmp |
        tmp _ self services.
        tmp removeKey: 
                (tmp keyAtIdentityValue: aComancheService ifAbsent: [ ^self ])
!

removeServiceNamed: aString
        self services removeKey: aString ifAbsent: []
!

replaceServiceNamed: aString with: aComancheService
        self services at: aString put: aComancheService
!

serviceNamed: aString
        ^self services at: aString ifAbsent: [nil]
!

services
        ^Services
! !

!ComancheService class methodsFor:'snapshots'!

shutDown
        self services do: [ :ea |
                ea prepareForShutdown
        ]
!

startUp
        self services do: [ :ea |
                ea prepareForStartup
        ].
        self services do: [ :ea |
                ea autoStart ifTrue: [ ea start ]
        ]
! !

!ComancheService methodsFor:'accessing'!

autoStart
    ^ autoStart == true
!

autoStart:aBoolean 
    autoStart := aBoolean
!

log
        ^log
!

log: aLog
        log _ aLog
!

module
    "return the value of the instance variable 'module' (automatically generated)"

    module isNil ifTrue:[
        self setupModule.
    ].

    ^ module
!

name
        ^self class nameForService: self
!

settingSelectors
    |slots|

    slots := super settingSelectors.
    slots remove:#module ifAbsent:[].
    ^ slots 
! !

!ComancheService methodsFor:'actions'!

processNetworkRequest:aComancheHttpRequest
    ^ self module process:aComancheHttpRequest.
!

separateHeaderFromData:aStream

    | buffer index|

    buffer := aStream contents.
    index := buffer indexOfSubCollection:(Comanche::HttpRequest endOfRequestHeaderMarker).
    ^ Array with:(buffer copyTo:index + 3) with:(buffer copyFrom:index + 4)
! !

!ComancheService methodsFor:'initialization'!

initialize
    super initialize.
    autoStart := false.
    module isNil ifTrue:[
        self setupModule.
    ].
!

setupModule
    self subclassResponsibility
! !

!ComancheService methodsFor:'printing & storing'!

nameInSettingsTable
    module notNil ifTrue:[
        ^ super nameInSettingsTable,'(',module className,')'

    ].
    ^ super nameInSettingsTable
!

printOn: aStream
    aStream nextPutAll: ('Service: ', self name).
    self isRunning ifTrue: [ 
            aStream nextPutAll: '[running]'
    ] ifFalse: [
            aStream nextPutAll: '[stopped]'
    ]
! !

!ComancheService methodsFor:'processing'!

process:aSTXHttpRequest
    |comancheHttpRequest response sqeakSocketStream headerAndData|

    self module isNil ifTrue:[
        self error:'No comancheModule'.
    ].

    sqeakSocketStream := Comanche::SimulatedSqueakSocketStream new
                        readStream:(aSTXHttpRequest data ? '') readStream;
                        writeStream:'' writeStream;
                        request:aSTXHttpRequest.

    comancheHttpRequest := Comanche::HttpRequest new
                        stxHttpRequest:aSTXHttpRequest;
                        stream:sqeakSocketStream;
                        method:aSTXHttpRequest method;
                        initUrl:aSTXHttpRequest rawUrl;
                        remoteAddress:aSTXHttpRequest peer hostAddress;
                        timestamp: Time totalSeconds;
                        header:aSTXHttpRequest fieldValues;
                        propertyAt:#localPort put:httpServer port;
                        "propertyAt:#localAddress put:self halt;"
                        rawPostFields;
                        yourself.
    
    response := self processNetworkRequest:comancheHttpRequest.
    response isNil ifTrue:[
        ^ self
    ].
    response isString ifTrue:[
        aSTXHttpRequest nextPutAll:response.
        ^ self.
    ].

    response := response asHttpResponseTo:comancheHttpRequest.

    response request:comancheHttpRequest.

    response 
        fieldAt: 'Connection' 
        put:(aSTXHttpRequest connection isKeepAlive ifTrue:['Keep-Alive'] ifFalse:['close']).

    response writeOn:sqeakSocketStream writeStream.
    headerAndData := self separateHeaderFromData:sqeakSocketStream writeStream.
    aSTXHttpRequest replyHeader:headerAndData first.
    aSTXHttpRequest nextPutAll:headerAndData second.
! !

!ComancheService methodsFor:'registration'!

prepareForShutdown
    "The smalltalk system is about to shutdown"
!

prepareForStartup
    "The smalltalk system just started up"
!

registerAs:aString
    self unregister.
    self class addService:self named:aString.

    self linkNames:(OrderedCollection with:'/' , aString).
    httpServer notNil ifTrue:[
        self registerServiceOn:httpServer.
    ].
!

unregister
    self stop.
    self class removeService:self
! !

!ComancheService methodsFor:'starting and stopping'!

start
        "Subclasses may override this method to take whatever
        is necessary to start this service"

        "self subclassResponsibility"
!

stop
        "Subclasses may override this method to take whatever
        is necessary to start this service"

        "self subclassResponsibility"
! !

!ComancheService class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/ComancheService.st,v 1.26 2010/06/14 10:38:37 cg Exp $'
!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/ComancheService.st,v 1.26 2010/06/14 10:38:37 cg Exp $'
! !

ComancheService initialize!
