"{ Package: 'stx:goodies/webServer/comanche' }"

"{ NameSpace: Comanche }"

Object subclass:#SimulatedSqueakFileDirectory
	instanceVariableNames:'fileName'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-STX-Support'
!


!SimulatedSqueakFileDirectory class methodsFor:'create/delete file'!

default
    ^ self on:(Filename defaultDirectory pathName)
!

deleteFilePath: fullPathToAFile
        "Delete the file after finding its directory"

        | dir |
        dir _ self on: (self dirPathFor: fullPathToAFile).
        dir deleteFileNamed: (self localNameFor: fullPathToAFile).
!

localNameFor: fullName
        "Return the local part the given name."

        self
                splitName: fullName
                to: [:dirPath :localName | ^ localName]
!

on: pathString
        "Return a new file directory for the given path, of the appropriate FileDirectory subclass for the current OS platform."

        | pathName |
        "If path ends with a delimiter (: or /) then remove it"
        ((pathName _ pathString) endsWith: self pathNameDelimiter asString) ifTrue: [
                pathName _ pathName copyFrom: 1 to: pathName size - 1].
        ^ self new fileName:pathName asFilename
! !

!SimulatedSqueakFileDirectory class methodsFor:'name utilities'!

dirPathFor: fullName
        "Return the directory part the given name."

        self
                splitName: fullName
                to: [:dirPath :localName | ^ dirPath]
!

splitName: fullName to: pathAndNameBlock
        "Take the file name and convert it to the path name of a directory and a local file name within that directory. FileName must be of the form: <dirPath><delimiter><localName>, where <dirPath><delimiter> is optional. The <dirPath> part may contain delimiters."

        | delimiter i dirName localName |
        delimiter _ self pathNameDelimiter.
        (i _ fullName findLast: [:c | c = delimiter]) = 0
                ifTrue:
                        [dirName _ String new.
                        localName _ fullName]
                ifFalse:
                        [dirName _ fullName copyFrom: 1 to: (i - 1 max: 1).
                        localName _ fullName copyFrom: i + 1 to: fullName size].

        ^ pathAndNameBlock value: dirName value: localName 
! !

!SimulatedSqueakFileDirectory class methodsFor:'queries'!

maxFileNameLength

    ^ OperatingSystem maxFileNameLength
!

pathNameDelimiter
"return the active directory class's directory seperator character"
        ^ Filename separator
!

slash
        ^ self pathNameDelimiter asString
! !

!SimulatedSqueakFileDirectory methodsFor:'accessing'!

checkName: aFileName fixErrors: fixing
        "Check a string aFileName for validity as a file name. Answer the original file name if it is valid. If the name is not valid (e.g., it is too long or contains illegal characters) and fixing is false, raise an error. If fixing is true, fix the name (usually by truncating and/or tranforming characters), and answer the corrected name. The default behavior is just to truncate the name to the maximum length for this platform. Subclasses can do any kind of checking and correction appropriate for their platform."

        | maxLength |
        aFileName size = 0 ifTrue: [self error: 'zero length file name'].
        maxLength _ self class maxFileNameLength.
        aFileName size > maxLength ifTrue: [
                fixing
                        ifTrue: [^ aFileName contractTo: maxLength]
                        ifFalse: [self error: 'file name is too long']].

        ^ aFileName
!

deleteFileNamed: localFileName
        "Delete the file with the given name in this directory."

    ^ self deleteFileNamed: localFileName ifAbsent:[]
!

deleteFileNamed: localFileName ifAbsent:aBlock
        "Delete the file with the given name in this directory."

        | fullName file|
        fullName _ self fullNameFor: localFileName.
        file := fullName asFilename.
        file exists ifTrue:[
            fullName asFilename delete
        ] ifFalse:[
            aBlock value
        ].
!

fileName:something
    "set the value of the instance variable 'fileName' (automatically generated)"

    fileName := something.
!

fullNameFor: fileName
        "Return a corrected, fully-qualified name for the given file name. If the given name is already a full path (i.e., it contains a delimiter character), assume it is already a fully-qualified name. Otherwise, prefix it with the path to this directory. In either case, correct the local part of the file name."
        "Details: Note that path relative to a directory, such as '../../foo' are disallowed by this algorithm."
        | correctedLocalName prefix |
        self class splitName: fileName to:
                [:filePath :localName |
                        correctedLocalName _ localName isEmpty 
                                ifFalse: [self checkName: localName fixErrors: true]
                                ifTrue: [localName].
                        prefix _ self fullPathFor: filePath].
        prefix isEmpty
                ifTrue: [^correctedLocalName].
        prefix last = self pathNameDelimiter
                ifTrue:[^ prefix, correctedLocalName]
                ifFalse:[^ prefix, self slash, correctedLocalName]
!

fullPathFor: path
        ^path isEmpty ifTrue:[fileName pathName] ifFalse:[path]
!

newFileNamed: localFileName
    |s mime|

    s := SimulatedSqueakFileStream for:(fileName construct:localFileName) writeStream.
    mime := MIMETypes mimeTypeForFilename:localFileName.
    mime isImage ifTrue:[
        s binary.
    ].
    ^ s.
!

oldFileNamed: localFileName
    |s mime|

    Smalltalk isSmalltalkX ifTrue:[
        s := SimulatedSqueakFileStream for:(fileName construct:localFileName) appendingWriteStream.
    ] ifFalse:[
        s := SimulatedSqueakFileStream for:(fileName construct:localFileName) writeStream.
    ].
    mime := MIMETypes mimeTypeForFilename:localFileName.
    mime isImage ifTrue:[
        s binary.
    ].
    ^ s.
!

pathNameDelimiter
        "Return the delimiter character for this kind of directory. This depends on the current platform."

        ^ self class pathNameDelimiter
! !

!SimulatedSqueakFileDirectory methodsFor:'create/delete file'!

deleteDirectory: dName

    |rem|

    rem := fileName construct:dName.
    rem exists ifTrue:[
        rem removeDirectory
    ].
! !

!SimulatedSqueakFileDirectory methodsFor:'operations'!

containingDirectory
    ^ self class new 
        fileName:(fileName directory).
!

copyFile: fileStream1 toFile: fileStream2
    | buffer |

    buffer _ String new: 50000.
    [fileStream1 atEnd] whileFalse:
            [fileStream2 nextPutAll: (fileStream1 nextInto: buffer)].
!

createDirectory:dirName

    ^ (fileName asFilename construct:dirName) makeDirectory.
!

directoryExists:subFileName
    |fn|

    fn := fileName construct:subFileName.
    ^ fn exists and:[fn isDirectory]
!

directoryNamed:subDirName
    ^ self class new 
        fileName:(fileName construct:subDirName).
!

directoryNames
    ^ fileName directoryContentsAsFilenames 
        select:[:each | each isDirectory]
        thenCollect:[:each | each baseName].
!

entries
    ^ fileName directoryContents collect:[:eachFile | self entryAt:eachFile]
!

entryAt:aBaseName
    " Squeak DirectoryEntry "
    |fn|

    fn := fileName construct:aBaseName.
    ^ Structure     
        with:('name' -> aBaseName)
        with:('creationTime' -> (fn creationTime ? fn modificationTime))
        with:('modificationTime' -> fn modificationTime)
        with:('dirFlag' -> fn isDirectory)
        with:('fileSize' -> fn fileSize)
!

fileExists:subFileName
    |fn|

    fn := fileName construct:subFileName.
    ^ fn exists and:[fn isDirectory not]
!

fileNamed:localFileName
    |s mime|

    s := SimulatedSqueakFileStream for:(fileName construct:localFileName) readWriteStream.
    mime := MIMETypes mimeTypeForFilename:localFileName.
    mime isImage ifTrue:[
        s binary.
    ].
    ^ s.
!

fileNames
    ^ fileName directoryContentsAsFilenames 
        select:[:each | each isDirectory not]
        thenCollect:[:each | each baseName].
!

pathParts
    |components|

    components := fileName components.
    (components startsWith:(Filename rootDirectory name)) ifTrue:[
        components := components copyFrom:2
    ].
    ^ components
!

readOnlyFileNamed:subFileName
    |stream squeakStream mime|

    stream := (fileName construct:subFileName) readStream.
    stream isNil ifTrue:[
        ^ nil
    ].
    squeakStream := SimulatedSqueakFileStream for:stream.
    mime := MIMETypes mimeTypeForFilename:subFileName.
    mime isImage ifTrue:[
        squeakStream binary.
    ].
    ^ squeakStream.
! !

!SimulatedSqueakFileDirectory methodsFor:'printing'!

printOn:aStream
    aStream 
        nextPutAll:self class name;
        nextPutAll:' on ';
        nextPutAll:fileName pathName
! !

!SimulatedSqueakFileDirectory methodsFor:'queries'!

slash
    ^self class slash
! !

!SimulatedSqueakFileDirectory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/SimulatedSqueakFileDirectory.st,v 1.16 2006/10/12 08:43:34 cg Exp $'
! !
