"{ Package: 'stx:goodies/webServer/comanche/htmlview' }"

"{ NameSpace: HV }"

Object subclass:#HVHtmlBuilder
	instanceVariableNames:'request view isPost fields elements variables forms elementStack
		maxCol currentCol'
	classVariableNames:''
	poolDictionaries:''
	category:'HV-base'
!

HVHtmlBuilder comment:'HVHtmlBuilder builds HTML pages and handles HTML Forms. It has a reference to the HVHttpView that created it (for various services) and to the request of that view.

Typical use is to instantiate it (HVHttpView has a method #builder which creates one), feed it "build messages" and then finally calling #render which will stream out the resulting HTML page. This is an example:

builder _ self builder.
builder start; h1: ''A heading''; postForm; inputText: #name; submit; endForm; end.
builder render

During the build some of the "build messages" return objects that can be used for input and output - typically objects representing Forms, Inputfields, Submit buttons etc. The messages #postForm, #inputText: and #submit all returned objects in the code above but the code chose to ignore them.

The contents message causes the builder to assemble the finished HTML document and return it as a String, but normally you just return the builder itself since HVHtmlBuilder also implements #asHttpResponseTo: which means that it can be returned as a result to Comanche from an ''url'' method in a HVHttpView class. So you do not need to call #render and actually, in order to compose pages from multiple methods it is better if you do not call #render.

...

elements - An OrderedCollection holding all elements that constitute the page.
elementStack - An OrderedCollection acting as a stack for certain elements like for example forms.

'
!


!HVHtmlBuilder class methodsFor:'instance creation'!

request: aRequest view: aView
	^self new request: aRequest view: aView; yourself
! !

!HVHtmlBuilder methodsFor:'accessing'!

request
	^request
!

view
	^view
! !

!HVHtmlBuilder methodsFor:'as yet unclassified'!

end
	^elements add: '</body></html>'
!

start
	"Start the HTML page by embedding two variables,
	head and body with normal default values."

	elements add: '<html><head>'.
	self var: #head value: '<title>', view title, '</title>'.
	elements add: '</head>'.
	self var: #bodytag value: '<body>'
!

tableHeads: heads
	"Add a table. Use #row, #cell, #endRow, #endCell, #endTable."

	elements add: '<table border="0"><head>'.
	heads do: [:head | elements add: '<th class="thgrey">', head, '</th>'].
	elements add: '</head>'
!

zebraTable: objects heads: heads block: aBlock
	"Add a table. Efficient implementation but the extraction
	block <aBlock> can not use the builder."

	| even row |
	elements add: '<table border="0"><head>'.
	heads do: [:head | elements add: '<th class="thgrey">', head, '</th>'].
	elements add: '</head>'.
	even _ false.
	objects do: [:obj |
		row _ aBlock value: obj.
		row ifNotNil: [
			even
				ifTrue: [ elements add: '<tr class="trgrey">']
				ifFalse: [ elements add: '<tr class="trlgrey">'].
			even _ even not.
			row do: [:val | elements add: '<td class="tdp">', val asString, '</td>' ].
			elements add: '</tr>'
		]
	].
	elements add: '</table>'
! !

!HVHtmlBuilder methodsFor:'comanche'!

asHttpResponseTo: req
	"We implement this so that a builder can be returned as a
	result from a method rather than a HTML String.
	This also enables nestling of builders in order to compose
	pages by calling other 'submethods'."

	^(HttpResponse fromString: self render)
		fieldAt: 'Pragma' put: 'no-cache';
		fieldAt: 'Cache-Control' put: 'no-cache'
		
! !

!HVHtmlBuilder methodsFor:'composition'!

head
	"Create and return a subbuilder for the head of the document.
	This is a variable reserved automatically in #start."

	| child |
	self var: #head value: (child _ HVHtmlBuilder request: request view: view).
	child title: view title.
	^child
!

subBuilder
	"Create and return a subbuilder."

	| child |
	self var: self nextVarName value: (child _ HVHtmlBuilder request: request view: view).
	child title: view title.
	^child
!

submit: var value: default

	| value |
	value _ fields includesKey: var.
	elements add: '<input type="submit" name="', var, '" value="', default, '" />'.
	^value
! !

!HVHtmlBuilder methodsFor:'conditionals'!

ifPost: aBlock
	"If it is a POST we run the block."

	^self isPost ifTrue:[aBlock value]
!

ifSubmit: aBlock
	"If it is a POST or a GET with fields we run the block."

	request isPostRequest ifTrue:[^aBlock value].
	request isGetRequest ifTrue:[request getFields isEmpty ifFalse:[^aBlock value]]
!

isPost
	"Is it a POST?"

	^request isPostRequest
! !

!HVHtmlBuilder methodsFor:'html'!

b: text
	text class == String ifTrue:[^elements add: '<b>', text, '</b>'].
	elements add: '<b>'.
	text value: self.
	elements add: '</b>'
!

blockquote
	elements add: '<blockquote>'
!

blockquote: text
	text class == String ifTrue:[^elements add: '<blockquote>', text, '</blockquote>'].
	elements add: '<blockquote>'.
	text value: self.
	elements add: '</blockquote>'
!

br
	^elements add: '<br />'
!

endBlockquote
	elements add: '</blockquote>'
!

getImage: url
	"Embed an image."

	^'<img src="', url, '" alt="no description" />'
!

getImage: url description: desc
	"Embed an image."

	^'<img src="', url, '" alt="', desc, '" />'
!

hr
	^elements add: '<hr />'
!

html: aString
	^elements add: aString
!

i: text
	text class == String ifTrue:[^elements add: '<i>', text, '</i>'].
	elements add: '<i>'.
	text value: self.
	elements add: '</i>'
!

image: url
	"Embed an image."

	^self image: url description: 'no description'
!

image: url description: desc
	"Embed an image."

	^elements add: '<img src="', url, '" alt="', desc, '" />'
!

nbsp
	^elements add: '&nbsp;'
!

ol: items
	"Make an ordered list by using asString on the elements."
	
	^self ol: items do: [:i | self html: i asString ]
!

ol: items do: aBlock
	"Make an ordered list but we let the block
	add the item to the builder itself."
	
	elements add: '<ol>'.
	items do: [:i |
		elements add: '<li>'.
		aBlock value: i.
		elements add: '</li>' ].
	elements add: '</ol>'
!

ol: items extract: aBlock
	"Make an ordered list with the value of the block
	as the item, use #ol:do: for fancier stuff."
	
	elements add: '<ol>'.
	items do: [:i | elements add: '<li>', (aBlock value: i), '</li>' ].
	elements add: '</ol>'
!

p: text
	text class == String ifTrue:[^elements add: '<p>', text, '</p>'].
	elements add: '<p>'.
	text value: self.
	elements add: '</p>'
!

pre: text
	text class == String ifTrue:[^elements add: '<pre>', text, '</pre>'].
	elements add: '<pre>'.
	text value: self.
	elements add: '</pre>'
!

tag: tag around: text
	text class == String ifTrue:[^elements add: '<',tag,'>', text, '<',tag,'>'].
	elements add: '<',tag,'>'.
	text value: self.
	elements add: '<',tag,'>'
!

ul: items
	"Make an unordered list by using asString on the elements."
	
	^self ul: items do: [:i | self html: i asString ]
!

ul: items do: aBlock
	"Make an unordered list but we let the block
	add the item to the builder itself."

	elements add: '<ul>'.
	items do: [:i |
		elements add: '<li>'.
		aBlock value: i.
		elements add: '</li>' ].
	elements add: '</ul>'
! !

!HVHtmlBuilder methodsFor:'html-alignment'!

colTab
	currentCol = maxCol
		ifTrue:[elements add: '</td></tr><tr><td>'.
				currentCol _ 1]
		ifFalse:[	elements add: '</td><td>'.
				currentCol _ currentCol + 1]
!

colTabTop
	currentCol = maxCol
		ifTrue:[elements add: '</td></tr><tr valign="top"><td>'.
				currentCol _ 1]
		ifFalse:[	elements add: '</td><td>'.
				currentCol _ currentCol + 1]
!

columns: cols
	elements add: '<table border="0" cellpadding="0" cellspacing="3"><tr><td>'.
	maxCol _ cols.
	currentCol _ 1
!

columns: cols heads: heads
	elements add: '<table border="0" cellpadding="0" cellspacing="3">'.
	self heads: heads.
	elements add: '<tr><td>'.
	maxCol _ cols.
	currentCol _ 1
!

columnsWide: cols
	elements add: '<table border="0" cellpadding="0" cellspacing="3" width="100%"><tr><td>'.
	maxCol _ cols.
	currentCol _ 1
!

columnsWideTop: cols
	elements add: '<table border="0" cellpadding="0" cellspacing="3" width="100%"><tr  valign="top"><td>'.
	maxCol _ cols.
	currentCol _ 1
!

endColumns
	elements add: '</td></tr></table>'
! !

!HVHtmlBuilder methodsFor:'html-forms'!

addForm: aForm
	"Add the form both to the list of forms in this builder,
	the element stack and to the list of elements to render.
	The form is also given an id for this page."

	forms addLast: aForm.
	aForm id: forms size.
	elementStack addLast: aForm.
	^elements add: aForm

!

addHtmlInput: aVar
	"Add the input variable in the correct Form and
	also add it to the collection of elements."

	self currentForm add: aVar.
	^elements add: aVar

!

currentForm
	"Find current form by looking in the stack.
	We could probably cache currentForm..."

	elementStack reverseDo: [:e | e isForm ifTrue:[^e]].
	^nil

!

endDiv
	^self endTag
!

endForm
	^self endTag
!

getForm
	"Start a GET form that will return to the same url
	as this request."

	^self addForm: (HVHtmlForm getAction: request url)
!

inputCheckbox

	^self inputCheckbox: self nextVarName value: false

!

inputCheckbox: name

	^self inputCheckbox: name value: false

!

inputCheckbox: var value: default

	| val |
	val _ default.
	self ifSubmit: [
		val _ (fields includesKey: var)
			ifTrue:[ (fields at: var) = 'on' ]
			ifFalse: [ false ]].
	^self addHtmlInput: (HVHtmlInputCheckbox name: var value: val)
!

inputCheckboxValue: default

	^self inputCheckbox: self nextVarName value: default

!

inputFile

	^self inputFile: self nextVarName value: false

!

inputFile: name

	^self inputFile: name value: false

!

inputFile: name value: default 

	^self addHtmlInput: (HVHtmlInputFile name: name value: (fields at: name ifAbsent:[default]))


!

inputFileValue: default

	^self inputFile: self nextVarName value: default

!

inputHidden

	^self inputHidden: self nextVarName value: ''
!

inputHidden: name

	^self inputHidden: name value: ''
!

inputHidden: var value: default 

	^self addHtmlInput: (HVHtmlInputHidden name: var value: (fields at: var ifAbsent:[default]))

!

inputHiddenValue: default

	^self inputHidden: self nextVarName value: default
!

inputNumber

	^self inputNumber: self nextVarName value: ''

!

inputNumber: name

	^self inputNumber: name value: ''

!

inputNumber: name size: size

	^self inputNumber: name size: size value: ''
!

inputNumber: name size: size value: default 

	^self addHtmlInput: (HVHtmlInputNumber name: name value: (fields at: name ifAbsent:[default ifNil: [0]]) size: size)

!

inputNumber: name value: value

	^self inputNumber: name size: 20 value: value

!

inputNumberSize: size

	^self inputNumber: self nextVarName size: size value: ''
!

inputNumberSize: size value: default

	^self inputNumber: self nextVarName size: size value: default
!

inputNumberValue: default

	^self inputNumber: self nextVarName size: 20 value: default
!

inputPassword

	^self inputPassword: self nextVarName value: ''

!

inputPassword: name

	^self inputPassword: name value: ''

!

inputPassword: name size: size

	^self inputPassword: name size: size value: ''
!

inputPassword: name size: size value: default 

	^self addHtmlInput: (HVHtmlInputPassword name: name value: (fields at: name ifAbsent:[default]) size: size)


!

inputPassword: name value: default

	^self inputPassword: name size: 30 value: default
!

inputPasswordSize: size

	^self inputPassword: self nextVarName size: size value: ''
!

inputText

	^self inputText: self nextVarName value: ''

!

inputText: var

	^self inputText: var value: ''

!

inputText: var size: size

	^self inputText: var size: size value: ''
!

inputText: name size: size value: default 

	^self addHtmlInput: (HVHtmlInputText name: name value: (fields at: name ifAbsent:[default]) size: size)


!

inputText: var value: default

	^self inputText: var size: 30 value: default
!

inputTextSize: size

	^self inputText: self nextVarName size: size value: ''
!

inputTextSize: size value: default

	^self inputText: self nextVarName size: size value: default
!

inputTextValue: default

	^self inputText: self nextVarName value: default

!

postForm
	"Start a POST form with an action url that is
	the same as this request."

	^self addForm: (HVHtmlForm postAction: request url)
!

postFormLocal: aString
	"Start a POST form with an action url based
	on the local view."

	^self addForm: (HVHtmlForm postAction: view prefix, aString)
!

postFormMultiPart
	"Start a POST form with an action url that is
	the same as this request and with encoding as multi part."

	^self addForm: (HVHtmlForm postAction: request url enctype: 'multipart/form-data')
!

postFormMultiPartLocal: aString
	"Start a POST form with an action url that is
	based on the view and with encoding as multi part."

	^self addForm: (HVHtmlForm postAction: view prefix, aString enctype: 'multipart/form-data')
!

reset

	^self reset: 'Reset'
!

reset: text

	^elements add: '<input type="reset" value="', text, '" />'
!

submit

	^self submit: 'Submit'
!

submit: text

	^self submit: text value: text
!

textArea

	^self textArea: self nextVarName rows: 5 cols: 40 value: ''
!

textArea: name

	^self textArea: name rows: 5 cols: 40 value: ''
!

textArea: var rows: rows cols: cols

	^self textArea: var rows: rows cols: cols value: ''
!

textArea: var value: default

	^self textArea: var rows: 4 cols: 20 value: default
!

textAreaRows: rows cols: cols

	^self textArea: self nextVarName rows: rows cols: cols value: ''
!

textAreaRows: rows cols: cols value: default

	^self textArea: self nextVarName rows: rows cols: cols value: default
! !

!HVHtmlBuilder methodsFor:'html-headings'!

divClass: aString
	"Start a div with a class."

	| div |
	elementStack addLast: (div _ HVHtmlDiv class: aString).
	^elements add: div
!

divId: anId
	"Start a div with an id."

	| div |
	elementStack addLast: (div _ HVHtmlDiv id: anId).
	^elements add: div
!

green: text
	text class == String ifTrue:[^elements add:'<font color="green">', text, '</font>'].
	elements add: '<font color="green">'.
	text value: self.
	elements add: '</font>'
!

h1: text
	text class == String ifTrue:[^elements add: '<h1>', text, '</h1>'].
	elements add: '<h1>'.
	text value: self.
	elements add: '</h1>'
!

h2: text
	text class == String ifTrue:[^elements add: '<h2>', text, '</h2>'].
	elements add: '<h2>'.
	text numArgs = 0
		ifTrue:[ text value ]
		ifFalse:[ text value: self].
	elements add: '</h2>'
!

h3: text
	text class == String ifTrue:[^elements add: '<h3>', text, '</h3>'].
	elements add: '<h3>'.
	text value: self.
	elements add: '</h3>'
!

red: text
	text class == String ifTrue:[^elements add:'<font color="red">', text, '</font>'].
	elements add: '<font color="red">'.
	text value: self.
	elements add: '</font>'
!

title: text
	text class == String ifTrue:[^elements add: '<title>', text, '</title>'].
	elements add: '<title>'.
	text value: self.
	elements add: '</title>'
! !

!HVHtmlBuilder methodsFor:'html-links'!

getLink: url
	"Create a HTML link."

	^self getLink: url text: url
!

getLink: url text: text
	"Create a HTML link."

	^'<a href="', url, '">', text, '</a>'
!

getLinkLocal: relativeUrl text: text
	"Return a local HTML link to the view."

	^self getLink: view prefix, relativeUrl text: text
!

link: url text: text
	"Embed a HTML link."

	^elements add: '<a href="', url, '">', text, '</a>'
!

linkBack
	"Return a local backlink."

	^self linkBack: 'Back'
!

linkBack: text
	"Return a local backlink."

	^self link: view prefix text: text
!

linkBackParent
	"Return a parent backlink."

	^self link: view parent prefix text: 'Back'
!

linkBackParent: text
	"Return a parent backlink."

	^self link: view parent prefix text: text
!

linkLocal: relativeUrl text: text
	"Make a local HTML link to the view."

	^self link: view prefix, relativeUrl text: text
!

linkPrefix: prefix text: text
	"Embed a link to the view with that prefix."

	| target |
	target _ view.
	[target prefix = prefix ifTrue:[ ^self link: target prefix text: text ].
	target _ target parent.
	target notNil] whileTrue.
	^self link: prefix text: 'No view found for this link'
!

srcLink
	| url slash |
	(request url = view prefix)
		ifTrue:[ url _ request url, '/srcdefault' ]
		ifFalse:[	slash _ request url findLast: [:c | c = $/ ].
			url _ (request url copyFrom: 1 to: slash), 'src', (request url copyFrom: slash + 1 to: request url size)].
	^self html: '<font size=-1>'; link: url text: 'See source for this page'
! !

!HVHtmlBuilder methodsFor:'html-tables'!

cell
	elements add: '<td class="tdp">'
!

cell: text
	text class == BlockContext ifFalse:[^elements add: '<td>', text, '</td>'].
	elements add: '<td>'.
	text value: self.
	elements add: '</td>'
!

cellCenter
	elements add: '<td class="tdpcenter">'
!

data
	elements add: '<td class="tdp">'
!

endCell
	elements add: '</td>'
!

endData
	elements add: '</td>'
!

endP
	elements add: '</p>'
!

endRow
	elements add: '</tr>'
!

endTable
	elements add: '</table>'
!

greyRow
	elements add: '<tr class="trgrey">'
!

heads: heads
	"Add heads to a table that has already been started."

	elements add: '<head>'.
	heads do: [:head | elements add: '<th align="left">', head, '</th>'].
	elements add: '</head>'
!

lightGreyRow
	elements add: '<tr class="trlgrey">'
!

p
	elements add: '<p>'
!

row
	elements add: '<tr>'
!

tab
	elements add: '</td><td class="tdp">'
!

table
	elements add: '<table border="0">'
!

table: objects heads: heads block: aBlock
	"Add a table. Efficient implementation but the extraction
	block <aBlock> can not use the builder."

	elements add: '<table border="1">'.
	self heads: heads.
	objects do: [:obj |
		elements add: '<tr>'.
		(aBlock value: obj) do: [:val |
			elements add: '<td>', val asString, '</td>' ].
		elements add: '</tr>'.
	].
	elements add: '</table>'
!

table: objects heads: heads cellBlock: aBlock
	"Add a table. This version takes a 2-argument (object and builder)
	block which feeds the cell values into the builder using #cell:."

	elements add: '<table border="1"><head>'.
	heads do: [:head | elements add: '<th>', head, '</th>'].
	elements add: '</head>'.
	objects do: [:obj |
		elements add: '<tr>'.
		aBlock value: obj value: self.
		elements add: '</tr>'.
	].
	elements add: '</table>'
! !

!HVHtmlBuilder methodsFor:'initialize-release'!

request: aRequest view: aView

	view _ aView.
	request _ aRequest.
	isPost _ request isPostRequest.
	isPost ifTrue: [fields _ request postFields] ifFalse: [fields _ request getFields].
	elements _ OrderedCollection new: 1000.
	elementStack _ OrderedCollection new.
	forms _ OrderedCollection new.
	variables _ Dictionary new
! !

!HVHtmlBuilder methodsFor:'private'!

asString
	^self render
!

at: var ifAbsent: block
	"Low level access to fields - parameters to GET or POST."

	^fields at: var ifAbsent: [^block value]
!

booleanAt: var ifAbsent: block
	"Customized version for checkboxes."

	^(fields at: var ifAbsent: [^block value]) = 'on'
!

numberAt: var ifAbsent: block
	"Customized version to get an expected number."

	^(fields at: var ifAbsent: [^block value]) asNumber
!

render
	"Concatenate everything together and substituting variables
	in the progress. First we check if we are timing this page.
	If we are then we need to calculate the stop time and incorporate
	the time at the bottom of the page."

	| start stream |
	start _ request propertyAt: #startTime ifAbsent: [nil].
	start ifNotNil: [elements removeLast ].
	stream _ WriteStream on: (String new: 50000).
	elements do: [:element |
		(element isKindOf: String)
			ifTrue:[ stream nextPutAll: element ]
			ifFalse:[ element renderFullOn: stream ]].
	start ifNotNil: [
		stream nextPutAll: '<br><font size=-1>Time in ms: ', (Time millisecondClockValue - start) asString, '</font>', String crlf.
		^stream contents
	].
	^stream contents

!

select: aName objects: objects nilLabel: label value: default

	^self select: aName objects: objects size: 1 nilLabel: label value: default
!

select: aName objects: objects size: size nilLabel: label value: default
	"Selects always have values that are integers from 1.
	If there is a value submitted we find the object at that
	position, otherwise we use the supplied default value."

	^self select: aName objects: objects values: nil size: size nilLabel: label selector: nil value: default
!

select: aName objects: objects size: size value: default
	^self select: aName objects: objects values: nil size: size nilLabel: #None selector: nil value: default
!

select: aName objects: objects value: default

	^self select: aName objects: objects size: 1 value: default
!

select: aName objects: objects values: values size: size nilLabel: label selector: sel value: default
	"Selects always have values that are integers from 1.
	If there is a value submitted we find the object at that
	position, otherwise we use the supplied default value."

	| val submitted v |
	submitted _ fields at: aName ifAbsent:[nil].
	submitted
		ifNotNil: [
			values
				ifNil: [
					default isCollection
						ifTrue:[val _ OrderedCollection new.
							(submitted findTokens: Character space)
								do: [:subm |
									v _ objects at: subm asNumber ifAbsent: [nil].
									v ifNotNil: [val add: v]]]
						ifFalse:[val _ objects at: submitted asNumber ifAbsent: [nil]]]
				ifNotNil: [
					default isCollection
						ifTrue: [val _ submitted findTokens: Character space]
						ifFalse: [val _ submitted].
					(val = 'nil') ifTrue:[val _ nil].
					]]
		ifNil: [val _ default].
	^self addHtmlInput: (HVHtmlSelect name: aName objects: objects values: values size: size nilLabel: label selector: sel value: val)
!

selectObjects: objects

	^self select: self nextVarName objects: objects size: 1 value: nil
!

selectObjects: objects value: default

	^self select: self nextVarName objects: objects size: 1 value: default
!

selectObjectsFirstSelected: objects

	^self select: self nextVarName objects: objects size: 1 value: (objects isEmpty ifTrue:[nil] ifFalse:[objects first])
!

selectObjectsFirstSelected: objects selector: sel

	^self select: self nextVarName objects: objects values: nil size: 1 nilLabel: #None selector: sel value: (objects isEmpty ifTrue:[nil] ifFalse:[objects first])
!

selectObjectsFirstSelected: objects values: values

	^self select: self nextVarName objects: objects values: values size: 1 nilLabel: #None selector: nil value: (objects isEmpty ifTrue:[nil] ifFalse:[values first])
!

selectObjectsNilFirst: objects

	^self select: self nextVarName objects: (objects copy addFirst: nil; yourself) size: 1 value: nil
!

selectObjectsNilFirst: objects nilLabel: label

	^self select: self nextVarName objects: (objects copy addFirst: nil; yourself) size: 1 nilLabel: label value: nil
!

selectObjectsNilFirst: objects nilLabel: label selector: sel value: default

	^self select: self nextVarName objects: (objects asOrderedCollection addFirst: nil; yourself) values: nil size: 1 nilLabel: label selector: sel value: default
!

selectObjectsNilFirst: objects nilLabel: label value: default

	^self select: self nextVarName objects: (objects asOrderedCollection addFirst: nil; yourself) size: 1 nilLabel: label value: default
!

selectObjectsNilFirst: objects value: default

	^self select: self nextVarName objects: (objects asOrderedCollection addFirst: nil; yourself) size: 1 value: default
!

selectObjectsNilFirst: objects values: values nilLabel: label

	^self select: self nextVarName objects: (objects copy addFirst: nil; yourself) values: (values copy addFirst: nil; yourself) size: 1 nilLabel: label selector: nil value: nil
!

submitted: aString
	"Low level access - is a certain field available?"

	fields at: aString ifAbsent: [^false].
	^true
!

textArea: name rows: rows cols: cols value: default

	^self addHtmlInput: (HVHtmlTextArea name: name rows: rows cols: cols value: (fields at: name ifAbsent:[default]))
! !

!HVHtmlBuilder methodsFor:'variables'!

addVar: aVar
	"Add the variable in the correct Form and
	add it to the collection of elements."

	self currentForm add: aVar.
	^elements add: aVar

!

bodytag
	"This is a way to be able to customize the body.
	Perhaps there are much better ways but this was simple."
	
	^variables at: #bodytag ifAbsent: [nil]

!

endTag
	"We assume that the last element on the stack is the correct one."

	^elements add: (HVHtmlEndtag for: elementStack removeLast)
!

findVar: aName
	"Find a named variable."

	^variables at: aName ifAbsent: [nil]
!

nextVarName
	"The current Form enumerates it's own variables."

	^self currentForm nextVarName
!

toc
	"Return an empty table of contents."

	^self tocValue: nil
!

toc: aName value: aValue
	"Return a table of contents with a given name and value."

	| var |
	var _ HVTableOfContents name: aName value: aValue builder: self.
	variables at: aName put: var.
	^elements add: var

!

tocValue: aValue
	"Return a table of contents with a given value."

	^self toc: variables size fullPrintString value: aValue

!

var
	"Return a variable node. The name will be a single number.
	A variable node can be given content using #value:
	later on after the page has been built but before it is rendered."

	^self var: (variables size fullPrintString) value: ''
!

var: aName
	"Return a variable node.
	A variable node can be given content using #value:
	later on after the page has been built."

	^self var: aName value: ''
!

var: aName value: aValue
	"Return a variable node.
	A variable node can be given content using #value:
	later on after the page has been built."

	| var |
	var _ HVSimpleVariable name: aName value: aValue.
	variables at: aName put: var.
	^elements add: var

! !

!HVHtmlBuilder class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/htmlview/HVHtmlBuilder.st,v 1.1 2003/05/19 18:49:50 cg Exp $'
! !
