"{ Package: 'stx:goodies/webServer/comanche/swiki' }"

"{ NameSpace: Comanche }"

SwikiEntry subclass:#SwikiDirectory
	instanceVariableNames:'dir dirServeCache dirRefsCache fileServeCache fileRefsCache'
	classVariableNames:''
	poolDictionaries:''
	category:'Comanche-Swiki-FileServer'
!

SwikiDirectory comment:'SwikiDirectory

is a cacheing file directory that allows you to reference and serve files in it.

INSTANCE VARIABLES:
dir = the corresponding FileDirectory
dirServeCache = a dictionary mapping relative HTTP paths to the corresponding subdirectories
dirRefsCache = a dictionary mapping directory names to the corresponding directories
fileServeCache = a dictionary mapping relative HTTP paths to the corresponding files
fileRefsCache = a dictionary mapping file names to the corresponding SwikiFileVersions'
!


!SwikiDirectory class methodsFor:'instance creation'!

fromArray: arr onDir: dir
	| instance |
	instance _ self fromArray: arr.
	instance dir: dir.
	instance initialize.
	^instance
!

onDir: dir
	^self fromArray: (dir containingDirectory entryAt: (dir pathParts last)) onDir: dir
! !

!SwikiDirectory methodsFor:'accessing'!

dir
	^dir
!

dir: aFileDirectory
	dir _ aFileDirectory
!

directoriesAlphabetically
	^dirRefsCache values asSortedCollection: [:a :b | (a name asLowercase) < (b name asLowercase)]
!

fileVersionsAt: key
	^fileRefsCache at: key ifAbsent: [nil]
!

filesAlphabetically
	^fileRefsCache values asSortedCollection: [:a :b | (a name asLowercase) < (b name asLowercase)]
!

httpPath
	^''
!

localHttpPath
	^''
!

size
	^(fileRefsCache size) + (dirRefsCache size)
! !

!SwikiDirectory methodsFor:'cacheing'!

addToDirRefsCache: aSubSwikiDir at: key
	dirRefsCache at: key put: aSubSwikiDir
	
!

addToDirServeCache: aSubSwikiDir at: key
	dirServeCache at: key put: aSubSwikiDir
!

addToFileRefsCache: entry at: fullName
        | versions |
        (versions _ fileRefsCache at: fullName ifAbsent: [nil]) ifNil: [
                versions _ SwikiFileVersions named: fullName.
                fileRefsCache at: fullName put: versions].
        versions addEntry: entry
!

addToFileServeCache: entry at: key
	fileServeCache at: key put: entry
!

directoryNamed: dirName
	| subSwikiDir |
	^self directoryNamed: dirName ifAbsent: ["Create One"
		dir createDirectory: dirName.
		subSwikiDir _ SwikiSubDirectory fromArray: (dir entryAt: dirName) onDir: (dir directoryNamed: dirName) containingDirectory: self.
		self addToDirRefsCache: subSwikiDir at: subSwikiDir name.
		subSwikiDir]
!

directoryNamed: dirName ifAbsent: aBlock
	^dirRefsCache at: dirName ifAbsent: aBlock
!

fileRefsCacheAt: key ifAbsent: block
	^fileRefsCache at: key ifAbsent: block
!

fileServeCacheAt: key ifAbsent: block
	^fileServeCache at: key ifAbsent: block
! !

!SwikiDirectory methodsFor:'file operations'!

addAliasFrom: key to: value
	| aliasFile |
	aliasFile _ dir fileNamed: '.aliases'.
	aliasFile setToEnd.
	aliasFile
		nextPutAll: key;
		nextPut: $	;
		nextPutAll: value;
		nextPutAll: String crlf;
		close
!

aliasDict
	| return key value |
	return _ Dictionary new.
	(dir fileExists: '.aliases') ifFalse: [^return].
	((dir readOnlyFileNamed: '.aliases') contentsOfEntireFile findTokens: String crlf)
		do: [:lineEntry | (lineEntry includes: $	) ifTrue: [
			key _ lineEntry copyUpTo: $	.
			value _ lineEntry copyFrom: (key size + 2) to: (lineEntry size).
			return at: key put: value]].
	^return
!

fileEntries
        ^(dir entries select: [:arr | (arr at: 4) not and: [(arr at: 1) ~= '.aliases']]) collect: [:arr | SwikiFile fromArray: arr onDir: dir]
!

moveFileNamed: fullPath toFileName: fileName
        | rootName actualFileName versions entry fromFile toFile |
        rootName _ self class rootNameFor: fileName.
        (versions _ fileRefsCache at: rootName ifAbsent: [nil])
                ifNil: [actualFileName _ dir checkName: rootName fixErrors: true.
                        (rootName = actualFileName) ifFalse: ["Add alias"
                                self addAliasFrom: actualFileName to: rootName]]
                ifNotNil: [actualFileName _ versions nextVersionName].
        "Move file to new location"
        [dir rename: fullPath toBe: actualFileName] ifError: [:err :rcvr |
                "Copy and delete instead"
                fromFile _ FileStream readOnlyFileNamed: fullPath.
                toFile _ dir newFileNamed: actualFileName.
                dir copyFile: fromFile toFile: toFile.
                fromFile close.
                toFile close.
                (Smalltalk dialectName = 'SmalltalkX') ifTrue:[
                    SimulatedSqueakFileDirectory deleteFilePath: fullPath
                ] ifFalse:[
                    FileDirectory deleteFilePath: fullPath    
                ]].
        "Get and Add Entry"
        entry _ SwikiFile fromArray: (dir entryAt: actualFileName) onDir: dir.
        self addToFileRefsCache: entry at: rootName.
        self addToFileServeCache: entry at: entry httpName.
        "Return Versions"
        ^fileRefsCache at: rootName
!

readOnlyFile: aSwikiFile
        ^ dir readOnlyFileNamed: aSwikiFile name.
! !

!SwikiDirectory methodsFor:'initialization'!

initialize
	"Fill Directories and Files"
	self initializeDirectories.
	self initializeFiles
!

initializeDirectories
	| subSwikiDir |
	"Initialize Caches"
	dirServeCache _ Dictionary new.
	dirRefsCache _ Dictionary new.
	"Add subSwikiDirs"
	(dir entries select: [:entry | entry at: 4]) do: [:arr |
		subSwikiDir _ SwikiSubDirectory fromArray: arr onDir: (dir directoryNamed: (arr at: 1)) containingDirectory: self.
		"dirRefsCache"
		self addToDirRefsCache: subSwikiDir at: subSwikiDir name]
!

initializeFiles
	| aliasDict rootName |
	"Initialize Caches"
	fileServeCache _ Dictionary new.
	fileRefsCache _ Dictionary new.
	"Add Files"
	aliasDict _ self aliasDict.
	self fileEntries do: [:entry | rootName _ entry rootName.
		(entry name = rootName)
			ifFalse: [rootName _ aliasDict at: entry name ifAbsent: [rootName]].
		self addToFileRefsCache: entry at: (aliasDict at: rootName ifAbsent: [rootName]).
		self addToFileServeCache: entry at: entry httpName]
! !

!SwikiDirectory methodsFor:'referencing'!

referenceDirFs: fs options: dict
	^fs referenceDirectory: self options: dict
!

referenceFileNamed: aFileName fs: fs options: dict
        | versionEntry |
        ^(versionEntry _ fileRefsCache at: aFileName ifAbsent: [nil])
                ifNil: [fs referenceMissingFileNamed: aFileName onDir: self options: dict]
                ifNotNil: [fs referenceFile: versionEntry onDir: self options: dict]
! !

!SwikiDirectory methodsFor:'serving'!

serveDirFs: fs options: dict
	^fs serveDirectory: self options: dict
!

serveFilePath: aFilePath fs: fs options: dict
	| fileEntry |
	^(fileEntry _ fileServeCache at: aFilePath ifAbsent: [nil])
		ifNil: [fs serveMissingFilePath: aFilePath onDir: self options: dict]
		ifNotNil: [fs serveFile: fileEntry onDir: self options: dict]
!

servePath: aPath fs: fs options: dict
	| dirPath filePath dirEntry |
	(aPath includes: $/) ifFalse: [^(aPath = '')
		ifTrue: [self serveDirFs: fs options: dict]
		ifFalse: [self serveFilePath: aPath fs: fs options: dict]].
	(aPath last = $/) ifTrue: [^(dirEntry _ dirServeCache at: aPath ifAbsent: [nil])
		ifNil: [filePath _ (aPath copyUpTo: '/'), '/'.
			(dirEntry _ dirServeCache at: filePath ifAbsent: [nil])
				ifNil: [fs serveMissingFilePath: aPath onDir: self options: dict]
				ifNotNil: [dirEntry servePath: (aPath copyFrom: (filePath size + 1) to: (aPath size)) fs: fs options: dict]]
		ifNotNil: [dirEntry serveDirFs: fs options: dict]].
	dirPath _ (aPath copyUpToLast: $/), '/'.
	(dirEntry _ dirServeCache at: dirPath ifAbsent: [nil]) ifNotNil: [
		^dirEntry serveFilePath: (aPath copyAfterLast: $/) fs: fs options: dict].
	filePath _ (aPath copyUpTo: '/'), '/'.
	^(dirEntry _ dirServeCache at: filePath ifAbsent: [nil])
		ifNil: [fs serveMissingFilePath: aPath onDir: self options: dict]
		ifNotNil: [dirEntry servePath: (aPath copyFrom: (filePath size + 1) to: (aPath size)) fs: fs options: dict]
! !

!SwikiDirectory methodsFor:'testing'!

directoryExists: dirName
	^dirRefsCache includesKey: dirName
!

fileExists: fileName
	^dir fileExists: fileName
!

fileRefsExists: fileName
	^fileRefsCache includesKey: fileName
!

fileServeExists: fileName
	^fileServeCache includesKey: fileName
!

isASubDirectory
	^false
! !

!SwikiDirectory class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/webServer/comanche/swiki/SwikiDirectory.st,v 1.9 2003/08/29 17:08:46 cg Exp $'
! !
