"{ Package: 'stx:goodies/xml/sixx' }"

Object subclass:#SixxReadStream
	instanceVariableNames:'stream sixxElements contextDictionary position limit
		cachedObjects'
	classVariableNames:''
	poolDictionaries:''
	category:'SIXX-Core'
!

!SixxReadStream class methodsFor:'documentation'!

documentation
"
    I represent a ReadStream for SIXX format.

    I can be used as follows:

        srs := SixxReadStream readOnlyFileNamed: ('obj.sixx').
        obj := srs next.
        srs close.
        obj inspect.

        srs := SixxReadStream readOnlyFileNamed: ('objs.sixx').
        objs := srs contents.
        srs close.
        objs inspect.

    ---
    [:masashi | ^umezawa]
"
! !

!SixxReadStream class methodsFor:'instance creation'!

new
	self error: 'Streams are created with on:'
!

on: readStream 
	^ self basicNew on: readStream
!

readOnlyFileNamed: fileName 
	^ self basicNew readOnlyFileNamed: fileName
! !

!SixxReadStream methodsFor:'accessing'!

cachedObjects
	^ cachedObjects
!

contextDictionary
	^ contextDictionary
!

limit
	^ limit
!

sixxElements
	^ sixxElements
!

stream
	^ stream
! !

!SixxReadStream methodsFor:'file open/close'!

close
	self stream close
! !

!SixxReadStream methodsFor:'initializing'!

initialize
	contextDictionary := IdentityDictionary new.
	position := 0.
	sixxElements := self parseStream: stream.
	limit := sixxElements isNil
				ifTrue: [0]
				ifFalse: [sixxElements size].
	cachedObjects := Dictionary new
!

on: aStream 
	stream := aStream.
	self initialize
!

readOnlyFileNamed: fileName 
	stream := SixxPortableUtil readFileStreamNamed: fileName.
	self initialize
! !

!SixxReadStream methodsFor:'private'!

checkSixxElement: sixxElement 
	(SixxXmlUtil nameFrom: sixxElement)
			= 'sixx'
		ifFalse: [self error: 'unexpected element: '
					, (SixxXmlUtil nameFrom: sixxElement)]
!

parseStream: aStream 
        | root str recoverTried |
        (aStream isNil or: [aStream size = 0])  ifTrue: [^nil].
        str := aStream.
        recoverTried := false.
        [root := SixxXmlUtil parseXml: str]
                on: SixxXmlParseError 
                do: [:ex | 
                    (SixxSetting useRecoveryRead and: [recoverTried not])
                        ifTrue: 
                                [str := self recoveredStreamFrom: str.
                                recoverTried := true.
                                ex retry]
                        ifFalse: [ex pass]].
        self checkSixxElement: root.
        ^SixxXmlUtil elementsFrom: root
!

readSixxFromSixxElement: sixxElement 
	^ Object readSixxFromSixxElement: sixxElement context: self contextDictionary
! !

!SixxReadStream methodsFor:'private-accessing'!

privatePosition: anInteger 
	
	position := anInteger
! !

!SixxReadStream methodsFor:'private-recovery'!

findStringIndexes: subString from: sourceString 
	| foundIndexes index |
	foundIndexes := OrderedCollection new.
	index := 1.
	[index = 0]
		whileFalse: [index := sourceString findString: subString startingAt: index.
			index = 0
				ifFalse: [foundIndexes add: index.
					index := index + 1]].
	^ foundIndexes
!

recoveredStreamFrom: aStream 
        | contents startIndexes endIndexes recoverStream |

        contents := aStream contents.

        startIndexes := self findStringIndexes: '<sixx.object' from: contents.
        endIndexes := self findStringIndexes: '</sixx.object>' from: contents.

        recoverStream := ReadWriteStream with: (contents copyFrom: 1 to: (startIndexes last -1)).
        (startIndexes size - endIndexes size -1) timesRepeat: [recoverStream nextPutAll: '</sixx.object>'].
        recoverStream nextPutAll: '</sixx>'.

        ^ recoverStream reset
! !

!SixxReadStream methodsFor:'reading'!

contents
	^(self next: self size) asArray
!

next
      self atEnd ifTrue:[^ nil].

      position := position + 1. 
      ^ self cachedObjects
            at: position
            ifAbsentPut: [self readSixxFromSixxElement: (sixxElements at: position)]
!

next: anInteger 
	| ord |
	anInteger < 0 ifTrue: [^nil].
	ord := OrderedCollection new: anInteger.
	anInteger
		timesRepeat: [self atEnd
				ifFalse: [ord add: self next]].
	^ ord
!

peek
	| nextObj |
	nextObj := self next.
	self privatePosition: self position - 1.
	^ nextObj
!

position
	^ position
!

position: anInteger 
        (anInteger < 0 or: [anInteger > limit])
                ifTrue: [self error: 'out of bounds'].
        self reset.
        self next: anInteger
!

reset
	self privatePosition: 0
!

size
	^ self limit
!

skip: anInteger
	self position: self position + anInteger.
! !

!SixxReadStream methodsFor:'testing'!

atEnd
	^ self position >= self limit
! !

!SixxReadStream class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/xml/sixx/SixxReadStream.st,v 1.4 2003/10/08 15:40:03 cg Exp $'
! !
