"{ Package: 'stx:goodies/xml/xmlrpc' }"

"{ NameSpace: XMLRPC }"

Object subclass:#XMLRPCProxy
	instanceVariableNames:'url'
	classVariableNames:''
	poolDictionaries:''
	category:'Net-Communication-XMLRPC-Client'
!

XMLRPCProxy comment:'Proxy class to issue XML-RPC requests.
  Example (invoking a method that takes a single integer argument):
     u _ Url absoluteFromText: ''http://bleu.west.spy.net/servlet/net.spy.rpc.XMLRPC''.
       proxy _ XMLRPCProxy withUrl: u.
        proxy invokeMethod: ''echo.echoInt'' withArgs: #(1).'
!

!XMLRPCProxy class methodsFor:'documentation'!

documentation
"
    Actually, should be called XMLRPXClient...

    Assume, that you have an XML-RPC service running on <url>;
    then,
        client := XMLRPCProxy withUrl:<url>
    returns a proxy object, which can be sent messages via:
        client invokeMethod:'nameOfOperation'
    or:
        client invokeMethod:'nameOfOperation' withArgs:<arrayOfArguments>
"
!

examples
"
    Talking to an arbitrary XML-RPC service which supports protocol-introspection:
                                                                                            [exBegin]
        |client|

        client := XMLRPCProxy withUrl:'http://localhost:8082/rpc/xmlrpc'.
        client invokeMethod:'system.listMethods'
                                                                                            [exEnd]
                                                                                            [exBegin]
        |client|

        client := XMLRPCProxy withUrl:'http://localhost:8082/rpc/xmlrpc'.
        client invokeMethod:'system.getCapabilities'
                                                                                            [exEnd]

    Talking to a JIRA XML-RPC service:
                                                                                            [exBegin]
        |client|

        client := XMLRPCProxy withUrl:'http://localhost:8081/rpc/xmlrpc'.
        client invokeMethod:'login' with:'cg' with:'cg'.
        client invokeMethod:'logout'.
                                                                                            [exEnd]

    Talking to an expeccoNET XML-RPC service:
                                                                                            [exBegin]
        |client|

        client := XMLRPCProxy withUrl:'http://localhost:8082/rpc/xmlrpc'.
        client invokeMethod:'expeccoNET1.login' with:'cg' with:'cg'.
        client invokeMethod:'expeccoNET1.logout'.
                                                                                            [exEnd]
    Talking to an expecco XML-RPC service:
                                                                                            [exBegin]
        |client|

        client := XMLRPCProxy withUrl:'http://localhost:9090/rpc/xmlrpc'.
        client invokeMethod:'expecco.loadProjectFromURL' with:'someURL'.
                                                                                            [exEnd]
"
! !

!XMLRPCProxy class methodsFor:'instance creation'!

withUrl:u 
    "Get a new instance of XMLRPCProxy to connect to a given URL."
    
    ^ self new url:u
! !

!XMLRPCProxy methodsFor:'accessing'!

url
    "Get the URL against which requests will be invoked."
    
    ^ url
! !

!XMLRPCProxy methodsFor:'initializing'!

url:aUrl 
    "Set the URL that will be servicing the requests."
    
    url := aUrl
! !

!XMLRPCProxy methodsFor:'invoking'!

doesNotUnderstand:aMessage
    "/ cg: I dont think this is a good idea - it makes debugging somewhat more
    "/ difficult (although it makes writing code less painful).

    self halt.
    ^ self invokeMethod:(aMessage selector) withArguments:(aMessage arguments).
!

invokeMethod:method 
    "Invoke a method on the XML-RPC server with no arguments."
    
    ^ self invokeMethod:method withArguments:nil
!

invokeMethod:method with:arg1 
    "Invoke a method on the XML-RPC server.
     arg is a parameter to be passed.  
     For example, if the remote method is called ``test.Method'' and takes one int, 
     you'd invoke is like this:
          anXMLRPCProxy invokeMethod: 'test.method' with: 815
    "
    
    ^ self invokeMethod:method withArguments:(Array with:arg1)
!

invokeMethod:method with:arg1 with:arg2
    "Invoke a method on the XML-RPC server.
     arg is a parameter to be passed.  
     For example, if the remote method is called ``test.Method'' and takes two ints, 
     you'd invoke is like this:
          anXMLRPCProxy invokeMethod: 'test.method' with: 815 with: 999
    "
    
    ^ self invokeMethod:method withArguments:(Array with:arg1 with:arg2)
!

invokeMethod:method with:arg1 with:arg2 with:arg3
    "Invoke a method on the XML-RPC server.
     arg is a parameter to be passed.  
     For example, if the remote method is called ``test.Method'' and takes two ints, 
     you'd invoke is like this:
          anXMLRPCProxy invokeMethod: 'test.method' with: 815 with: 999
    "
    
    ^ self invokeMethod:method withArguments:(Array with:arg1 with:arg2 with:arg3)
!

invokeMethod:method withArgs:args 
    "backward compatibility"
    
    ^ self invokeMethod:method withArguments:args
!

invokeMethod:method withArguments:args 
    "Invoke a method on the XML-RPC server.
     args is a list of parameters to be passed.  
     For example, if the remote method is called ``test.Method'' and takes two ints, 
     you'd invoke is like this:
          anXMLRPCProxy invokeMethod: 'test.method' withArgs: #(19 77)
    "
    
    |req res rv|

    req := self buildXmlRpc:method withArgs:args.
    res := self sendXmlRpc:req.
    rv := self processResponse:res.
    ^ rv
! !

!XMLRPCProxy methodsFor:'private'!

buildXmlRpc:method withArgs:args 
    "Build the XMLRPC data required to invoke the given method with the 
     given collection of arguments."
    
    ^ String streamContents:[:str |
        |e|

        str 
            nextPutAll:'<?xml version="1.0"?><methodCall><methodName>';
            nextPutAll:method;
            nextPutAll:'</methodName><params>'. 

        args notEmptyOrNil ifTrue:[
            e := XMLRPCEncoder new.
            args do:[:p | 
                str nextPutAll:'<param>'; nextPutAll:(e encode:p); nextPutAll:'</param>'
            ]
        ].
        str nextPutAll:'</params></methodCall>']
!

processFault:xmlFault 
    "Process an XML-RPC fault into an XMLRPCFaultException"
    
    |d decoder|

    decoder := XMLRPCDecoder new.
    d := decoder decode:xmlFault.

    "/ a fault happened on the server-side
    "/ let's hope, that the given fault string helps...
    XMLRPCFaultException 
        signal:(d at:'faultCode')
        withMessage:(d at:'faultString').
!

processResponse:xmlStringOrStream 
    "Process the results."
    
    |xmlStream xmldoc mr fr pr|

    xmlStream := xmlStringOrStream readStream.
    xmldoc := YAXO::XMLDOMParser parseDocumentFrom:xmlStream.
    mr := xmldoc elementAt:'methodResponse'.
    mr isNil ifTrue:[ XMLRPCException signal:'methodResponse not found in XML Stream.'. ^ nil ].
     "See if it's a fault response."
    fr := mr elementAt:'fault'.
    fr notNil ifTrue:[ self processFault:fr ].
     "If we made it this far, it's a valid response, look for the results."
    pr := mr elementAt:'params'.
    pr isNil ifTrue:[ XMLRPCException signal:'Params not found in XML-RPC response.'.  ^ nil ].
     "Decode, and return the response."
    ^ XMLRPCDecoder new decode:pr
!

sendXmlRpc:data 
    "Send the XML-RPC data and get the response. Return the raw XML."

    Smalltalk isSmalltalkX ifTrue:[
        |response|

        response := HTTPInterface post:url with:data.
        response responseCode = 200 ifFalse:[
            self error:('HTTP-error: %1 (%2)' bindWith:response responseCode with:(response response))
        ].
        ^ response data
    ] ifFalse:[
        "/ squeak

        |s req res xml|

        s := HTTPSocket 
                initHTTPSocket:url
                ifError:[:msg | 
                    XMLRPCException 
                        signal:'Error connecting ' , url asString , '(' , msg , ')'
                ].
        req := 'POST ' , url fullPath , ' HTTP/1.0' , String crlf , 'Host: ' 
                    , url authority , String crlf 
                    , 'Content-type: text/xml' , String crlf 
                    , 'Content-length:  ' , data size asString 
                    , String crlf 
                        , 'User-Agent:  Spy Internetworking XML-RPC client for Smalltalk' 
                        , String crlf , String crlf 
                    , data.
         "Send the command"
        s sendCommand:req.
         "Make sure the result was valid."
        s responseCode = 200 ifFalse:[
            'XXX Raise exception.'
        ].
         "Get the data up to the first double-newline (indicating end of headers)"
        res := s getResponseUpTo:String crlf , String crlf.
        xml := s getRestOfBuffer:(res at:3) contents.
        ^ xml
    ].
! !

!XMLRPCProxy class methodsFor:'documentation'!

version_CVS
    ^ '$Header: /cvs/stx/stx/goodies/xml/xmlrpc/XMLRPCProxy.st,v 1.11 2010/04/06 14:27:53 cg Exp $'
! !
