"{ Package: 'stx:liboracle8' }"

OCI8Variable subclass:#OCI8BindVariable
	instanceVariableNames:'name needsBind maxElementSize setup value'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8BindVariable primitiveDefinitions!
%{

/*
 * includes, defines, structure definitions
 * and typedefs come here.
 */

#include <oci.h>
#include <xa.h>
#include <stdio.h>

/*  internal/external datatype codes */
#define VARCHAR2_TYPE           SQLT_CHR
#define NUMBER_TYPE             SQLT_NUM
#define INTEGER_TYPE            SQLT_INT
#define FLOAT_TYPE              SQLT_FLT
#define STRING_TYPE             SQLT_STR
#define VARNUM_TYPE             SQLT_VNU
#define LONG_TYPE               SQLT_LNG
#define VARCHAR_TYPE            SQLT_VCS
#define ROWID_TYPE              SQLT_RID
#define DATE_TYPE               SQLT_DAT
#define VARRAW_TYPE             SQLT_VBI
#define RAW_TYPE                SQLT_BIN
#define LONG_RAW_TYPE           SQLT_LBI
#define UNSIGNED_INT_TYPE       SQLT_UIN
#define LONG_VARCHAR_TYPE       SQLT_LVC
#define LONG_VARRAW_TYPE        SQLT_LVB
#define CHAR_TYPE               SQLT_AFC
#define CHARZ_TYPE              SQLT_AVC
#define ROWID_DESC_TYPE         SQLT_RDD
#define NAMED_DATA_TYPE         SQLT_NTY
#define REF_TYPE                SQLT_REF
#define CHARACTER_LOB_TYPE      SQLT_CLOB
#define BINARY_LOB_TYPE         SQLT_BLOB
#define BINARY_FILE_TYPE        SQLT_FILE
#define OCI_STRING_TYPE         SQLT_VST
#define OCI_DATE_TYPE           SQLT_ODT
%}

! !

!OCI8BindVariable class methodsFor:'documentation'!

documentation
"
    Abstract superclass for bind variables. They are used to bind an variable in a statement
    to external buffers. Three subclasses handle the insert of values into the database:

        OCI8MultipleValueBindVariable
                Used, if an array should be insert into the database. More then one value
                is given for each bind variable.

        OCI8ReturnValueBindVariable
                Used for variables in a returning clause of a statement. The data is
                fetched via callback functions.

        OCI8SingleValueBindVariable
                Used, if one value for each bind variable should be insert into the database.

    Each bind variable represent one variable in the statement text.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8MultipleValueBindVariable
        OCI8ReturnValueBindVariable
        OCI8SingleValueBindVariable
        OCI8Variable

    [instance variables:]

        name <Symbol>
            holds the name of the bind variable. It must be exactly the same
            as the name of the placeholder in the statement.

        needsBind <Boolean>
            holds if the variable must be binded

        maxElementSize <Integer>
           holds the maximal size of a element in the value instance variables.

        setup <Boolean>
            holds if the variables needs a set up. All buffers of the receiver must be
            initialized then. This is set to true if a new values is added to the variable.
            Then all buffers must be reinitialized.

        value <Object>
            holds the value of the variable. This value is written into
            the data buffer. (Null values are handles via the indicator array).

    [class variables:]
        none
"
!

history
    "Created: / 30.8.2000 / 11:14:05 / ps"
! !

!OCI8BindVariable methodsFor:'accessing'!

isSetup
    "returns if the receiver is already set up.
     This means the external bytes are set up for the current value(s) of the
     receiver. If the value(s) of the receiver changes the setup state switch
     to false.

     <return: Boolean>
    "

    setup isNil ifTrue:[
        setup := false
    ].
    ^ setup
!

maxElementSize
"
returns the maximal size of the values.

<return: Integer>
"
    ^ maxElementSize
!

maxElementSize:anInteger
"
sets the maximal size of the values.

<return: self>
"

    maxElementSize := anInteger.
!

name
"
returns the name of the bind variable. It must be exactly the same
as the name of the placeholder in the statement.

<return: Symbol>
"

    ^ name
!

name:aSymbol
"
sets the name of the bind variable. It must be exactly the same
as the name of the placeholder in the statement.

<return: self>
"

    name := aSymbol.
!

needsBind
"
returns if the variables must be binded

<return: Boolean>
"
    needsBind isNil ifTrue:[
        needsBind := true
    ].
    ^ needsBind
!

needsBind:aBoolean
"
sets if the variables must be binded

<return: self>
"
    needsBind := aBoolean.
!

numberOfElements
"
returns the number of elements which are stored in the value of the
receiver.

<return: Integer>
"
    self subclassResponsibility
!

setup:aBoolean
"
sets if the variables needs a set up.
All buffers of the receiver must be initialized then. This is set to true
if a new values is added to the receiver. Then all buffers must
be reinitialized.

<return: self>
"

    setup := aBoolean.
!

value
"
returns the value of the receiver. This value is written into
the data buffer. (Null values are handles via the indicator array).

<return: Object>
"
    ^ value
! !

!OCI8BindVariable methodsFor:'binding'!

bindForStatement:anOCI8Statement
"
bind the receiver with buffers and OCI_DEFAULT mode 

<return: self>
"
    anOCI8Statement ociLib ociBindByName:anOCI8Statement handle 
                              bindHandle:self handle 
                                   cType:self externalCType
                                  buffer:dataBuffer
                              bufferSize:maxElementSize
                                    name:name
                      nullValueIndicator:indicatorArray
                      elementLengthArray:elementLengthArray
                         returnCodeArray:nil
                                    mode:#'OCI_DEFAULT'  
                          errorOCIHandle:anOCI8Statement errorHandle.
    needsBind := false.
! !

!OCI8BindVariable methodsFor:'constants'!

handleType
"
default handle type for the bind variable. If using another mode, the given oci session,
statement mechanismen cannot be used.

<return: Symbol>
"
    ^#'OCI_HTYPE_BIND'.

! !

!OCI8BindVariable methodsFor:'setup'!

setupForStatement:anOCI8Statement 
"
sets up all necassary buffers for the receiver and copies
the value to the external bytes buffer.

<return: self>
"   
    self isSetup ifFalse:[
        self setupExternalBytes.
        self setup:true.
        self needsBind ifTrue:[
            self bindForStatement:anOCI8Statement.
        ].
    ].
! !

!OCI8BindVariable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8BindVariable.st,v 1.29 2002/04/02 18:11:19 stefan Exp $'
! !
