"{ Package: 'stx:liboracle8' }"

OCI8Statement subclass:#OCI8QueryStatement
	instanceVariableNames:'columnDescriptors defined fetchingFinished ociDefineVariables
		pieceSize prefetchedRows dynamicDefineVariables objectClass'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Statements'
!

Object subclass:#OCIRowObject
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	privateIn:OCI8QueryStatement
!

!OCI8QueryStatement class methodsFor:'documentation'!

documentation
"
    All query statements are represented by this class.
    To execute a data manipulation statement call:
        OCI8Session>>executeStatement:from:
        OCI8Session>>executeStatement:from:into:
        OCI8Session>>executeStatement:from:into:numberOfRows:
        OCI8Session>>executeStatement:into:
        OCI8Session>>executeStatement:into:numberOfRows:

    When an query statement is executed an fetch will be done automatically. The number
    of rows to be returned can be set. If this is done the rest of the rows can be fetched later.
    For example:
    session executeStatement:selectStatement into:resultStream numberOfRows:5.
    session fetchDataForStatement:selectStatement into:resultStream numberOfRows:10.
    Only the number of rows available will be fetched. No error signal is raised if
    the numbers in numberOfRows:anInteger is higher than the number of rows returned.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8Statement
        OCI8DDStatement
        OCI8DMStatement

    [instance variables:]
        columnDescriptors  <OrderedCollection>
            if a statement is executed the column descriptors (which have a description of the
            selected columns) are available here.

        defined <Boolean>
            is true if the statement define variables are generated and setup. False if not.

        fetchingFinished <Boolean>
            is true if the statements fetch operation is finished. Prevents that any other 
            oci fetch operation could be call if the flag is true. Because that would raise an
            oci error.

        ociDefineVariables <OrderedCollection>
            the define variables for each column are hold here.

        pieceSize <Integer>
            holds how many bytes should be fetched for a piecewise operation. If the statement
            includes a long or long raw column, the rows are fetched pieces wise.
            The number of bytes to be fetched by each server round trip has an important impact
            onto the performace of statement execution. A higher number reduces the necessary
            server roundtrips. The default is 30000 bytes rows per roundtrip.

        prefetchedRows <Integer>
            holds how many rows should be prefetched for the statement by the oci lib.
            The number of prefetched rows has an important impact onto the performace
            of statement execution. A higher number reduces the necessary server roundtrips.
            The default is 100 rows per roundtrip.

    [class variables:]
            none
"
!

history
    "Created: / 15.3.2000 / 11:54:00 / ps"
! !

!OCI8QueryStatement class methodsFor:'examples'!

example_createTable
"
create the table querytest for the examples of this class and insert some values.

                                        [exBegin]
    |session dropTableStatement createTableStatement insertStatement data|

    session := OCI8Session default.
    dropTableStatement := session newStatementFor:'drop table querytest'.
    createTableStatement := session newStatementFor:'create table querytest (id number, name varchar2(100))'.
    insertStatement := session newStatementFor:'insert into querytest values(:id, :name)'.
    insertStatement integerBindVariable:#id.
    insertStatement stringBindVariable:#name.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    OCIErrorSignal handle:[:ex |
    ] do:[
            session executeStatement:dropTableStatement.
    ].
    OCIErrorSignal handle:[:ex |
    ] do:[
            session executeStatement:createTableStatement.
    ].

    data := OrderedCollection new.
    1 to: 10000 do:[:i | data add:(Array with:i with:('number ',i printString))].
    session executeStatement:insertStatement from:data.

    session disconnect.
                                        [exEnd]
"


!

example_selectStatement1
"
read from the test table an inspect the column descriptor of the statement
(they hold the description of the column you read), the number of rows and
the content received. All rows are fetched

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement into:resultStream.
    selectStatement columnDescriptors inspect.
    selectStatement processedRows inspect.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"


!

example_selectStatement2
"
read from the test table an inspect the column descriptor of the statement
(that is the description of the tables you read), the number of rows read and
the content received. Only 5 rows are fetched via the implicit fetch at execution
time of the statement.

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement into:resultStream numberOfRows:5.
    selectStatement columnDescriptors inspect.
    selectStatement processedRows inspect.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"


!

example_selectStatement2b
"
read from the test table an inspect the column descriptor of the statement
(that is the description of the tables you read), the number of rows read and
the content received. Only 5 rows are fetched via the implicit fetch at execution
time of the statement. The other 10 rows are fetched with an extra call.

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement into:resultStream numberOfRows:5.
    session fetchDataForStatement:selectStatement into:resultStream numberOfRows:10.

    selectStatement columnDescriptors inspect.
    selectStatement processedRows inspect.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement2c
"
read from the test table an inspect the content in stream resultStream1 and resultStream2.
All rows are fetched via the implicit fetch at execution
time of the statement, because numberOfRows is nil. The rextra fetch will return
an empty stream.

                                        [exBegin]
    |session selectStatement resultStream1 resultStream2|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest'.
    selectStatement prefetchedRows:1000.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream1 := WriteStream on:Array new.
    session executeStatement:selectStatement into:resultStream1 numberOfRows:nil.
    resultStream1 contents inspect.
    resultStream2 := WriteStream on:Array new.
    session fetchDataForStatement:selectStatement into:resultStream2 numberOfRows:100.
    resultStream2 contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement3a
"
read from the test table the row with id = 10 and inspect the content received.
All variables in the statement must be bound via

    OCI8Statement>>byteBindVariable:       (for variables which hold bytes e.g. ByteArray)                 or
    OCI8Statement>>numberBindVariable:     (for variables which hold numbers e.g. Integer)         or
    OCI8Statement>>stringBindVariable:     (for variables which hold strings e.g. String)

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest where id = :id'.
    selectStatement numberBindVariable:#id.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement from:#(10) into:resultStream.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement3b
"
read from the test table the rows with name like '1%9%' and inspect the content received.
All variables in the statement must be bound via

    OCI8Statement>>byteBindVariable:       (for variables which hold bytes e.g. ByteArray)                 or
    OCI8Statement>>numberBindVariable:     (for variables which hold numbers e.g. Integer)         or
    OCI8Statement>>stringBindVariable:     (for variables which hold strings e.g. String)

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest where name like :name'.
    selectStatement stringBindVariable:#name.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement from:#('number 1%9%') into:resultStream.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement3c
"
read from the test table the rows with id like '%9%' and name like '1%9%' and inspect the content received.
All variables in the statement must be bound via

    OCI8Statement>>byteBindVariable:       (for variables which hold bytes e.g. ByteArray)                 or
    OCI8Statement>>numberBindVariable:     (for variables which hold numbers e.g. Integer)         or
    OCI8Statement>>stringBindVariable:     (for variables which hold strings e.g. String)

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select id from querytest where id like :id and name like :name'.
    selectStatement stringBindVariable:#name.
    selectStatement stringBindVariable:#id.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement from:#('%9%' 'number 1%9%') into:resultStream.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement3d
"
first read from the test table the rows with id like '%9%' and name like '1%9%'.
Second append to the same result stream the rows with id like '%8%' and name like '1%8%'
Inspect the content received.
All variables in the statement must be bound via

    OCI8Statement>>byteBindVariable:       (for variables which hold bytes e.g. ByteArray)                 or
    OCI8Statement>>numberBindVariable:     (for variables which hold numbers e.g. Integer)         or
    OCI8Statement>>stringBindVariable:     (for variables which hold strings e.g. String)

                                        [exBegin]
    |session selectStatement resultStream|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select id from querytest where id like :id and name like :name'.
    selectStatement stringBindVariable:#name.
    selectStatement stringBindVariable:#id.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    resultStream := WriteStream on:Array new.
    session executeStatement:selectStatement from:#('%9%' 'number 1%9%') into:resultStream numberOfRows:50.
    session executeStatement:selectStatement from:#('%8%' 'number 1%8%') into:resultStream numberOfRows:50.
    resultStream contents inspect.

    session disconnect.
                                        [exEnd]
"
!

example_selectStatement4
"
read all rows from the test table. Show the content received in a separte thread.
All variables in the statement must be bound via

    OCI8Statement>>byteBindVariable:       (for variables which hold bytes e.g. ByteArray)                 or
    OCI8Statement>>numberBindVariable:     (for variables which hold numbers e.g. Integer)         or
    OCI8Statement>>stringBindVariable:     (for variables which hold strings e.g. String)

                                        [exBegin]
    |session selectStatement resultStream resultSet p|

    session := OCI8Session default.
    selectStatement := session newStatementFor:'select * from querytest'.

    resultSet := CollectingSharedQueueStream new.
    p := [
        |result|

        [result := resultSet next.
        result notNil]
        whileTrue:[
            Transcript showCR:result.                                          
        ]
    ] fork.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    [
        session executeStatement:selectStatement into:resultSet.
    ] on:GenericException do:[:ex|
        ex printOn:Transcript.
        p terminate.
    ].
    resultSet close.

    session disconnect.
                                        [exEnd]
"
! !

!OCI8QueryStatement methodsFor:'accessing'!

columnDescriptors
"
returns the column descriptors which have a description of the
selected columns.

<return: OrderedCollection>
"

    columnDescriptors isNil ifTrue:[
        columnDescriptors := OrderedCollection new.
    ].
    ^columnDescriptors
!

pieceSize
"
returns how many bytes should be fetched for a piecewise operation. If the statement
includes a long or long raw column, the rows are fetched pieces wise.
The number of bytes to be fetched by each server round trip has an important impact
onto the performace of statement execution. A higher number reduces the necessary
server roundtrips. The default is 30000 bytes rows per roundtrip.

<return: Integer>
"
    pieceSize isNil ifTrue:[
        pieceSize := 30000.
    ].
    ^ pieceSize
!

pieceSize:anInteger
"
sets how many bytes should be fetched for each piecewise operation,
if the receiver is a piecewise fetch. The receiver is a piecewice operation if
one of the define variables is a dynamic variable.

<return: Integer>
"

    pieceSize := anInteger.
!

prefetchedRows
"
returns how many rows should be prefetched for the receiver by the oci lib.
The number of prefetched rows has an important impact onto the performace
of statement execution. A higher number reduces the necessary server roundtrips.

<return: Integer>
"
    prefetchedRows isNil ifTrue:[
        prefetchedRows := 100.
    ].
    ^ prefetchedRows
!

prefetchedRows:anInteger
"
sets how many rows should be prefetched for the receiver by the oci lib.
The number of prefetched rows has an important impact onto the performace
of statement execution. A higher number reduces the necassary server roundtrips.

<return: self>
"

    prefetchedRows := anInteger.
! !

!OCI8QueryStatement methodsFor:'binding'!

createBindVariablesFor:aCollection
"
creates bind variables out of the bind descriptions. In a query statement only single values
can be passed for the variables. For example: select * from test where id in(:ids)
only one value (e.g. 100) can be passed to the ids bind variable. That is a restriction of
the oci lib!! Hence only single valued bind variables are used here.
For the above example a solution is the following construction:
select * from test where id in(:id1 :id2 :id3 :id4)

<return: self>
"
    |userBindDescriptor element bindVar bindVarName numberOfBindVars|

    "generate new bind variables only when statement isn't binded"
    self binded ifTrue:[^self].

    "get number of bind variables needed"
    aCollection isEmpty ifTrue:[
        numberOfBindVars := 0.
    ] ifFalse:[
        ((element := aCollection first) isArray or:[element isOrderedCollection]) ifTrue:[
            numberOfBindVars := element size.
        ] ifFalse:[
            numberOfBindVars := aCollection size.
        ].
    ].

    "check if correct number of values for bind variables are available"
    numberOfBindVars = self bindNames size ifFalse:[
        ^ OCIErrorSignal raiseErrorString:' - wrong number of values for bind variables specified in ''from'' stream/collection'.
    ].

    "generate bind variables"
    1 to: numberOfBindVars do:[:index |
        bindVarName := self bindNameAtPosition:index.
        userBindDescriptor := self bindNameFor:bindVarName.
        userBindDescriptor isNil ifTrue:[
            OCIErrorSignal raiseErrorString:' - undefine bind variable for index ', index printString.
        ].
        bindVar := OCI8SingleValueBindVariable name:(userBindDescriptor key) converter:(userBindDescriptor value).
        self bindVariables add:bindVar.
    ].
    binded := false.
!

doBindingFor:aCollection
"
creates bind variables out of the description parameters and bind it to the statement via an oci call.
Raise an error when the number of elements in the bind variable are different.

<return: self>
"
    "create bind variables out of the definition in bindDefinition"
    self createBindVariablesFor:aCollection.
    "set value of bind variables"
    self setValueOfBindVariablesFor:aCollection.
    "set up each bind variable and bind it to the statement"
    self doBindingVariables.

    self binded:true.
!

setValueOfBindVariablesFor:aCollection
"
sets values from the collection aCollection to the bind varaibles.
The format of the collection could be:
#(el_1 el_2 ... el_n) or
#(#(el_1 el_2 ... el_n))

<return: self>
"
    |bindVar element row| 

    "if collection is empty nothing to do"
    aCollection isEmpty ifTrue:[^self].
    "get row from collection"
    ((element := aCollection first) isArray or:[element isOrderedCollection]) ifTrue:[
        row := element.
    ] ifFalse:[
        row := aCollection.
    ].

    "set values to the bind variables"
    1 to: row size do:[:index |
        bindVar := bindVariables at:index.
        bindVar addValue:(row at:index)
    ]
! !

!OCI8QueryStatement methodsFor:'execution'!

describe
    "describes a query statement"

    |status|

    self fetchingFinished:false.
    abortState := false.
    status := self ociLib ociStatementExecute:self
                   serviceOCIHandle:self serviceHandle
                     errorOCIHandle:self errorHandle
                               mode:#'OCI_DESCRIBE_ONLY'.

    status ~~ #'OCI_SUCCESS' ifTrue:[ |exception|
        exception :=
            self ociLib createExceptionFor:status errorHandle:self errorHandle.
        exception statement:self.
        exception raiseRequest.
    ].

    self setDefineVariablesAndColumnDescriptors.
!

execute
    "executes a query statement"

    |dataAvailable|

    self setPrefetchRowCount.
    self fetchingFinished:false.

    "execute statement"
    dataAvailable := super execute.
    dataAvailable ifTrue:[
        self setDefineVariablesAndColumnDescriptors.
    ].
    ^ dataAvailable.
!

executeFrom:anInsertStreamOrCollection
     "Execute a query statement and bind the variables.
      Return true, if data is available"

    |content|

    "check for correct type of anInsertStreamOrCollection"
    anInsertStreamOrCollection isStream ifTrue:[
        content := anInsertStreamOrCollection contents.
    ] ifFalse:[
        anInsertStreamOrCollection isCollection ifTrue:[
            content := anInsertStreamOrCollection.
        ] ifFalse:[
            OCIErrorSignal raiseErrorString:' - exepected stream or collection in ''from'''.
        ].
    ].
    "bind variable"
    self doBindingFor:content.

    "execute statement"
    ^ self execute.
!

executeFrom:anInsertStreamOrCollection into:aReturningStream
"
executes a query statement. The values from the stream/collection will be copied to the
variables in the statement text. All result of the statement will be insert into the stream aReturningStream.
"
    ^ self executeFrom:anInsertStreamOrCollection into:aReturningStream numberOfRows:nil
!

executeFrom:anInsertStreamOrCollection into:aReturningStream numberOfRows:aRowIntegerOrNil
"
executes a query statement. The values from the stream/collection will be copied to the
variables in the statement text.
Into the stream aReturningStream the result of the statements will be insert.
If a  aRowIntegerOrNil is nil all results of the statement will be insert into the stream aReturningStream,
if not aRowIntegerOrNil number of results of the statement will be insert into the stream aReturningStream.

<return: Boolean>
"
    |dataAvailable content|

    "check for correct type of anInsertStreamOrCollection"
    anInsertStreamOrCollection isStream ifTrue:[
        content := anInsertStreamOrCollection contents.
    ] ifFalse:[
        anInsertStreamOrCollection isCollection ifTrue:[
            content := anInsertStreamOrCollection.
        ] ifFalse:[
            OCIErrorSignal raiseErrorString:' - exepected stream or collection in ''from'''.
        ].
    ].
    "bind variable"
    self doBindingFor:content.

    "execute statement"
    dataAvailable := self execute.

    "fetch data"
    (dataAvailable and:[aRowIntegerOrNil ~~ 0]) ifTrue:[
        self fetchDataInto:aReturningStream numberOfRows:aRowIntegerOrNil.
    ].
    ^ dataAvailable
!

executeInto:aReturningStream
"
executes a query statement with no variables in the statement text.
All results of the statement will be insert into the stream aReturningStream.
"
    ^ self executeFrom:#() into:aReturningStream
!

executeInto:aReturningStream numberOfRows:aRowIntegerOrNil
"
executes a query statement with no variables in the statement text.
Into the stream aReturningStream the result of the statements will be insert.
If a  aRowIntegerOrNil is nil all results of the statement will be insert into the stream aReturningStream,
if not aRowIntegerOrNil number of results of the statement will be insert into the stream aReturningStream.
"
    ^ self executeFrom:#() into:aReturningStream numberOfRows:aRowIntegerOrNil
! !

!OCI8QueryStatement methodsFor:'fetching'!

fetchDataInto:aReturningStream numberOfRows:aRowIntegerOrNil
"
fetched data for the receiver. Into the stream aReturningStream the result of the receiver will be insert.
If a  aRowIntegerOrNil is nil all results of the receiver will be insert into the stream aReturningStream,
if not aRowIntegerOrNil number of results of the receiver will be insert into the stream aReturningStream.

<return: self>
"
    |finished numerOfRows serviceHandle errorHandle|

    "if number of rows to fetch is 0 - return"
    (aRowIntegerOrNil == 0 or:[self fetchingFinished]) ifTrue:[
        ^self
    ].
    "return if the statement should be aborted"
    abortState ifTrue:[
        ^ self
    ].
    "the receiver must be executed or must be fetched to do another fetch operation"
    (self isExecuted or:[self isFetched]) ifFalse:[
        OCIErrorSignal raiseErrorString:' - cannot fetch data from statement when first fetch has not been finished'.
    ].
    serviceHandle := self serviceHandle.
    errorHandle:= self errorHandle.
    numerOfRows := 0.
    finished := false.
    self setFetching.
    [
        OCINoDataSignal handle:[:ex |
            finished := true.
            self fetchingFinished:true.
        ] do:[
            dynamicDefineVariables notEmptyOrNil ifTrue:[
                "if one of the define variables of the receiver is a dynamic variable,"
                "then the data must be fetched by a piecewise operation."

                "problems with OCI8 and unblock connection.
                 so block here... (unblockConnection toggles unblock mode"
                session unblockConnection.
                [
                    |result|
                    result := self ociLib ociStatementPiecewiseFetch:self
                                               serviceOCIHandle:serviceHandle
                                                      pieceSize:self pieceSize
                                                   numberOfRows:1 
                                                 errorOCIHandle:errorHandle.
                    dynamicDefineVariables do:[:eachVariable|  |stream|
                        stream := result at:eachVariable handle handle ifAbsent:[].
                        stream notNil ifTrue:[
                            eachVariable dataBuffer:stream contents.
                        ].
                    ].                
                ] ensure:[
                    session unblockConnection.
                ].
            ] ifFalse:[
                "normal fetch"
                self ociLib ociStatementFetch:self
                             serviceOCIHandle:serviceHandle
                                 numberOfRows:1 
                               errorOCIHandle:errorHandle.
            ].
            "fill result collection if the fetch wasnt aborted"
            (self isAborted or:[self shouldAbort]) ifTrue:[
                self setAborted.
                finished := true.
            ] ifFalse:[
                aReturningStream nextPut:(ociDefineVariables collect:[:eachVar | eachVar getValue]).
                numerOfRows := numerOfRows + 1.
                "set finished to true if the number of already fetched rows
                 is  higer than the number to fetch. 
                 If maximum number is not specified fetch all available rows."

                (aRowIntegerOrNil isNil or:[numerOfRows < aRowIntegerOrNil]) ifFalse:[
                    finished := true.
                ]
            ].
        ].
    ] doUntil:[finished].

    self setFetched.
!

next
    "fetch the next row from the cursor.
     OCINoDataSignal is raised, if no more rows are available"

    |serviceHandle errorHandle|

    serviceHandle := self serviceHandle.
    errorHandle:= self errorHandle.
    dynamicDefineVariables notEmptyOrNil ifTrue:[
        "if one of the define variables of the receiver is a dynamic variable,"
        "then the data must be fetched by a piecewise operation."
        session unblockConnection.
        [
            |result|
            result := self ociLib ociStatementPiecewiseFetch:self
                                       serviceOCIHandle:serviceHandle
                                              pieceSize:self pieceSize
                                           numberOfRows:1 
                                         errorOCIHandle:errorHandle.
            dynamicDefineVariables do:[:eachVariable|  |stream|
                stream := result at:eachVariable handle handle ifAbsent:[].
                stream notNil ifTrue:[
                    eachVariable dataBuffer:stream contents.
                ].
            ].                
        ] ensure:[
            session unblockConnection.
        ].
    ] ifFalse:[
        "normal fetch"
        self ociLib ociStatementFetch:self
                     serviceOCIHandle:serviceHandle
                         numberOfRows:1 
                       errorOCIHandle:errorHandle.
    ].
    "fill result collection only if the fetch hasn't been aborted"
    abortState ifTrue:[
        ^ nil.
    ].

    "ociDefineVariables is an array, so #collect: generates arrays"
    ^ ociDefineVariables collect:[:eachVar | eachVar getValue].
!

nextAsObject
    "answer the current row as an object which understands getter methods as per column"

    |names inst session columns|
    columns := self next.
    session := self session.
    columnDescriptors isNil ifTrue:[
        ^ nil
    ].

    objectClass isNil ifTrue:[
        names := columnDescriptors collect:[:aDesc |aDesc name asSymbol].  
        objectClass := session cachedRowObjectClasses at:names ifAbsentPut:[
            |cls itsSuperClass|

            itsSuperClass := OCIRowObject.

            cls := ClassDescription basicNew.
            cls
                setSuperclass:itsSuperClass
                methodDictionary:(MethodDictionary new:names size)
                instSize:0
                flags:(ClassDescription flagBehavior).

            cls instSize:columns size + (itsSuperClass instSize).
            cls setInstanceVariableString:names.
            names do:[:eachInstVarName |
                |m|

                m := Compiler 
                            compile:(eachInstVarName , ' ^ ',eachInstVarName)
                            forClass:cls
                            install:false.

                "/ cg: can use primAddSelector (which does not flush caches),
                "/ because this is a new class, which has never been instanciated
                "/ (therefore, no caches refer to this class yet).
                cls 
                    primAddSelector:eachInstVarName asSymbol
                    withMethod:m.

                m := Compiler 
                            compile:(eachInstVarName, ':anObject', ' ',eachInstVarName, ':= anObject')
                            forClass:cls
                            install:false.
                cls 
                    primAddSelector:(eachInstVarName,':') asSymbol
                    withMethod:m.
            ].
            cls
        ].
    ].           
    inst := (Array new:objectClass superclass instSize) , columns.    
    inst changeClassTo:objectClass.
    ^ inst
! !

!OCI8QueryStatement methodsFor:'private accessing'!

columnDescriptors:aCollection
"
sets the column descriptors which have a description of the
selected rows.

<return: self>
"

    columnDescriptors := aCollection.
!

defined
"
returns if the receivers define variables are generated and setup.

<return: Boolean>
"
    defined isNil ifTrue:[
	defined := false.
    ].
    ^ defined
!

defined:aBoolean
"
sets if the receivers define variables must be generated

<return: Boolean>
"

    defined := aBoolean.
!

fetchingFinished
"
returns if the receivers fetch operation is finished. No
oci fetch operation should be call if the flag is true

<return: Boolean>
"
    fetchingFinished isNil ifTrue:[
        fetchingFinished := false.
    ].

    ^ fetchingFinished
!

fetchingFinished:aBoolean
"
sets if the receivers fetch operation is finished. No
oci fetch operation should be call if the flag is true

<return: self>
"

    fetchingFinished := aBoolean.
!

ociDefineVariables
"
returns the define variables of the receiver.

<return: OrderedCollection>
"
    ociDefineVariables isNil ifTrue:[
        ociDefineVariables := #().
    ].
    ^ ociDefineVariables
!

ociDefineVariables:aCollection

    ociDefineVariables := aCollection asArray.
    dynamicDefineVariables := ociDefineVariables select:[:var| var isDynamic] ifNone:[].
! !

!OCI8QueryStatement methodsFor:'queries'!

numberOfColumns
   "answer the number of columns returned by this query"

   ^ ociDefineVariables size
!

processedRows
   "answer the number of rows fetched by the user.
    Redefined to not use cached value, since this changes with each fetch"

   ^ self getProcessedRows
! !

!OCI8QueryStatement methodsFor:'setting'!

setDefineVariablesAndColumnDescriptors
"
if the statement isnt defined the column descriptors which have a description of the
selected columns and the define variables (from the column descriptors) will be computed.

<return: self>
"
    |position defineVariable|

    self isDefined ifTrue:[^self].
    "set column descriptors"
    self columnDescriptors:(OCI8ColumnDescriptor newForStatement:self) asArray.
    "set define varaibles"
    position := 1.
    self ociDefineVariables:(
            columnDescriptors collect:[:eachDescriptor|
                defineVariable := OCI8DefineVariable newFor:self 
                                                 columnDescriptor:eachDescriptor 
                                                         position:position.
                position := position + 1.
                defineVariable.
            ]
    ).

    self defined:true.
!

setPrefetchRowCount
"
sets how many rows should be prefetched for the receiver by the oci lib.
The number of prefetched rows has an important impact on the performace
of statement execution. A higher number reduces the necassary server roundtrips.

<return: self>
"
    self ociLib ociAttrSet:self handle
                 attribute:self prefetchedRows 
             attributeType:#'OCI_ATTR_PREFETCH_ROWS'
            errorOCIHandle:self errorHandle
! !

!OCI8QueryStatement methodsFor:'setup'!

initialize
    "statement has been prepared"

    self computeBindDescriptors.
! !

!OCI8QueryStatement methodsFor:'subclass responsibility'!

iterator
"
returns the number of many times a statement must be executed.
For query statements, the iterator is set to 0, because no prefetch is done.
All rows are fetched via oci fetch calls.

<return: Integer>
"
    ^0
! !

!OCI8QueryStatement methodsFor:'testing'!

isDefined
"
returns true if the receivers define variables are generated and setup
if not false

<return: Boolean>
"
    ^self defined

!

isQueryStatement
"
returns if the statement is a query statement

<return: Boolean>
"
    ^true

! !

!OCI8QueryStatement::OCIRowObject class methodsFor:'documentation'!

documentation
"
    
    ODBCDataObject is the abstract superclass of row objects created by a cursor.

    The subclasses of ODBCDataObject allow access to row objects through generating an interface 
    based on the names of the column descriptors in the original SELECT statement. Single getter and setter
    methods are generated upon creation of a subclass and multi-setter methods are generated at run time.

    A cache of data objects classses are held in each session for performance reasons.
    


    [author:]
        james (james@CHANTI)

    [instance variables:]

    [class variables:]

    [see also:]

"
!

history
    "Created: / 12-01-2006 / 12:01:08 / james"
! !

!OCI8QueryStatement::OCIRowObject methodsFor:'adaption'!

doesNotUnderstand:aMessage
    |keywords s s2 instVars|
    keywords := aMessage selector keywords.
    s := String new writeStream.
    s2 := String new writeStream.
    instVars := self class instanceVariableNames.
    keywords do:[:aKeyword |
        |withoutColon|
        withoutColon := aKeyword copyWithoutLast:1.
        (instVars includes:withoutColon) ifFalse:[
            super doesNotUnderstand:aMessage.
        ].
        s nextPutAll:aKeyword, '_a',withoutColon.
        s nextPut:Character space.

        s2 nextPutAll:withoutColon, ' := _a',withoutColon.
        s2 nextPutAll:'. '.
    ].
    s nextPutAll:s2 contents.

    self class compile:s contents classified:'adapted accessors'.
    aMessage sendTo:self.
! !

!OCI8QueryStatement::OCIRowObject methodsFor:'conversion'!

asArray
    ^ self shallowCopy changeClassTo:Array.
! !

!OCI8QueryStatement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8QueryStatement.st,v 1.49 2006/09/27 14:49:52 james Exp $'
! !
