"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Session
	instanceVariableNames:'connected environmentHandle errorHandle ociLib serverHandle
		serviceHandle userSessionHandle weakChildren
		cachedRowObjectClasses'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Session'
!

!OCI8Session class methodsFor:'documentation'!

documentation
"
    Framework for accessing oracle databases.
    For accessing an oracle database create an instance of OCI8Session with:
        OCI8Session class>>default.

    Connect to a database with:
        OCI8Session>>connectWithUsername:password:dbname:

    Disconnect from a database with:
        OCI8Session>>disconnect
    A disconnect is done if your instance of OCI8Session is finalized.

    Create a statement with:
        OCI8Session>>newStatementFor:
        Returns an instance of OCI8DDStatement, OCI8DMStatement or OCI8QueryStatement.

    Execute a statement with (does an implicit fetch for OCI8QueryStatement):
        OCI8Session>>executeStatement:                         (for OCI8DDStatement and OCI8DMStatement)
        OCI8Session>>executeStatement:from:                    (for OCI8DMStatement and OCI8QueryStatement)
        OCI8Session>>executeStatement:from:into:               (for OCI8DMStatement and OCI8QueryStatement)
        OCI8Session>>executeStatement:from:into:numberOfRows:  (for OCI8QueryStatement)
        OCI8Session>>executeStatement:into:                    (for OCI8DMStatement and OCI8QueryStatement)
        OCI8Session>>executeFrom:into:numberOfRows:            (for OCI8QueryStatement)

    Fetch (more) results from a statement (OCI8QueryStatement) with (used when the results are
    restricted by numberOfRows):
        OCI8Session>>fetchDataForStatement:anOCI8Statement into:               (for OCI8QueryStatement)
        OCI8Session>>fetchDataForStatement:anOCI8Statement into:numberOfRows:  (for OCI8QueryStatement)

    To create transcations call:
        OCI8Session>>newReadOnlyTransaction
        OCI8Session>>newReadWriteTransaction

    For detailed information see ee the examples in OCI8Session, OCI8DDStatement, OCI8DMStatement,
    OCI8QueryStatement and OCI8Transaction.

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8DDStatement
        OCI8DMStatement
        OCI8QueryStatement
        OCI8Statement
        OCI8Transaction

    [instance variables:]

        connected <Boolean>
            Is true if a session is connected to a database, false if not.

        environmentHandle <OCI8Handle>
            Holds the environment handle of the oci lib for the session. All other handles
            are subhandles of the environment handle. In the finalization of a session only
            the environment handle needs to be freed. All other subhandles are released implicit.
        
        errorHandle <OCI8Handle>
            Holds an error handle. To most of the oci methods a error handle is passed to
            get more information in case of an error. Used for the internal error handling of
            the oci lib.

        ociLib <Class>
            Holds an instance of OCI8Lib. All oci methods are available
            an insctance of the oci call interface lib.

        serverHandle <OCI8Handle>
            Holds an handle to the oci server. The server handle is attached to the
            service handle with the database name given in the connection method.

        serviceHandle <OCI8Handle>
            Holds an handle to the oci service. The server handle and user handle are
            attached to this handle.

        userSessionHandle <OCI8Handle>
            Holds an handle to an oci user session. The user session handle is attached to the
            service handle with the username, password given in the connection method.

        weakChildren <WeakIdentitySet>
            Holds all dependent objects (statements, transactions) in a weak set which are generated by the session.
            In the finalization of the session the environment handle is freed and implicitly
            all dependent handles. Therefore the dependent handles in all statements which
            were not yet finialized are invalidated. That prevents that they are freed in the finalization
            of the statements.

    [class variables:]
        none
"
!

history
    "Created: / 13.3.2000 / 17:20:38 / ps"
! !

!OCI8Session class methodsFor:'instance creation'!

default
"
creates a new session and initializes in the default oci mode.

<return: OCI8Session>
"
   ^ self basicNew initialize:#(#OCI_DEFAULT)
!

new
"
do not use Session new. Use one default mode.

<return: OCI8Session>
"
   self error:'use default mode instead'


!

shared
"
creates a new session and initializes it shared.

<return: OCI8Session>
"
   self error:'not supported yet. Use default mode instead.'


!

threaded
"
creates a new session and initializes it threaded.

<return: OCI8Session>
"
   self error:'not supported yet. Use default mode instead.'


! !

!OCI8Session class methodsFor:'examples'!

example_ConnectToDatabase
"  
    create a session and connect to a db
                                                    [exBegin]
    |session|
    session := OCI8Session default.
    session connectWithUsername:'scott' password:'tiger' dbname:'test'.
    Transcript showCR:'Connected'.
    Delay waitForSeconds:5.
    session disconnect.
    Transcript showCR:'Disconnected'.
                                                    [exEnd]
"

!

example_ShowTableInformation
"  
show the column informations for a table

                                        [exBegin]
    |session|
    session := OCI8Session default.
    session connectWithUsername:'scott' password:'tiger' dbname:'test'.
    (session columnsOfTable:'emp') inspect.
    session disconnect.
                                        [exEnd]

"

!

example_ShowTables
"  
show the column informations for a table

                                        [exBegin]
    |session statement s|
    session := OCI8Session default.
    session connectWithUsername:'system' password:'manager' dbname:'secrec'.
    (statement := session newStatementFor:'select * from dba_tables order by owner') executeInto:(s := #() writeStream).
    statement columnDescriptors inspect.
    s contents inspect.
    session disconnect.
                                        [exEnd]

"

! !

!OCI8Session methodsFor:'accessing'!

cachedRowObjectClasses
    cachedRowObjectClasses isNil ifTrue:[
        cachedRowObjectClasses := Dictionary new.
    ].
    ^ cachedRowObjectClasses
!

connected
"
returns if the receiver is connected to a database.

<return: Boolean>
"
    connected isNil ifTrue:[
	connected := false.
    ].
    ^ connected
!

ociLib
"
returns the oci database lib which is used for the oci calls

<return: OCI8Lib>
"
    ociLib isNil ifTrue:[
	ociLib := self databaseLib.
    ].
    ^ ociLib
! !

!OCI8Session methodsFor:'connection'!

connectWithUsername:aUsername password:aPassword dbname:aDBName
"
establish a server to the database dbname and start a user session with username and password.
If connected do nothing.

<return: self>
"
    self isConnected ifTrue:[^self].
    self initializeServerFor:aDBName.
    self beginUserSessionFor:aUsername password:aPassword.
    "unblock the connection"
    self unblockConnection.
    self connected:true.
!

disconnect
"
terminates a session and connection. The server, user session 
and service context handles are deallocated implicitly. Its possible to disconnect
while an statement is executed.

<return: self>
"
    self isConnected ifFalse:[^self].

    Error handle:[:ex |
    ] do:[
        self endUserSession.
    ].
    self releaseUserSessionHandle.

    Error handle:[:ex |
    ] do:[
        self detachServer.
    ].
    self releaseServerHandle.

    self connected:false.
! !

!OCI8Session methodsFor:'constants'!

databaseLib
"
returns the default oci lib, which is used for the oci calls.

<return: OCI8Lib>
"
    ^OCI8Lib new
!

statementClass
"
returns the default abstract statment class, which generates 
instances of dm-, dd- or query-statments.

<return: OCI8Statement class>
"
    ^OCI8Statement
!

transactionClass
"
returns the default transaction class, which generates 
instances of transactions.

<return: OCI8Transaction class>
"
    ^OCI8Transaction


! !

!OCI8Session methodsFor:'describe table'!

columnsOfTable:aTableName
"
describes a table with the name aTableName. A collection of column
descriptors is returned. Each of them describes a column in the table.

<return: OrderedCollection with:OCI8ColumnDescriptor>
"

    |describeHandle parameterHandle numberOfCols listHandle parameterColl|

    "describe table"
    [
        describeHandle := self newDescribeHandle.
        self ociLib ociDescribeAny:self serviceHandle 
                            schemaName:aTableName 
                            schemaType:#'OCI_PTYPE_TABLE' 
                            newHandle:describeHandle 
                            errorOCIHandle:self errorHandle.
        "get parameter handle"
        parameterHandle := OCI8Handle new type:#'OCI_DTYPE_PARAM'.
        parameterHandle handle:(self ociLib ociAttrGet:describeHandle 
                                         attributeType:#'OCI_ATTR_PARAM' 
                                        errorOCIHandle:self errorHandle).
        "get number of columns"
        numberOfCols := self ociLib ociAttrGet:parameterHandle 
                                    attributeType:#'OCI_ATTR_NUM_COLS' 
                                    errorOCIHandle:self errorHandle.
        "get the handle to the column list of the table"
        listHandle := OCI8Handle new type:#'OCI_DTYPE_PARAM'.
        listHandle handle:(self ociLib ociAttrGet:parameterHandle 
                                    attributeType:#'OCI_ATTR_LIST_COLUMNS' 
                                    errorOCIHandle:self errorHandle).

        parameterColl := Array new:numberOfCols.
        1 to:numberOfCols do:[:i| 
            parameterHandle := self ociLib ociParamGet:listHandle
                                              position:i
                                        errorOCIHandle:self errorHandle.
            parameterColl at:i put:(
                OCI8ColumnDescriptor newForParameterHandle:parameterHandle   
                                                    ociLib:self ociLib 
                                               errorHandle:self errorHandle).
        ].
    ] ensure:[
        self releaseOCIHandle:describeHandle.
    ].

    ^ parameterColl
! !

!OCI8Session methodsFor:'finalization'!

finalize
"
this is invoked when an instance of myself is simply forgotten.
Release the handles of the receiver and do a logoff from the database if necessary.

<return: self>
"
    self releaseHandles

    "Modified: / 16.2.2000 / 13:17:00 / ps"
! !

!OCI8Session methodsFor:'initialize & release'!

initialize:aModeColl
"
creates a new instance of an oci8 lib. The enviroment and error handle
will be initialized. The receiver is entered in a registry. Therefore dispose will be
called for the receiver during finialization.

<return: self>
"
"/    KeptSessions := nil isNil ifTrue:[
"/        KeptSessions := Set new.
"/    ].
"/    KeptSessions add:self.

    self initializeEnvironment:aModeColl.


!

initializeEnvironment:aModeColl
"
initialize the enviroment handle. The receiver is entered in a registry.
Therefore dispose will be call for the receiver while finialization.

<return: self>
"   
    self ociLib ociEnvCreate:self environmentHandle mode:aModeColl.
!

releaseHandles
"
release the external bytes of the service, error and environment handles and
remove the receiver from the registry.

<return: self>
"
    |tmpHandle|

    "end user session and server detach must be called"
    "because if transaction commit has not been called"
    "any pending transactions are automatically rolled back"
    "if end user sesion and server detach is succesfull a commit will be done"
    self isUserSessionHandleInitialized ifTrue:[
        Error handle:[:ex|
        ] do:[
            self endUserSession
        ].
    ].

    "detach from server if server handle initialized"
    self isServerHandleInitialized ifTrue:[
        Error handle:[:ex|
        ] do:[
            self detachServer.
        ].
    ].

    "clear sub-handles in dependent objects because they are implicitly deallocated with the environment handle"
    self weakChildren do:[:eachChild|
        (eachChild notNil and:[eachChild ~~ 0]) ifTrue:[
            eachChild invalidateHandles.
        ].
    ].
    weakChildren := nil.

    "release environment handle"
    "all other subhandles are released implicitly with the environment handle"
    tmpHandle := environmentHandle.
    environmentHandle := nil.
    self releaseOCIHandle:tmpHandle.

    "remove the receiver from the registry"
    self unregisterForFinalization.

    "Modified: / 16.2.2000 / 13:01:45 / ps"

! !

!OCI8Session methodsFor:'oci handles'!

newDescribeHandle
"
returns a new oci handle for the type #OCI_HTYPE_DESCRIBE.

<return: OCIHandle>
"
    ^self ociLib ociHandleAlloc:self environmentHandle type:#'OCI_HTYPE_DESCRIBE'


!

newErrorHandle
"
returns a new oci handle for the type #'OCI_HTYPE_ERROR'.

<return: OCIHandle>
"
    ^self ociLib ociHandleAlloc:self environmentHandle type:#'OCI_HTYPE_ERROR'


!

newServerHandle
"
returns a new oci handle for the type #'OCI_HTYPE_SERVER'.

<return: OCIHandle>
"
    ^self ociLib ociHandleAlloc:self environmentHandle type:#'OCI_HTYPE_SERVER'


!

newServiceHandle
"
returns a new oci handle for the type #'OCI_HTYPE_SVCCTX'.

<return: OCIHandle>
"
    ^self ociLib ociHandleAlloc:self environmentHandle type:#'OCI_HTYPE_SVCCTX'


!

newUserSessionHandle
"
returns a new oci handle for the type #'OCI_HTYPE_SESSION'.

<return: OCIHandle>
"
    ^self ociLib ociHandleAlloc:self environmentHandle type:#'OCI_HTYPE_SESSION'


!

releaseOCIHandle:anOCI8Handle
"
release the external bytes of the oci handle.

<return: self>
"  
    (anOCI8Handle notNil and:[anOCI8Handle isValid]) ifTrue:[
	self ociLib ociHandleFree:anOCI8Handle.
    ].
!

releaseServerHandle
"
sets the server handle to nil and free it.
The finalization copy of the receiver must be updated.

<return: self>
"
    |tmpHandle|

    tmpHandle := serverHandle.
    serverHandle := nil.
    self releaseOCIHandle:tmpHandle.
    self reRegisterForFinalization.
!

releaseUserSessionHandle
"
sets the user session handle to nil and free it.
The finalization copy of the receiver must be updated.

<return: self>
"
    |tmpHandle|

    tmpHandle := userSessionHandle.
    userSessionHandle := nil.
    self releaseOCIHandle:tmpHandle.
    self reRegisterForFinalization.
! !

!OCI8Session methodsFor:'private accessing'!

connected:aBoolean
"
sets the connection state of the session.

<return: self>
"

    connected := aBoolean.
    self reRegisterForFinalization.

!

environmentHandle
"
returns the environment handle. 
All other handles are subhandles of the eviroment handle.
This handle must be freed in the finalization via an oci call.
The oci lib will free automaticaly all subhandles.

The finalization copy of the receiver must be updated.

<return: OCI8Handle>
"
    environmentHandle isNil ifTrue:[
        environmentHandle := OCI8Handle new type:#'OCI_HTYPE_ENV'.
        self reRegisterForFinalization.
    ].
    ^ environmentHandle

!

errorHandle
"
returns the error handler from the session. The handle is used
to get specific informations when an error occured.
The finalization copy of the receiver must be updated.

<return: OCI8Handle>
"
    errorHandle isNil ifTrue:[
        errorHandle := self newErrorHandle.
        self reRegisterForFinalization.
    ].
    ^ errorHandle

!

serverHandle
"
returns the server handle. The finalization copy 
of the receiver must be updated.

<return: OCI8Handle>
"
    serverHandle isNil ifTrue:[
	serverHandle := self newServerHandle.
	self reRegisterForFinalization.
    ].
    ^ serverHandle
!

serviceHandle
"
returns the service handle. The finalization copy 
of the receiver must be updated.

<return: OCI8Handle>
"
    serviceHandle isNil ifTrue:[
	serviceHandle := self newServiceHandle.
	self reRegisterForFinalization.
    ].
    ^serviceHandle
!

userSessionHandle
"
returns the user session handle. The finalization copy 
of the receiver must be updated.

<return: OCI8Handle>
"
    userSessionHandle isNil ifTrue:[
	userSessionHandle := self newUserSessionHandle.
	self reRegisterForFinalization.
    ].
    ^ userSessionHandle

!

weakChildren
"
returns a WeakIdentitySet which holds all references to generated 
object whose handles are freed together with the emvironment handle, e.g. statements, transactions.
The set is used to nil out all handles when the receiver is finalized
because when finializing the receiver the eviroment handle is freed. This
includes the release of all dependent handles.

<return: WeakIdentitySet>
"
    weakChildren isNil ifTrue:[
        weakChildren := WeakIdentitySet new.
        self reRegisterForFinalization.
    ].
    ^ weakChildren
! !

!OCI8Session methodsFor:'private connection'!

beginUserSessionFor:aUsername password:aPassword
"
sets the username and password properties in the user session handle and
connects to the server. Add the session handle to the session attribute
of the service context. 

<return: self>
"
    "set username attribute in user session handle"
    self ociLib ociAttrSet:self userSessionHandle
                 attribute:aUsername 
             attributeType:#'OCI_ATTR_USERNAME' 
            errorOCIHandle:self errorHandle.
    "set password attribute in user session handle"
    self ociLib ociAttrSet:self userSessionHandle
                 attribute:aPassword 
             attributeType:#'OCI_ATTR_PASSWORD' 
            errorOCIHandle:self errorHandle.
    "start session"
    self ociLib ociSessionBegin:self serviceHandle 
                    userSession:self userSessionHandle 
                     credential:#'OCI_CRED_RDBMS'
                           mode:#'OCI_DEFAULT' 
                    errorHandle:self errorHandle.
    "set the user session in the service context"
    self ociLib ociAttrSet:self serviceHandle
                 attribute:self userSessionHandle handle 
             attributeType:#'OCI_ATTR_SESSION' 
            errorOCIHandle:self errorHandle.
!

detachServer
"
detaches from the server.

<return: self>
"
    self ociLib ociServerDetach:self serverHandle
                    errorHandle:self errorHandle.

!

endUserSession
"
terminates user session.

<return: self>
"
    self ociLib ociSessionEnd:self serviceHandle 
                  userSession:self userSessionHandle 
                  errorHandle:self errorHandle.
!

initializeServerFor:aDBName
"
establishes an access path to the server for the database dbname by attaching.
Sets the server attribute to the server handle in the service context.

<return: self>
"
    "if server handle already initialized return"
    self isServerHandleInitialized ifTrue:[^self].
    "create server context"
    self ociLib ociServerAttach:self serverHandle 
                         dbname:aDBName 
                    errorHandle:self errorHandle.
    "set attribute server context in the service context"
    self ociLib ociAttrSet:self serviceHandle
                 attribute:self serverHandle handle 
             attributeType:#'OCI_ATTR_SERVER' 
            errorOCIHandle:self errorHandle.

!

unblockConnection
"
when unblocking the receiver, an oci call returns immediatly and
the lib must poll the result. This prevents smalltalk to be blocked
while doing an oci call.

<return: self>
"
    self ociLib ociAttrSet:self serverHandle
		 attribute:0 
	     attributeType:#'OCI_ATTR_NONBLOCKING_MODE' 
	    errorOCIHandle:self errorHandle
! !

!OCI8Session methodsFor:'statement executing'!

execute:aStatementText
    "execute the statementText and return the result.
     The statement (i.e. the Database cursor) is released after execution"

    |statement result|

    statement := self statementClass newForSession:self text:aStatementText.
    result := statement execute.
    statement release.
    ^ result.


!

executeStatement:anOCI8Statement
"
executes an OCI8DDStatement or OCI8DMStatement statement. No input or output 
streams/collections are needed for the statement.

<return: self>
"
    self isConnected ifTrue:[
        anOCI8Statement execute.
    ] ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ]
!

executeStatement:anOCI8Statement from:anInsertStreamOrCollection
"
executes an OCI8DMStatement or OCI8QueryStatement statement.
The values for the varibales in the statement text are get from the stream/collection anInsertStreamOrCollection.

<return: self>
"
    self executeStatement:anOCI8Statement from:anInsertStreamOrCollection into:nil.
!

executeStatement:anOCI8Statement from:anInsertStreamOrCollection into:aReturningStream
"
executes an OCI8DMStatement or OCI8QueryStatement statement.
The values for the varibales in the statement text are get from the stream/collection anInsertStreamOrCollection.
Into the stream aReturningStream the result of the statements will be insert.

<return: self>
"
    self isConnected ifTrue:[
        anOCI8Statement executeFrom:anInsertStreamOrCollection into:aReturningStream.
    ] ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ]
!

executeStatement:anOCI8Statement from:anInsertStreamOrCollection into:aReturningStream numberOfRows:aRowIntegerOrNil
"
executes an OCI8QueryStatement statement.
The values for the varibales in the statement text are get from the stream/collection anInsertStreamOrCollection.
Into the stream aReturningStream the result of the statements will be insert.
If a  aRowIntegerOrNil is nil all results of the statement will be insert into the stream aReturningStream,
if not aRowIntegerOrNil number of results of the statement will be insert into the stream aReturningStream.

<return: self>
"
    self isConnected ifTrue:[
        anOCI8Statement executeFrom:anInsertStreamOrCollection into:aReturningStream numberOfRows:aRowIntegerOrNil.
    ] ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ]
!

executeStatement:anOCI8Statement into:aReturningStream
"
executes an OCI8DMStatement and OCI8QueryStatement statement.
Into the stream aReturningStream the result of the statements will be insert.

<return: self>
"
    self isConnected ifTrue:[
        anOCI8Statement executeInto:aReturningStream.
    ] ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ]
!

executeStatement:anOCI8Statement into:aReturningStream numberOfRows:aRowIntegerOrNil
"
executes an OCI8QueryStatement statement.
Into the stream aReturningStream the result of the statements will be insert.
If a  aRowIntegerOrNil is nil all results of the statement will be insert into the stream aReturningStream,
if not aRowIntegerOrNil number of results of the statement will be insert into the stream aReturningStream.

<return: self>
"
    self isConnected ifTrue:[
        anOCI8Statement executeInto:aReturningStream numberOfRows:aRowIntegerOrNil.
    ] ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ]
! !

!OCI8Session methodsFor:'statement generating'!

newStatementFor:aStatementText
"
returns a new statement for the text aStatementText. Three different statement type
could be returned depending on the text. An OCI8QueryStatement is returned if the
text specified a select statement. An OCI8DMStatement (Data Manipulation) is returned
for insert, update etc statement texts. An OCI8DDStatement (Data Definition) si returned
for create, drop etc statement texts.

<return: OCI8QueryStatement | OCI8DMStatement | OCI8DDStatement >
"

    ^ self statementClass newForSession:self text:aStatementText.
!

registerNewStatement:aStatement
    "register a new transaction, that has got an OCI handle.
     The transaction is remembered, so that the handle can be invalidated
     as soon as the session is disposed (and all dependent handles are freed)."

    self weakChildren add:aStatement
!

unregisterStatement:aStatement
    "unregister a statement, that has got an OCI handle"

    self weakChildren remove:aStatement ifAbsent:[].
! !

!OCI8Session methodsFor:'testing'!

isConnected
"
test if the receiver is connected.

<return: Boolean>
"

    ^self connected
!

isServerHandleInitialized
"
test if the server handle is setup

<return: Boolean>
"

    ^serverHandle notNil
!

isUserSessionHandleInitialized
"
test if the user session handle is setup

<return: Boolean>
"

    ^userSessionHandle notNil
! !

!OCI8Session methodsFor:'transaction'!

commit
    "commit the current transaction"

    self ociLib ociTransactionCommit:self serviceHandle
                                flag:#'OCI_DEFAULT'
                      errorOCIHandle:self errorHandle
!

newReadOnlyTransaction
"
returns a new instance of the oci trancation class. The transaction
will be read only. No changes can be made to the database then.

<return: OCI8Transaction>
"
    ^self newReadOnlyTransactionWithTimeOut:0
!

newReadOnlyTransactionWithTimeOut:aTimeOutInteger
"
returns a new instance of the oci trancation class. The transaction
will be read only. No changes can be made to the database then.
If the transaction is detached, the transaction is rolled back, if
it isnt attached after aTimeOutInteger seconds.

<return: OCI8Transaction>
"
    self isConnected ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ].
    ^self transactionClass newReadOnlyFor:self timeOut:aTimeOutInteger
!

newReadWriteTransaction
"
returns a new instance of the oci trancation class. The transaction
will be read- and writeable.

<return: OCI8Transaction>
"
    ^self newReadWriteTransactionWithTimeOut:0
!

newReadWriteTransactionWithTimeOut:aTimeOutInteger
"
returns a new instance of the oci trancation class. The transaction
will be read- and writeable.
If the transaction is detached, the transaction is rolled back, if
it isnt attached after aTimeOutInteger seconds.

<return: OCI8Transaction>
"
    self isConnected ifFalse:[
        OCILogOnErrorSignal raiseErrorString:' - not connected'
    ].
    ^self transactionClass newReadWriteFor:self timeOut:aTimeOutInteger
!

registerNewTransaction:aTransaction
    "register a new transaction, that has got an OCI handle.
     The transaction is remembered, so that the handle can be invalidated
     as soon as the session is disposed (and all dependent handles are freed)."

    self weakChildren add:aTransaction
!

rollback
    "rollback the current transaction"

    self ociLib ociTransactionRollback:self serviceHandle
                        errorOCIHandle:self errorHandle
! !

!OCI8Session class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Session.st,v 1.61 2006/09/19 10:35:02 james Exp $'
! !
