"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Statement
	instanceVariableNames:'abortState bindDescriptors binded bindNames bindVariables
		executeMode handle processedRows session state text textBuffer'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Statements'
!

!OCI8Statement class methodsFor:'documentation'!

documentation
"
    abstract super class for oci 8 statements. There are three different kinds of statements. Query,
    data definition and data manipulation statements. 

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8DDStatement
        OCI8DMStatement
        OCI8QueryStatement

    [instance variables:]

        abortState <Boolean>
            can be set to true, if the statement should be aborted, e.g. to
            cancel long running statements.

        arrayBind <Boolean>
            is true if the statement does an array insert (multiple values for one bind variable),
            false if not.

        bindDescriptors
            holds a collection of descriptors which describes the bind variabels in the receivers text
            (see OCI8BindDescriptor).

        binded <Boolean>
            is true if the statement is binded else false

        bindNames <OrderedCollection>
            a collection into which dictionaries (name -> an OCI8Converter) are added.
            Those dictionaries describe the information needed to generate bind variables.

        bindVariables <OrderedCollection>
            collection of bind variables for the receiver. These are
            automatical generated by the describtions out of the instance variable
            bindNames.

        executeMode <Symbol>
            describes how this statement is executed; if not specified the mode is OCI_DEFAULT

        handle <OCI8Handle>
            holds the external address (handle) and type (a symbol) 
            for the oci handle

        processedRows <Integer>
            returns the number of rows affected by the statement

        session <OCI8Session>
            holds the session which created the statement. Used to access some
            parameters of the session.

        state <Sumbol>
            returns the state of a statement. Possible values are:
                #prepared   first state of an statement. After generating it 
                            the state of it will be prepared
                #executing  state during the execution of a statement
                #executed   state after the execution of a statement
                #fetching   state during the fetch operation of a statement
                #fetched    state after the fetch operation of a statement

        text <String>
            the text of the statement

        textBuffer

    [class variables:]
"
!

history
    "Created: / 14.3.2000 / 07:57:01 / ps"
! !

!OCI8Statement class methodsFor:'instance creation'!

newForSession:aSession text:aStatementText
"
returns an instance of the correct subclass of OCI8Statement for the statement text.
Three diverent types of statements exist:
1. dd (data definition) statements
2. dm (data manipulation) statements
3. query statements

To generate such an instance a new statement handle is allocated and the statement
is prepared. Then the statement type can be queried by an oci call (OCIAttrGet).
The symbol returned by stmtType:statementHandle:errorOCIHandle: determines the
subclass of which an instance is created by statementFor:.
At least set the parameters of the statement and register it for finalization.

<return: OCI8QueryStatement OCI8SelectStatement | OCI8InsertStatement >
"

    |statementHandle statementType aStatement aStatementTextBuffer|

    statementHandle := self newStatementHandleFor:aSession environmentHandle ociLib:aSession ociLib.
    aStatementTextBuffer := aSession ociLib bufferForSize:aStatementText size + 1.
    aStatementTextBuffer stringAt:1 put:aStatementText.
    [
        aSession ociLib ociStatementPrepare:statementHandle
                                 textBuffer:aStatementTextBuffer
                                   language:self defaultLanguage
                                       mode:self defaultMode
                             errorOCIHandle:aSession errorHandle.
    ] ifCurtailed:[
        aStatementTextBuffer free.
        aStatementTextBuffer := nil.
    ].
    statementType:= self stmtType:aSession ociLib statementHandle:statementHandle errorOCIHandle:aSession errorHandle.
    aStatement := self statementFor:statementType.
    aSession registerNewStatement:aStatement.
    aStatement handle:statementHandle.
    aStatement text:aStatementText.
    aStatement textBuffer:aStatementTextBuffer.
    aStatement session:aSession.
    aStatement setPrepared.
    aStatement initialize.
    aStatement registerForFinalization.
    ^ aStatement.
!

statementFor:aStatementTypeSymbol
"
returns an statement instance for a spezific symbol. There are three different
statement classes: data manipulation, data definition and query statements.
One of the three classes is instanciated for the type symbol.

<return: OCI8QueryStatement | OCI8DMLStatement | OCI8DDLStatement>
"

    aStatementTypeSymbol == #'OCI_STMT_SELECT' ifTrue:[
        "query statement"
        ^ OCI8QueryStatement new
    ].
    (aStatementTypeSymbol == #'OCI_STMT_DECLARE' or:[
     aStatementTypeSymbol == #'OCI_STMT_INSERT' or:[
     aStatementTypeSymbol == #'OCI_STMT_UPDATE' or:[
     aStatementTypeSymbol == #'OCI_STMT_DELETE' or:[
     aStatementTypeSymbol == #'OCI_STMT_BEGIN' or:[
     aStatementTypeSymbol == #'OCI_STMT_INSERT']]]]]) ifTrue:[
        "DM (data manipulation) statement"
        ^ OCI8DMStatement new
    ].
    (aStatementTypeSymbol == #'OCI_STMT_CREATE' or:[
     aStatementTypeSymbol == #'OCI_STMT_ALTER' or:[
     aStatementTypeSymbol == #'OCI_STMT_DROP']]) ifTrue:[
        "DD (data definition) statement"
        ^ OCI8DDStatement new
    ].
    "allow other statements to be executed"
    "therefore they will be an instance of OCI8DDStatement"
    "for example truncate table <tablename>"
    (aStatementTypeSymbol == #'OCI_STMT_OTHER') ifTrue:[
        "other statement"
        ^ OCI8DDStatement new
    ].


    ^ OCIInternalError raiseErrorString:' - unknown statement type symbol: ', aStatementTypeSymbol printString
! !

!OCI8Statement class methodsFor:'constants'!

commitOnSuccessMode
"
<return: Symbol>
"
    ^#'OCI_COMMIT_ON_SUCCESS'

!

defaultLanguage
"
<return: Symbol>
"
    ^#'OCI_NTV_SYNTAX'


!

defaultMode
"
<return: Symbol>
"
    ^#'OCI_DEFAULT'

!

newStatementHandleFor:anOCI8EnvironmentHandle ociLib:anOCILib
"

<return: OCI8Handle>
"
    ^anOCILib ociHandleAlloc:anOCI8EnvironmentHandle type:#OCI_HTYPE_STMT





!

stmtType:anOCI8Lib statementHandle:anOCI8StatementHandle errorOCIHandle:anOCI8ErrorHandle
"
returns the type of the statement. The statement must be prepared before
calling this method.

Possible return values are
	#'OCI_STMT_SELECT'      SELECT statement
	#'OCI_STMT_UPDATE'      UPDATE statement
	#'OCI_STMT_DELETE'      DELETE statement
	#'OCI_STMT_INSERT'      INSERT statement
	#'OCI_STMT_CREATE'      CREATE statement
	#'OCI_STMT_DROP'        DROP statement
	#'OCI_STMT_ALTER'       ALTER statement
	#'OCI_STMT_BEGIN'       BEGIN PL/SQL statement
	#'OCI_STMT_DECLARE'     DECLARE PL/SQL statement

<return: Symbol>
"

    |statementType|

    statementType := anOCI8Lib ociAttrGet:anOCI8StatementHandle
			    attributeType:#'OCI_ATTR_STMT_TYPE'
			   errorOCIHandle:anOCI8ErrorHandle.

    ^anOCI8Lib class symbolForNumericStatementType:statementType


! !

!OCI8Statement class methodsFor:'examples'!

examples_dateTest1
"
insert a date
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table datetest'.
        createTableStatement := session newStatementFor:'create table datetest (d date)'.
        insertStatement := session newStatementFor:'insert into datetest values (:dat)'.
        selectStatement := session newStatementFor:'select * from datetest'.
        insertStatement dateBindVariable:#dat.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        session executeStatement:insertStatement from:(Array with:Timestamp now).
        resultStream := WriteStream on:Array new.
        session executeStatement:selectStatement into:resultStream.
        (resultStream contents first first) inspect.
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]
"
!

examples_numberTest1
"
Example 1:
insert and select positive fixed points;
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table numbertest'.
        createTableStatement := session newStatementFor:'create table numbertest (num number(12,5))'.
        insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
        selectStatement := session newStatementFor:'select num from numbertest where rownum = 1'.
        deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
        insertStatement numberBindVariable:#num.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        number := (9999531/10) asFixedPoint.
        [   |result|

            session executeStatement:insertStatement from:(Array with:number).
            resultStream := WriteStream on:Array new.
            session executeStatement:selectStatement into:resultStream.
            result := resultStream contents first first.
            Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
            session executeStatement:deleteStatement.
            number := number + (0.2 asFixedPoint).
        ] doWhile:[number <= ((9999531/10)+100)].
    ] ensure:[
        session disconnect.
    ].
                                                                [exEnd]

Example 2:
insert and select negative fixed points;
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table numbertest'.
        createTableStatement := session newStatementFor:'create table numbertest (num number(12,5))'.
        insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
        selectStatement := session newStatementFor:'select num from numbertest where rownum = 1'.
        deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
        insertStatement numberBindVariable:#num.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        number := (-99997/10) asFixedPoint.
        [   |result|

            session executeStatement:insertStatement from:(Array with:number).
            resultStream := WriteStream on:Array new.
            session executeStatement:selectStatement into:resultStream.
            result := resultStream contents first first.
            Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
            session executeStatement:deleteStatement.
            number := number + (0.2 asFixedPoint).
        ]doWhile:[number <= ((-99997/10)+100)].
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]

"
!

examples_numberTest2
"
Example 2:
insert and select positive floats
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table numbertest'.
        createTableStatement := session newStatementFor:'create table numbertest (num float)'.
        insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
        selectStatement := session newStatementFor:'select num from numbertest'.
        deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
        insertStatement numberBindVariable:#num.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        number := 5.9876.

        [   |result|

            session executeStatement:insertStatement from:(Array with:number).
            resultStream := WriteStream on:Array new.
            session executeStatement:selectStatement into:resultStream.
            result := resultStream contents first first.
            Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
            session executeStatement:deleteStatement.
            number := number + 0.02.
        ]doWhile:[number <= (5.9876+10.0)].
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]


Example 2:
insert and select positive floats;
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.

    dropTableStatement := session newStatementFor:'drop table numbertest'.
    createTableStatement := session newStatementFor:'create table numbertest (num float)'.
    insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
    selectStatement := session newStatementFor:'select num from numbertest where rownum = 1'.
    deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
    insertStatement numberBindVariable:#num.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    OCIErrorSignal handle:[:ex |
    ] do:[
        session executeStatement:dropTableStatement.
    ].
    session executeStatement:createTableStatement.

    number := 999.0000.

    [   |result|

        session executeStatement:insertStatement from:(Array with:number).
        resultStream := WriteStream on:Array new.
        session executeStatement:selectStatement into:resultStream.
        result := resultStream contents first first.
        Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
        session executeStatement:deleteStatement.
        number := number + 0.001.
    ]doWhile:[number <= (999.0000+1.0)].

    session disconnect.
                                                                [exEnd]
"
!

examples_numberTest3
"
Example 1:
insert and select positve integers
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table numbertest'.
        createTableStatement := session newStatementFor:'create table numbertest (num number)'.
        insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
        selectStatement := session newStatementFor:'select num from numbertest where rownum = 1'.
        deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
        insertStatement numberBindVariable:#num.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        number := 99999999999999.

        [   |result|

            session executeStatement:insertStatement from:(Array with:number).
            resultStream := WriteStream on:Array new.
            session executeStatement:selectStatement into:resultStream.
            result := resultStream contents first first.
            Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
            session executeStatement:deleteStatement.
            number := number + 1.
        ]doWhile:[number <= (99999999999999+1000)].
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]


Example 2:
insert and select negative integers
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement number resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table numbertest'.
        createTableStatement := session newStatementFor:'create table numbertest (num number)'.
        insertStatement := session newStatementFor:'insert into numbertest values (:num)'.
        selectStatement := session newStatementFor:'select num from numbertest where rownum = 1'.
        deleteStatement := session newStatementFor:'delete from numbertest where rownum = 1'.
        insertStatement numberBindVariable:#num.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        number := -99999999999999.

        [   |result|

            session executeStatement:insertStatement from:(Array with:number).
            resultStream := WriteStream on:Array new.
            session executeStatement:selectStatement into:resultStream.
            result := resultStream contents first first.
            Transcript showCR:'insert number: ',number printString,' selected number: ',result printString.
            session executeStatement:deleteStatement.
            number := number + 1.
        ]doWhile:[number <= (-99999999999999+1000)].
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]
"
!

examples_stringTest1
"
Example 1:
insert and select positve integers
                                                                [exBegin]
    |session dropTableStatement createTableStatement insertStatement selectStatement
     deleteStatement longString resultStream|

    session := OCI8Session default.
    [
        dropTableStatement := session newStatementFor:'drop table stringtest'.
        createTableStatement := session newStatementFor:'create table stringtest (string clob)'.
        insertStatement := session newStatementFor:'insert into stringtest values (:string)'.
        selectStatement := session newStatementFor:'select string from stringtest where rownum = 1'.
        deleteStatement := session newStatementFor:'delete from stringtest where rownum = 1'.
        insertStatement stringBindVariable:#string.

        session connectWithUsername:'scott' password:'tiger' dbname:'test'.

        OCIErrorSignal handle:[:ex |
        ] do:[
            session executeStatement:dropTableStatement.
        ].
        session executeStatement:createTableStatement.

        longString := (1 to: 10000) inject:'' into:[:accu :el| accu , el printString].

        session executeStatement:insertStatement from:(Array with:longString).
        resultStream := WriteStream on:Array new.
        session executeStatement:selectStatement into:resultStream.
        (resultStream contents first first) inspect.
    ] ensure:[
        session disconnect.
    ]
                                                                [exEnd]
"
! !

!OCI8Statement methodsFor:'accessing'!

abortState
"
returns true, if the receiver should be aborted (e.g. to cancel a long running statement),
false if not.

<return: Boolean>
"
    abortState isNil ifTrue:[
        abortState := false.
    ].
    ^ abortState
!

abortState:aBoolean
"
set to true if the receiver should be aborted (e.g. to cancel a long running statement).

<return: self>
"
    abortState := aBoolean.
!

bindDescriptors
"
returns the descriptors which describes the bind variabels in the receivers text
(see OCI8BindDescriptor).

<return: Array of:OCI8BindDescriptor|nil>
"
    ^bindDescriptors
!

bindDescriptors:anArray
"
sets the descriptors which describes the bind variabels in the receivers text
(see OCI8BindDescriptor).

<return: self>
"

    bindDescriptors := anArray.
!

bindNames
"
returns a collection into which dictionaries (name -> an OCI8Converter) are added.
Those dictionaries describe the information needed to generate bind variables.

<return: OrderedCollection>
"
    bindNames isNil ifTrue:[
        bindNames := OrderedCollection new.
    ].
    ^bindNames
!

bindVariables
"
returns the collection of bind variables for the receiver. These are
automatical generated by the describtions out of the instance variable
bindNames.

<return: OrderedCollection>
"
    bindVariables isNil ifTrue:[
        bindVariables := OrderedCollection new
    ].
    ^bindVariables

!

binded
"
returns true if the statement is binded, false if not

<return: aBoolean>
"
    binded isNil ifTrue:[
        binded := false.
    ].
    ^ binded
!

binded:aBoolean
"
sets if the statement is binded

<return: self>
"
    binded := aBoolean.
!

errorHandle
"
returns the error handle from the session. The handle is used
to get specific informations when an error occured.

<return: OCI8Handle>
"
    ^self session errorHandle
!

executeMode
"
returns the mode in which the statement is executed. When not specified
the mode is OCI_DEFAULT.

<return: Symbol>
"   
    executeMode isNil ifTrue:[
        executeMode := self defaultMode
    ].
    ^executeMode


!

handle
"
returns the handle of the receiver.

<return: OCI8Handle>
"
    ^ handle
!

handle:anOCI8Handle
"
sets the handle of the receiver.

<return: self>
"

    handle := anOCI8Handle.
!

language
"
returns the default language of the receiver.

<return: Symbol>
"
    ^self defaultLanguage
!

ociLib
"
returns the oci lib from the session. The lib is used to call
the oci methods for the receiver.

<return: OCI8Lib>
"
    ^ session ociLib
!

prepareMode
"
returns the default prepare mode of the receiver.

<return: Symbol>
"
    ^self defaultPrepareMode
!

processedRows:anInteger
"
sets how many rows were affected by the receiver. If the receiver
was cancelled during execution or no row was affected 0 is returned.

<return: self>
"

    processedRows := anInteger.
!

rowOffset
"
returns the starting index from which the data in an array bind is relevant
for this multiple row execution. Returning 0 inserts elements starting with
the first one.

<return: Integer>
"
    ^0


!

serviceHandle
"
returns the service handle from the session.

<return: OCI8Handle>
"
    ^self session serviceHandle
!

session
"
returns the session which generates the receiver.

<return: OCI8Session>
"

    ^ session
!

session:anOCI8Session
"
sets the session which generates the receiver.

<return: self>
"

    session := anOCI8Session.
!

state
"
returns the state of the receiver. Possible values are:

    #prepared   first state of an statement. After generating it 
                the state of it will be prepared
    #executing  state during the execution of a statement
    #executed   state after the execution of a statement
    #fetching   state during the fetch operation of a statement
    #fetched    state after the fetch operation of a statement

<return: Symbol>
"
    ^ state
!

textBuffer
"
returns the buffer for the statements text. Is used to provide
a fix text buffer for the oci lib.

<return: ExternalBytes>
"

    ^ textBuffer
!

textBuffer:anExternalBytes
"
sets the buffer for the statements text. Is used to provide
a fix text buffer for the oci lib.

<return: self>
"
    textBuffer := anExternalBytes.
! !

!OCI8Statement methodsFor:'accessing variables'!

bindNameFor:aSymbol
"
returns a association (name -> an OCI8Converter) for the name aStringOrSymbol.

<return: Association|nil>
"
    ^self bindNames detect:[:eachBindNameAssoc |
        eachBindNameAssoc key = aSymbol
    ] ifNone:nil.

! !

!OCI8Statement methodsFor:'binding'!

bindNameAtPosition:anIndex
"
returns the name of the variable from the statement text for the
index anIndex.

<return: Symbol>
"
    |bindName|

    bindName := self bindDescriptors at:anIndex ifAbsent:nil.
    bindName isNil ifTrue:[
        OCIErrorSignal raiseErrorString:' - non existant bind variable index ', anIndex printString,'. Check statement text.'.
    ] ifFalse:[
        ^ bindName name
    ]
!

checkBindVariablesForSameSize
"
Checks if in the receivers bind variables have all the same number of 
elements defined for checkin. If not raise an error.

<return: self>
"

    ((self bindVariables collect:[:eachBindVar |
        eachBindVar numberOfElements
    ]) asSet size > 1) ifTrue:[
        OCIErrorSignal raiseErrorString:' - different number of values in bind variables'
    ].
!

computeBindDescriptors
"
compute the bind descriptors of the receiver.

<return: self>
"
    self bindDescriptors:(self ociLib ociStmtGetBindInfo:self handle errorOCIHandle:self errorHandle)

!

doBindingVariables
"
setup each bind variable and bind it to the receiver

<return: self>
"
    bindVariables notNil ifTrue:[
        bindVariables do:[:eachBindVar |
            eachBindVar setupForStatement:self.
        ].
    ].
!

indexForBindName:aSymbol
"
returns the index for a bind name in the statement text.
Raises an error if the name isnt in the bind desciptors.

<return: Integer>
"
    |i|

    i := 1.
    self bindDescriptors do:[:each|
        each name = aSymbol ifTrue:[
            ^ i
        ].            
        i := i+1.
    ].

    OCIErrorSignal raiseErrorString:' - unbound variables. Check statement text.'.
    ^ 0
!

removeAllBindVariables
"
removes all bindvariables of the receiver. This happens on each reallocation 
of bind variables of the receiver.

<return: self>
"
    self bindVariables removeAll
! !

!OCI8Statement methodsFor:'computation'!

computeProcessedRows
    "compute the number of rows affected by the receiver

     <return: Integer>"

    ^ processedRows := self getProcessedRows.
!

getProcessedRows
    "answer the number of rows fetched from the receiver
     <return: Integer>"

    ^ self ociLib ociAttrGet:self handle
               attributeType:#'OCI_ATTR_ROW_COUNT'
              errorOCIHandle:self errorHandle.
!

processedRows
"
returns how many rows were affected by the receiver. If the receiver
was cancelled during execution or no row was affected 0 is returned.

<return: Integer>
"
    processedRows isNil ifTrue:[
        processedRows := 0.
    ].
    ^ processedRows
! !

!OCI8Statement methodsFor:'constants'!

commitOnSuccessMode
"
returns the symbol for commit on success. In mode #'OCI_COMMIT_ON_SUCCESS'
the receiver is commited after successful execution.

<return: Symbol>
"
    ^self class commitOnSuccessMode
!

defaultLanguage
"
returns the symbol #'OCI_NTV_SYNTAX' for the receivers syntax definition.
With #'OCI_NTV_SYNTAX' the syntax depends upon the version of the server.

<return: Symbol>
"
    ^self class defaultLanguage
!

defaultMode
"
returns the mode in which this statement is executed when nothing else is specified.
For all statements this is OCI_DEFAULT.

<return: Symbol>
"
    ^self class defaultMode
!

defaultPrepareMode
"
returns the symbol #'OCI_DEFAULT' for the default mode
of statement preparing.

<return: Symbol>
"
    ^self defaultMode
! !

!OCI8Statement methodsFor:'execution'!

execute
    "call the oci execute method for the receiver. If no data is
     available for the receiver the dataAvailable is set to false else true.
     Used to know if a fetch is necessary.

     <return: Boolean>"

    |status|

    abortState := false.
    status := self ociLib ociStatementExecute:self
                   serviceOCIHandle:self serviceHandle
                     errorOCIHandle:self errorHandle
                               mode:self executeMode.

    status ~~ #'OCI_SUCCESS' ifTrue:[ |exception|
        status == #'OCI_NO_DATA' ifTrue:[
            ^ false.    "there is no data available"
        ].
        exception :=
            self ociLib createExceptionFor:status errorHandle:self errorHandle.
        exception statement:self.
        exception raiseRequest.
    ].
    ^ true
! !

!OCI8Statement methodsFor:'generate bind-variable'!

addBindName:aStringOrSymbol converter:anOCI8Converter
"
adds a new description (name and converter) for a variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"

    self bindNames add:(aStringOrSymbol asUppercase asSymbol -> anOCI8Converter)
!

byteBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'byte' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8ByteArrayConverter new
!

dateBindVariable:aStringOrSymbol
    "adds a new description (name and converter) for a date variable with name
     aStringOrSymbol in the statement text to the receiver.
     From this descriptions the OCI8BindVariables are generated.

     <return: self>"

    self addBindName:aStringOrSymbol converter:OCI8DateConverter new
!

fixedPointBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'fixed point' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8NumberConverter new
!

floatBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'float' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8FloatConverter new
!

integerBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'integer' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8NumberConverter new
!

numberBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'number' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8NumberConverter new
!

stringBindVariable:aStringOrSymbol
"
adds a new description (name and converter) for a 'string' variable with name
aStringOrSymbol in the statement text to the receiver.
From this descriptions the OCI8BindVariables are generated.

<return: self>
"
    self addBindName:aStringOrSymbol converter:OCI8StringConverter new
! !

!OCI8Statement methodsFor:'private accessing'!

state:aSymbol
"
returns the state of the receiver. Possible values are:

    #prepared   first state of an statement. After generating it 
                the state of it will be prepared
    #executing  state during the execution of a statement
    #executed   state after the execution of a statement
    #fetching   state during the fetch operation of a statement
    #fetched    state after the fetch operation of a statement

<return: self>
"

    state := aSymbol.
!

text
"
returns the statements text

<return: aString>
"
    ^ text
!

text:aString
"
sets the statements text

<return: self>
"
    text := aString.
! !

!OCI8Statement methodsFor:'releasing'!

finalize
    "this is invoked when an instance of myself has been garbage collected.
     Release the handle of the receiver. This will auto free any bind
     and/or define pointers. Release the text buffer.

     <return: self>"

    handle notNil ifTrue:[
        self ociLib ociHandleFree:handle.
        handle := nil.
    ].
    textBuffer notNil ifTrue:[
        textBuffer free.
        textBuffer := nil.
    ].

    "Modified: / 16.2.2000 / 13:17:00 / ps"
!

invalidateHandles
    "
        this is invoked when my handle is no longer valid.

        <return: self>
    "

    handle := nil.
    self reRegisterForFinalization.
!

release
    "release the transaction"

    self unregisterForFinalization.
    self finalize.
    session notNil ifTrue:[
        session unregisterStatement:self.
    ].
! !

!OCI8Statement methodsFor:'set states'!

setAborted
"
sets the statements abort state to true. The statement will be aborted if
not finished execution.

<return: self>
"
    abortState := true
!

setExecuted
"
sets the statements state executed.

<return: self>
"
    self state:#executed
!

setExecuting
"
sets the statements state executing.

<return: self>
"
    self state:#executing
!

setFetched
"
sets the statements state fetched.

<return: self>
"
    self state:#fetched
!

setFetching
"
sets the statements state fetching.

<return: self>
"
    self state:#fetching
!

setPrepared
"
sets the statements state prepared.

<return: self>
"
    self state:#prepared
! !

!OCI8Statement methodsFor:'subclass responsibility'!

iterator
"
returns the number of many times a statement must be executed.
A data manipulation statement must be executed at least once. 
If bind variables exists the statement must be executed n times.
Where n is the number of rows which should be insert.

<return: Integer>
"
    self subclassResponsibility
! !

!OCI8Statement methodsFor:'testing'!

isAborted
"
returns true if the statements abort state is set

<return: Boolean>
"

    ^ abortState
!

isBinded
"
returns, if the receiver (a statement) is binded. This test is
used by the database connection lib to check if a low level
bind is already done for the receiver.

<return: Boolean>
"

    ^ binded
!

isDDLStatement
"
returns if the statement is a data definition statement

<return: Boolean>
"
    ^false




!

isDMLStatement
"
returns if the statement is a data manipulation statement

<return: Boolean>
"
    ^false


!

isExecuted
"
returns true if the statement is in an executing operation

<return: Boolean>
"

    ^state == #executed
!

isExecuting
"
returns true if the statement is in an executing operation

<return: Boolean>
"

    ^state == #executing
!

isFetched
"
returns true if the statement fetching operation is finished

<return: Boolean>
"

    ^state == #fetched
!

isQueryStatement
"
returns if the statement is a query statement

<return: Boolean>
"
    ^false


!

parseErrorOffset
    "answer the offset of the parse error in the statement text"

    ^ session ociLib ociAttrGet:handle
                      attributeType:#'OCI_ATTR_PARSE_ERROR_OFFSET'
                      errorOCIHandle:self errorHandle.
!

shouldAbort
"
returns true if the statement should be aborted

<return: Boolean>
"
    ^ abortState
!

statementType
    "
     returns the type of the statement. The statement must be prepared before
     calling this method.

Possible return values are
        #'OCI_STMT_SELECT'      SELECT statement
        #'OCI_STMT_UPDATE'      UPDATE statement
        #'OCI_STMT_DELETE'      DELETE statement
        #'OCI_STMT_INSERT'      INSERT statement
        #'OCI_STMT_CREATE'      CREATE statement
        #'OCI_STMT_DROP'        DROP statement
        #'OCI_STMT_ALTER'       ALTER statement
        #'OCI_STMT_BEGIN'       BEGIN PL/SQL statement
        #'OCI_STMT_DECLARE'     DECLARE PL/SQL statement

<return: Symbol>
"

    |statementType|

    statementType := self ociLib ociAttrGet:handle
                            attributeType:#'OCI_ATTR_STMT_TYPE'
                           errorOCIHandle:self errorHandle.

    ^ OCI8Lib symbolForNumericStatementType:statementType
! !

!OCI8Statement class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Statement.st,v 1.68 2006/09/11 09:19:08 james Exp $'
! !
