"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Transaction
	instanceVariableNames:'flags timeout handle session'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Transactions'
!

!OCI8Transaction class methodsFor:'documentation'!

documentation
"
    To create a transaction:
        OCI8Session>>newReadOnlyTransaction
        OCI8Session>>newReadOnlyTransactionWithTimeOut:
        OCI8Session>>newReadWriteTransaction
        OCI8Session>>newReadWriteTransactionWithTimeOut:

    If a transaction is created with OCI8Session>>newReadOnlyTransaction or
    OCI8Session>>newReadWriteTransaction then the transaction will be rollback
    immeditially if it is detached because the time out integer of the transaction
    is set to 0.  To create a transaction which is detach- and resumeable call
    OCI8Session>>newReadOnlyTransactionWithTimeOut: or OCI8Session>>newReadWriteTransactionWithTimeOut:.

    To commit or rollback a transaction:
        OCI8Transaction>>commit
        OCI8Transaction>>commitOnePhase
        OCI8Transaction>>rollback

    To detach or resume a transaction:
        OCI8Transaction>>detach
        OCI8Transaction>>resume:

    To prepare a transaction:
        OCI8Transaction>>prepare

    [author:]
        Pierre Schwarz

    [see also:]
        OCI8DDStatement
        OCI8DMStatement
        OCI8QueryStatement

    [instance variables:]

        flags <Collection>
            a collection with the symbols representing the mode
            of a transaction.

        handle <OCI8Handle>
            holds the external address (handle) and type (a symbol) 
            for the oci handle

        session <OCI8Session>
            holds the session which created the transaction. Used to access some
            parameters of the session.

    [class variables:]
        none
"
!

history
    "Created: / 28.8.2000 / 09:26:27 / ps"
! !

!OCI8Transaction class methodsFor:'instance creation'!

newFor:anOCI8Session timeOut:aTimeOutInteger flags:aFlagCollection
"        
begins a new transaction for the session. The timeout parameter aTimeOutInteger indicates
the number of seconds the transaction can be inactive before it is automatically aborted by the system.
A transaction is inactive between the time it is detached and the time it is resumed.

<return: OCI8Transaction>
"

    ^ self new session:anOCI8Session timeout:aTimeOutInteger flags:aFlagCollection




!

newReadOnlyFor:anOCI8Session
"        
starts a new read only transaction for the session. If the transaction becomes
detached it will be rolled back immeditally because 0 seconds are set for the
transaction timeout.

<return: OCI8Transaction>
"

    ^self newReadOnlyFor:anOCI8Session timeOut:0




!

newReadOnlyFor:anOCI8Session timeOut:aTimeOutInteger
"        
starts a new read only transaction for the session. The timeout parameter aTimeOutInteger indicates
the number of seconds the transaction can be inactive before it is automatically aborted by the system.
A transaction is inactive between the time it is detached and the time it is resumed.

<return: OCI8Transaction>
"
    ^self newFor:anOCI8Session timeOut:aTimeOutInteger flags:self readOnlyFlagCollection 




!

newReadWriteFor:anOCI8Session
"        
starts a new read and writeable transaction for the session. If the transaction becomes
detached it will be rolled back immeditally because 0 seconds are set for the
transaction timeout.

<return: OCI8Transaction>
"

    ^self newReadWriteFor:anOCI8Session timeOut:0




!

newReadWriteFor:anOCI8Session timeOut:aTimeOutInteger
"        
starts a new read and writeable transaction for the session. The timeout parameter aTimeOutInteger indicates
the number of seconds the transaction can be inactive before it is automatically aborted by the system.
A transaction is inactive between the time it is detached and the time it is resumed.

<return: OCI8Transaction>
"
    ^self newFor:anOCI8Session timeOut:aTimeOutInteger flags:self readWriteFlagCollection




! !

!OCI8Transaction class methodsFor:'constants'!

readOnlyFlagCollection
"
returns a collection with the symbols representing the flags of
a read only transaction.

<return: Collection of:<Symbol>>
"

    ^#(#'OCI_TRANS_READONLY')
!

readWriteFlagCollection
"
returns a collection with the symbols representing the flags of
a writeable transaction.

<return: Collection of:<Symbol>>
"

    ^#(#'OCI_TRANS_NEW')
! !

!OCI8Transaction class methodsFor:'examples'!

example_createTable
"
create the table transactiontest for the examples of this class.
                                                                [exBegin]
    |session transaction dropTableStatement createTableStatement|

    session := OCI8Session default.
    dropTableStatement := session newStatementFor:'drop table transactiontest'.
    createTableStatement := session newStatementFor:'create table transactiontest (testnumber integer)'.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.
    transaction := session newReadWriteTransaction.

    OCIErrorSignal handle:[:ex |
        OCIErrorSignal handle:[:ex |
        ] do:[
            transaction rollback.
        ].
    ] do:[
        session executeStatement:dropTableStatement.
    ].

    OCIErrorSignal handle:[:ex |
        OCIErrorSignal handle:[:ex |
        ] do:[
            transaction rollback.
        ].
    ] do:[
        session executeStatement:createTableStatement.
    ].

    session disconnect.
                                                                [exEnd]
"


!

example_transactionTest1
"
insert integers and rollback each insert, if integer is odd

                                                                [exBegin]
    |session truncateStatement insertStatement insertStream transaction|

    session := OCI8Session default.
    truncateStatement := session newStatementFor:'truncate table transactiontest'.
    insertStatement := session newStatementFor:'insert into transactiontest values (:testnumber)'.
    insertStatement integerBindVariable:#testnumber.

    session connectWithUsername:'scott' password:'tiger' dbname:'test'.

    session executeStatement:truncateStatement.

    12 to: 22 do:[:eachNumber|
        transaction := session newReadWriteTransaction.
        session executeStatement:insertStatement from:(Array with:eachNumber).
        eachNumber odd ifTrue:[
            transaction rollback.
        ] ifFalse:[
            transaction commit.
        ]
    ].
    session disconnect.
                                                                [exEnd]

"

! !

!OCI8Transaction methodsFor:'accessing'!

handle
"
returns the handle of the receiver.

<return: OCI8Handle>
"

    ^ handle
!

handle:anOCI8Handle
"
sets the handle of the receiver.

<return: self>
"

    handle := anOCI8Handle.
!

session
"
returns the session which generates the receiver.

<return: OCI8Session>
"

    ^ session
!

session:anOCI8Session
"
sets the session which generates the receiver.

<return: self>
"

    session := anOCI8Session.
!

timeout
    "return the value of the instance variable 'timeout' (automatically generated)"

    ^ timeout
!

timeout:something
    "set the value of the instance variable 'timeout' (automatically generated)"

    timeout := something.
! !

!OCI8Transaction methodsFor:'accessing handles'!

errorHandle
"
returns the error handler from the session. The handle is used
to get specific informations when an error occured.

<return: OCI8Handle>
"
    ^session errorHandle



!

ociLib
"
returns the oci lib from the session. The lib is used to call
the oci methods for the receiver.

<return: OCI8Lib>
"
    ^session ociLib



!

serviceHandle
"
returns the service handle from the session.

<return: OCI8Handle>
"
    ^session serviceHandle



! !

!OCI8Transaction methodsFor:'commit'!

commit
"
one phase commit of the receiver.

<return: self>
"
    self commitOnePhase



!

commitOnePhase
"
one phase commit of the receiver.

<return: self>
"
    self ociLib ociTransactionCommit:self serviceHandle
				flag:#'OCI_DEFAULT'
		      errorOCIHandle:self errorHandle



!

commitTwoPhase
"
two phase commit of the receiver. Only needed if the oci application
manageing global transactions.

<return: self>
"
    self ociLib ociTransactionCommit:self serviceHandle
				flag:#'OCI_TRANS_TWOPHASE'
		      errorOCIHandle:self errorHandle



! !

!OCI8Transaction methodsFor:'detach'!

detach
"
detache of the receiver.

<return: self>
"
    self ociLib ociTransactionDetach:self serviceHandle
		      errorOCIHandle:self errorHandle




! !

!OCI8Transaction methodsFor:'finalization'!

finalize
"
when an instance of myself is finalized release the handle of the receiver
and remove the receiver from the registry.

<return: self>
"
    |tmpHandle|

    (handle notNil and:[handle isValid and:[session notNil]]) ifTrue:[
        tmpHandle := handle.
        handle := nil.
        self ociLib ociHandleFree:tmpHandle
    ].
!

invalidateHandles
    "
        this is invoked when my handle is no longer valid.

        <return: self>
    "

    handle := nil.
    self reRegisterForFinalization.

! !

!OCI8Transaction methodsFor:'initialization'!

session:anOCI8Session timeout:aTimeOutInteger flags:aFlagCollection
"        
begins a new transaction for the session. The timeout parameter aTimeOutInteger indicates
the number of seconds the transaction can be inactive before it is automatically aborted by the system.
A transaction is inactive between the time it is detached and the time it is resumed.

<return: OCI8Transaction>
"

    "new transaction handle"
    anOCI8Session registerNewTransaction:self.
    handle  := anOCI8Session ociLib ociHandleAlloc:anOCI8Session environmentHandle type:#OCI_HTYPE_TRANS.
    session := anOCI8Session.
    flags   := aFlagCollection.
    timeout := aTimeOutInteger.

    self registerForFinalization.
    self startTransaction.



!

startTransaction
    "start a new transaction.
     The prvious transaction (if one exists) is committed"

    |ociLib|

    ociLib := session ociLib.

    "when starting a new transaction the active one is commited"
    "this commits also the implicit transaction"
    ociLib ociTransactionCommit:session serviceHandle
                           flag:#'OCI_DEFAULT'
                 errorOCIHandle:session errorHandle.

    "set the transaction attribute of the service handle
     to the new transaction handle"
    ociLib ociAttrSet:session serviceHandle
            attribute:handle handle 
        attributeType:#'OCI_ATTR_TRANS' 
       errorOCIHandle:session errorHandle.

    "set the beginning of the new transaction"
    ociLib ociTransactionStart:session serviceHandle
                          time:timeout
                         flags:flags
                errorOCIHandle:session errorHandle.

! !

!OCI8Transaction methodsFor:'prepare'!

prepare
"
prepare of the receiver.

<return: self>
"
    self ociLib ociTransactionPrepare:self serviceHandle
		       errorOCIHandle:self errorHandle





! !

!OCI8Transaction methodsFor:'private accessing'!

flags
"
returns the collection with the symbols representing the mode
of the receiver.

<return: Collection>
"

    ^ flags
!

flags:aCollection
"
sets the collection with the symbols representing the mode
of the receiver.

<return: self>
"

    flags := aCollection.
! !

!OCI8Transaction methodsFor:'resume'!

resume:aTimeInteger
"
starts a new read only transaction for the session. The time aTimeInteger, in seconds,
is the time to wait for a transaction to become available for resumption.

<return: self>
"
    self ociLib ociTransactionStart:self serviceHandle
			       time:aTimeInteger
			      flags:#(#'OCI_TRANS_RESUME')
		     errorOCIHandle:self errorHandle


! !

!OCI8Transaction methodsFor:'rollback'!

rollback
"
rollback of the receiver.

<return: self>
"
    self ociLib ociTransactionRollback:self serviceHandle
			errorOCIHandle:self errorHandle





! !

!OCI8Transaction class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Transaction.st,v 1.15 2002/03/04 14:36:02 stefan Exp $'
! !
