"{ Package: 'stx:liboracle8' }"

Object subclass:#OCI8Variable
	instanceVariableNames:'handle converter dataBuffer elementLengthArray indicatorArray'
	classVariableNames:''
	poolDictionaries:''
	category:'DataBase-Oracle8-Variables'
!

!OCI8Variable class methodsFor:'documentation'!

documentation
"
    Subclasses of this abstract class are OCI8BindVariable and OCI8DefineVariable.
    OCI8BindVariable defines variables from which values are insert into the database.
    OCI8DefineVariable are variables which are filled with the values fron the database.
    Common behaviour for the two subclasses is implemented here.

    [author:]
	Pierre Schwarz

    [see also:]
	OCI8BindVariable
	OCI8DefaultDefineVariable
	OCI8DefineVariable
	OCI8DynamicDefineVariable
	OCI8MultipleValueBindVariable
	OCI8ReturnValueBindVariable
	OCI8SingleValueBindVariable

    [instance variables:]

	converter <OCI8Converter>
	    The external c type of the converter determines the conversion of the values
	    in the data buffer. For bind variables the converted values represented by the
	    bind varaibles are copied into the databuffer. For define variables the values in
	    the datas buffer are converted into the correct smalltalk type. The subclasses
	    of OCI8Converter determine the conversion parameters.

	dataBuffer <ExternalBytes|ByteArray|nil>
	    for bind variables the values in the data are insert into the
	    database. For define variables they represent the returned values.
	    Nil values are handled via the indicator array.

	elementLengthArray <ExternalBytes|nil>
	    the values represents the lenght for the corresponding value in the
	    data buffer.

	handle <OCI8Handle>
	    holds the external address (handle) and type (a symbol) 
	    for the oci handle

	indicatorArray <ExternalBytes|nil>
	    the values in this buffer indicate if the corresponding value in the
	    data buffer is a null value. Null values are indicated with an -1.
	    Not null values with 0.

	returnCodeArray <ExternalBytes|nil>
	    the values represents the return codes for the corresponding value in the
	    data buffer. For exapmle an error code is set here, if a values has been
	    truncated.

    [class variables:]
	none
"
!

history
    "Created: / 25.8.2000 / 15:35:20 / ps"
! !

!OCI8Variable methodsFor:'accessing'!

converter
"
returns a converter needed for the bind operation of the receiver.

<return: OCI8ByteArrayConverter|OCI8DateConverter|OCI8FixedPointConverter|OCI8FloatConverter|OCI8IntegerConverter>
"

    ^ converter
!

converter:anOCI8Converter
"
sets a converter needed for the bind operation of the receiver.
Possible converters are:
    OCI8ByteArrayConverter
    OCI8DateConverter
    OCI8FixedPointConverter
    OCI8FloatConverter
    OCI8IntegerConverter

<return: self>
"

    converter := anOCI8Converter.
!

dataBuffer
"
returns the buffer where the values for the receiver are insert
from the oci c functions.

<return: ExternalBytes|ByteArray|nil>
"
    ^dataBuffer


!

dataBuffer:anExternalBytes
"
sets the buffer where the values for the receiver are insert
from the oci c functions.

<return: self>
"
    dataBuffer := anExternalBytes.
    self reRegisterForFinalization.


!

dataBufferSize
"
returns the size of the buffer

<return: Integer>
"
    ^ dataBuffer size
!

elementLengthArray
"
returns the buffer where the lenght of the values for the receiver are insert
from the oci c functions.

<return: ExternalBytes|nil>
"
    ^ elementLengthArray

!

elementLengthArray:anExternalBytesOrNil
"
sets the buffer where the lenght of the values for the receiver are insert
from the oci c functions.

<return: self>
"
    anExternalBytesOrNil notNil ifTrue:[
	anExternalBytesOrNil atAllPut:0.
    ].
    elementLengthArray := anExternalBytesOrNil.
    self reRegisterForFinalization.
!

externalCType
"
returns the symbol needed for the bind operation of the receiver.

<return: Symbol>
"
    ^ converter externalCType
!

handle
    "return the value of the instance variable 'handle' (automatically generated)"

    handle isNil ifTrue:[
	handle := OCI8Handle new type:self handleType
    ].
    ^ handle
!

indicatorArray
"
returns the buffer which indicates if its a null value for the receiver.
The values are insert from the oci c functions.

<return: ExternalBytes|nil>
"

    ^ indicatorArray
!

indicatorArray:anExternalBytesOrNil
"
sets the buffer which indicates if its a null value for the receiver.
The values are insert from the oci c functions.

<return: self>
"

    anExternalBytesOrNil notNil ifTrue:[
	anExternalBytesOrNil atAllPut:0.
    ].
    indicatorArray := anExternalBytesOrNil.
    self reRegisterForFinalization.
! !

!OCI8Variable methodsFor:'buffer accessing'!

getValue
    "fetch the data from the external bytes aBuffer.
     The data in aBuffer is stored there in the external c type format of the used converter. 
     The data is converted into the smalltalk format by the converter.

     <return: String|Integer|Float|ByteArray|nil>"

    |size|

    "a nil value is indicated in the indicator array"
    ((indicatorArray shortAt:1) == -1) ifTrue:[
        ^ nil
    ].
    size := elementLengthArray unsignedShortAt:1.
    ^ converter getDataFromBuffer:(dataBuffer referenceToBytesFrom:1 to:size) size:size.
! !

!OCI8Variable methodsFor:'finalization'!

finalize
    "Release the different external bytes"

    dataBuffer notNil ifTrue:[
	dataBuffer free.
	dataBuffer := nil
    ].
    indicatorArray notNil ifTrue:[
	indicatorArray free.
	indicatorArray := nil
    ].
    elementLengthArray notNil ifTrue:[
	elementLengthArray free.
	elementLengthArray := nil
    ]
    "do not free the handle here, it is freed implicitly by OCI lib when the
     statement handle is released"
! !

!OCI8Variable methodsFor:'testing'!

isArrayBind
"
returns true if the receiver hold more than one value

<return: Boolean>
"

    ^false
!

isReturnVariable
"
returns if the receiver is a bind variable which is used to return
values. Used in returning clauses of statements.

<return: Boolean>
"

    ^ false

! !

!OCI8Variable class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/liboracle8/OCI8Variable.st,v 1.30 2004/11/09 08:32:18 stefan Exp $'
! !
