/* 
 * Browser Plugin for Smalltalk\X on Unix platforms
 * 
 * Author:  eXept Software AG <bg@exept.de>
 */

#define XP_UNIX
#include <npapi.h>
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

/* config.h is in the top build dir of the VM */
#ifndef VM_LIBDIR
# include "config.h"
#endif

#undef DEBUG

#ifdef DEBUG
static void DPRINT(char *format, ...)
{
  static int debug= 42;
  if (42 == debug) 
    debug= (NULL != getenv("NPSQUEAK_DEBUG"));

  if (!debug) 
    {
      return;
    }
  else
    {
      static FILE *file= 0;
      if (!file) 
	{
	  file= fopen("/tmp/npsqueak.log", "a+");//here squeak, because i don't know, if there are any other references to this file
	  fprintf(file, "=== START PLUGIN ===\n");
	}

      {
	va_list ap;
	va_start(ap, format);
	vfprintf(file, format, ap);
	va_end(ap);
	fflush(file);
      }
    }
}
#else
static void DPRINT(char *format, ...) { }
#endif

/***********************************************************************
 * Plugin instance data
 ***********************************************************************/

#define CMD_BROWSER_WINDOW 1
#define CMD_GET_URL        2
#define CMD_POST_URL       3
#define CMD_RECEIVE_DATA   4

#define MAX_STREAMS 128

#define STX_READ  0
#define PLUGIN_WRITE 1
#define PLUGIN_READ  2
#define STX_WRITE 3

/* plugin state */
typedef struct StxPlugin {
  NPP instance;                    /* plugin instance */
  pid_t pid;                       /* the child process pid */
  Display *display;
  Window nswindow;                 /* the netscape window */
  Window stxwindow;                 /* the Stx window */
  XtInputId input;                 /* handler for command pipe */
  Bool embedded;                   /* false if we have the whole window */
  char **argv;                     /* the commandline for Stx vm */
  int  argc;
  char vmName[PATH_MAX];
  char imageName[PATH_MAX];
  int pipes[4];                    /* 4 ends of 2 pipes */
  char* srcUrl;                    /* set by browser in first NewStream */
  char* srcFilename;
  int   srcId;                     /* if requested */
} StxPlugin;

/* URL notify data */
typedef struct StxStream {
  int id;                          /* request id (0 if finished)  */
} StxStream;

/***********************************************************************
 * Prototypes
 ***********************************************************************/

static void DeliverFile(StxPlugin *, int id, const char* fname);
static void SetWindow(StxPlugin*, Window window, int width, int height);
static void SetUpWindow(StxPlugin*);
static void SetUpStxWindow(StxPlugin*);
static void Run(StxPlugin*);
static void GetUrl(StxPlugin*);
static void PostUrl(StxPlugin*);
static void DestroyCallback(Widget widget, StxPlugin *, XtPointer calldata);
static void InputCallback(StxPlugin *, int *source, XtInputId*);

static char* NPN_StrDup(const char* s)
{
  return strcpy(NPN_MemAlloc(strlen(s)+1), s);
}


/***********************************************************************
 * Plugin registration
 ***********************************************************************/

char*
NPP_GetMIMEDescription(void)
{
  return("application/x-SmalltalkX-source:sts:Smalltalk\\X source"
	 ";application/x-SmalltalkX-object:sqo:Smalltalk\\X object"
	 ";application/x-SmalltalkX-project:pr:Smalltalk\\X project");
}

NPError
NPP_GetValue(void *instance, NPPVariable variable, void *value)
{
  switch (variable) {
  case NPPVpluginNameString:
    *((char **)value)= "Smalltalk\\X";//here org only Squeak
    break;
  case NPPVpluginDescriptionString:
    *((char **)value) =
      "<a href=\"http://www.exept.de/\">Smalltalk\\X</a> is a modern"
      " Smalltalk environment.<P>"
      "The Smalltalk\\X Plugin for Unix was developed by"
      " <a href=\"mailto:bg@exept.de\">eXept Software AG</a>";
    break;
  default:
    return NPERR_GENERIC_ERROR;
  }
  return NPERR_NO_ERROR;
}

/***********************************************************************
 * Plugin loading and termination
 ***********************************************************************/

NPError 
NPP_New(NPMIMEType pluginType, NPP instance, uint16 mode, int16 argc,
		char* argn[], char* argv[], NPSavedData* saved)
{
  StxPlugin *plugin;
  char basedir[PATH_MAX];
  if (instance == NULL)
    return NPERR_INVALID_INSTANCE_ERROR;
  plugin= (StxPlugin*) NPN_MemAlloc(sizeof(StxPlugin));
  if (!plugin)
    return NPERR_OUT_OF_MEMORY_ERROR;
  plugin->argv= (char**) NPN_MemAlloc(sizeof(char*) * (16+2*argc));
  if (!plugin->argv)
    return NPERR_OUT_OF_MEMORY_ERROR;
  {
    char* ev= getenv("HOME");
    if (ev) {
      strcpy(basedir, ev);
      strcat(basedir, "/.npsqueak/");//here squeak, because this subfolder of your homefolder is generated by make install (on the top)
    } else {
      basedir[0]= '\0';
      fprintf(stderr, "Smalltalk\\X Plugin: No home directory?!\n");
      return NPERR_GENERIC_ERROR;
    }
  }
  /* Default settings */
  plugin->instance= instance;
  plugin->pid= 0;
  plugin->nswindow= 0;
  plugin->stxwindow= 0;
  plugin->display= NULL;
  plugin->input= 0;
  plugin->embedded= (mode == NP_EMBED);
  plugin->srcUrl= NULL;
  plugin->srcFilename= NULL;
  plugin->srcId= -1;
  strcpy(plugin->vmName, basedir);
  strcat(plugin->vmName, "npsqueakvm");//here squeak, because I don't know, if this name is refered by something else
  strcpy(plugin->imageName, basedir);
  strcat(plugin->imageName, "SqueakPlugin.image");//here squeak, because I don't know, if this name is refered by something else
  plugin->argv[0]= NPN_StrDup(plugin->vmName);
  plugin->argv[1]= NPN_StrDup("-display");
  plugin->argv[2]= NULL;              /* inserted later */
  plugin->argv[3]= NPN_StrDup("-browserPipes");
  plugin->argv[4]= NULL;              /* inserted later */
  plugin->argv[5]= NULL;              /* inserted later */
  plugin->argv[6]= NPN_StrDup(plugin->imageName); 
  plugin->argv[7]= NPN_StrDup("");    /* empty document file on cmdline! */ 
  plugin->argc= 8;
  if (plugin->embedded) {
    int i;
    for (i= 0; i < argc; i++) {
      plugin->argv[plugin->argc++]= NPN_StrDup(argn[i]);
      plugin->argv[plugin->argc++]= NPN_StrDup(argv[i] ? argv[i] : "");
      if (strcasecmp("SRC", argn[i]) == 0)
	plugin->srcUrl= NPN_StrDup(argv[i]);
    }
    if (!plugin->srcUrl)
      plugin->srcUrl= NPN_StrDup(""); /* we were embedded without a SRC */
  } else {
    /* if not embedded srcUrl will be set in NewStream */
    plugin->srcUrl= NULL;
  }
  plugin->argv[plugin->argc]= 0; 
  if (pipe(&plugin->pipes[STX_READ])
      || pipe(&plugin->pipes[PLUGIN_READ])) {
    perror("Smalltalk\\X Plugin: Creating pipes failed");
    return NPERR_GENERIC_ERROR;
  }
  DPRINT("NP: Created pipes (VM read: %d <- %d, NP read: %d <- %d)\n", 
	 plugin->pipes[STX_READ],
	 plugin->pipes[PLUGIN_WRITE],
	 plugin->pipes[PLUGIN_READ],
	 plugin->pipes[STX_WRITE]);
  instance->pdata= (void*) plugin;
  return NPERR_NO_ERROR;
}

NPError 
NPP_Destroy(NPP instance, NPSavedData** save)
{
  StxPlugin *plugin;
  if (!instance)
    return NPERR_INVALID_INSTANCE_ERROR;
  plugin= (StxPlugin*) instance->pdata;
  if (plugin) {
    int i;
    if (plugin->pid) {
      kill(plugin->pid, SIGTERM);
      plugin->pid= 0;
    }
    if (plugin->input) {
      XtRemoveInput(plugin->input);
    }
    for (i=0; i<4; i++)
      if (plugin->pipes[i]) {
	close(plugin->pipes[i]);
	plugin->pipes[i]= 0;
      }
    if (plugin->srcUrl) {
      NPN_MemFree(plugin->srcUrl);
      plugin->srcUrl= NULL;
    }
    if (plugin->srcFilename) {
      NPN_MemFree(plugin->srcFilename);
      plugin->srcFilename= NULL;
    }
    if (plugin->argv) {
      for (i=0; i<plugin->argc; i++) {
	if (plugin->argv[i])
	  NPN_MemFree(plugin->argv[i]);
      }
      plugin->argc= 0;
      NPN_MemFree(plugin->argv);
      plugin->argv= NULL;
    }
    NPN_MemFree(plugin);
  }
  instance->pdata= NULL;
  return NPERR_NO_ERROR;
}

/***********************************************************************
 * Plugin events we need to handle
 ***********************************************************************/

NPError 
NPP_SetWindow(NPP instance, NPWindow *pNPWindow)
{
  StxPlugin *plugin;
  if (!instance)
    return NPERR_INVALID_INSTANCE_ERROR;
  plugin= (StxPlugin*) instance->pdata;
  if (!plugin)
    return NPERR_GENERIC_ERROR;
  if (pNPWindow == NULL) 
    return NPERR_NO_ERROR;

  if (!plugin->display) {
    /* first time only */
    plugin->display= ((NPSetWindowCallbackStruct *)pNPWindow->ws_info)->display;
  }
  SetWindow(plugin, (Window) pNPWindow->window, 
		  pNPWindow->width, pNPWindow->height);
  if (!plugin->pid)
    Run(plugin);
  return NPERR_NO_ERROR;
}


NPError 
NPP_NewStream(NPP instance, NPMIMEType type, NPStream *stream, NPBool seekable, uint16 *stype)
{
  StxPlugin *plugin= (StxPlugin*) instance->pdata;
  DPRINT("NP: NewStream(%s, id=%i)\n", stream->url, 
	   stream->notifyData ? ((StxStream*) stream->notifyData)->id : -1);
  if (!stream->notifyData && !plugin->srcUrl) {
    /* We did not request this stream, so it is our SRC file. */
    plugin->srcUrl= NPN_StrDup(stream->url);
    plugin->argv[plugin->argc++]= NPN_StrDup("SRC");
    plugin->argv[plugin->argc++]= NPN_StrDup(plugin->srcUrl);
    DPRINT("NP:   got srcUrl=%s\n", plugin->srcUrl);
    Run(plugin);
  }
    
  *stype= NP_ASFILEONLY;          /* We want the file after download */
  return NPERR_NO_ERROR;
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
  /* We'll clean up in URLNotify */
  DPRINT("NP: DestroyStream(%s, id=%i)\n", stream->url, 
	 stream->notifyData ? ((StxStream*) stream->notifyData)->id : -1);
  return NPERR_NO_ERROR;
}


void 
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
  int id= stream->notifyData ? ((StxStream*) stream->notifyData)->id : -1;
  char lname[PATH_MAX];
  StxPlugin *plugin= (StxPlugin*) instance->pdata;
  DPRINT("NP: StreamAsFile(%s, id=%i)\n", stream->url, id);
  DPRINT("NP:   fname=%s\n", fname ? fname : "<NULL>");
  if (!plugin || !fname) return;

  /* need to copy file because it might be deleted after return */
  strncpy(lname, fname, PATH_MAX);
  strcat(lname, "$");
  DPRINT("NP:  lname=%s\n", lname);
  if (-1 == link(fname, lname))
    DPRINT("NP:   Link failed: %s\n", strerror(errno));
  fname= lname;

  if (!stream->notifyData && !plugin->srcFilename) {
    /* We did not request this stream, so it is our SRC file. */
    plugin->srcFilename= NPN_StrDup(fname);
    DPRINT("NP:   got srcFilename=%s\n", plugin->srcFilename);
    if (plugin->srcId >= 0) {
      /* plugin wanted it already */
      DeliverFile(plugin, plugin->srcId, plugin->srcFilename);
      plugin->srcId= -1;
    }
    return;
  }

  DeliverFile(plugin, id, fname);

  /* signal URLNotify that we're done */
  ((StxStream*) stream->notifyData)->id= -1;
}

void
NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
{
  int id= notifyData ? ((StxStream*) notifyData)->id : -1;
  int ok= reason == NPRES_DONE;
  StxPlugin *plugin= (StxPlugin*) instance->pdata;
  DPRINT("NP: URLNotify(%s, id=%i, ok=%i)\n", url, id, ok);
  if (notifyData) NPN_MemFree(notifyData);
  if (!plugin || -1 == id) return;

  DeliverFile(plugin, id, NULL);
}

/***********************************************************************
 * Plugin stubs
 ***********************************************************************/


NPError
NPP_Initialize(void)
{
  return NPERR_NO_ERROR;
}


void
NPP_Shutdown(void)
{
}


/* We don't have an associated java class */

jref
NPP_GetJavaClass()
{
  return NULL;
}

/* We don't really stream */

int32 
NPP_WriteReady(NPP instance, NPStream *stream)
{
  return 0X0FFFFFFF;
}


int32 
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
  return len;
}


/* We don't print */

void 
NPP_Print(NPP instance, NPPrint* printInfo)
{
}


/***********************************************************************
 * Our functions
 ***********************************************************************/

/* Read from command pipe. */
static void
Receive(StxPlugin *plugin, void *buf, size_t count)
{
  ssize_t n;
  do {
    n= read(plugin->pipes[PLUGIN_READ], buf, count);
  } while (n == -1 && errno == EINTR);
  if (n == -1)
    perror("Smalltalk\\X plugin pipe read failed:");
  if (n < count)
    fprintf(stderr, "Smalltalk\\X plugin read too few data from pipe\n");
}


/* Write to command pipe. */
static void
Send(StxPlugin *plugin, const void *buf, size_t count)
{
  ssize_t n;
  do {
    n= write(plugin->pipes[PLUGIN_WRITE], buf, count);
  } while (n == -1 && errno == EINTR);
  if (n == -1)
    perror("Smalltalk\\X plugin write failed:");
  if (n < count)
    fprintf(stderr, "Smalltalk\\X plugin wrote too few data to pipe\n");
}

static void
SendInt(StxPlugin *plugin, int value)
{
  Send(plugin, &value, 4);
}


static void 
DeliverFile(StxPlugin *plugin, int id, const char* fname)
{
  int ok= fname != NULL;
  DPRINT("NP:   Send RECEIVE_DATA id=%i state=%i\n", id, ok);

  errno= 0;
  SendInt(plugin, CMD_RECEIVE_DATA);
  SendInt(plugin, id);
  SendInt(plugin, ok);

  if (ok) {
    int length= strlen(fname);
    SendInt(plugin, length);
    Send(plugin, fname, length);
  }

  if (errno)
    perror("Smalltalk\\X Plugin (StreamAsFile)");
}


static void 
Run(StxPlugin *plugin)
{
  if (plugin->pid || !plugin->nswindow || !plugin->srcUrl)
    return;
  
  plugin->pid= fork();
  
  if (plugin->pid == -1) {
    perror("Smalltalk\\X fork() failed");
    plugin->pid= 0;
    return;
  }
  DPRINT("NP: fork() -> %i\n", plugin->pid);
  if (plugin->pid == 0) {
    char tmp1[16], tmp2[16];
    plugin->argv[2]= NPN_StrDup(DisplayString(plugin->display));
    sprintf(tmp1, "%i", plugin->pipes[STX_READ]);
    plugin->argv[4]= NPN_StrDup(tmp1);
    sprintf(tmp2, "%i", plugin->pipes[STX_WRITE]);
    plugin->argv[5]= NPN_StrDup(tmp2);
    DPRINT("NP(child): Running Smalltalk\\X VM with arguments\n");
    {
      int i;
      for (i= 1; i<plugin->argc; i++)
	DPRINT("    %s\n", plugin->argv[i]);
    }
    /* this is from the XLib manual ... */
    if ((fcntl(ConnectionNumber(plugin->display), F_SETFD, FD_CLOEXEC)) == -1)
      DPRINT("NP: Cannot disinherit X connection fd\n");
    DPRINT("NP(child): trying %s\n", plugin->vmName);
    execv(plugin->vmName, plugin->argv);

    strcpy(plugin->vmName, VM_LIBDIR "/npsqueakrun");//here squeak, because I don't know, if this name is refered by something else
    NPN_MemFree(plugin->argv[0]);
    plugin->argv[0]= NPN_StrDup(plugin->vmName);
    DPRINT("NP(child): trying %s\n", plugin->vmName);
    execv(plugin->vmName, plugin->argv);

    fprintf(stderr, "Smalltalk\\X Plugin: running \"%s\"\n", plugin->vmName);
    perror("Smalltalk\\X execv() failed");
    _exit(1);
  } 

  {
    XtAppContext app= XtDisplayToApplicationContext(plugin->display);
    plugin->input= XtAppAddInput(app,
				 plugin->pipes[PLUGIN_READ],
				 (XtPointer) XtInputReadMask,
				 (XtInputCallbackProc) InputCallback,
				 plugin);
  }

  DPRINT("NP: Sending browser window=0x%X\n", plugin->nswindow);
  SendInt(plugin, plugin->nswindow);
}


static void
SetWindow(StxPlugin *plugin, Window window, int width, int height)
{
  DPRINT("NP: SetWindow(0x%X, %i@%i)\n", window, width, height);
  if (plugin->nswindow == window) {
    XResizeWindow(plugin->display, plugin->nswindow, width, height);
  } else {
    /* New window */
    plugin->nswindow= window;
    SetUpWindow(plugin);
    if (plugin->stxwindow) {
      DPRINT("NP: Reparenting to plugin window 0x%X\n", plugin->nswindow);
      XReparentWindow(plugin->display, plugin->stxwindow, plugin->nswindow, 0, 0);
      XMapWindow(plugin->display, plugin->stxwindow);
      
      SendInt(plugin, CMD_BROWSER_WINDOW);
      SendInt(plugin, plugin->nswindow);
    }
  }
  if (plugin->stxwindow)
    XResizeWindow(plugin->display, plugin->stxwindow, width, height);
}


static void
SetUpWindow(StxPlugin *plugin)
{
  Widget w= XtWindowToWidget(plugin->display, plugin->nswindow);
  DPRINT("NP: SetUpWindow(0x%X)\n", plugin->nswindow);
  XSelectInput(plugin->display, plugin->nswindow, 0); 
  if (plugin->embedded) {
    /* need to capture destroys when page is re-layouted */
    XtAddCallback(w, XtNdestroyCallback, 
		  (XtCallbackProc) DestroyCallback, plugin);
  }
}


static void
SetUpStxWindow(StxPlugin *plugin)
{
  Receive(plugin, &plugin->stxwindow, 4);
  DPRINT("NP: got stx window=0x%X\n", plugin->stxwindow);
  DPRINT("NP: resizing stx window\n");
  {
    XWindowAttributes attr;
    XGetWindowAttributes(plugin->display, plugin->nswindow, &attr);
    XResizeWindow(plugin->display, plugin->stxwindow, attr.width, attr.height);
  }
  DPRINT("NP: mapping stx window\n");
  XMapWindow(plugin->display, plugin->stxwindow);
}

static int
IgnoreErrors(Display *display, XErrorEvent *evt)
{
  DPRINT("NP: X Error ignored.\n");
  return 1;
}

static void
DestroyCallback(Widget widget, StxPlugin *plugin, XtPointer data)
{
  int (*previous)(Display *, XErrorEvent *);

  DPRINT("NP: DestroyCallback()\n");

  /* Ignore errors due to the window being closed */
  XSync(plugin->display, False);
  previous= XSetErrorHandler(IgnoreErrors);

  /* Save Smalltalk\\X window from being destroyed by page re-layout */
  DPRINT("NP: Reparenting to root window\n");
  XUnmapWindow(plugin->display, plugin->stxwindow);
  XReparentWindow(plugin->display, plugin->stxwindow,
		  DefaultRootWindow(plugin->display), 0, 0);

  /* Report errors */
  XSync(plugin->display, False);
  XSetErrorHandler(previous);
}

static void 
InputCallback(StxPlugin *plugin, int *source, XtInputId* id)
{
  int cmd;
  DPRINT("NP: InputCallback()\n");
  if (!plugin->stxwindow)
    {
      /* read stxwindow */
      SetUpStxWindow(plugin);
      return;
    }
  Receive(plugin, &cmd, 4);
  switch (cmd) {
  case CMD_GET_URL: 
    GetUrl(plugin);
    break;
  case CMD_POST_URL: 
    PostUrl(plugin);
    break;
  default:
    fprintf(stderr, "Unknown command from Smalltalk\\X: %i\n", cmd);
  }
}


static void
GetUrl(StxPlugin *plugin)
{
  char *url, *target;
  int id, urlSize, targetSize;

  errno= 0;
  Receive(plugin, &id, 4);
  /* Read URL from pipe */
  Receive(plugin, &urlSize, 4);
  if (urlSize > 0) {
    url= NPN_MemAlloc(urlSize+1);
    Receive(plugin, url, urlSize);
    url[urlSize]= 0;
  } else url= NULL;
  /* Read target from pipe */
  Receive(plugin, &targetSize, 4);
  if (targetSize > 0) {
    target= NPN_MemAlloc(targetSize+1);
    Receive(plugin, target, targetSize);
    target[targetSize]= 0;
  } else target= NULL;

  if (errno) {
    perror("Smalltalk\\X Plugin (GetUrl)");
  } else {
    DPRINT("NP: GetUrl(%s, %s)\n", url, target ? target : "NULL");
    if (strcmp(url, plugin->srcUrl)==0) {
      if (plugin->srcFilename)
	DeliverFile(plugin, id, plugin->srcFilename);
      else
	plugin->srcId= id;
    } else {
      StxStream* notifyData= (StxStream*) NPN_MemAlloc(sizeof(StxStream));
      if (!notifyData) { 
	fprintf(stderr, "Smalltalk\\X Plugin (GetUrl): alloc failed\n");
      } else {
	DPRINT("NP: GetURLNotify(%s, id=%i)\n", url, id);
	notifyData->id= id;
	NPN_GetURLNotify(plugin->instance, url, target, notifyData);
      }
    }
  }

  if (url) NPN_MemFree(url);
  if (target) NPN_MemFree(target);
}

static void
PostUrl(StxPlugin *plugin)
{
  char *url, *target, *data;
  int id, urlSize, targetSize, dataSize;

  errno= 0;
  Receive(plugin, &id, 4);
  /* Read URL from pipe */
  Receive(plugin, &urlSize, 4);
  if (urlSize > 0) {
    url= NPN_MemAlloc(urlSize+1);
    Receive(plugin, url, urlSize);
    url[urlSize]= 0;
  } else url= NULL;
  /* Read target from pipe */
  Receive(plugin, &targetSize, 4);
  if (targetSize > 0) {
    target= NPN_MemAlloc(targetSize+1);
    Receive(plugin, target, targetSize);
    target[targetSize]= 0;
  } else target= NULL;
  /* Read post data from pipe */
  Receive(plugin, &dataSize, 4);
  if (dataSize > 0) {
    data= NPN_MemAlloc(dataSize);
    Receive(plugin, data, dataSize);
  } else data= NULL;

  if (errno) {
    perror("Smalltalk\\X Plugin (PostUrl)");
  } else {
    StxStream* notifyData= (StxStream*) NPN_MemAlloc(sizeof(StxStream));
    if (!notifyData) { 
      fprintf(stderr, "Smalltalk\\X Plugin (PostUrl): alloc failed\n");
    } else {
      DPRINT("NP: PostURLNotify(%s, id=%i)\n", url, id);
      notifyData->id= id;
      NPN_PostURLNotify(plugin->instance, url, target, 
			dataSize, data, FALSE, notifyData);
    }
  }

  if (url) NPN_MemFree(url);
  if (target) NPN_MemFree(target);
  if (data) NPN_MemFree(data);
}
