"{ Package: 'stx:goodies/soap/splash' }"

"{ NameSpace: SOAP }"

Model subclass:#SplashModelAbstract
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'SOAP-Splash'
!

SplashModelAbstract comment:'I am an abstract model for Splash that provides helper methods to construct Splash Requests, send them, and extract the return.

My methods in the "dotnet" category are useful in sending requests to DOTNET servers

My methods in the "rpcenc" category are useful in sending requests to RPC/Encoded servers.

Subclass should define:
	methodNS 	<Symbol>		the operation''s namespace (this is almost always 
					the same for all operations in a service)
	serviceUrl 	<String>		the service''s endPoint url (In a WSDL this is the addressLocation)

See Client samples for examples

==========
DOTNET implementation notes
Most of the dotnet helper construction methods require an operation name as an argument. This can be found in the service''s WSDL. DOTNET services are not rpc/encoded so there are no rules (without using the Schema)  for what a request and response message should look like. There is a common pattern based on the operation''s name and this class exploits that pattern.'
!


!SplashModelAbstract methodsFor:'accessing-private'!

serviceUrl 
	"Private - Answer the endpoint Url for the service.
	(Can be located in a WSDL in the service/port addressLocation attribute)"

	self subclassResponsibility
! !

!SplashModelAbstract methodsFor:'constants-private'!

methodNS
	self subclassResponsibility
!

schemaNS
	"Private - Answer the targetNamespace of the schema for this service.
	Note: this assumes the service is implemented as a 'typical' .NET service"

	^self methodNS
! !

!SplashModelAbstract methodsFor:'dotnet-private'!

exchangeFor: anOpName parameters: anArrayOfAssocs
	"Private - Answer a <SptRequestingExchange> with a soap request envelope built using the typical .NET pattern of:
	-one body child named as the operation, qualified in the single schema's targetNamespace
	-a number of parameters qualified in that same namespace"

	| request |
	request := SplashDOCRequest new.
	request 
		addNETMessage: anOpName
		ns: self schemaNS
		parameters: anArrayOfAssocs.
	^self exchange: request op: anOpName
!

responseFor: anOpName parameters: anArrayOfAssocs
	"Private - Answer a <SplashResponse>.
	-Build an envelope, create an exchange, process the exchange (ie send it) and put the response soap envelope into a SplashResponse object.
	-This will signal an error if the transport fails."

	| exchange |
	exchange := self exchangeFor: anOpName parameters: anArrayOfAssocs.
	self doExchange: exchange.
	^(SplashResponse exchange: exchange)
		beLiteralEnc;
		yourself
!

returnFor: anOpName parameters: anArrayOfAssocs
	"Private - Answer the decoded object of the first child of the first child of the soap:Body node of a soap response.
	... that is where the .NET 'typically' puts the interesting value (ie it mirrors the return parameter of a SOAP section 7 return parameter.
	-This will signal an error if the transport fails.
	-This will signal an error if the server's response is a soap:Fault"

	^(self responseFor: anOpName parameters: anArrayOfAssocs) return
!

returnFor: anOpName parameters: anArrayOfAssocs as: aTypeName
	"Private - Same semantics as #returnFor:parameters: except 
	-The typeName is used to force a conversion. (examples: 'xsd:boolean' 'xsd:float')."

	^(self responseFor: anOpName parameters: anArrayOfAssocs) returnAs: aTypeName
! !

!SplashModelAbstract methodsFor:'event handling'!

onStateChanged: aStateString
	"As an exchange <SptRequestingExchange> is processed, it goes through a number of state changes"

	self trigger: #stateChanged: with: aStateString
! !

!SplashModelAbstract methodsFor:'helpers-private'!

soapActionFor: anOpString
	"Private - This seems to be the common pattern for DOTNET services but may not always be the case"

	^self schemaNS last = $/
		ifTrue: [self schemaNS asString ,  anOpString]
		ifFalse: [self schemaNS asString , '/' ,  anOpString]

! !

!SplashModelAbstract methodsFor:'operations-private'!

doExchange: anExchange
	"Private - Process the exchange using default transport.
	-Firstly register to receive state change events from the exchange
	-Lastly check that the final state was Success (Note: this does not check for a Fault envelope, only that the binding successfully received a response"

	anExchange when: #stateChanged: send: #onStateChanged: to: self.
	anExchange processUsingDefault.
	anExchange assertSuccessState.


	
!

exchange: request op: opName
	"Private - Answer a <SptRequestingExchange> that can be used to process the <SplashRequest>"

	^request exchangeTo: self serviceUrl soapAction: (self soapActionFor: opName)
! !

!SplashModelAbstract methodsFor:'rpcenc-private'!

getResponse: aRequest op: anOpName
	"Private - Answer a <SplashResponse>.
	-Create an exchange, process it (ie send it), and answer an object that can be used to access the server's response.
	-This will signal an error if the transport fails"

	| exchange |
	exchange := self exchange: aRequest op: anOpName.
	self trigger: #aboutToProcessExchange: with: exchange.
	self doExchange: exchange.
	^SplashResponse exchange: exchange
!

getReturn: aRequest op: anOpName
	"Private - Answer the 'return' parameter (defined by SOAP1.1 section 7) 
	-This will signal an error if the transport fails.
	-This will signal an error if the server's response is a soap:Fault"

	^(self getResponse: aRequest op: anOpName) return
!

getReturn: aRequest op: anOpName as: aTypeName
	"Private - Answer the 'return' parameter (defined by SOAP1.1 section 7) 
	-The typeName is used to force a conversion. (examples: 'xsd:boolean' 'xsd:float').
	-This will signal an error if the transport fails.
	-This will signal an error if the server's response is a soap:Fault"

	^(self getResponse: aRequest op: anOpName) returnAs: aTypeName
!

getVoidReturnFor: anOpName
	| request |
	request := SplashRPCRequest method: anOpName ns: self methodNS.
	^self getReturn: request op: anOpName
! !

!SplashModelAbstract class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/goodies/soap/splash/SplashModelAbstract.st,v 1.2 2002/11/18 19:03:00 cg Exp $'
! !
